// Autogenerated: 'src/ExtractionOCaml/word_by_word_montgomery' --lang Zig --internal-static --public-function-case camelCase --private-function-case camelCase --public-type-case UpperCamelCase --private-type-case UpperCamelCase --no-prefix-fiat --package-name p384 '' 32 '2^384 - 2^128 - 2^96 + 2^32 - 1' mul square add sub opp from_montgomery to_montgomery nonzero selectznz to_bytes from_bytes one msat divstep divstep_precomp
// curve description (via package name): p384
// machine_wordsize = 32 (from "32")
// requested operations: mul, square, add, sub, opp, from_montgomery, to_montgomery, nonzero, selectznz, to_bytes, from_bytes, one, msat, divstep, divstep_precomp
// m = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff (from "2^384 - 2^128 - 2^96 + 2^32 - 1")
//
// NOTE: In addition to the bounds specified above each function, all
//   functions synthesized for this Montgomery arithmetic require the
//   input to be strictly less than the prime modulus (m), and also
//   require the input to be in the unique saturated representation.
//   All functions also ensure that these two properties are true of
//   return values.
//
// Computed values:
//   eval z = z[0] + (z[1] << 32) + (z[2] << 64) + (z[3] << 96) + (z[4] << 128) + (z[5] << 160) + (z[6] << 192) + (z[7] << 224) + (z[8] << 256) + (z[9] << 0x120) + (z[10] << 0x140) + (z[11] << 0x160)
//   bytes_eval z = z[0] + (z[1] << 8) + (z[2] << 16) + (z[3] << 24) + (z[4] << 32) + (z[5] << 40) + (z[6] << 48) + (z[7] << 56) + (z[8] << 64) + (z[9] << 72) + (z[10] << 80) + (z[11] << 88) + (z[12] << 96) + (z[13] << 104) + (z[14] << 112) + (z[15] << 120) + (z[16] << 128) + (z[17] << 136) + (z[18] << 144) + (z[19] << 152) + (z[20] << 160) + (z[21] << 168) + (z[22] << 176) + (z[23] << 184) + (z[24] << 192) + (z[25] << 200) + (z[26] << 208) + (z[27] << 216) + (z[28] << 224) + (z[29] << 232) + (z[30] << 240) + (z[31] << 248) + (z[32] << 256) + (z[33] << 0x108) + (z[34] << 0x110) + (z[35] << 0x118) + (z[36] << 0x120) + (z[37] << 0x128) + (z[38] << 0x130) + (z[39] << 0x138) + (z[40] << 0x140) + (z[41] << 0x148) + (z[42] << 0x150) + (z[43] << 0x158) + (z[44] << 0x160) + (z[45] << 0x168) + (z[46] << 0x170) + (z[47] << 0x178)
//   twos_complement_eval z = let x1 := z[0] + (z[1] << 32) + (z[2] << 64) + (z[3] << 96) + (z[4] << 128) + (z[5] << 160) + (z[6] << 192) + (z[7] << 224) + (z[8] << 256) + (z[9] << 0x120) + (z[10] << 0x140) + (z[11] << 0x160) in
//                            if x1 & (2^384-1) < 2^383 then x1 & (2^384-1) else (x1 & (2^384-1)) - 2^384

const std = @import("std");
const mode = @import("builtin").mode; // Checked arithmetic is disabled in non-debug modes to avoid side channels

inline fn cast(comptime DestType: type, target: anytype) DestType {
    @setEvalBranchQuota(10000);
    if (@typeInfo(@TypeOf(target)) == .int) {
        const dest = @typeInfo(DestType).int;
        const source = @typeInfo(@TypeOf(target)).int;
        if (dest.bits < source.bits) {
            const T = std.meta.Int(source.signedness, dest.bits);
            return @bitCast(@as(T, @truncate(target)));
        }
    }
    return target;
}

// The type MontgomeryDomainFieldElement is a field element in the Montgomery domain.
// Bounds: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub const MontgomeryDomainFieldElement = [12]u32;

// The type NonMontgomeryDomainFieldElement is a field element NOT in the Montgomery domain.
// Bounds: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub const NonMontgomeryDomainFieldElement = [12]u32;

/// The function addcarryxU32 is an addition with carry.
///
/// Postconditions:
///   out1 = (arg1 + arg2 + arg3) mod 2^32
///   out2 = ⌊(arg1 + arg2 + arg3) / 2^32⌋
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [0x0 ~> 0xffffffff]
///   arg3: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [0x0 ~> 0x1]
inline fn addcarryxU32(out1: *u32, out2: *u1, arg1: u1, arg2: u32, arg3: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = ((cast(u64, arg1) + cast(u64, arg2)) + cast(u64, arg3));
    const x2 = cast(u32, (x1 & cast(u64, 0xffffffff)));
    const x3 = cast(u1, (x1 >> 32));
    out1.* = x2;
    out2.* = x3;
}

/// The function subborrowxU32 is a subtraction with borrow.
///
/// Postconditions:
///   out1 = (-arg1 + arg2 + -arg3) mod 2^32
///   out2 = -⌊(-arg1 + arg2 + -arg3) / 2^32⌋
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [0x0 ~> 0xffffffff]
///   arg3: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [0x0 ~> 0x1]
inline fn subborrowxU32(out1: *u32, out2: *u1, arg1: u1, arg2: u32, arg3: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = ((cast(i64, arg2) - cast(i64, arg1)) - cast(i64, arg3));
    const x2 = cast(i1, (x1 >> 32));
    const x3 = cast(u32, (x1 & cast(i64, 0xffffffff)));
    out1.* = x3;
    out2.* = cast(u1, (cast(i2, 0x0) - cast(i2, x2)));
}

/// The function mulxU32 is a multiplication, returning the full double-width result.
///
/// Postconditions:
///   out1 = (arg1 * arg2) mod 2^32
///   out2 = ⌊arg1 * arg2 / 2^32⌋
///
/// Input Bounds:
///   arg1: [0x0 ~> 0xffffffff]
///   arg2: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [0x0 ~> 0xffffffff]
inline fn mulxU32(out1: *u32, out2: *u32, arg1: u32, arg2: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (cast(u64, arg1) * cast(u64, arg2));
    const x2 = cast(u32, (x1 & cast(u64, 0xffffffff)));
    const x3 = cast(u32, (x1 >> 32));
    out1.* = x2;
    out2.* = x3;
}

/// The function cmovznzU32 is a single-word conditional move.
///
/// Postconditions:
///   out1 = (if arg1 = 0 then arg2 else arg3)
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [0x0 ~> 0xffffffff]
///   arg3: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
inline fn cmovznzU32(out1: *u32, arg1: u1, arg2: u32, arg3: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (~(~arg1));
    const x2 = cast(u32, (cast(i64, cast(i1, (cast(i2, 0x0) - cast(i2, x1)))) & cast(i64, 0xffffffff)));
    const x3 = ((x2 & arg3) | ((~x2) & arg2));
    out1.* = x3;
}

/// The function mul multiplies two field elements in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
///   0 ≤ eval arg2 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) * eval (from_montgomery arg2)) mod m
///   0 ≤ eval out1 < m
///
pub fn mul(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement, arg2: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[1]);
    const x2 = (arg1[2]);
    const x3 = (arg1[3]);
    const x4 = (arg1[4]);
    const x5 = (arg1[5]);
    const x6 = (arg1[6]);
    const x7 = (arg1[7]);
    const x8 = (arg1[8]);
    const x9 = (arg1[9]);
    const x10 = (arg1[10]);
    const x11 = (arg1[11]);
    const x12 = (arg1[0]);
    var x13: u32 = undefined;
    var x14: u32 = undefined;
    mulxU32(&x13, &x14, x12, (arg2[11]));
    var x15: u32 = undefined;
    var x16: u32 = undefined;
    mulxU32(&x15, &x16, x12, (arg2[10]));
    var x17: u32 = undefined;
    var x18: u32 = undefined;
    mulxU32(&x17, &x18, x12, (arg2[9]));
    var x19: u32 = undefined;
    var x20: u32 = undefined;
    mulxU32(&x19, &x20, x12, (arg2[8]));
    var x21: u32 = undefined;
    var x22: u32 = undefined;
    mulxU32(&x21, &x22, x12, (arg2[7]));
    var x23: u32 = undefined;
    var x24: u32 = undefined;
    mulxU32(&x23, &x24, x12, (arg2[6]));
    var x25: u32 = undefined;
    var x26: u32 = undefined;
    mulxU32(&x25, &x26, x12, (arg2[5]));
    var x27: u32 = undefined;
    var x28: u32 = undefined;
    mulxU32(&x27, &x28, x12, (arg2[4]));
    var x29: u32 = undefined;
    var x30: u32 = undefined;
    mulxU32(&x29, &x30, x12, (arg2[3]));
    var x31: u32 = undefined;
    var x32: u32 = undefined;
    mulxU32(&x31, &x32, x12, (arg2[2]));
    var x33: u32 = undefined;
    var x34: u32 = undefined;
    mulxU32(&x33, &x34, x12, (arg2[1]));
    var x35: u32 = undefined;
    var x36: u32 = undefined;
    mulxU32(&x35, &x36, x12, (arg2[0]));
    var x37: u32 = undefined;
    var x38: u1 = undefined;
    addcarryxU32(&x37, &x38, 0x0, x36, x33);
    var x39: u32 = undefined;
    var x40: u1 = undefined;
    addcarryxU32(&x39, &x40, x38, x34, x31);
    var x41: u32 = undefined;
    var x42: u1 = undefined;
    addcarryxU32(&x41, &x42, x40, x32, x29);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    addcarryxU32(&x43, &x44, x42, x30, x27);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    addcarryxU32(&x45, &x46, x44, x28, x25);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    addcarryxU32(&x47, &x48, x46, x26, x23);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    addcarryxU32(&x49, &x50, x48, x24, x21);
    var x51: u32 = undefined;
    var x52: u1 = undefined;
    addcarryxU32(&x51, &x52, x50, x22, x19);
    var x53: u32 = undefined;
    var x54: u1 = undefined;
    addcarryxU32(&x53, &x54, x52, x20, x17);
    var x55: u32 = undefined;
    var x56: u1 = undefined;
    addcarryxU32(&x55, &x56, x54, x18, x15);
    var x57: u32 = undefined;
    var x58: u1 = undefined;
    addcarryxU32(&x57, &x58, x56, x16, x13);
    const x59 = (cast(u32, x58) + x14);
    var x60: u32 = undefined;
    var x61: u32 = undefined;
    mulxU32(&x60, &x61, x35, 0xffffffff);
    var x62: u32 = undefined;
    var x63: u32 = undefined;
    mulxU32(&x62, &x63, x35, 0xffffffff);
    var x64: u32 = undefined;
    var x65: u32 = undefined;
    mulxU32(&x64, &x65, x35, 0xffffffff);
    var x66: u32 = undefined;
    var x67: u32 = undefined;
    mulxU32(&x66, &x67, x35, 0xffffffff);
    var x68: u32 = undefined;
    var x69: u32 = undefined;
    mulxU32(&x68, &x69, x35, 0xffffffff);
    var x70: u32 = undefined;
    var x71: u32 = undefined;
    mulxU32(&x70, &x71, x35, 0xffffffff);
    var x72: u32 = undefined;
    var x73: u32 = undefined;
    mulxU32(&x72, &x73, x35, 0xffffffff);
    var x74: u32 = undefined;
    var x75: u32 = undefined;
    mulxU32(&x74, &x75, x35, 0xfffffffe);
    var x76: u32 = undefined;
    var x77: u32 = undefined;
    mulxU32(&x76, &x77, x35, 0xffffffff);
    var x78: u32 = undefined;
    var x79: u32 = undefined;
    mulxU32(&x78, &x79, x35, 0xffffffff);
    var x80: u32 = undefined;
    var x81: u1 = undefined;
    addcarryxU32(&x80, &x81, 0x0, x77, x74);
    var x82: u32 = undefined;
    var x83: u1 = undefined;
    addcarryxU32(&x82, &x83, x81, x75, x72);
    var x84: u32 = undefined;
    var x85: u1 = undefined;
    addcarryxU32(&x84, &x85, x83, x73, x70);
    var x86: u32 = undefined;
    var x87: u1 = undefined;
    addcarryxU32(&x86, &x87, x85, x71, x68);
    var x88: u32 = undefined;
    var x89: u1 = undefined;
    addcarryxU32(&x88, &x89, x87, x69, x66);
    var x90: u32 = undefined;
    var x91: u1 = undefined;
    addcarryxU32(&x90, &x91, x89, x67, x64);
    var x92: u32 = undefined;
    var x93: u1 = undefined;
    addcarryxU32(&x92, &x93, x91, x65, x62);
    var x94: u32 = undefined;
    var x95: u1 = undefined;
    addcarryxU32(&x94, &x95, x93, x63, x60);
    const x96 = (cast(u32, x95) + x61);
    var x97: u32 = undefined;
    var x98: u1 = undefined;
    addcarryxU32(&x97, &x98, 0x0, x35, x78);
    var x99: u32 = undefined;
    var x100: u1 = undefined;
    addcarryxU32(&x99, &x100, x98, x37, x79);
    var x101: u32 = undefined;
    var x102: u1 = undefined;
    addcarryxU32(&x101, &x102, x100, x39, cast(u32, 0x0));
    var x103: u32 = undefined;
    var x104: u1 = undefined;
    addcarryxU32(&x103, &x104, x102, x41, x76);
    var x105: u32 = undefined;
    var x106: u1 = undefined;
    addcarryxU32(&x105, &x106, x104, x43, x80);
    var x107: u32 = undefined;
    var x108: u1 = undefined;
    addcarryxU32(&x107, &x108, x106, x45, x82);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    addcarryxU32(&x109, &x110, x108, x47, x84);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    addcarryxU32(&x111, &x112, x110, x49, x86);
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    addcarryxU32(&x113, &x114, x112, x51, x88);
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    addcarryxU32(&x115, &x116, x114, x53, x90);
    var x117: u32 = undefined;
    var x118: u1 = undefined;
    addcarryxU32(&x117, &x118, x116, x55, x92);
    var x119: u32 = undefined;
    var x120: u1 = undefined;
    addcarryxU32(&x119, &x120, x118, x57, x94);
    var x121: u32 = undefined;
    var x122: u1 = undefined;
    addcarryxU32(&x121, &x122, x120, x59, x96);
    var x123: u32 = undefined;
    var x124: u32 = undefined;
    mulxU32(&x123, &x124, x1, (arg2[11]));
    var x125: u32 = undefined;
    var x126: u32 = undefined;
    mulxU32(&x125, &x126, x1, (arg2[10]));
    var x127: u32 = undefined;
    var x128: u32 = undefined;
    mulxU32(&x127, &x128, x1, (arg2[9]));
    var x129: u32 = undefined;
    var x130: u32 = undefined;
    mulxU32(&x129, &x130, x1, (arg2[8]));
    var x131: u32 = undefined;
    var x132: u32 = undefined;
    mulxU32(&x131, &x132, x1, (arg2[7]));
    var x133: u32 = undefined;
    var x134: u32 = undefined;
    mulxU32(&x133, &x134, x1, (arg2[6]));
    var x135: u32 = undefined;
    var x136: u32 = undefined;
    mulxU32(&x135, &x136, x1, (arg2[5]));
    var x137: u32 = undefined;
    var x138: u32 = undefined;
    mulxU32(&x137, &x138, x1, (arg2[4]));
    var x139: u32 = undefined;
    var x140: u32 = undefined;
    mulxU32(&x139, &x140, x1, (arg2[3]));
    var x141: u32 = undefined;
    var x142: u32 = undefined;
    mulxU32(&x141, &x142, x1, (arg2[2]));
    var x143: u32 = undefined;
    var x144: u32 = undefined;
    mulxU32(&x143, &x144, x1, (arg2[1]));
    var x145: u32 = undefined;
    var x146: u32 = undefined;
    mulxU32(&x145, &x146, x1, (arg2[0]));
    var x147: u32 = undefined;
    var x148: u1 = undefined;
    addcarryxU32(&x147, &x148, 0x0, x146, x143);
    var x149: u32 = undefined;
    var x150: u1 = undefined;
    addcarryxU32(&x149, &x150, x148, x144, x141);
    var x151: u32 = undefined;
    var x152: u1 = undefined;
    addcarryxU32(&x151, &x152, x150, x142, x139);
    var x153: u32 = undefined;
    var x154: u1 = undefined;
    addcarryxU32(&x153, &x154, x152, x140, x137);
    var x155: u32 = undefined;
    var x156: u1 = undefined;
    addcarryxU32(&x155, &x156, x154, x138, x135);
    var x157: u32 = undefined;
    var x158: u1 = undefined;
    addcarryxU32(&x157, &x158, x156, x136, x133);
    var x159: u32 = undefined;
    var x160: u1 = undefined;
    addcarryxU32(&x159, &x160, x158, x134, x131);
    var x161: u32 = undefined;
    var x162: u1 = undefined;
    addcarryxU32(&x161, &x162, x160, x132, x129);
    var x163: u32 = undefined;
    var x164: u1 = undefined;
    addcarryxU32(&x163, &x164, x162, x130, x127);
    var x165: u32 = undefined;
    var x166: u1 = undefined;
    addcarryxU32(&x165, &x166, x164, x128, x125);
    var x167: u32 = undefined;
    var x168: u1 = undefined;
    addcarryxU32(&x167, &x168, x166, x126, x123);
    const x169 = (cast(u32, x168) + x124);
    var x170: u32 = undefined;
    var x171: u1 = undefined;
    addcarryxU32(&x170, &x171, 0x0, x99, x145);
    var x172: u32 = undefined;
    var x173: u1 = undefined;
    addcarryxU32(&x172, &x173, x171, x101, x147);
    var x174: u32 = undefined;
    var x175: u1 = undefined;
    addcarryxU32(&x174, &x175, x173, x103, x149);
    var x176: u32 = undefined;
    var x177: u1 = undefined;
    addcarryxU32(&x176, &x177, x175, x105, x151);
    var x178: u32 = undefined;
    var x179: u1 = undefined;
    addcarryxU32(&x178, &x179, x177, x107, x153);
    var x180: u32 = undefined;
    var x181: u1 = undefined;
    addcarryxU32(&x180, &x181, x179, x109, x155);
    var x182: u32 = undefined;
    var x183: u1 = undefined;
    addcarryxU32(&x182, &x183, x181, x111, x157);
    var x184: u32 = undefined;
    var x185: u1 = undefined;
    addcarryxU32(&x184, &x185, x183, x113, x159);
    var x186: u32 = undefined;
    var x187: u1 = undefined;
    addcarryxU32(&x186, &x187, x185, x115, x161);
    var x188: u32 = undefined;
    var x189: u1 = undefined;
    addcarryxU32(&x188, &x189, x187, x117, x163);
    var x190: u32 = undefined;
    var x191: u1 = undefined;
    addcarryxU32(&x190, &x191, x189, x119, x165);
    var x192: u32 = undefined;
    var x193: u1 = undefined;
    addcarryxU32(&x192, &x193, x191, x121, x167);
    var x194: u32 = undefined;
    var x195: u1 = undefined;
    addcarryxU32(&x194, &x195, x193, cast(u32, x122), x169);
    var x196: u32 = undefined;
    var x197: u32 = undefined;
    mulxU32(&x196, &x197, x170, 0xffffffff);
    var x198: u32 = undefined;
    var x199: u32 = undefined;
    mulxU32(&x198, &x199, x170, 0xffffffff);
    var x200: u32 = undefined;
    var x201: u32 = undefined;
    mulxU32(&x200, &x201, x170, 0xffffffff);
    var x202: u32 = undefined;
    var x203: u32 = undefined;
    mulxU32(&x202, &x203, x170, 0xffffffff);
    var x204: u32 = undefined;
    var x205: u32 = undefined;
    mulxU32(&x204, &x205, x170, 0xffffffff);
    var x206: u32 = undefined;
    var x207: u32 = undefined;
    mulxU32(&x206, &x207, x170, 0xffffffff);
    var x208: u32 = undefined;
    var x209: u32 = undefined;
    mulxU32(&x208, &x209, x170, 0xffffffff);
    var x210: u32 = undefined;
    var x211: u32 = undefined;
    mulxU32(&x210, &x211, x170, 0xfffffffe);
    var x212: u32 = undefined;
    var x213: u32 = undefined;
    mulxU32(&x212, &x213, x170, 0xffffffff);
    var x214: u32 = undefined;
    var x215: u32 = undefined;
    mulxU32(&x214, &x215, x170, 0xffffffff);
    var x216: u32 = undefined;
    var x217: u1 = undefined;
    addcarryxU32(&x216, &x217, 0x0, x213, x210);
    var x218: u32 = undefined;
    var x219: u1 = undefined;
    addcarryxU32(&x218, &x219, x217, x211, x208);
    var x220: u32 = undefined;
    var x221: u1 = undefined;
    addcarryxU32(&x220, &x221, x219, x209, x206);
    var x222: u32 = undefined;
    var x223: u1 = undefined;
    addcarryxU32(&x222, &x223, x221, x207, x204);
    var x224: u32 = undefined;
    var x225: u1 = undefined;
    addcarryxU32(&x224, &x225, x223, x205, x202);
    var x226: u32 = undefined;
    var x227: u1 = undefined;
    addcarryxU32(&x226, &x227, x225, x203, x200);
    var x228: u32 = undefined;
    var x229: u1 = undefined;
    addcarryxU32(&x228, &x229, x227, x201, x198);
    var x230: u32 = undefined;
    var x231: u1 = undefined;
    addcarryxU32(&x230, &x231, x229, x199, x196);
    const x232 = (cast(u32, x231) + x197);
    var x233: u32 = undefined;
    var x234: u1 = undefined;
    addcarryxU32(&x233, &x234, 0x0, x170, x214);
    var x235: u32 = undefined;
    var x236: u1 = undefined;
    addcarryxU32(&x235, &x236, x234, x172, x215);
    var x237: u32 = undefined;
    var x238: u1 = undefined;
    addcarryxU32(&x237, &x238, x236, x174, cast(u32, 0x0));
    var x239: u32 = undefined;
    var x240: u1 = undefined;
    addcarryxU32(&x239, &x240, x238, x176, x212);
    var x241: u32 = undefined;
    var x242: u1 = undefined;
    addcarryxU32(&x241, &x242, x240, x178, x216);
    var x243: u32 = undefined;
    var x244: u1 = undefined;
    addcarryxU32(&x243, &x244, x242, x180, x218);
    var x245: u32 = undefined;
    var x246: u1 = undefined;
    addcarryxU32(&x245, &x246, x244, x182, x220);
    var x247: u32 = undefined;
    var x248: u1 = undefined;
    addcarryxU32(&x247, &x248, x246, x184, x222);
    var x249: u32 = undefined;
    var x250: u1 = undefined;
    addcarryxU32(&x249, &x250, x248, x186, x224);
    var x251: u32 = undefined;
    var x252: u1 = undefined;
    addcarryxU32(&x251, &x252, x250, x188, x226);
    var x253: u32 = undefined;
    var x254: u1 = undefined;
    addcarryxU32(&x253, &x254, x252, x190, x228);
    var x255: u32 = undefined;
    var x256: u1 = undefined;
    addcarryxU32(&x255, &x256, x254, x192, x230);
    var x257: u32 = undefined;
    var x258: u1 = undefined;
    addcarryxU32(&x257, &x258, x256, x194, x232);
    const x259 = (cast(u32, x258) + cast(u32, x195));
    var x260: u32 = undefined;
    var x261: u32 = undefined;
    mulxU32(&x260, &x261, x2, (arg2[11]));
    var x262: u32 = undefined;
    var x263: u32 = undefined;
    mulxU32(&x262, &x263, x2, (arg2[10]));
    var x264: u32 = undefined;
    var x265: u32 = undefined;
    mulxU32(&x264, &x265, x2, (arg2[9]));
    var x266: u32 = undefined;
    var x267: u32 = undefined;
    mulxU32(&x266, &x267, x2, (arg2[8]));
    var x268: u32 = undefined;
    var x269: u32 = undefined;
    mulxU32(&x268, &x269, x2, (arg2[7]));
    var x270: u32 = undefined;
    var x271: u32 = undefined;
    mulxU32(&x270, &x271, x2, (arg2[6]));
    var x272: u32 = undefined;
    var x273: u32 = undefined;
    mulxU32(&x272, &x273, x2, (arg2[5]));
    var x274: u32 = undefined;
    var x275: u32 = undefined;
    mulxU32(&x274, &x275, x2, (arg2[4]));
    var x276: u32 = undefined;
    var x277: u32 = undefined;
    mulxU32(&x276, &x277, x2, (arg2[3]));
    var x278: u32 = undefined;
    var x279: u32 = undefined;
    mulxU32(&x278, &x279, x2, (arg2[2]));
    var x280: u32 = undefined;
    var x281: u32 = undefined;
    mulxU32(&x280, &x281, x2, (arg2[1]));
    var x282: u32 = undefined;
    var x283: u32 = undefined;
    mulxU32(&x282, &x283, x2, (arg2[0]));
    var x284: u32 = undefined;
    var x285: u1 = undefined;
    addcarryxU32(&x284, &x285, 0x0, x283, x280);
    var x286: u32 = undefined;
    var x287: u1 = undefined;
    addcarryxU32(&x286, &x287, x285, x281, x278);
    var x288: u32 = undefined;
    var x289: u1 = undefined;
    addcarryxU32(&x288, &x289, x287, x279, x276);
    var x290: u32 = undefined;
    var x291: u1 = undefined;
    addcarryxU32(&x290, &x291, x289, x277, x274);
    var x292: u32 = undefined;
    var x293: u1 = undefined;
    addcarryxU32(&x292, &x293, x291, x275, x272);
    var x294: u32 = undefined;
    var x295: u1 = undefined;
    addcarryxU32(&x294, &x295, x293, x273, x270);
    var x296: u32 = undefined;
    var x297: u1 = undefined;
    addcarryxU32(&x296, &x297, x295, x271, x268);
    var x298: u32 = undefined;
    var x299: u1 = undefined;
    addcarryxU32(&x298, &x299, x297, x269, x266);
    var x300: u32 = undefined;
    var x301: u1 = undefined;
    addcarryxU32(&x300, &x301, x299, x267, x264);
    var x302: u32 = undefined;
    var x303: u1 = undefined;
    addcarryxU32(&x302, &x303, x301, x265, x262);
    var x304: u32 = undefined;
    var x305: u1 = undefined;
    addcarryxU32(&x304, &x305, x303, x263, x260);
    const x306 = (cast(u32, x305) + x261);
    var x307: u32 = undefined;
    var x308: u1 = undefined;
    addcarryxU32(&x307, &x308, 0x0, x235, x282);
    var x309: u32 = undefined;
    var x310: u1 = undefined;
    addcarryxU32(&x309, &x310, x308, x237, x284);
    var x311: u32 = undefined;
    var x312: u1 = undefined;
    addcarryxU32(&x311, &x312, x310, x239, x286);
    var x313: u32 = undefined;
    var x314: u1 = undefined;
    addcarryxU32(&x313, &x314, x312, x241, x288);
    var x315: u32 = undefined;
    var x316: u1 = undefined;
    addcarryxU32(&x315, &x316, x314, x243, x290);
    var x317: u32 = undefined;
    var x318: u1 = undefined;
    addcarryxU32(&x317, &x318, x316, x245, x292);
    var x319: u32 = undefined;
    var x320: u1 = undefined;
    addcarryxU32(&x319, &x320, x318, x247, x294);
    var x321: u32 = undefined;
    var x322: u1 = undefined;
    addcarryxU32(&x321, &x322, x320, x249, x296);
    var x323: u32 = undefined;
    var x324: u1 = undefined;
    addcarryxU32(&x323, &x324, x322, x251, x298);
    var x325: u32 = undefined;
    var x326: u1 = undefined;
    addcarryxU32(&x325, &x326, x324, x253, x300);
    var x327: u32 = undefined;
    var x328: u1 = undefined;
    addcarryxU32(&x327, &x328, x326, x255, x302);
    var x329: u32 = undefined;
    var x330: u1 = undefined;
    addcarryxU32(&x329, &x330, x328, x257, x304);
    var x331: u32 = undefined;
    var x332: u1 = undefined;
    addcarryxU32(&x331, &x332, x330, x259, x306);
    var x333: u32 = undefined;
    var x334: u32 = undefined;
    mulxU32(&x333, &x334, x307, 0xffffffff);
    var x335: u32 = undefined;
    var x336: u32 = undefined;
    mulxU32(&x335, &x336, x307, 0xffffffff);
    var x337: u32 = undefined;
    var x338: u32 = undefined;
    mulxU32(&x337, &x338, x307, 0xffffffff);
    var x339: u32 = undefined;
    var x340: u32 = undefined;
    mulxU32(&x339, &x340, x307, 0xffffffff);
    var x341: u32 = undefined;
    var x342: u32 = undefined;
    mulxU32(&x341, &x342, x307, 0xffffffff);
    var x343: u32 = undefined;
    var x344: u32 = undefined;
    mulxU32(&x343, &x344, x307, 0xffffffff);
    var x345: u32 = undefined;
    var x346: u32 = undefined;
    mulxU32(&x345, &x346, x307, 0xffffffff);
    var x347: u32 = undefined;
    var x348: u32 = undefined;
    mulxU32(&x347, &x348, x307, 0xfffffffe);
    var x349: u32 = undefined;
    var x350: u32 = undefined;
    mulxU32(&x349, &x350, x307, 0xffffffff);
    var x351: u32 = undefined;
    var x352: u32 = undefined;
    mulxU32(&x351, &x352, x307, 0xffffffff);
    var x353: u32 = undefined;
    var x354: u1 = undefined;
    addcarryxU32(&x353, &x354, 0x0, x350, x347);
    var x355: u32 = undefined;
    var x356: u1 = undefined;
    addcarryxU32(&x355, &x356, x354, x348, x345);
    var x357: u32 = undefined;
    var x358: u1 = undefined;
    addcarryxU32(&x357, &x358, x356, x346, x343);
    var x359: u32 = undefined;
    var x360: u1 = undefined;
    addcarryxU32(&x359, &x360, x358, x344, x341);
    var x361: u32 = undefined;
    var x362: u1 = undefined;
    addcarryxU32(&x361, &x362, x360, x342, x339);
    var x363: u32 = undefined;
    var x364: u1 = undefined;
    addcarryxU32(&x363, &x364, x362, x340, x337);
    var x365: u32 = undefined;
    var x366: u1 = undefined;
    addcarryxU32(&x365, &x366, x364, x338, x335);
    var x367: u32 = undefined;
    var x368: u1 = undefined;
    addcarryxU32(&x367, &x368, x366, x336, x333);
    const x369 = (cast(u32, x368) + x334);
    var x370: u32 = undefined;
    var x371: u1 = undefined;
    addcarryxU32(&x370, &x371, 0x0, x307, x351);
    var x372: u32 = undefined;
    var x373: u1 = undefined;
    addcarryxU32(&x372, &x373, x371, x309, x352);
    var x374: u32 = undefined;
    var x375: u1 = undefined;
    addcarryxU32(&x374, &x375, x373, x311, cast(u32, 0x0));
    var x376: u32 = undefined;
    var x377: u1 = undefined;
    addcarryxU32(&x376, &x377, x375, x313, x349);
    var x378: u32 = undefined;
    var x379: u1 = undefined;
    addcarryxU32(&x378, &x379, x377, x315, x353);
    var x380: u32 = undefined;
    var x381: u1 = undefined;
    addcarryxU32(&x380, &x381, x379, x317, x355);
    var x382: u32 = undefined;
    var x383: u1 = undefined;
    addcarryxU32(&x382, &x383, x381, x319, x357);
    var x384: u32 = undefined;
    var x385: u1 = undefined;
    addcarryxU32(&x384, &x385, x383, x321, x359);
    var x386: u32 = undefined;
    var x387: u1 = undefined;
    addcarryxU32(&x386, &x387, x385, x323, x361);
    var x388: u32 = undefined;
    var x389: u1 = undefined;
    addcarryxU32(&x388, &x389, x387, x325, x363);
    var x390: u32 = undefined;
    var x391: u1 = undefined;
    addcarryxU32(&x390, &x391, x389, x327, x365);
    var x392: u32 = undefined;
    var x393: u1 = undefined;
    addcarryxU32(&x392, &x393, x391, x329, x367);
    var x394: u32 = undefined;
    var x395: u1 = undefined;
    addcarryxU32(&x394, &x395, x393, x331, x369);
    const x396 = (cast(u32, x395) + cast(u32, x332));
    var x397: u32 = undefined;
    var x398: u32 = undefined;
    mulxU32(&x397, &x398, x3, (arg2[11]));
    var x399: u32 = undefined;
    var x400: u32 = undefined;
    mulxU32(&x399, &x400, x3, (arg2[10]));
    var x401: u32 = undefined;
    var x402: u32 = undefined;
    mulxU32(&x401, &x402, x3, (arg2[9]));
    var x403: u32 = undefined;
    var x404: u32 = undefined;
    mulxU32(&x403, &x404, x3, (arg2[8]));
    var x405: u32 = undefined;
    var x406: u32 = undefined;
    mulxU32(&x405, &x406, x3, (arg2[7]));
    var x407: u32 = undefined;
    var x408: u32 = undefined;
    mulxU32(&x407, &x408, x3, (arg2[6]));
    var x409: u32 = undefined;
    var x410: u32 = undefined;
    mulxU32(&x409, &x410, x3, (arg2[5]));
    var x411: u32 = undefined;
    var x412: u32 = undefined;
    mulxU32(&x411, &x412, x3, (arg2[4]));
    var x413: u32 = undefined;
    var x414: u32 = undefined;
    mulxU32(&x413, &x414, x3, (arg2[3]));
    var x415: u32 = undefined;
    var x416: u32 = undefined;
    mulxU32(&x415, &x416, x3, (arg2[2]));
    var x417: u32 = undefined;
    var x418: u32 = undefined;
    mulxU32(&x417, &x418, x3, (arg2[1]));
    var x419: u32 = undefined;
    var x420: u32 = undefined;
    mulxU32(&x419, &x420, x3, (arg2[0]));
    var x421: u32 = undefined;
    var x422: u1 = undefined;
    addcarryxU32(&x421, &x422, 0x0, x420, x417);
    var x423: u32 = undefined;
    var x424: u1 = undefined;
    addcarryxU32(&x423, &x424, x422, x418, x415);
    var x425: u32 = undefined;
    var x426: u1 = undefined;
    addcarryxU32(&x425, &x426, x424, x416, x413);
    var x427: u32 = undefined;
    var x428: u1 = undefined;
    addcarryxU32(&x427, &x428, x426, x414, x411);
    var x429: u32 = undefined;
    var x430: u1 = undefined;
    addcarryxU32(&x429, &x430, x428, x412, x409);
    var x431: u32 = undefined;
    var x432: u1 = undefined;
    addcarryxU32(&x431, &x432, x430, x410, x407);
    var x433: u32 = undefined;
    var x434: u1 = undefined;
    addcarryxU32(&x433, &x434, x432, x408, x405);
    var x435: u32 = undefined;
    var x436: u1 = undefined;
    addcarryxU32(&x435, &x436, x434, x406, x403);
    var x437: u32 = undefined;
    var x438: u1 = undefined;
    addcarryxU32(&x437, &x438, x436, x404, x401);
    var x439: u32 = undefined;
    var x440: u1 = undefined;
    addcarryxU32(&x439, &x440, x438, x402, x399);
    var x441: u32 = undefined;
    var x442: u1 = undefined;
    addcarryxU32(&x441, &x442, x440, x400, x397);
    const x443 = (cast(u32, x442) + x398);
    var x444: u32 = undefined;
    var x445: u1 = undefined;
    addcarryxU32(&x444, &x445, 0x0, x372, x419);
    var x446: u32 = undefined;
    var x447: u1 = undefined;
    addcarryxU32(&x446, &x447, x445, x374, x421);
    var x448: u32 = undefined;
    var x449: u1 = undefined;
    addcarryxU32(&x448, &x449, x447, x376, x423);
    var x450: u32 = undefined;
    var x451: u1 = undefined;
    addcarryxU32(&x450, &x451, x449, x378, x425);
    var x452: u32 = undefined;
    var x453: u1 = undefined;
    addcarryxU32(&x452, &x453, x451, x380, x427);
    var x454: u32 = undefined;
    var x455: u1 = undefined;
    addcarryxU32(&x454, &x455, x453, x382, x429);
    var x456: u32 = undefined;
    var x457: u1 = undefined;
    addcarryxU32(&x456, &x457, x455, x384, x431);
    var x458: u32 = undefined;
    var x459: u1 = undefined;
    addcarryxU32(&x458, &x459, x457, x386, x433);
    var x460: u32 = undefined;
    var x461: u1 = undefined;
    addcarryxU32(&x460, &x461, x459, x388, x435);
    var x462: u32 = undefined;
    var x463: u1 = undefined;
    addcarryxU32(&x462, &x463, x461, x390, x437);
    var x464: u32 = undefined;
    var x465: u1 = undefined;
    addcarryxU32(&x464, &x465, x463, x392, x439);
    var x466: u32 = undefined;
    var x467: u1 = undefined;
    addcarryxU32(&x466, &x467, x465, x394, x441);
    var x468: u32 = undefined;
    var x469: u1 = undefined;
    addcarryxU32(&x468, &x469, x467, x396, x443);
    var x470: u32 = undefined;
    var x471: u32 = undefined;
    mulxU32(&x470, &x471, x444, 0xffffffff);
    var x472: u32 = undefined;
    var x473: u32 = undefined;
    mulxU32(&x472, &x473, x444, 0xffffffff);
    var x474: u32 = undefined;
    var x475: u32 = undefined;
    mulxU32(&x474, &x475, x444, 0xffffffff);
    var x476: u32 = undefined;
    var x477: u32 = undefined;
    mulxU32(&x476, &x477, x444, 0xffffffff);
    var x478: u32 = undefined;
    var x479: u32 = undefined;
    mulxU32(&x478, &x479, x444, 0xffffffff);
    var x480: u32 = undefined;
    var x481: u32 = undefined;
    mulxU32(&x480, &x481, x444, 0xffffffff);
    var x482: u32 = undefined;
    var x483: u32 = undefined;
    mulxU32(&x482, &x483, x444, 0xffffffff);
    var x484: u32 = undefined;
    var x485: u32 = undefined;
    mulxU32(&x484, &x485, x444, 0xfffffffe);
    var x486: u32 = undefined;
    var x487: u32 = undefined;
    mulxU32(&x486, &x487, x444, 0xffffffff);
    var x488: u32 = undefined;
    var x489: u32 = undefined;
    mulxU32(&x488, &x489, x444, 0xffffffff);
    var x490: u32 = undefined;
    var x491: u1 = undefined;
    addcarryxU32(&x490, &x491, 0x0, x487, x484);
    var x492: u32 = undefined;
    var x493: u1 = undefined;
    addcarryxU32(&x492, &x493, x491, x485, x482);
    var x494: u32 = undefined;
    var x495: u1 = undefined;
    addcarryxU32(&x494, &x495, x493, x483, x480);
    var x496: u32 = undefined;
    var x497: u1 = undefined;
    addcarryxU32(&x496, &x497, x495, x481, x478);
    var x498: u32 = undefined;
    var x499: u1 = undefined;
    addcarryxU32(&x498, &x499, x497, x479, x476);
    var x500: u32 = undefined;
    var x501: u1 = undefined;
    addcarryxU32(&x500, &x501, x499, x477, x474);
    var x502: u32 = undefined;
    var x503: u1 = undefined;
    addcarryxU32(&x502, &x503, x501, x475, x472);
    var x504: u32 = undefined;
    var x505: u1 = undefined;
    addcarryxU32(&x504, &x505, x503, x473, x470);
    const x506 = (cast(u32, x505) + x471);
    var x507: u32 = undefined;
    var x508: u1 = undefined;
    addcarryxU32(&x507, &x508, 0x0, x444, x488);
    var x509: u32 = undefined;
    var x510: u1 = undefined;
    addcarryxU32(&x509, &x510, x508, x446, x489);
    var x511: u32 = undefined;
    var x512: u1 = undefined;
    addcarryxU32(&x511, &x512, x510, x448, cast(u32, 0x0));
    var x513: u32 = undefined;
    var x514: u1 = undefined;
    addcarryxU32(&x513, &x514, x512, x450, x486);
    var x515: u32 = undefined;
    var x516: u1 = undefined;
    addcarryxU32(&x515, &x516, x514, x452, x490);
    var x517: u32 = undefined;
    var x518: u1 = undefined;
    addcarryxU32(&x517, &x518, x516, x454, x492);
    var x519: u32 = undefined;
    var x520: u1 = undefined;
    addcarryxU32(&x519, &x520, x518, x456, x494);
    var x521: u32 = undefined;
    var x522: u1 = undefined;
    addcarryxU32(&x521, &x522, x520, x458, x496);
    var x523: u32 = undefined;
    var x524: u1 = undefined;
    addcarryxU32(&x523, &x524, x522, x460, x498);
    var x525: u32 = undefined;
    var x526: u1 = undefined;
    addcarryxU32(&x525, &x526, x524, x462, x500);
    var x527: u32 = undefined;
    var x528: u1 = undefined;
    addcarryxU32(&x527, &x528, x526, x464, x502);
    var x529: u32 = undefined;
    var x530: u1 = undefined;
    addcarryxU32(&x529, &x530, x528, x466, x504);
    var x531: u32 = undefined;
    var x532: u1 = undefined;
    addcarryxU32(&x531, &x532, x530, x468, x506);
    const x533 = (cast(u32, x532) + cast(u32, x469));
    var x534: u32 = undefined;
    var x535: u32 = undefined;
    mulxU32(&x534, &x535, x4, (arg2[11]));
    var x536: u32 = undefined;
    var x537: u32 = undefined;
    mulxU32(&x536, &x537, x4, (arg2[10]));
    var x538: u32 = undefined;
    var x539: u32 = undefined;
    mulxU32(&x538, &x539, x4, (arg2[9]));
    var x540: u32 = undefined;
    var x541: u32 = undefined;
    mulxU32(&x540, &x541, x4, (arg2[8]));
    var x542: u32 = undefined;
    var x543: u32 = undefined;
    mulxU32(&x542, &x543, x4, (arg2[7]));
    var x544: u32 = undefined;
    var x545: u32 = undefined;
    mulxU32(&x544, &x545, x4, (arg2[6]));
    var x546: u32 = undefined;
    var x547: u32 = undefined;
    mulxU32(&x546, &x547, x4, (arg2[5]));
    var x548: u32 = undefined;
    var x549: u32 = undefined;
    mulxU32(&x548, &x549, x4, (arg2[4]));
    var x550: u32 = undefined;
    var x551: u32 = undefined;
    mulxU32(&x550, &x551, x4, (arg2[3]));
    var x552: u32 = undefined;
    var x553: u32 = undefined;
    mulxU32(&x552, &x553, x4, (arg2[2]));
    var x554: u32 = undefined;
    var x555: u32 = undefined;
    mulxU32(&x554, &x555, x4, (arg2[1]));
    var x556: u32 = undefined;
    var x557: u32 = undefined;
    mulxU32(&x556, &x557, x4, (arg2[0]));
    var x558: u32 = undefined;
    var x559: u1 = undefined;
    addcarryxU32(&x558, &x559, 0x0, x557, x554);
    var x560: u32 = undefined;
    var x561: u1 = undefined;
    addcarryxU32(&x560, &x561, x559, x555, x552);
    var x562: u32 = undefined;
    var x563: u1 = undefined;
    addcarryxU32(&x562, &x563, x561, x553, x550);
    var x564: u32 = undefined;
    var x565: u1 = undefined;
    addcarryxU32(&x564, &x565, x563, x551, x548);
    var x566: u32 = undefined;
    var x567: u1 = undefined;
    addcarryxU32(&x566, &x567, x565, x549, x546);
    var x568: u32 = undefined;
    var x569: u1 = undefined;
    addcarryxU32(&x568, &x569, x567, x547, x544);
    var x570: u32 = undefined;
    var x571: u1 = undefined;
    addcarryxU32(&x570, &x571, x569, x545, x542);
    var x572: u32 = undefined;
    var x573: u1 = undefined;
    addcarryxU32(&x572, &x573, x571, x543, x540);
    var x574: u32 = undefined;
    var x575: u1 = undefined;
    addcarryxU32(&x574, &x575, x573, x541, x538);
    var x576: u32 = undefined;
    var x577: u1 = undefined;
    addcarryxU32(&x576, &x577, x575, x539, x536);
    var x578: u32 = undefined;
    var x579: u1 = undefined;
    addcarryxU32(&x578, &x579, x577, x537, x534);
    const x580 = (cast(u32, x579) + x535);
    var x581: u32 = undefined;
    var x582: u1 = undefined;
    addcarryxU32(&x581, &x582, 0x0, x509, x556);
    var x583: u32 = undefined;
    var x584: u1 = undefined;
    addcarryxU32(&x583, &x584, x582, x511, x558);
    var x585: u32 = undefined;
    var x586: u1 = undefined;
    addcarryxU32(&x585, &x586, x584, x513, x560);
    var x587: u32 = undefined;
    var x588: u1 = undefined;
    addcarryxU32(&x587, &x588, x586, x515, x562);
    var x589: u32 = undefined;
    var x590: u1 = undefined;
    addcarryxU32(&x589, &x590, x588, x517, x564);
    var x591: u32 = undefined;
    var x592: u1 = undefined;
    addcarryxU32(&x591, &x592, x590, x519, x566);
    var x593: u32 = undefined;
    var x594: u1 = undefined;
    addcarryxU32(&x593, &x594, x592, x521, x568);
    var x595: u32 = undefined;
    var x596: u1 = undefined;
    addcarryxU32(&x595, &x596, x594, x523, x570);
    var x597: u32 = undefined;
    var x598: u1 = undefined;
    addcarryxU32(&x597, &x598, x596, x525, x572);
    var x599: u32 = undefined;
    var x600: u1 = undefined;
    addcarryxU32(&x599, &x600, x598, x527, x574);
    var x601: u32 = undefined;
    var x602: u1 = undefined;
    addcarryxU32(&x601, &x602, x600, x529, x576);
    var x603: u32 = undefined;
    var x604: u1 = undefined;
    addcarryxU32(&x603, &x604, x602, x531, x578);
    var x605: u32 = undefined;
    var x606: u1 = undefined;
    addcarryxU32(&x605, &x606, x604, x533, x580);
    var x607: u32 = undefined;
    var x608: u32 = undefined;
    mulxU32(&x607, &x608, x581, 0xffffffff);
    var x609: u32 = undefined;
    var x610: u32 = undefined;
    mulxU32(&x609, &x610, x581, 0xffffffff);
    var x611: u32 = undefined;
    var x612: u32 = undefined;
    mulxU32(&x611, &x612, x581, 0xffffffff);
    var x613: u32 = undefined;
    var x614: u32 = undefined;
    mulxU32(&x613, &x614, x581, 0xffffffff);
    var x615: u32 = undefined;
    var x616: u32 = undefined;
    mulxU32(&x615, &x616, x581, 0xffffffff);
    var x617: u32 = undefined;
    var x618: u32 = undefined;
    mulxU32(&x617, &x618, x581, 0xffffffff);
    var x619: u32 = undefined;
    var x620: u32 = undefined;
    mulxU32(&x619, &x620, x581, 0xffffffff);
    var x621: u32 = undefined;
    var x622: u32 = undefined;
    mulxU32(&x621, &x622, x581, 0xfffffffe);
    var x623: u32 = undefined;
    var x624: u32 = undefined;
    mulxU32(&x623, &x624, x581, 0xffffffff);
    var x625: u32 = undefined;
    var x626: u32 = undefined;
    mulxU32(&x625, &x626, x581, 0xffffffff);
    var x627: u32 = undefined;
    var x628: u1 = undefined;
    addcarryxU32(&x627, &x628, 0x0, x624, x621);
    var x629: u32 = undefined;
    var x630: u1 = undefined;
    addcarryxU32(&x629, &x630, x628, x622, x619);
    var x631: u32 = undefined;
    var x632: u1 = undefined;
    addcarryxU32(&x631, &x632, x630, x620, x617);
    var x633: u32 = undefined;
    var x634: u1 = undefined;
    addcarryxU32(&x633, &x634, x632, x618, x615);
    var x635: u32 = undefined;
    var x636: u1 = undefined;
    addcarryxU32(&x635, &x636, x634, x616, x613);
    var x637: u32 = undefined;
    var x638: u1 = undefined;
    addcarryxU32(&x637, &x638, x636, x614, x611);
    var x639: u32 = undefined;
    var x640: u1 = undefined;
    addcarryxU32(&x639, &x640, x638, x612, x609);
    var x641: u32 = undefined;
    var x642: u1 = undefined;
    addcarryxU32(&x641, &x642, x640, x610, x607);
    const x643 = (cast(u32, x642) + x608);
    var x644: u32 = undefined;
    var x645: u1 = undefined;
    addcarryxU32(&x644, &x645, 0x0, x581, x625);
    var x646: u32 = undefined;
    var x647: u1 = undefined;
    addcarryxU32(&x646, &x647, x645, x583, x626);
    var x648: u32 = undefined;
    var x649: u1 = undefined;
    addcarryxU32(&x648, &x649, x647, x585, cast(u32, 0x0));
    var x650: u32 = undefined;
    var x651: u1 = undefined;
    addcarryxU32(&x650, &x651, x649, x587, x623);
    var x652: u32 = undefined;
    var x653: u1 = undefined;
    addcarryxU32(&x652, &x653, x651, x589, x627);
    var x654: u32 = undefined;
    var x655: u1 = undefined;
    addcarryxU32(&x654, &x655, x653, x591, x629);
    var x656: u32 = undefined;
    var x657: u1 = undefined;
    addcarryxU32(&x656, &x657, x655, x593, x631);
    var x658: u32 = undefined;
    var x659: u1 = undefined;
    addcarryxU32(&x658, &x659, x657, x595, x633);
    var x660: u32 = undefined;
    var x661: u1 = undefined;
    addcarryxU32(&x660, &x661, x659, x597, x635);
    var x662: u32 = undefined;
    var x663: u1 = undefined;
    addcarryxU32(&x662, &x663, x661, x599, x637);
    var x664: u32 = undefined;
    var x665: u1 = undefined;
    addcarryxU32(&x664, &x665, x663, x601, x639);
    var x666: u32 = undefined;
    var x667: u1 = undefined;
    addcarryxU32(&x666, &x667, x665, x603, x641);
    var x668: u32 = undefined;
    var x669: u1 = undefined;
    addcarryxU32(&x668, &x669, x667, x605, x643);
    const x670 = (cast(u32, x669) + cast(u32, x606));
    var x671: u32 = undefined;
    var x672: u32 = undefined;
    mulxU32(&x671, &x672, x5, (arg2[11]));
    var x673: u32 = undefined;
    var x674: u32 = undefined;
    mulxU32(&x673, &x674, x5, (arg2[10]));
    var x675: u32 = undefined;
    var x676: u32 = undefined;
    mulxU32(&x675, &x676, x5, (arg2[9]));
    var x677: u32 = undefined;
    var x678: u32 = undefined;
    mulxU32(&x677, &x678, x5, (arg2[8]));
    var x679: u32 = undefined;
    var x680: u32 = undefined;
    mulxU32(&x679, &x680, x5, (arg2[7]));
    var x681: u32 = undefined;
    var x682: u32 = undefined;
    mulxU32(&x681, &x682, x5, (arg2[6]));
    var x683: u32 = undefined;
    var x684: u32 = undefined;
    mulxU32(&x683, &x684, x5, (arg2[5]));
    var x685: u32 = undefined;
    var x686: u32 = undefined;
    mulxU32(&x685, &x686, x5, (arg2[4]));
    var x687: u32 = undefined;
    var x688: u32 = undefined;
    mulxU32(&x687, &x688, x5, (arg2[3]));
    var x689: u32 = undefined;
    var x690: u32 = undefined;
    mulxU32(&x689, &x690, x5, (arg2[2]));
    var x691: u32 = undefined;
    var x692: u32 = undefined;
    mulxU32(&x691, &x692, x5, (arg2[1]));
    var x693: u32 = undefined;
    var x694: u32 = undefined;
    mulxU32(&x693, &x694, x5, (arg2[0]));
    var x695: u32 = undefined;
    var x696: u1 = undefined;
    addcarryxU32(&x695, &x696, 0x0, x694, x691);
    var x697: u32 = undefined;
    var x698: u1 = undefined;
    addcarryxU32(&x697, &x698, x696, x692, x689);
    var x699: u32 = undefined;
    var x700: u1 = undefined;
    addcarryxU32(&x699, &x700, x698, x690, x687);
    var x701: u32 = undefined;
    var x702: u1 = undefined;
    addcarryxU32(&x701, &x702, x700, x688, x685);
    var x703: u32 = undefined;
    var x704: u1 = undefined;
    addcarryxU32(&x703, &x704, x702, x686, x683);
    var x705: u32 = undefined;
    var x706: u1 = undefined;
    addcarryxU32(&x705, &x706, x704, x684, x681);
    var x707: u32 = undefined;
    var x708: u1 = undefined;
    addcarryxU32(&x707, &x708, x706, x682, x679);
    var x709: u32 = undefined;
    var x710: u1 = undefined;
    addcarryxU32(&x709, &x710, x708, x680, x677);
    var x711: u32 = undefined;
    var x712: u1 = undefined;
    addcarryxU32(&x711, &x712, x710, x678, x675);
    var x713: u32 = undefined;
    var x714: u1 = undefined;
    addcarryxU32(&x713, &x714, x712, x676, x673);
    var x715: u32 = undefined;
    var x716: u1 = undefined;
    addcarryxU32(&x715, &x716, x714, x674, x671);
    const x717 = (cast(u32, x716) + x672);
    var x718: u32 = undefined;
    var x719: u1 = undefined;
    addcarryxU32(&x718, &x719, 0x0, x646, x693);
    var x720: u32 = undefined;
    var x721: u1 = undefined;
    addcarryxU32(&x720, &x721, x719, x648, x695);
    var x722: u32 = undefined;
    var x723: u1 = undefined;
    addcarryxU32(&x722, &x723, x721, x650, x697);
    var x724: u32 = undefined;
    var x725: u1 = undefined;
    addcarryxU32(&x724, &x725, x723, x652, x699);
    var x726: u32 = undefined;
    var x727: u1 = undefined;
    addcarryxU32(&x726, &x727, x725, x654, x701);
    var x728: u32 = undefined;
    var x729: u1 = undefined;
    addcarryxU32(&x728, &x729, x727, x656, x703);
    var x730: u32 = undefined;
    var x731: u1 = undefined;
    addcarryxU32(&x730, &x731, x729, x658, x705);
    var x732: u32 = undefined;
    var x733: u1 = undefined;
    addcarryxU32(&x732, &x733, x731, x660, x707);
    var x734: u32 = undefined;
    var x735: u1 = undefined;
    addcarryxU32(&x734, &x735, x733, x662, x709);
    var x736: u32 = undefined;
    var x737: u1 = undefined;
    addcarryxU32(&x736, &x737, x735, x664, x711);
    var x738: u32 = undefined;
    var x739: u1 = undefined;
    addcarryxU32(&x738, &x739, x737, x666, x713);
    var x740: u32 = undefined;
    var x741: u1 = undefined;
    addcarryxU32(&x740, &x741, x739, x668, x715);
    var x742: u32 = undefined;
    var x743: u1 = undefined;
    addcarryxU32(&x742, &x743, x741, x670, x717);
    var x744: u32 = undefined;
    var x745: u32 = undefined;
    mulxU32(&x744, &x745, x718, 0xffffffff);
    var x746: u32 = undefined;
    var x747: u32 = undefined;
    mulxU32(&x746, &x747, x718, 0xffffffff);
    var x748: u32 = undefined;
    var x749: u32 = undefined;
    mulxU32(&x748, &x749, x718, 0xffffffff);
    var x750: u32 = undefined;
    var x751: u32 = undefined;
    mulxU32(&x750, &x751, x718, 0xffffffff);
    var x752: u32 = undefined;
    var x753: u32 = undefined;
    mulxU32(&x752, &x753, x718, 0xffffffff);
    var x754: u32 = undefined;
    var x755: u32 = undefined;
    mulxU32(&x754, &x755, x718, 0xffffffff);
    var x756: u32 = undefined;
    var x757: u32 = undefined;
    mulxU32(&x756, &x757, x718, 0xffffffff);
    var x758: u32 = undefined;
    var x759: u32 = undefined;
    mulxU32(&x758, &x759, x718, 0xfffffffe);
    var x760: u32 = undefined;
    var x761: u32 = undefined;
    mulxU32(&x760, &x761, x718, 0xffffffff);
    var x762: u32 = undefined;
    var x763: u32 = undefined;
    mulxU32(&x762, &x763, x718, 0xffffffff);
    var x764: u32 = undefined;
    var x765: u1 = undefined;
    addcarryxU32(&x764, &x765, 0x0, x761, x758);
    var x766: u32 = undefined;
    var x767: u1 = undefined;
    addcarryxU32(&x766, &x767, x765, x759, x756);
    var x768: u32 = undefined;
    var x769: u1 = undefined;
    addcarryxU32(&x768, &x769, x767, x757, x754);
    var x770: u32 = undefined;
    var x771: u1 = undefined;
    addcarryxU32(&x770, &x771, x769, x755, x752);
    var x772: u32 = undefined;
    var x773: u1 = undefined;
    addcarryxU32(&x772, &x773, x771, x753, x750);
    var x774: u32 = undefined;
    var x775: u1 = undefined;
    addcarryxU32(&x774, &x775, x773, x751, x748);
    var x776: u32 = undefined;
    var x777: u1 = undefined;
    addcarryxU32(&x776, &x777, x775, x749, x746);
    var x778: u32 = undefined;
    var x779: u1 = undefined;
    addcarryxU32(&x778, &x779, x777, x747, x744);
    const x780 = (cast(u32, x779) + x745);
    var x781: u32 = undefined;
    var x782: u1 = undefined;
    addcarryxU32(&x781, &x782, 0x0, x718, x762);
    var x783: u32 = undefined;
    var x784: u1 = undefined;
    addcarryxU32(&x783, &x784, x782, x720, x763);
    var x785: u32 = undefined;
    var x786: u1 = undefined;
    addcarryxU32(&x785, &x786, x784, x722, cast(u32, 0x0));
    var x787: u32 = undefined;
    var x788: u1 = undefined;
    addcarryxU32(&x787, &x788, x786, x724, x760);
    var x789: u32 = undefined;
    var x790: u1 = undefined;
    addcarryxU32(&x789, &x790, x788, x726, x764);
    var x791: u32 = undefined;
    var x792: u1 = undefined;
    addcarryxU32(&x791, &x792, x790, x728, x766);
    var x793: u32 = undefined;
    var x794: u1 = undefined;
    addcarryxU32(&x793, &x794, x792, x730, x768);
    var x795: u32 = undefined;
    var x796: u1 = undefined;
    addcarryxU32(&x795, &x796, x794, x732, x770);
    var x797: u32 = undefined;
    var x798: u1 = undefined;
    addcarryxU32(&x797, &x798, x796, x734, x772);
    var x799: u32 = undefined;
    var x800: u1 = undefined;
    addcarryxU32(&x799, &x800, x798, x736, x774);
    var x801: u32 = undefined;
    var x802: u1 = undefined;
    addcarryxU32(&x801, &x802, x800, x738, x776);
    var x803: u32 = undefined;
    var x804: u1 = undefined;
    addcarryxU32(&x803, &x804, x802, x740, x778);
    var x805: u32 = undefined;
    var x806: u1 = undefined;
    addcarryxU32(&x805, &x806, x804, x742, x780);
    const x807 = (cast(u32, x806) + cast(u32, x743));
    var x808: u32 = undefined;
    var x809: u32 = undefined;
    mulxU32(&x808, &x809, x6, (arg2[11]));
    var x810: u32 = undefined;
    var x811: u32 = undefined;
    mulxU32(&x810, &x811, x6, (arg2[10]));
    var x812: u32 = undefined;
    var x813: u32 = undefined;
    mulxU32(&x812, &x813, x6, (arg2[9]));
    var x814: u32 = undefined;
    var x815: u32 = undefined;
    mulxU32(&x814, &x815, x6, (arg2[8]));
    var x816: u32 = undefined;
    var x817: u32 = undefined;
    mulxU32(&x816, &x817, x6, (arg2[7]));
    var x818: u32 = undefined;
    var x819: u32 = undefined;
    mulxU32(&x818, &x819, x6, (arg2[6]));
    var x820: u32 = undefined;
    var x821: u32 = undefined;
    mulxU32(&x820, &x821, x6, (arg2[5]));
    var x822: u32 = undefined;
    var x823: u32 = undefined;
    mulxU32(&x822, &x823, x6, (arg2[4]));
    var x824: u32 = undefined;
    var x825: u32 = undefined;
    mulxU32(&x824, &x825, x6, (arg2[3]));
    var x826: u32 = undefined;
    var x827: u32 = undefined;
    mulxU32(&x826, &x827, x6, (arg2[2]));
    var x828: u32 = undefined;
    var x829: u32 = undefined;
    mulxU32(&x828, &x829, x6, (arg2[1]));
    var x830: u32 = undefined;
    var x831: u32 = undefined;
    mulxU32(&x830, &x831, x6, (arg2[0]));
    var x832: u32 = undefined;
    var x833: u1 = undefined;
    addcarryxU32(&x832, &x833, 0x0, x831, x828);
    var x834: u32 = undefined;
    var x835: u1 = undefined;
    addcarryxU32(&x834, &x835, x833, x829, x826);
    var x836: u32 = undefined;
    var x837: u1 = undefined;
    addcarryxU32(&x836, &x837, x835, x827, x824);
    var x838: u32 = undefined;
    var x839: u1 = undefined;
    addcarryxU32(&x838, &x839, x837, x825, x822);
    var x840: u32 = undefined;
    var x841: u1 = undefined;
    addcarryxU32(&x840, &x841, x839, x823, x820);
    var x842: u32 = undefined;
    var x843: u1 = undefined;
    addcarryxU32(&x842, &x843, x841, x821, x818);
    var x844: u32 = undefined;
    var x845: u1 = undefined;
    addcarryxU32(&x844, &x845, x843, x819, x816);
    var x846: u32 = undefined;
    var x847: u1 = undefined;
    addcarryxU32(&x846, &x847, x845, x817, x814);
    var x848: u32 = undefined;
    var x849: u1 = undefined;
    addcarryxU32(&x848, &x849, x847, x815, x812);
    var x850: u32 = undefined;
    var x851: u1 = undefined;
    addcarryxU32(&x850, &x851, x849, x813, x810);
    var x852: u32 = undefined;
    var x853: u1 = undefined;
    addcarryxU32(&x852, &x853, x851, x811, x808);
    const x854 = (cast(u32, x853) + x809);
    var x855: u32 = undefined;
    var x856: u1 = undefined;
    addcarryxU32(&x855, &x856, 0x0, x783, x830);
    var x857: u32 = undefined;
    var x858: u1 = undefined;
    addcarryxU32(&x857, &x858, x856, x785, x832);
    var x859: u32 = undefined;
    var x860: u1 = undefined;
    addcarryxU32(&x859, &x860, x858, x787, x834);
    var x861: u32 = undefined;
    var x862: u1 = undefined;
    addcarryxU32(&x861, &x862, x860, x789, x836);
    var x863: u32 = undefined;
    var x864: u1 = undefined;
    addcarryxU32(&x863, &x864, x862, x791, x838);
    var x865: u32 = undefined;
    var x866: u1 = undefined;
    addcarryxU32(&x865, &x866, x864, x793, x840);
    var x867: u32 = undefined;
    var x868: u1 = undefined;
    addcarryxU32(&x867, &x868, x866, x795, x842);
    var x869: u32 = undefined;
    var x870: u1 = undefined;
    addcarryxU32(&x869, &x870, x868, x797, x844);
    var x871: u32 = undefined;
    var x872: u1 = undefined;
    addcarryxU32(&x871, &x872, x870, x799, x846);
    var x873: u32 = undefined;
    var x874: u1 = undefined;
    addcarryxU32(&x873, &x874, x872, x801, x848);
    var x875: u32 = undefined;
    var x876: u1 = undefined;
    addcarryxU32(&x875, &x876, x874, x803, x850);
    var x877: u32 = undefined;
    var x878: u1 = undefined;
    addcarryxU32(&x877, &x878, x876, x805, x852);
    var x879: u32 = undefined;
    var x880: u1 = undefined;
    addcarryxU32(&x879, &x880, x878, x807, x854);
    var x881: u32 = undefined;
    var x882: u32 = undefined;
    mulxU32(&x881, &x882, x855, 0xffffffff);
    var x883: u32 = undefined;
    var x884: u32 = undefined;
    mulxU32(&x883, &x884, x855, 0xffffffff);
    var x885: u32 = undefined;
    var x886: u32 = undefined;
    mulxU32(&x885, &x886, x855, 0xffffffff);
    var x887: u32 = undefined;
    var x888: u32 = undefined;
    mulxU32(&x887, &x888, x855, 0xffffffff);
    var x889: u32 = undefined;
    var x890: u32 = undefined;
    mulxU32(&x889, &x890, x855, 0xffffffff);
    var x891: u32 = undefined;
    var x892: u32 = undefined;
    mulxU32(&x891, &x892, x855, 0xffffffff);
    var x893: u32 = undefined;
    var x894: u32 = undefined;
    mulxU32(&x893, &x894, x855, 0xffffffff);
    var x895: u32 = undefined;
    var x896: u32 = undefined;
    mulxU32(&x895, &x896, x855, 0xfffffffe);
    var x897: u32 = undefined;
    var x898: u32 = undefined;
    mulxU32(&x897, &x898, x855, 0xffffffff);
    var x899: u32 = undefined;
    var x900: u32 = undefined;
    mulxU32(&x899, &x900, x855, 0xffffffff);
    var x901: u32 = undefined;
    var x902: u1 = undefined;
    addcarryxU32(&x901, &x902, 0x0, x898, x895);
    var x903: u32 = undefined;
    var x904: u1 = undefined;
    addcarryxU32(&x903, &x904, x902, x896, x893);
    var x905: u32 = undefined;
    var x906: u1 = undefined;
    addcarryxU32(&x905, &x906, x904, x894, x891);
    var x907: u32 = undefined;
    var x908: u1 = undefined;
    addcarryxU32(&x907, &x908, x906, x892, x889);
    var x909: u32 = undefined;
    var x910: u1 = undefined;
    addcarryxU32(&x909, &x910, x908, x890, x887);
    var x911: u32 = undefined;
    var x912: u1 = undefined;
    addcarryxU32(&x911, &x912, x910, x888, x885);
    var x913: u32 = undefined;
    var x914: u1 = undefined;
    addcarryxU32(&x913, &x914, x912, x886, x883);
    var x915: u32 = undefined;
    var x916: u1 = undefined;
    addcarryxU32(&x915, &x916, x914, x884, x881);
    const x917 = (cast(u32, x916) + x882);
    var x918: u32 = undefined;
    var x919: u1 = undefined;
    addcarryxU32(&x918, &x919, 0x0, x855, x899);
    var x920: u32 = undefined;
    var x921: u1 = undefined;
    addcarryxU32(&x920, &x921, x919, x857, x900);
    var x922: u32 = undefined;
    var x923: u1 = undefined;
    addcarryxU32(&x922, &x923, x921, x859, cast(u32, 0x0));
    var x924: u32 = undefined;
    var x925: u1 = undefined;
    addcarryxU32(&x924, &x925, x923, x861, x897);
    var x926: u32 = undefined;
    var x927: u1 = undefined;
    addcarryxU32(&x926, &x927, x925, x863, x901);
    var x928: u32 = undefined;
    var x929: u1 = undefined;
    addcarryxU32(&x928, &x929, x927, x865, x903);
    var x930: u32 = undefined;
    var x931: u1 = undefined;
    addcarryxU32(&x930, &x931, x929, x867, x905);
    var x932: u32 = undefined;
    var x933: u1 = undefined;
    addcarryxU32(&x932, &x933, x931, x869, x907);
    var x934: u32 = undefined;
    var x935: u1 = undefined;
    addcarryxU32(&x934, &x935, x933, x871, x909);
    var x936: u32 = undefined;
    var x937: u1 = undefined;
    addcarryxU32(&x936, &x937, x935, x873, x911);
    var x938: u32 = undefined;
    var x939: u1 = undefined;
    addcarryxU32(&x938, &x939, x937, x875, x913);
    var x940: u32 = undefined;
    var x941: u1 = undefined;
    addcarryxU32(&x940, &x941, x939, x877, x915);
    var x942: u32 = undefined;
    var x943: u1 = undefined;
    addcarryxU32(&x942, &x943, x941, x879, x917);
    const x944 = (cast(u32, x943) + cast(u32, x880));
    var x945: u32 = undefined;
    var x946: u32 = undefined;
    mulxU32(&x945, &x946, x7, (arg2[11]));
    var x947: u32 = undefined;
    var x948: u32 = undefined;
    mulxU32(&x947, &x948, x7, (arg2[10]));
    var x949: u32 = undefined;
    var x950: u32 = undefined;
    mulxU32(&x949, &x950, x7, (arg2[9]));
    var x951: u32 = undefined;
    var x952: u32 = undefined;
    mulxU32(&x951, &x952, x7, (arg2[8]));
    var x953: u32 = undefined;
    var x954: u32 = undefined;
    mulxU32(&x953, &x954, x7, (arg2[7]));
    var x955: u32 = undefined;
    var x956: u32 = undefined;
    mulxU32(&x955, &x956, x7, (arg2[6]));
    var x957: u32 = undefined;
    var x958: u32 = undefined;
    mulxU32(&x957, &x958, x7, (arg2[5]));
    var x959: u32 = undefined;
    var x960: u32 = undefined;
    mulxU32(&x959, &x960, x7, (arg2[4]));
    var x961: u32 = undefined;
    var x962: u32 = undefined;
    mulxU32(&x961, &x962, x7, (arg2[3]));
    var x963: u32 = undefined;
    var x964: u32 = undefined;
    mulxU32(&x963, &x964, x7, (arg2[2]));
    var x965: u32 = undefined;
    var x966: u32 = undefined;
    mulxU32(&x965, &x966, x7, (arg2[1]));
    var x967: u32 = undefined;
    var x968: u32 = undefined;
    mulxU32(&x967, &x968, x7, (arg2[0]));
    var x969: u32 = undefined;
    var x970: u1 = undefined;
    addcarryxU32(&x969, &x970, 0x0, x968, x965);
    var x971: u32 = undefined;
    var x972: u1 = undefined;
    addcarryxU32(&x971, &x972, x970, x966, x963);
    var x973: u32 = undefined;
    var x974: u1 = undefined;
    addcarryxU32(&x973, &x974, x972, x964, x961);
    var x975: u32 = undefined;
    var x976: u1 = undefined;
    addcarryxU32(&x975, &x976, x974, x962, x959);
    var x977: u32 = undefined;
    var x978: u1 = undefined;
    addcarryxU32(&x977, &x978, x976, x960, x957);
    var x979: u32 = undefined;
    var x980: u1 = undefined;
    addcarryxU32(&x979, &x980, x978, x958, x955);
    var x981: u32 = undefined;
    var x982: u1 = undefined;
    addcarryxU32(&x981, &x982, x980, x956, x953);
    var x983: u32 = undefined;
    var x984: u1 = undefined;
    addcarryxU32(&x983, &x984, x982, x954, x951);
    var x985: u32 = undefined;
    var x986: u1 = undefined;
    addcarryxU32(&x985, &x986, x984, x952, x949);
    var x987: u32 = undefined;
    var x988: u1 = undefined;
    addcarryxU32(&x987, &x988, x986, x950, x947);
    var x989: u32 = undefined;
    var x990: u1 = undefined;
    addcarryxU32(&x989, &x990, x988, x948, x945);
    const x991 = (cast(u32, x990) + x946);
    var x992: u32 = undefined;
    var x993: u1 = undefined;
    addcarryxU32(&x992, &x993, 0x0, x920, x967);
    var x994: u32 = undefined;
    var x995: u1 = undefined;
    addcarryxU32(&x994, &x995, x993, x922, x969);
    var x996: u32 = undefined;
    var x997: u1 = undefined;
    addcarryxU32(&x996, &x997, x995, x924, x971);
    var x998: u32 = undefined;
    var x999: u1 = undefined;
    addcarryxU32(&x998, &x999, x997, x926, x973);
    var x1000: u32 = undefined;
    var x1001: u1 = undefined;
    addcarryxU32(&x1000, &x1001, x999, x928, x975);
    var x1002: u32 = undefined;
    var x1003: u1 = undefined;
    addcarryxU32(&x1002, &x1003, x1001, x930, x977);
    var x1004: u32 = undefined;
    var x1005: u1 = undefined;
    addcarryxU32(&x1004, &x1005, x1003, x932, x979);
    var x1006: u32 = undefined;
    var x1007: u1 = undefined;
    addcarryxU32(&x1006, &x1007, x1005, x934, x981);
    var x1008: u32 = undefined;
    var x1009: u1 = undefined;
    addcarryxU32(&x1008, &x1009, x1007, x936, x983);
    var x1010: u32 = undefined;
    var x1011: u1 = undefined;
    addcarryxU32(&x1010, &x1011, x1009, x938, x985);
    var x1012: u32 = undefined;
    var x1013: u1 = undefined;
    addcarryxU32(&x1012, &x1013, x1011, x940, x987);
    var x1014: u32 = undefined;
    var x1015: u1 = undefined;
    addcarryxU32(&x1014, &x1015, x1013, x942, x989);
    var x1016: u32 = undefined;
    var x1017: u1 = undefined;
    addcarryxU32(&x1016, &x1017, x1015, x944, x991);
    var x1018: u32 = undefined;
    var x1019: u32 = undefined;
    mulxU32(&x1018, &x1019, x992, 0xffffffff);
    var x1020: u32 = undefined;
    var x1021: u32 = undefined;
    mulxU32(&x1020, &x1021, x992, 0xffffffff);
    var x1022: u32 = undefined;
    var x1023: u32 = undefined;
    mulxU32(&x1022, &x1023, x992, 0xffffffff);
    var x1024: u32 = undefined;
    var x1025: u32 = undefined;
    mulxU32(&x1024, &x1025, x992, 0xffffffff);
    var x1026: u32 = undefined;
    var x1027: u32 = undefined;
    mulxU32(&x1026, &x1027, x992, 0xffffffff);
    var x1028: u32 = undefined;
    var x1029: u32 = undefined;
    mulxU32(&x1028, &x1029, x992, 0xffffffff);
    var x1030: u32 = undefined;
    var x1031: u32 = undefined;
    mulxU32(&x1030, &x1031, x992, 0xffffffff);
    var x1032: u32 = undefined;
    var x1033: u32 = undefined;
    mulxU32(&x1032, &x1033, x992, 0xfffffffe);
    var x1034: u32 = undefined;
    var x1035: u32 = undefined;
    mulxU32(&x1034, &x1035, x992, 0xffffffff);
    var x1036: u32 = undefined;
    var x1037: u32 = undefined;
    mulxU32(&x1036, &x1037, x992, 0xffffffff);
    var x1038: u32 = undefined;
    var x1039: u1 = undefined;
    addcarryxU32(&x1038, &x1039, 0x0, x1035, x1032);
    var x1040: u32 = undefined;
    var x1041: u1 = undefined;
    addcarryxU32(&x1040, &x1041, x1039, x1033, x1030);
    var x1042: u32 = undefined;
    var x1043: u1 = undefined;
    addcarryxU32(&x1042, &x1043, x1041, x1031, x1028);
    var x1044: u32 = undefined;
    var x1045: u1 = undefined;
    addcarryxU32(&x1044, &x1045, x1043, x1029, x1026);
    var x1046: u32 = undefined;
    var x1047: u1 = undefined;
    addcarryxU32(&x1046, &x1047, x1045, x1027, x1024);
    var x1048: u32 = undefined;
    var x1049: u1 = undefined;
    addcarryxU32(&x1048, &x1049, x1047, x1025, x1022);
    var x1050: u32 = undefined;
    var x1051: u1 = undefined;
    addcarryxU32(&x1050, &x1051, x1049, x1023, x1020);
    var x1052: u32 = undefined;
    var x1053: u1 = undefined;
    addcarryxU32(&x1052, &x1053, x1051, x1021, x1018);
    const x1054 = (cast(u32, x1053) + x1019);
    var x1055: u32 = undefined;
    var x1056: u1 = undefined;
    addcarryxU32(&x1055, &x1056, 0x0, x992, x1036);
    var x1057: u32 = undefined;
    var x1058: u1 = undefined;
    addcarryxU32(&x1057, &x1058, x1056, x994, x1037);
    var x1059: u32 = undefined;
    var x1060: u1 = undefined;
    addcarryxU32(&x1059, &x1060, x1058, x996, cast(u32, 0x0));
    var x1061: u32 = undefined;
    var x1062: u1 = undefined;
    addcarryxU32(&x1061, &x1062, x1060, x998, x1034);
    var x1063: u32 = undefined;
    var x1064: u1 = undefined;
    addcarryxU32(&x1063, &x1064, x1062, x1000, x1038);
    var x1065: u32 = undefined;
    var x1066: u1 = undefined;
    addcarryxU32(&x1065, &x1066, x1064, x1002, x1040);
    var x1067: u32 = undefined;
    var x1068: u1 = undefined;
    addcarryxU32(&x1067, &x1068, x1066, x1004, x1042);
    var x1069: u32 = undefined;
    var x1070: u1 = undefined;
    addcarryxU32(&x1069, &x1070, x1068, x1006, x1044);
    var x1071: u32 = undefined;
    var x1072: u1 = undefined;
    addcarryxU32(&x1071, &x1072, x1070, x1008, x1046);
    var x1073: u32 = undefined;
    var x1074: u1 = undefined;
    addcarryxU32(&x1073, &x1074, x1072, x1010, x1048);
    var x1075: u32 = undefined;
    var x1076: u1 = undefined;
    addcarryxU32(&x1075, &x1076, x1074, x1012, x1050);
    var x1077: u32 = undefined;
    var x1078: u1 = undefined;
    addcarryxU32(&x1077, &x1078, x1076, x1014, x1052);
    var x1079: u32 = undefined;
    var x1080: u1 = undefined;
    addcarryxU32(&x1079, &x1080, x1078, x1016, x1054);
    const x1081 = (cast(u32, x1080) + cast(u32, x1017));
    var x1082: u32 = undefined;
    var x1083: u32 = undefined;
    mulxU32(&x1082, &x1083, x8, (arg2[11]));
    var x1084: u32 = undefined;
    var x1085: u32 = undefined;
    mulxU32(&x1084, &x1085, x8, (arg2[10]));
    var x1086: u32 = undefined;
    var x1087: u32 = undefined;
    mulxU32(&x1086, &x1087, x8, (arg2[9]));
    var x1088: u32 = undefined;
    var x1089: u32 = undefined;
    mulxU32(&x1088, &x1089, x8, (arg2[8]));
    var x1090: u32 = undefined;
    var x1091: u32 = undefined;
    mulxU32(&x1090, &x1091, x8, (arg2[7]));
    var x1092: u32 = undefined;
    var x1093: u32 = undefined;
    mulxU32(&x1092, &x1093, x8, (arg2[6]));
    var x1094: u32 = undefined;
    var x1095: u32 = undefined;
    mulxU32(&x1094, &x1095, x8, (arg2[5]));
    var x1096: u32 = undefined;
    var x1097: u32 = undefined;
    mulxU32(&x1096, &x1097, x8, (arg2[4]));
    var x1098: u32 = undefined;
    var x1099: u32 = undefined;
    mulxU32(&x1098, &x1099, x8, (arg2[3]));
    var x1100: u32 = undefined;
    var x1101: u32 = undefined;
    mulxU32(&x1100, &x1101, x8, (arg2[2]));
    var x1102: u32 = undefined;
    var x1103: u32 = undefined;
    mulxU32(&x1102, &x1103, x8, (arg2[1]));
    var x1104: u32 = undefined;
    var x1105: u32 = undefined;
    mulxU32(&x1104, &x1105, x8, (arg2[0]));
    var x1106: u32 = undefined;
    var x1107: u1 = undefined;
    addcarryxU32(&x1106, &x1107, 0x0, x1105, x1102);
    var x1108: u32 = undefined;
    var x1109: u1 = undefined;
    addcarryxU32(&x1108, &x1109, x1107, x1103, x1100);
    var x1110: u32 = undefined;
    var x1111: u1 = undefined;
    addcarryxU32(&x1110, &x1111, x1109, x1101, x1098);
    var x1112: u32 = undefined;
    var x1113: u1 = undefined;
    addcarryxU32(&x1112, &x1113, x1111, x1099, x1096);
    var x1114: u32 = undefined;
    var x1115: u1 = undefined;
    addcarryxU32(&x1114, &x1115, x1113, x1097, x1094);
    var x1116: u32 = undefined;
    var x1117: u1 = undefined;
    addcarryxU32(&x1116, &x1117, x1115, x1095, x1092);
    var x1118: u32 = undefined;
    var x1119: u1 = undefined;
    addcarryxU32(&x1118, &x1119, x1117, x1093, x1090);
    var x1120: u32 = undefined;
    var x1121: u1 = undefined;
    addcarryxU32(&x1120, &x1121, x1119, x1091, x1088);
    var x1122: u32 = undefined;
    var x1123: u1 = undefined;
    addcarryxU32(&x1122, &x1123, x1121, x1089, x1086);
    var x1124: u32 = undefined;
    var x1125: u1 = undefined;
    addcarryxU32(&x1124, &x1125, x1123, x1087, x1084);
    var x1126: u32 = undefined;
    var x1127: u1 = undefined;
    addcarryxU32(&x1126, &x1127, x1125, x1085, x1082);
    const x1128 = (cast(u32, x1127) + x1083);
    var x1129: u32 = undefined;
    var x1130: u1 = undefined;
    addcarryxU32(&x1129, &x1130, 0x0, x1057, x1104);
    var x1131: u32 = undefined;
    var x1132: u1 = undefined;
    addcarryxU32(&x1131, &x1132, x1130, x1059, x1106);
    var x1133: u32 = undefined;
    var x1134: u1 = undefined;
    addcarryxU32(&x1133, &x1134, x1132, x1061, x1108);
    var x1135: u32 = undefined;
    var x1136: u1 = undefined;
    addcarryxU32(&x1135, &x1136, x1134, x1063, x1110);
    var x1137: u32 = undefined;
    var x1138: u1 = undefined;
    addcarryxU32(&x1137, &x1138, x1136, x1065, x1112);
    var x1139: u32 = undefined;
    var x1140: u1 = undefined;
    addcarryxU32(&x1139, &x1140, x1138, x1067, x1114);
    var x1141: u32 = undefined;
    var x1142: u1 = undefined;
    addcarryxU32(&x1141, &x1142, x1140, x1069, x1116);
    var x1143: u32 = undefined;
    var x1144: u1 = undefined;
    addcarryxU32(&x1143, &x1144, x1142, x1071, x1118);
    var x1145: u32 = undefined;
    var x1146: u1 = undefined;
    addcarryxU32(&x1145, &x1146, x1144, x1073, x1120);
    var x1147: u32 = undefined;
    var x1148: u1 = undefined;
    addcarryxU32(&x1147, &x1148, x1146, x1075, x1122);
    var x1149: u32 = undefined;
    var x1150: u1 = undefined;
    addcarryxU32(&x1149, &x1150, x1148, x1077, x1124);
    var x1151: u32 = undefined;
    var x1152: u1 = undefined;
    addcarryxU32(&x1151, &x1152, x1150, x1079, x1126);
    var x1153: u32 = undefined;
    var x1154: u1 = undefined;
    addcarryxU32(&x1153, &x1154, x1152, x1081, x1128);
    var x1155: u32 = undefined;
    var x1156: u32 = undefined;
    mulxU32(&x1155, &x1156, x1129, 0xffffffff);
    var x1157: u32 = undefined;
    var x1158: u32 = undefined;
    mulxU32(&x1157, &x1158, x1129, 0xffffffff);
    var x1159: u32 = undefined;
    var x1160: u32 = undefined;
    mulxU32(&x1159, &x1160, x1129, 0xffffffff);
    var x1161: u32 = undefined;
    var x1162: u32 = undefined;
    mulxU32(&x1161, &x1162, x1129, 0xffffffff);
    var x1163: u32 = undefined;
    var x1164: u32 = undefined;
    mulxU32(&x1163, &x1164, x1129, 0xffffffff);
    var x1165: u32 = undefined;
    var x1166: u32 = undefined;
    mulxU32(&x1165, &x1166, x1129, 0xffffffff);
    var x1167: u32 = undefined;
    var x1168: u32 = undefined;
    mulxU32(&x1167, &x1168, x1129, 0xffffffff);
    var x1169: u32 = undefined;
    var x1170: u32 = undefined;
    mulxU32(&x1169, &x1170, x1129, 0xfffffffe);
    var x1171: u32 = undefined;
    var x1172: u32 = undefined;
    mulxU32(&x1171, &x1172, x1129, 0xffffffff);
    var x1173: u32 = undefined;
    var x1174: u32 = undefined;
    mulxU32(&x1173, &x1174, x1129, 0xffffffff);
    var x1175: u32 = undefined;
    var x1176: u1 = undefined;
    addcarryxU32(&x1175, &x1176, 0x0, x1172, x1169);
    var x1177: u32 = undefined;
    var x1178: u1 = undefined;
    addcarryxU32(&x1177, &x1178, x1176, x1170, x1167);
    var x1179: u32 = undefined;
    var x1180: u1 = undefined;
    addcarryxU32(&x1179, &x1180, x1178, x1168, x1165);
    var x1181: u32 = undefined;
    var x1182: u1 = undefined;
    addcarryxU32(&x1181, &x1182, x1180, x1166, x1163);
    var x1183: u32 = undefined;
    var x1184: u1 = undefined;
    addcarryxU32(&x1183, &x1184, x1182, x1164, x1161);
    var x1185: u32 = undefined;
    var x1186: u1 = undefined;
    addcarryxU32(&x1185, &x1186, x1184, x1162, x1159);
    var x1187: u32 = undefined;
    var x1188: u1 = undefined;
    addcarryxU32(&x1187, &x1188, x1186, x1160, x1157);
    var x1189: u32 = undefined;
    var x1190: u1 = undefined;
    addcarryxU32(&x1189, &x1190, x1188, x1158, x1155);
    const x1191 = (cast(u32, x1190) + x1156);
    var x1192: u32 = undefined;
    var x1193: u1 = undefined;
    addcarryxU32(&x1192, &x1193, 0x0, x1129, x1173);
    var x1194: u32 = undefined;
    var x1195: u1 = undefined;
    addcarryxU32(&x1194, &x1195, x1193, x1131, x1174);
    var x1196: u32 = undefined;
    var x1197: u1 = undefined;
    addcarryxU32(&x1196, &x1197, x1195, x1133, cast(u32, 0x0));
    var x1198: u32 = undefined;
    var x1199: u1 = undefined;
    addcarryxU32(&x1198, &x1199, x1197, x1135, x1171);
    var x1200: u32 = undefined;
    var x1201: u1 = undefined;
    addcarryxU32(&x1200, &x1201, x1199, x1137, x1175);
    var x1202: u32 = undefined;
    var x1203: u1 = undefined;
    addcarryxU32(&x1202, &x1203, x1201, x1139, x1177);
    var x1204: u32 = undefined;
    var x1205: u1 = undefined;
    addcarryxU32(&x1204, &x1205, x1203, x1141, x1179);
    var x1206: u32 = undefined;
    var x1207: u1 = undefined;
    addcarryxU32(&x1206, &x1207, x1205, x1143, x1181);
    var x1208: u32 = undefined;
    var x1209: u1 = undefined;
    addcarryxU32(&x1208, &x1209, x1207, x1145, x1183);
    var x1210: u32 = undefined;
    var x1211: u1 = undefined;
    addcarryxU32(&x1210, &x1211, x1209, x1147, x1185);
    var x1212: u32 = undefined;
    var x1213: u1 = undefined;
    addcarryxU32(&x1212, &x1213, x1211, x1149, x1187);
    var x1214: u32 = undefined;
    var x1215: u1 = undefined;
    addcarryxU32(&x1214, &x1215, x1213, x1151, x1189);
    var x1216: u32 = undefined;
    var x1217: u1 = undefined;
    addcarryxU32(&x1216, &x1217, x1215, x1153, x1191);
    const x1218 = (cast(u32, x1217) + cast(u32, x1154));
    var x1219: u32 = undefined;
    var x1220: u32 = undefined;
    mulxU32(&x1219, &x1220, x9, (arg2[11]));
    var x1221: u32 = undefined;
    var x1222: u32 = undefined;
    mulxU32(&x1221, &x1222, x9, (arg2[10]));
    var x1223: u32 = undefined;
    var x1224: u32 = undefined;
    mulxU32(&x1223, &x1224, x9, (arg2[9]));
    var x1225: u32 = undefined;
    var x1226: u32 = undefined;
    mulxU32(&x1225, &x1226, x9, (arg2[8]));
    var x1227: u32 = undefined;
    var x1228: u32 = undefined;
    mulxU32(&x1227, &x1228, x9, (arg2[7]));
    var x1229: u32 = undefined;
    var x1230: u32 = undefined;
    mulxU32(&x1229, &x1230, x9, (arg2[6]));
    var x1231: u32 = undefined;
    var x1232: u32 = undefined;
    mulxU32(&x1231, &x1232, x9, (arg2[5]));
    var x1233: u32 = undefined;
    var x1234: u32 = undefined;
    mulxU32(&x1233, &x1234, x9, (arg2[4]));
    var x1235: u32 = undefined;
    var x1236: u32 = undefined;
    mulxU32(&x1235, &x1236, x9, (arg2[3]));
    var x1237: u32 = undefined;
    var x1238: u32 = undefined;
    mulxU32(&x1237, &x1238, x9, (arg2[2]));
    var x1239: u32 = undefined;
    var x1240: u32 = undefined;
    mulxU32(&x1239, &x1240, x9, (arg2[1]));
    var x1241: u32 = undefined;
    var x1242: u32 = undefined;
    mulxU32(&x1241, &x1242, x9, (arg2[0]));
    var x1243: u32 = undefined;
    var x1244: u1 = undefined;
    addcarryxU32(&x1243, &x1244, 0x0, x1242, x1239);
    var x1245: u32 = undefined;
    var x1246: u1 = undefined;
    addcarryxU32(&x1245, &x1246, x1244, x1240, x1237);
    var x1247: u32 = undefined;
    var x1248: u1 = undefined;
    addcarryxU32(&x1247, &x1248, x1246, x1238, x1235);
    var x1249: u32 = undefined;
    var x1250: u1 = undefined;
    addcarryxU32(&x1249, &x1250, x1248, x1236, x1233);
    var x1251: u32 = undefined;
    var x1252: u1 = undefined;
    addcarryxU32(&x1251, &x1252, x1250, x1234, x1231);
    var x1253: u32 = undefined;
    var x1254: u1 = undefined;
    addcarryxU32(&x1253, &x1254, x1252, x1232, x1229);
    var x1255: u32 = undefined;
    var x1256: u1 = undefined;
    addcarryxU32(&x1255, &x1256, x1254, x1230, x1227);
    var x1257: u32 = undefined;
    var x1258: u1 = undefined;
    addcarryxU32(&x1257, &x1258, x1256, x1228, x1225);
    var x1259: u32 = undefined;
    var x1260: u1 = undefined;
    addcarryxU32(&x1259, &x1260, x1258, x1226, x1223);
    var x1261: u32 = undefined;
    var x1262: u1 = undefined;
    addcarryxU32(&x1261, &x1262, x1260, x1224, x1221);
    var x1263: u32 = undefined;
    var x1264: u1 = undefined;
    addcarryxU32(&x1263, &x1264, x1262, x1222, x1219);
    const x1265 = (cast(u32, x1264) + x1220);
    var x1266: u32 = undefined;
    var x1267: u1 = undefined;
    addcarryxU32(&x1266, &x1267, 0x0, x1194, x1241);
    var x1268: u32 = undefined;
    var x1269: u1 = undefined;
    addcarryxU32(&x1268, &x1269, x1267, x1196, x1243);
    var x1270: u32 = undefined;
    var x1271: u1 = undefined;
    addcarryxU32(&x1270, &x1271, x1269, x1198, x1245);
    var x1272: u32 = undefined;
    var x1273: u1 = undefined;
    addcarryxU32(&x1272, &x1273, x1271, x1200, x1247);
    var x1274: u32 = undefined;
    var x1275: u1 = undefined;
    addcarryxU32(&x1274, &x1275, x1273, x1202, x1249);
    var x1276: u32 = undefined;
    var x1277: u1 = undefined;
    addcarryxU32(&x1276, &x1277, x1275, x1204, x1251);
    var x1278: u32 = undefined;
    var x1279: u1 = undefined;
    addcarryxU32(&x1278, &x1279, x1277, x1206, x1253);
    var x1280: u32 = undefined;
    var x1281: u1 = undefined;
    addcarryxU32(&x1280, &x1281, x1279, x1208, x1255);
    var x1282: u32 = undefined;
    var x1283: u1 = undefined;
    addcarryxU32(&x1282, &x1283, x1281, x1210, x1257);
    var x1284: u32 = undefined;
    var x1285: u1 = undefined;
    addcarryxU32(&x1284, &x1285, x1283, x1212, x1259);
    var x1286: u32 = undefined;
    var x1287: u1 = undefined;
    addcarryxU32(&x1286, &x1287, x1285, x1214, x1261);
    var x1288: u32 = undefined;
    var x1289: u1 = undefined;
    addcarryxU32(&x1288, &x1289, x1287, x1216, x1263);
    var x1290: u32 = undefined;
    var x1291: u1 = undefined;
    addcarryxU32(&x1290, &x1291, x1289, x1218, x1265);
    var x1292: u32 = undefined;
    var x1293: u32 = undefined;
    mulxU32(&x1292, &x1293, x1266, 0xffffffff);
    var x1294: u32 = undefined;
    var x1295: u32 = undefined;
    mulxU32(&x1294, &x1295, x1266, 0xffffffff);
    var x1296: u32 = undefined;
    var x1297: u32 = undefined;
    mulxU32(&x1296, &x1297, x1266, 0xffffffff);
    var x1298: u32 = undefined;
    var x1299: u32 = undefined;
    mulxU32(&x1298, &x1299, x1266, 0xffffffff);
    var x1300: u32 = undefined;
    var x1301: u32 = undefined;
    mulxU32(&x1300, &x1301, x1266, 0xffffffff);
    var x1302: u32 = undefined;
    var x1303: u32 = undefined;
    mulxU32(&x1302, &x1303, x1266, 0xffffffff);
    var x1304: u32 = undefined;
    var x1305: u32 = undefined;
    mulxU32(&x1304, &x1305, x1266, 0xffffffff);
    var x1306: u32 = undefined;
    var x1307: u32 = undefined;
    mulxU32(&x1306, &x1307, x1266, 0xfffffffe);
    var x1308: u32 = undefined;
    var x1309: u32 = undefined;
    mulxU32(&x1308, &x1309, x1266, 0xffffffff);
    var x1310: u32 = undefined;
    var x1311: u32 = undefined;
    mulxU32(&x1310, &x1311, x1266, 0xffffffff);
    var x1312: u32 = undefined;
    var x1313: u1 = undefined;
    addcarryxU32(&x1312, &x1313, 0x0, x1309, x1306);
    var x1314: u32 = undefined;
    var x1315: u1 = undefined;
    addcarryxU32(&x1314, &x1315, x1313, x1307, x1304);
    var x1316: u32 = undefined;
    var x1317: u1 = undefined;
    addcarryxU32(&x1316, &x1317, x1315, x1305, x1302);
    var x1318: u32 = undefined;
    var x1319: u1 = undefined;
    addcarryxU32(&x1318, &x1319, x1317, x1303, x1300);
    var x1320: u32 = undefined;
    var x1321: u1 = undefined;
    addcarryxU32(&x1320, &x1321, x1319, x1301, x1298);
    var x1322: u32 = undefined;
    var x1323: u1 = undefined;
    addcarryxU32(&x1322, &x1323, x1321, x1299, x1296);
    var x1324: u32 = undefined;
    var x1325: u1 = undefined;
    addcarryxU32(&x1324, &x1325, x1323, x1297, x1294);
    var x1326: u32 = undefined;
    var x1327: u1 = undefined;
    addcarryxU32(&x1326, &x1327, x1325, x1295, x1292);
    const x1328 = (cast(u32, x1327) + x1293);
    var x1329: u32 = undefined;
    var x1330: u1 = undefined;
    addcarryxU32(&x1329, &x1330, 0x0, x1266, x1310);
    var x1331: u32 = undefined;
    var x1332: u1 = undefined;
    addcarryxU32(&x1331, &x1332, x1330, x1268, x1311);
    var x1333: u32 = undefined;
    var x1334: u1 = undefined;
    addcarryxU32(&x1333, &x1334, x1332, x1270, cast(u32, 0x0));
    var x1335: u32 = undefined;
    var x1336: u1 = undefined;
    addcarryxU32(&x1335, &x1336, x1334, x1272, x1308);
    var x1337: u32 = undefined;
    var x1338: u1 = undefined;
    addcarryxU32(&x1337, &x1338, x1336, x1274, x1312);
    var x1339: u32 = undefined;
    var x1340: u1 = undefined;
    addcarryxU32(&x1339, &x1340, x1338, x1276, x1314);
    var x1341: u32 = undefined;
    var x1342: u1 = undefined;
    addcarryxU32(&x1341, &x1342, x1340, x1278, x1316);
    var x1343: u32 = undefined;
    var x1344: u1 = undefined;
    addcarryxU32(&x1343, &x1344, x1342, x1280, x1318);
    var x1345: u32 = undefined;
    var x1346: u1 = undefined;
    addcarryxU32(&x1345, &x1346, x1344, x1282, x1320);
    var x1347: u32 = undefined;
    var x1348: u1 = undefined;
    addcarryxU32(&x1347, &x1348, x1346, x1284, x1322);
    var x1349: u32 = undefined;
    var x1350: u1 = undefined;
    addcarryxU32(&x1349, &x1350, x1348, x1286, x1324);
    var x1351: u32 = undefined;
    var x1352: u1 = undefined;
    addcarryxU32(&x1351, &x1352, x1350, x1288, x1326);
    var x1353: u32 = undefined;
    var x1354: u1 = undefined;
    addcarryxU32(&x1353, &x1354, x1352, x1290, x1328);
    const x1355 = (cast(u32, x1354) + cast(u32, x1291));
    var x1356: u32 = undefined;
    var x1357: u32 = undefined;
    mulxU32(&x1356, &x1357, x10, (arg2[11]));
    var x1358: u32 = undefined;
    var x1359: u32 = undefined;
    mulxU32(&x1358, &x1359, x10, (arg2[10]));
    var x1360: u32 = undefined;
    var x1361: u32 = undefined;
    mulxU32(&x1360, &x1361, x10, (arg2[9]));
    var x1362: u32 = undefined;
    var x1363: u32 = undefined;
    mulxU32(&x1362, &x1363, x10, (arg2[8]));
    var x1364: u32 = undefined;
    var x1365: u32 = undefined;
    mulxU32(&x1364, &x1365, x10, (arg2[7]));
    var x1366: u32 = undefined;
    var x1367: u32 = undefined;
    mulxU32(&x1366, &x1367, x10, (arg2[6]));
    var x1368: u32 = undefined;
    var x1369: u32 = undefined;
    mulxU32(&x1368, &x1369, x10, (arg2[5]));
    var x1370: u32 = undefined;
    var x1371: u32 = undefined;
    mulxU32(&x1370, &x1371, x10, (arg2[4]));
    var x1372: u32 = undefined;
    var x1373: u32 = undefined;
    mulxU32(&x1372, &x1373, x10, (arg2[3]));
    var x1374: u32 = undefined;
    var x1375: u32 = undefined;
    mulxU32(&x1374, &x1375, x10, (arg2[2]));
    var x1376: u32 = undefined;
    var x1377: u32 = undefined;
    mulxU32(&x1376, &x1377, x10, (arg2[1]));
    var x1378: u32 = undefined;
    var x1379: u32 = undefined;
    mulxU32(&x1378, &x1379, x10, (arg2[0]));
    var x1380: u32 = undefined;
    var x1381: u1 = undefined;
    addcarryxU32(&x1380, &x1381, 0x0, x1379, x1376);
    var x1382: u32 = undefined;
    var x1383: u1 = undefined;
    addcarryxU32(&x1382, &x1383, x1381, x1377, x1374);
    var x1384: u32 = undefined;
    var x1385: u1 = undefined;
    addcarryxU32(&x1384, &x1385, x1383, x1375, x1372);
    var x1386: u32 = undefined;
    var x1387: u1 = undefined;
    addcarryxU32(&x1386, &x1387, x1385, x1373, x1370);
    var x1388: u32 = undefined;
    var x1389: u1 = undefined;
    addcarryxU32(&x1388, &x1389, x1387, x1371, x1368);
    var x1390: u32 = undefined;
    var x1391: u1 = undefined;
    addcarryxU32(&x1390, &x1391, x1389, x1369, x1366);
    var x1392: u32 = undefined;
    var x1393: u1 = undefined;
    addcarryxU32(&x1392, &x1393, x1391, x1367, x1364);
    var x1394: u32 = undefined;
    var x1395: u1 = undefined;
    addcarryxU32(&x1394, &x1395, x1393, x1365, x1362);
    var x1396: u32 = undefined;
    var x1397: u1 = undefined;
    addcarryxU32(&x1396, &x1397, x1395, x1363, x1360);
    var x1398: u32 = undefined;
    var x1399: u1 = undefined;
    addcarryxU32(&x1398, &x1399, x1397, x1361, x1358);
    var x1400: u32 = undefined;
    var x1401: u1 = undefined;
    addcarryxU32(&x1400, &x1401, x1399, x1359, x1356);
    const x1402 = (cast(u32, x1401) + x1357);
    var x1403: u32 = undefined;
    var x1404: u1 = undefined;
    addcarryxU32(&x1403, &x1404, 0x0, x1331, x1378);
    var x1405: u32 = undefined;
    var x1406: u1 = undefined;
    addcarryxU32(&x1405, &x1406, x1404, x1333, x1380);
    var x1407: u32 = undefined;
    var x1408: u1 = undefined;
    addcarryxU32(&x1407, &x1408, x1406, x1335, x1382);
    var x1409: u32 = undefined;
    var x1410: u1 = undefined;
    addcarryxU32(&x1409, &x1410, x1408, x1337, x1384);
    var x1411: u32 = undefined;
    var x1412: u1 = undefined;
    addcarryxU32(&x1411, &x1412, x1410, x1339, x1386);
    var x1413: u32 = undefined;
    var x1414: u1 = undefined;
    addcarryxU32(&x1413, &x1414, x1412, x1341, x1388);
    var x1415: u32 = undefined;
    var x1416: u1 = undefined;
    addcarryxU32(&x1415, &x1416, x1414, x1343, x1390);
    var x1417: u32 = undefined;
    var x1418: u1 = undefined;
    addcarryxU32(&x1417, &x1418, x1416, x1345, x1392);
    var x1419: u32 = undefined;
    var x1420: u1 = undefined;
    addcarryxU32(&x1419, &x1420, x1418, x1347, x1394);
    var x1421: u32 = undefined;
    var x1422: u1 = undefined;
    addcarryxU32(&x1421, &x1422, x1420, x1349, x1396);
    var x1423: u32 = undefined;
    var x1424: u1 = undefined;
    addcarryxU32(&x1423, &x1424, x1422, x1351, x1398);
    var x1425: u32 = undefined;
    var x1426: u1 = undefined;
    addcarryxU32(&x1425, &x1426, x1424, x1353, x1400);
    var x1427: u32 = undefined;
    var x1428: u1 = undefined;
    addcarryxU32(&x1427, &x1428, x1426, x1355, x1402);
    var x1429: u32 = undefined;
    var x1430: u32 = undefined;
    mulxU32(&x1429, &x1430, x1403, 0xffffffff);
    var x1431: u32 = undefined;
    var x1432: u32 = undefined;
    mulxU32(&x1431, &x1432, x1403, 0xffffffff);
    var x1433: u32 = undefined;
    var x1434: u32 = undefined;
    mulxU32(&x1433, &x1434, x1403, 0xffffffff);
    var x1435: u32 = undefined;
    var x1436: u32 = undefined;
    mulxU32(&x1435, &x1436, x1403, 0xffffffff);
    var x1437: u32 = undefined;
    var x1438: u32 = undefined;
    mulxU32(&x1437, &x1438, x1403, 0xffffffff);
    var x1439: u32 = undefined;
    var x1440: u32 = undefined;
    mulxU32(&x1439, &x1440, x1403, 0xffffffff);
    var x1441: u32 = undefined;
    var x1442: u32 = undefined;
    mulxU32(&x1441, &x1442, x1403, 0xffffffff);
    var x1443: u32 = undefined;
    var x1444: u32 = undefined;
    mulxU32(&x1443, &x1444, x1403, 0xfffffffe);
    var x1445: u32 = undefined;
    var x1446: u32 = undefined;
    mulxU32(&x1445, &x1446, x1403, 0xffffffff);
    var x1447: u32 = undefined;
    var x1448: u32 = undefined;
    mulxU32(&x1447, &x1448, x1403, 0xffffffff);
    var x1449: u32 = undefined;
    var x1450: u1 = undefined;
    addcarryxU32(&x1449, &x1450, 0x0, x1446, x1443);
    var x1451: u32 = undefined;
    var x1452: u1 = undefined;
    addcarryxU32(&x1451, &x1452, x1450, x1444, x1441);
    var x1453: u32 = undefined;
    var x1454: u1 = undefined;
    addcarryxU32(&x1453, &x1454, x1452, x1442, x1439);
    var x1455: u32 = undefined;
    var x1456: u1 = undefined;
    addcarryxU32(&x1455, &x1456, x1454, x1440, x1437);
    var x1457: u32 = undefined;
    var x1458: u1 = undefined;
    addcarryxU32(&x1457, &x1458, x1456, x1438, x1435);
    var x1459: u32 = undefined;
    var x1460: u1 = undefined;
    addcarryxU32(&x1459, &x1460, x1458, x1436, x1433);
    var x1461: u32 = undefined;
    var x1462: u1 = undefined;
    addcarryxU32(&x1461, &x1462, x1460, x1434, x1431);
    var x1463: u32 = undefined;
    var x1464: u1 = undefined;
    addcarryxU32(&x1463, &x1464, x1462, x1432, x1429);
    const x1465 = (cast(u32, x1464) + x1430);
    var x1466: u32 = undefined;
    var x1467: u1 = undefined;
    addcarryxU32(&x1466, &x1467, 0x0, x1403, x1447);
    var x1468: u32 = undefined;
    var x1469: u1 = undefined;
    addcarryxU32(&x1468, &x1469, x1467, x1405, x1448);
    var x1470: u32 = undefined;
    var x1471: u1 = undefined;
    addcarryxU32(&x1470, &x1471, x1469, x1407, cast(u32, 0x0));
    var x1472: u32 = undefined;
    var x1473: u1 = undefined;
    addcarryxU32(&x1472, &x1473, x1471, x1409, x1445);
    var x1474: u32 = undefined;
    var x1475: u1 = undefined;
    addcarryxU32(&x1474, &x1475, x1473, x1411, x1449);
    var x1476: u32 = undefined;
    var x1477: u1 = undefined;
    addcarryxU32(&x1476, &x1477, x1475, x1413, x1451);
    var x1478: u32 = undefined;
    var x1479: u1 = undefined;
    addcarryxU32(&x1478, &x1479, x1477, x1415, x1453);
    var x1480: u32 = undefined;
    var x1481: u1 = undefined;
    addcarryxU32(&x1480, &x1481, x1479, x1417, x1455);
    var x1482: u32 = undefined;
    var x1483: u1 = undefined;
    addcarryxU32(&x1482, &x1483, x1481, x1419, x1457);
    var x1484: u32 = undefined;
    var x1485: u1 = undefined;
    addcarryxU32(&x1484, &x1485, x1483, x1421, x1459);
    var x1486: u32 = undefined;
    var x1487: u1 = undefined;
    addcarryxU32(&x1486, &x1487, x1485, x1423, x1461);
    var x1488: u32 = undefined;
    var x1489: u1 = undefined;
    addcarryxU32(&x1488, &x1489, x1487, x1425, x1463);
    var x1490: u32 = undefined;
    var x1491: u1 = undefined;
    addcarryxU32(&x1490, &x1491, x1489, x1427, x1465);
    const x1492 = (cast(u32, x1491) + cast(u32, x1428));
    var x1493: u32 = undefined;
    var x1494: u32 = undefined;
    mulxU32(&x1493, &x1494, x11, (arg2[11]));
    var x1495: u32 = undefined;
    var x1496: u32 = undefined;
    mulxU32(&x1495, &x1496, x11, (arg2[10]));
    var x1497: u32 = undefined;
    var x1498: u32 = undefined;
    mulxU32(&x1497, &x1498, x11, (arg2[9]));
    var x1499: u32 = undefined;
    var x1500: u32 = undefined;
    mulxU32(&x1499, &x1500, x11, (arg2[8]));
    var x1501: u32 = undefined;
    var x1502: u32 = undefined;
    mulxU32(&x1501, &x1502, x11, (arg2[7]));
    var x1503: u32 = undefined;
    var x1504: u32 = undefined;
    mulxU32(&x1503, &x1504, x11, (arg2[6]));
    var x1505: u32 = undefined;
    var x1506: u32 = undefined;
    mulxU32(&x1505, &x1506, x11, (arg2[5]));
    var x1507: u32 = undefined;
    var x1508: u32 = undefined;
    mulxU32(&x1507, &x1508, x11, (arg2[4]));
    var x1509: u32 = undefined;
    var x1510: u32 = undefined;
    mulxU32(&x1509, &x1510, x11, (arg2[3]));
    var x1511: u32 = undefined;
    var x1512: u32 = undefined;
    mulxU32(&x1511, &x1512, x11, (arg2[2]));
    var x1513: u32 = undefined;
    var x1514: u32 = undefined;
    mulxU32(&x1513, &x1514, x11, (arg2[1]));
    var x1515: u32 = undefined;
    var x1516: u32 = undefined;
    mulxU32(&x1515, &x1516, x11, (arg2[0]));
    var x1517: u32 = undefined;
    var x1518: u1 = undefined;
    addcarryxU32(&x1517, &x1518, 0x0, x1516, x1513);
    var x1519: u32 = undefined;
    var x1520: u1 = undefined;
    addcarryxU32(&x1519, &x1520, x1518, x1514, x1511);
    var x1521: u32 = undefined;
    var x1522: u1 = undefined;
    addcarryxU32(&x1521, &x1522, x1520, x1512, x1509);
    var x1523: u32 = undefined;
    var x1524: u1 = undefined;
    addcarryxU32(&x1523, &x1524, x1522, x1510, x1507);
    var x1525: u32 = undefined;
    var x1526: u1 = undefined;
    addcarryxU32(&x1525, &x1526, x1524, x1508, x1505);
    var x1527: u32 = undefined;
    var x1528: u1 = undefined;
    addcarryxU32(&x1527, &x1528, x1526, x1506, x1503);
    var x1529: u32 = undefined;
    var x1530: u1 = undefined;
    addcarryxU32(&x1529, &x1530, x1528, x1504, x1501);
    var x1531: u32 = undefined;
    var x1532: u1 = undefined;
    addcarryxU32(&x1531, &x1532, x1530, x1502, x1499);
    var x1533: u32 = undefined;
    var x1534: u1 = undefined;
    addcarryxU32(&x1533, &x1534, x1532, x1500, x1497);
    var x1535: u32 = undefined;
    var x1536: u1 = undefined;
    addcarryxU32(&x1535, &x1536, x1534, x1498, x1495);
    var x1537: u32 = undefined;
    var x1538: u1 = undefined;
    addcarryxU32(&x1537, &x1538, x1536, x1496, x1493);
    const x1539 = (cast(u32, x1538) + x1494);
    var x1540: u32 = undefined;
    var x1541: u1 = undefined;
    addcarryxU32(&x1540, &x1541, 0x0, x1468, x1515);
    var x1542: u32 = undefined;
    var x1543: u1 = undefined;
    addcarryxU32(&x1542, &x1543, x1541, x1470, x1517);
    var x1544: u32 = undefined;
    var x1545: u1 = undefined;
    addcarryxU32(&x1544, &x1545, x1543, x1472, x1519);
    var x1546: u32 = undefined;
    var x1547: u1 = undefined;
    addcarryxU32(&x1546, &x1547, x1545, x1474, x1521);
    var x1548: u32 = undefined;
    var x1549: u1 = undefined;
    addcarryxU32(&x1548, &x1549, x1547, x1476, x1523);
    var x1550: u32 = undefined;
    var x1551: u1 = undefined;
    addcarryxU32(&x1550, &x1551, x1549, x1478, x1525);
    var x1552: u32 = undefined;
    var x1553: u1 = undefined;
    addcarryxU32(&x1552, &x1553, x1551, x1480, x1527);
    var x1554: u32 = undefined;
    var x1555: u1 = undefined;
    addcarryxU32(&x1554, &x1555, x1553, x1482, x1529);
    var x1556: u32 = undefined;
    var x1557: u1 = undefined;
    addcarryxU32(&x1556, &x1557, x1555, x1484, x1531);
    var x1558: u32 = undefined;
    var x1559: u1 = undefined;
    addcarryxU32(&x1558, &x1559, x1557, x1486, x1533);
    var x1560: u32 = undefined;
    var x1561: u1 = undefined;
    addcarryxU32(&x1560, &x1561, x1559, x1488, x1535);
    var x1562: u32 = undefined;
    var x1563: u1 = undefined;
    addcarryxU32(&x1562, &x1563, x1561, x1490, x1537);
    var x1564: u32 = undefined;
    var x1565: u1 = undefined;
    addcarryxU32(&x1564, &x1565, x1563, x1492, x1539);
    var x1566: u32 = undefined;
    var x1567: u32 = undefined;
    mulxU32(&x1566, &x1567, x1540, 0xffffffff);
    var x1568: u32 = undefined;
    var x1569: u32 = undefined;
    mulxU32(&x1568, &x1569, x1540, 0xffffffff);
    var x1570: u32 = undefined;
    var x1571: u32 = undefined;
    mulxU32(&x1570, &x1571, x1540, 0xffffffff);
    var x1572: u32 = undefined;
    var x1573: u32 = undefined;
    mulxU32(&x1572, &x1573, x1540, 0xffffffff);
    var x1574: u32 = undefined;
    var x1575: u32 = undefined;
    mulxU32(&x1574, &x1575, x1540, 0xffffffff);
    var x1576: u32 = undefined;
    var x1577: u32 = undefined;
    mulxU32(&x1576, &x1577, x1540, 0xffffffff);
    var x1578: u32 = undefined;
    var x1579: u32 = undefined;
    mulxU32(&x1578, &x1579, x1540, 0xffffffff);
    var x1580: u32 = undefined;
    var x1581: u32 = undefined;
    mulxU32(&x1580, &x1581, x1540, 0xfffffffe);
    var x1582: u32 = undefined;
    var x1583: u32 = undefined;
    mulxU32(&x1582, &x1583, x1540, 0xffffffff);
    var x1584: u32 = undefined;
    var x1585: u32 = undefined;
    mulxU32(&x1584, &x1585, x1540, 0xffffffff);
    var x1586: u32 = undefined;
    var x1587: u1 = undefined;
    addcarryxU32(&x1586, &x1587, 0x0, x1583, x1580);
    var x1588: u32 = undefined;
    var x1589: u1 = undefined;
    addcarryxU32(&x1588, &x1589, x1587, x1581, x1578);
    var x1590: u32 = undefined;
    var x1591: u1 = undefined;
    addcarryxU32(&x1590, &x1591, x1589, x1579, x1576);
    var x1592: u32 = undefined;
    var x1593: u1 = undefined;
    addcarryxU32(&x1592, &x1593, x1591, x1577, x1574);
    var x1594: u32 = undefined;
    var x1595: u1 = undefined;
    addcarryxU32(&x1594, &x1595, x1593, x1575, x1572);
    var x1596: u32 = undefined;
    var x1597: u1 = undefined;
    addcarryxU32(&x1596, &x1597, x1595, x1573, x1570);
    var x1598: u32 = undefined;
    var x1599: u1 = undefined;
    addcarryxU32(&x1598, &x1599, x1597, x1571, x1568);
    var x1600: u32 = undefined;
    var x1601: u1 = undefined;
    addcarryxU32(&x1600, &x1601, x1599, x1569, x1566);
    const x1602 = (cast(u32, x1601) + x1567);
    var x1603: u32 = undefined;
    var x1604: u1 = undefined;
    addcarryxU32(&x1603, &x1604, 0x0, x1540, x1584);
    var x1605: u32 = undefined;
    var x1606: u1 = undefined;
    addcarryxU32(&x1605, &x1606, x1604, x1542, x1585);
    var x1607: u32 = undefined;
    var x1608: u1 = undefined;
    addcarryxU32(&x1607, &x1608, x1606, x1544, cast(u32, 0x0));
    var x1609: u32 = undefined;
    var x1610: u1 = undefined;
    addcarryxU32(&x1609, &x1610, x1608, x1546, x1582);
    var x1611: u32 = undefined;
    var x1612: u1 = undefined;
    addcarryxU32(&x1611, &x1612, x1610, x1548, x1586);
    var x1613: u32 = undefined;
    var x1614: u1 = undefined;
    addcarryxU32(&x1613, &x1614, x1612, x1550, x1588);
    var x1615: u32 = undefined;
    var x1616: u1 = undefined;
    addcarryxU32(&x1615, &x1616, x1614, x1552, x1590);
    var x1617: u32 = undefined;
    var x1618: u1 = undefined;
    addcarryxU32(&x1617, &x1618, x1616, x1554, x1592);
    var x1619: u32 = undefined;
    var x1620: u1 = undefined;
    addcarryxU32(&x1619, &x1620, x1618, x1556, x1594);
    var x1621: u32 = undefined;
    var x1622: u1 = undefined;
    addcarryxU32(&x1621, &x1622, x1620, x1558, x1596);
    var x1623: u32 = undefined;
    var x1624: u1 = undefined;
    addcarryxU32(&x1623, &x1624, x1622, x1560, x1598);
    var x1625: u32 = undefined;
    var x1626: u1 = undefined;
    addcarryxU32(&x1625, &x1626, x1624, x1562, x1600);
    var x1627: u32 = undefined;
    var x1628: u1 = undefined;
    addcarryxU32(&x1627, &x1628, x1626, x1564, x1602);
    const x1629 = (cast(u32, x1628) + cast(u32, x1565));
    var x1630: u32 = undefined;
    var x1631: u1 = undefined;
    subborrowxU32(&x1630, &x1631, 0x0, x1605, 0xffffffff);
    var x1632: u32 = undefined;
    var x1633: u1 = undefined;
    subborrowxU32(&x1632, &x1633, x1631, x1607, cast(u32, 0x0));
    var x1634: u32 = undefined;
    var x1635: u1 = undefined;
    subborrowxU32(&x1634, &x1635, x1633, x1609, cast(u32, 0x0));
    var x1636: u32 = undefined;
    var x1637: u1 = undefined;
    subborrowxU32(&x1636, &x1637, x1635, x1611, 0xffffffff);
    var x1638: u32 = undefined;
    var x1639: u1 = undefined;
    subborrowxU32(&x1638, &x1639, x1637, x1613, 0xfffffffe);
    var x1640: u32 = undefined;
    var x1641: u1 = undefined;
    subborrowxU32(&x1640, &x1641, x1639, x1615, 0xffffffff);
    var x1642: u32 = undefined;
    var x1643: u1 = undefined;
    subborrowxU32(&x1642, &x1643, x1641, x1617, 0xffffffff);
    var x1644: u32 = undefined;
    var x1645: u1 = undefined;
    subborrowxU32(&x1644, &x1645, x1643, x1619, 0xffffffff);
    var x1646: u32 = undefined;
    var x1647: u1 = undefined;
    subborrowxU32(&x1646, &x1647, x1645, x1621, 0xffffffff);
    var x1648: u32 = undefined;
    var x1649: u1 = undefined;
    subborrowxU32(&x1648, &x1649, x1647, x1623, 0xffffffff);
    var x1650: u32 = undefined;
    var x1651: u1 = undefined;
    subborrowxU32(&x1650, &x1651, x1649, x1625, 0xffffffff);
    var x1652: u32 = undefined;
    var x1653: u1 = undefined;
    subborrowxU32(&x1652, &x1653, x1651, x1627, 0xffffffff);
    var x1654: u32 = undefined;
    var x1655: u1 = undefined;
    subborrowxU32(&x1654, &x1655, x1653, x1629, cast(u32, 0x0));
    var x1656: u32 = undefined;
    cmovznzU32(&x1656, x1655, x1630, x1605);
    var x1657: u32 = undefined;
    cmovznzU32(&x1657, x1655, x1632, x1607);
    var x1658: u32 = undefined;
    cmovznzU32(&x1658, x1655, x1634, x1609);
    var x1659: u32 = undefined;
    cmovznzU32(&x1659, x1655, x1636, x1611);
    var x1660: u32 = undefined;
    cmovznzU32(&x1660, x1655, x1638, x1613);
    var x1661: u32 = undefined;
    cmovznzU32(&x1661, x1655, x1640, x1615);
    var x1662: u32 = undefined;
    cmovznzU32(&x1662, x1655, x1642, x1617);
    var x1663: u32 = undefined;
    cmovznzU32(&x1663, x1655, x1644, x1619);
    var x1664: u32 = undefined;
    cmovznzU32(&x1664, x1655, x1646, x1621);
    var x1665: u32 = undefined;
    cmovznzU32(&x1665, x1655, x1648, x1623);
    var x1666: u32 = undefined;
    cmovznzU32(&x1666, x1655, x1650, x1625);
    var x1667: u32 = undefined;
    cmovznzU32(&x1667, x1655, x1652, x1627);
    out1[0] = x1656;
    out1[1] = x1657;
    out1[2] = x1658;
    out1[3] = x1659;
    out1[4] = x1660;
    out1[5] = x1661;
    out1[6] = x1662;
    out1[7] = x1663;
    out1[8] = x1664;
    out1[9] = x1665;
    out1[10] = x1666;
    out1[11] = x1667;
}

/// The function square squares a field element in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) * eval (from_montgomery arg1)) mod m
///   0 ≤ eval out1 < m
///
pub fn square(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[1]);
    const x2 = (arg1[2]);
    const x3 = (arg1[3]);
    const x4 = (arg1[4]);
    const x5 = (arg1[5]);
    const x6 = (arg1[6]);
    const x7 = (arg1[7]);
    const x8 = (arg1[8]);
    const x9 = (arg1[9]);
    const x10 = (arg1[10]);
    const x11 = (arg1[11]);
    const x12 = (arg1[0]);
    var x13: u32 = undefined;
    var x14: u32 = undefined;
    mulxU32(&x13, &x14, x12, (arg1[11]));
    var x15: u32 = undefined;
    var x16: u32 = undefined;
    mulxU32(&x15, &x16, x12, (arg1[10]));
    var x17: u32 = undefined;
    var x18: u32 = undefined;
    mulxU32(&x17, &x18, x12, (arg1[9]));
    var x19: u32 = undefined;
    var x20: u32 = undefined;
    mulxU32(&x19, &x20, x12, (arg1[8]));
    var x21: u32 = undefined;
    var x22: u32 = undefined;
    mulxU32(&x21, &x22, x12, (arg1[7]));
    var x23: u32 = undefined;
    var x24: u32 = undefined;
    mulxU32(&x23, &x24, x12, (arg1[6]));
    var x25: u32 = undefined;
    var x26: u32 = undefined;
    mulxU32(&x25, &x26, x12, (arg1[5]));
    var x27: u32 = undefined;
    var x28: u32 = undefined;
    mulxU32(&x27, &x28, x12, (arg1[4]));
    var x29: u32 = undefined;
    var x30: u32 = undefined;
    mulxU32(&x29, &x30, x12, (arg1[3]));
    var x31: u32 = undefined;
    var x32: u32 = undefined;
    mulxU32(&x31, &x32, x12, (arg1[2]));
    var x33: u32 = undefined;
    var x34: u32 = undefined;
    mulxU32(&x33, &x34, x12, (arg1[1]));
    var x35: u32 = undefined;
    var x36: u32 = undefined;
    mulxU32(&x35, &x36, x12, (arg1[0]));
    var x37: u32 = undefined;
    var x38: u1 = undefined;
    addcarryxU32(&x37, &x38, 0x0, x36, x33);
    var x39: u32 = undefined;
    var x40: u1 = undefined;
    addcarryxU32(&x39, &x40, x38, x34, x31);
    var x41: u32 = undefined;
    var x42: u1 = undefined;
    addcarryxU32(&x41, &x42, x40, x32, x29);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    addcarryxU32(&x43, &x44, x42, x30, x27);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    addcarryxU32(&x45, &x46, x44, x28, x25);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    addcarryxU32(&x47, &x48, x46, x26, x23);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    addcarryxU32(&x49, &x50, x48, x24, x21);
    var x51: u32 = undefined;
    var x52: u1 = undefined;
    addcarryxU32(&x51, &x52, x50, x22, x19);
    var x53: u32 = undefined;
    var x54: u1 = undefined;
    addcarryxU32(&x53, &x54, x52, x20, x17);
    var x55: u32 = undefined;
    var x56: u1 = undefined;
    addcarryxU32(&x55, &x56, x54, x18, x15);
    var x57: u32 = undefined;
    var x58: u1 = undefined;
    addcarryxU32(&x57, &x58, x56, x16, x13);
    const x59 = (cast(u32, x58) + x14);
    var x60: u32 = undefined;
    var x61: u32 = undefined;
    mulxU32(&x60, &x61, x35, 0xffffffff);
    var x62: u32 = undefined;
    var x63: u32 = undefined;
    mulxU32(&x62, &x63, x35, 0xffffffff);
    var x64: u32 = undefined;
    var x65: u32 = undefined;
    mulxU32(&x64, &x65, x35, 0xffffffff);
    var x66: u32 = undefined;
    var x67: u32 = undefined;
    mulxU32(&x66, &x67, x35, 0xffffffff);
    var x68: u32 = undefined;
    var x69: u32 = undefined;
    mulxU32(&x68, &x69, x35, 0xffffffff);
    var x70: u32 = undefined;
    var x71: u32 = undefined;
    mulxU32(&x70, &x71, x35, 0xffffffff);
    var x72: u32 = undefined;
    var x73: u32 = undefined;
    mulxU32(&x72, &x73, x35, 0xffffffff);
    var x74: u32 = undefined;
    var x75: u32 = undefined;
    mulxU32(&x74, &x75, x35, 0xfffffffe);
    var x76: u32 = undefined;
    var x77: u32 = undefined;
    mulxU32(&x76, &x77, x35, 0xffffffff);
    var x78: u32 = undefined;
    var x79: u32 = undefined;
    mulxU32(&x78, &x79, x35, 0xffffffff);
    var x80: u32 = undefined;
    var x81: u1 = undefined;
    addcarryxU32(&x80, &x81, 0x0, x77, x74);
    var x82: u32 = undefined;
    var x83: u1 = undefined;
    addcarryxU32(&x82, &x83, x81, x75, x72);
    var x84: u32 = undefined;
    var x85: u1 = undefined;
    addcarryxU32(&x84, &x85, x83, x73, x70);
    var x86: u32 = undefined;
    var x87: u1 = undefined;
    addcarryxU32(&x86, &x87, x85, x71, x68);
    var x88: u32 = undefined;
    var x89: u1 = undefined;
    addcarryxU32(&x88, &x89, x87, x69, x66);
    var x90: u32 = undefined;
    var x91: u1 = undefined;
    addcarryxU32(&x90, &x91, x89, x67, x64);
    var x92: u32 = undefined;
    var x93: u1 = undefined;
    addcarryxU32(&x92, &x93, x91, x65, x62);
    var x94: u32 = undefined;
    var x95: u1 = undefined;
    addcarryxU32(&x94, &x95, x93, x63, x60);
    const x96 = (cast(u32, x95) + x61);
    var x97: u32 = undefined;
    var x98: u1 = undefined;
    addcarryxU32(&x97, &x98, 0x0, x35, x78);
    var x99: u32 = undefined;
    var x100: u1 = undefined;
    addcarryxU32(&x99, &x100, x98, x37, x79);
    var x101: u32 = undefined;
    var x102: u1 = undefined;
    addcarryxU32(&x101, &x102, x100, x39, cast(u32, 0x0));
    var x103: u32 = undefined;
    var x104: u1 = undefined;
    addcarryxU32(&x103, &x104, x102, x41, x76);
    var x105: u32 = undefined;
    var x106: u1 = undefined;
    addcarryxU32(&x105, &x106, x104, x43, x80);
    var x107: u32 = undefined;
    var x108: u1 = undefined;
    addcarryxU32(&x107, &x108, x106, x45, x82);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    addcarryxU32(&x109, &x110, x108, x47, x84);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    addcarryxU32(&x111, &x112, x110, x49, x86);
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    addcarryxU32(&x113, &x114, x112, x51, x88);
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    addcarryxU32(&x115, &x116, x114, x53, x90);
    var x117: u32 = undefined;
    var x118: u1 = undefined;
    addcarryxU32(&x117, &x118, x116, x55, x92);
    var x119: u32 = undefined;
    var x120: u1 = undefined;
    addcarryxU32(&x119, &x120, x118, x57, x94);
    var x121: u32 = undefined;
    var x122: u1 = undefined;
    addcarryxU32(&x121, &x122, x120, x59, x96);
    var x123: u32 = undefined;
    var x124: u32 = undefined;
    mulxU32(&x123, &x124, x1, (arg1[11]));
    var x125: u32 = undefined;
    var x126: u32 = undefined;
    mulxU32(&x125, &x126, x1, (arg1[10]));
    var x127: u32 = undefined;
    var x128: u32 = undefined;
    mulxU32(&x127, &x128, x1, (arg1[9]));
    var x129: u32 = undefined;
    var x130: u32 = undefined;
    mulxU32(&x129, &x130, x1, (arg1[8]));
    var x131: u32 = undefined;
    var x132: u32 = undefined;
    mulxU32(&x131, &x132, x1, (arg1[7]));
    var x133: u32 = undefined;
    var x134: u32 = undefined;
    mulxU32(&x133, &x134, x1, (arg1[6]));
    var x135: u32 = undefined;
    var x136: u32 = undefined;
    mulxU32(&x135, &x136, x1, (arg1[5]));
    var x137: u32 = undefined;
    var x138: u32 = undefined;
    mulxU32(&x137, &x138, x1, (arg1[4]));
    var x139: u32 = undefined;
    var x140: u32 = undefined;
    mulxU32(&x139, &x140, x1, (arg1[3]));
    var x141: u32 = undefined;
    var x142: u32 = undefined;
    mulxU32(&x141, &x142, x1, (arg1[2]));
    var x143: u32 = undefined;
    var x144: u32 = undefined;
    mulxU32(&x143, &x144, x1, (arg1[1]));
    var x145: u32 = undefined;
    var x146: u32 = undefined;
    mulxU32(&x145, &x146, x1, (arg1[0]));
    var x147: u32 = undefined;
    var x148: u1 = undefined;
    addcarryxU32(&x147, &x148, 0x0, x146, x143);
    var x149: u32 = undefined;
    var x150: u1 = undefined;
    addcarryxU32(&x149, &x150, x148, x144, x141);
    var x151: u32 = undefined;
    var x152: u1 = undefined;
    addcarryxU32(&x151, &x152, x150, x142, x139);
    var x153: u32 = undefined;
    var x154: u1 = undefined;
    addcarryxU32(&x153, &x154, x152, x140, x137);
    var x155: u32 = undefined;
    var x156: u1 = undefined;
    addcarryxU32(&x155, &x156, x154, x138, x135);
    var x157: u32 = undefined;
    var x158: u1 = undefined;
    addcarryxU32(&x157, &x158, x156, x136, x133);
    var x159: u32 = undefined;
    var x160: u1 = undefined;
    addcarryxU32(&x159, &x160, x158, x134, x131);
    var x161: u32 = undefined;
    var x162: u1 = undefined;
    addcarryxU32(&x161, &x162, x160, x132, x129);
    var x163: u32 = undefined;
    var x164: u1 = undefined;
    addcarryxU32(&x163, &x164, x162, x130, x127);
    var x165: u32 = undefined;
    var x166: u1 = undefined;
    addcarryxU32(&x165, &x166, x164, x128, x125);
    var x167: u32 = undefined;
    var x168: u1 = undefined;
    addcarryxU32(&x167, &x168, x166, x126, x123);
    const x169 = (cast(u32, x168) + x124);
    var x170: u32 = undefined;
    var x171: u1 = undefined;
    addcarryxU32(&x170, &x171, 0x0, x99, x145);
    var x172: u32 = undefined;
    var x173: u1 = undefined;
    addcarryxU32(&x172, &x173, x171, x101, x147);
    var x174: u32 = undefined;
    var x175: u1 = undefined;
    addcarryxU32(&x174, &x175, x173, x103, x149);
    var x176: u32 = undefined;
    var x177: u1 = undefined;
    addcarryxU32(&x176, &x177, x175, x105, x151);
    var x178: u32 = undefined;
    var x179: u1 = undefined;
    addcarryxU32(&x178, &x179, x177, x107, x153);
    var x180: u32 = undefined;
    var x181: u1 = undefined;
    addcarryxU32(&x180, &x181, x179, x109, x155);
    var x182: u32 = undefined;
    var x183: u1 = undefined;
    addcarryxU32(&x182, &x183, x181, x111, x157);
    var x184: u32 = undefined;
    var x185: u1 = undefined;
    addcarryxU32(&x184, &x185, x183, x113, x159);
    var x186: u32 = undefined;
    var x187: u1 = undefined;
    addcarryxU32(&x186, &x187, x185, x115, x161);
    var x188: u32 = undefined;
    var x189: u1 = undefined;
    addcarryxU32(&x188, &x189, x187, x117, x163);
    var x190: u32 = undefined;
    var x191: u1 = undefined;
    addcarryxU32(&x190, &x191, x189, x119, x165);
    var x192: u32 = undefined;
    var x193: u1 = undefined;
    addcarryxU32(&x192, &x193, x191, x121, x167);
    var x194: u32 = undefined;
    var x195: u1 = undefined;
    addcarryxU32(&x194, &x195, x193, cast(u32, x122), x169);
    var x196: u32 = undefined;
    var x197: u32 = undefined;
    mulxU32(&x196, &x197, x170, 0xffffffff);
    var x198: u32 = undefined;
    var x199: u32 = undefined;
    mulxU32(&x198, &x199, x170, 0xffffffff);
    var x200: u32 = undefined;
    var x201: u32 = undefined;
    mulxU32(&x200, &x201, x170, 0xffffffff);
    var x202: u32 = undefined;
    var x203: u32 = undefined;
    mulxU32(&x202, &x203, x170, 0xffffffff);
    var x204: u32 = undefined;
    var x205: u32 = undefined;
    mulxU32(&x204, &x205, x170, 0xffffffff);
    var x206: u32 = undefined;
    var x207: u32 = undefined;
    mulxU32(&x206, &x207, x170, 0xffffffff);
    var x208: u32 = undefined;
    var x209: u32 = undefined;
    mulxU32(&x208, &x209, x170, 0xffffffff);
    var x210: u32 = undefined;
    var x211: u32 = undefined;
    mulxU32(&x210, &x211, x170, 0xfffffffe);
    var x212: u32 = undefined;
    var x213: u32 = undefined;
    mulxU32(&x212, &x213, x170, 0xffffffff);
    var x214: u32 = undefined;
    var x215: u32 = undefined;
    mulxU32(&x214, &x215, x170, 0xffffffff);
    var x216: u32 = undefined;
    var x217: u1 = undefined;
    addcarryxU32(&x216, &x217, 0x0, x213, x210);
    var x218: u32 = undefined;
    var x219: u1 = undefined;
    addcarryxU32(&x218, &x219, x217, x211, x208);
    var x220: u32 = undefined;
    var x221: u1 = undefined;
    addcarryxU32(&x220, &x221, x219, x209, x206);
    var x222: u32 = undefined;
    var x223: u1 = undefined;
    addcarryxU32(&x222, &x223, x221, x207, x204);
    var x224: u32 = undefined;
    var x225: u1 = undefined;
    addcarryxU32(&x224, &x225, x223, x205, x202);
    var x226: u32 = undefined;
    var x227: u1 = undefined;
    addcarryxU32(&x226, &x227, x225, x203, x200);
    var x228: u32 = undefined;
    var x229: u1 = undefined;
    addcarryxU32(&x228, &x229, x227, x201, x198);
    var x230: u32 = undefined;
    var x231: u1 = undefined;
    addcarryxU32(&x230, &x231, x229, x199, x196);
    const x232 = (cast(u32, x231) + x197);
    var x233: u32 = undefined;
    var x234: u1 = undefined;
    addcarryxU32(&x233, &x234, 0x0, x170, x214);
    var x235: u32 = undefined;
    var x236: u1 = undefined;
    addcarryxU32(&x235, &x236, x234, x172, x215);
    var x237: u32 = undefined;
    var x238: u1 = undefined;
    addcarryxU32(&x237, &x238, x236, x174, cast(u32, 0x0));
    var x239: u32 = undefined;
    var x240: u1 = undefined;
    addcarryxU32(&x239, &x240, x238, x176, x212);
    var x241: u32 = undefined;
    var x242: u1 = undefined;
    addcarryxU32(&x241, &x242, x240, x178, x216);
    var x243: u32 = undefined;
    var x244: u1 = undefined;
    addcarryxU32(&x243, &x244, x242, x180, x218);
    var x245: u32 = undefined;
    var x246: u1 = undefined;
    addcarryxU32(&x245, &x246, x244, x182, x220);
    var x247: u32 = undefined;
    var x248: u1 = undefined;
    addcarryxU32(&x247, &x248, x246, x184, x222);
    var x249: u32 = undefined;
    var x250: u1 = undefined;
    addcarryxU32(&x249, &x250, x248, x186, x224);
    var x251: u32 = undefined;
    var x252: u1 = undefined;
    addcarryxU32(&x251, &x252, x250, x188, x226);
    var x253: u32 = undefined;
    var x254: u1 = undefined;
    addcarryxU32(&x253, &x254, x252, x190, x228);
    var x255: u32 = undefined;
    var x256: u1 = undefined;
    addcarryxU32(&x255, &x256, x254, x192, x230);
    var x257: u32 = undefined;
    var x258: u1 = undefined;
    addcarryxU32(&x257, &x258, x256, x194, x232);
    const x259 = (cast(u32, x258) + cast(u32, x195));
    var x260: u32 = undefined;
    var x261: u32 = undefined;
    mulxU32(&x260, &x261, x2, (arg1[11]));
    var x262: u32 = undefined;
    var x263: u32 = undefined;
    mulxU32(&x262, &x263, x2, (arg1[10]));
    var x264: u32 = undefined;
    var x265: u32 = undefined;
    mulxU32(&x264, &x265, x2, (arg1[9]));
    var x266: u32 = undefined;
    var x267: u32 = undefined;
    mulxU32(&x266, &x267, x2, (arg1[8]));
    var x268: u32 = undefined;
    var x269: u32 = undefined;
    mulxU32(&x268, &x269, x2, (arg1[7]));
    var x270: u32 = undefined;
    var x271: u32 = undefined;
    mulxU32(&x270, &x271, x2, (arg1[6]));
    var x272: u32 = undefined;
    var x273: u32 = undefined;
    mulxU32(&x272, &x273, x2, (arg1[5]));
    var x274: u32 = undefined;
    var x275: u32 = undefined;
    mulxU32(&x274, &x275, x2, (arg1[4]));
    var x276: u32 = undefined;
    var x277: u32 = undefined;
    mulxU32(&x276, &x277, x2, (arg1[3]));
    var x278: u32 = undefined;
    var x279: u32 = undefined;
    mulxU32(&x278, &x279, x2, (arg1[2]));
    var x280: u32 = undefined;
    var x281: u32 = undefined;
    mulxU32(&x280, &x281, x2, (arg1[1]));
    var x282: u32 = undefined;
    var x283: u32 = undefined;
    mulxU32(&x282, &x283, x2, (arg1[0]));
    var x284: u32 = undefined;
    var x285: u1 = undefined;
    addcarryxU32(&x284, &x285, 0x0, x283, x280);
    var x286: u32 = undefined;
    var x287: u1 = undefined;
    addcarryxU32(&x286, &x287, x285, x281, x278);
    var x288: u32 = undefined;
    var x289: u1 = undefined;
    addcarryxU32(&x288, &x289, x287, x279, x276);
    var x290: u32 = undefined;
    var x291: u1 = undefined;
    addcarryxU32(&x290, &x291, x289, x277, x274);
    var x292: u32 = undefined;
    var x293: u1 = undefined;
    addcarryxU32(&x292, &x293, x291, x275, x272);
    var x294: u32 = undefined;
    var x295: u1 = undefined;
    addcarryxU32(&x294, &x295, x293, x273, x270);
    var x296: u32 = undefined;
    var x297: u1 = undefined;
    addcarryxU32(&x296, &x297, x295, x271, x268);
    var x298: u32 = undefined;
    var x299: u1 = undefined;
    addcarryxU32(&x298, &x299, x297, x269, x266);
    var x300: u32 = undefined;
    var x301: u1 = undefined;
    addcarryxU32(&x300, &x301, x299, x267, x264);
    var x302: u32 = undefined;
    var x303: u1 = undefined;
    addcarryxU32(&x302, &x303, x301, x265, x262);
    var x304: u32 = undefined;
    var x305: u1 = undefined;
    addcarryxU32(&x304, &x305, x303, x263, x260);
    const x306 = (cast(u32, x305) + x261);
    var x307: u32 = undefined;
    var x308: u1 = undefined;
    addcarryxU32(&x307, &x308, 0x0, x235, x282);
    var x309: u32 = undefined;
    var x310: u1 = undefined;
    addcarryxU32(&x309, &x310, x308, x237, x284);
    var x311: u32 = undefined;
    var x312: u1 = undefined;
    addcarryxU32(&x311, &x312, x310, x239, x286);
    var x313: u32 = undefined;
    var x314: u1 = undefined;
    addcarryxU32(&x313, &x314, x312, x241, x288);
    var x315: u32 = undefined;
    var x316: u1 = undefined;
    addcarryxU32(&x315, &x316, x314, x243, x290);
    var x317: u32 = undefined;
    var x318: u1 = undefined;
    addcarryxU32(&x317, &x318, x316, x245, x292);
    var x319: u32 = undefined;
    var x320: u1 = undefined;
    addcarryxU32(&x319, &x320, x318, x247, x294);
    var x321: u32 = undefined;
    var x322: u1 = undefined;
    addcarryxU32(&x321, &x322, x320, x249, x296);
    var x323: u32 = undefined;
    var x324: u1 = undefined;
    addcarryxU32(&x323, &x324, x322, x251, x298);
    var x325: u32 = undefined;
    var x326: u1 = undefined;
    addcarryxU32(&x325, &x326, x324, x253, x300);
    var x327: u32 = undefined;
    var x328: u1 = undefined;
    addcarryxU32(&x327, &x328, x326, x255, x302);
    var x329: u32 = undefined;
    var x330: u1 = undefined;
    addcarryxU32(&x329, &x330, x328, x257, x304);
    var x331: u32 = undefined;
    var x332: u1 = undefined;
    addcarryxU32(&x331, &x332, x330, x259, x306);
    var x333: u32 = undefined;
    var x334: u32 = undefined;
    mulxU32(&x333, &x334, x307, 0xffffffff);
    var x335: u32 = undefined;
    var x336: u32 = undefined;
    mulxU32(&x335, &x336, x307, 0xffffffff);
    var x337: u32 = undefined;
    var x338: u32 = undefined;
    mulxU32(&x337, &x338, x307, 0xffffffff);
    var x339: u32 = undefined;
    var x340: u32 = undefined;
    mulxU32(&x339, &x340, x307, 0xffffffff);
    var x341: u32 = undefined;
    var x342: u32 = undefined;
    mulxU32(&x341, &x342, x307, 0xffffffff);
    var x343: u32 = undefined;
    var x344: u32 = undefined;
    mulxU32(&x343, &x344, x307, 0xffffffff);
    var x345: u32 = undefined;
    var x346: u32 = undefined;
    mulxU32(&x345, &x346, x307, 0xffffffff);
    var x347: u32 = undefined;
    var x348: u32 = undefined;
    mulxU32(&x347, &x348, x307, 0xfffffffe);
    var x349: u32 = undefined;
    var x350: u32 = undefined;
    mulxU32(&x349, &x350, x307, 0xffffffff);
    var x351: u32 = undefined;
    var x352: u32 = undefined;
    mulxU32(&x351, &x352, x307, 0xffffffff);
    var x353: u32 = undefined;
    var x354: u1 = undefined;
    addcarryxU32(&x353, &x354, 0x0, x350, x347);
    var x355: u32 = undefined;
    var x356: u1 = undefined;
    addcarryxU32(&x355, &x356, x354, x348, x345);
    var x357: u32 = undefined;
    var x358: u1 = undefined;
    addcarryxU32(&x357, &x358, x356, x346, x343);
    var x359: u32 = undefined;
    var x360: u1 = undefined;
    addcarryxU32(&x359, &x360, x358, x344, x341);
    var x361: u32 = undefined;
    var x362: u1 = undefined;
    addcarryxU32(&x361, &x362, x360, x342, x339);
    var x363: u32 = undefined;
    var x364: u1 = undefined;
    addcarryxU32(&x363, &x364, x362, x340, x337);
    var x365: u32 = undefined;
    var x366: u1 = undefined;
    addcarryxU32(&x365, &x366, x364, x338, x335);
    var x367: u32 = undefined;
    var x368: u1 = undefined;
    addcarryxU32(&x367, &x368, x366, x336, x333);
    const x369 = (cast(u32, x368) + x334);
    var x370: u32 = undefined;
    var x371: u1 = undefined;
    addcarryxU32(&x370, &x371, 0x0, x307, x351);
    var x372: u32 = undefined;
    var x373: u1 = undefined;
    addcarryxU32(&x372, &x373, x371, x309, x352);
    var x374: u32 = undefined;
    var x375: u1 = undefined;
    addcarryxU32(&x374, &x375, x373, x311, cast(u32, 0x0));
    var x376: u32 = undefined;
    var x377: u1 = undefined;
    addcarryxU32(&x376, &x377, x375, x313, x349);
    var x378: u32 = undefined;
    var x379: u1 = undefined;
    addcarryxU32(&x378, &x379, x377, x315, x353);
    var x380: u32 = undefined;
    var x381: u1 = undefined;
    addcarryxU32(&x380, &x381, x379, x317, x355);
    var x382: u32 = undefined;
    var x383: u1 = undefined;
    addcarryxU32(&x382, &x383, x381, x319, x357);
    var x384: u32 = undefined;
    var x385: u1 = undefined;
    addcarryxU32(&x384, &x385, x383, x321, x359);
    var x386: u32 = undefined;
    var x387: u1 = undefined;
    addcarryxU32(&x386, &x387, x385, x323, x361);
    var x388: u32 = undefined;
    var x389: u1 = undefined;
    addcarryxU32(&x388, &x389, x387, x325, x363);
    var x390: u32 = undefined;
    var x391: u1 = undefined;
    addcarryxU32(&x390, &x391, x389, x327, x365);
    var x392: u32 = undefined;
    var x393: u1 = undefined;
    addcarryxU32(&x392, &x393, x391, x329, x367);
    var x394: u32 = undefined;
    var x395: u1 = undefined;
    addcarryxU32(&x394, &x395, x393, x331, x369);
    const x396 = (cast(u32, x395) + cast(u32, x332));
    var x397: u32 = undefined;
    var x398: u32 = undefined;
    mulxU32(&x397, &x398, x3, (arg1[11]));
    var x399: u32 = undefined;
    var x400: u32 = undefined;
    mulxU32(&x399, &x400, x3, (arg1[10]));
    var x401: u32 = undefined;
    var x402: u32 = undefined;
    mulxU32(&x401, &x402, x3, (arg1[9]));
    var x403: u32 = undefined;
    var x404: u32 = undefined;
    mulxU32(&x403, &x404, x3, (arg1[8]));
    var x405: u32 = undefined;
    var x406: u32 = undefined;
    mulxU32(&x405, &x406, x3, (arg1[7]));
    var x407: u32 = undefined;
    var x408: u32 = undefined;
    mulxU32(&x407, &x408, x3, (arg1[6]));
    var x409: u32 = undefined;
    var x410: u32 = undefined;
    mulxU32(&x409, &x410, x3, (arg1[5]));
    var x411: u32 = undefined;
    var x412: u32 = undefined;
    mulxU32(&x411, &x412, x3, (arg1[4]));
    var x413: u32 = undefined;
    var x414: u32 = undefined;
    mulxU32(&x413, &x414, x3, (arg1[3]));
    var x415: u32 = undefined;
    var x416: u32 = undefined;
    mulxU32(&x415, &x416, x3, (arg1[2]));
    var x417: u32 = undefined;
    var x418: u32 = undefined;
    mulxU32(&x417, &x418, x3, (arg1[1]));
    var x419: u32 = undefined;
    var x420: u32 = undefined;
    mulxU32(&x419, &x420, x3, (arg1[0]));
    var x421: u32 = undefined;
    var x422: u1 = undefined;
    addcarryxU32(&x421, &x422, 0x0, x420, x417);
    var x423: u32 = undefined;
    var x424: u1 = undefined;
    addcarryxU32(&x423, &x424, x422, x418, x415);
    var x425: u32 = undefined;
    var x426: u1 = undefined;
    addcarryxU32(&x425, &x426, x424, x416, x413);
    var x427: u32 = undefined;
    var x428: u1 = undefined;
    addcarryxU32(&x427, &x428, x426, x414, x411);
    var x429: u32 = undefined;
    var x430: u1 = undefined;
    addcarryxU32(&x429, &x430, x428, x412, x409);
    var x431: u32 = undefined;
    var x432: u1 = undefined;
    addcarryxU32(&x431, &x432, x430, x410, x407);
    var x433: u32 = undefined;
    var x434: u1 = undefined;
    addcarryxU32(&x433, &x434, x432, x408, x405);
    var x435: u32 = undefined;
    var x436: u1 = undefined;
    addcarryxU32(&x435, &x436, x434, x406, x403);
    var x437: u32 = undefined;
    var x438: u1 = undefined;
    addcarryxU32(&x437, &x438, x436, x404, x401);
    var x439: u32 = undefined;
    var x440: u1 = undefined;
    addcarryxU32(&x439, &x440, x438, x402, x399);
    var x441: u32 = undefined;
    var x442: u1 = undefined;
    addcarryxU32(&x441, &x442, x440, x400, x397);
    const x443 = (cast(u32, x442) + x398);
    var x444: u32 = undefined;
    var x445: u1 = undefined;
    addcarryxU32(&x444, &x445, 0x0, x372, x419);
    var x446: u32 = undefined;
    var x447: u1 = undefined;
    addcarryxU32(&x446, &x447, x445, x374, x421);
    var x448: u32 = undefined;
    var x449: u1 = undefined;
    addcarryxU32(&x448, &x449, x447, x376, x423);
    var x450: u32 = undefined;
    var x451: u1 = undefined;
    addcarryxU32(&x450, &x451, x449, x378, x425);
    var x452: u32 = undefined;
    var x453: u1 = undefined;
    addcarryxU32(&x452, &x453, x451, x380, x427);
    var x454: u32 = undefined;
    var x455: u1 = undefined;
    addcarryxU32(&x454, &x455, x453, x382, x429);
    var x456: u32 = undefined;
    var x457: u1 = undefined;
    addcarryxU32(&x456, &x457, x455, x384, x431);
    var x458: u32 = undefined;
    var x459: u1 = undefined;
    addcarryxU32(&x458, &x459, x457, x386, x433);
    var x460: u32 = undefined;
    var x461: u1 = undefined;
    addcarryxU32(&x460, &x461, x459, x388, x435);
    var x462: u32 = undefined;
    var x463: u1 = undefined;
    addcarryxU32(&x462, &x463, x461, x390, x437);
    var x464: u32 = undefined;
    var x465: u1 = undefined;
    addcarryxU32(&x464, &x465, x463, x392, x439);
    var x466: u32 = undefined;
    var x467: u1 = undefined;
    addcarryxU32(&x466, &x467, x465, x394, x441);
    var x468: u32 = undefined;
    var x469: u1 = undefined;
    addcarryxU32(&x468, &x469, x467, x396, x443);
    var x470: u32 = undefined;
    var x471: u32 = undefined;
    mulxU32(&x470, &x471, x444, 0xffffffff);
    var x472: u32 = undefined;
    var x473: u32 = undefined;
    mulxU32(&x472, &x473, x444, 0xffffffff);
    var x474: u32 = undefined;
    var x475: u32 = undefined;
    mulxU32(&x474, &x475, x444, 0xffffffff);
    var x476: u32 = undefined;
    var x477: u32 = undefined;
    mulxU32(&x476, &x477, x444, 0xffffffff);
    var x478: u32 = undefined;
    var x479: u32 = undefined;
    mulxU32(&x478, &x479, x444, 0xffffffff);
    var x480: u32 = undefined;
    var x481: u32 = undefined;
    mulxU32(&x480, &x481, x444, 0xffffffff);
    var x482: u32 = undefined;
    var x483: u32 = undefined;
    mulxU32(&x482, &x483, x444, 0xffffffff);
    var x484: u32 = undefined;
    var x485: u32 = undefined;
    mulxU32(&x484, &x485, x444, 0xfffffffe);
    var x486: u32 = undefined;
    var x487: u32 = undefined;
    mulxU32(&x486, &x487, x444, 0xffffffff);
    var x488: u32 = undefined;
    var x489: u32 = undefined;
    mulxU32(&x488, &x489, x444, 0xffffffff);
    var x490: u32 = undefined;
    var x491: u1 = undefined;
    addcarryxU32(&x490, &x491, 0x0, x487, x484);
    var x492: u32 = undefined;
    var x493: u1 = undefined;
    addcarryxU32(&x492, &x493, x491, x485, x482);
    var x494: u32 = undefined;
    var x495: u1 = undefined;
    addcarryxU32(&x494, &x495, x493, x483, x480);
    var x496: u32 = undefined;
    var x497: u1 = undefined;
    addcarryxU32(&x496, &x497, x495, x481, x478);
    var x498: u32 = undefined;
    var x499: u1 = undefined;
    addcarryxU32(&x498, &x499, x497, x479, x476);
    var x500: u32 = undefined;
    var x501: u1 = undefined;
    addcarryxU32(&x500, &x501, x499, x477, x474);
    var x502: u32 = undefined;
    var x503: u1 = undefined;
    addcarryxU32(&x502, &x503, x501, x475, x472);
    var x504: u32 = undefined;
    var x505: u1 = undefined;
    addcarryxU32(&x504, &x505, x503, x473, x470);
    const x506 = (cast(u32, x505) + x471);
    var x507: u32 = undefined;
    var x508: u1 = undefined;
    addcarryxU32(&x507, &x508, 0x0, x444, x488);
    var x509: u32 = undefined;
    var x510: u1 = undefined;
    addcarryxU32(&x509, &x510, x508, x446, x489);
    var x511: u32 = undefined;
    var x512: u1 = undefined;
    addcarryxU32(&x511, &x512, x510, x448, cast(u32, 0x0));
    var x513: u32 = undefined;
    var x514: u1 = undefined;
    addcarryxU32(&x513, &x514, x512, x450, x486);
    var x515: u32 = undefined;
    var x516: u1 = undefined;
    addcarryxU32(&x515, &x516, x514, x452, x490);
    var x517: u32 = undefined;
    var x518: u1 = undefined;
    addcarryxU32(&x517, &x518, x516, x454, x492);
    var x519: u32 = undefined;
    var x520: u1 = undefined;
    addcarryxU32(&x519, &x520, x518, x456, x494);
    var x521: u32 = undefined;
    var x522: u1 = undefined;
    addcarryxU32(&x521, &x522, x520, x458, x496);
    var x523: u32 = undefined;
    var x524: u1 = undefined;
    addcarryxU32(&x523, &x524, x522, x460, x498);
    var x525: u32 = undefined;
    var x526: u1 = undefined;
    addcarryxU32(&x525, &x526, x524, x462, x500);
    var x527: u32 = undefined;
    var x528: u1 = undefined;
    addcarryxU32(&x527, &x528, x526, x464, x502);
    var x529: u32 = undefined;
    var x530: u1 = undefined;
    addcarryxU32(&x529, &x530, x528, x466, x504);
    var x531: u32 = undefined;
    var x532: u1 = undefined;
    addcarryxU32(&x531, &x532, x530, x468, x506);
    const x533 = (cast(u32, x532) + cast(u32, x469));
    var x534: u32 = undefined;
    var x535: u32 = undefined;
    mulxU32(&x534, &x535, x4, (arg1[11]));
    var x536: u32 = undefined;
    var x537: u32 = undefined;
    mulxU32(&x536, &x537, x4, (arg1[10]));
    var x538: u32 = undefined;
    var x539: u32 = undefined;
    mulxU32(&x538, &x539, x4, (arg1[9]));
    var x540: u32 = undefined;
    var x541: u32 = undefined;
    mulxU32(&x540, &x541, x4, (arg1[8]));
    var x542: u32 = undefined;
    var x543: u32 = undefined;
    mulxU32(&x542, &x543, x4, (arg1[7]));
    var x544: u32 = undefined;
    var x545: u32 = undefined;
    mulxU32(&x544, &x545, x4, (arg1[6]));
    var x546: u32 = undefined;
    var x547: u32 = undefined;
    mulxU32(&x546, &x547, x4, (arg1[5]));
    var x548: u32 = undefined;
    var x549: u32 = undefined;
    mulxU32(&x548, &x549, x4, (arg1[4]));
    var x550: u32 = undefined;
    var x551: u32 = undefined;
    mulxU32(&x550, &x551, x4, (arg1[3]));
    var x552: u32 = undefined;
    var x553: u32 = undefined;
    mulxU32(&x552, &x553, x4, (arg1[2]));
    var x554: u32 = undefined;
    var x555: u32 = undefined;
    mulxU32(&x554, &x555, x4, (arg1[1]));
    var x556: u32 = undefined;
    var x557: u32 = undefined;
    mulxU32(&x556, &x557, x4, (arg1[0]));
    var x558: u32 = undefined;
    var x559: u1 = undefined;
    addcarryxU32(&x558, &x559, 0x0, x557, x554);
    var x560: u32 = undefined;
    var x561: u1 = undefined;
    addcarryxU32(&x560, &x561, x559, x555, x552);
    var x562: u32 = undefined;
    var x563: u1 = undefined;
    addcarryxU32(&x562, &x563, x561, x553, x550);
    var x564: u32 = undefined;
    var x565: u1 = undefined;
    addcarryxU32(&x564, &x565, x563, x551, x548);
    var x566: u32 = undefined;
    var x567: u1 = undefined;
    addcarryxU32(&x566, &x567, x565, x549, x546);
    var x568: u32 = undefined;
    var x569: u1 = undefined;
    addcarryxU32(&x568, &x569, x567, x547, x544);
    var x570: u32 = undefined;
    var x571: u1 = undefined;
    addcarryxU32(&x570, &x571, x569, x545, x542);
    var x572: u32 = undefined;
    var x573: u1 = undefined;
    addcarryxU32(&x572, &x573, x571, x543, x540);
    var x574: u32 = undefined;
    var x575: u1 = undefined;
    addcarryxU32(&x574, &x575, x573, x541, x538);
    var x576: u32 = undefined;
    var x577: u1 = undefined;
    addcarryxU32(&x576, &x577, x575, x539, x536);
    var x578: u32 = undefined;
    var x579: u1 = undefined;
    addcarryxU32(&x578, &x579, x577, x537, x534);
    const x580 = (cast(u32, x579) + x535);
    var x581: u32 = undefined;
    var x582: u1 = undefined;
    addcarryxU32(&x581, &x582, 0x0, x509, x556);
    var x583: u32 = undefined;
    var x584: u1 = undefined;
    addcarryxU32(&x583, &x584, x582, x511, x558);
    var x585: u32 = undefined;
    var x586: u1 = undefined;
    addcarryxU32(&x585, &x586, x584, x513, x560);
    var x587: u32 = undefined;
    var x588: u1 = undefined;
    addcarryxU32(&x587, &x588, x586, x515, x562);
    var x589: u32 = undefined;
    var x590: u1 = undefined;
    addcarryxU32(&x589, &x590, x588, x517, x564);
    var x591: u32 = undefined;
    var x592: u1 = undefined;
    addcarryxU32(&x591, &x592, x590, x519, x566);
    var x593: u32 = undefined;
    var x594: u1 = undefined;
    addcarryxU32(&x593, &x594, x592, x521, x568);
    var x595: u32 = undefined;
    var x596: u1 = undefined;
    addcarryxU32(&x595, &x596, x594, x523, x570);
    var x597: u32 = undefined;
    var x598: u1 = undefined;
    addcarryxU32(&x597, &x598, x596, x525, x572);
    var x599: u32 = undefined;
    var x600: u1 = undefined;
    addcarryxU32(&x599, &x600, x598, x527, x574);
    var x601: u32 = undefined;
    var x602: u1 = undefined;
    addcarryxU32(&x601, &x602, x600, x529, x576);
    var x603: u32 = undefined;
    var x604: u1 = undefined;
    addcarryxU32(&x603, &x604, x602, x531, x578);
    var x605: u32 = undefined;
    var x606: u1 = undefined;
    addcarryxU32(&x605, &x606, x604, x533, x580);
    var x607: u32 = undefined;
    var x608: u32 = undefined;
    mulxU32(&x607, &x608, x581, 0xffffffff);
    var x609: u32 = undefined;
    var x610: u32 = undefined;
    mulxU32(&x609, &x610, x581, 0xffffffff);
    var x611: u32 = undefined;
    var x612: u32 = undefined;
    mulxU32(&x611, &x612, x581, 0xffffffff);
    var x613: u32 = undefined;
    var x614: u32 = undefined;
    mulxU32(&x613, &x614, x581, 0xffffffff);
    var x615: u32 = undefined;
    var x616: u32 = undefined;
    mulxU32(&x615, &x616, x581, 0xffffffff);
    var x617: u32 = undefined;
    var x618: u32 = undefined;
    mulxU32(&x617, &x618, x581, 0xffffffff);
    var x619: u32 = undefined;
    var x620: u32 = undefined;
    mulxU32(&x619, &x620, x581, 0xffffffff);
    var x621: u32 = undefined;
    var x622: u32 = undefined;
    mulxU32(&x621, &x622, x581, 0xfffffffe);
    var x623: u32 = undefined;
    var x624: u32 = undefined;
    mulxU32(&x623, &x624, x581, 0xffffffff);
    var x625: u32 = undefined;
    var x626: u32 = undefined;
    mulxU32(&x625, &x626, x581, 0xffffffff);
    var x627: u32 = undefined;
    var x628: u1 = undefined;
    addcarryxU32(&x627, &x628, 0x0, x624, x621);
    var x629: u32 = undefined;
    var x630: u1 = undefined;
    addcarryxU32(&x629, &x630, x628, x622, x619);
    var x631: u32 = undefined;
    var x632: u1 = undefined;
    addcarryxU32(&x631, &x632, x630, x620, x617);
    var x633: u32 = undefined;
    var x634: u1 = undefined;
    addcarryxU32(&x633, &x634, x632, x618, x615);
    var x635: u32 = undefined;
    var x636: u1 = undefined;
    addcarryxU32(&x635, &x636, x634, x616, x613);
    var x637: u32 = undefined;
    var x638: u1 = undefined;
    addcarryxU32(&x637, &x638, x636, x614, x611);
    var x639: u32 = undefined;
    var x640: u1 = undefined;
    addcarryxU32(&x639, &x640, x638, x612, x609);
    var x641: u32 = undefined;
    var x642: u1 = undefined;
    addcarryxU32(&x641, &x642, x640, x610, x607);
    const x643 = (cast(u32, x642) + x608);
    var x644: u32 = undefined;
    var x645: u1 = undefined;
    addcarryxU32(&x644, &x645, 0x0, x581, x625);
    var x646: u32 = undefined;
    var x647: u1 = undefined;
    addcarryxU32(&x646, &x647, x645, x583, x626);
    var x648: u32 = undefined;
    var x649: u1 = undefined;
    addcarryxU32(&x648, &x649, x647, x585, cast(u32, 0x0));
    var x650: u32 = undefined;
    var x651: u1 = undefined;
    addcarryxU32(&x650, &x651, x649, x587, x623);
    var x652: u32 = undefined;
    var x653: u1 = undefined;
    addcarryxU32(&x652, &x653, x651, x589, x627);
    var x654: u32 = undefined;
    var x655: u1 = undefined;
    addcarryxU32(&x654, &x655, x653, x591, x629);
    var x656: u32 = undefined;
    var x657: u1 = undefined;
    addcarryxU32(&x656, &x657, x655, x593, x631);
    var x658: u32 = undefined;
    var x659: u1 = undefined;
    addcarryxU32(&x658, &x659, x657, x595, x633);
    var x660: u32 = undefined;
    var x661: u1 = undefined;
    addcarryxU32(&x660, &x661, x659, x597, x635);
    var x662: u32 = undefined;
    var x663: u1 = undefined;
    addcarryxU32(&x662, &x663, x661, x599, x637);
    var x664: u32 = undefined;
    var x665: u1 = undefined;
    addcarryxU32(&x664, &x665, x663, x601, x639);
    var x666: u32 = undefined;
    var x667: u1 = undefined;
    addcarryxU32(&x666, &x667, x665, x603, x641);
    var x668: u32 = undefined;
    var x669: u1 = undefined;
    addcarryxU32(&x668, &x669, x667, x605, x643);
    const x670 = (cast(u32, x669) + cast(u32, x606));
    var x671: u32 = undefined;
    var x672: u32 = undefined;
    mulxU32(&x671, &x672, x5, (arg1[11]));
    var x673: u32 = undefined;
    var x674: u32 = undefined;
    mulxU32(&x673, &x674, x5, (arg1[10]));
    var x675: u32 = undefined;
    var x676: u32 = undefined;
    mulxU32(&x675, &x676, x5, (arg1[9]));
    var x677: u32 = undefined;
    var x678: u32 = undefined;
    mulxU32(&x677, &x678, x5, (arg1[8]));
    var x679: u32 = undefined;
    var x680: u32 = undefined;
    mulxU32(&x679, &x680, x5, (arg1[7]));
    var x681: u32 = undefined;
    var x682: u32 = undefined;
    mulxU32(&x681, &x682, x5, (arg1[6]));
    var x683: u32 = undefined;
    var x684: u32 = undefined;
    mulxU32(&x683, &x684, x5, (arg1[5]));
    var x685: u32 = undefined;
    var x686: u32 = undefined;
    mulxU32(&x685, &x686, x5, (arg1[4]));
    var x687: u32 = undefined;
    var x688: u32 = undefined;
    mulxU32(&x687, &x688, x5, (arg1[3]));
    var x689: u32 = undefined;
    var x690: u32 = undefined;
    mulxU32(&x689, &x690, x5, (arg1[2]));
    var x691: u32 = undefined;
    var x692: u32 = undefined;
    mulxU32(&x691, &x692, x5, (arg1[1]));
    var x693: u32 = undefined;
    var x694: u32 = undefined;
    mulxU32(&x693, &x694, x5, (arg1[0]));
    var x695: u32 = undefined;
    var x696: u1 = undefined;
    addcarryxU32(&x695, &x696, 0x0, x694, x691);
    var x697: u32 = undefined;
    var x698: u1 = undefined;
    addcarryxU32(&x697, &x698, x696, x692, x689);
    var x699: u32 = undefined;
    var x700: u1 = undefined;
    addcarryxU32(&x699, &x700, x698, x690, x687);
    var x701: u32 = undefined;
    var x702: u1 = undefined;
    addcarryxU32(&x701, &x702, x700, x688, x685);
    var x703: u32 = undefined;
    var x704: u1 = undefined;
    addcarryxU32(&x703, &x704, x702, x686, x683);
    var x705: u32 = undefined;
    var x706: u1 = undefined;
    addcarryxU32(&x705, &x706, x704, x684, x681);
    var x707: u32 = undefined;
    var x708: u1 = undefined;
    addcarryxU32(&x707, &x708, x706, x682, x679);
    var x709: u32 = undefined;
    var x710: u1 = undefined;
    addcarryxU32(&x709, &x710, x708, x680, x677);
    var x711: u32 = undefined;
    var x712: u1 = undefined;
    addcarryxU32(&x711, &x712, x710, x678, x675);
    var x713: u32 = undefined;
    var x714: u1 = undefined;
    addcarryxU32(&x713, &x714, x712, x676, x673);
    var x715: u32 = undefined;
    var x716: u1 = undefined;
    addcarryxU32(&x715, &x716, x714, x674, x671);
    const x717 = (cast(u32, x716) + x672);
    var x718: u32 = undefined;
    var x719: u1 = undefined;
    addcarryxU32(&x718, &x719, 0x0, x646, x693);
    var x720: u32 = undefined;
    var x721: u1 = undefined;
    addcarryxU32(&x720, &x721, x719, x648, x695);
    var x722: u32 = undefined;
    var x723: u1 = undefined;
    addcarryxU32(&x722, &x723, x721, x650, x697);
    var x724: u32 = undefined;
    var x725: u1 = undefined;
    addcarryxU32(&x724, &x725, x723, x652, x699);
    var x726: u32 = undefined;
    var x727: u1 = undefined;
    addcarryxU32(&x726, &x727, x725, x654, x701);
    var x728: u32 = undefined;
    var x729: u1 = undefined;
    addcarryxU32(&x728, &x729, x727, x656, x703);
    var x730: u32 = undefined;
    var x731: u1 = undefined;
    addcarryxU32(&x730, &x731, x729, x658, x705);
    var x732: u32 = undefined;
    var x733: u1 = undefined;
    addcarryxU32(&x732, &x733, x731, x660, x707);
    var x734: u32 = undefined;
    var x735: u1 = undefined;
    addcarryxU32(&x734, &x735, x733, x662, x709);
    var x736: u32 = undefined;
    var x737: u1 = undefined;
    addcarryxU32(&x736, &x737, x735, x664, x711);
    var x738: u32 = undefined;
    var x739: u1 = undefined;
    addcarryxU32(&x738, &x739, x737, x666, x713);
    var x740: u32 = undefined;
    var x741: u1 = undefined;
    addcarryxU32(&x740, &x741, x739, x668, x715);
    var x742: u32 = undefined;
    var x743: u1 = undefined;
    addcarryxU32(&x742, &x743, x741, x670, x717);
    var x744: u32 = undefined;
    var x745: u32 = undefined;
    mulxU32(&x744, &x745, x718, 0xffffffff);
    var x746: u32 = undefined;
    var x747: u32 = undefined;
    mulxU32(&x746, &x747, x718, 0xffffffff);
    var x748: u32 = undefined;
    var x749: u32 = undefined;
    mulxU32(&x748, &x749, x718, 0xffffffff);
    var x750: u32 = undefined;
    var x751: u32 = undefined;
    mulxU32(&x750, &x751, x718, 0xffffffff);
    var x752: u32 = undefined;
    var x753: u32 = undefined;
    mulxU32(&x752, &x753, x718, 0xffffffff);
    var x754: u32 = undefined;
    var x755: u32 = undefined;
    mulxU32(&x754, &x755, x718, 0xffffffff);
    var x756: u32 = undefined;
    var x757: u32 = undefined;
    mulxU32(&x756, &x757, x718, 0xffffffff);
    var x758: u32 = undefined;
    var x759: u32 = undefined;
    mulxU32(&x758, &x759, x718, 0xfffffffe);
    var x760: u32 = undefined;
    var x761: u32 = undefined;
    mulxU32(&x760, &x761, x718, 0xffffffff);
    var x762: u32 = undefined;
    var x763: u32 = undefined;
    mulxU32(&x762, &x763, x718, 0xffffffff);
    var x764: u32 = undefined;
    var x765: u1 = undefined;
    addcarryxU32(&x764, &x765, 0x0, x761, x758);
    var x766: u32 = undefined;
    var x767: u1 = undefined;
    addcarryxU32(&x766, &x767, x765, x759, x756);
    var x768: u32 = undefined;
    var x769: u1 = undefined;
    addcarryxU32(&x768, &x769, x767, x757, x754);
    var x770: u32 = undefined;
    var x771: u1 = undefined;
    addcarryxU32(&x770, &x771, x769, x755, x752);
    var x772: u32 = undefined;
    var x773: u1 = undefined;
    addcarryxU32(&x772, &x773, x771, x753, x750);
    var x774: u32 = undefined;
    var x775: u1 = undefined;
    addcarryxU32(&x774, &x775, x773, x751, x748);
    var x776: u32 = undefined;
    var x777: u1 = undefined;
    addcarryxU32(&x776, &x777, x775, x749, x746);
    var x778: u32 = undefined;
    var x779: u1 = undefined;
    addcarryxU32(&x778, &x779, x777, x747, x744);
    const x780 = (cast(u32, x779) + x745);
    var x781: u32 = undefined;
    var x782: u1 = undefined;
    addcarryxU32(&x781, &x782, 0x0, x718, x762);
    var x783: u32 = undefined;
    var x784: u1 = undefined;
    addcarryxU32(&x783, &x784, x782, x720, x763);
    var x785: u32 = undefined;
    var x786: u1 = undefined;
    addcarryxU32(&x785, &x786, x784, x722, cast(u32, 0x0));
    var x787: u32 = undefined;
    var x788: u1 = undefined;
    addcarryxU32(&x787, &x788, x786, x724, x760);
    var x789: u32 = undefined;
    var x790: u1 = undefined;
    addcarryxU32(&x789, &x790, x788, x726, x764);
    var x791: u32 = undefined;
    var x792: u1 = undefined;
    addcarryxU32(&x791, &x792, x790, x728, x766);
    var x793: u32 = undefined;
    var x794: u1 = undefined;
    addcarryxU32(&x793, &x794, x792, x730, x768);
    var x795: u32 = undefined;
    var x796: u1 = undefined;
    addcarryxU32(&x795, &x796, x794, x732, x770);
    var x797: u32 = undefined;
    var x798: u1 = undefined;
    addcarryxU32(&x797, &x798, x796, x734, x772);
    var x799: u32 = undefined;
    var x800: u1 = undefined;
    addcarryxU32(&x799, &x800, x798, x736, x774);
    var x801: u32 = undefined;
    var x802: u1 = undefined;
    addcarryxU32(&x801, &x802, x800, x738, x776);
    var x803: u32 = undefined;
    var x804: u1 = undefined;
    addcarryxU32(&x803, &x804, x802, x740, x778);
    var x805: u32 = undefined;
    var x806: u1 = undefined;
    addcarryxU32(&x805, &x806, x804, x742, x780);
    const x807 = (cast(u32, x806) + cast(u32, x743));
    var x808: u32 = undefined;
    var x809: u32 = undefined;
    mulxU32(&x808, &x809, x6, (arg1[11]));
    var x810: u32 = undefined;
    var x811: u32 = undefined;
    mulxU32(&x810, &x811, x6, (arg1[10]));
    var x812: u32 = undefined;
    var x813: u32 = undefined;
    mulxU32(&x812, &x813, x6, (arg1[9]));
    var x814: u32 = undefined;
    var x815: u32 = undefined;
    mulxU32(&x814, &x815, x6, (arg1[8]));
    var x816: u32 = undefined;
    var x817: u32 = undefined;
    mulxU32(&x816, &x817, x6, (arg1[7]));
    var x818: u32 = undefined;
    var x819: u32 = undefined;
    mulxU32(&x818, &x819, x6, (arg1[6]));
    var x820: u32 = undefined;
    var x821: u32 = undefined;
    mulxU32(&x820, &x821, x6, (arg1[5]));
    var x822: u32 = undefined;
    var x823: u32 = undefined;
    mulxU32(&x822, &x823, x6, (arg1[4]));
    var x824: u32 = undefined;
    var x825: u32 = undefined;
    mulxU32(&x824, &x825, x6, (arg1[3]));
    var x826: u32 = undefined;
    var x827: u32 = undefined;
    mulxU32(&x826, &x827, x6, (arg1[2]));
    var x828: u32 = undefined;
    var x829: u32 = undefined;
    mulxU32(&x828, &x829, x6, (arg1[1]));
    var x830: u32 = undefined;
    var x831: u32 = undefined;
    mulxU32(&x830, &x831, x6, (arg1[0]));
    var x832: u32 = undefined;
    var x833: u1 = undefined;
    addcarryxU32(&x832, &x833, 0x0, x831, x828);
    var x834: u32 = undefined;
    var x835: u1 = undefined;
    addcarryxU32(&x834, &x835, x833, x829, x826);
    var x836: u32 = undefined;
    var x837: u1 = undefined;
    addcarryxU32(&x836, &x837, x835, x827, x824);
    var x838: u32 = undefined;
    var x839: u1 = undefined;
    addcarryxU32(&x838, &x839, x837, x825, x822);
    var x840: u32 = undefined;
    var x841: u1 = undefined;
    addcarryxU32(&x840, &x841, x839, x823, x820);
    var x842: u32 = undefined;
    var x843: u1 = undefined;
    addcarryxU32(&x842, &x843, x841, x821, x818);
    var x844: u32 = undefined;
    var x845: u1 = undefined;
    addcarryxU32(&x844, &x845, x843, x819, x816);
    var x846: u32 = undefined;
    var x847: u1 = undefined;
    addcarryxU32(&x846, &x847, x845, x817, x814);
    var x848: u32 = undefined;
    var x849: u1 = undefined;
    addcarryxU32(&x848, &x849, x847, x815, x812);
    var x850: u32 = undefined;
    var x851: u1 = undefined;
    addcarryxU32(&x850, &x851, x849, x813, x810);
    var x852: u32 = undefined;
    var x853: u1 = undefined;
    addcarryxU32(&x852, &x853, x851, x811, x808);
    const x854 = (cast(u32, x853) + x809);
    var x855: u32 = undefined;
    var x856: u1 = undefined;
    addcarryxU32(&x855, &x856, 0x0, x783, x830);
    var x857: u32 = undefined;
    var x858: u1 = undefined;
    addcarryxU32(&x857, &x858, x856, x785, x832);
    var x859: u32 = undefined;
    var x860: u1 = undefined;
    addcarryxU32(&x859, &x860, x858, x787, x834);
    var x861: u32 = undefined;
    var x862: u1 = undefined;
    addcarryxU32(&x861, &x862, x860, x789, x836);
    var x863: u32 = undefined;
    var x864: u1 = undefined;
    addcarryxU32(&x863, &x864, x862, x791, x838);
    var x865: u32 = undefined;
    var x866: u1 = undefined;
    addcarryxU32(&x865, &x866, x864, x793, x840);
    var x867: u32 = undefined;
    var x868: u1 = undefined;
    addcarryxU32(&x867, &x868, x866, x795, x842);
    var x869: u32 = undefined;
    var x870: u1 = undefined;
    addcarryxU32(&x869, &x870, x868, x797, x844);
    var x871: u32 = undefined;
    var x872: u1 = undefined;
    addcarryxU32(&x871, &x872, x870, x799, x846);
    var x873: u32 = undefined;
    var x874: u1 = undefined;
    addcarryxU32(&x873, &x874, x872, x801, x848);
    var x875: u32 = undefined;
    var x876: u1 = undefined;
    addcarryxU32(&x875, &x876, x874, x803, x850);
    var x877: u32 = undefined;
    var x878: u1 = undefined;
    addcarryxU32(&x877, &x878, x876, x805, x852);
    var x879: u32 = undefined;
    var x880: u1 = undefined;
    addcarryxU32(&x879, &x880, x878, x807, x854);
    var x881: u32 = undefined;
    var x882: u32 = undefined;
    mulxU32(&x881, &x882, x855, 0xffffffff);
    var x883: u32 = undefined;
    var x884: u32 = undefined;
    mulxU32(&x883, &x884, x855, 0xffffffff);
    var x885: u32 = undefined;
    var x886: u32 = undefined;
    mulxU32(&x885, &x886, x855, 0xffffffff);
    var x887: u32 = undefined;
    var x888: u32 = undefined;
    mulxU32(&x887, &x888, x855, 0xffffffff);
    var x889: u32 = undefined;
    var x890: u32 = undefined;
    mulxU32(&x889, &x890, x855, 0xffffffff);
    var x891: u32 = undefined;
    var x892: u32 = undefined;
    mulxU32(&x891, &x892, x855, 0xffffffff);
    var x893: u32 = undefined;
    var x894: u32 = undefined;
    mulxU32(&x893, &x894, x855, 0xffffffff);
    var x895: u32 = undefined;
    var x896: u32 = undefined;
    mulxU32(&x895, &x896, x855, 0xfffffffe);
    var x897: u32 = undefined;
    var x898: u32 = undefined;
    mulxU32(&x897, &x898, x855, 0xffffffff);
    var x899: u32 = undefined;
    var x900: u32 = undefined;
    mulxU32(&x899, &x900, x855, 0xffffffff);
    var x901: u32 = undefined;
    var x902: u1 = undefined;
    addcarryxU32(&x901, &x902, 0x0, x898, x895);
    var x903: u32 = undefined;
    var x904: u1 = undefined;
    addcarryxU32(&x903, &x904, x902, x896, x893);
    var x905: u32 = undefined;
    var x906: u1 = undefined;
    addcarryxU32(&x905, &x906, x904, x894, x891);
    var x907: u32 = undefined;
    var x908: u1 = undefined;
    addcarryxU32(&x907, &x908, x906, x892, x889);
    var x909: u32 = undefined;
    var x910: u1 = undefined;
    addcarryxU32(&x909, &x910, x908, x890, x887);
    var x911: u32 = undefined;
    var x912: u1 = undefined;
    addcarryxU32(&x911, &x912, x910, x888, x885);
    var x913: u32 = undefined;
    var x914: u1 = undefined;
    addcarryxU32(&x913, &x914, x912, x886, x883);
    var x915: u32 = undefined;
    var x916: u1 = undefined;
    addcarryxU32(&x915, &x916, x914, x884, x881);
    const x917 = (cast(u32, x916) + x882);
    var x918: u32 = undefined;
    var x919: u1 = undefined;
    addcarryxU32(&x918, &x919, 0x0, x855, x899);
    var x920: u32 = undefined;
    var x921: u1 = undefined;
    addcarryxU32(&x920, &x921, x919, x857, x900);
    var x922: u32 = undefined;
    var x923: u1 = undefined;
    addcarryxU32(&x922, &x923, x921, x859, cast(u32, 0x0));
    var x924: u32 = undefined;
    var x925: u1 = undefined;
    addcarryxU32(&x924, &x925, x923, x861, x897);
    var x926: u32 = undefined;
    var x927: u1 = undefined;
    addcarryxU32(&x926, &x927, x925, x863, x901);
    var x928: u32 = undefined;
    var x929: u1 = undefined;
    addcarryxU32(&x928, &x929, x927, x865, x903);
    var x930: u32 = undefined;
    var x931: u1 = undefined;
    addcarryxU32(&x930, &x931, x929, x867, x905);
    var x932: u32 = undefined;
    var x933: u1 = undefined;
    addcarryxU32(&x932, &x933, x931, x869, x907);
    var x934: u32 = undefined;
    var x935: u1 = undefined;
    addcarryxU32(&x934, &x935, x933, x871, x909);
    var x936: u32 = undefined;
    var x937: u1 = undefined;
    addcarryxU32(&x936, &x937, x935, x873, x911);
    var x938: u32 = undefined;
    var x939: u1 = undefined;
    addcarryxU32(&x938, &x939, x937, x875, x913);
    var x940: u32 = undefined;
    var x941: u1 = undefined;
    addcarryxU32(&x940, &x941, x939, x877, x915);
    var x942: u32 = undefined;
    var x943: u1 = undefined;
    addcarryxU32(&x942, &x943, x941, x879, x917);
    const x944 = (cast(u32, x943) + cast(u32, x880));
    var x945: u32 = undefined;
    var x946: u32 = undefined;
    mulxU32(&x945, &x946, x7, (arg1[11]));
    var x947: u32 = undefined;
    var x948: u32 = undefined;
    mulxU32(&x947, &x948, x7, (arg1[10]));
    var x949: u32 = undefined;
    var x950: u32 = undefined;
    mulxU32(&x949, &x950, x7, (arg1[9]));
    var x951: u32 = undefined;
    var x952: u32 = undefined;
    mulxU32(&x951, &x952, x7, (arg1[8]));
    var x953: u32 = undefined;
    var x954: u32 = undefined;
    mulxU32(&x953, &x954, x7, (arg1[7]));
    var x955: u32 = undefined;
    var x956: u32 = undefined;
    mulxU32(&x955, &x956, x7, (arg1[6]));
    var x957: u32 = undefined;
    var x958: u32 = undefined;
    mulxU32(&x957, &x958, x7, (arg1[5]));
    var x959: u32 = undefined;
    var x960: u32 = undefined;
    mulxU32(&x959, &x960, x7, (arg1[4]));
    var x961: u32 = undefined;
    var x962: u32 = undefined;
    mulxU32(&x961, &x962, x7, (arg1[3]));
    var x963: u32 = undefined;
    var x964: u32 = undefined;
    mulxU32(&x963, &x964, x7, (arg1[2]));
    var x965: u32 = undefined;
    var x966: u32 = undefined;
    mulxU32(&x965, &x966, x7, (arg1[1]));
    var x967: u32 = undefined;
    var x968: u32 = undefined;
    mulxU32(&x967, &x968, x7, (arg1[0]));
    var x969: u32 = undefined;
    var x970: u1 = undefined;
    addcarryxU32(&x969, &x970, 0x0, x968, x965);
    var x971: u32 = undefined;
    var x972: u1 = undefined;
    addcarryxU32(&x971, &x972, x970, x966, x963);
    var x973: u32 = undefined;
    var x974: u1 = undefined;
    addcarryxU32(&x973, &x974, x972, x964, x961);
    var x975: u32 = undefined;
    var x976: u1 = undefined;
    addcarryxU32(&x975, &x976, x974, x962, x959);
    var x977: u32 = undefined;
    var x978: u1 = undefined;
    addcarryxU32(&x977, &x978, x976, x960, x957);
    var x979: u32 = undefined;
    var x980: u1 = undefined;
    addcarryxU32(&x979, &x980, x978, x958, x955);
    var x981: u32 = undefined;
    var x982: u1 = undefined;
    addcarryxU32(&x981, &x982, x980, x956, x953);
    var x983: u32 = undefined;
    var x984: u1 = undefined;
    addcarryxU32(&x983, &x984, x982, x954, x951);
    var x985: u32 = undefined;
    var x986: u1 = undefined;
    addcarryxU32(&x985, &x986, x984, x952, x949);
    var x987: u32 = undefined;
    var x988: u1 = undefined;
    addcarryxU32(&x987, &x988, x986, x950, x947);
    var x989: u32 = undefined;
    var x990: u1 = undefined;
    addcarryxU32(&x989, &x990, x988, x948, x945);
    const x991 = (cast(u32, x990) + x946);
    var x992: u32 = undefined;
    var x993: u1 = undefined;
    addcarryxU32(&x992, &x993, 0x0, x920, x967);
    var x994: u32 = undefined;
    var x995: u1 = undefined;
    addcarryxU32(&x994, &x995, x993, x922, x969);
    var x996: u32 = undefined;
    var x997: u1 = undefined;
    addcarryxU32(&x996, &x997, x995, x924, x971);
    var x998: u32 = undefined;
    var x999: u1 = undefined;
    addcarryxU32(&x998, &x999, x997, x926, x973);
    var x1000: u32 = undefined;
    var x1001: u1 = undefined;
    addcarryxU32(&x1000, &x1001, x999, x928, x975);
    var x1002: u32 = undefined;
    var x1003: u1 = undefined;
    addcarryxU32(&x1002, &x1003, x1001, x930, x977);
    var x1004: u32 = undefined;
    var x1005: u1 = undefined;
    addcarryxU32(&x1004, &x1005, x1003, x932, x979);
    var x1006: u32 = undefined;
    var x1007: u1 = undefined;
    addcarryxU32(&x1006, &x1007, x1005, x934, x981);
    var x1008: u32 = undefined;
    var x1009: u1 = undefined;
    addcarryxU32(&x1008, &x1009, x1007, x936, x983);
    var x1010: u32 = undefined;
    var x1011: u1 = undefined;
    addcarryxU32(&x1010, &x1011, x1009, x938, x985);
    var x1012: u32 = undefined;
    var x1013: u1 = undefined;
    addcarryxU32(&x1012, &x1013, x1011, x940, x987);
    var x1014: u32 = undefined;
    var x1015: u1 = undefined;
    addcarryxU32(&x1014, &x1015, x1013, x942, x989);
    var x1016: u32 = undefined;
    var x1017: u1 = undefined;
    addcarryxU32(&x1016, &x1017, x1015, x944, x991);
    var x1018: u32 = undefined;
    var x1019: u32 = undefined;
    mulxU32(&x1018, &x1019, x992, 0xffffffff);
    var x1020: u32 = undefined;
    var x1021: u32 = undefined;
    mulxU32(&x1020, &x1021, x992, 0xffffffff);
    var x1022: u32 = undefined;
    var x1023: u32 = undefined;
    mulxU32(&x1022, &x1023, x992, 0xffffffff);
    var x1024: u32 = undefined;
    var x1025: u32 = undefined;
    mulxU32(&x1024, &x1025, x992, 0xffffffff);
    var x1026: u32 = undefined;
    var x1027: u32 = undefined;
    mulxU32(&x1026, &x1027, x992, 0xffffffff);
    var x1028: u32 = undefined;
    var x1029: u32 = undefined;
    mulxU32(&x1028, &x1029, x992, 0xffffffff);
    var x1030: u32 = undefined;
    var x1031: u32 = undefined;
    mulxU32(&x1030, &x1031, x992, 0xffffffff);
    var x1032: u32 = undefined;
    var x1033: u32 = undefined;
    mulxU32(&x1032, &x1033, x992, 0xfffffffe);
    var x1034: u32 = undefined;
    var x1035: u32 = undefined;
    mulxU32(&x1034, &x1035, x992, 0xffffffff);
    var x1036: u32 = undefined;
    var x1037: u32 = undefined;
    mulxU32(&x1036, &x1037, x992, 0xffffffff);
    var x1038: u32 = undefined;
    var x1039: u1 = undefined;
    addcarryxU32(&x1038, &x1039, 0x0, x1035, x1032);
    var x1040: u32 = undefined;
    var x1041: u1 = undefined;
    addcarryxU32(&x1040, &x1041, x1039, x1033, x1030);
    var x1042: u32 = undefined;
    var x1043: u1 = undefined;
    addcarryxU32(&x1042, &x1043, x1041, x1031, x1028);
    var x1044: u32 = undefined;
    var x1045: u1 = undefined;
    addcarryxU32(&x1044, &x1045, x1043, x1029, x1026);
    var x1046: u32 = undefined;
    var x1047: u1 = undefined;
    addcarryxU32(&x1046, &x1047, x1045, x1027, x1024);
    var x1048: u32 = undefined;
    var x1049: u1 = undefined;
    addcarryxU32(&x1048, &x1049, x1047, x1025, x1022);
    var x1050: u32 = undefined;
    var x1051: u1 = undefined;
    addcarryxU32(&x1050, &x1051, x1049, x1023, x1020);
    var x1052: u32 = undefined;
    var x1053: u1 = undefined;
    addcarryxU32(&x1052, &x1053, x1051, x1021, x1018);
    const x1054 = (cast(u32, x1053) + x1019);
    var x1055: u32 = undefined;
    var x1056: u1 = undefined;
    addcarryxU32(&x1055, &x1056, 0x0, x992, x1036);
    var x1057: u32 = undefined;
    var x1058: u1 = undefined;
    addcarryxU32(&x1057, &x1058, x1056, x994, x1037);
    var x1059: u32 = undefined;
    var x1060: u1 = undefined;
    addcarryxU32(&x1059, &x1060, x1058, x996, cast(u32, 0x0));
    var x1061: u32 = undefined;
    var x1062: u1 = undefined;
    addcarryxU32(&x1061, &x1062, x1060, x998, x1034);
    var x1063: u32 = undefined;
    var x1064: u1 = undefined;
    addcarryxU32(&x1063, &x1064, x1062, x1000, x1038);
    var x1065: u32 = undefined;
    var x1066: u1 = undefined;
    addcarryxU32(&x1065, &x1066, x1064, x1002, x1040);
    var x1067: u32 = undefined;
    var x1068: u1 = undefined;
    addcarryxU32(&x1067, &x1068, x1066, x1004, x1042);
    var x1069: u32 = undefined;
    var x1070: u1 = undefined;
    addcarryxU32(&x1069, &x1070, x1068, x1006, x1044);
    var x1071: u32 = undefined;
    var x1072: u1 = undefined;
    addcarryxU32(&x1071, &x1072, x1070, x1008, x1046);
    var x1073: u32 = undefined;
    var x1074: u1 = undefined;
    addcarryxU32(&x1073, &x1074, x1072, x1010, x1048);
    var x1075: u32 = undefined;
    var x1076: u1 = undefined;
    addcarryxU32(&x1075, &x1076, x1074, x1012, x1050);
    var x1077: u32 = undefined;
    var x1078: u1 = undefined;
    addcarryxU32(&x1077, &x1078, x1076, x1014, x1052);
    var x1079: u32 = undefined;
    var x1080: u1 = undefined;
    addcarryxU32(&x1079, &x1080, x1078, x1016, x1054);
    const x1081 = (cast(u32, x1080) + cast(u32, x1017));
    var x1082: u32 = undefined;
    var x1083: u32 = undefined;
    mulxU32(&x1082, &x1083, x8, (arg1[11]));
    var x1084: u32 = undefined;
    var x1085: u32 = undefined;
    mulxU32(&x1084, &x1085, x8, (arg1[10]));
    var x1086: u32 = undefined;
    var x1087: u32 = undefined;
    mulxU32(&x1086, &x1087, x8, (arg1[9]));
    var x1088: u32 = undefined;
    var x1089: u32 = undefined;
    mulxU32(&x1088, &x1089, x8, (arg1[8]));
    var x1090: u32 = undefined;
    var x1091: u32 = undefined;
    mulxU32(&x1090, &x1091, x8, (arg1[7]));
    var x1092: u32 = undefined;
    var x1093: u32 = undefined;
    mulxU32(&x1092, &x1093, x8, (arg1[6]));
    var x1094: u32 = undefined;
    var x1095: u32 = undefined;
    mulxU32(&x1094, &x1095, x8, (arg1[5]));
    var x1096: u32 = undefined;
    var x1097: u32 = undefined;
    mulxU32(&x1096, &x1097, x8, (arg1[4]));
    var x1098: u32 = undefined;
    var x1099: u32 = undefined;
    mulxU32(&x1098, &x1099, x8, (arg1[3]));
    var x1100: u32 = undefined;
    var x1101: u32 = undefined;
    mulxU32(&x1100, &x1101, x8, (arg1[2]));
    var x1102: u32 = undefined;
    var x1103: u32 = undefined;
    mulxU32(&x1102, &x1103, x8, (arg1[1]));
    var x1104: u32 = undefined;
    var x1105: u32 = undefined;
    mulxU32(&x1104, &x1105, x8, (arg1[0]));
    var x1106: u32 = undefined;
    var x1107: u1 = undefined;
    addcarryxU32(&x1106, &x1107, 0x0, x1105, x1102);
    var x1108: u32 = undefined;
    var x1109: u1 = undefined;
    addcarryxU32(&x1108, &x1109, x1107, x1103, x1100);
    var x1110: u32 = undefined;
    var x1111: u1 = undefined;
    addcarryxU32(&x1110, &x1111, x1109, x1101, x1098);
    var x1112: u32 = undefined;
    var x1113: u1 = undefined;
    addcarryxU32(&x1112, &x1113, x1111, x1099, x1096);
    var x1114: u32 = undefined;
    var x1115: u1 = undefined;
    addcarryxU32(&x1114, &x1115, x1113, x1097, x1094);
    var x1116: u32 = undefined;
    var x1117: u1 = undefined;
    addcarryxU32(&x1116, &x1117, x1115, x1095, x1092);
    var x1118: u32 = undefined;
    var x1119: u1 = undefined;
    addcarryxU32(&x1118, &x1119, x1117, x1093, x1090);
    var x1120: u32 = undefined;
    var x1121: u1 = undefined;
    addcarryxU32(&x1120, &x1121, x1119, x1091, x1088);
    var x1122: u32 = undefined;
    var x1123: u1 = undefined;
    addcarryxU32(&x1122, &x1123, x1121, x1089, x1086);
    var x1124: u32 = undefined;
    var x1125: u1 = undefined;
    addcarryxU32(&x1124, &x1125, x1123, x1087, x1084);
    var x1126: u32 = undefined;
    var x1127: u1 = undefined;
    addcarryxU32(&x1126, &x1127, x1125, x1085, x1082);
    const x1128 = (cast(u32, x1127) + x1083);
    var x1129: u32 = undefined;
    var x1130: u1 = undefined;
    addcarryxU32(&x1129, &x1130, 0x0, x1057, x1104);
    var x1131: u32 = undefined;
    var x1132: u1 = undefined;
    addcarryxU32(&x1131, &x1132, x1130, x1059, x1106);
    var x1133: u32 = undefined;
    var x1134: u1 = undefined;
    addcarryxU32(&x1133, &x1134, x1132, x1061, x1108);
    var x1135: u32 = undefined;
    var x1136: u1 = undefined;
    addcarryxU32(&x1135, &x1136, x1134, x1063, x1110);
    var x1137: u32 = undefined;
    var x1138: u1 = undefined;
    addcarryxU32(&x1137, &x1138, x1136, x1065, x1112);
    var x1139: u32 = undefined;
    var x1140: u1 = undefined;
    addcarryxU32(&x1139, &x1140, x1138, x1067, x1114);
    var x1141: u32 = undefined;
    var x1142: u1 = undefined;
    addcarryxU32(&x1141, &x1142, x1140, x1069, x1116);
    var x1143: u32 = undefined;
    var x1144: u1 = undefined;
    addcarryxU32(&x1143, &x1144, x1142, x1071, x1118);
    var x1145: u32 = undefined;
    var x1146: u1 = undefined;
    addcarryxU32(&x1145, &x1146, x1144, x1073, x1120);
    var x1147: u32 = undefined;
    var x1148: u1 = undefined;
    addcarryxU32(&x1147, &x1148, x1146, x1075, x1122);
    var x1149: u32 = undefined;
    var x1150: u1 = undefined;
    addcarryxU32(&x1149, &x1150, x1148, x1077, x1124);
    var x1151: u32 = undefined;
    var x1152: u1 = undefined;
    addcarryxU32(&x1151, &x1152, x1150, x1079, x1126);
    var x1153: u32 = undefined;
    var x1154: u1 = undefined;
    addcarryxU32(&x1153, &x1154, x1152, x1081, x1128);
    var x1155: u32 = undefined;
    var x1156: u32 = undefined;
    mulxU32(&x1155, &x1156, x1129, 0xffffffff);
    var x1157: u32 = undefined;
    var x1158: u32 = undefined;
    mulxU32(&x1157, &x1158, x1129, 0xffffffff);
    var x1159: u32 = undefined;
    var x1160: u32 = undefined;
    mulxU32(&x1159, &x1160, x1129, 0xffffffff);
    var x1161: u32 = undefined;
    var x1162: u32 = undefined;
    mulxU32(&x1161, &x1162, x1129, 0xffffffff);
    var x1163: u32 = undefined;
    var x1164: u32 = undefined;
    mulxU32(&x1163, &x1164, x1129, 0xffffffff);
    var x1165: u32 = undefined;
    var x1166: u32 = undefined;
    mulxU32(&x1165, &x1166, x1129, 0xffffffff);
    var x1167: u32 = undefined;
    var x1168: u32 = undefined;
    mulxU32(&x1167, &x1168, x1129, 0xffffffff);
    var x1169: u32 = undefined;
    var x1170: u32 = undefined;
    mulxU32(&x1169, &x1170, x1129, 0xfffffffe);
    var x1171: u32 = undefined;
    var x1172: u32 = undefined;
    mulxU32(&x1171, &x1172, x1129, 0xffffffff);
    var x1173: u32 = undefined;
    var x1174: u32 = undefined;
    mulxU32(&x1173, &x1174, x1129, 0xffffffff);
    var x1175: u32 = undefined;
    var x1176: u1 = undefined;
    addcarryxU32(&x1175, &x1176, 0x0, x1172, x1169);
    var x1177: u32 = undefined;
    var x1178: u1 = undefined;
    addcarryxU32(&x1177, &x1178, x1176, x1170, x1167);
    var x1179: u32 = undefined;
    var x1180: u1 = undefined;
    addcarryxU32(&x1179, &x1180, x1178, x1168, x1165);
    var x1181: u32 = undefined;
    var x1182: u1 = undefined;
    addcarryxU32(&x1181, &x1182, x1180, x1166, x1163);
    var x1183: u32 = undefined;
    var x1184: u1 = undefined;
    addcarryxU32(&x1183, &x1184, x1182, x1164, x1161);
    var x1185: u32 = undefined;
    var x1186: u1 = undefined;
    addcarryxU32(&x1185, &x1186, x1184, x1162, x1159);
    var x1187: u32 = undefined;
    var x1188: u1 = undefined;
    addcarryxU32(&x1187, &x1188, x1186, x1160, x1157);
    var x1189: u32 = undefined;
    var x1190: u1 = undefined;
    addcarryxU32(&x1189, &x1190, x1188, x1158, x1155);
    const x1191 = (cast(u32, x1190) + x1156);
    var x1192: u32 = undefined;
    var x1193: u1 = undefined;
    addcarryxU32(&x1192, &x1193, 0x0, x1129, x1173);
    var x1194: u32 = undefined;
    var x1195: u1 = undefined;
    addcarryxU32(&x1194, &x1195, x1193, x1131, x1174);
    var x1196: u32 = undefined;
    var x1197: u1 = undefined;
    addcarryxU32(&x1196, &x1197, x1195, x1133, cast(u32, 0x0));
    var x1198: u32 = undefined;
    var x1199: u1 = undefined;
    addcarryxU32(&x1198, &x1199, x1197, x1135, x1171);
    var x1200: u32 = undefined;
    var x1201: u1 = undefined;
    addcarryxU32(&x1200, &x1201, x1199, x1137, x1175);
    var x1202: u32 = undefined;
    var x1203: u1 = undefined;
    addcarryxU32(&x1202, &x1203, x1201, x1139, x1177);
    var x1204: u32 = undefined;
    var x1205: u1 = undefined;
    addcarryxU32(&x1204, &x1205, x1203, x1141, x1179);
    var x1206: u32 = undefined;
    var x1207: u1 = undefined;
    addcarryxU32(&x1206, &x1207, x1205, x1143, x1181);
    var x1208: u32 = undefined;
    var x1209: u1 = undefined;
    addcarryxU32(&x1208, &x1209, x1207, x1145, x1183);
    var x1210: u32 = undefined;
    var x1211: u1 = undefined;
    addcarryxU32(&x1210, &x1211, x1209, x1147, x1185);
    var x1212: u32 = undefined;
    var x1213: u1 = undefined;
    addcarryxU32(&x1212, &x1213, x1211, x1149, x1187);
    var x1214: u32 = undefined;
    var x1215: u1 = undefined;
    addcarryxU32(&x1214, &x1215, x1213, x1151, x1189);
    var x1216: u32 = undefined;
    var x1217: u1 = undefined;
    addcarryxU32(&x1216, &x1217, x1215, x1153, x1191);
    const x1218 = (cast(u32, x1217) + cast(u32, x1154));
    var x1219: u32 = undefined;
    var x1220: u32 = undefined;
    mulxU32(&x1219, &x1220, x9, (arg1[11]));
    var x1221: u32 = undefined;
    var x1222: u32 = undefined;
    mulxU32(&x1221, &x1222, x9, (arg1[10]));
    var x1223: u32 = undefined;
    var x1224: u32 = undefined;
    mulxU32(&x1223, &x1224, x9, (arg1[9]));
    var x1225: u32 = undefined;
    var x1226: u32 = undefined;
    mulxU32(&x1225, &x1226, x9, (arg1[8]));
    var x1227: u32 = undefined;
    var x1228: u32 = undefined;
    mulxU32(&x1227, &x1228, x9, (arg1[7]));
    var x1229: u32 = undefined;
    var x1230: u32 = undefined;
    mulxU32(&x1229, &x1230, x9, (arg1[6]));
    var x1231: u32 = undefined;
    var x1232: u32 = undefined;
    mulxU32(&x1231, &x1232, x9, (arg1[5]));
    var x1233: u32 = undefined;
    var x1234: u32 = undefined;
    mulxU32(&x1233, &x1234, x9, (arg1[4]));
    var x1235: u32 = undefined;
    var x1236: u32 = undefined;
    mulxU32(&x1235, &x1236, x9, (arg1[3]));
    var x1237: u32 = undefined;
    var x1238: u32 = undefined;
    mulxU32(&x1237, &x1238, x9, (arg1[2]));
    var x1239: u32 = undefined;
    var x1240: u32 = undefined;
    mulxU32(&x1239, &x1240, x9, (arg1[1]));
    var x1241: u32 = undefined;
    var x1242: u32 = undefined;
    mulxU32(&x1241, &x1242, x9, (arg1[0]));
    var x1243: u32 = undefined;
    var x1244: u1 = undefined;
    addcarryxU32(&x1243, &x1244, 0x0, x1242, x1239);
    var x1245: u32 = undefined;
    var x1246: u1 = undefined;
    addcarryxU32(&x1245, &x1246, x1244, x1240, x1237);
    var x1247: u32 = undefined;
    var x1248: u1 = undefined;
    addcarryxU32(&x1247, &x1248, x1246, x1238, x1235);
    var x1249: u32 = undefined;
    var x1250: u1 = undefined;
    addcarryxU32(&x1249, &x1250, x1248, x1236, x1233);
    var x1251: u32 = undefined;
    var x1252: u1 = undefined;
    addcarryxU32(&x1251, &x1252, x1250, x1234, x1231);
    var x1253: u32 = undefined;
    var x1254: u1 = undefined;
    addcarryxU32(&x1253, &x1254, x1252, x1232, x1229);
    var x1255: u32 = undefined;
    var x1256: u1 = undefined;
    addcarryxU32(&x1255, &x1256, x1254, x1230, x1227);
    var x1257: u32 = undefined;
    var x1258: u1 = undefined;
    addcarryxU32(&x1257, &x1258, x1256, x1228, x1225);
    var x1259: u32 = undefined;
    var x1260: u1 = undefined;
    addcarryxU32(&x1259, &x1260, x1258, x1226, x1223);
    var x1261: u32 = undefined;
    var x1262: u1 = undefined;
    addcarryxU32(&x1261, &x1262, x1260, x1224, x1221);
    var x1263: u32 = undefined;
    var x1264: u1 = undefined;
    addcarryxU32(&x1263, &x1264, x1262, x1222, x1219);
    const x1265 = (cast(u32, x1264) + x1220);
    var x1266: u32 = undefined;
    var x1267: u1 = undefined;
    addcarryxU32(&x1266, &x1267, 0x0, x1194, x1241);
    var x1268: u32 = undefined;
    var x1269: u1 = undefined;
    addcarryxU32(&x1268, &x1269, x1267, x1196, x1243);
    var x1270: u32 = undefined;
    var x1271: u1 = undefined;
    addcarryxU32(&x1270, &x1271, x1269, x1198, x1245);
    var x1272: u32 = undefined;
    var x1273: u1 = undefined;
    addcarryxU32(&x1272, &x1273, x1271, x1200, x1247);
    var x1274: u32 = undefined;
    var x1275: u1 = undefined;
    addcarryxU32(&x1274, &x1275, x1273, x1202, x1249);
    var x1276: u32 = undefined;
    var x1277: u1 = undefined;
    addcarryxU32(&x1276, &x1277, x1275, x1204, x1251);
    var x1278: u32 = undefined;
    var x1279: u1 = undefined;
    addcarryxU32(&x1278, &x1279, x1277, x1206, x1253);
    var x1280: u32 = undefined;
    var x1281: u1 = undefined;
    addcarryxU32(&x1280, &x1281, x1279, x1208, x1255);
    var x1282: u32 = undefined;
    var x1283: u1 = undefined;
    addcarryxU32(&x1282, &x1283, x1281, x1210, x1257);
    var x1284: u32 = undefined;
    var x1285: u1 = undefined;
    addcarryxU32(&x1284, &x1285, x1283, x1212, x1259);
    var x1286: u32 = undefined;
    var x1287: u1 = undefined;
    addcarryxU32(&x1286, &x1287, x1285, x1214, x1261);
    var x1288: u32 = undefined;
    var x1289: u1 = undefined;
    addcarryxU32(&x1288, &x1289, x1287, x1216, x1263);
    var x1290: u32 = undefined;
    var x1291: u1 = undefined;
    addcarryxU32(&x1290, &x1291, x1289, x1218, x1265);
    var x1292: u32 = undefined;
    var x1293: u32 = undefined;
    mulxU32(&x1292, &x1293, x1266, 0xffffffff);
    var x1294: u32 = undefined;
    var x1295: u32 = undefined;
    mulxU32(&x1294, &x1295, x1266, 0xffffffff);
    var x1296: u32 = undefined;
    var x1297: u32 = undefined;
    mulxU32(&x1296, &x1297, x1266, 0xffffffff);
    var x1298: u32 = undefined;
    var x1299: u32 = undefined;
    mulxU32(&x1298, &x1299, x1266, 0xffffffff);
    var x1300: u32 = undefined;
    var x1301: u32 = undefined;
    mulxU32(&x1300, &x1301, x1266, 0xffffffff);
    var x1302: u32 = undefined;
    var x1303: u32 = undefined;
    mulxU32(&x1302, &x1303, x1266, 0xffffffff);
    var x1304: u32 = undefined;
    var x1305: u32 = undefined;
    mulxU32(&x1304, &x1305, x1266, 0xffffffff);
    var x1306: u32 = undefined;
    var x1307: u32 = undefined;
    mulxU32(&x1306, &x1307, x1266, 0xfffffffe);
    var x1308: u32 = undefined;
    var x1309: u32 = undefined;
    mulxU32(&x1308, &x1309, x1266, 0xffffffff);
    var x1310: u32 = undefined;
    var x1311: u32 = undefined;
    mulxU32(&x1310, &x1311, x1266, 0xffffffff);
    var x1312: u32 = undefined;
    var x1313: u1 = undefined;
    addcarryxU32(&x1312, &x1313, 0x0, x1309, x1306);
    var x1314: u32 = undefined;
    var x1315: u1 = undefined;
    addcarryxU32(&x1314, &x1315, x1313, x1307, x1304);
    var x1316: u32 = undefined;
    var x1317: u1 = undefined;
    addcarryxU32(&x1316, &x1317, x1315, x1305, x1302);
    var x1318: u32 = undefined;
    var x1319: u1 = undefined;
    addcarryxU32(&x1318, &x1319, x1317, x1303, x1300);
    var x1320: u32 = undefined;
    var x1321: u1 = undefined;
    addcarryxU32(&x1320, &x1321, x1319, x1301, x1298);
    var x1322: u32 = undefined;
    var x1323: u1 = undefined;
    addcarryxU32(&x1322, &x1323, x1321, x1299, x1296);
    var x1324: u32 = undefined;
    var x1325: u1 = undefined;
    addcarryxU32(&x1324, &x1325, x1323, x1297, x1294);
    var x1326: u32 = undefined;
    var x1327: u1 = undefined;
    addcarryxU32(&x1326, &x1327, x1325, x1295, x1292);
    const x1328 = (cast(u32, x1327) + x1293);
    var x1329: u32 = undefined;
    var x1330: u1 = undefined;
    addcarryxU32(&x1329, &x1330, 0x0, x1266, x1310);
    var x1331: u32 = undefined;
    var x1332: u1 = undefined;
    addcarryxU32(&x1331, &x1332, x1330, x1268, x1311);
    var x1333: u32 = undefined;
    var x1334: u1 = undefined;
    addcarryxU32(&x1333, &x1334, x1332, x1270, cast(u32, 0x0));
    var x1335: u32 = undefined;
    var x1336: u1 = undefined;
    addcarryxU32(&x1335, &x1336, x1334, x1272, x1308);
    var x1337: u32 = undefined;
    var x1338: u1 = undefined;
    addcarryxU32(&x1337, &x1338, x1336, x1274, x1312);
    var x1339: u32 = undefined;
    var x1340: u1 = undefined;
    addcarryxU32(&x1339, &x1340, x1338, x1276, x1314);
    var x1341: u32 = undefined;
    var x1342: u1 = undefined;
    addcarryxU32(&x1341, &x1342, x1340, x1278, x1316);
    var x1343: u32 = undefined;
    var x1344: u1 = undefined;
    addcarryxU32(&x1343, &x1344, x1342, x1280, x1318);
    var x1345: u32 = undefined;
    var x1346: u1 = undefined;
    addcarryxU32(&x1345, &x1346, x1344, x1282, x1320);
    var x1347: u32 = undefined;
    var x1348: u1 = undefined;
    addcarryxU32(&x1347, &x1348, x1346, x1284, x1322);
    var x1349: u32 = undefined;
    var x1350: u1 = undefined;
    addcarryxU32(&x1349, &x1350, x1348, x1286, x1324);
    var x1351: u32 = undefined;
    var x1352: u1 = undefined;
    addcarryxU32(&x1351, &x1352, x1350, x1288, x1326);
    var x1353: u32 = undefined;
    var x1354: u1 = undefined;
    addcarryxU32(&x1353, &x1354, x1352, x1290, x1328);
    const x1355 = (cast(u32, x1354) + cast(u32, x1291));
    var x1356: u32 = undefined;
    var x1357: u32 = undefined;
    mulxU32(&x1356, &x1357, x10, (arg1[11]));
    var x1358: u32 = undefined;
    var x1359: u32 = undefined;
    mulxU32(&x1358, &x1359, x10, (arg1[10]));
    var x1360: u32 = undefined;
    var x1361: u32 = undefined;
    mulxU32(&x1360, &x1361, x10, (arg1[9]));
    var x1362: u32 = undefined;
    var x1363: u32 = undefined;
    mulxU32(&x1362, &x1363, x10, (arg1[8]));
    var x1364: u32 = undefined;
    var x1365: u32 = undefined;
    mulxU32(&x1364, &x1365, x10, (arg1[7]));
    var x1366: u32 = undefined;
    var x1367: u32 = undefined;
    mulxU32(&x1366, &x1367, x10, (arg1[6]));
    var x1368: u32 = undefined;
    var x1369: u32 = undefined;
    mulxU32(&x1368, &x1369, x10, (arg1[5]));
    var x1370: u32 = undefined;
    var x1371: u32 = undefined;
    mulxU32(&x1370, &x1371, x10, (arg1[4]));
    var x1372: u32 = undefined;
    var x1373: u32 = undefined;
    mulxU32(&x1372, &x1373, x10, (arg1[3]));
    var x1374: u32 = undefined;
    var x1375: u32 = undefined;
    mulxU32(&x1374, &x1375, x10, (arg1[2]));
    var x1376: u32 = undefined;
    var x1377: u32 = undefined;
    mulxU32(&x1376, &x1377, x10, (arg1[1]));
    var x1378: u32 = undefined;
    var x1379: u32 = undefined;
    mulxU32(&x1378, &x1379, x10, (arg1[0]));
    var x1380: u32 = undefined;
    var x1381: u1 = undefined;
    addcarryxU32(&x1380, &x1381, 0x0, x1379, x1376);
    var x1382: u32 = undefined;
    var x1383: u1 = undefined;
    addcarryxU32(&x1382, &x1383, x1381, x1377, x1374);
    var x1384: u32 = undefined;
    var x1385: u1 = undefined;
    addcarryxU32(&x1384, &x1385, x1383, x1375, x1372);
    var x1386: u32 = undefined;
    var x1387: u1 = undefined;
    addcarryxU32(&x1386, &x1387, x1385, x1373, x1370);
    var x1388: u32 = undefined;
    var x1389: u1 = undefined;
    addcarryxU32(&x1388, &x1389, x1387, x1371, x1368);
    var x1390: u32 = undefined;
    var x1391: u1 = undefined;
    addcarryxU32(&x1390, &x1391, x1389, x1369, x1366);
    var x1392: u32 = undefined;
    var x1393: u1 = undefined;
    addcarryxU32(&x1392, &x1393, x1391, x1367, x1364);
    var x1394: u32 = undefined;
    var x1395: u1 = undefined;
    addcarryxU32(&x1394, &x1395, x1393, x1365, x1362);
    var x1396: u32 = undefined;
    var x1397: u1 = undefined;
    addcarryxU32(&x1396, &x1397, x1395, x1363, x1360);
    var x1398: u32 = undefined;
    var x1399: u1 = undefined;
    addcarryxU32(&x1398, &x1399, x1397, x1361, x1358);
    var x1400: u32 = undefined;
    var x1401: u1 = undefined;
    addcarryxU32(&x1400, &x1401, x1399, x1359, x1356);
    const x1402 = (cast(u32, x1401) + x1357);
    var x1403: u32 = undefined;
    var x1404: u1 = undefined;
    addcarryxU32(&x1403, &x1404, 0x0, x1331, x1378);
    var x1405: u32 = undefined;
    var x1406: u1 = undefined;
    addcarryxU32(&x1405, &x1406, x1404, x1333, x1380);
    var x1407: u32 = undefined;
    var x1408: u1 = undefined;
    addcarryxU32(&x1407, &x1408, x1406, x1335, x1382);
    var x1409: u32 = undefined;
    var x1410: u1 = undefined;
    addcarryxU32(&x1409, &x1410, x1408, x1337, x1384);
    var x1411: u32 = undefined;
    var x1412: u1 = undefined;
    addcarryxU32(&x1411, &x1412, x1410, x1339, x1386);
    var x1413: u32 = undefined;
    var x1414: u1 = undefined;
    addcarryxU32(&x1413, &x1414, x1412, x1341, x1388);
    var x1415: u32 = undefined;
    var x1416: u1 = undefined;
    addcarryxU32(&x1415, &x1416, x1414, x1343, x1390);
    var x1417: u32 = undefined;
    var x1418: u1 = undefined;
    addcarryxU32(&x1417, &x1418, x1416, x1345, x1392);
    var x1419: u32 = undefined;
    var x1420: u1 = undefined;
    addcarryxU32(&x1419, &x1420, x1418, x1347, x1394);
    var x1421: u32 = undefined;
    var x1422: u1 = undefined;
    addcarryxU32(&x1421, &x1422, x1420, x1349, x1396);
    var x1423: u32 = undefined;
    var x1424: u1 = undefined;
    addcarryxU32(&x1423, &x1424, x1422, x1351, x1398);
    var x1425: u32 = undefined;
    var x1426: u1 = undefined;
    addcarryxU32(&x1425, &x1426, x1424, x1353, x1400);
    var x1427: u32 = undefined;
    var x1428: u1 = undefined;
    addcarryxU32(&x1427, &x1428, x1426, x1355, x1402);
    var x1429: u32 = undefined;
    var x1430: u32 = undefined;
    mulxU32(&x1429, &x1430, x1403, 0xffffffff);
    var x1431: u32 = undefined;
    var x1432: u32 = undefined;
    mulxU32(&x1431, &x1432, x1403, 0xffffffff);
    var x1433: u32 = undefined;
    var x1434: u32 = undefined;
    mulxU32(&x1433, &x1434, x1403, 0xffffffff);
    var x1435: u32 = undefined;
    var x1436: u32 = undefined;
    mulxU32(&x1435, &x1436, x1403, 0xffffffff);
    var x1437: u32 = undefined;
    var x1438: u32 = undefined;
    mulxU32(&x1437, &x1438, x1403, 0xffffffff);
    var x1439: u32 = undefined;
    var x1440: u32 = undefined;
    mulxU32(&x1439, &x1440, x1403, 0xffffffff);
    var x1441: u32 = undefined;
    var x1442: u32 = undefined;
    mulxU32(&x1441, &x1442, x1403, 0xffffffff);
    var x1443: u32 = undefined;
    var x1444: u32 = undefined;
    mulxU32(&x1443, &x1444, x1403, 0xfffffffe);
    var x1445: u32 = undefined;
    var x1446: u32 = undefined;
    mulxU32(&x1445, &x1446, x1403, 0xffffffff);
    var x1447: u32 = undefined;
    var x1448: u32 = undefined;
    mulxU32(&x1447, &x1448, x1403, 0xffffffff);
    var x1449: u32 = undefined;
    var x1450: u1 = undefined;
    addcarryxU32(&x1449, &x1450, 0x0, x1446, x1443);
    var x1451: u32 = undefined;
    var x1452: u1 = undefined;
    addcarryxU32(&x1451, &x1452, x1450, x1444, x1441);
    var x1453: u32 = undefined;
    var x1454: u1 = undefined;
    addcarryxU32(&x1453, &x1454, x1452, x1442, x1439);
    var x1455: u32 = undefined;
    var x1456: u1 = undefined;
    addcarryxU32(&x1455, &x1456, x1454, x1440, x1437);
    var x1457: u32 = undefined;
    var x1458: u1 = undefined;
    addcarryxU32(&x1457, &x1458, x1456, x1438, x1435);
    var x1459: u32 = undefined;
    var x1460: u1 = undefined;
    addcarryxU32(&x1459, &x1460, x1458, x1436, x1433);
    var x1461: u32 = undefined;
    var x1462: u1 = undefined;
    addcarryxU32(&x1461, &x1462, x1460, x1434, x1431);
    var x1463: u32 = undefined;
    var x1464: u1 = undefined;
    addcarryxU32(&x1463, &x1464, x1462, x1432, x1429);
    const x1465 = (cast(u32, x1464) + x1430);
    var x1466: u32 = undefined;
    var x1467: u1 = undefined;
    addcarryxU32(&x1466, &x1467, 0x0, x1403, x1447);
    var x1468: u32 = undefined;
    var x1469: u1 = undefined;
    addcarryxU32(&x1468, &x1469, x1467, x1405, x1448);
    var x1470: u32 = undefined;
    var x1471: u1 = undefined;
    addcarryxU32(&x1470, &x1471, x1469, x1407, cast(u32, 0x0));
    var x1472: u32 = undefined;
    var x1473: u1 = undefined;
    addcarryxU32(&x1472, &x1473, x1471, x1409, x1445);
    var x1474: u32 = undefined;
    var x1475: u1 = undefined;
    addcarryxU32(&x1474, &x1475, x1473, x1411, x1449);
    var x1476: u32 = undefined;
    var x1477: u1 = undefined;
    addcarryxU32(&x1476, &x1477, x1475, x1413, x1451);
    var x1478: u32 = undefined;
    var x1479: u1 = undefined;
    addcarryxU32(&x1478, &x1479, x1477, x1415, x1453);
    var x1480: u32 = undefined;
    var x1481: u1 = undefined;
    addcarryxU32(&x1480, &x1481, x1479, x1417, x1455);
    var x1482: u32 = undefined;
    var x1483: u1 = undefined;
    addcarryxU32(&x1482, &x1483, x1481, x1419, x1457);
    var x1484: u32 = undefined;
    var x1485: u1 = undefined;
    addcarryxU32(&x1484, &x1485, x1483, x1421, x1459);
    var x1486: u32 = undefined;
    var x1487: u1 = undefined;
    addcarryxU32(&x1486, &x1487, x1485, x1423, x1461);
    var x1488: u32 = undefined;
    var x1489: u1 = undefined;
    addcarryxU32(&x1488, &x1489, x1487, x1425, x1463);
    var x1490: u32 = undefined;
    var x1491: u1 = undefined;
    addcarryxU32(&x1490, &x1491, x1489, x1427, x1465);
    const x1492 = (cast(u32, x1491) + cast(u32, x1428));
    var x1493: u32 = undefined;
    var x1494: u32 = undefined;
    mulxU32(&x1493, &x1494, x11, (arg1[11]));
    var x1495: u32 = undefined;
    var x1496: u32 = undefined;
    mulxU32(&x1495, &x1496, x11, (arg1[10]));
    var x1497: u32 = undefined;
    var x1498: u32 = undefined;
    mulxU32(&x1497, &x1498, x11, (arg1[9]));
    var x1499: u32 = undefined;
    var x1500: u32 = undefined;
    mulxU32(&x1499, &x1500, x11, (arg1[8]));
    var x1501: u32 = undefined;
    var x1502: u32 = undefined;
    mulxU32(&x1501, &x1502, x11, (arg1[7]));
    var x1503: u32 = undefined;
    var x1504: u32 = undefined;
    mulxU32(&x1503, &x1504, x11, (arg1[6]));
    var x1505: u32 = undefined;
    var x1506: u32 = undefined;
    mulxU32(&x1505, &x1506, x11, (arg1[5]));
    var x1507: u32 = undefined;
    var x1508: u32 = undefined;
    mulxU32(&x1507, &x1508, x11, (arg1[4]));
    var x1509: u32 = undefined;
    var x1510: u32 = undefined;
    mulxU32(&x1509, &x1510, x11, (arg1[3]));
    var x1511: u32 = undefined;
    var x1512: u32 = undefined;
    mulxU32(&x1511, &x1512, x11, (arg1[2]));
    var x1513: u32 = undefined;
    var x1514: u32 = undefined;
    mulxU32(&x1513, &x1514, x11, (arg1[1]));
    var x1515: u32 = undefined;
    var x1516: u32 = undefined;
    mulxU32(&x1515, &x1516, x11, (arg1[0]));
    var x1517: u32 = undefined;
    var x1518: u1 = undefined;
    addcarryxU32(&x1517, &x1518, 0x0, x1516, x1513);
    var x1519: u32 = undefined;
    var x1520: u1 = undefined;
    addcarryxU32(&x1519, &x1520, x1518, x1514, x1511);
    var x1521: u32 = undefined;
    var x1522: u1 = undefined;
    addcarryxU32(&x1521, &x1522, x1520, x1512, x1509);
    var x1523: u32 = undefined;
    var x1524: u1 = undefined;
    addcarryxU32(&x1523, &x1524, x1522, x1510, x1507);
    var x1525: u32 = undefined;
    var x1526: u1 = undefined;
    addcarryxU32(&x1525, &x1526, x1524, x1508, x1505);
    var x1527: u32 = undefined;
    var x1528: u1 = undefined;
    addcarryxU32(&x1527, &x1528, x1526, x1506, x1503);
    var x1529: u32 = undefined;
    var x1530: u1 = undefined;
    addcarryxU32(&x1529, &x1530, x1528, x1504, x1501);
    var x1531: u32 = undefined;
    var x1532: u1 = undefined;
    addcarryxU32(&x1531, &x1532, x1530, x1502, x1499);
    var x1533: u32 = undefined;
    var x1534: u1 = undefined;
    addcarryxU32(&x1533, &x1534, x1532, x1500, x1497);
    var x1535: u32 = undefined;
    var x1536: u1 = undefined;
    addcarryxU32(&x1535, &x1536, x1534, x1498, x1495);
    var x1537: u32 = undefined;
    var x1538: u1 = undefined;
    addcarryxU32(&x1537, &x1538, x1536, x1496, x1493);
    const x1539 = (cast(u32, x1538) + x1494);
    var x1540: u32 = undefined;
    var x1541: u1 = undefined;
    addcarryxU32(&x1540, &x1541, 0x0, x1468, x1515);
    var x1542: u32 = undefined;
    var x1543: u1 = undefined;
    addcarryxU32(&x1542, &x1543, x1541, x1470, x1517);
    var x1544: u32 = undefined;
    var x1545: u1 = undefined;
    addcarryxU32(&x1544, &x1545, x1543, x1472, x1519);
    var x1546: u32 = undefined;
    var x1547: u1 = undefined;
    addcarryxU32(&x1546, &x1547, x1545, x1474, x1521);
    var x1548: u32 = undefined;
    var x1549: u1 = undefined;
    addcarryxU32(&x1548, &x1549, x1547, x1476, x1523);
    var x1550: u32 = undefined;
    var x1551: u1 = undefined;
    addcarryxU32(&x1550, &x1551, x1549, x1478, x1525);
    var x1552: u32 = undefined;
    var x1553: u1 = undefined;
    addcarryxU32(&x1552, &x1553, x1551, x1480, x1527);
    var x1554: u32 = undefined;
    var x1555: u1 = undefined;
    addcarryxU32(&x1554, &x1555, x1553, x1482, x1529);
    var x1556: u32 = undefined;
    var x1557: u1 = undefined;
    addcarryxU32(&x1556, &x1557, x1555, x1484, x1531);
    var x1558: u32 = undefined;
    var x1559: u1 = undefined;
    addcarryxU32(&x1558, &x1559, x1557, x1486, x1533);
    var x1560: u32 = undefined;
    var x1561: u1 = undefined;
    addcarryxU32(&x1560, &x1561, x1559, x1488, x1535);
    var x1562: u32 = undefined;
    var x1563: u1 = undefined;
    addcarryxU32(&x1562, &x1563, x1561, x1490, x1537);
    var x1564: u32 = undefined;
    var x1565: u1 = undefined;
    addcarryxU32(&x1564, &x1565, x1563, x1492, x1539);
    var x1566: u32 = undefined;
    var x1567: u32 = undefined;
    mulxU32(&x1566, &x1567, x1540, 0xffffffff);
    var x1568: u32 = undefined;
    var x1569: u32 = undefined;
    mulxU32(&x1568, &x1569, x1540, 0xffffffff);
    var x1570: u32 = undefined;
    var x1571: u32 = undefined;
    mulxU32(&x1570, &x1571, x1540, 0xffffffff);
    var x1572: u32 = undefined;
    var x1573: u32 = undefined;
    mulxU32(&x1572, &x1573, x1540, 0xffffffff);
    var x1574: u32 = undefined;
    var x1575: u32 = undefined;
    mulxU32(&x1574, &x1575, x1540, 0xffffffff);
    var x1576: u32 = undefined;
    var x1577: u32 = undefined;
    mulxU32(&x1576, &x1577, x1540, 0xffffffff);
    var x1578: u32 = undefined;
    var x1579: u32 = undefined;
    mulxU32(&x1578, &x1579, x1540, 0xffffffff);
    var x1580: u32 = undefined;
    var x1581: u32 = undefined;
    mulxU32(&x1580, &x1581, x1540, 0xfffffffe);
    var x1582: u32 = undefined;
    var x1583: u32 = undefined;
    mulxU32(&x1582, &x1583, x1540, 0xffffffff);
    var x1584: u32 = undefined;
    var x1585: u32 = undefined;
    mulxU32(&x1584, &x1585, x1540, 0xffffffff);
    var x1586: u32 = undefined;
    var x1587: u1 = undefined;
    addcarryxU32(&x1586, &x1587, 0x0, x1583, x1580);
    var x1588: u32 = undefined;
    var x1589: u1 = undefined;
    addcarryxU32(&x1588, &x1589, x1587, x1581, x1578);
    var x1590: u32 = undefined;
    var x1591: u1 = undefined;
    addcarryxU32(&x1590, &x1591, x1589, x1579, x1576);
    var x1592: u32 = undefined;
    var x1593: u1 = undefined;
    addcarryxU32(&x1592, &x1593, x1591, x1577, x1574);
    var x1594: u32 = undefined;
    var x1595: u1 = undefined;
    addcarryxU32(&x1594, &x1595, x1593, x1575, x1572);
    var x1596: u32 = undefined;
    var x1597: u1 = undefined;
    addcarryxU32(&x1596, &x1597, x1595, x1573, x1570);
    var x1598: u32 = undefined;
    var x1599: u1 = undefined;
    addcarryxU32(&x1598, &x1599, x1597, x1571, x1568);
    var x1600: u32 = undefined;
    var x1601: u1 = undefined;
    addcarryxU32(&x1600, &x1601, x1599, x1569, x1566);
    const x1602 = (cast(u32, x1601) + x1567);
    var x1603: u32 = undefined;
    var x1604: u1 = undefined;
    addcarryxU32(&x1603, &x1604, 0x0, x1540, x1584);
    var x1605: u32 = undefined;
    var x1606: u1 = undefined;
    addcarryxU32(&x1605, &x1606, x1604, x1542, x1585);
    var x1607: u32 = undefined;
    var x1608: u1 = undefined;
    addcarryxU32(&x1607, &x1608, x1606, x1544, cast(u32, 0x0));
    var x1609: u32 = undefined;
    var x1610: u1 = undefined;
    addcarryxU32(&x1609, &x1610, x1608, x1546, x1582);
    var x1611: u32 = undefined;
    var x1612: u1 = undefined;
    addcarryxU32(&x1611, &x1612, x1610, x1548, x1586);
    var x1613: u32 = undefined;
    var x1614: u1 = undefined;
    addcarryxU32(&x1613, &x1614, x1612, x1550, x1588);
    var x1615: u32 = undefined;
    var x1616: u1 = undefined;
    addcarryxU32(&x1615, &x1616, x1614, x1552, x1590);
    var x1617: u32 = undefined;
    var x1618: u1 = undefined;
    addcarryxU32(&x1617, &x1618, x1616, x1554, x1592);
    var x1619: u32 = undefined;
    var x1620: u1 = undefined;
    addcarryxU32(&x1619, &x1620, x1618, x1556, x1594);
    var x1621: u32 = undefined;
    var x1622: u1 = undefined;
    addcarryxU32(&x1621, &x1622, x1620, x1558, x1596);
    var x1623: u32 = undefined;
    var x1624: u1 = undefined;
    addcarryxU32(&x1623, &x1624, x1622, x1560, x1598);
    var x1625: u32 = undefined;
    var x1626: u1 = undefined;
    addcarryxU32(&x1625, &x1626, x1624, x1562, x1600);
    var x1627: u32 = undefined;
    var x1628: u1 = undefined;
    addcarryxU32(&x1627, &x1628, x1626, x1564, x1602);
    const x1629 = (cast(u32, x1628) + cast(u32, x1565));
    var x1630: u32 = undefined;
    var x1631: u1 = undefined;
    subborrowxU32(&x1630, &x1631, 0x0, x1605, 0xffffffff);
    var x1632: u32 = undefined;
    var x1633: u1 = undefined;
    subborrowxU32(&x1632, &x1633, x1631, x1607, cast(u32, 0x0));
    var x1634: u32 = undefined;
    var x1635: u1 = undefined;
    subborrowxU32(&x1634, &x1635, x1633, x1609, cast(u32, 0x0));
    var x1636: u32 = undefined;
    var x1637: u1 = undefined;
    subborrowxU32(&x1636, &x1637, x1635, x1611, 0xffffffff);
    var x1638: u32 = undefined;
    var x1639: u1 = undefined;
    subborrowxU32(&x1638, &x1639, x1637, x1613, 0xfffffffe);
    var x1640: u32 = undefined;
    var x1641: u1 = undefined;
    subborrowxU32(&x1640, &x1641, x1639, x1615, 0xffffffff);
    var x1642: u32 = undefined;
    var x1643: u1 = undefined;
    subborrowxU32(&x1642, &x1643, x1641, x1617, 0xffffffff);
    var x1644: u32 = undefined;
    var x1645: u1 = undefined;
    subborrowxU32(&x1644, &x1645, x1643, x1619, 0xffffffff);
    var x1646: u32 = undefined;
    var x1647: u1 = undefined;
    subborrowxU32(&x1646, &x1647, x1645, x1621, 0xffffffff);
    var x1648: u32 = undefined;
    var x1649: u1 = undefined;
    subborrowxU32(&x1648, &x1649, x1647, x1623, 0xffffffff);
    var x1650: u32 = undefined;
    var x1651: u1 = undefined;
    subborrowxU32(&x1650, &x1651, x1649, x1625, 0xffffffff);
    var x1652: u32 = undefined;
    var x1653: u1 = undefined;
    subborrowxU32(&x1652, &x1653, x1651, x1627, 0xffffffff);
    var x1654: u32 = undefined;
    var x1655: u1 = undefined;
    subborrowxU32(&x1654, &x1655, x1653, x1629, cast(u32, 0x0));
    var x1656: u32 = undefined;
    cmovznzU32(&x1656, x1655, x1630, x1605);
    var x1657: u32 = undefined;
    cmovznzU32(&x1657, x1655, x1632, x1607);
    var x1658: u32 = undefined;
    cmovznzU32(&x1658, x1655, x1634, x1609);
    var x1659: u32 = undefined;
    cmovznzU32(&x1659, x1655, x1636, x1611);
    var x1660: u32 = undefined;
    cmovznzU32(&x1660, x1655, x1638, x1613);
    var x1661: u32 = undefined;
    cmovznzU32(&x1661, x1655, x1640, x1615);
    var x1662: u32 = undefined;
    cmovznzU32(&x1662, x1655, x1642, x1617);
    var x1663: u32 = undefined;
    cmovznzU32(&x1663, x1655, x1644, x1619);
    var x1664: u32 = undefined;
    cmovznzU32(&x1664, x1655, x1646, x1621);
    var x1665: u32 = undefined;
    cmovznzU32(&x1665, x1655, x1648, x1623);
    var x1666: u32 = undefined;
    cmovznzU32(&x1666, x1655, x1650, x1625);
    var x1667: u32 = undefined;
    cmovznzU32(&x1667, x1655, x1652, x1627);
    out1[0] = x1656;
    out1[1] = x1657;
    out1[2] = x1658;
    out1[3] = x1659;
    out1[4] = x1660;
    out1[5] = x1661;
    out1[6] = x1662;
    out1[7] = x1663;
    out1[8] = x1664;
    out1[9] = x1665;
    out1[10] = x1666;
    out1[11] = x1667;
}

/// The function add adds two field elements in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
///   0 ≤ eval arg2 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) + eval (from_montgomery arg2)) mod m
///   0 ≤ eval out1 < m
///
pub fn add(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement, arg2: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    addcarryxU32(&x1, &x2, 0x0, (arg1[0]), (arg2[0]));
    var x3: u32 = undefined;
    var x4: u1 = undefined;
    addcarryxU32(&x3, &x4, x2, (arg1[1]), (arg2[1]));
    var x5: u32 = undefined;
    var x6: u1 = undefined;
    addcarryxU32(&x5, &x6, x4, (arg1[2]), (arg2[2]));
    var x7: u32 = undefined;
    var x8: u1 = undefined;
    addcarryxU32(&x7, &x8, x6, (arg1[3]), (arg2[3]));
    var x9: u32 = undefined;
    var x10: u1 = undefined;
    addcarryxU32(&x9, &x10, x8, (arg1[4]), (arg2[4]));
    var x11: u32 = undefined;
    var x12: u1 = undefined;
    addcarryxU32(&x11, &x12, x10, (arg1[5]), (arg2[5]));
    var x13: u32 = undefined;
    var x14: u1 = undefined;
    addcarryxU32(&x13, &x14, x12, (arg1[6]), (arg2[6]));
    var x15: u32 = undefined;
    var x16: u1 = undefined;
    addcarryxU32(&x15, &x16, x14, (arg1[7]), (arg2[7]));
    var x17: u32 = undefined;
    var x18: u1 = undefined;
    addcarryxU32(&x17, &x18, x16, (arg1[8]), (arg2[8]));
    var x19: u32 = undefined;
    var x20: u1 = undefined;
    addcarryxU32(&x19, &x20, x18, (arg1[9]), (arg2[9]));
    var x21: u32 = undefined;
    var x22: u1 = undefined;
    addcarryxU32(&x21, &x22, x20, (arg1[10]), (arg2[10]));
    var x23: u32 = undefined;
    var x24: u1 = undefined;
    addcarryxU32(&x23, &x24, x22, (arg1[11]), (arg2[11]));
    var x25: u32 = undefined;
    var x26: u1 = undefined;
    subborrowxU32(&x25, &x26, 0x0, x1, 0xffffffff);
    var x27: u32 = undefined;
    var x28: u1 = undefined;
    subborrowxU32(&x27, &x28, x26, x3, cast(u32, 0x0));
    var x29: u32 = undefined;
    var x30: u1 = undefined;
    subborrowxU32(&x29, &x30, x28, x5, cast(u32, 0x0));
    var x31: u32 = undefined;
    var x32: u1 = undefined;
    subborrowxU32(&x31, &x32, x30, x7, 0xffffffff);
    var x33: u32 = undefined;
    var x34: u1 = undefined;
    subborrowxU32(&x33, &x34, x32, x9, 0xfffffffe);
    var x35: u32 = undefined;
    var x36: u1 = undefined;
    subborrowxU32(&x35, &x36, x34, x11, 0xffffffff);
    var x37: u32 = undefined;
    var x38: u1 = undefined;
    subborrowxU32(&x37, &x38, x36, x13, 0xffffffff);
    var x39: u32 = undefined;
    var x40: u1 = undefined;
    subborrowxU32(&x39, &x40, x38, x15, 0xffffffff);
    var x41: u32 = undefined;
    var x42: u1 = undefined;
    subborrowxU32(&x41, &x42, x40, x17, 0xffffffff);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    subborrowxU32(&x43, &x44, x42, x19, 0xffffffff);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    subborrowxU32(&x45, &x46, x44, x21, 0xffffffff);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    subborrowxU32(&x47, &x48, x46, x23, 0xffffffff);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    subborrowxU32(&x49, &x50, x48, cast(u32, x24), cast(u32, 0x0));
    var x51: u32 = undefined;
    cmovznzU32(&x51, x50, x25, x1);
    var x52: u32 = undefined;
    cmovznzU32(&x52, x50, x27, x3);
    var x53: u32 = undefined;
    cmovznzU32(&x53, x50, x29, x5);
    var x54: u32 = undefined;
    cmovznzU32(&x54, x50, x31, x7);
    var x55: u32 = undefined;
    cmovznzU32(&x55, x50, x33, x9);
    var x56: u32 = undefined;
    cmovznzU32(&x56, x50, x35, x11);
    var x57: u32 = undefined;
    cmovznzU32(&x57, x50, x37, x13);
    var x58: u32 = undefined;
    cmovznzU32(&x58, x50, x39, x15);
    var x59: u32 = undefined;
    cmovznzU32(&x59, x50, x41, x17);
    var x60: u32 = undefined;
    cmovznzU32(&x60, x50, x43, x19);
    var x61: u32 = undefined;
    cmovznzU32(&x61, x50, x45, x21);
    var x62: u32 = undefined;
    cmovznzU32(&x62, x50, x47, x23);
    out1[0] = x51;
    out1[1] = x52;
    out1[2] = x53;
    out1[3] = x54;
    out1[4] = x55;
    out1[5] = x56;
    out1[6] = x57;
    out1[7] = x58;
    out1[8] = x59;
    out1[9] = x60;
    out1[10] = x61;
    out1[11] = x62;
}

/// The function sub subtracts two field elements in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
///   0 ≤ eval arg2 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) - eval (from_montgomery arg2)) mod m
///   0 ≤ eval out1 < m
///
pub fn sub(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement, arg2: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    subborrowxU32(&x1, &x2, 0x0, (arg1[0]), (arg2[0]));
    var x3: u32 = undefined;
    var x4: u1 = undefined;
    subborrowxU32(&x3, &x4, x2, (arg1[1]), (arg2[1]));
    var x5: u32 = undefined;
    var x6: u1 = undefined;
    subborrowxU32(&x5, &x6, x4, (arg1[2]), (arg2[2]));
    var x7: u32 = undefined;
    var x8: u1 = undefined;
    subborrowxU32(&x7, &x8, x6, (arg1[3]), (arg2[3]));
    var x9: u32 = undefined;
    var x10: u1 = undefined;
    subborrowxU32(&x9, &x10, x8, (arg1[4]), (arg2[4]));
    var x11: u32 = undefined;
    var x12: u1 = undefined;
    subborrowxU32(&x11, &x12, x10, (arg1[5]), (arg2[5]));
    var x13: u32 = undefined;
    var x14: u1 = undefined;
    subborrowxU32(&x13, &x14, x12, (arg1[6]), (arg2[6]));
    var x15: u32 = undefined;
    var x16: u1 = undefined;
    subborrowxU32(&x15, &x16, x14, (arg1[7]), (arg2[7]));
    var x17: u32 = undefined;
    var x18: u1 = undefined;
    subborrowxU32(&x17, &x18, x16, (arg1[8]), (arg2[8]));
    var x19: u32 = undefined;
    var x20: u1 = undefined;
    subborrowxU32(&x19, &x20, x18, (arg1[9]), (arg2[9]));
    var x21: u32 = undefined;
    var x22: u1 = undefined;
    subborrowxU32(&x21, &x22, x20, (arg1[10]), (arg2[10]));
    var x23: u32 = undefined;
    var x24: u1 = undefined;
    subborrowxU32(&x23, &x24, x22, (arg1[11]), (arg2[11]));
    var x25: u32 = undefined;
    cmovznzU32(&x25, x24, cast(u32, 0x0), 0xffffffff);
    var x26: u32 = undefined;
    var x27: u1 = undefined;
    addcarryxU32(&x26, &x27, 0x0, x1, x25);
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, x27, x3, cast(u32, 0x0));
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, x5, cast(u32, 0x0));
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, x7, x25);
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, x9, (x25 & 0xfffffffe));
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, x11, x25);
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, x37, x13, x25);
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, x39, x15, x25);
    var x42: u32 = undefined;
    var x43: u1 = undefined;
    addcarryxU32(&x42, &x43, x41, x17, x25);
    var x44: u32 = undefined;
    var x45: u1 = undefined;
    addcarryxU32(&x44, &x45, x43, x19, x25);
    var x46: u32 = undefined;
    var x47: u1 = undefined;
    addcarryxU32(&x46, &x47, x45, x21, x25);
    var x48: u32 = undefined;
    var x49: u1 = undefined;
    addcarryxU32(&x48, &x49, x47, x23, x25);
    out1[0] = x26;
    out1[1] = x28;
    out1[2] = x30;
    out1[3] = x32;
    out1[4] = x34;
    out1[5] = x36;
    out1[6] = x38;
    out1[7] = x40;
    out1[8] = x42;
    out1[9] = x44;
    out1[10] = x46;
    out1[11] = x48;
}

/// The function opp negates a field element in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = -eval (from_montgomery arg1) mod m
///   0 ≤ eval out1 < m
///
pub fn opp(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    subborrowxU32(&x1, &x2, 0x0, cast(u32, 0x0), (arg1[0]));
    var x3: u32 = undefined;
    var x4: u1 = undefined;
    subborrowxU32(&x3, &x4, x2, cast(u32, 0x0), (arg1[1]));
    var x5: u32 = undefined;
    var x6: u1 = undefined;
    subborrowxU32(&x5, &x6, x4, cast(u32, 0x0), (arg1[2]));
    var x7: u32 = undefined;
    var x8: u1 = undefined;
    subborrowxU32(&x7, &x8, x6, cast(u32, 0x0), (arg1[3]));
    var x9: u32 = undefined;
    var x10: u1 = undefined;
    subborrowxU32(&x9, &x10, x8, cast(u32, 0x0), (arg1[4]));
    var x11: u32 = undefined;
    var x12: u1 = undefined;
    subborrowxU32(&x11, &x12, x10, cast(u32, 0x0), (arg1[5]));
    var x13: u32 = undefined;
    var x14: u1 = undefined;
    subborrowxU32(&x13, &x14, x12, cast(u32, 0x0), (arg1[6]));
    var x15: u32 = undefined;
    var x16: u1 = undefined;
    subborrowxU32(&x15, &x16, x14, cast(u32, 0x0), (arg1[7]));
    var x17: u32 = undefined;
    var x18: u1 = undefined;
    subborrowxU32(&x17, &x18, x16, cast(u32, 0x0), (arg1[8]));
    var x19: u32 = undefined;
    var x20: u1 = undefined;
    subborrowxU32(&x19, &x20, x18, cast(u32, 0x0), (arg1[9]));
    var x21: u32 = undefined;
    var x22: u1 = undefined;
    subborrowxU32(&x21, &x22, x20, cast(u32, 0x0), (arg1[10]));
    var x23: u32 = undefined;
    var x24: u1 = undefined;
    subborrowxU32(&x23, &x24, x22, cast(u32, 0x0), (arg1[11]));
    var x25: u32 = undefined;
    cmovznzU32(&x25, x24, cast(u32, 0x0), 0xffffffff);
    var x26: u32 = undefined;
    var x27: u1 = undefined;
    addcarryxU32(&x26, &x27, 0x0, x1, x25);
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, x27, x3, cast(u32, 0x0));
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, x5, cast(u32, 0x0));
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, x7, x25);
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, x9, (x25 & 0xfffffffe));
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, x11, x25);
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, x37, x13, x25);
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, x39, x15, x25);
    var x42: u32 = undefined;
    var x43: u1 = undefined;
    addcarryxU32(&x42, &x43, x41, x17, x25);
    var x44: u32 = undefined;
    var x45: u1 = undefined;
    addcarryxU32(&x44, &x45, x43, x19, x25);
    var x46: u32 = undefined;
    var x47: u1 = undefined;
    addcarryxU32(&x46, &x47, x45, x21, x25);
    var x48: u32 = undefined;
    var x49: u1 = undefined;
    addcarryxU32(&x48, &x49, x47, x23, x25);
    out1[0] = x26;
    out1[1] = x28;
    out1[2] = x30;
    out1[3] = x32;
    out1[4] = x34;
    out1[5] = x36;
    out1[6] = x38;
    out1[7] = x40;
    out1[8] = x42;
    out1[9] = x44;
    out1[10] = x46;
    out1[11] = x48;
}

/// The function fromMontgomery translates a field element out of the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval out1 mod m = (eval arg1 * ((2^32)⁻¹ mod m)^12) mod m
///   0 ≤ eval out1 < m
///
pub fn fromMontgomery(out1: *NonMontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[0]);
    var x2: u32 = undefined;
    var x3: u32 = undefined;
    mulxU32(&x2, &x3, x1, 0xffffffff);
    var x4: u32 = undefined;
    var x5: u32 = undefined;
    mulxU32(&x4, &x5, x1, 0xffffffff);
    var x6: u32 = undefined;
    var x7: u32 = undefined;
    mulxU32(&x6, &x7, x1, 0xffffffff);
    var x8: u32 = undefined;
    var x9: u32 = undefined;
    mulxU32(&x8, &x9, x1, 0xffffffff);
    var x10: u32 = undefined;
    var x11: u32 = undefined;
    mulxU32(&x10, &x11, x1, 0xffffffff);
    var x12: u32 = undefined;
    var x13: u32 = undefined;
    mulxU32(&x12, &x13, x1, 0xffffffff);
    var x14: u32 = undefined;
    var x15: u32 = undefined;
    mulxU32(&x14, &x15, x1, 0xffffffff);
    var x16: u32 = undefined;
    var x17: u32 = undefined;
    mulxU32(&x16, &x17, x1, 0xfffffffe);
    var x18: u32 = undefined;
    var x19: u32 = undefined;
    mulxU32(&x18, &x19, x1, 0xffffffff);
    var x20: u32 = undefined;
    var x21: u32 = undefined;
    mulxU32(&x20, &x21, x1, 0xffffffff);
    var x22: u32 = undefined;
    var x23: u1 = undefined;
    addcarryxU32(&x22, &x23, 0x0, x19, x16);
    var x24: u32 = undefined;
    var x25: u1 = undefined;
    addcarryxU32(&x24, &x25, x23, x17, x14);
    var x26: u32 = undefined;
    var x27: u1 = undefined;
    addcarryxU32(&x26, &x27, x25, x15, x12);
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, x27, x13, x10);
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, x11, x8);
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, x9, x6);
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, x7, x4);
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, x5, x2);
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, 0x0, x1, x20);
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, 0x0, (cast(u32, x39) + x21), (arg1[1]));
    var x42: u32 = undefined;
    var x43: u32 = undefined;
    mulxU32(&x42, &x43, x40, 0xffffffff);
    var x44: u32 = undefined;
    var x45: u32 = undefined;
    mulxU32(&x44, &x45, x40, 0xffffffff);
    var x46: u32 = undefined;
    var x47: u32 = undefined;
    mulxU32(&x46, &x47, x40, 0xffffffff);
    var x48: u32 = undefined;
    var x49: u32 = undefined;
    mulxU32(&x48, &x49, x40, 0xffffffff);
    var x50: u32 = undefined;
    var x51: u32 = undefined;
    mulxU32(&x50, &x51, x40, 0xffffffff);
    var x52: u32 = undefined;
    var x53: u32 = undefined;
    mulxU32(&x52, &x53, x40, 0xffffffff);
    var x54: u32 = undefined;
    var x55: u32 = undefined;
    mulxU32(&x54, &x55, x40, 0xffffffff);
    var x56: u32 = undefined;
    var x57: u32 = undefined;
    mulxU32(&x56, &x57, x40, 0xfffffffe);
    var x58: u32 = undefined;
    var x59: u32 = undefined;
    mulxU32(&x58, &x59, x40, 0xffffffff);
    var x60: u32 = undefined;
    var x61: u32 = undefined;
    mulxU32(&x60, &x61, x40, 0xffffffff);
    var x62: u32 = undefined;
    var x63: u1 = undefined;
    addcarryxU32(&x62, &x63, 0x0, x59, x56);
    var x64: u32 = undefined;
    var x65: u1 = undefined;
    addcarryxU32(&x64, &x65, x63, x57, x54);
    var x66: u32 = undefined;
    var x67: u1 = undefined;
    addcarryxU32(&x66, &x67, x65, x55, x52);
    var x68: u32 = undefined;
    var x69: u1 = undefined;
    addcarryxU32(&x68, &x69, x67, x53, x50);
    var x70: u32 = undefined;
    var x71: u1 = undefined;
    addcarryxU32(&x70, &x71, x69, x51, x48);
    var x72: u32 = undefined;
    var x73: u1 = undefined;
    addcarryxU32(&x72, &x73, x71, x49, x46);
    var x74: u32 = undefined;
    var x75: u1 = undefined;
    addcarryxU32(&x74, &x75, x73, x47, x44);
    var x76: u32 = undefined;
    var x77: u1 = undefined;
    addcarryxU32(&x76, &x77, x75, x45, x42);
    var x78: u32 = undefined;
    var x79: u1 = undefined;
    addcarryxU32(&x78, &x79, 0x0, x40, x60);
    var x80: u32 = undefined;
    var x81: u1 = undefined;
    addcarryxU32(&x80, &x81, x79, cast(u32, x41), x61);
    var x82: u32 = undefined;
    var x83: u1 = undefined;
    addcarryxU32(&x82, &x83, x81, x18, cast(u32, 0x0));
    var x84: u32 = undefined;
    var x85: u1 = undefined;
    addcarryxU32(&x84, &x85, x83, x22, x58);
    var x86: u32 = undefined;
    var x87: u1 = undefined;
    addcarryxU32(&x86, &x87, x85, x24, x62);
    var x88: u32 = undefined;
    var x89: u1 = undefined;
    addcarryxU32(&x88, &x89, x87, x26, x64);
    var x90: u32 = undefined;
    var x91: u1 = undefined;
    addcarryxU32(&x90, &x91, x89, x28, x66);
    var x92: u32 = undefined;
    var x93: u1 = undefined;
    addcarryxU32(&x92, &x93, x91, x30, x68);
    var x94: u32 = undefined;
    var x95: u1 = undefined;
    addcarryxU32(&x94, &x95, x93, x32, x70);
    var x96: u32 = undefined;
    var x97: u1 = undefined;
    addcarryxU32(&x96, &x97, x95, x34, x72);
    var x98: u32 = undefined;
    var x99: u1 = undefined;
    addcarryxU32(&x98, &x99, x97, x36, x74);
    var x100: u32 = undefined;
    var x101: u1 = undefined;
    addcarryxU32(&x100, &x101, x99, (cast(u32, x37) + x3), x76);
    var x102: u32 = undefined;
    var x103: u1 = undefined;
    addcarryxU32(&x102, &x103, x101, cast(u32, 0x0), (cast(u32, x77) + x43));
    var x104: u32 = undefined;
    var x105: u1 = undefined;
    addcarryxU32(&x104, &x105, 0x0, x80, (arg1[2]));
    var x106: u32 = undefined;
    var x107: u1 = undefined;
    addcarryxU32(&x106, &x107, x105, x82, cast(u32, 0x0));
    var x108: u32 = undefined;
    var x109: u1 = undefined;
    addcarryxU32(&x108, &x109, x107, x84, cast(u32, 0x0));
    var x110: u32 = undefined;
    var x111: u1 = undefined;
    addcarryxU32(&x110, &x111, x109, x86, cast(u32, 0x0));
    var x112: u32 = undefined;
    var x113: u1 = undefined;
    addcarryxU32(&x112, &x113, x111, x88, cast(u32, 0x0));
    var x114: u32 = undefined;
    var x115: u1 = undefined;
    addcarryxU32(&x114, &x115, x113, x90, cast(u32, 0x0));
    var x116: u32 = undefined;
    var x117: u1 = undefined;
    addcarryxU32(&x116, &x117, x115, x92, cast(u32, 0x0));
    var x118: u32 = undefined;
    var x119: u1 = undefined;
    addcarryxU32(&x118, &x119, x117, x94, cast(u32, 0x0));
    var x120: u32 = undefined;
    var x121: u1 = undefined;
    addcarryxU32(&x120, &x121, x119, x96, cast(u32, 0x0));
    var x122: u32 = undefined;
    var x123: u1 = undefined;
    addcarryxU32(&x122, &x123, x121, x98, cast(u32, 0x0));
    var x124: u32 = undefined;
    var x125: u1 = undefined;
    addcarryxU32(&x124, &x125, x123, x100, cast(u32, 0x0));
    var x126: u32 = undefined;
    var x127: u1 = undefined;
    addcarryxU32(&x126, &x127, x125, x102, cast(u32, 0x0));
    var x128: u32 = undefined;
    var x129: u32 = undefined;
    mulxU32(&x128, &x129, x104, 0xffffffff);
    var x130: u32 = undefined;
    var x131: u32 = undefined;
    mulxU32(&x130, &x131, x104, 0xffffffff);
    var x132: u32 = undefined;
    var x133: u32 = undefined;
    mulxU32(&x132, &x133, x104, 0xffffffff);
    var x134: u32 = undefined;
    var x135: u32 = undefined;
    mulxU32(&x134, &x135, x104, 0xffffffff);
    var x136: u32 = undefined;
    var x137: u32 = undefined;
    mulxU32(&x136, &x137, x104, 0xffffffff);
    var x138: u32 = undefined;
    var x139: u32 = undefined;
    mulxU32(&x138, &x139, x104, 0xffffffff);
    var x140: u32 = undefined;
    var x141: u32 = undefined;
    mulxU32(&x140, &x141, x104, 0xffffffff);
    var x142: u32 = undefined;
    var x143: u32 = undefined;
    mulxU32(&x142, &x143, x104, 0xfffffffe);
    var x144: u32 = undefined;
    var x145: u32 = undefined;
    mulxU32(&x144, &x145, x104, 0xffffffff);
    var x146: u32 = undefined;
    var x147: u32 = undefined;
    mulxU32(&x146, &x147, x104, 0xffffffff);
    var x148: u32 = undefined;
    var x149: u1 = undefined;
    addcarryxU32(&x148, &x149, 0x0, x145, x142);
    var x150: u32 = undefined;
    var x151: u1 = undefined;
    addcarryxU32(&x150, &x151, x149, x143, x140);
    var x152: u32 = undefined;
    var x153: u1 = undefined;
    addcarryxU32(&x152, &x153, x151, x141, x138);
    var x154: u32 = undefined;
    var x155: u1 = undefined;
    addcarryxU32(&x154, &x155, x153, x139, x136);
    var x156: u32 = undefined;
    var x157: u1 = undefined;
    addcarryxU32(&x156, &x157, x155, x137, x134);
    var x158: u32 = undefined;
    var x159: u1 = undefined;
    addcarryxU32(&x158, &x159, x157, x135, x132);
    var x160: u32 = undefined;
    var x161: u1 = undefined;
    addcarryxU32(&x160, &x161, x159, x133, x130);
    var x162: u32 = undefined;
    var x163: u1 = undefined;
    addcarryxU32(&x162, &x163, x161, x131, x128);
    var x164: u32 = undefined;
    var x165: u1 = undefined;
    addcarryxU32(&x164, &x165, 0x0, x104, x146);
    var x166: u32 = undefined;
    var x167: u1 = undefined;
    addcarryxU32(&x166, &x167, x165, x106, x147);
    var x168: u32 = undefined;
    var x169: u1 = undefined;
    addcarryxU32(&x168, &x169, x167, x108, cast(u32, 0x0));
    var x170: u32 = undefined;
    var x171: u1 = undefined;
    addcarryxU32(&x170, &x171, x169, x110, x144);
    var x172: u32 = undefined;
    var x173: u1 = undefined;
    addcarryxU32(&x172, &x173, x171, x112, x148);
    var x174: u32 = undefined;
    var x175: u1 = undefined;
    addcarryxU32(&x174, &x175, x173, x114, x150);
    var x176: u32 = undefined;
    var x177: u1 = undefined;
    addcarryxU32(&x176, &x177, x175, x116, x152);
    var x178: u32 = undefined;
    var x179: u1 = undefined;
    addcarryxU32(&x178, &x179, x177, x118, x154);
    var x180: u32 = undefined;
    var x181: u1 = undefined;
    addcarryxU32(&x180, &x181, x179, x120, x156);
    var x182: u32 = undefined;
    var x183: u1 = undefined;
    addcarryxU32(&x182, &x183, x181, x122, x158);
    var x184: u32 = undefined;
    var x185: u1 = undefined;
    addcarryxU32(&x184, &x185, x183, x124, x160);
    var x186: u32 = undefined;
    var x187: u1 = undefined;
    addcarryxU32(&x186, &x187, x185, x126, x162);
    var x188: u32 = undefined;
    var x189: u1 = undefined;
    addcarryxU32(&x188, &x189, x187, (cast(u32, x127) + cast(u32, x103)), (cast(u32, x163) + x129));
    var x190: u32 = undefined;
    var x191: u1 = undefined;
    addcarryxU32(&x190, &x191, 0x0, x166, (arg1[3]));
    var x192: u32 = undefined;
    var x193: u1 = undefined;
    addcarryxU32(&x192, &x193, x191, x168, cast(u32, 0x0));
    var x194: u32 = undefined;
    var x195: u1 = undefined;
    addcarryxU32(&x194, &x195, x193, x170, cast(u32, 0x0));
    var x196: u32 = undefined;
    var x197: u1 = undefined;
    addcarryxU32(&x196, &x197, x195, x172, cast(u32, 0x0));
    var x198: u32 = undefined;
    var x199: u1 = undefined;
    addcarryxU32(&x198, &x199, x197, x174, cast(u32, 0x0));
    var x200: u32 = undefined;
    var x201: u1 = undefined;
    addcarryxU32(&x200, &x201, x199, x176, cast(u32, 0x0));
    var x202: u32 = undefined;
    var x203: u1 = undefined;
    addcarryxU32(&x202, &x203, x201, x178, cast(u32, 0x0));
    var x204: u32 = undefined;
    var x205: u1 = undefined;
    addcarryxU32(&x204, &x205, x203, x180, cast(u32, 0x0));
    var x206: u32 = undefined;
    var x207: u1 = undefined;
    addcarryxU32(&x206, &x207, x205, x182, cast(u32, 0x0));
    var x208: u32 = undefined;
    var x209: u1 = undefined;
    addcarryxU32(&x208, &x209, x207, x184, cast(u32, 0x0));
    var x210: u32 = undefined;
    var x211: u1 = undefined;
    addcarryxU32(&x210, &x211, x209, x186, cast(u32, 0x0));
    var x212: u32 = undefined;
    var x213: u1 = undefined;
    addcarryxU32(&x212, &x213, x211, x188, cast(u32, 0x0));
    var x214: u32 = undefined;
    var x215: u32 = undefined;
    mulxU32(&x214, &x215, x190, 0xffffffff);
    var x216: u32 = undefined;
    var x217: u32 = undefined;
    mulxU32(&x216, &x217, x190, 0xffffffff);
    var x218: u32 = undefined;
    var x219: u32 = undefined;
    mulxU32(&x218, &x219, x190, 0xffffffff);
    var x220: u32 = undefined;
    var x221: u32 = undefined;
    mulxU32(&x220, &x221, x190, 0xffffffff);
    var x222: u32 = undefined;
    var x223: u32 = undefined;
    mulxU32(&x222, &x223, x190, 0xffffffff);
    var x224: u32 = undefined;
    var x225: u32 = undefined;
    mulxU32(&x224, &x225, x190, 0xffffffff);
    var x226: u32 = undefined;
    var x227: u32 = undefined;
    mulxU32(&x226, &x227, x190, 0xffffffff);
    var x228: u32 = undefined;
    var x229: u32 = undefined;
    mulxU32(&x228, &x229, x190, 0xfffffffe);
    var x230: u32 = undefined;
    var x231: u32 = undefined;
    mulxU32(&x230, &x231, x190, 0xffffffff);
    var x232: u32 = undefined;
    var x233: u32 = undefined;
    mulxU32(&x232, &x233, x190, 0xffffffff);
    var x234: u32 = undefined;
    var x235: u1 = undefined;
    addcarryxU32(&x234, &x235, 0x0, x231, x228);
    var x236: u32 = undefined;
    var x237: u1 = undefined;
    addcarryxU32(&x236, &x237, x235, x229, x226);
    var x238: u32 = undefined;
    var x239: u1 = undefined;
    addcarryxU32(&x238, &x239, x237, x227, x224);
    var x240: u32 = undefined;
    var x241: u1 = undefined;
    addcarryxU32(&x240, &x241, x239, x225, x222);
    var x242: u32 = undefined;
    var x243: u1 = undefined;
    addcarryxU32(&x242, &x243, x241, x223, x220);
    var x244: u32 = undefined;
    var x245: u1 = undefined;
    addcarryxU32(&x244, &x245, x243, x221, x218);
    var x246: u32 = undefined;
    var x247: u1 = undefined;
    addcarryxU32(&x246, &x247, x245, x219, x216);
    var x248: u32 = undefined;
    var x249: u1 = undefined;
    addcarryxU32(&x248, &x249, x247, x217, x214);
    var x250: u32 = undefined;
    var x251: u1 = undefined;
    addcarryxU32(&x250, &x251, 0x0, x190, x232);
    var x252: u32 = undefined;
    var x253: u1 = undefined;
    addcarryxU32(&x252, &x253, x251, x192, x233);
    var x254: u32 = undefined;
    var x255: u1 = undefined;
    addcarryxU32(&x254, &x255, x253, x194, cast(u32, 0x0));
    var x256: u32 = undefined;
    var x257: u1 = undefined;
    addcarryxU32(&x256, &x257, x255, x196, x230);
    var x258: u32 = undefined;
    var x259: u1 = undefined;
    addcarryxU32(&x258, &x259, x257, x198, x234);
    var x260: u32 = undefined;
    var x261: u1 = undefined;
    addcarryxU32(&x260, &x261, x259, x200, x236);
    var x262: u32 = undefined;
    var x263: u1 = undefined;
    addcarryxU32(&x262, &x263, x261, x202, x238);
    var x264: u32 = undefined;
    var x265: u1 = undefined;
    addcarryxU32(&x264, &x265, x263, x204, x240);
    var x266: u32 = undefined;
    var x267: u1 = undefined;
    addcarryxU32(&x266, &x267, x265, x206, x242);
    var x268: u32 = undefined;
    var x269: u1 = undefined;
    addcarryxU32(&x268, &x269, x267, x208, x244);
    var x270: u32 = undefined;
    var x271: u1 = undefined;
    addcarryxU32(&x270, &x271, x269, x210, x246);
    var x272: u32 = undefined;
    var x273: u1 = undefined;
    addcarryxU32(&x272, &x273, x271, x212, x248);
    var x274: u32 = undefined;
    var x275: u1 = undefined;
    addcarryxU32(&x274, &x275, x273, (cast(u32, x213) + cast(u32, x189)), (cast(u32, x249) + x215));
    var x276: u32 = undefined;
    var x277: u1 = undefined;
    addcarryxU32(&x276, &x277, 0x0, x252, (arg1[4]));
    var x278: u32 = undefined;
    var x279: u1 = undefined;
    addcarryxU32(&x278, &x279, x277, x254, cast(u32, 0x0));
    var x280: u32 = undefined;
    var x281: u1 = undefined;
    addcarryxU32(&x280, &x281, x279, x256, cast(u32, 0x0));
    var x282: u32 = undefined;
    var x283: u1 = undefined;
    addcarryxU32(&x282, &x283, x281, x258, cast(u32, 0x0));
    var x284: u32 = undefined;
    var x285: u1 = undefined;
    addcarryxU32(&x284, &x285, x283, x260, cast(u32, 0x0));
    var x286: u32 = undefined;
    var x287: u1 = undefined;
    addcarryxU32(&x286, &x287, x285, x262, cast(u32, 0x0));
    var x288: u32 = undefined;
    var x289: u1 = undefined;
    addcarryxU32(&x288, &x289, x287, x264, cast(u32, 0x0));
    var x290: u32 = undefined;
    var x291: u1 = undefined;
    addcarryxU32(&x290, &x291, x289, x266, cast(u32, 0x0));
    var x292: u32 = undefined;
    var x293: u1 = undefined;
    addcarryxU32(&x292, &x293, x291, x268, cast(u32, 0x0));
    var x294: u32 = undefined;
    var x295: u1 = undefined;
    addcarryxU32(&x294, &x295, x293, x270, cast(u32, 0x0));
    var x296: u32 = undefined;
    var x297: u1 = undefined;
    addcarryxU32(&x296, &x297, x295, x272, cast(u32, 0x0));
    var x298: u32 = undefined;
    var x299: u1 = undefined;
    addcarryxU32(&x298, &x299, x297, x274, cast(u32, 0x0));
    var x300: u32 = undefined;
    var x301: u32 = undefined;
    mulxU32(&x300, &x301, x276, 0xffffffff);
    var x302: u32 = undefined;
    var x303: u32 = undefined;
    mulxU32(&x302, &x303, x276, 0xffffffff);
    var x304: u32 = undefined;
    var x305: u32 = undefined;
    mulxU32(&x304, &x305, x276, 0xffffffff);
    var x306: u32 = undefined;
    var x307: u32 = undefined;
    mulxU32(&x306, &x307, x276, 0xffffffff);
    var x308: u32 = undefined;
    var x309: u32 = undefined;
    mulxU32(&x308, &x309, x276, 0xffffffff);
    var x310: u32 = undefined;
    var x311: u32 = undefined;
    mulxU32(&x310, &x311, x276, 0xffffffff);
    var x312: u32 = undefined;
    var x313: u32 = undefined;
    mulxU32(&x312, &x313, x276, 0xffffffff);
    var x314: u32 = undefined;
    var x315: u32 = undefined;
    mulxU32(&x314, &x315, x276, 0xfffffffe);
    var x316: u32 = undefined;
    var x317: u32 = undefined;
    mulxU32(&x316, &x317, x276, 0xffffffff);
    var x318: u32 = undefined;
    var x319: u32 = undefined;
    mulxU32(&x318, &x319, x276, 0xffffffff);
    var x320: u32 = undefined;
    var x321: u1 = undefined;
    addcarryxU32(&x320, &x321, 0x0, x317, x314);
    var x322: u32 = undefined;
    var x323: u1 = undefined;
    addcarryxU32(&x322, &x323, x321, x315, x312);
    var x324: u32 = undefined;
    var x325: u1 = undefined;
    addcarryxU32(&x324, &x325, x323, x313, x310);
    var x326: u32 = undefined;
    var x327: u1 = undefined;
    addcarryxU32(&x326, &x327, x325, x311, x308);
    var x328: u32 = undefined;
    var x329: u1 = undefined;
    addcarryxU32(&x328, &x329, x327, x309, x306);
    var x330: u32 = undefined;
    var x331: u1 = undefined;
    addcarryxU32(&x330, &x331, x329, x307, x304);
    var x332: u32 = undefined;
    var x333: u1 = undefined;
    addcarryxU32(&x332, &x333, x331, x305, x302);
    var x334: u32 = undefined;
    var x335: u1 = undefined;
    addcarryxU32(&x334, &x335, x333, x303, x300);
    var x336: u32 = undefined;
    var x337: u1 = undefined;
    addcarryxU32(&x336, &x337, 0x0, x276, x318);
    var x338: u32 = undefined;
    var x339: u1 = undefined;
    addcarryxU32(&x338, &x339, x337, x278, x319);
    var x340: u32 = undefined;
    var x341: u1 = undefined;
    addcarryxU32(&x340, &x341, x339, x280, cast(u32, 0x0));
    var x342: u32 = undefined;
    var x343: u1 = undefined;
    addcarryxU32(&x342, &x343, x341, x282, x316);
    var x344: u32 = undefined;
    var x345: u1 = undefined;
    addcarryxU32(&x344, &x345, x343, x284, x320);
    var x346: u32 = undefined;
    var x347: u1 = undefined;
    addcarryxU32(&x346, &x347, x345, x286, x322);
    var x348: u32 = undefined;
    var x349: u1 = undefined;
    addcarryxU32(&x348, &x349, x347, x288, x324);
    var x350: u32 = undefined;
    var x351: u1 = undefined;
    addcarryxU32(&x350, &x351, x349, x290, x326);
    var x352: u32 = undefined;
    var x353: u1 = undefined;
    addcarryxU32(&x352, &x353, x351, x292, x328);
    var x354: u32 = undefined;
    var x355: u1 = undefined;
    addcarryxU32(&x354, &x355, x353, x294, x330);
    var x356: u32 = undefined;
    var x357: u1 = undefined;
    addcarryxU32(&x356, &x357, x355, x296, x332);
    var x358: u32 = undefined;
    var x359: u1 = undefined;
    addcarryxU32(&x358, &x359, x357, x298, x334);
    var x360: u32 = undefined;
    var x361: u1 = undefined;
    addcarryxU32(&x360, &x361, x359, (cast(u32, x299) + cast(u32, x275)), (cast(u32, x335) + x301));
    var x362: u32 = undefined;
    var x363: u1 = undefined;
    addcarryxU32(&x362, &x363, 0x0, x338, (arg1[5]));
    var x364: u32 = undefined;
    var x365: u1 = undefined;
    addcarryxU32(&x364, &x365, x363, x340, cast(u32, 0x0));
    var x366: u32 = undefined;
    var x367: u1 = undefined;
    addcarryxU32(&x366, &x367, x365, x342, cast(u32, 0x0));
    var x368: u32 = undefined;
    var x369: u1 = undefined;
    addcarryxU32(&x368, &x369, x367, x344, cast(u32, 0x0));
    var x370: u32 = undefined;
    var x371: u1 = undefined;
    addcarryxU32(&x370, &x371, x369, x346, cast(u32, 0x0));
    var x372: u32 = undefined;
    var x373: u1 = undefined;
    addcarryxU32(&x372, &x373, x371, x348, cast(u32, 0x0));
    var x374: u32 = undefined;
    var x375: u1 = undefined;
    addcarryxU32(&x374, &x375, x373, x350, cast(u32, 0x0));
    var x376: u32 = undefined;
    var x377: u1 = undefined;
    addcarryxU32(&x376, &x377, x375, x352, cast(u32, 0x0));
    var x378: u32 = undefined;
    var x379: u1 = undefined;
    addcarryxU32(&x378, &x379, x377, x354, cast(u32, 0x0));
    var x380: u32 = undefined;
    var x381: u1 = undefined;
    addcarryxU32(&x380, &x381, x379, x356, cast(u32, 0x0));
    var x382: u32 = undefined;
    var x383: u1 = undefined;
    addcarryxU32(&x382, &x383, x381, x358, cast(u32, 0x0));
    var x384: u32 = undefined;
    var x385: u1 = undefined;
    addcarryxU32(&x384, &x385, x383, x360, cast(u32, 0x0));
    var x386: u32 = undefined;
    var x387: u32 = undefined;
    mulxU32(&x386, &x387, x362, 0xffffffff);
    var x388: u32 = undefined;
    var x389: u32 = undefined;
    mulxU32(&x388, &x389, x362, 0xffffffff);
    var x390: u32 = undefined;
    var x391: u32 = undefined;
    mulxU32(&x390, &x391, x362, 0xffffffff);
    var x392: u32 = undefined;
    var x393: u32 = undefined;
    mulxU32(&x392, &x393, x362, 0xffffffff);
    var x394: u32 = undefined;
    var x395: u32 = undefined;
    mulxU32(&x394, &x395, x362, 0xffffffff);
    var x396: u32 = undefined;
    var x397: u32 = undefined;
    mulxU32(&x396, &x397, x362, 0xffffffff);
    var x398: u32 = undefined;
    var x399: u32 = undefined;
    mulxU32(&x398, &x399, x362, 0xffffffff);
    var x400: u32 = undefined;
    var x401: u32 = undefined;
    mulxU32(&x400, &x401, x362, 0xfffffffe);
    var x402: u32 = undefined;
    var x403: u32 = undefined;
    mulxU32(&x402, &x403, x362, 0xffffffff);
    var x404: u32 = undefined;
    var x405: u32 = undefined;
    mulxU32(&x404, &x405, x362, 0xffffffff);
    var x406: u32 = undefined;
    var x407: u1 = undefined;
    addcarryxU32(&x406, &x407, 0x0, x403, x400);
    var x408: u32 = undefined;
    var x409: u1 = undefined;
    addcarryxU32(&x408, &x409, x407, x401, x398);
    var x410: u32 = undefined;
    var x411: u1 = undefined;
    addcarryxU32(&x410, &x411, x409, x399, x396);
    var x412: u32 = undefined;
    var x413: u1 = undefined;
    addcarryxU32(&x412, &x413, x411, x397, x394);
    var x414: u32 = undefined;
    var x415: u1 = undefined;
    addcarryxU32(&x414, &x415, x413, x395, x392);
    var x416: u32 = undefined;
    var x417: u1 = undefined;
    addcarryxU32(&x416, &x417, x415, x393, x390);
    var x418: u32 = undefined;
    var x419: u1 = undefined;
    addcarryxU32(&x418, &x419, x417, x391, x388);
    var x420: u32 = undefined;
    var x421: u1 = undefined;
    addcarryxU32(&x420, &x421, x419, x389, x386);
    var x422: u32 = undefined;
    var x423: u1 = undefined;
    addcarryxU32(&x422, &x423, 0x0, x362, x404);
    var x424: u32 = undefined;
    var x425: u1 = undefined;
    addcarryxU32(&x424, &x425, x423, x364, x405);
    var x426: u32 = undefined;
    var x427: u1 = undefined;
    addcarryxU32(&x426, &x427, x425, x366, cast(u32, 0x0));
    var x428: u32 = undefined;
    var x429: u1 = undefined;
    addcarryxU32(&x428, &x429, x427, x368, x402);
    var x430: u32 = undefined;
    var x431: u1 = undefined;
    addcarryxU32(&x430, &x431, x429, x370, x406);
    var x432: u32 = undefined;
    var x433: u1 = undefined;
    addcarryxU32(&x432, &x433, x431, x372, x408);
    var x434: u32 = undefined;
    var x435: u1 = undefined;
    addcarryxU32(&x434, &x435, x433, x374, x410);
    var x436: u32 = undefined;
    var x437: u1 = undefined;
    addcarryxU32(&x436, &x437, x435, x376, x412);
    var x438: u32 = undefined;
    var x439: u1 = undefined;
    addcarryxU32(&x438, &x439, x437, x378, x414);
    var x440: u32 = undefined;
    var x441: u1 = undefined;
    addcarryxU32(&x440, &x441, x439, x380, x416);
    var x442: u32 = undefined;
    var x443: u1 = undefined;
    addcarryxU32(&x442, &x443, x441, x382, x418);
    var x444: u32 = undefined;
    var x445: u1 = undefined;
    addcarryxU32(&x444, &x445, x443, x384, x420);
    var x446: u32 = undefined;
    var x447: u1 = undefined;
    addcarryxU32(&x446, &x447, x445, (cast(u32, x385) + cast(u32, x361)), (cast(u32, x421) + x387));
    var x448: u32 = undefined;
    var x449: u1 = undefined;
    addcarryxU32(&x448, &x449, 0x0, x424, (arg1[6]));
    var x450: u32 = undefined;
    var x451: u1 = undefined;
    addcarryxU32(&x450, &x451, x449, x426, cast(u32, 0x0));
    var x452: u32 = undefined;
    var x453: u1 = undefined;
    addcarryxU32(&x452, &x453, x451, x428, cast(u32, 0x0));
    var x454: u32 = undefined;
    var x455: u1 = undefined;
    addcarryxU32(&x454, &x455, x453, x430, cast(u32, 0x0));
    var x456: u32 = undefined;
    var x457: u1 = undefined;
    addcarryxU32(&x456, &x457, x455, x432, cast(u32, 0x0));
    var x458: u32 = undefined;
    var x459: u1 = undefined;
    addcarryxU32(&x458, &x459, x457, x434, cast(u32, 0x0));
    var x460: u32 = undefined;
    var x461: u1 = undefined;
    addcarryxU32(&x460, &x461, x459, x436, cast(u32, 0x0));
    var x462: u32 = undefined;
    var x463: u1 = undefined;
    addcarryxU32(&x462, &x463, x461, x438, cast(u32, 0x0));
    var x464: u32 = undefined;
    var x465: u1 = undefined;
    addcarryxU32(&x464, &x465, x463, x440, cast(u32, 0x0));
    var x466: u32 = undefined;
    var x467: u1 = undefined;
    addcarryxU32(&x466, &x467, x465, x442, cast(u32, 0x0));
    var x468: u32 = undefined;
    var x469: u1 = undefined;
    addcarryxU32(&x468, &x469, x467, x444, cast(u32, 0x0));
    var x470: u32 = undefined;
    var x471: u1 = undefined;
    addcarryxU32(&x470, &x471, x469, x446, cast(u32, 0x0));
    var x472: u32 = undefined;
    var x473: u32 = undefined;
    mulxU32(&x472, &x473, x448, 0xffffffff);
    var x474: u32 = undefined;
    var x475: u32 = undefined;
    mulxU32(&x474, &x475, x448, 0xffffffff);
    var x476: u32 = undefined;
    var x477: u32 = undefined;
    mulxU32(&x476, &x477, x448, 0xffffffff);
    var x478: u32 = undefined;
    var x479: u32 = undefined;
    mulxU32(&x478, &x479, x448, 0xffffffff);
    var x480: u32 = undefined;
    var x481: u32 = undefined;
    mulxU32(&x480, &x481, x448, 0xffffffff);
    var x482: u32 = undefined;
    var x483: u32 = undefined;
    mulxU32(&x482, &x483, x448, 0xffffffff);
    var x484: u32 = undefined;
    var x485: u32 = undefined;
    mulxU32(&x484, &x485, x448, 0xffffffff);
    var x486: u32 = undefined;
    var x487: u32 = undefined;
    mulxU32(&x486, &x487, x448, 0xfffffffe);
    var x488: u32 = undefined;
    var x489: u32 = undefined;
    mulxU32(&x488, &x489, x448, 0xffffffff);
    var x490: u32 = undefined;
    var x491: u32 = undefined;
    mulxU32(&x490, &x491, x448, 0xffffffff);
    var x492: u32 = undefined;
    var x493: u1 = undefined;
    addcarryxU32(&x492, &x493, 0x0, x489, x486);
    var x494: u32 = undefined;
    var x495: u1 = undefined;
    addcarryxU32(&x494, &x495, x493, x487, x484);
    var x496: u32 = undefined;
    var x497: u1 = undefined;
    addcarryxU32(&x496, &x497, x495, x485, x482);
    var x498: u32 = undefined;
    var x499: u1 = undefined;
    addcarryxU32(&x498, &x499, x497, x483, x480);
    var x500: u32 = undefined;
    var x501: u1 = undefined;
    addcarryxU32(&x500, &x501, x499, x481, x478);
    var x502: u32 = undefined;
    var x503: u1 = undefined;
    addcarryxU32(&x502, &x503, x501, x479, x476);
    var x504: u32 = undefined;
    var x505: u1 = undefined;
    addcarryxU32(&x504, &x505, x503, x477, x474);
    var x506: u32 = undefined;
    var x507: u1 = undefined;
    addcarryxU32(&x506, &x507, x505, x475, x472);
    var x508: u32 = undefined;
    var x509: u1 = undefined;
    addcarryxU32(&x508, &x509, 0x0, x448, x490);
    var x510: u32 = undefined;
    var x511: u1 = undefined;
    addcarryxU32(&x510, &x511, x509, x450, x491);
    var x512: u32 = undefined;
    var x513: u1 = undefined;
    addcarryxU32(&x512, &x513, x511, x452, cast(u32, 0x0));
    var x514: u32 = undefined;
    var x515: u1 = undefined;
    addcarryxU32(&x514, &x515, x513, x454, x488);
    var x516: u32 = undefined;
    var x517: u1 = undefined;
    addcarryxU32(&x516, &x517, x515, x456, x492);
    var x518: u32 = undefined;
    var x519: u1 = undefined;
    addcarryxU32(&x518, &x519, x517, x458, x494);
    var x520: u32 = undefined;
    var x521: u1 = undefined;
    addcarryxU32(&x520, &x521, x519, x460, x496);
    var x522: u32 = undefined;
    var x523: u1 = undefined;
    addcarryxU32(&x522, &x523, x521, x462, x498);
    var x524: u32 = undefined;
    var x525: u1 = undefined;
    addcarryxU32(&x524, &x525, x523, x464, x500);
    var x526: u32 = undefined;
    var x527: u1 = undefined;
    addcarryxU32(&x526, &x527, x525, x466, x502);
    var x528: u32 = undefined;
    var x529: u1 = undefined;
    addcarryxU32(&x528, &x529, x527, x468, x504);
    var x530: u32 = undefined;
    var x531: u1 = undefined;
    addcarryxU32(&x530, &x531, x529, x470, x506);
    var x532: u32 = undefined;
    var x533: u1 = undefined;
    addcarryxU32(&x532, &x533, x531, (cast(u32, x471) + cast(u32, x447)), (cast(u32, x507) + x473));
    var x534: u32 = undefined;
    var x535: u1 = undefined;
    addcarryxU32(&x534, &x535, 0x0, x510, (arg1[7]));
    var x536: u32 = undefined;
    var x537: u1 = undefined;
    addcarryxU32(&x536, &x537, x535, x512, cast(u32, 0x0));
    var x538: u32 = undefined;
    var x539: u1 = undefined;
    addcarryxU32(&x538, &x539, x537, x514, cast(u32, 0x0));
    var x540: u32 = undefined;
    var x541: u1 = undefined;
    addcarryxU32(&x540, &x541, x539, x516, cast(u32, 0x0));
    var x542: u32 = undefined;
    var x543: u1 = undefined;
    addcarryxU32(&x542, &x543, x541, x518, cast(u32, 0x0));
    var x544: u32 = undefined;
    var x545: u1 = undefined;
    addcarryxU32(&x544, &x545, x543, x520, cast(u32, 0x0));
    var x546: u32 = undefined;
    var x547: u1 = undefined;
    addcarryxU32(&x546, &x547, x545, x522, cast(u32, 0x0));
    var x548: u32 = undefined;
    var x549: u1 = undefined;
    addcarryxU32(&x548, &x549, x547, x524, cast(u32, 0x0));
    var x550: u32 = undefined;
    var x551: u1 = undefined;
    addcarryxU32(&x550, &x551, x549, x526, cast(u32, 0x0));
    var x552: u32 = undefined;
    var x553: u1 = undefined;
    addcarryxU32(&x552, &x553, x551, x528, cast(u32, 0x0));
    var x554: u32 = undefined;
    var x555: u1 = undefined;
    addcarryxU32(&x554, &x555, x553, x530, cast(u32, 0x0));
    var x556: u32 = undefined;
    var x557: u1 = undefined;
    addcarryxU32(&x556, &x557, x555, x532, cast(u32, 0x0));
    var x558: u32 = undefined;
    var x559: u32 = undefined;
    mulxU32(&x558, &x559, x534, 0xffffffff);
    var x560: u32 = undefined;
    var x561: u32 = undefined;
    mulxU32(&x560, &x561, x534, 0xffffffff);
    var x562: u32 = undefined;
    var x563: u32 = undefined;
    mulxU32(&x562, &x563, x534, 0xffffffff);
    var x564: u32 = undefined;
    var x565: u32 = undefined;
    mulxU32(&x564, &x565, x534, 0xffffffff);
    var x566: u32 = undefined;
    var x567: u32 = undefined;
    mulxU32(&x566, &x567, x534, 0xffffffff);
    var x568: u32 = undefined;
    var x569: u32 = undefined;
    mulxU32(&x568, &x569, x534, 0xffffffff);
    var x570: u32 = undefined;
    var x571: u32 = undefined;
    mulxU32(&x570, &x571, x534, 0xffffffff);
    var x572: u32 = undefined;
    var x573: u32 = undefined;
    mulxU32(&x572, &x573, x534, 0xfffffffe);
    var x574: u32 = undefined;
    var x575: u32 = undefined;
    mulxU32(&x574, &x575, x534, 0xffffffff);
    var x576: u32 = undefined;
    var x577: u32 = undefined;
    mulxU32(&x576, &x577, x534, 0xffffffff);
    var x578: u32 = undefined;
    var x579: u1 = undefined;
    addcarryxU32(&x578, &x579, 0x0, x575, x572);
    var x580: u32 = undefined;
    var x581: u1 = undefined;
    addcarryxU32(&x580, &x581, x579, x573, x570);
    var x582: u32 = undefined;
    var x583: u1 = undefined;
    addcarryxU32(&x582, &x583, x581, x571, x568);
    var x584: u32 = undefined;
    var x585: u1 = undefined;
    addcarryxU32(&x584, &x585, x583, x569, x566);
    var x586: u32 = undefined;
    var x587: u1 = undefined;
    addcarryxU32(&x586, &x587, x585, x567, x564);
    var x588: u32 = undefined;
    var x589: u1 = undefined;
    addcarryxU32(&x588, &x589, x587, x565, x562);
    var x590: u32 = undefined;
    var x591: u1 = undefined;
    addcarryxU32(&x590, &x591, x589, x563, x560);
    var x592: u32 = undefined;
    var x593: u1 = undefined;
    addcarryxU32(&x592, &x593, x591, x561, x558);
    var x594: u32 = undefined;
    var x595: u1 = undefined;
    addcarryxU32(&x594, &x595, 0x0, x534, x576);
    var x596: u32 = undefined;
    var x597: u1 = undefined;
    addcarryxU32(&x596, &x597, x595, x536, x577);
    var x598: u32 = undefined;
    var x599: u1 = undefined;
    addcarryxU32(&x598, &x599, x597, x538, cast(u32, 0x0));
    var x600: u32 = undefined;
    var x601: u1 = undefined;
    addcarryxU32(&x600, &x601, x599, x540, x574);
    var x602: u32 = undefined;
    var x603: u1 = undefined;
    addcarryxU32(&x602, &x603, x601, x542, x578);
    var x604: u32 = undefined;
    var x605: u1 = undefined;
    addcarryxU32(&x604, &x605, x603, x544, x580);
    var x606: u32 = undefined;
    var x607: u1 = undefined;
    addcarryxU32(&x606, &x607, x605, x546, x582);
    var x608: u32 = undefined;
    var x609: u1 = undefined;
    addcarryxU32(&x608, &x609, x607, x548, x584);
    var x610: u32 = undefined;
    var x611: u1 = undefined;
    addcarryxU32(&x610, &x611, x609, x550, x586);
    var x612: u32 = undefined;
    var x613: u1 = undefined;
    addcarryxU32(&x612, &x613, x611, x552, x588);
    var x614: u32 = undefined;
    var x615: u1 = undefined;
    addcarryxU32(&x614, &x615, x613, x554, x590);
    var x616: u32 = undefined;
    var x617: u1 = undefined;
    addcarryxU32(&x616, &x617, x615, x556, x592);
    var x618: u32 = undefined;
    var x619: u1 = undefined;
    addcarryxU32(&x618, &x619, x617, (cast(u32, x557) + cast(u32, x533)), (cast(u32, x593) + x559));
    var x620: u32 = undefined;
    var x621: u1 = undefined;
    addcarryxU32(&x620, &x621, 0x0, x596, (arg1[8]));
    var x622: u32 = undefined;
    var x623: u1 = undefined;
    addcarryxU32(&x622, &x623, x621, x598, cast(u32, 0x0));
    var x624: u32 = undefined;
    var x625: u1 = undefined;
    addcarryxU32(&x624, &x625, x623, x600, cast(u32, 0x0));
    var x626: u32 = undefined;
    var x627: u1 = undefined;
    addcarryxU32(&x626, &x627, x625, x602, cast(u32, 0x0));
    var x628: u32 = undefined;
    var x629: u1 = undefined;
    addcarryxU32(&x628, &x629, x627, x604, cast(u32, 0x0));
    var x630: u32 = undefined;
    var x631: u1 = undefined;
    addcarryxU32(&x630, &x631, x629, x606, cast(u32, 0x0));
    var x632: u32 = undefined;
    var x633: u1 = undefined;
    addcarryxU32(&x632, &x633, x631, x608, cast(u32, 0x0));
    var x634: u32 = undefined;
    var x635: u1 = undefined;
    addcarryxU32(&x634, &x635, x633, x610, cast(u32, 0x0));
    var x636: u32 = undefined;
    var x637: u1 = undefined;
    addcarryxU32(&x636, &x637, x635, x612, cast(u32, 0x0));
    var x638: u32 = undefined;
    var x639: u1 = undefined;
    addcarryxU32(&x638, &x639, x637, x614, cast(u32, 0x0));
    var x640: u32 = undefined;
    var x641: u1 = undefined;
    addcarryxU32(&x640, &x641, x639, x616, cast(u32, 0x0));
    var x642: u32 = undefined;
    var x643: u1 = undefined;
    addcarryxU32(&x642, &x643, x641, x618, cast(u32, 0x0));
    var x644: u32 = undefined;
    var x645: u32 = undefined;
    mulxU32(&x644, &x645, x620, 0xffffffff);
    var x646: u32 = undefined;
    var x647: u32 = undefined;
    mulxU32(&x646, &x647, x620, 0xffffffff);
    var x648: u32 = undefined;
    var x649: u32 = undefined;
    mulxU32(&x648, &x649, x620, 0xffffffff);
    var x650: u32 = undefined;
    var x651: u32 = undefined;
    mulxU32(&x650, &x651, x620, 0xffffffff);
    var x652: u32 = undefined;
    var x653: u32 = undefined;
    mulxU32(&x652, &x653, x620, 0xffffffff);
    var x654: u32 = undefined;
    var x655: u32 = undefined;
    mulxU32(&x654, &x655, x620, 0xffffffff);
    var x656: u32 = undefined;
    var x657: u32 = undefined;
    mulxU32(&x656, &x657, x620, 0xffffffff);
    var x658: u32 = undefined;
    var x659: u32 = undefined;
    mulxU32(&x658, &x659, x620, 0xfffffffe);
    var x660: u32 = undefined;
    var x661: u32 = undefined;
    mulxU32(&x660, &x661, x620, 0xffffffff);
    var x662: u32 = undefined;
    var x663: u32 = undefined;
    mulxU32(&x662, &x663, x620, 0xffffffff);
    var x664: u32 = undefined;
    var x665: u1 = undefined;
    addcarryxU32(&x664, &x665, 0x0, x661, x658);
    var x666: u32 = undefined;
    var x667: u1 = undefined;
    addcarryxU32(&x666, &x667, x665, x659, x656);
    var x668: u32 = undefined;
    var x669: u1 = undefined;
    addcarryxU32(&x668, &x669, x667, x657, x654);
    var x670: u32 = undefined;
    var x671: u1 = undefined;
    addcarryxU32(&x670, &x671, x669, x655, x652);
    var x672: u32 = undefined;
    var x673: u1 = undefined;
    addcarryxU32(&x672, &x673, x671, x653, x650);
    var x674: u32 = undefined;
    var x675: u1 = undefined;
    addcarryxU32(&x674, &x675, x673, x651, x648);
    var x676: u32 = undefined;
    var x677: u1 = undefined;
    addcarryxU32(&x676, &x677, x675, x649, x646);
    var x678: u32 = undefined;
    var x679: u1 = undefined;
    addcarryxU32(&x678, &x679, x677, x647, x644);
    var x680: u32 = undefined;
    var x681: u1 = undefined;
    addcarryxU32(&x680, &x681, 0x0, x620, x662);
    var x682: u32 = undefined;
    var x683: u1 = undefined;
    addcarryxU32(&x682, &x683, x681, x622, x663);
    var x684: u32 = undefined;
    var x685: u1 = undefined;
    addcarryxU32(&x684, &x685, x683, x624, cast(u32, 0x0));
    var x686: u32 = undefined;
    var x687: u1 = undefined;
    addcarryxU32(&x686, &x687, x685, x626, x660);
    var x688: u32 = undefined;
    var x689: u1 = undefined;
    addcarryxU32(&x688, &x689, x687, x628, x664);
    var x690: u32 = undefined;
    var x691: u1 = undefined;
    addcarryxU32(&x690, &x691, x689, x630, x666);
    var x692: u32 = undefined;
    var x693: u1 = undefined;
    addcarryxU32(&x692, &x693, x691, x632, x668);
    var x694: u32 = undefined;
    var x695: u1 = undefined;
    addcarryxU32(&x694, &x695, x693, x634, x670);
    var x696: u32 = undefined;
    var x697: u1 = undefined;
    addcarryxU32(&x696, &x697, x695, x636, x672);
    var x698: u32 = undefined;
    var x699: u1 = undefined;
    addcarryxU32(&x698, &x699, x697, x638, x674);
    var x700: u32 = undefined;
    var x701: u1 = undefined;
    addcarryxU32(&x700, &x701, x699, x640, x676);
    var x702: u32 = undefined;
    var x703: u1 = undefined;
    addcarryxU32(&x702, &x703, x701, x642, x678);
    var x704: u32 = undefined;
    var x705: u1 = undefined;
    addcarryxU32(&x704, &x705, x703, (cast(u32, x643) + cast(u32, x619)), (cast(u32, x679) + x645));
    var x706: u32 = undefined;
    var x707: u1 = undefined;
    addcarryxU32(&x706, &x707, 0x0, x682, (arg1[9]));
    var x708: u32 = undefined;
    var x709: u1 = undefined;
    addcarryxU32(&x708, &x709, x707, x684, cast(u32, 0x0));
    var x710: u32 = undefined;
    var x711: u1 = undefined;
    addcarryxU32(&x710, &x711, x709, x686, cast(u32, 0x0));
    var x712: u32 = undefined;
    var x713: u1 = undefined;
    addcarryxU32(&x712, &x713, x711, x688, cast(u32, 0x0));
    var x714: u32 = undefined;
    var x715: u1 = undefined;
    addcarryxU32(&x714, &x715, x713, x690, cast(u32, 0x0));
    var x716: u32 = undefined;
    var x717: u1 = undefined;
    addcarryxU32(&x716, &x717, x715, x692, cast(u32, 0x0));
    var x718: u32 = undefined;
    var x719: u1 = undefined;
    addcarryxU32(&x718, &x719, x717, x694, cast(u32, 0x0));
    var x720: u32 = undefined;
    var x721: u1 = undefined;
    addcarryxU32(&x720, &x721, x719, x696, cast(u32, 0x0));
    var x722: u32 = undefined;
    var x723: u1 = undefined;
    addcarryxU32(&x722, &x723, x721, x698, cast(u32, 0x0));
    var x724: u32 = undefined;
    var x725: u1 = undefined;
    addcarryxU32(&x724, &x725, x723, x700, cast(u32, 0x0));
    var x726: u32 = undefined;
    var x727: u1 = undefined;
    addcarryxU32(&x726, &x727, x725, x702, cast(u32, 0x0));
    var x728: u32 = undefined;
    var x729: u1 = undefined;
    addcarryxU32(&x728, &x729, x727, x704, cast(u32, 0x0));
    var x730: u32 = undefined;
    var x731: u32 = undefined;
    mulxU32(&x730, &x731, x706, 0xffffffff);
    var x732: u32 = undefined;
    var x733: u32 = undefined;
    mulxU32(&x732, &x733, x706, 0xffffffff);
    var x734: u32 = undefined;
    var x735: u32 = undefined;
    mulxU32(&x734, &x735, x706, 0xffffffff);
    var x736: u32 = undefined;
    var x737: u32 = undefined;
    mulxU32(&x736, &x737, x706, 0xffffffff);
    var x738: u32 = undefined;
    var x739: u32 = undefined;
    mulxU32(&x738, &x739, x706, 0xffffffff);
    var x740: u32 = undefined;
    var x741: u32 = undefined;
    mulxU32(&x740, &x741, x706, 0xffffffff);
    var x742: u32 = undefined;
    var x743: u32 = undefined;
    mulxU32(&x742, &x743, x706, 0xffffffff);
    var x744: u32 = undefined;
    var x745: u32 = undefined;
    mulxU32(&x744, &x745, x706, 0xfffffffe);
    var x746: u32 = undefined;
    var x747: u32 = undefined;
    mulxU32(&x746, &x747, x706, 0xffffffff);
    var x748: u32 = undefined;
    var x749: u32 = undefined;
    mulxU32(&x748, &x749, x706, 0xffffffff);
    var x750: u32 = undefined;
    var x751: u1 = undefined;
    addcarryxU32(&x750, &x751, 0x0, x747, x744);
    var x752: u32 = undefined;
    var x753: u1 = undefined;
    addcarryxU32(&x752, &x753, x751, x745, x742);
    var x754: u32 = undefined;
    var x755: u1 = undefined;
    addcarryxU32(&x754, &x755, x753, x743, x740);
    var x756: u32 = undefined;
    var x757: u1 = undefined;
    addcarryxU32(&x756, &x757, x755, x741, x738);
    var x758: u32 = undefined;
    var x759: u1 = undefined;
    addcarryxU32(&x758, &x759, x757, x739, x736);
    var x760: u32 = undefined;
    var x761: u1 = undefined;
    addcarryxU32(&x760, &x761, x759, x737, x734);
    var x762: u32 = undefined;
    var x763: u1 = undefined;
    addcarryxU32(&x762, &x763, x761, x735, x732);
    var x764: u32 = undefined;
    var x765: u1 = undefined;
    addcarryxU32(&x764, &x765, x763, x733, x730);
    var x766: u32 = undefined;
    var x767: u1 = undefined;
    addcarryxU32(&x766, &x767, 0x0, x706, x748);
    var x768: u32 = undefined;
    var x769: u1 = undefined;
    addcarryxU32(&x768, &x769, x767, x708, x749);
    var x770: u32 = undefined;
    var x771: u1 = undefined;
    addcarryxU32(&x770, &x771, x769, x710, cast(u32, 0x0));
    var x772: u32 = undefined;
    var x773: u1 = undefined;
    addcarryxU32(&x772, &x773, x771, x712, x746);
    var x774: u32 = undefined;
    var x775: u1 = undefined;
    addcarryxU32(&x774, &x775, x773, x714, x750);
    var x776: u32 = undefined;
    var x777: u1 = undefined;
    addcarryxU32(&x776, &x777, x775, x716, x752);
    var x778: u32 = undefined;
    var x779: u1 = undefined;
    addcarryxU32(&x778, &x779, x777, x718, x754);
    var x780: u32 = undefined;
    var x781: u1 = undefined;
    addcarryxU32(&x780, &x781, x779, x720, x756);
    var x782: u32 = undefined;
    var x783: u1 = undefined;
    addcarryxU32(&x782, &x783, x781, x722, x758);
    var x784: u32 = undefined;
    var x785: u1 = undefined;
    addcarryxU32(&x784, &x785, x783, x724, x760);
    var x786: u32 = undefined;
    var x787: u1 = undefined;
    addcarryxU32(&x786, &x787, x785, x726, x762);
    var x788: u32 = undefined;
    var x789: u1 = undefined;
    addcarryxU32(&x788, &x789, x787, x728, x764);
    var x790: u32 = undefined;
    var x791: u1 = undefined;
    addcarryxU32(&x790, &x791, x789, (cast(u32, x729) + cast(u32, x705)), (cast(u32, x765) + x731));
    var x792: u32 = undefined;
    var x793: u1 = undefined;
    addcarryxU32(&x792, &x793, 0x0, x768, (arg1[10]));
    var x794: u32 = undefined;
    var x795: u1 = undefined;
    addcarryxU32(&x794, &x795, x793, x770, cast(u32, 0x0));
    var x796: u32 = undefined;
    var x797: u1 = undefined;
    addcarryxU32(&x796, &x797, x795, x772, cast(u32, 0x0));
    var x798: u32 = undefined;
    var x799: u1 = undefined;
    addcarryxU32(&x798, &x799, x797, x774, cast(u32, 0x0));
    var x800: u32 = undefined;
    var x801: u1 = undefined;
    addcarryxU32(&x800, &x801, x799, x776, cast(u32, 0x0));
    var x802: u32 = undefined;
    var x803: u1 = undefined;
    addcarryxU32(&x802, &x803, x801, x778, cast(u32, 0x0));
    var x804: u32 = undefined;
    var x805: u1 = undefined;
    addcarryxU32(&x804, &x805, x803, x780, cast(u32, 0x0));
    var x806: u32 = undefined;
    var x807: u1 = undefined;
    addcarryxU32(&x806, &x807, x805, x782, cast(u32, 0x0));
    var x808: u32 = undefined;
    var x809: u1 = undefined;
    addcarryxU32(&x808, &x809, x807, x784, cast(u32, 0x0));
    var x810: u32 = undefined;
    var x811: u1 = undefined;
    addcarryxU32(&x810, &x811, x809, x786, cast(u32, 0x0));
    var x812: u32 = undefined;
    var x813: u1 = undefined;
    addcarryxU32(&x812, &x813, x811, x788, cast(u32, 0x0));
    var x814: u32 = undefined;
    var x815: u1 = undefined;
    addcarryxU32(&x814, &x815, x813, x790, cast(u32, 0x0));
    var x816: u32 = undefined;
    var x817: u32 = undefined;
    mulxU32(&x816, &x817, x792, 0xffffffff);
    var x818: u32 = undefined;
    var x819: u32 = undefined;
    mulxU32(&x818, &x819, x792, 0xffffffff);
    var x820: u32 = undefined;
    var x821: u32 = undefined;
    mulxU32(&x820, &x821, x792, 0xffffffff);
    var x822: u32 = undefined;
    var x823: u32 = undefined;
    mulxU32(&x822, &x823, x792, 0xffffffff);
    var x824: u32 = undefined;
    var x825: u32 = undefined;
    mulxU32(&x824, &x825, x792, 0xffffffff);
    var x826: u32 = undefined;
    var x827: u32 = undefined;
    mulxU32(&x826, &x827, x792, 0xffffffff);
    var x828: u32 = undefined;
    var x829: u32 = undefined;
    mulxU32(&x828, &x829, x792, 0xffffffff);
    var x830: u32 = undefined;
    var x831: u32 = undefined;
    mulxU32(&x830, &x831, x792, 0xfffffffe);
    var x832: u32 = undefined;
    var x833: u32 = undefined;
    mulxU32(&x832, &x833, x792, 0xffffffff);
    var x834: u32 = undefined;
    var x835: u32 = undefined;
    mulxU32(&x834, &x835, x792, 0xffffffff);
    var x836: u32 = undefined;
    var x837: u1 = undefined;
    addcarryxU32(&x836, &x837, 0x0, x833, x830);
    var x838: u32 = undefined;
    var x839: u1 = undefined;
    addcarryxU32(&x838, &x839, x837, x831, x828);
    var x840: u32 = undefined;
    var x841: u1 = undefined;
    addcarryxU32(&x840, &x841, x839, x829, x826);
    var x842: u32 = undefined;
    var x843: u1 = undefined;
    addcarryxU32(&x842, &x843, x841, x827, x824);
    var x844: u32 = undefined;
    var x845: u1 = undefined;
    addcarryxU32(&x844, &x845, x843, x825, x822);
    var x846: u32 = undefined;
    var x847: u1 = undefined;
    addcarryxU32(&x846, &x847, x845, x823, x820);
    var x848: u32 = undefined;
    var x849: u1 = undefined;
    addcarryxU32(&x848, &x849, x847, x821, x818);
    var x850: u32 = undefined;
    var x851: u1 = undefined;
    addcarryxU32(&x850, &x851, x849, x819, x816);
    var x852: u32 = undefined;
    var x853: u1 = undefined;
    addcarryxU32(&x852, &x853, 0x0, x792, x834);
    var x854: u32 = undefined;
    var x855: u1 = undefined;
    addcarryxU32(&x854, &x855, x853, x794, x835);
    var x856: u32 = undefined;
    var x857: u1 = undefined;
    addcarryxU32(&x856, &x857, x855, x796, cast(u32, 0x0));
    var x858: u32 = undefined;
    var x859: u1 = undefined;
    addcarryxU32(&x858, &x859, x857, x798, x832);
    var x860: u32 = undefined;
    var x861: u1 = undefined;
    addcarryxU32(&x860, &x861, x859, x800, x836);
    var x862: u32 = undefined;
    var x863: u1 = undefined;
    addcarryxU32(&x862, &x863, x861, x802, x838);
    var x864: u32 = undefined;
    var x865: u1 = undefined;
    addcarryxU32(&x864, &x865, x863, x804, x840);
    var x866: u32 = undefined;
    var x867: u1 = undefined;
    addcarryxU32(&x866, &x867, x865, x806, x842);
    var x868: u32 = undefined;
    var x869: u1 = undefined;
    addcarryxU32(&x868, &x869, x867, x808, x844);
    var x870: u32 = undefined;
    var x871: u1 = undefined;
    addcarryxU32(&x870, &x871, x869, x810, x846);
    var x872: u32 = undefined;
    var x873: u1 = undefined;
    addcarryxU32(&x872, &x873, x871, x812, x848);
    var x874: u32 = undefined;
    var x875: u1 = undefined;
    addcarryxU32(&x874, &x875, x873, x814, x850);
    var x876: u32 = undefined;
    var x877: u1 = undefined;
    addcarryxU32(&x876, &x877, x875, (cast(u32, x815) + cast(u32, x791)), (cast(u32, x851) + x817));
    var x878: u32 = undefined;
    var x879: u1 = undefined;
    addcarryxU32(&x878, &x879, 0x0, x854, (arg1[11]));
    var x880: u32 = undefined;
    var x881: u1 = undefined;
    addcarryxU32(&x880, &x881, x879, x856, cast(u32, 0x0));
    var x882: u32 = undefined;
    var x883: u1 = undefined;
    addcarryxU32(&x882, &x883, x881, x858, cast(u32, 0x0));
    var x884: u32 = undefined;
    var x885: u1 = undefined;
    addcarryxU32(&x884, &x885, x883, x860, cast(u32, 0x0));
    var x886: u32 = undefined;
    var x887: u1 = undefined;
    addcarryxU32(&x886, &x887, x885, x862, cast(u32, 0x0));
    var x888: u32 = undefined;
    var x889: u1 = undefined;
    addcarryxU32(&x888, &x889, x887, x864, cast(u32, 0x0));
    var x890: u32 = undefined;
    var x891: u1 = undefined;
    addcarryxU32(&x890, &x891, x889, x866, cast(u32, 0x0));
    var x892: u32 = undefined;
    var x893: u1 = undefined;
    addcarryxU32(&x892, &x893, x891, x868, cast(u32, 0x0));
    var x894: u32 = undefined;
    var x895: u1 = undefined;
    addcarryxU32(&x894, &x895, x893, x870, cast(u32, 0x0));
    var x896: u32 = undefined;
    var x897: u1 = undefined;
    addcarryxU32(&x896, &x897, x895, x872, cast(u32, 0x0));
    var x898: u32 = undefined;
    var x899: u1 = undefined;
    addcarryxU32(&x898, &x899, x897, x874, cast(u32, 0x0));
    var x900: u32 = undefined;
    var x901: u1 = undefined;
    addcarryxU32(&x900, &x901, x899, x876, cast(u32, 0x0));
    var x902: u32 = undefined;
    var x903: u32 = undefined;
    mulxU32(&x902, &x903, x878, 0xffffffff);
    var x904: u32 = undefined;
    var x905: u32 = undefined;
    mulxU32(&x904, &x905, x878, 0xffffffff);
    var x906: u32 = undefined;
    var x907: u32 = undefined;
    mulxU32(&x906, &x907, x878, 0xffffffff);
    var x908: u32 = undefined;
    var x909: u32 = undefined;
    mulxU32(&x908, &x909, x878, 0xffffffff);
    var x910: u32 = undefined;
    var x911: u32 = undefined;
    mulxU32(&x910, &x911, x878, 0xffffffff);
    var x912: u32 = undefined;
    var x913: u32 = undefined;
    mulxU32(&x912, &x913, x878, 0xffffffff);
    var x914: u32 = undefined;
    var x915: u32 = undefined;
    mulxU32(&x914, &x915, x878, 0xffffffff);
    var x916: u32 = undefined;
    var x917: u32 = undefined;
    mulxU32(&x916, &x917, x878, 0xfffffffe);
    var x918: u32 = undefined;
    var x919: u32 = undefined;
    mulxU32(&x918, &x919, x878, 0xffffffff);
    var x920: u32 = undefined;
    var x921: u32 = undefined;
    mulxU32(&x920, &x921, x878, 0xffffffff);
    var x922: u32 = undefined;
    var x923: u1 = undefined;
    addcarryxU32(&x922, &x923, 0x0, x919, x916);
    var x924: u32 = undefined;
    var x925: u1 = undefined;
    addcarryxU32(&x924, &x925, x923, x917, x914);
    var x926: u32 = undefined;
    var x927: u1 = undefined;
    addcarryxU32(&x926, &x927, x925, x915, x912);
    var x928: u32 = undefined;
    var x929: u1 = undefined;
    addcarryxU32(&x928, &x929, x927, x913, x910);
    var x930: u32 = undefined;
    var x931: u1 = undefined;
    addcarryxU32(&x930, &x931, x929, x911, x908);
    var x932: u32 = undefined;
    var x933: u1 = undefined;
    addcarryxU32(&x932, &x933, x931, x909, x906);
    var x934: u32 = undefined;
    var x935: u1 = undefined;
    addcarryxU32(&x934, &x935, x933, x907, x904);
    var x936: u32 = undefined;
    var x937: u1 = undefined;
    addcarryxU32(&x936, &x937, x935, x905, x902);
    var x938: u32 = undefined;
    var x939: u1 = undefined;
    addcarryxU32(&x938, &x939, 0x0, x878, x920);
    var x940: u32 = undefined;
    var x941: u1 = undefined;
    addcarryxU32(&x940, &x941, x939, x880, x921);
    var x942: u32 = undefined;
    var x943: u1 = undefined;
    addcarryxU32(&x942, &x943, x941, x882, cast(u32, 0x0));
    var x944: u32 = undefined;
    var x945: u1 = undefined;
    addcarryxU32(&x944, &x945, x943, x884, x918);
    var x946: u32 = undefined;
    var x947: u1 = undefined;
    addcarryxU32(&x946, &x947, x945, x886, x922);
    var x948: u32 = undefined;
    var x949: u1 = undefined;
    addcarryxU32(&x948, &x949, x947, x888, x924);
    var x950: u32 = undefined;
    var x951: u1 = undefined;
    addcarryxU32(&x950, &x951, x949, x890, x926);
    var x952: u32 = undefined;
    var x953: u1 = undefined;
    addcarryxU32(&x952, &x953, x951, x892, x928);
    var x954: u32 = undefined;
    var x955: u1 = undefined;
    addcarryxU32(&x954, &x955, x953, x894, x930);
    var x956: u32 = undefined;
    var x957: u1 = undefined;
    addcarryxU32(&x956, &x957, x955, x896, x932);
    var x958: u32 = undefined;
    var x959: u1 = undefined;
    addcarryxU32(&x958, &x959, x957, x898, x934);
    var x960: u32 = undefined;
    var x961: u1 = undefined;
    addcarryxU32(&x960, &x961, x959, x900, x936);
    var x962: u32 = undefined;
    var x963: u1 = undefined;
    addcarryxU32(&x962, &x963, x961, (cast(u32, x901) + cast(u32, x877)), (cast(u32, x937) + x903));
    var x964: u32 = undefined;
    var x965: u1 = undefined;
    subborrowxU32(&x964, &x965, 0x0, x940, 0xffffffff);
    var x966: u32 = undefined;
    var x967: u1 = undefined;
    subborrowxU32(&x966, &x967, x965, x942, cast(u32, 0x0));
    var x968: u32 = undefined;
    var x969: u1 = undefined;
    subborrowxU32(&x968, &x969, x967, x944, cast(u32, 0x0));
    var x970: u32 = undefined;
    var x971: u1 = undefined;
    subborrowxU32(&x970, &x971, x969, x946, 0xffffffff);
    var x972: u32 = undefined;
    var x973: u1 = undefined;
    subborrowxU32(&x972, &x973, x971, x948, 0xfffffffe);
    var x974: u32 = undefined;
    var x975: u1 = undefined;
    subborrowxU32(&x974, &x975, x973, x950, 0xffffffff);
    var x976: u32 = undefined;
    var x977: u1 = undefined;
    subborrowxU32(&x976, &x977, x975, x952, 0xffffffff);
    var x978: u32 = undefined;
    var x979: u1 = undefined;
    subborrowxU32(&x978, &x979, x977, x954, 0xffffffff);
    var x980: u32 = undefined;
    var x981: u1 = undefined;
    subborrowxU32(&x980, &x981, x979, x956, 0xffffffff);
    var x982: u32 = undefined;
    var x983: u1 = undefined;
    subborrowxU32(&x982, &x983, x981, x958, 0xffffffff);
    var x984: u32 = undefined;
    var x985: u1 = undefined;
    subborrowxU32(&x984, &x985, x983, x960, 0xffffffff);
    var x986: u32 = undefined;
    var x987: u1 = undefined;
    subborrowxU32(&x986, &x987, x985, x962, 0xffffffff);
    var x988: u32 = undefined;
    var x989: u1 = undefined;
    subborrowxU32(&x988, &x989, x987, cast(u32, x963), cast(u32, 0x0));
    var x990: u32 = undefined;
    cmovznzU32(&x990, x989, x964, x940);
    var x991: u32 = undefined;
    cmovznzU32(&x991, x989, x966, x942);
    var x992: u32 = undefined;
    cmovznzU32(&x992, x989, x968, x944);
    var x993: u32 = undefined;
    cmovznzU32(&x993, x989, x970, x946);
    var x994: u32 = undefined;
    cmovznzU32(&x994, x989, x972, x948);
    var x995: u32 = undefined;
    cmovznzU32(&x995, x989, x974, x950);
    var x996: u32 = undefined;
    cmovznzU32(&x996, x989, x976, x952);
    var x997: u32 = undefined;
    cmovznzU32(&x997, x989, x978, x954);
    var x998: u32 = undefined;
    cmovznzU32(&x998, x989, x980, x956);
    var x999: u32 = undefined;
    cmovznzU32(&x999, x989, x982, x958);
    var x1000: u32 = undefined;
    cmovznzU32(&x1000, x989, x984, x960);
    var x1001: u32 = undefined;
    cmovznzU32(&x1001, x989, x986, x962);
    out1[0] = x990;
    out1[1] = x991;
    out1[2] = x992;
    out1[3] = x993;
    out1[4] = x994;
    out1[5] = x995;
    out1[6] = x996;
    out1[7] = x997;
    out1[8] = x998;
    out1[9] = x999;
    out1[10] = x1000;
    out1[11] = x1001;
}

/// The function toMontgomery translates a field element into the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = eval arg1 mod m
///   0 ≤ eval out1 < m
///
pub fn toMontgomery(out1: *MontgomeryDomainFieldElement, arg1: NonMontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[1]);
    const x2 = (arg1[2]);
    const x3 = (arg1[3]);
    const x4 = (arg1[4]);
    const x5 = (arg1[5]);
    const x6 = (arg1[6]);
    const x7 = (arg1[7]);
    const x8 = (arg1[8]);
    const x9 = (arg1[9]);
    const x10 = (arg1[10]);
    const x11 = (arg1[11]);
    const x12 = (arg1[0]);
    var x13: u32 = undefined;
    var x14: u32 = undefined;
    mulxU32(&x13, &x14, x12, 0x2);
    var x15: u32 = undefined;
    var x16: u32 = undefined;
    mulxU32(&x15, &x16, x12, 0xfffffffe);
    var x17: u32 = undefined;
    var x18: u32 = undefined;
    mulxU32(&x17, &x18, x12, 0x2);
    var x19: u32 = undefined;
    var x20: u32 = undefined;
    mulxU32(&x19, &x20, x12, 0xfffffffe);
    var x21: u32 = undefined;
    var x22: u1 = undefined;
    addcarryxU32(&x21, &x22, 0x0, cast(u32, cast(u1, x14)), x12);
    var x23: u32 = undefined;
    var x24: u32 = undefined;
    mulxU32(&x23, &x24, x12, 0xffffffff);
    var x25: u32 = undefined;
    var x26: u32 = undefined;
    mulxU32(&x25, &x26, x12, 0xffffffff);
    var x27: u32 = undefined;
    var x28: u32 = undefined;
    mulxU32(&x27, &x28, x12, 0xffffffff);
    var x29: u32 = undefined;
    var x30: u32 = undefined;
    mulxU32(&x29, &x30, x12, 0xffffffff);
    var x31: u32 = undefined;
    var x32: u32 = undefined;
    mulxU32(&x31, &x32, x12, 0xffffffff);
    var x33: u32 = undefined;
    var x34: u32 = undefined;
    mulxU32(&x33, &x34, x12, 0xffffffff);
    var x35: u32 = undefined;
    var x36: u32 = undefined;
    mulxU32(&x35, &x36, x12, 0xffffffff);
    var x37: u32 = undefined;
    var x38: u32 = undefined;
    mulxU32(&x37, &x38, x12, 0xfffffffe);
    var x39: u32 = undefined;
    var x40: u32 = undefined;
    mulxU32(&x39, &x40, x12, 0xffffffff);
    var x41: u32 = undefined;
    var x42: u32 = undefined;
    mulxU32(&x41, &x42, x12, 0xffffffff);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    addcarryxU32(&x43, &x44, 0x0, x40, x37);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    addcarryxU32(&x45, &x46, x44, x38, x35);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    addcarryxU32(&x47, &x48, x46, x36, x33);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    addcarryxU32(&x49, &x50, x48, x34, x31);
    var x51: u32 = undefined;
    var x52: u1 = undefined;
    addcarryxU32(&x51, &x52, x50, x32, x29);
    var x53: u32 = undefined;
    var x54: u1 = undefined;
    addcarryxU32(&x53, &x54, x52, x30, x27);
    var x55: u32 = undefined;
    var x56: u1 = undefined;
    addcarryxU32(&x55, &x56, x54, x28, x25);
    var x57: u32 = undefined;
    var x58: u1 = undefined;
    addcarryxU32(&x57, &x58, x56, x26, x23);
    var x59: u32 = undefined;
    var x60: u1 = undefined;
    addcarryxU32(&x59, &x60, 0x0, x12, x41);
    var x61: u32 = undefined;
    var x62: u1 = undefined;
    addcarryxU32(&x61, &x62, x60, x19, x42);
    var x63: u32 = undefined;
    var x64: u1 = undefined;
    addcarryxU32(&x63, &x64, 0x0, x17, x39);
    var x65: u32 = undefined;
    var x66: u1 = undefined;
    addcarryxU32(&x65, &x66, x64, cast(u32, cast(u1, x18)), x43);
    var x67: u32 = undefined;
    var x68: u1 = undefined;
    addcarryxU32(&x67, &x68, x66, x15, x45);
    var x69: u32 = undefined;
    var x70: u1 = undefined;
    addcarryxU32(&x69, &x70, x68, x16, x47);
    var x71: u32 = undefined;
    var x72: u1 = undefined;
    addcarryxU32(&x71, &x72, x70, x13, x49);
    var x73: u32 = undefined;
    var x74: u1 = undefined;
    addcarryxU32(&x73, &x74, x72, x21, x51);
    var x75: u32 = undefined;
    var x76: u1 = undefined;
    addcarryxU32(&x75, &x76, x74, cast(u32, x22), x53);
    var x77: u32 = undefined;
    var x78: u1 = undefined;
    addcarryxU32(&x77, &x78, x76, cast(u32, 0x0), x55);
    var x79: u32 = undefined;
    var x80: u1 = undefined;
    addcarryxU32(&x79, &x80, x78, cast(u32, 0x0), x57);
    var x81: u32 = undefined;
    var x82: u1 = undefined;
    addcarryxU32(&x81, &x82, x80, cast(u32, 0x0), (cast(u32, x58) + x24));
    var x83: u32 = undefined;
    var x84: u32 = undefined;
    mulxU32(&x83, &x84, x1, 0x2);
    var x85: u32 = undefined;
    var x86: u32 = undefined;
    mulxU32(&x85, &x86, x1, 0xfffffffe);
    var x87: u32 = undefined;
    var x88: u32 = undefined;
    mulxU32(&x87, &x88, x1, 0x2);
    var x89: u32 = undefined;
    var x90: u32 = undefined;
    mulxU32(&x89, &x90, x1, 0xfffffffe);
    var x91: u32 = undefined;
    var x92: u1 = undefined;
    addcarryxU32(&x91, &x92, 0x0, cast(u32, cast(u1, x84)), x1);
    var x93: u32 = undefined;
    var x94: u1 = undefined;
    addcarryxU32(&x93, &x94, 0x0, x61, x1);
    var x95: u32 = undefined;
    var x96: u1 = undefined;
    addcarryxU32(&x95, &x96, x94, (cast(u32, x62) + x20), x89);
    var x97: u32 = undefined;
    var x98: u1 = undefined;
    addcarryxU32(&x97, &x98, x96, x63, x90);
    var x99: u32 = undefined;
    var x100: u1 = undefined;
    addcarryxU32(&x99, &x100, x98, x65, x87);
    var x101: u32 = undefined;
    var x102: u1 = undefined;
    addcarryxU32(&x101, &x102, x100, x67, cast(u32, cast(u1, x88)));
    var x103: u32 = undefined;
    var x104: u1 = undefined;
    addcarryxU32(&x103, &x104, x102, x69, x85);
    var x105: u32 = undefined;
    var x106: u1 = undefined;
    addcarryxU32(&x105, &x106, x104, x71, x86);
    var x107: u32 = undefined;
    var x108: u1 = undefined;
    addcarryxU32(&x107, &x108, x106, x73, x83);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    addcarryxU32(&x109, &x110, x108, x75, x91);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    addcarryxU32(&x111, &x112, x110, x77, cast(u32, x92));
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    addcarryxU32(&x113, &x114, x112, x79, cast(u32, 0x0));
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    addcarryxU32(&x115, &x116, x114, x81, cast(u32, 0x0));
    var x117: u32 = undefined;
    var x118: u32 = undefined;
    mulxU32(&x117, &x118, x93, 0xffffffff);
    var x119: u32 = undefined;
    var x120: u32 = undefined;
    mulxU32(&x119, &x120, x93, 0xffffffff);
    var x121: u32 = undefined;
    var x122: u32 = undefined;
    mulxU32(&x121, &x122, x93, 0xffffffff);
    var x123: u32 = undefined;
    var x124: u32 = undefined;
    mulxU32(&x123, &x124, x93, 0xffffffff);
    var x125: u32 = undefined;
    var x126: u32 = undefined;
    mulxU32(&x125, &x126, x93, 0xffffffff);
    var x127: u32 = undefined;
    var x128: u32 = undefined;
    mulxU32(&x127, &x128, x93, 0xffffffff);
    var x129: u32 = undefined;
    var x130: u32 = undefined;
    mulxU32(&x129, &x130, x93, 0xffffffff);
    var x131: u32 = undefined;
    var x132: u32 = undefined;
    mulxU32(&x131, &x132, x93, 0xfffffffe);
    var x133: u32 = undefined;
    var x134: u32 = undefined;
    mulxU32(&x133, &x134, x93, 0xffffffff);
    var x135: u32 = undefined;
    var x136: u32 = undefined;
    mulxU32(&x135, &x136, x93, 0xffffffff);
    var x137: u32 = undefined;
    var x138: u1 = undefined;
    addcarryxU32(&x137, &x138, 0x0, x134, x131);
    var x139: u32 = undefined;
    var x140: u1 = undefined;
    addcarryxU32(&x139, &x140, x138, x132, x129);
    var x141: u32 = undefined;
    var x142: u1 = undefined;
    addcarryxU32(&x141, &x142, x140, x130, x127);
    var x143: u32 = undefined;
    var x144: u1 = undefined;
    addcarryxU32(&x143, &x144, x142, x128, x125);
    var x145: u32 = undefined;
    var x146: u1 = undefined;
    addcarryxU32(&x145, &x146, x144, x126, x123);
    var x147: u32 = undefined;
    var x148: u1 = undefined;
    addcarryxU32(&x147, &x148, x146, x124, x121);
    var x149: u32 = undefined;
    var x150: u1 = undefined;
    addcarryxU32(&x149, &x150, x148, x122, x119);
    var x151: u32 = undefined;
    var x152: u1 = undefined;
    addcarryxU32(&x151, &x152, x150, x120, x117);
    var x153: u32 = undefined;
    var x154: u1 = undefined;
    addcarryxU32(&x153, &x154, 0x0, x93, x135);
    var x155: u32 = undefined;
    var x156: u1 = undefined;
    addcarryxU32(&x155, &x156, x154, x95, x136);
    var x157: u32 = undefined;
    var x158: u1 = undefined;
    addcarryxU32(&x157, &x158, x156, x97, cast(u32, 0x0));
    var x159: u32 = undefined;
    var x160: u1 = undefined;
    addcarryxU32(&x159, &x160, x158, x99, x133);
    var x161: u32 = undefined;
    var x162: u1 = undefined;
    addcarryxU32(&x161, &x162, x160, x101, x137);
    var x163: u32 = undefined;
    var x164: u1 = undefined;
    addcarryxU32(&x163, &x164, x162, x103, x139);
    var x165: u32 = undefined;
    var x166: u1 = undefined;
    addcarryxU32(&x165, &x166, x164, x105, x141);
    var x167: u32 = undefined;
    var x168: u1 = undefined;
    addcarryxU32(&x167, &x168, x166, x107, x143);
    var x169: u32 = undefined;
    var x170: u1 = undefined;
    addcarryxU32(&x169, &x170, x168, x109, x145);
    var x171: u32 = undefined;
    var x172: u1 = undefined;
    addcarryxU32(&x171, &x172, x170, x111, x147);
    var x173: u32 = undefined;
    var x174: u1 = undefined;
    addcarryxU32(&x173, &x174, x172, x113, x149);
    var x175: u32 = undefined;
    var x176: u1 = undefined;
    addcarryxU32(&x175, &x176, x174, x115, x151);
    var x177: u32 = undefined;
    var x178: u1 = undefined;
    addcarryxU32(&x177, &x178, x176, (cast(u32, x116) + cast(u32, x82)), (cast(u32, x152) + x118));
    var x179: u32 = undefined;
    var x180: u32 = undefined;
    mulxU32(&x179, &x180, x2, 0x2);
    var x181: u32 = undefined;
    var x182: u32 = undefined;
    mulxU32(&x181, &x182, x2, 0xfffffffe);
    var x183: u32 = undefined;
    var x184: u32 = undefined;
    mulxU32(&x183, &x184, x2, 0x2);
    var x185: u32 = undefined;
    var x186: u32 = undefined;
    mulxU32(&x185, &x186, x2, 0xfffffffe);
    var x187: u32 = undefined;
    var x188: u1 = undefined;
    addcarryxU32(&x187, &x188, 0x0, cast(u32, cast(u1, x180)), x2);
    var x189: u32 = undefined;
    var x190: u1 = undefined;
    addcarryxU32(&x189, &x190, 0x0, x155, x2);
    var x191: u32 = undefined;
    var x192: u1 = undefined;
    addcarryxU32(&x191, &x192, x190, x157, x185);
    var x193: u32 = undefined;
    var x194: u1 = undefined;
    addcarryxU32(&x193, &x194, x192, x159, x186);
    var x195: u32 = undefined;
    var x196: u1 = undefined;
    addcarryxU32(&x195, &x196, x194, x161, x183);
    var x197: u32 = undefined;
    var x198: u1 = undefined;
    addcarryxU32(&x197, &x198, x196, x163, cast(u32, cast(u1, x184)));
    var x199: u32 = undefined;
    var x200: u1 = undefined;
    addcarryxU32(&x199, &x200, x198, x165, x181);
    var x201: u32 = undefined;
    var x202: u1 = undefined;
    addcarryxU32(&x201, &x202, x200, x167, x182);
    var x203: u32 = undefined;
    var x204: u1 = undefined;
    addcarryxU32(&x203, &x204, x202, x169, x179);
    var x205: u32 = undefined;
    var x206: u1 = undefined;
    addcarryxU32(&x205, &x206, x204, x171, x187);
    var x207: u32 = undefined;
    var x208: u1 = undefined;
    addcarryxU32(&x207, &x208, x206, x173, cast(u32, x188));
    var x209: u32 = undefined;
    var x210: u1 = undefined;
    addcarryxU32(&x209, &x210, x208, x175, cast(u32, 0x0));
    var x211: u32 = undefined;
    var x212: u1 = undefined;
    addcarryxU32(&x211, &x212, x210, x177, cast(u32, 0x0));
    var x213: u32 = undefined;
    var x214: u32 = undefined;
    mulxU32(&x213, &x214, x189, 0xffffffff);
    var x215: u32 = undefined;
    var x216: u32 = undefined;
    mulxU32(&x215, &x216, x189, 0xffffffff);
    var x217: u32 = undefined;
    var x218: u32 = undefined;
    mulxU32(&x217, &x218, x189, 0xffffffff);
    var x219: u32 = undefined;
    var x220: u32 = undefined;
    mulxU32(&x219, &x220, x189, 0xffffffff);
    var x221: u32 = undefined;
    var x222: u32 = undefined;
    mulxU32(&x221, &x222, x189, 0xffffffff);
    var x223: u32 = undefined;
    var x224: u32 = undefined;
    mulxU32(&x223, &x224, x189, 0xffffffff);
    var x225: u32 = undefined;
    var x226: u32 = undefined;
    mulxU32(&x225, &x226, x189, 0xffffffff);
    var x227: u32 = undefined;
    var x228: u32 = undefined;
    mulxU32(&x227, &x228, x189, 0xfffffffe);
    var x229: u32 = undefined;
    var x230: u32 = undefined;
    mulxU32(&x229, &x230, x189, 0xffffffff);
    var x231: u32 = undefined;
    var x232: u32 = undefined;
    mulxU32(&x231, &x232, x189, 0xffffffff);
    var x233: u32 = undefined;
    var x234: u1 = undefined;
    addcarryxU32(&x233, &x234, 0x0, x230, x227);
    var x235: u32 = undefined;
    var x236: u1 = undefined;
    addcarryxU32(&x235, &x236, x234, x228, x225);
    var x237: u32 = undefined;
    var x238: u1 = undefined;
    addcarryxU32(&x237, &x238, x236, x226, x223);
    var x239: u32 = undefined;
    var x240: u1 = undefined;
    addcarryxU32(&x239, &x240, x238, x224, x221);
    var x241: u32 = undefined;
    var x242: u1 = undefined;
    addcarryxU32(&x241, &x242, x240, x222, x219);
    var x243: u32 = undefined;
    var x244: u1 = undefined;
    addcarryxU32(&x243, &x244, x242, x220, x217);
    var x245: u32 = undefined;
    var x246: u1 = undefined;
    addcarryxU32(&x245, &x246, x244, x218, x215);
    var x247: u32 = undefined;
    var x248: u1 = undefined;
    addcarryxU32(&x247, &x248, x246, x216, x213);
    var x249: u32 = undefined;
    var x250: u1 = undefined;
    addcarryxU32(&x249, &x250, 0x0, x189, x231);
    var x251: u32 = undefined;
    var x252: u1 = undefined;
    addcarryxU32(&x251, &x252, x250, x191, x232);
    var x253: u32 = undefined;
    var x254: u1 = undefined;
    addcarryxU32(&x253, &x254, x252, x193, cast(u32, 0x0));
    var x255: u32 = undefined;
    var x256: u1 = undefined;
    addcarryxU32(&x255, &x256, x254, x195, x229);
    var x257: u32 = undefined;
    var x258: u1 = undefined;
    addcarryxU32(&x257, &x258, x256, x197, x233);
    var x259: u32 = undefined;
    var x260: u1 = undefined;
    addcarryxU32(&x259, &x260, x258, x199, x235);
    var x261: u32 = undefined;
    var x262: u1 = undefined;
    addcarryxU32(&x261, &x262, x260, x201, x237);
    var x263: u32 = undefined;
    var x264: u1 = undefined;
    addcarryxU32(&x263, &x264, x262, x203, x239);
    var x265: u32 = undefined;
    var x266: u1 = undefined;
    addcarryxU32(&x265, &x266, x264, x205, x241);
    var x267: u32 = undefined;
    var x268: u1 = undefined;
    addcarryxU32(&x267, &x268, x266, x207, x243);
    var x269: u32 = undefined;
    var x270: u1 = undefined;
    addcarryxU32(&x269, &x270, x268, x209, x245);
    var x271: u32 = undefined;
    var x272: u1 = undefined;
    addcarryxU32(&x271, &x272, x270, x211, x247);
    var x273: u32 = undefined;
    var x274: u1 = undefined;
    addcarryxU32(&x273, &x274, x272, (cast(u32, x212) + cast(u32, x178)), (cast(u32, x248) + x214));
    var x275: u32 = undefined;
    var x276: u32 = undefined;
    mulxU32(&x275, &x276, x3, 0x2);
    var x277: u32 = undefined;
    var x278: u32 = undefined;
    mulxU32(&x277, &x278, x3, 0xfffffffe);
    var x279: u32 = undefined;
    var x280: u32 = undefined;
    mulxU32(&x279, &x280, x3, 0x2);
    var x281: u32 = undefined;
    var x282: u32 = undefined;
    mulxU32(&x281, &x282, x3, 0xfffffffe);
    var x283: u32 = undefined;
    var x284: u1 = undefined;
    addcarryxU32(&x283, &x284, 0x0, cast(u32, cast(u1, x276)), x3);
    var x285: u32 = undefined;
    var x286: u1 = undefined;
    addcarryxU32(&x285, &x286, 0x0, x251, x3);
    var x287: u32 = undefined;
    var x288: u1 = undefined;
    addcarryxU32(&x287, &x288, x286, x253, x281);
    var x289: u32 = undefined;
    var x290: u1 = undefined;
    addcarryxU32(&x289, &x290, x288, x255, x282);
    var x291: u32 = undefined;
    var x292: u1 = undefined;
    addcarryxU32(&x291, &x292, x290, x257, x279);
    var x293: u32 = undefined;
    var x294: u1 = undefined;
    addcarryxU32(&x293, &x294, x292, x259, cast(u32, cast(u1, x280)));
    var x295: u32 = undefined;
    var x296: u1 = undefined;
    addcarryxU32(&x295, &x296, x294, x261, x277);
    var x297: u32 = undefined;
    var x298: u1 = undefined;
    addcarryxU32(&x297, &x298, x296, x263, x278);
    var x299: u32 = undefined;
    var x300: u1 = undefined;
    addcarryxU32(&x299, &x300, x298, x265, x275);
    var x301: u32 = undefined;
    var x302: u1 = undefined;
    addcarryxU32(&x301, &x302, x300, x267, x283);
    var x303: u32 = undefined;
    var x304: u1 = undefined;
    addcarryxU32(&x303, &x304, x302, x269, cast(u32, x284));
    var x305: u32 = undefined;
    var x306: u1 = undefined;
    addcarryxU32(&x305, &x306, x304, x271, cast(u32, 0x0));
    var x307: u32 = undefined;
    var x308: u1 = undefined;
    addcarryxU32(&x307, &x308, x306, x273, cast(u32, 0x0));
    var x309: u32 = undefined;
    var x310: u32 = undefined;
    mulxU32(&x309, &x310, x285, 0xffffffff);
    var x311: u32 = undefined;
    var x312: u32 = undefined;
    mulxU32(&x311, &x312, x285, 0xffffffff);
    var x313: u32 = undefined;
    var x314: u32 = undefined;
    mulxU32(&x313, &x314, x285, 0xffffffff);
    var x315: u32 = undefined;
    var x316: u32 = undefined;
    mulxU32(&x315, &x316, x285, 0xffffffff);
    var x317: u32 = undefined;
    var x318: u32 = undefined;
    mulxU32(&x317, &x318, x285, 0xffffffff);
    var x319: u32 = undefined;
    var x320: u32 = undefined;
    mulxU32(&x319, &x320, x285, 0xffffffff);
    var x321: u32 = undefined;
    var x322: u32 = undefined;
    mulxU32(&x321, &x322, x285, 0xffffffff);
    var x323: u32 = undefined;
    var x324: u32 = undefined;
    mulxU32(&x323, &x324, x285, 0xfffffffe);
    var x325: u32 = undefined;
    var x326: u32 = undefined;
    mulxU32(&x325, &x326, x285, 0xffffffff);
    var x327: u32 = undefined;
    var x328: u32 = undefined;
    mulxU32(&x327, &x328, x285, 0xffffffff);
    var x329: u32 = undefined;
    var x330: u1 = undefined;
    addcarryxU32(&x329, &x330, 0x0, x326, x323);
    var x331: u32 = undefined;
    var x332: u1 = undefined;
    addcarryxU32(&x331, &x332, x330, x324, x321);
    var x333: u32 = undefined;
    var x334: u1 = undefined;
    addcarryxU32(&x333, &x334, x332, x322, x319);
    var x335: u32 = undefined;
    var x336: u1 = undefined;
    addcarryxU32(&x335, &x336, x334, x320, x317);
    var x337: u32 = undefined;
    var x338: u1 = undefined;
    addcarryxU32(&x337, &x338, x336, x318, x315);
    var x339: u32 = undefined;
    var x340: u1 = undefined;
    addcarryxU32(&x339, &x340, x338, x316, x313);
    var x341: u32 = undefined;
    var x342: u1 = undefined;
    addcarryxU32(&x341, &x342, x340, x314, x311);
    var x343: u32 = undefined;
    var x344: u1 = undefined;
    addcarryxU32(&x343, &x344, x342, x312, x309);
    var x345: u32 = undefined;
    var x346: u1 = undefined;
    addcarryxU32(&x345, &x346, 0x0, x285, x327);
    var x347: u32 = undefined;
    var x348: u1 = undefined;
    addcarryxU32(&x347, &x348, x346, x287, x328);
    var x349: u32 = undefined;
    var x350: u1 = undefined;
    addcarryxU32(&x349, &x350, x348, x289, cast(u32, 0x0));
    var x351: u32 = undefined;
    var x352: u1 = undefined;
    addcarryxU32(&x351, &x352, x350, x291, x325);
    var x353: u32 = undefined;
    var x354: u1 = undefined;
    addcarryxU32(&x353, &x354, x352, x293, x329);
    var x355: u32 = undefined;
    var x356: u1 = undefined;
    addcarryxU32(&x355, &x356, x354, x295, x331);
    var x357: u32 = undefined;
    var x358: u1 = undefined;
    addcarryxU32(&x357, &x358, x356, x297, x333);
    var x359: u32 = undefined;
    var x360: u1 = undefined;
    addcarryxU32(&x359, &x360, x358, x299, x335);
    var x361: u32 = undefined;
    var x362: u1 = undefined;
    addcarryxU32(&x361, &x362, x360, x301, x337);
    var x363: u32 = undefined;
    var x364: u1 = undefined;
    addcarryxU32(&x363, &x364, x362, x303, x339);
    var x365: u32 = undefined;
    var x366: u1 = undefined;
    addcarryxU32(&x365, &x366, x364, x305, x341);
    var x367: u32 = undefined;
    var x368: u1 = undefined;
    addcarryxU32(&x367, &x368, x366, x307, x343);
    var x369: u32 = undefined;
    var x370: u1 = undefined;
    addcarryxU32(&x369, &x370, x368, (cast(u32, x308) + cast(u32, x274)), (cast(u32, x344) + x310));
    var x371: u32 = undefined;
    var x372: u32 = undefined;
    mulxU32(&x371, &x372, x4, 0x2);
    var x373: u32 = undefined;
    var x374: u32 = undefined;
    mulxU32(&x373, &x374, x4, 0xfffffffe);
    var x375: u32 = undefined;
    var x376: u32 = undefined;
    mulxU32(&x375, &x376, x4, 0x2);
    var x377: u32 = undefined;
    var x378: u32 = undefined;
    mulxU32(&x377, &x378, x4, 0xfffffffe);
    var x379: u32 = undefined;
    var x380: u1 = undefined;
    addcarryxU32(&x379, &x380, 0x0, cast(u32, cast(u1, x372)), x4);
    var x381: u32 = undefined;
    var x382: u1 = undefined;
    addcarryxU32(&x381, &x382, 0x0, x347, x4);
    var x383: u32 = undefined;
    var x384: u1 = undefined;
    addcarryxU32(&x383, &x384, x382, x349, x377);
    var x385: u32 = undefined;
    var x386: u1 = undefined;
    addcarryxU32(&x385, &x386, x384, x351, x378);
    var x387: u32 = undefined;
    var x388: u1 = undefined;
    addcarryxU32(&x387, &x388, x386, x353, x375);
    var x389: u32 = undefined;
    var x390: u1 = undefined;
    addcarryxU32(&x389, &x390, x388, x355, cast(u32, cast(u1, x376)));
    var x391: u32 = undefined;
    var x392: u1 = undefined;
    addcarryxU32(&x391, &x392, x390, x357, x373);
    var x393: u32 = undefined;
    var x394: u1 = undefined;
    addcarryxU32(&x393, &x394, x392, x359, x374);
    var x395: u32 = undefined;
    var x396: u1 = undefined;
    addcarryxU32(&x395, &x396, x394, x361, x371);
    var x397: u32 = undefined;
    var x398: u1 = undefined;
    addcarryxU32(&x397, &x398, x396, x363, x379);
    var x399: u32 = undefined;
    var x400: u1 = undefined;
    addcarryxU32(&x399, &x400, x398, x365, cast(u32, x380));
    var x401: u32 = undefined;
    var x402: u1 = undefined;
    addcarryxU32(&x401, &x402, x400, x367, cast(u32, 0x0));
    var x403: u32 = undefined;
    var x404: u1 = undefined;
    addcarryxU32(&x403, &x404, x402, x369, cast(u32, 0x0));
    var x405: u32 = undefined;
    var x406: u32 = undefined;
    mulxU32(&x405, &x406, x381, 0xffffffff);
    var x407: u32 = undefined;
    var x408: u32 = undefined;
    mulxU32(&x407, &x408, x381, 0xffffffff);
    var x409: u32 = undefined;
    var x410: u32 = undefined;
    mulxU32(&x409, &x410, x381, 0xffffffff);
    var x411: u32 = undefined;
    var x412: u32 = undefined;
    mulxU32(&x411, &x412, x381, 0xffffffff);
    var x413: u32 = undefined;
    var x414: u32 = undefined;
    mulxU32(&x413, &x414, x381, 0xffffffff);
    var x415: u32 = undefined;
    var x416: u32 = undefined;
    mulxU32(&x415, &x416, x381, 0xffffffff);
    var x417: u32 = undefined;
    var x418: u32 = undefined;
    mulxU32(&x417, &x418, x381, 0xffffffff);
    var x419: u32 = undefined;
    var x420: u32 = undefined;
    mulxU32(&x419, &x420, x381, 0xfffffffe);
    var x421: u32 = undefined;
    var x422: u32 = undefined;
    mulxU32(&x421, &x422, x381, 0xffffffff);
    var x423: u32 = undefined;
    var x424: u32 = undefined;
    mulxU32(&x423, &x424, x381, 0xffffffff);
    var x425: u32 = undefined;
    var x426: u1 = undefined;
    addcarryxU32(&x425, &x426, 0x0, x422, x419);
    var x427: u32 = undefined;
    var x428: u1 = undefined;
    addcarryxU32(&x427, &x428, x426, x420, x417);
    var x429: u32 = undefined;
    var x430: u1 = undefined;
    addcarryxU32(&x429, &x430, x428, x418, x415);
    var x431: u32 = undefined;
    var x432: u1 = undefined;
    addcarryxU32(&x431, &x432, x430, x416, x413);
    var x433: u32 = undefined;
    var x434: u1 = undefined;
    addcarryxU32(&x433, &x434, x432, x414, x411);
    var x435: u32 = undefined;
    var x436: u1 = undefined;
    addcarryxU32(&x435, &x436, x434, x412, x409);
    var x437: u32 = undefined;
    var x438: u1 = undefined;
    addcarryxU32(&x437, &x438, x436, x410, x407);
    var x439: u32 = undefined;
    var x440: u1 = undefined;
    addcarryxU32(&x439, &x440, x438, x408, x405);
    var x441: u32 = undefined;
    var x442: u1 = undefined;
    addcarryxU32(&x441, &x442, 0x0, x381, x423);
    var x443: u32 = undefined;
    var x444: u1 = undefined;
    addcarryxU32(&x443, &x444, x442, x383, x424);
    var x445: u32 = undefined;
    var x446: u1 = undefined;
    addcarryxU32(&x445, &x446, x444, x385, cast(u32, 0x0));
    var x447: u32 = undefined;
    var x448: u1 = undefined;
    addcarryxU32(&x447, &x448, x446, x387, x421);
    var x449: u32 = undefined;
    var x450: u1 = undefined;
    addcarryxU32(&x449, &x450, x448, x389, x425);
    var x451: u32 = undefined;
    var x452: u1 = undefined;
    addcarryxU32(&x451, &x452, x450, x391, x427);
    var x453: u32 = undefined;
    var x454: u1 = undefined;
    addcarryxU32(&x453, &x454, x452, x393, x429);
    var x455: u32 = undefined;
    var x456: u1 = undefined;
    addcarryxU32(&x455, &x456, x454, x395, x431);
    var x457: u32 = undefined;
    var x458: u1 = undefined;
    addcarryxU32(&x457, &x458, x456, x397, x433);
    var x459: u32 = undefined;
    var x460: u1 = undefined;
    addcarryxU32(&x459, &x460, x458, x399, x435);
    var x461: u32 = undefined;
    var x462: u1 = undefined;
    addcarryxU32(&x461, &x462, x460, x401, x437);
    var x463: u32 = undefined;
    var x464: u1 = undefined;
    addcarryxU32(&x463, &x464, x462, x403, x439);
    var x465: u32 = undefined;
    var x466: u1 = undefined;
    addcarryxU32(&x465, &x466, x464, (cast(u32, x404) + cast(u32, x370)), (cast(u32, x440) + x406));
    var x467: u32 = undefined;
    var x468: u32 = undefined;
    mulxU32(&x467, &x468, x5, 0x2);
    var x469: u32 = undefined;
    var x470: u32 = undefined;
    mulxU32(&x469, &x470, x5, 0xfffffffe);
    var x471: u32 = undefined;
    var x472: u32 = undefined;
    mulxU32(&x471, &x472, x5, 0x2);
    var x473: u32 = undefined;
    var x474: u32 = undefined;
    mulxU32(&x473, &x474, x5, 0xfffffffe);
    var x475: u32 = undefined;
    var x476: u1 = undefined;
    addcarryxU32(&x475, &x476, 0x0, cast(u32, cast(u1, x468)), x5);
    var x477: u32 = undefined;
    var x478: u1 = undefined;
    addcarryxU32(&x477, &x478, 0x0, x443, x5);
    var x479: u32 = undefined;
    var x480: u1 = undefined;
    addcarryxU32(&x479, &x480, x478, x445, x473);
    var x481: u32 = undefined;
    var x482: u1 = undefined;
    addcarryxU32(&x481, &x482, x480, x447, x474);
    var x483: u32 = undefined;
    var x484: u1 = undefined;
    addcarryxU32(&x483, &x484, x482, x449, x471);
    var x485: u32 = undefined;
    var x486: u1 = undefined;
    addcarryxU32(&x485, &x486, x484, x451, cast(u32, cast(u1, x472)));
    var x487: u32 = undefined;
    var x488: u1 = undefined;
    addcarryxU32(&x487, &x488, x486, x453, x469);
    var x489: u32 = undefined;
    var x490: u1 = undefined;
    addcarryxU32(&x489, &x490, x488, x455, x470);
    var x491: u32 = undefined;
    var x492: u1 = undefined;
    addcarryxU32(&x491, &x492, x490, x457, x467);
    var x493: u32 = undefined;
    var x494: u1 = undefined;
    addcarryxU32(&x493, &x494, x492, x459, x475);
    var x495: u32 = undefined;
    var x496: u1 = undefined;
    addcarryxU32(&x495, &x496, x494, x461, cast(u32, x476));
    var x497: u32 = undefined;
    var x498: u1 = undefined;
    addcarryxU32(&x497, &x498, x496, x463, cast(u32, 0x0));
    var x499: u32 = undefined;
    var x500: u1 = undefined;
    addcarryxU32(&x499, &x500, x498, x465, cast(u32, 0x0));
    var x501: u32 = undefined;
    var x502: u32 = undefined;
    mulxU32(&x501, &x502, x477, 0xffffffff);
    var x503: u32 = undefined;
    var x504: u32 = undefined;
    mulxU32(&x503, &x504, x477, 0xffffffff);
    var x505: u32 = undefined;
    var x506: u32 = undefined;
    mulxU32(&x505, &x506, x477, 0xffffffff);
    var x507: u32 = undefined;
    var x508: u32 = undefined;
    mulxU32(&x507, &x508, x477, 0xffffffff);
    var x509: u32 = undefined;
    var x510: u32 = undefined;
    mulxU32(&x509, &x510, x477, 0xffffffff);
    var x511: u32 = undefined;
    var x512: u32 = undefined;
    mulxU32(&x511, &x512, x477, 0xffffffff);
    var x513: u32 = undefined;
    var x514: u32 = undefined;
    mulxU32(&x513, &x514, x477, 0xffffffff);
    var x515: u32 = undefined;
    var x516: u32 = undefined;
    mulxU32(&x515, &x516, x477, 0xfffffffe);
    var x517: u32 = undefined;
    var x518: u32 = undefined;
    mulxU32(&x517, &x518, x477, 0xffffffff);
    var x519: u32 = undefined;
    var x520: u32 = undefined;
    mulxU32(&x519, &x520, x477, 0xffffffff);
    var x521: u32 = undefined;
    var x522: u1 = undefined;
    addcarryxU32(&x521, &x522, 0x0, x518, x515);
    var x523: u32 = undefined;
    var x524: u1 = undefined;
    addcarryxU32(&x523, &x524, x522, x516, x513);
    var x525: u32 = undefined;
    var x526: u1 = undefined;
    addcarryxU32(&x525, &x526, x524, x514, x511);
    var x527: u32 = undefined;
    var x528: u1 = undefined;
    addcarryxU32(&x527, &x528, x526, x512, x509);
    var x529: u32 = undefined;
    var x530: u1 = undefined;
    addcarryxU32(&x529, &x530, x528, x510, x507);
    var x531: u32 = undefined;
    var x532: u1 = undefined;
    addcarryxU32(&x531, &x532, x530, x508, x505);
    var x533: u32 = undefined;
    var x534: u1 = undefined;
    addcarryxU32(&x533, &x534, x532, x506, x503);
    var x535: u32 = undefined;
    var x536: u1 = undefined;
    addcarryxU32(&x535, &x536, x534, x504, x501);
    var x537: u32 = undefined;
    var x538: u1 = undefined;
    addcarryxU32(&x537, &x538, 0x0, x477, x519);
    var x539: u32 = undefined;
    var x540: u1 = undefined;
    addcarryxU32(&x539, &x540, x538, x479, x520);
    var x541: u32 = undefined;
    var x542: u1 = undefined;
    addcarryxU32(&x541, &x542, x540, x481, cast(u32, 0x0));
    var x543: u32 = undefined;
    var x544: u1 = undefined;
    addcarryxU32(&x543, &x544, x542, x483, x517);
    var x545: u32 = undefined;
    var x546: u1 = undefined;
    addcarryxU32(&x545, &x546, x544, x485, x521);
    var x547: u32 = undefined;
    var x548: u1 = undefined;
    addcarryxU32(&x547, &x548, x546, x487, x523);
    var x549: u32 = undefined;
    var x550: u1 = undefined;
    addcarryxU32(&x549, &x550, x548, x489, x525);
    var x551: u32 = undefined;
    var x552: u1 = undefined;
    addcarryxU32(&x551, &x552, x550, x491, x527);
    var x553: u32 = undefined;
    var x554: u1 = undefined;
    addcarryxU32(&x553, &x554, x552, x493, x529);
    var x555: u32 = undefined;
    var x556: u1 = undefined;
    addcarryxU32(&x555, &x556, x554, x495, x531);
    var x557: u32 = undefined;
    var x558: u1 = undefined;
    addcarryxU32(&x557, &x558, x556, x497, x533);
    var x559: u32 = undefined;
    var x560: u1 = undefined;
    addcarryxU32(&x559, &x560, x558, x499, x535);
    var x561: u32 = undefined;
    var x562: u1 = undefined;
    addcarryxU32(&x561, &x562, x560, (cast(u32, x500) + cast(u32, x466)), (cast(u32, x536) + x502));
    var x563: u32 = undefined;
    var x564: u32 = undefined;
    mulxU32(&x563, &x564, x6, 0x2);
    var x565: u32 = undefined;
    var x566: u32 = undefined;
    mulxU32(&x565, &x566, x6, 0xfffffffe);
    var x567: u32 = undefined;
    var x568: u32 = undefined;
    mulxU32(&x567, &x568, x6, 0x2);
    var x569: u32 = undefined;
    var x570: u32 = undefined;
    mulxU32(&x569, &x570, x6, 0xfffffffe);
    var x571: u32 = undefined;
    var x572: u1 = undefined;
    addcarryxU32(&x571, &x572, 0x0, cast(u32, cast(u1, x564)), x6);
    var x573: u32 = undefined;
    var x574: u1 = undefined;
    addcarryxU32(&x573, &x574, 0x0, x539, x6);
    var x575: u32 = undefined;
    var x576: u1 = undefined;
    addcarryxU32(&x575, &x576, x574, x541, x569);
    var x577: u32 = undefined;
    var x578: u1 = undefined;
    addcarryxU32(&x577, &x578, x576, x543, x570);
    var x579: u32 = undefined;
    var x580: u1 = undefined;
    addcarryxU32(&x579, &x580, x578, x545, x567);
    var x581: u32 = undefined;
    var x582: u1 = undefined;
    addcarryxU32(&x581, &x582, x580, x547, cast(u32, cast(u1, x568)));
    var x583: u32 = undefined;
    var x584: u1 = undefined;
    addcarryxU32(&x583, &x584, x582, x549, x565);
    var x585: u32 = undefined;
    var x586: u1 = undefined;
    addcarryxU32(&x585, &x586, x584, x551, x566);
    var x587: u32 = undefined;
    var x588: u1 = undefined;
    addcarryxU32(&x587, &x588, x586, x553, x563);
    var x589: u32 = undefined;
    var x590: u1 = undefined;
    addcarryxU32(&x589, &x590, x588, x555, x571);
    var x591: u32 = undefined;
    var x592: u1 = undefined;
    addcarryxU32(&x591, &x592, x590, x557, cast(u32, x572));
    var x593: u32 = undefined;
    var x594: u1 = undefined;
    addcarryxU32(&x593, &x594, x592, x559, cast(u32, 0x0));
    var x595: u32 = undefined;
    var x596: u1 = undefined;
    addcarryxU32(&x595, &x596, x594, x561, cast(u32, 0x0));
    var x597: u32 = undefined;
    var x598: u32 = undefined;
    mulxU32(&x597, &x598, x573, 0xffffffff);
    var x599: u32 = undefined;
    var x600: u32 = undefined;
    mulxU32(&x599, &x600, x573, 0xffffffff);
    var x601: u32 = undefined;
    var x602: u32 = undefined;
    mulxU32(&x601, &x602, x573, 0xffffffff);
    var x603: u32 = undefined;
    var x604: u32 = undefined;
    mulxU32(&x603, &x604, x573, 0xffffffff);
    var x605: u32 = undefined;
    var x606: u32 = undefined;
    mulxU32(&x605, &x606, x573, 0xffffffff);
    var x607: u32 = undefined;
    var x608: u32 = undefined;
    mulxU32(&x607, &x608, x573, 0xffffffff);
    var x609: u32 = undefined;
    var x610: u32 = undefined;
    mulxU32(&x609, &x610, x573, 0xffffffff);
    var x611: u32 = undefined;
    var x612: u32 = undefined;
    mulxU32(&x611, &x612, x573, 0xfffffffe);
    var x613: u32 = undefined;
    var x614: u32 = undefined;
    mulxU32(&x613, &x614, x573, 0xffffffff);
    var x615: u32 = undefined;
    var x616: u32 = undefined;
    mulxU32(&x615, &x616, x573, 0xffffffff);
    var x617: u32 = undefined;
    var x618: u1 = undefined;
    addcarryxU32(&x617, &x618, 0x0, x614, x611);
    var x619: u32 = undefined;
    var x620: u1 = undefined;
    addcarryxU32(&x619, &x620, x618, x612, x609);
    var x621: u32 = undefined;
    var x622: u1 = undefined;
    addcarryxU32(&x621, &x622, x620, x610, x607);
    var x623: u32 = undefined;
    var x624: u1 = undefined;
    addcarryxU32(&x623, &x624, x622, x608, x605);
    var x625: u32 = undefined;
    var x626: u1 = undefined;
    addcarryxU32(&x625, &x626, x624, x606, x603);
    var x627: u32 = undefined;
    var x628: u1 = undefined;
    addcarryxU32(&x627, &x628, x626, x604, x601);
    var x629: u32 = undefined;
    var x630: u1 = undefined;
    addcarryxU32(&x629, &x630, x628, x602, x599);
    var x631: u32 = undefined;
    var x632: u1 = undefined;
    addcarryxU32(&x631, &x632, x630, x600, x597);
    var x633: u32 = undefined;
    var x634: u1 = undefined;
    addcarryxU32(&x633, &x634, 0x0, x573, x615);
    var x635: u32 = undefined;
    var x636: u1 = undefined;
    addcarryxU32(&x635, &x636, x634, x575, x616);
    var x637: u32 = undefined;
    var x638: u1 = undefined;
    addcarryxU32(&x637, &x638, x636, x577, cast(u32, 0x0));
    var x639: u32 = undefined;
    var x640: u1 = undefined;
    addcarryxU32(&x639, &x640, x638, x579, x613);
    var x641: u32 = undefined;
    var x642: u1 = undefined;
    addcarryxU32(&x641, &x642, x640, x581, x617);
    var x643: u32 = undefined;
    var x644: u1 = undefined;
    addcarryxU32(&x643, &x644, x642, x583, x619);
    var x645: u32 = undefined;
    var x646: u1 = undefined;
    addcarryxU32(&x645, &x646, x644, x585, x621);
    var x647: u32 = undefined;
    var x648: u1 = undefined;
    addcarryxU32(&x647, &x648, x646, x587, x623);
    var x649: u32 = undefined;
    var x650: u1 = undefined;
    addcarryxU32(&x649, &x650, x648, x589, x625);
    var x651: u32 = undefined;
    var x652: u1 = undefined;
    addcarryxU32(&x651, &x652, x650, x591, x627);
    var x653: u32 = undefined;
    var x654: u1 = undefined;
    addcarryxU32(&x653, &x654, x652, x593, x629);
    var x655: u32 = undefined;
    var x656: u1 = undefined;
    addcarryxU32(&x655, &x656, x654, x595, x631);
    var x657: u32 = undefined;
    var x658: u1 = undefined;
    addcarryxU32(&x657, &x658, x656, (cast(u32, x596) + cast(u32, x562)), (cast(u32, x632) + x598));
    var x659: u32 = undefined;
    var x660: u32 = undefined;
    mulxU32(&x659, &x660, x7, 0x2);
    var x661: u32 = undefined;
    var x662: u32 = undefined;
    mulxU32(&x661, &x662, x7, 0xfffffffe);
    var x663: u32 = undefined;
    var x664: u32 = undefined;
    mulxU32(&x663, &x664, x7, 0x2);
    var x665: u32 = undefined;
    var x666: u32 = undefined;
    mulxU32(&x665, &x666, x7, 0xfffffffe);
    var x667: u32 = undefined;
    var x668: u1 = undefined;
    addcarryxU32(&x667, &x668, 0x0, cast(u32, cast(u1, x660)), x7);
    var x669: u32 = undefined;
    var x670: u1 = undefined;
    addcarryxU32(&x669, &x670, 0x0, x635, x7);
    var x671: u32 = undefined;
    var x672: u1 = undefined;
    addcarryxU32(&x671, &x672, x670, x637, x665);
    var x673: u32 = undefined;
    var x674: u1 = undefined;
    addcarryxU32(&x673, &x674, x672, x639, x666);
    var x675: u32 = undefined;
    var x676: u1 = undefined;
    addcarryxU32(&x675, &x676, x674, x641, x663);
    var x677: u32 = undefined;
    var x678: u1 = undefined;
    addcarryxU32(&x677, &x678, x676, x643, cast(u32, cast(u1, x664)));
    var x679: u32 = undefined;
    var x680: u1 = undefined;
    addcarryxU32(&x679, &x680, x678, x645, x661);
    var x681: u32 = undefined;
    var x682: u1 = undefined;
    addcarryxU32(&x681, &x682, x680, x647, x662);
    var x683: u32 = undefined;
    var x684: u1 = undefined;
    addcarryxU32(&x683, &x684, x682, x649, x659);
    var x685: u32 = undefined;
    var x686: u1 = undefined;
    addcarryxU32(&x685, &x686, x684, x651, x667);
    var x687: u32 = undefined;
    var x688: u1 = undefined;
    addcarryxU32(&x687, &x688, x686, x653, cast(u32, x668));
    var x689: u32 = undefined;
    var x690: u1 = undefined;
    addcarryxU32(&x689, &x690, x688, x655, cast(u32, 0x0));
    var x691: u32 = undefined;
    var x692: u1 = undefined;
    addcarryxU32(&x691, &x692, x690, x657, cast(u32, 0x0));
    var x693: u32 = undefined;
    var x694: u32 = undefined;
    mulxU32(&x693, &x694, x669, 0xffffffff);
    var x695: u32 = undefined;
    var x696: u32 = undefined;
    mulxU32(&x695, &x696, x669, 0xffffffff);
    var x697: u32 = undefined;
    var x698: u32 = undefined;
    mulxU32(&x697, &x698, x669, 0xffffffff);
    var x699: u32 = undefined;
    var x700: u32 = undefined;
    mulxU32(&x699, &x700, x669, 0xffffffff);
    var x701: u32 = undefined;
    var x702: u32 = undefined;
    mulxU32(&x701, &x702, x669, 0xffffffff);
    var x703: u32 = undefined;
    var x704: u32 = undefined;
    mulxU32(&x703, &x704, x669, 0xffffffff);
    var x705: u32 = undefined;
    var x706: u32 = undefined;
    mulxU32(&x705, &x706, x669, 0xffffffff);
    var x707: u32 = undefined;
    var x708: u32 = undefined;
    mulxU32(&x707, &x708, x669, 0xfffffffe);
    var x709: u32 = undefined;
    var x710: u32 = undefined;
    mulxU32(&x709, &x710, x669, 0xffffffff);
    var x711: u32 = undefined;
    var x712: u32 = undefined;
    mulxU32(&x711, &x712, x669, 0xffffffff);
    var x713: u32 = undefined;
    var x714: u1 = undefined;
    addcarryxU32(&x713, &x714, 0x0, x710, x707);
    var x715: u32 = undefined;
    var x716: u1 = undefined;
    addcarryxU32(&x715, &x716, x714, x708, x705);
    var x717: u32 = undefined;
    var x718: u1 = undefined;
    addcarryxU32(&x717, &x718, x716, x706, x703);
    var x719: u32 = undefined;
    var x720: u1 = undefined;
    addcarryxU32(&x719, &x720, x718, x704, x701);
    var x721: u32 = undefined;
    var x722: u1 = undefined;
    addcarryxU32(&x721, &x722, x720, x702, x699);
    var x723: u32 = undefined;
    var x724: u1 = undefined;
    addcarryxU32(&x723, &x724, x722, x700, x697);
    var x725: u32 = undefined;
    var x726: u1 = undefined;
    addcarryxU32(&x725, &x726, x724, x698, x695);
    var x727: u32 = undefined;
    var x728: u1 = undefined;
    addcarryxU32(&x727, &x728, x726, x696, x693);
    var x729: u32 = undefined;
    var x730: u1 = undefined;
    addcarryxU32(&x729, &x730, 0x0, x669, x711);
    var x731: u32 = undefined;
    var x732: u1 = undefined;
    addcarryxU32(&x731, &x732, x730, x671, x712);
    var x733: u32 = undefined;
    var x734: u1 = undefined;
    addcarryxU32(&x733, &x734, x732, x673, cast(u32, 0x0));
    var x735: u32 = undefined;
    var x736: u1 = undefined;
    addcarryxU32(&x735, &x736, x734, x675, x709);
    var x737: u32 = undefined;
    var x738: u1 = undefined;
    addcarryxU32(&x737, &x738, x736, x677, x713);
    var x739: u32 = undefined;
    var x740: u1 = undefined;
    addcarryxU32(&x739, &x740, x738, x679, x715);
    var x741: u32 = undefined;
    var x742: u1 = undefined;
    addcarryxU32(&x741, &x742, x740, x681, x717);
    var x743: u32 = undefined;
    var x744: u1 = undefined;
    addcarryxU32(&x743, &x744, x742, x683, x719);
    var x745: u32 = undefined;
    var x746: u1 = undefined;
    addcarryxU32(&x745, &x746, x744, x685, x721);
    var x747: u32 = undefined;
    var x748: u1 = undefined;
    addcarryxU32(&x747, &x748, x746, x687, x723);
    var x749: u32 = undefined;
    var x750: u1 = undefined;
    addcarryxU32(&x749, &x750, x748, x689, x725);
    var x751: u32 = undefined;
    var x752: u1 = undefined;
    addcarryxU32(&x751, &x752, x750, x691, x727);
    var x753: u32 = undefined;
    var x754: u1 = undefined;
    addcarryxU32(&x753, &x754, x752, (cast(u32, x692) + cast(u32, x658)), (cast(u32, x728) + x694));
    var x755: u32 = undefined;
    var x756: u32 = undefined;
    mulxU32(&x755, &x756, x8, 0x2);
    var x757: u32 = undefined;
    var x758: u32 = undefined;
    mulxU32(&x757, &x758, x8, 0xfffffffe);
    var x759: u32 = undefined;
    var x760: u32 = undefined;
    mulxU32(&x759, &x760, x8, 0x2);
    var x761: u32 = undefined;
    var x762: u32 = undefined;
    mulxU32(&x761, &x762, x8, 0xfffffffe);
    var x763: u32 = undefined;
    var x764: u1 = undefined;
    addcarryxU32(&x763, &x764, 0x0, cast(u32, cast(u1, x756)), x8);
    var x765: u32 = undefined;
    var x766: u1 = undefined;
    addcarryxU32(&x765, &x766, 0x0, x731, x8);
    var x767: u32 = undefined;
    var x768: u1 = undefined;
    addcarryxU32(&x767, &x768, x766, x733, x761);
    var x769: u32 = undefined;
    var x770: u1 = undefined;
    addcarryxU32(&x769, &x770, x768, x735, x762);
    var x771: u32 = undefined;
    var x772: u1 = undefined;
    addcarryxU32(&x771, &x772, x770, x737, x759);
    var x773: u32 = undefined;
    var x774: u1 = undefined;
    addcarryxU32(&x773, &x774, x772, x739, cast(u32, cast(u1, x760)));
    var x775: u32 = undefined;
    var x776: u1 = undefined;
    addcarryxU32(&x775, &x776, x774, x741, x757);
    var x777: u32 = undefined;
    var x778: u1 = undefined;
    addcarryxU32(&x777, &x778, x776, x743, x758);
    var x779: u32 = undefined;
    var x780: u1 = undefined;
    addcarryxU32(&x779, &x780, x778, x745, x755);
    var x781: u32 = undefined;
    var x782: u1 = undefined;
    addcarryxU32(&x781, &x782, x780, x747, x763);
    var x783: u32 = undefined;
    var x784: u1 = undefined;
    addcarryxU32(&x783, &x784, x782, x749, cast(u32, x764));
    var x785: u32 = undefined;
    var x786: u1 = undefined;
    addcarryxU32(&x785, &x786, x784, x751, cast(u32, 0x0));
    var x787: u32 = undefined;
    var x788: u1 = undefined;
    addcarryxU32(&x787, &x788, x786, x753, cast(u32, 0x0));
    var x789: u32 = undefined;
    var x790: u32 = undefined;
    mulxU32(&x789, &x790, x765, 0xffffffff);
    var x791: u32 = undefined;
    var x792: u32 = undefined;
    mulxU32(&x791, &x792, x765, 0xffffffff);
    var x793: u32 = undefined;
    var x794: u32 = undefined;
    mulxU32(&x793, &x794, x765, 0xffffffff);
    var x795: u32 = undefined;
    var x796: u32 = undefined;
    mulxU32(&x795, &x796, x765, 0xffffffff);
    var x797: u32 = undefined;
    var x798: u32 = undefined;
    mulxU32(&x797, &x798, x765, 0xffffffff);
    var x799: u32 = undefined;
    var x800: u32 = undefined;
    mulxU32(&x799, &x800, x765, 0xffffffff);
    var x801: u32 = undefined;
    var x802: u32 = undefined;
    mulxU32(&x801, &x802, x765, 0xffffffff);
    var x803: u32 = undefined;
    var x804: u32 = undefined;
    mulxU32(&x803, &x804, x765, 0xfffffffe);
    var x805: u32 = undefined;
    var x806: u32 = undefined;
    mulxU32(&x805, &x806, x765, 0xffffffff);
    var x807: u32 = undefined;
    var x808: u32 = undefined;
    mulxU32(&x807, &x808, x765, 0xffffffff);
    var x809: u32 = undefined;
    var x810: u1 = undefined;
    addcarryxU32(&x809, &x810, 0x0, x806, x803);
    var x811: u32 = undefined;
    var x812: u1 = undefined;
    addcarryxU32(&x811, &x812, x810, x804, x801);
    var x813: u32 = undefined;
    var x814: u1 = undefined;
    addcarryxU32(&x813, &x814, x812, x802, x799);
    var x815: u32 = undefined;
    var x816: u1 = undefined;
    addcarryxU32(&x815, &x816, x814, x800, x797);
    var x817: u32 = undefined;
    var x818: u1 = undefined;
    addcarryxU32(&x817, &x818, x816, x798, x795);
    var x819: u32 = undefined;
    var x820: u1 = undefined;
    addcarryxU32(&x819, &x820, x818, x796, x793);
    var x821: u32 = undefined;
    var x822: u1 = undefined;
    addcarryxU32(&x821, &x822, x820, x794, x791);
    var x823: u32 = undefined;
    var x824: u1 = undefined;
    addcarryxU32(&x823, &x824, x822, x792, x789);
    var x825: u32 = undefined;
    var x826: u1 = undefined;
    addcarryxU32(&x825, &x826, 0x0, x765, x807);
    var x827: u32 = undefined;
    var x828: u1 = undefined;
    addcarryxU32(&x827, &x828, x826, x767, x808);
    var x829: u32 = undefined;
    var x830: u1 = undefined;
    addcarryxU32(&x829, &x830, x828, x769, cast(u32, 0x0));
    var x831: u32 = undefined;
    var x832: u1 = undefined;
    addcarryxU32(&x831, &x832, x830, x771, x805);
    var x833: u32 = undefined;
    var x834: u1 = undefined;
    addcarryxU32(&x833, &x834, x832, x773, x809);
    var x835: u32 = undefined;
    var x836: u1 = undefined;
    addcarryxU32(&x835, &x836, x834, x775, x811);
    var x837: u32 = undefined;
    var x838: u1 = undefined;
    addcarryxU32(&x837, &x838, x836, x777, x813);
    var x839: u32 = undefined;
    var x840: u1 = undefined;
    addcarryxU32(&x839, &x840, x838, x779, x815);
    var x841: u32 = undefined;
    var x842: u1 = undefined;
    addcarryxU32(&x841, &x842, x840, x781, x817);
    var x843: u32 = undefined;
    var x844: u1 = undefined;
    addcarryxU32(&x843, &x844, x842, x783, x819);
    var x845: u32 = undefined;
    var x846: u1 = undefined;
    addcarryxU32(&x845, &x846, x844, x785, x821);
    var x847: u32 = undefined;
    var x848: u1 = undefined;
    addcarryxU32(&x847, &x848, x846, x787, x823);
    var x849: u32 = undefined;
    var x850: u1 = undefined;
    addcarryxU32(&x849, &x850, x848, (cast(u32, x788) + cast(u32, x754)), (cast(u32, x824) + x790));
    var x851: u32 = undefined;
    var x852: u32 = undefined;
    mulxU32(&x851, &x852, x9, 0x2);
    var x853: u32 = undefined;
    var x854: u32 = undefined;
    mulxU32(&x853, &x854, x9, 0xfffffffe);
    var x855: u32 = undefined;
    var x856: u32 = undefined;
    mulxU32(&x855, &x856, x9, 0x2);
    var x857: u32 = undefined;
    var x858: u32 = undefined;
    mulxU32(&x857, &x858, x9, 0xfffffffe);
    var x859: u32 = undefined;
    var x860: u1 = undefined;
    addcarryxU32(&x859, &x860, 0x0, cast(u32, cast(u1, x852)), x9);
    var x861: u32 = undefined;
    var x862: u1 = undefined;
    addcarryxU32(&x861, &x862, 0x0, x827, x9);
    var x863: u32 = undefined;
    var x864: u1 = undefined;
    addcarryxU32(&x863, &x864, x862, x829, x857);
    var x865: u32 = undefined;
    var x866: u1 = undefined;
    addcarryxU32(&x865, &x866, x864, x831, x858);
    var x867: u32 = undefined;
    var x868: u1 = undefined;
    addcarryxU32(&x867, &x868, x866, x833, x855);
    var x869: u32 = undefined;
    var x870: u1 = undefined;
    addcarryxU32(&x869, &x870, x868, x835, cast(u32, cast(u1, x856)));
    var x871: u32 = undefined;
    var x872: u1 = undefined;
    addcarryxU32(&x871, &x872, x870, x837, x853);
    var x873: u32 = undefined;
    var x874: u1 = undefined;
    addcarryxU32(&x873, &x874, x872, x839, x854);
    var x875: u32 = undefined;
    var x876: u1 = undefined;
    addcarryxU32(&x875, &x876, x874, x841, x851);
    var x877: u32 = undefined;
    var x878: u1 = undefined;
    addcarryxU32(&x877, &x878, x876, x843, x859);
    var x879: u32 = undefined;
    var x880: u1 = undefined;
    addcarryxU32(&x879, &x880, x878, x845, cast(u32, x860));
    var x881: u32 = undefined;
    var x882: u1 = undefined;
    addcarryxU32(&x881, &x882, x880, x847, cast(u32, 0x0));
    var x883: u32 = undefined;
    var x884: u1 = undefined;
    addcarryxU32(&x883, &x884, x882, x849, cast(u32, 0x0));
    var x885: u32 = undefined;
    var x886: u32 = undefined;
    mulxU32(&x885, &x886, x861, 0xffffffff);
    var x887: u32 = undefined;
    var x888: u32 = undefined;
    mulxU32(&x887, &x888, x861, 0xffffffff);
    var x889: u32 = undefined;
    var x890: u32 = undefined;
    mulxU32(&x889, &x890, x861, 0xffffffff);
    var x891: u32 = undefined;
    var x892: u32 = undefined;
    mulxU32(&x891, &x892, x861, 0xffffffff);
    var x893: u32 = undefined;
    var x894: u32 = undefined;
    mulxU32(&x893, &x894, x861, 0xffffffff);
    var x895: u32 = undefined;
    var x896: u32 = undefined;
    mulxU32(&x895, &x896, x861, 0xffffffff);
    var x897: u32 = undefined;
    var x898: u32 = undefined;
    mulxU32(&x897, &x898, x861, 0xffffffff);
    var x899: u32 = undefined;
    var x900: u32 = undefined;
    mulxU32(&x899, &x900, x861, 0xfffffffe);
    var x901: u32 = undefined;
    var x902: u32 = undefined;
    mulxU32(&x901, &x902, x861, 0xffffffff);
    var x903: u32 = undefined;
    var x904: u32 = undefined;
    mulxU32(&x903, &x904, x861, 0xffffffff);
    var x905: u32 = undefined;
    var x906: u1 = undefined;
    addcarryxU32(&x905, &x906, 0x0, x902, x899);
    var x907: u32 = undefined;
    var x908: u1 = undefined;
    addcarryxU32(&x907, &x908, x906, x900, x897);
    var x909: u32 = undefined;
    var x910: u1 = undefined;
    addcarryxU32(&x909, &x910, x908, x898, x895);
    var x911: u32 = undefined;
    var x912: u1 = undefined;
    addcarryxU32(&x911, &x912, x910, x896, x893);
    var x913: u32 = undefined;
    var x914: u1 = undefined;
    addcarryxU32(&x913, &x914, x912, x894, x891);
    var x915: u32 = undefined;
    var x916: u1 = undefined;
    addcarryxU32(&x915, &x916, x914, x892, x889);
    var x917: u32 = undefined;
    var x918: u1 = undefined;
    addcarryxU32(&x917, &x918, x916, x890, x887);
    var x919: u32 = undefined;
    var x920: u1 = undefined;
    addcarryxU32(&x919, &x920, x918, x888, x885);
    var x921: u32 = undefined;
    var x922: u1 = undefined;
    addcarryxU32(&x921, &x922, 0x0, x861, x903);
    var x923: u32 = undefined;
    var x924: u1 = undefined;
    addcarryxU32(&x923, &x924, x922, x863, x904);
    var x925: u32 = undefined;
    var x926: u1 = undefined;
    addcarryxU32(&x925, &x926, x924, x865, cast(u32, 0x0));
    var x927: u32 = undefined;
    var x928: u1 = undefined;
    addcarryxU32(&x927, &x928, x926, x867, x901);
    var x929: u32 = undefined;
    var x930: u1 = undefined;
    addcarryxU32(&x929, &x930, x928, x869, x905);
    var x931: u32 = undefined;
    var x932: u1 = undefined;
    addcarryxU32(&x931, &x932, x930, x871, x907);
    var x933: u32 = undefined;
    var x934: u1 = undefined;
    addcarryxU32(&x933, &x934, x932, x873, x909);
    var x935: u32 = undefined;
    var x936: u1 = undefined;
    addcarryxU32(&x935, &x936, x934, x875, x911);
    var x937: u32 = undefined;
    var x938: u1 = undefined;
    addcarryxU32(&x937, &x938, x936, x877, x913);
    var x939: u32 = undefined;
    var x940: u1 = undefined;
    addcarryxU32(&x939, &x940, x938, x879, x915);
    var x941: u32 = undefined;
    var x942: u1 = undefined;
    addcarryxU32(&x941, &x942, x940, x881, x917);
    var x943: u32 = undefined;
    var x944: u1 = undefined;
    addcarryxU32(&x943, &x944, x942, x883, x919);
    var x945: u32 = undefined;
    var x946: u1 = undefined;
    addcarryxU32(&x945, &x946, x944, (cast(u32, x884) + cast(u32, x850)), (cast(u32, x920) + x886));
    var x947: u32 = undefined;
    var x948: u32 = undefined;
    mulxU32(&x947, &x948, x10, 0x2);
    var x949: u32 = undefined;
    var x950: u32 = undefined;
    mulxU32(&x949, &x950, x10, 0xfffffffe);
    var x951: u32 = undefined;
    var x952: u32 = undefined;
    mulxU32(&x951, &x952, x10, 0x2);
    var x953: u32 = undefined;
    var x954: u32 = undefined;
    mulxU32(&x953, &x954, x10, 0xfffffffe);
    var x955: u32 = undefined;
    var x956: u1 = undefined;
    addcarryxU32(&x955, &x956, 0x0, cast(u32, cast(u1, x948)), x10);
    var x957: u32 = undefined;
    var x958: u1 = undefined;
    addcarryxU32(&x957, &x958, 0x0, x923, x10);
    var x959: u32 = undefined;
    var x960: u1 = undefined;
    addcarryxU32(&x959, &x960, x958, x925, x953);
    var x961: u32 = undefined;
    var x962: u1 = undefined;
    addcarryxU32(&x961, &x962, x960, x927, x954);
    var x963: u32 = undefined;
    var x964: u1 = undefined;
    addcarryxU32(&x963, &x964, x962, x929, x951);
    var x965: u32 = undefined;
    var x966: u1 = undefined;
    addcarryxU32(&x965, &x966, x964, x931, cast(u32, cast(u1, x952)));
    var x967: u32 = undefined;
    var x968: u1 = undefined;
    addcarryxU32(&x967, &x968, x966, x933, x949);
    var x969: u32 = undefined;
    var x970: u1 = undefined;
    addcarryxU32(&x969, &x970, x968, x935, x950);
    var x971: u32 = undefined;
    var x972: u1 = undefined;
    addcarryxU32(&x971, &x972, x970, x937, x947);
    var x973: u32 = undefined;
    var x974: u1 = undefined;
    addcarryxU32(&x973, &x974, x972, x939, x955);
    var x975: u32 = undefined;
    var x976: u1 = undefined;
    addcarryxU32(&x975, &x976, x974, x941, cast(u32, x956));
    var x977: u32 = undefined;
    var x978: u1 = undefined;
    addcarryxU32(&x977, &x978, x976, x943, cast(u32, 0x0));
    var x979: u32 = undefined;
    var x980: u1 = undefined;
    addcarryxU32(&x979, &x980, x978, x945, cast(u32, 0x0));
    var x981: u32 = undefined;
    var x982: u32 = undefined;
    mulxU32(&x981, &x982, x957, 0xffffffff);
    var x983: u32 = undefined;
    var x984: u32 = undefined;
    mulxU32(&x983, &x984, x957, 0xffffffff);
    var x985: u32 = undefined;
    var x986: u32 = undefined;
    mulxU32(&x985, &x986, x957, 0xffffffff);
    var x987: u32 = undefined;
    var x988: u32 = undefined;
    mulxU32(&x987, &x988, x957, 0xffffffff);
    var x989: u32 = undefined;
    var x990: u32 = undefined;
    mulxU32(&x989, &x990, x957, 0xffffffff);
    var x991: u32 = undefined;
    var x992: u32 = undefined;
    mulxU32(&x991, &x992, x957, 0xffffffff);
    var x993: u32 = undefined;
    var x994: u32 = undefined;
    mulxU32(&x993, &x994, x957, 0xffffffff);
    var x995: u32 = undefined;
    var x996: u32 = undefined;
    mulxU32(&x995, &x996, x957, 0xfffffffe);
    var x997: u32 = undefined;
    var x998: u32 = undefined;
    mulxU32(&x997, &x998, x957, 0xffffffff);
    var x999: u32 = undefined;
    var x1000: u32 = undefined;
    mulxU32(&x999, &x1000, x957, 0xffffffff);
    var x1001: u32 = undefined;
    var x1002: u1 = undefined;
    addcarryxU32(&x1001, &x1002, 0x0, x998, x995);
    var x1003: u32 = undefined;
    var x1004: u1 = undefined;
    addcarryxU32(&x1003, &x1004, x1002, x996, x993);
    var x1005: u32 = undefined;
    var x1006: u1 = undefined;
    addcarryxU32(&x1005, &x1006, x1004, x994, x991);
    var x1007: u32 = undefined;
    var x1008: u1 = undefined;
    addcarryxU32(&x1007, &x1008, x1006, x992, x989);
    var x1009: u32 = undefined;
    var x1010: u1 = undefined;
    addcarryxU32(&x1009, &x1010, x1008, x990, x987);
    var x1011: u32 = undefined;
    var x1012: u1 = undefined;
    addcarryxU32(&x1011, &x1012, x1010, x988, x985);
    var x1013: u32 = undefined;
    var x1014: u1 = undefined;
    addcarryxU32(&x1013, &x1014, x1012, x986, x983);
    var x1015: u32 = undefined;
    var x1016: u1 = undefined;
    addcarryxU32(&x1015, &x1016, x1014, x984, x981);
    var x1017: u32 = undefined;
    var x1018: u1 = undefined;
    addcarryxU32(&x1017, &x1018, 0x0, x957, x999);
    var x1019: u32 = undefined;
    var x1020: u1 = undefined;
    addcarryxU32(&x1019, &x1020, x1018, x959, x1000);
    var x1021: u32 = undefined;
    var x1022: u1 = undefined;
    addcarryxU32(&x1021, &x1022, x1020, x961, cast(u32, 0x0));
    var x1023: u32 = undefined;
    var x1024: u1 = undefined;
    addcarryxU32(&x1023, &x1024, x1022, x963, x997);
    var x1025: u32 = undefined;
    var x1026: u1 = undefined;
    addcarryxU32(&x1025, &x1026, x1024, x965, x1001);
    var x1027: u32 = undefined;
    var x1028: u1 = undefined;
    addcarryxU32(&x1027, &x1028, x1026, x967, x1003);
    var x1029: u32 = undefined;
    var x1030: u1 = undefined;
    addcarryxU32(&x1029, &x1030, x1028, x969, x1005);
    var x1031: u32 = undefined;
    var x1032: u1 = undefined;
    addcarryxU32(&x1031, &x1032, x1030, x971, x1007);
    var x1033: u32 = undefined;
    var x1034: u1 = undefined;
    addcarryxU32(&x1033, &x1034, x1032, x973, x1009);
    var x1035: u32 = undefined;
    var x1036: u1 = undefined;
    addcarryxU32(&x1035, &x1036, x1034, x975, x1011);
    var x1037: u32 = undefined;
    var x1038: u1 = undefined;
    addcarryxU32(&x1037, &x1038, x1036, x977, x1013);
    var x1039: u32 = undefined;
    var x1040: u1 = undefined;
    addcarryxU32(&x1039, &x1040, x1038, x979, x1015);
    var x1041: u32 = undefined;
    var x1042: u1 = undefined;
    addcarryxU32(&x1041, &x1042, x1040, (cast(u32, x980) + cast(u32, x946)), (cast(u32, x1016) + x982));
    var x1043: u32 = undefined;
    var x1044: u32 = undefined;
    mulxU32(&x1043, &x1044, x11, 0x2);
    var x1045: u32 = undefined;
    var x1046: u32 = undefined;
    mulxU32(&x1045, &x1046, x11, 0xfffffffe);
    var x1047: u32 = undefined;
    var x1048: u32 = undefined;
    mulxU32(&x1047, &x1048, x11, 0x2);
    var x1049: u32 = undefined;
    var x1050: u32 = undefined;
    mulxU32(&x1049, &x1050, x11, 0xfffffffe);
    var x1051: u32 = undefined;
    var x1052: u1 = undefined;
    addcarryxU32(&x1051, &x1052, 0x0, cast(u32, cast(u1, x1044)), x11);
    var x1053: u32 = undefined;
    var x1054: u1 = undefined;
    addcarryxU32(&x1053, &x1054, 0x0, x1019, x11);
    var x1055: u32 = undefined;
    var x1056: u1 = undefined;
    addcarryxU32(&x1055, &x1056, x1054, x1021, x1049);
    var x1057: u32 = undefined;
    var x1058: u1 = undefined;
    addcarryxU32(&x1057, &x1058, x1056, x1023, x1050);
    var x1059: u32 = undefined;
    var x1060: u1 = undefined;
    addcarryxU32(&x1059, &x1060, x1058, x1025, x1047);
    var x1061: u32 = undefined;
    var x1062: u1 = undefined;
    addcarryxU32(&x1061, &x1062, x1060, x1027, cast(u32, cast(u1, x1048)));
    var x1063: u32 = undefined;
    var x1064: u1 = undefined;
    addcarryxU32(&x1063, &x1064, x1062, x1029, x1045);
    var x1065: u32 = undefined;
    var x1066: u1 = undefined;
    addcarryxU32(&x1065, &x1066, x1064, x1031, x1046);
    var x1067: u32 = undefined;
    var x1068: u1 = undefined;
    addcarryxU32(&x1067, &x1068, x1066, x1033, x1043);
    var x1069: u32 = undefined;
    var x1070: u1 = undefined;
    addcarryxU32(&x1069, &x1070, x1068, x1035, x1051);
    var x1071: u32 = undefined;
    var x1072: u1 = undefined;
    addcarryxU32(&x1071, &x1072, x1070, x1037, cast(u32, x1052));
    var x1073: u32 = undefined;
    var x1074: u1 = undefined;
    addcarryxU32(&x1073, &x1074, x1072, x1039, cast(u32, 0x0));
    var x1075: u32 = undefined;
    var x1076: u1 = undefined;
    addcarryxU32(&x1075, &x1076, x1074, x1041, cast(u32, 0x0));
    var x1077: u32 = undefined;
    var x1078: u32 = undefined;
    mulxU32(&x1077, &x1078, x1053, 0xffffffff);
    var x1079: u32 = undefined;
    var x1080: u32 = undefined;
    mulxU32(&x1079, &x1080, x1053, 0xffffffff);
    var x1081: u32 = undefined;
    var x1082: u32 = undefined;
    mulxU32(&x1081, &x1082, x1053, 0xffffffff);
    var x1083: u32 = undefined;
    var x1084: u32 = undefined;
    mulxU32(&x1083, &x1084, x1053, 0xffffffff);
    var x1085: u32 = undefined;
    var x1086: u32 = undefined;
    mulxU32(&x1085, &x1086, x1053, 0xffffffff);
    var x1087: u32 = undefined;
    var x1088: u32 = undefined;
    mulxU32(&x1087, &x1088, x1053, 0xffffffff);
    var x1089: u32 = undefined;
    var x1090: u32 = undefined;
    mulxU32(&x1089, &x1090, x1053, 0xffffffff);
    var x1091: u32 = undefined;
    var x1092: u32 = undefined;
    mulxU32(&x1091, &x1092, x1053, 0xfffffffe);
    var x1093: u32 = undefined;
    var x1094: u32 = undefined;
    mulxU32(&x1093, &x1094, x1053, 0xffffffff);
    var x1095: u32 = undefined;
    var x1096: u32 = undefined;
    mulxU32(&x1095, &x1096, x1053, 0xffffffff);
    var x1097: u32 = undefined;
    var x1098: u1 = undefined;
    addcarryxU32(&x1097, &x1098, 0x0, x1094, x1091);
    var x1099: u32 = undefined;
    var x1100: u1 = undefined;
    addcarryxU32(&x1099, &x1100, x1098, x1092, x1089);
    var x1101: u32 = undefined;
    var x1102: u1 = undefined;
    addcarryxU32(&x1101, &x1102, x1100, x1090, x1087);
    var x1103: u32 = undefined;
    var x1104: u1 = undefined;
    addcarryxU32(&x1103, &x1104, x1102, x1088, x1085);
    var x1105: u32 = undefined;
    var x1106: u1 = undefined;
    addcarryxU32(&x1105, &x1106, x1104, x1086, x1083);
    var x1107: u32 = undefined;
    var x1108: u1 = undefined;
    addcarryxU32(&x1107, &x1108, x1106, x1084, x1081);
    var x1109: u32 = undefined;
    var x1110: u1 = undefined;
    addcarryxU32(&x1109, &x1110, x1108, x1082, x1079);
    var x1111: u32 = undefined;
    var x1112: u1 = undefined;
    addcarryxU32(&x1111, &x1112, x1110, x1080, x1077);
    var x1113: u32 = undefined;
    var x1114: u1 = undefined;
    addcarryxU32(&x1113, &x1114, 0x0, x1053, x1095);
    var x1115: u32 = undefined;
    var x1116: u1 = undefined;
    addcarryxU32(&x1115, &x1116, x1114, x1055, x1096);
    var x1117: u32 = undefined;
    var x1118: u1 = undefined;
    addcarryxU32(&x1117, &x1118, x1116, x1057, cast(u32, 0x0));
    var x1119: u32 = undefined;
    var x1120: u1 = undefined;
    addcarryxU32(&x1119, &x1120, x1118, x1059, x1093);
    var x1121: u32 = undefined;
    var x1122: u1 = undefined;
    addcarryxU32(&x1121, &x1122, x1120, x1061, x1097);
    var x1123: u32 = undefined;
    var x1124: u1 = undefined;
    addcarryxU32(&x1123, &x1124, x1122, x1063, x1099);
    var x1125: u32 = undefined;
    var x1126: u1 = undefined;
    addcarryxU32(&x1125, &x1126, x1124, x1065, x1101);
    var x1127: u32 = undefined;
    var x1128: u1 = undefined;
    addcarryxU32(&x1127, &x1128, x1126, x1067, x1103);
    var x1129: u32 = undefined;
    var x1130: u1 = undefined;
    addcarryxU32(&x1129, &x1130, x1128, x1069, x1105);
    var x1131: u32 = undefined;
    var x1132: u1 = undefined;
    addcarryxU32(&x1131, &x1132, x1130, x1071, x1107);
    var x1133: u32 = undefined;
    var x1134: u1 = undefined;
    addcarryxU32(&x1133, &x1134, x1132, x1073, x1109);
    var x1135: u32 = undefined;
    var x1136: u1 = undefined;
    addcarryxU32(&x1135, &x1136, x1134, x1075, x1111);
    var x1137: u32 = undefined;
    var x1138: u1 = undefined;
    addcarryxU32(&x1137, &x1138, x1136, (cast(u32, x1076) + cast(u32, x1042)), (cast(u32, x1112) + x1078));
    var x1139: u32 = undefined;
    var x1140: u1 = undefined;
    subborrowxU32(&x1139, &x1140, 0x0, x1115, 0xffffffff);
    var x1141: u32 = undefined;
    var x1142: u1 = undefined;
    subborrowxU32(&x1141, &x1142, x1140, x1117, cast(u32, 0x0));
    var x1143: u32 = undefined;
    var x1144: u1 = undefined;
    subborrowxU32(&x1143, &x1144, x1142, x1119, cast(u32, 0x0));
    var x1145: u32 = undefined;
    var x1146: u1 = undefined;
    subborrowxU32(&x1145, &x1146, x1144, x1121, 0xffffffff);
    var x1147: u32 = undefined;
    var x1148: u1 = undefined;
    subborrowxU32(&x1147, &x1148, x1146, x1123, 0xfffffffe);
    var x1149: u32 = undefined;
    var x1150: u1 = undefined;
    subborrowxU32(&x1149, &x1150, x1148, x1125, 0xffffffff);
    var x1151: u32 = undefined;
    var x1152: u1 = undefined;
    subborrowxU32(&x1151, &x1152, x1150, x1127, 0xffffffff);
    var x1153: u32 = undefined;
    var x1154: u1 = undefined;
    subborrowxU32(&x1153, &x1154, x1152, x1129, 0xffffffff);
    var x1155: u32 = undefined;
    var x1156: u1 = undefined;
    subborrowxU32(&x1155, &x1156, x1154, x1131, 0xffffffff);
    var x1157: u32 = undefined;
    var x1158: u1 = undefined;
    subborrowxU32(&x1157, &x1158, x1156, x1133, 0xffffffff);
    var x1159: u32 = undefined;
    var x1160: u1 = undefined;
    subborrowxU32(&x1159, &x1160, x1158, x1135, 0xffffffff);
    var x1161: u32 = undefined;
    var x1162: u1 = undefined;
    subborrowxU32(&x1161, &x1162, x1160, x1137, 0xffffffff);
    var x1163: u32 = undefined;
    var x1164: u1 = undefined;
    subborrowxU32(&x1163, &x1164, x1162, cast(u32, x1138), cast(u32, 0x0));
    var x1165: u32 = undefined;
    cmovznzU32(&x1165, x1164, x1139, x1115);
    var x1166: u32 = undefined;
    cmovznzU32(&x1166, x1164, x1141, x1117);
    var x1167: u32 = undefined;
    cmovznzU32(&x1167, x1164, x1143, x1119);
    var x1168: u32 = undefined;
    cmovznzU32(&x1168, x1164, x1145, x1121);
    var x1169: u32 = undefined;
    cmovznzU32(&x1169, x1164, x1147, x1123);
    var x1170: u32 = undefined;
    cmovznzU32(&x1170, x1164, x1149, x1125);
    var x1171: u32 = undefined;
    cmovznzU32(&x1171, x1164, x1151, x1127);
    var x1172: u32 = undefined;
    cmovznzU32(&x1172, x1164, x1153, x1129);
    var x1173: u32 = undefined;
    cmovznzU32(&x1173, x1164, x1155, x1131);
    var x1174: u32 = undefined;
    cmovznzU32(&x1174, x1164, x1157, x1133);
    var x1175: u32 = undefined;
    cmovznzU32(&x1175, x1164, x1159, x1135);
    var x1176: u32 = undefined;
    cmovznzU32(&x1176, x1164, x1161, x1137);
    out1[0] = x1165;
    out1[1] = x1166;
    out1[2] = x1167;
    out1[3] = x1168;
    out1[4] = x1169;
    out1[5] = x1170;
    out1[6] = x1171;
    out1[7] = x1172;
    out1[8] = x1173;
    out1[9] = x1174;
    out1[10] = x1175;
    out1[11] = x1176;
}

/// The function nonzero outputs a single non-zero word if the input is non-zero and zero otherwise.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   out1 = 0 ↔ eval (from_montgomery arg1) mod m = 0
///
/// Input Bounds:
///   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
pub fn nonzero(out1: *u32, arg1: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = ((arg1[0]) | ((arg1[1]) | ((arg1[2]) | ((arg1[3]) | ((arg1[4]) | ((arg1[5]) | ((arg1[6]) | ((arg1[7]) | ((arg1[8]) | ((arg1[9]) | ((arg1[10]) | (arg1[11]))))))))))));
    out1.* = x1;
}

/// The function selectznz is a multi-limb conditional select.
///
/// Postconditions:
///   out1 = (if arg1 = 0 then arg2 else arg3)
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn selectznz(out1: *[12]u32, arg1: u1, arg2: [12]u32, arg3: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    cmovznzU32(&x1, arg1, (arg2[0]), (arg3[0]));
    var x2: u32 = undefined;
    cmovznzU32(&x2, arg1, (arg2[1]), (arg3[1]));
    var x3: u32 = undefined;
    cmovznzU32(&x3, arg1, (arg2[2]), (arg3[2]));
    var x4: u32 = undefined;
    cmovznzU32(&x4, arg1, (arg2[3]), (arg3[3]));
    var x5: u32 = undefined;
    cmovznzU32(&x5, arg1, (arg2[4]), (arg3[4]));
    var x6: u32 = undefined;
    cmovznzU32(&x6, arg1, (arg2[5]), (arg3[5]));
    var x7: u32 = undefined;
    cmovznzU32(&x7, arg1, (arg2[6]), (arg3[6]));
    var x8: u32 = undefined;
    cmovznzU32(&x8, arg1, (arg2[7]), (arg3[7]));
    var x9: u32 = undefined;
    cmovznzU32(&x9, arg1, (arg2[8]), (arg3[8]));
    var x10: u32 = undefined;
    cmovznzU32(&x10, arg1, (arg2[9]), (arg3[9]));
    var x11: u32 = undefined;
    cmovznzU32(&x11, arg1, (arg2[10]), (arg3[10]));
    var x12: u32 = undefined;
    cmovznzU32(&x12, arg1, (arg2[11]), (arg3[11]));
    out1[0] = x1;
    out1[1] = x2;
    out1[2] = x3;
    out1[3] = x4;
    out1[4] = x5;
    out1[5] = x6;
    out1[6] = x7;
    out1[7] = x8;
    out1[8] = x9;
    out1[9] = x10;
    out1[10] = x11;
    out1[11] = x12;
}

/// The function toBytes serializes a field element NOT in the Montgomery domain to bytes in little-endian order.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   out1 = map (λ x, ⌊((eval arg1 mod m) mod 2^(8 * (x + 1))) / 2^(8 * x)⌋) [0..47]
///
/// Input Bounds:
///   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
pub fn toBytes(out1: *[48]u8, arg1: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[11]);
    const x2 = (arg1[10]);
    const x3 = (arg1[9]);
    const x4 = (arg1[8]);
    const x5 = (arg1[7]);
    const x6 = (arg1[6]);
    const x7 = (arg1[5]);
    const x8 = (arg1[4]);
    const x9 = (arg1[3]);
    const x10 = (arg1[2]);
    const x11 = (arg1[1]);
    const x12 = (arg1[0]);
    const x13 = cast(u8, (x12 & cast(u32, 0xff)));
    const x14 = (x12 >> 8);
    const x15 = cast(u8, (x14 & cast(u32, 0xff)));
    const x16 = (x14 >> 8);
    const x17 = cast(u8, (x16 & cast(u32, 0xff)));
    const x18 = cast(u8, (x16 >> 8));
    const x19 = cast(u8, (x11 & cast(u32, 0xff)));
    const x20 = (x11 >> 8);
    const x21 = cast(u8, (x20 & cast(u32, 0xff)));
    const x22 = (x20 >> 8);
    const x23 = cast(u8, (x22 & cast(u32, 0xff)));
    const x24 = cast(u8, (x22 >> 8));
    const x25 = cast(u8, (x10 & cast(u32, 0xff)));
    const x26 = (x10 >> 8);
    const x27 = cast(u8, (x26 & cast(u32, 0xff)));
    const x28 = (x26 >> 8);
    const x29 = cast(u8, (x28 & cast(u32, 0xff)));
    const x30 = cast(u8, (x28 >> 8));
    const x31 = cast(u8, (x9 & cast(u32, 0xff)));
    const x32 = (x9 >> 8);
    const x33 = cast(u8, (x32 & cast(u32, 0xff)));
    const x34 = (x32 >> 8);
    const x35 = cast(u8, (x34 & cast(u32, 0xff)));
    const x36 = cast(u8, (x34 >> 8));
    const x37 = cast(u8, (x8 & cast(u32, 0xff)));
    const x38 = (x8 >> 8);
    const x39 = cast(u8, (x38 & cast(u32, 0xff)));
    const x40 = (x38 >> 8);
    const x41 = cast(u8, (x40 & cast(u32, 0xff)));
    const x42 = cast(u8, (x40 >> 8));
    const x43 = cast(u8, (x7 & cast(u32, 0xff)));
    const x44 = (x7 >> 8);
    const x45 = cast(u8, (x44 & cast(u32, 0xff)));
    const x46 = (x44 >> 8);
    const x47 = cast(u8, (x46 & cast(u32, 0xff)));
    const x48 = cast(u8, (x46 >> 8));
    const x49 = cast(u8, (x6 & cast(u32, 0xff)));
    const x50 = (x6 >> 8);
    const x51 = cast(u8, (x50 & cast(u32, 0xff)));
    const x52 = (x50 >> 8);
    const x53 = cast(u8, (x52 & cast(u32, 0xff)));
    const x54 = cast(u8, (x52 >> 8));
    const x55 = cast(u8, (x5 & cast(u32, 0xff)));
    const x56 = (x5 >> 8);
    const x57 = cast(u8, (x56 & cast(u32, 0xff)));
    const x58 = (x56 >> 8);
    const x59 = cast(u8, (x58 & cast(u32, 0xff)));
    const x60 = cast(u8, (x58 >> 8));
    const x61 = cast(u8, (x4 & cast(u32, 0xff)));
    const x62 = (x4 >> 8);
    const x63 = cast(u8, (x62 & cast(u32, 0xff)));
    const x64 = (x62 >> 8);
    const x65 = cast(u8, (x64 & cast(u32, 0xff)));
    const x66 = cast(u8, (x64 >> 8));
    const x67 = cast(u8, (x3 & cast(u32, 0xff)));
    const x68 = (x3 >> 8);
    const x69 = cast(u8, (x68 & cast(u32, 0xff)));
    const x70 = (x68 >> 8);
    const x71 = cast(u8, (x70 & cast(u32, 0xff)));
    const x72 = cast(u8, (x70 >> 8));
    const x73 = cast(u8, (x2 & cast(u32, 0xff)));
    const x74 = (x2 >> 8);
    const x75 = cast(u8, (x74 & cast(u32, 0xff)));
    const x76 = (x74 >> 8);
    const x77 = cast(u8, (x76 & cast(u32, 0xff)));
    const x78 = cast(u8, (x76 >> 8));
    const x79 = cast(u8, (x1 & cast(u32, 0xff)));
    const x80 = (x1 >> 8);
    const x81 = cast(u8, (x80 & cast(u32, 0xff)));
    const x82 = (x80 >> 8);
    const x83 = cast(u8, (x82 & cast(u32, 0xff)));
    const x84 = cast(u8, (x82 >> 8));
    out1[0] = x13;
    out1[1] = x15;
    out1[2] = x17;
    out1[3] = x18;
    out1[4] = x19;
    out1[5] = x21;
    out1[6] = x23;
    out1[7] = x24;
    out1[8] = x25;
    out1[9] = x27;
    out1[10] = x29;
    out1[11] = x30;
    out1[12] = x31;
    out1[13] = x33;
    out1[14] = x35;
    out1[15] = x36;
    out1[16] = x37;
    out1[17] = x39;
    out1[18] = x41;
    out1[19] = x42;
    out1[20] = x43;
    out1[21] = x45;
    out1[22] = x47;
    out1[23] = x48;
    out1[24] = x49;
    out1[25] = x51;
    out1[26] = x53;
    out1[27] = x54;
    out1[28] = x55;
    out1[29] = x57;
    out1[30] = x59;
    out1[31] = x60;
    out1[32] = x61;
    out1[33] = x63;
    out1[34] = x65;
    out1[35] = x66;
    out1[36] = x67;
    out1[37] = x69;
    out1[38] = x71;
    out1[39] = x72;
    out1[40] = x73;
    out1[41] = x75;
    out1[42] = x77;
    out1[43] = x78;
    out1[44] = x79;
    out1[45] = x81;
    out1[46] = x83;
    out1[47] = x84;
}

/// The function fromBytes deserializes a field element NOT in the Montgomery domain from bytes in little-endian order.
///
/// Preconditions:
///   0 ≤ bytes_eval arg1 < m
/// Postconditions:
///   eval out1 mod m = bytes_eval arg1 mod m
///   0 ≤ eval out1 < m
///
/// Input Bounds:
///   arg1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn fromBytes(out1: *[12]u32, arg1: [48]u8) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (cast(u32, (arg1[47])) << 24);
    const x2 = (cast(u32, (arg1[46])) << 16);
    const x3 = (cast(u32, (arg1[45])) << 8);
    const x4 = (arg1[44]);
    const x5 = (cast(u32, (arg1[43])) << 24);
    const x6 = (cast(u32, (arg1[42])) << 16);
    const x7 = (cast(u32, (arg1[41])) << 8);
    const x8 = (arg1[40]);
    const x9 = (cast(u32, (arg1[39])) << 24);
    const x10 = (cast(u32, (arg1[38])) << 16);
    const x11 = (cast(u32, (arg1[37])) << 8);
    const x12 = (arg1[36]);
    const x13 = (cast(u32, (arg1[35])) << 24);
    const x14 = (cast(u32, (arg1[34])) << 16);
    const x15 = (cast(u32, (arg1[33])) << 8);
    const x16 = (arg1[32]);
    const x17 = (cast(u32, (arg1[31])) << 24);
    const x18 = (cast(u32, (arg1[30])) << 16);
    const x19 = (cast(u32, (arg1[29])) << 8);
    const x20 = (arg1[28]);
    const x21 = (cast(u32, (arg1[27])) << 24);
    const x22 = (cast(u32, (arg1[26])) << 16);
    const x23 = (cast(u32, (arg1[25])) << 8);
    const x24 = (arg1[24]);
    const x25 = (cast(u32, (arg1[23])) << 24);
    const x26 = (cast(u32, (arg1[22])) << 16);
    const x27 = (cast(u32, (arg1[21])) << 8);
    const x28 = (arg1[20]);
    const x29 = (cast(u32, (arg1[19])) << 24);
    const x30 = (cast(u32, (arg1[18])) << 16);
    const x31 = (cast(u32, (arg1[17])) << 8);
    const x32 = (arg1[16]);
    const x33 = (cast(u32, (arg1[15])) << 24);
    const x34 = (cast(u32, (arg1[14])) << 16);
    const x35 = (cast(u32, (arg1[13])) << 8);
    const x36 = (arg1[12]);
    const x37 = (cast(u32, (arg1[11])) << 24);
    const x38 = (cast(u32, (arg1[10])) << 16);
    const x39 = (cast(u32, (arg1[9])) << 8);
    const x40 = (arg1[8]);
    const x41 = (cast(u32, (arg1[7])) << 24);
    const x42 = (cast(u32, (arg1[6])) << 16);
    const x43 = (cast(u32, (arg1[5])) << 8);
    const x44 = (arg1[4]);
    const x45 = (cast(u32, (arg1[3])) << 24);
    const x46 = (cast(u32, (arg1[2])) << 16);
    const x47 = (cast(u32, (arg1[1])) << 8);
    const x48 = (arg1[0]);
    const x49 = (x47 + cast(u32, x48));
    const x50 = (x46 + x49);
    const x51 = (x45 + x50);
    const x52 = (x43 + cast(u32, x44));
    const x53 = (x42 + x52);
    const x54 = (x41 + x53);
    const x55 = (x39 + cast(u32, x40));
    const x56 = (x38 + x55);
    const x57 = (x37 + x56);
    const x58 = (x35 + cast(u32, x36));
    const x59 = (x34 + x58);
    const x60 = (x33 + x59);
    const x61 = (x31 + cast(u32, x32));
    const x62 = (x30 + x61);
    const x63 = (x29 + x62);
    const x64 = (x27 + cast(u32, x28));
    const x65 = (x26 + x64);
    const x66 = (x25 + x65);
    const x67 = (x23 + cast(u32, x24));
    const x68 = (x22 + x67);
    const x69 = (x21 + x68);
    const x70 = (x19 + cast(u32, x20));
    const x71 = (x18 + x70);
    const x72 = (x17 + x71);
    const x73 = (x15 + cast(u32, x16));
    const x74 = (x14 + x73);
    const x75 = (x13 + x74);
    const x76 = (x11 + cast(u32, x12));
    const x77 = (x10 + x76);
    const x78 = (x9 + x77);
    const x79 = (x7 + cast(u32, x8));
    const x80 = (x6 + x79);
    const x81 = (x5 + x80);
    const x82 = (x3 + cast(u32, x4));
    const x83 = (x2 + x82);
    const x84 = (x1 + x83);
    out1[0] = x51;
    out1[1] = x54;
    out1[2] = x57;
    out1[3] = x60;
    out1[4] = x63;
    out1[5] = x66;
    out1[6] = x69;
    out1[7] = x72;
    out1[8] = x75;
    out1[9] = x78;
    out1[10] = x81;
    out1[11] = x84;
}

/// The function setOne returns the field element one in the Montgomery domain.
///
/// Postconditions:
///   eval (from_montgomery out1) mod m = 1 mod m
///   0 ≤ eval out1 < m
///
pub fn setOne(out1: *MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    out1[0] = cast(u32, 0x1);
    out1[1] = 0xffffffff;
    out1[2] = 0xffffffff;
    out1[3] = cast(u32, 0x0);
    out1[4] = cast(u32, 0x1);
    out1[5] = cast(u32, 0x0);
    out1[6] = cast(u32, 0x0);
    out1[7] = cast(u32, 0x0);
    out1[8] = cast(u32, 0x0);
    out1[9] = cast(u32, 0x0);
    out1[10] = cast(u32, 0x0);
    out1[11] = cast(u32, 0x0);
}

/// The function msat returns the saturated representation of the prime modulus.
///
/// Postconditions:
///   twos_complement_eval out1 = m
///   0 ≤ eval out1 < m
///
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn msat(out1: *[13]u32) void {
    @setRuntimeSafety(mode == .Debug);

    out1[0] = 0xffffffff;
    out1[1] = cast(u32, 0x0);
    out1[2] = cast(u32, 0x0);
    out1[3] = 0xffffffff;
    out1[4] = 0xfffffffe;
    out1[5] = 0xffffffff;
    out1[6] = 0xffffffff;
    out1[7] = 0xffffffff;
    out1[8] = 0xffffffff;
    out1[9] = 0xffffffff;
    out1[10] = 0xffffffff;
    out1[11] = 0xffffffff;
    out1[12] = cast(u32, 0x0);
}

/// The function divstep computes a divstep.
///
/// Preconditions:
///   0 ≤ eval arg4 < m
///   0 ≤ eval arg5 < m
/// Postconditions:
///   out1 = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then 1 - arg1 else 1 + arg1)
///   twos_complement_eval out2 = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then twos_complement_eval arg3 else twos_complement_eval arg2)
///   twos_complement_eval out3 = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then ⌊(twos_complement_eval arg3 - twos_complement_eval arg2) / 2⌋ else ⌊(twos_complement_eval arg3 + (twos_complement_eval arg3 mod 2) * twos_complement_eval arg2) / 2⌋)
///   eval (from_montgomery out4) mod m = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then (2 * eval (from_montgomery arg5)) mod m else (2 * eval (from_montgomery arg4)) mod m)
///   eval (from_montgomery out5) mod m = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then (eval (from_montgomery arg4) - eval (from_montgomery arg4)) mod m else (eval (from_montgomery arg5) + (twos_complement_eval arg3 mod 2) * eval (from_montgomery arg4)) mod m)
///   0 ≤ eval out5 < m
///   0 ≤ eval out5 < m
///   0 ≤ eval out2 < m
///   0 ≤ eval out3 < m
///
/// Input Bounds:
///   arg1: [0x0 ~> 0xffffffff]
///   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg4: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg5: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   out3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   out4: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   out5: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn divstep(out1: *u32, out2: *[13]u32, out3: *[13]u32, out4: *[12]u32, out5: *[12]u32, arg1: u32, arg2: [13]u32, arg3: [13]u32, arg4: [12]u32, arg5: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    addcarryxU32(&x1, &x2, 0x0, (~arg1), cast(u32, 0x1));
    const x3 = (cast(u1, (x1 >> 31)) & cast(u1, ((arg3[0]) & cast(u32, 0x1))));
    var x4: u32 = undefined;
    var x5: u1 = undefined;
    addcarryxU32(&x4, &x5, 0x0, (~arg1), cast(u32, 0x1));
    var x6: u32 = undefined;
    cmovznzU32(&x6, x3, arg1, x4);
    var x7: u32 = undefined;
    cmovznzU32(&x7, x3, (arg2[0]), (arg3[0]));
    var x8: u32 = undefined;
    cmovznzU32(&x8, x3, (arg2[1]), (arg3[1]));
    var x9: u32 = undefined;
    cmovznzU32(&x9, x3, (arg2[2]), (arg3[2]));
    var x10: u32 = undefined;
    cmovznzU32(&x10, x3, (arg2[3]), (arg3[3]));
    var x11: u32 = undefined;
    cmovznzU32(&x11, x3, (arg2[4]), (arg3[4]));
    var x12: u32 = undefined;
    cmovznzU32(&x12, x3, (arg2[5]), (arg3[5]));
    var x13: u32 = undefined;
    cmovznzU32(&x13, x3, (arg2[6]), (arg3[6]));
    var x14: u32 = undefined;
    cmovznzU32(&x14, x3, (arg2[7]), (arg3[7]));
    var x15: u32 = undefined;
    cmovznzU32(&x15, x3, (arg2[8]), (arg3[8]));
    var x16: u32 = undefined;
    cmovznzU32(&x16, x3, (arg2[9]), (arg3[9]));
    var x17: u32 = undefined;
    cmovznzU32(&x17, x3, (arg2[10]), (arg3[10]));
    var x18: u32 = undefined;
    cmovznzU32(&x18, x3, (arg2[11]), (arg3[11]));
    var x19: u32 = undefined;
    cmovznzU32(&x19, x3, (arg2[12]), (arg3[12]));
    var x20: u32 = undefined;
    var x21: u1 = undefined;
    addcarryxU32(&x20, &x21, 0x0, cast(u32, 0x1), (~(arg2[0])));
    var x22: u32 = undefined;
    var x23: u1 = undefined;
    addcarryxU32(&x22, &x23, x21, cast(u32, 0x0), (~(arg2[1])));
    var x24: u32 = undefined;
    var x25: u1 = undefined;
    addcarryxU32(&x24, &x25, x23, cast(u32, 0x0), (~(arg2[2])));
    var x26: u32 = undefined;
    var x27: u1 = undefined;
    addcarryxU32(&x26, &x27, x25, cast(u32, 0x0), (~(arg2[3])));
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, x27, cast(u32, 0x0), (~(arg2[4])));
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, cast(u32, 0x0), (~(arg2[5])));
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, cast(u32, 0x0), (~(arg2[6])));
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, cast(u32, 0x0), (~(arg2[7])));
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, cast(u32, 0x0), (~(arg2[8])));
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, x37, cast(u32, 0x0), (~(arg2[9])));
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, x39, cast(u32, 0x0), (~(arg2[10])));
    var x42: u32 = undefined;
    var x43: u1 = undefined;
    addcarryxU32(&x42, &x43, x41, cast(u32, 0x0), (~(arg2[11])));
    var x44: u32 = undefined;
    var x45: u1 = undefined;
    addcarryxU32(&x44, &x45, x43, cast(u32, 0x0), (~(arg2[12])));
    var x46: u32 = undefined;
    cmovznzU32(&x46, x3, (arg3[0]), x20);
    var x47: u32 = undefined;
    cmovznzU32(&x47, x3, (arg3[1]), x22);
    var x48: u32 = undefined;
    cmovznzU32(&x48, x3, (arg3[2]), x24);
    var x49: u32 = undefined;
    cmovznzU32(&x49, x3, (arg3[3]), x26);
    var x50: u32 = undefined;
    cmovznzU32(&x50, x3, (arg3[4]), x28);
    var x51: u32 = undefined;
    cmovznzU32(&x51, x3, (arg3[5]), x30);
    var x52: u32 = undefined;
    cmovznzU32(&x52, x3, (arg3[6]), x32);
    var x53: u32 = undefined;
    cmovznzU32(&x53, x3, (arg3[7]), x34);
    var x54: u32 = undefined;
    cmovznzU32(&x54, x3, (arg3[8]), x36);
    var x55: u32 = undefined;
    cmovznzU32(&x55, x3, (arg3[9]), x38);
    var x56: u32 = undefined;
    cmovznzU32(&x56, x3, (arg3[10]), x40);
    var x57: u32 = undefined;
    cmovznzU32(&x57, x3, (arg3[11]), x42);
    var x58: u32 = undefined;
    cmovznzU32(&x58, x3, (arg3[12]), x44);
    var x59: u32 = undefined;
    cmovznzU32(&x59, x3, (arg4[0]), (arg5[0]));
    var x60: u32 = undefined;
    cmovznzU32(&x60, x3, (arg4[1]), (arg5[1]));
    var x61: u32 = undefined;
    cmovznzU32(&x61, x3, (arg4[2]), (arg5[2]));
    var x62: u32 = undefined;
    cmovznzU32(&x62, x3, (arg4[3]), (arg5[3]));
    var x63: u32 = undefined;
    cmovznzU32(&x63, x3, (arg4[4]), (arg5[4]));
    var x64: u32 = undefined;
    cmovznzU32(&x64, x3, (arg4[5]), (arg5[5]));
    var x65: u32 = undefined;
    cmovznzU32(&x65, x3, (arg4[6]), (arg5[6]));
    var x66: u32 = undefined;
    cmovznzU32(&x66, x3, (arg4[7]), (arg5[7]));
    var x67: u32 = undefined;
    cmovznzU32(&x67, x3, (arg4[8]), (arg5[8]));
    var x68: u32 = undefined;
    cmovznzU32(&x68, x3, (arg4[9]), (arg5[9]));
    var x69: u32 = undefined;
    cmovznzU32(&x69, x3, (arg4[10]), (arg5[10]));
    var x70: u32 = undefined;
    cmovznzU32(&x70, x3, (arg4[11]), (arg5[11]));
    var x71: u32 = undefined;
    var x72: u1 = undefined;
    addcarryxU32(&x71, &x72, 0x0, x59, x59);
    var x73: u32 = undefined;
    var x74: u1 = undefined;
    addcarryxU32(&x73, &x74, x72, x60, x60);
    var x75: u32 = undefined;
    var x76: u1 = undefined;
    addcarryxU32(&x75, &x76, x74, x61, x61);
    var x77: u32 = undefined;
    var x78: u1 = undefined;
    addcarryxU32(&x77, &x78, x76, x62, x62);
    var x79: u32 = undefined;
    var x80: u1 = undefined;
    addcarryxU32(&x79, &x80, x78, x63, x63);
    var x81: u32 = undefined;
    var x82: u1 = undefined;
    addcarryxU32(&x81, &x82, x80, x64, x64);
    var x83: u32 = undefined;
    var x84: u1 = undefined;
    addcarryxU32(&x83, &x84, x82, x65, x65);
    var x85: u32 = undefined;
    var x86: u1 = undefined;
    addcarryxU32(&x85, &x86, x84, x66, x66);
    var x87: u32 = undefined;
    var x88: u1 = undefined;
    addcarryxU32(&x87, &x88, x86, x67, x67);
    var x89: u32 = undefined;
    var x90: u1 = undefined;
    addcarryxU32(&x89, &x90, x88, x68, x68);
    var x91: u32 = undefined;
    var x92: u1 = undefined;
    addcarryxU32(&x91, &x92, x90, x69, x69);
    var x93: u32 = undefined;
    var x94: u1 = undefined;
    addcarryxU32(&x93, &x94, x92, x70, x70);
    var x95: u32 = undefined;
    var x96: u1 = undefined;
    subborrowxU32(&x95, &x96, 0x0, x71, 0xffffffff);
    var x97: u32 = undefined;
    var x98: u1 = undefined;
    subborrowxU32(&x97, &x98, x96, x73, cast(u32, 0x0));
    var x99: u32 = undefined;
    var x100: u1 = undefined;
    subborrowxU32(&x99, &x100, x98, x75, cast(u32, 0x0));
    var x101: u32 = undefined;
    var x102: u1 = undefined;
    subborrowxU32(&x101, &x102, x100, x77, 0xffffffff);
    var x103: u32 = undefined;
    var x104: u1 = undefined;
    subborrowxU32(&x103, &x104, x102, x79, 0xfffffffe);
    var x105: u32 = undefined;
    var x106: u1 = undefined;
    subborrowxU32(&x105, &x106, x104, x81, 0xffffffff);
    var x107: u32 = undefined;
    var x108: u1 = undefined;
    subborrowxU32(&x107, &x108, x106, x83, 0xffffffff);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    subborrowxU32(&x109, &x110, x108, x85, 0xffffffff);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    subborrowxU32(&x111, &x112, x110, x87, 0xffffffff);
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    subborrowxU32(&x113, &x114, x112, x89, 0xffffffff);
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    subborrowxU32(&x115, &x116, x114, x91, 0xffffffff);
    var x117: u32 = undefined;
    var x118: u1 = undefined;
    subborrowxU32(&x117, &x118, x116, x93, 0xffffffff);
    var x119: u32 = undefined;
    var x120: u1 = undefined;
    subborrowxU32(&x119, &x120, x118, cast(u32, x94), cast(u32, 0x0));
    const x121 = (arg4[11]);
    const x122 = (arg4[10]);
    const x123 = (arg4[9]);
    const x124 = (arg4[8]);
    const x125 = (arg4[7]);
    const x126 = (arg4[6]);
    const x127 = (arg4[5]);
    const x128 = (arg4[4]);
    const x129 = (arg4[3]);
    const x130 = (arg4[2]);
    const x131 = (arg4[1]);
    const x132 = (arg4[0]);
    var x133: u32 = undefined;
    var x134: u1 = undefined;
    subborrowxU32(&x133, &x134, 0x0, cast(u32, 0x0), x132);
    var x135: u32 = undefined;
    var x136: u1 = undefined;
    subborrowxU32(&x135, &x136, x134, cast(u32, 0x0), x131);
    var x137: u32 = undefined;
    var x138: u1 = undefined;
    subborrowxU32(&x137, &x138, x136, cast(u32, 0x0), x130);
    var x139: u32 = undefined;
    var x140: u1 = undefined;
    subborrowxU32(&x139, &x140, x138, cast(u32, 0x0), x129);
    var x141: u32 = undefined;
    var x142: u1 = undefined;
    subborrowxU32(&x141, &x142, x140, cast(u32, 0x0), x128);
    var x143: u32 = undefined;
    var x144: u1 = undefined;
    subborrowxU32(&x143, &x144, x142, cast(u32, 0x0), x127);
    var x145: u32 = undefined;
    var x146: u1 = undefined;
    subborrowxU32(&x145, &x146, x144, cast(u32, 0x0), x126);
    var x147: u32 = undefined;
    var x148: u1 = undefined;
    subborrowxU32(&x147, &x148, x146, cast(u32, 0x0), x125);
    var x149: u32 = undefined;
    var x150: u1 = undefined;
    subborrowxU32(&x149, &x150, x148, cast(u32, 0x0), x124);
    var x151: u32 = undefined;
    var x152: u1 = undefined;
    subborrowxU32(&x151, &x152, x150, cast(u32, 0x0), x123);
    var x153: u32 = undefined;
    var x154: u1 = undefined;
    subborrowxU32(&x153, &x154, x152, cast(u32, 0x0), x122);
    var x155: u32 = undefined;
    var x156: u1 = undefined;
    subborrowxU32(&x155, &x156, x154, cast(u32, 0x0), x121);
    var x157: u32 = undefined;
    cmovznzU32(&x157, x156, cast(u32, 0x0), 0xffffffff);
    var x158: u32 = undefined;
    var x159: u1 = undefined;
    addcarryxU32(&x158, &x159, 0x0, x133, x157);
    var x160: u32 = undefined;
    var x161: u1 = undefined;
    addcarryxU32(&x160, &x161, x159, x135, cast(u32, 0x0));
    var x162: u32 = undefined;
    var x163: u1 = undefined;
    addcarryxU32(&x162, &x163, x161, x137, cast(u32, 0x0));
    var x164: u32 = undefined;
    var x165: u1 = undefined;
    addcarryxU32(&x164, &x165, x163, x139, x157);
    var x166: u32 = undefined;
    var x167: u1 = undefined;
    addcarryxU32(&x166, &x167, x165, x141, (x157 & 0xfffffffe));
    var x168: u32 = undefined;
    var x169: u1 = undefined;
    addcarryxU32(&x168, &x169, x167, x143, x157);
    var x170: u32 = undefined;
    var x171: u1 = undefined;
    addcarryxU32(&x170, &x171, x169, x145, x157);
    var x172: u32 = undefined;
    var x173: u1 = undefined;
    addcarryxU32(&x172, &x173, x171, x147, x157);
    var x174: u32 = undefined;
    var x175: u1 = undefined;
    addcarryxU32(&x174, &x175, x173, x149, x157);
    var x176: u32 = undefined;
    var x177: u1 = undefined;
    addcarryxU32(&x176, &x177, x175, x151, x157);
    var x178: u32 = undefined;
    var x179: u1 = undefined;
    addcarryxU32(&x178, &x179, x177, x153, x157);
    var x180: u32 = undefined;
    var x181: u1 = undefined;
    addcarryxU32(&x180, &x181, x179, x155, x157);
    var x182: u32 = undefined;
    cmovznzU32(&x182, x3, (arg5[0]), x158);
    var x183: u32 = undefined;
    cmovznzU32(&x183, x3, (arg5[1]), x160);
    var x184: u32 = undefined;
    cmovznzU32(&x184, x3, (arg5[2]), x162);
    var x185: u32 = undefined;
    cmovznzU32(&x185, x3, (arg5[3]), x164);
    var x186: u32 = undefined;
    cmovznzU32(&x186, x3, (arg5[4]), x166);
    var x187: u32 = undefined;
    cmovznzU32(&x187, x3, (arg5[5]), x168);
    var x188: u32 = undefined;
    cmovznzU32(&x188, x3, (arg5[6]), x170);
    var x189: u32 = undefined;
    cmovznzU32(&x189, x3, (arg5[7]), x172);
    var x190: u32 = undefined;
    cmovznzU32(&x190, x3, (arg5[8]), x174);
    var x191: u32 = undefined;
    cmovznzU32(&x191, x3, (arg5[9]), x176);
    var x192: u32 = undefined;
    cmovznzU32(&x192, x3, (arg5[10]), x178);
    var x193: u32 = undefined;
    cmovznzU32(&x193, x3, (arg5[11]), x180);
    const x194 = cast(u1, (x46 & cast(u32, 0x1)));
    var x195: u32 = undefined;
    cmovznzU32(&x195, x194, cast(u32, 0x0), x7);
    var x196: u32 = undefined;
    cmovznzU32(&x196, x194, cast(u32, 0x0), x8);
    var x197: u32 = undefined;
    cmovznzU32(&x197, x194, cast(u32, 0x0), x9);
    var x198: u32 = undefined;
    cmovznzU32(&x198, x194, cast(u32, 0x0), x10);
    var x199: u32 = undefined;
    cmovznzU32(&x199, x194, cast(u32, 0x0), x11);
    var x200: u32 = undefined;
    cmovznzU32(&x200, x194, cast(u32, 0x0), x12);
    var x201: u32 = undefined;
    cmovznzU32(&x201, x194, cast(u32, 0x0), x13);
    var x202: u32 = undefined;
    cmovznzU32(&x202, x194, cast(u32, 0x0), x14);
    var x203: u32 = undefined;
    cmovznzU32(&x203, x194, cast(u32, 0x0), x15);
    var x204: u32 = undefined;
    cmovznzU32(&x204, x194, cast(u32, 0x0), x16);
    var x205: u32 = undefined;
    cmovznzU32(&x205, x194, cast(u32, 0x0), x17);
    var x206: u32 = undefined;
    cmovznzU32(&x206, x194, cast(u32, 0x0), x18);
    var x207: u32 = undefined;
    cmovznzU32(&x207, x194, cast(u32, 0x0), x19);
    var x208: u32 = undefined;
    var x209: u1 = undefined;
    addcarryxU32(&x208, &x209, 0x0, x46, x195);
    var x210: u32 = undefined;
    var x211: u1 = undefined;
    addcarryxU32(&x210, &x211, x209, x47, x196);
    var x212: u32 = undefined;
    var x213: u1 = undefined;
    addcarryxU32(&x212, &x213, x211, x48, x197);
    var x214: u32 = undefined;
    var x215: u1 = undefined;
    addcarryxU32(&x214, &x215, x213, x49, x198);
    var x216: u32 = undefined;
    var x217: u1 = undefined;
    addcarryxU32(&x216, &x217, x215, x50, x199);
    var x218: u32 = undefined;
    var x219: u1 = undefined;
    addcarryxU32(&x218, &x219, x217, x51, x200);
    var x220: u32 = undefined;
    var x221: u1 = undefined;
    addcarryxU32(&x220, &x221, x219, x52, x201);
    var x222: u32 = undefined;
    var x223: u1 = undefined;
    addcarryxU32(&x222, &x223, x221, x53, x202);
    var x224: u32 = undefined;
    var x225: u1 = undefined;
    addcarryxU32(&x224, &x225, x223, x54, x203);
    var x226: u32 = undefined;
    var x227: u1 = undefined;
    addcarryxU32(&x226, &x227, x225, x55, x204);
    var x228: u32 = undefined;
    var x229: u1 = undefined;
    addcarryxU32(&x228, &x229, x227, x56, x205);
    var x230: u32 = undefined;
    var x231: u1 = undefined;
    addcarryxU32(&x230, &x231, x229, x57, x206);
    var x232: u32 = undefined;
    var x233: u1 = undefined;
    addcarryxU32(&x232, &x233, x231, x58, x207);
    var x234: u32 = undefined;
    cmovznzU32(&x234, x194, cast(u32, 0x0), x59);
    var x235: u32 = undefined;
    cmovznzU32(&x235, x194, cast(u32, 0x0), x60);
    var x236: u32 = undefined;
    cmovznzU32(&x236, x194, cast(u32, 0x0), x61);
    var x237: u32 = undefined;
    cmovznzU32(&x237, x194, cast(u32, 0x0), x62);
    var x238: u32 = undefined;
    cmovznzU32(&x238, x194, cast(u32, 0x0), x63);
    var x239: u32 = undefined;
    cmovznzU32(&x239, x194, cast(u32, 0x0), x64);
    var x240: u32 = undefined;
    cmovznzU32(&x240, x194, cast(u32, 0x0), x65);
    var x241: u32 = undefined;
    cmovznzU32(&x241, x194, cast(u32, 0x0), x66);
    var x242: u32 = undefined;
    cmovznzU32(&x242, x194, cast(u32, 0x0), x67);
    var x243: u32 = undefined;
    cmovznzU32(&x243, x194, cast(u32, 0x0), x68);
    var x244: u32 = undefined;
    cmovznzU32(&x244, x194, cast(u32, 0x0), x69);
    var x245: u32 = undefined;
    cmovznzU32(&x245, x194, cast(u32, 0x0), x70);
    var x246: u32 = undefined;
    var x247: u1 = undefined;
    addcarryxU32(&x246, &x247, 0x0, x182, x234);
    var x248: u32 = undefined;
    var x249: u1 = undefined;
    addcarryxU32(&x248, &x249, x247, x183, x235);
    var x250: u32 = undefined;
    var x251: u1 = undefined;
    addcarryxU32(&x250, &x251, x249, x184, x236);
    var x252: u32 = undefined;
    var x253: u1 = undefined;
    addcarryxU32(&x252, &x253, x251, x185, x237);
    var x254: u32 = undefined;
    var x255: u1 = undefined;
    addcarryxU32(&x254, &x255, x253, x186, x238);
    var x256: u32 = undefined;
    var x257: u1 = undefined;
    addcarryxU32(&x256, &x257, x255, x187, x239);
    var x258: u32 = undefined;
    var x259: u1 = undefined;
    addcarryxU32(&x258, &x259, x257, x188, x240);
    var x260: u32 = undefined;
    var x261: u1 = undefined;
    addcarryxU32(&x260, &x261, x259, x189, x241);
    var x262: u32 = undefined;
    var x263: u1 = undefined;
    addcarryxU32(&x262, &x263, x261, x190, x242);
    var x264: u32 = undefined;
    var x265: u1 = undefined;
    addcarryxU32(&x264, &x265, x263, x191, x243);
    var x266: u32 = undefined;
    var x267: u1 = undefined;
    addcarryxU32(&x266, &x267, x265, x192, x244);
    var x268: u32 = undefined;
    var x269: u1 = undefined;
    addcarryxU32(&x268, &x269, x267, x193, x245);
    var x270: u32 = undefined;
    var x271: u1 = undefined;
    subborrowxU32(&x270, &x271, 0x0, x246, 0xffffffff);
    var x272: u32 = undefined;
    var x273: u1 = undefined;
    subborrowxU32(&x272, &x273, x271, x248, cast(u32, 0x0));
    var x274: u32 = undefined;
    var x275: u1 = undefined;
    subborrowxU32(&x274, &x275, x273, x250, cast(u32, 0x0));
    var x276: u32 = undefined;
    var x277: u1 = undefined;
    subborrowxU32(&x276, &x277, x275, x252, 0xffffffff);
    var x278: u32 = undefined;
    var x279: u1 = undefined;
    subborrowxU32(&x278, &x279, x277, x254, 0xfffffffe);
    var x280: u32 = undefined;
    var x281: u1 = undefined;
    subborrowxU32(&x280, &x281, x279, x256, 0xffffffff);
    var x282: u32 = undefined;
    var x283: u1 = undefined;
    subborrowxU32(&x282, &x283, x281, x258, 0xffffffff);
    var x284: u32 = undefined;
    var x285: u1 = undefined;
    subborrowxU32(&x284, &x285, x283, x260, 0xffffffff);
    var x286: u32 = undefined;
    var x287: u1 = undefined;
    subborrowxU32(&x286, &x287, x285, x262, 0xffffffff);
    var x288: u32 = undefined;
    var x289: u1 = undefined;
    subborrowxU32(&x288, &x289, x287, x264, 0xffffffff);
    var x290: u32 = undefined;
    var x291: u1 = undefined;
    subborrowxU32(&x290, &x291, x289, x266, 0xffffffff);
    var x292: u32 = undefined;
    var x293: u1 = undefined;
    subborrowxU32(&x292, &x293, x291, x268, 0xffffffff);
    var x294: u32 = undefined;
    var x295: u1 = undefined;
    subborrowxU32(&x294, &x295, x293, cast(u32, x269), cast(u32, 0x0));
    var x296: u32 = undefined;
    var x297: u1 = undefined;
    addcarryxU32(&x296, &x297, 0x0, x6, cast(u32, 0x1));
    const x298 = ((x208 >> 1) | ((x210 << 31) & 0xffffffff));
    const x299 = ((x210 >> 1) | ((x212 << 31) & 0xffffffff));
    const x300 = ((x212 >> 1) | ((x214 << 31) & 0xffffffff));
    const x301 = ((x214 >> 1) | ((x216 << 31) & 0xffffffff));
    const x302 = ((x216 >> 1) | ((x218 << 31) & 0xffffffff));
    const x303 = ((x218 >> 1) | ((x220 << 31) & 0xffffffff));
    const x304 = ((x220 >> 1) | ((x222 << 31) & 0xffffffff));
    const x305 = ((x222 >> 1) | ((x224 << 31) & 0xffffffff));
    const x306 = ((x224 >> 1) | ((x226 << 31) & 0xffffffff));
    const x307 = ((x226 >> 1) | ((x228 << 31) & 0xffffffff));
    const x308 = ((x228 >> 1) | ((x230 << 31) & 0xffffffff));
    const x309 = ((x230 >> 1) | ((x232 << 31) & 0xffffffff));
    const x310 = ((x232 & 0x80000000) | (x232 >> 1));
    var x311: u32 = undefined;
    cmovznzU32(&x311, x120, x95, x71);
    var x312: u32 = undefined;
    cmovznzU32(&x312, x120, x97, x73);
    var x313: u32 = undefined;
    cmovznzU32(&x313, x120, x99, x75);
    var x314: u32 = undefined;
    cmovznzU32(&x314, x120, x101, x77);
    var x315: u32 = undefined;
    cmovznzU32(&x315, x120, x103, x79);
    var x316: u32 = undefined;
    cmovznzU32(&x316, x120, x105, x81);
    var x317: u32 = undefined;
    cmovznzU32(&x317, x120, x107, x83);
    var x318: u32 = undefined;
    cmovznzU32(&x318, x120, x109, x85);
    var x319: u32 = undefined;
    cmovznzU32(&x319, x120, x111, x87);
    var x320: u32 = undefined;
    cmovznzU32(&x320, x120, x113, x89);
    var x321: u32 = undefined;
    cmovznzU32(&x321, x120, x115, x91);
    var x322: u32 = undefined;
    cmovznzU32(&x322, x120, x117, x93);
    var x323: u32 = undefined;
    cmovznzU32(&x323, x295, x270, x246);
    var x324: u32 = undefined;
    cmovznzU32(&x324, x295, x272, x248);
    var x325: u32 = undefined;
    cmovznzU32(&x325, x295, x274, x250);
    var x326: u32 = undefined;
    cmovznzU32(&x326, x295, x276, x252);
    var x327: u32 = undefined;
    cmovznzU32(&x327, x295, x278, x254);
    var x328: u32 = undefined;
    cmovznzU32(&x328, x295, x280, x256);
    var x329: u32 = undefined;
    cmovznzU32(&x329, x295, x282, x258);
    var x330: u32 = undefined;
    cmovznzU32(&x330, x295, x284, x260);
    var x331: u32 = undefined;
    cmovznzU32(&x331, x295, x286, x262);
    var x332: u32 = undefined;
    cmovznzU32(&x332, x295, x288, x264);
    var x333: u32 = undefined;
    cmovznzU32(&x333, x295, x290, x266);
    var x334: u32 = undefined;
    cmovznzU32(&x334, x295, x292, x268);
    out1.* = x296;
    out2[0] = x7;
    out2[1] = x8;
    out2[2] = x9;
    out2[3] = x10;
    out2[4] = x11;
    out2[5] = x12;
    out2[6] = x13;
    out2[7] = x14;
    out2[8] = x15;
    out2[9] = x16;
    out2[10] = x17;
    out2[11] = x18;
    out2[12] = x19;
    out3[0] = x298;
    out3[1] = x299;
    out3[2] = x300;
    out3[3] = x301;
    out3[4] = x302;
    out3[5] = x303;
    out3[6] = x304;
    out3[7] = x305;
    out3[8] = x306;
    out3[9] = x307;
    out3[10] = x308;
    out3[11] = x309;
    out3[12] = x310;
    out4[0] = x311;
    out4[1] = x312;
    out4[2] = x313;
    out4[3] = x314;
    out4[4] = x315;
    out4[5] = x316;
    out4[6] = x317;
    out4[7] = x318;
    out4[8] = x319;
    out4[9] = x320;
    out4[10] = x321;
    out4[11] = x322;
    out5[0] = x323;
    out5[1] = x324;
    out5[2] = x325;
    out5[3] = x326;
    out5[4] = x327;
    out5[5] = x328;
    out5[6] = x329;
    out5[7] = x330;
    out5[8] = x331;
    out5[9] = x332;
    out5[10] = x333;
    out5[11] = x334;
}

/// The function divstepPrecomp returns the precomputed value for Bernstein-Yang-inversion (in montgomery form).
///
/// Postconditions:
///   eval (from_montgomery out1) = ⌊(m - 1) / 2⌋^(if ⌊log2 m⌋ + 1 < 46 then ⌊(49 * (⌊log2 m⌋ + 1) + 80) / 17⌋ else ⌊(49 * (⌊log2 m⌋ + 1) + 57) / 17⌋)
///   0 ≤ eval out1 < m
///
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn divstepPrecomp(out1: *[12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    out1[0] = 0xfff18fff;
    out1[1] = 0xfff69400;
    out1[2] = 0xffffd3ff;
    out1[3] = 0x2b7fe;
    out1[4] = 0xfffe97ff;
    out1[5] = 0xfffedbff;
    out1[6] = 0x2fff;
    out1[7] = 0x28400;
    out1[8] = 0x50400;
    out1[9] = 0x60400;
    out1[10] = 0x38000;
    out1[11] = 0xfffc4800;
}
