// Autogenerated: 'src/ExtractionOCaml/word_by_word_montgomery' --lang Zig --internal-static --public-function-case camelCase --private-function-case camelCase --public-type-case UpperCamelCase --private-type-case UpperCamelCase --no-prefix-fiat --package-name p384_scalar '' 32 '2^384 - 1388124618062372383947042015309946732620727252194336364173' mul square add sub opp from_montgomery to_montgomery nonzero selectznz to_bytes from_bytes one msat divstep divstep_precomp
// curve description (via package name): p384_scalar
// machine_wordsize = 32 (from "32")
// requested operations: mul, square, add, sub, opp, from_montgomery, to_montgomery, nonzero, selectznz, to_bytes, from_bytes, one, msat, divstep, divstep_precomp
// m = 0xffffffffffffffffffffffffffffffffffffffffffffffffc7634d81f4372ddf581a0db248b0a77aecec196accc52973 (from "2^384 - 1388124618062372383947042015309946732620727252194336364173")
//
// NOTE: In addition to the bounds specified above each function, all
//   functions synthesized for this Montgomery arithmetic require the
//   input to be strictly less than the prime modulus (m), and also
//   require the input to be in the unique saturated representation.
//   All functions also ensure that these two properties are true of
//   return values.
//
// Computed values:
//   eval z = z[0] + (z[1] << 32) + (z[2] << 64) + (z[3] << 96) + (z[4] << 128) + (z[5] << 160) + (z[6] << 192) + (z[7] << 224) + (z[8] << 256) + (z[9] << 0x120) + (z[10] << 0x140) + (z[11] << 0x160)
//   bytes_eval z = z[0] + (z[1] << 8) + (z[2] << 16) + (z[3] << 24) + (z[4] << 32) + (z[5] << 40) + (z[6] << 48) + (z[7] << 56) + (z[8] << 64) + (z[9] << 72) + (z[10] << 80) + (z[11] << 88) + (z[12] << 96) + (z[13] << 104) + (z[14] << 112) + (z[15] << 120) + (z[16] << 128) + (z[17] << 136) + (z[18] << 144) + (z[19] << 152) + (z[20] << 160) + (z[21] << 168) + (z[22] << 176) + (z[23] << 184) + (z[24] << 192) + (z[25] << 200) + (z[26] << 208) + (z[27] << 216) + (z[28] << 224) + (z[29] << 232) + (z[30] << 240) + (z[31] << 248) + (z[32] << 256) + (z[33] << 0x108) + (z[34] << 0x110) + (z[35] << 0x118) + (z[36] << 0x120) + (z[37] << 0x128) + (z[38] << 0x130) + (z[39] << 0x138) + (z[40] << 0x140) + (z[41] << 0x148) + (z[42] << 0x150) + (z[43] << 0x158) + (z[44] << 0x160) + (z[45] << 0x168) + (z[46] << 0x170) + (z[47] << 0x178)
//   twos_complement_eval z = let x1 := z[0] + (z[1] << 32) + (z[2] << 64) + (z[3] << 96) + (z[4] << 128) + (z[5] << 160) + (z[6] << 192) + (z[7] << 224) + (z[8] << 256) + (z[9] << 0x120) + (z[10] << 0x140) + (z[11] << 0x160) in
//                            if x1 & (2^384-1) < 2^383 then x1 & (2^384-1) else (x1 & (2^384-1)) - 2^384

const std = @import("std");
const mode = @import("builtin").mode; // Checked arithmetic is disabled in non-debug modes to avoid side channels

inline fn cast(comptime DestType: type, target: anytype) DestType {
    @setEvalBranchQuota(10000);
    if (@typeInfo(@TypeOf(target)) == .int) {
        const dest = @typeInfo(DestType).int;
        const source = @typeInfo(@TypeOf(target)).int;
        if (dest.bits < source.bits) {
            const T = std.meta.Int(source.signedness, dest.bits);
            return @bitCast(@as(T, @truncate(target)));
        }
    }
    return target;
}

// The type MontgomeryDomainFieldElement is a field element in the Montgomery domain.
// Bounds: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub const MontgomeryDomainFieldElement = [12]u32;

// The type NonMontgomeryDomainFieldElement is a field element NOT in the Montgomery domain.
// Bounds: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub const NonMontgomeryDomainFieldElement = [12]u32;

/// The function addcarryxU32 is an addition with carry.
///
/// Postconditions:
///   out1 = (arg1 + arg2 + arg3) mod 2^32
///   out2 = ⌊(arg1 + arg2 + arg3) / 2^32⌋
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [0x0 ~> 0xffffffff]
///   arg3: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [0x0 ~> 0x1]
inline fn addcarryxU32(out1: *u32, out2: *u1, arg1: u1, arg2: u32, arg3: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = ((cast(u64, arg1) + cast(u64, arg2)) + cast(u64, arg3));
    const x2 = cast(u32, (x1 & cast(u64, 0xffffffff)));
    const x3 = cast(u1, (x1 >> 32));
    out1.* = x2;
    out2.* = x3;
}

/// The function subborrowxU32 is a subtraction with borrow.
///
/// Postconditions:
///   out1 = (-arg1 + arg2 + -arg3) mod 2^32
///   out2 = -⌊(-arg1 + arg2 + -arg3) / 2^32⌋
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [0x0 ~> 0xffffffff]
///   arg3: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [0x0 ~> 0x1]
inline fn subborrowxU32(out1: *u32, out2: *u1, arg1: u1, arg2: u32, arg3: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = ((cast(i64, arg2) - cast(i64, arg1)) - cast(i64, arg3));
    const x2 = cast(i1, (x1 >> 32));
    const x3 = cast(u32, (x1 & cast(i64, 0xffffffff)));
    out1.* = x3;
    out2.* = cast(u1, (cast(i2, 0x0) - cast(i2, x2)));
}

/// The function mulxU32 is a multiplication, returning the full double-width result.
///
/// Postconditions:
///   out1 = (arg1 * arg2) mod 2^32
///   out2 = ⌊arg1 * arg2 / 2^32⌋
///
/// Input Bounds:
///   arg1: [0x0 ~> 0xffffffff]
///   arg2: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [0x0 ~> 0xffffffff]
inline fn mulxU32(out1: *u32, out2: *u32, arg1: u32, arg2: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (cast(u64, arg1) * cast(u64, arg2));
    const x2 = cast(u32, (x1 & cast(u64, 0xffffffff)));
    const x3 = cast(u32, (x1 >> 32));
    out1.* = x2;
    out2.* = x3;
}

/// The function cmovznzU32 is a single-word conditional move.
///
/// Postconditions:
///   out1 = (if arg1 = 0 then arg2 else arg3)
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [0x0 ~> 0xffffffff]
///   arg3: [0x0 ~> 0xffffffff]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
inline fn cmovznzU32(out1: *u32, arg1: u1, arg2: u32, arg3: u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (~(~arg1));
    const x2 = cast(u32, (cast(i64, cast(i1, (cast(i2, 0x0) - cast(i2, x1)))) & cast(i64, 0xffffffff)));
    const x3 = ((x2 & arg3) | ((~x2) & arg2));
    out1.* = x3;
}

/// The function mul multiplies two field elements in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
///   0 ≤ eval arg2 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) * eval (from_montgomery arg2)) mod m
///   0 ≤ eval out1 < m
///
pub fn mul(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement, arg2: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[1]);
    const x2 = (arg1[2]);
    const x3 = (arg1[3]);
    const x4 = (arg1[4]);
    const x5 = (arg1[5]);
    const x6 = (arg1[6]);
    const x7 = (arg1[7]);
    const x8 = (arg1[8]);
    const x9 = (arg1[9]);
    const x10 = (arg1[10]);
    const x11 = (arg1[11]);
    const x12 = (arg1[0]);
    var x13: u32 = undefined;
    var x14: u32 = undefined;
    mulxU32(&x13, &x14, x12, (arg2[11]));
    var x15: u32 = undefined;
    var x16: u32 = undefined;
    mulxU32(&x15, &x16, x12, (arg2[10]));
    var x17: u32 = undefined;
    var x18: u32 = undefined;
    mulxU32(&x17, &x18, x12, (arg2[9]));
    var x19: u32 = undefined;
    var x20: u32 = undefined;
    mulxU32(&x19, &x20, x12, (arg2[8]));
    var x21: u32 = undefined;
    var x22: u32 = undefined;
    mulxU32(&x21, &x22, x12, (arg2[7]));
    var x23: u32 = undefined;
    var x24: u32 = undefined;
    mulxU32(&x23, &x24, x12, (arg2[6]));
    var x25: u32 = undefined;
    var x26: u32 = undefined;
    mulxU32(&x25, &x26, x12, (arg2[5]));
    var x27: u32 = undefined;
    var x28: u32 = undefined;
    mulxU32(&x27, &x28, x12, (arg2[4]));
    var x29: u32 = undefined;
    var x30: u32 = undefined;
    mulxU32(&x29, &x30, x12, (arg2[3]));
    var x31: u32 = undefined;
    var x32: u32 = undefined;
    mulxU32(&x31, &x32, x12, (arg2[2]));
    var x33: u32 = undefined;
    var x34: u32 = undefined;
    mulxU32(&x33, &x34, x12, (arg2[1]));
    var x35: u32 = undefined;
    var x36: u32 = undefined;
    mulxU32(&x35, &x36, x12, (arg2[0]));
    var x37: u32 = undefined;
    var x38: u1 = undefined;
    addcarryxU32(&x37, &x38, 0x0, x36, x33);
    var x39: u32 = undefined;
    var x40: u1 = undefined;
    addcarryxU32(&x39, &x40, x38, x34, x31);
    var x41: u32 = undefined;
    var x42: u1 = undefined;
    addcarryxU32(&x41, &x42, x40, x32, x29);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    addcarryxU32(&x43, &x44, x42, x30, x27);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    addcarryxU32(&x45, &x46, x44, x28, x25);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    addcarryxU32(&x47, &x48, x46, x26, x23);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    addcarryxU32(&x49, &x50, x48, x24, x21);
    var x51: u32 = undefined;
    var x52: u1 = undefined;
    addcarryxU32(&x51, &x52, x50, x22, x19);
    var x53: u32 = undefined;
    var x54: u1 = undefined;
    addcarryxU32(&x53, &x54, x52, x20, x17);
    var x55: u32 = undefined;
    var x56: u1 = undefined;
    addcarryxU32(&x55, &x56, x54, x18, x15);
    var x57: u32 = undefined;
    var x58: u1 = undefined;
    addcarryxU32(&x57, &x58, x56, x16, x13);
    const x59 = (cast(u32, x58) + x14);
    var x60: u32 = undefined;
    var x61: u32 = undefined;
    mulxU32(&x60, &x61, x35, 0xe88fdc45);
    var x62: u32 = undefined;
    var x63: u32 = undefined;
    mulxU32(&x62, &x63, x60, 0xffffffff);
    var x64: u32 = undefined;
    var x65: u32 = undefined;
    mulxU32(&x64, &x65, x60, 0xffffffff);
    var x66: u32 = undefined;
    var x67: u32 = undefined;
    mulxU32(&x66, &x67, x60, 0xffffffff);
    var x68: u32 = undefined;
    var x69: u32 = undefined;
    mulxU32(&x68, &x69, x60, 0xffffffff);
    var x70: u32 = undefined;
    var x71: u32 = undefined;
    mulxU32(&x70, &x71, x60, 0xffffffff);
    var x72: u32 = undefined;
    var x73: u32 = undefined;
    mulxU32(&x72, &x73, x60, 0xffffffff);
    var x74: u32 = undefined;
    var x75: u32 = undefined;
    mulxU32(&x74, &x75, x60, 0xc7634d81);
    var x76: u32 = undefined;
    var x77: u32 = undefined;
    mulxU32(&x76, &x77, x60, 0xf4372ddf);
    var x78: u32 = undefined;
    var x79: u32 = undefined;
    mulxU32(&x78, &x79, x60, 0x581a0db2);
    var x80: u32 = undefined;
    var x81: u32 = undefined;
    mulxU32(&x80, &x81, x60, 0x48b0a77a);
    var x82: u32 = undefined;
    var x83: u32 = undefined;
    mulxU32(&x82, &x83, x60, 0xecec196a);
    var x84: u32 = undefined;
    var x85: u32 = undefined;
    mulxU32(&x84, &x85, x60, 0xccc52973);
    var x86: u32 = undefined;
    var x87: u1 = undefined;
    addcarryxU32(&x86, &x87, 0x0, x85, x82);
    var x88: u32 = undefined;
    var x89: u1 = undefined;
    addcarryxU32(&x88, &x89, x87, x83, x80);
    var x90: u32 = undefined;
    var x91: u1 = undefined;
    addcarryxU32(&x90, &x91, x89, x81, x78);
    var x92: u32 = undefined;
    var x93: u1 = undefined;
    addcarryxU32(&x92, &x93, x91, x79, x76);
    var x94: u32 = undefined;
    var x95: u1 = undefined;
    addcarryxU32(&x94, &x95, x93, x77, x74);
    var x96: u32 = undefined;
    var x97: u1 = undefined;
    addcarryxU32(&x96, &x97, x95, x75, x72);
    var x98: u32 = undefined;
    var x99: u1 = undefined;
    addcarryxU32(&x98, &x99, x97, x73, x70);
    var x100: u32 = undefined;
    var x101: u1 = undefined;
    addcarryxU32(&x100, &x101, x99, x71, x68);
    var x102: u32 = undefined;
    var x103: u1 = undefined;
    addcarryxU32(&x102, &x103, x101, x69, x66);
    var x104: u32 = undefined;
    var x105: u1 = undefined;
    addcarryxU32(&x104, &x105, x103, x67, x64);
    var x106: u32 = undefined;
    var x107: u1 = undefined;
    addcarryxU32(&x106, &x107, x105, x65, x62);
    const x108 = (cast(u32, x107) + x63);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    addcarryxU32(&x109, &x110, 0x0, x35, x84);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    addcarryxU32(&x111, &x112, x110, x37, x86);
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    addcarryxU32(&x113, &x114, x112, x39, x88);
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    addcarryxU32(&x115, &x116, x114, x41, x90);
    var x117: u32 = undefined;
    var x118: u1 = undefined;
    addcarryxU32(&x117, &x118, x116, x43, x92);
    var x119: u32 = undefined;
    var x120: u1 = undefined;
    addcarryxU32(&x119, &x120, x118, x45, x94);
    var x121: u32 = undefined;
    var x122: u1 = undefined;
    addcarryxU32(&x121, &x122, x120, x47, x96);
    var x123: u32 = undefined;
    var x124: u1 = undefined;
    addcarryxU32(&x123, &x124, x122, x49, x98);
    var x125: u32 = undefined;
    var x126: u1 = undefined;
    addcarryxU32(&x125, &x126, x124, x51, x100);
    var x127: u32 = undefined;
    var x128: u1 = undefined;
    addcarryxU32(&x127, &x128, x126, x53, x102);
    var x129: u32 = undefined;
    var x130: u1 = undefined;
    addcarryxU32(&x129, &x130, x128, x55, x104);
    var x131: u32 = undefined;
    var x132: u1 = undefined;
    addcarryxU32(&x131, &x132, x130, x57, x106);
    var x133: u32 = undefined;
    var x134: u1 = undefined;
    addcarryxU32(&x133, &x134, x132, x59, x108);
    var x135: u32 = undefined;
    var x136: u32 = undefined;
    mulxU32(&x135, &x136, x1, (arg2[11]));
    var x137: u32 = undefined;
    var x138: u32 = undefined;
    mulxU32(&x137, &x138, x1, (arg2[10]));
    var x139: u32 = undefined;
    var x140: u32 = undefined;
    mulxU32(&x139, &x140, x1, (arg2[9]));
    var x141: u32 = undefined;
    var x142: u32 = undefined;
    mulxU32(&x141, &x142, x1, (arg2[8]));
    var x143: u32 = undefined;
    var x144: u32 = undefined;
    mulxU32(&x143, &x144, x1, (arg2[7]));
    var x145: u32 = undefined;
    var x146: u32 = undefined;
    mulxU32(&x145, &x146, x1, (arg2[6]));
    var x147: u32 = undefined;
    var x148: u32 = undefined;
    mulxU32(&x147, &x148, x1, (arg2[5]));
    var x149: u32 = undefined;
    var x150: u32 = undefined;
    mulxU32(&x149, &x150, x1, (arg2[4]));
    var x151: u32 = undefined;
    var x152: u32 = undefined;
    mulxU32(&x151, &x152, x1, (arg2[3]));
    var x153: u32 = undefined;
    var x154: u32 = undefined;
    mulxU32(&x153, &x154, x1, (arg2[2]));
    var x155: u32 = undefined;
    var x156: u32 = undefined;
    mulxU32(&x155, &x156, x1, (arg2[1]));
    var x157: u32 = undefined;
    var x158: u32 = undefined;
    mulxU32(&x157, &x158, x1, (arg2[0]));
    var x159: u32 = undefined;
    var x160: u1 = undefined;
    addcarryxU32(&x159, &x160, 0x0, x158, x155);
    var x161: u32 = undefined;
    var x162: u1 = undefined;
    addcarryxU32(&x161, &x162, x160, x156, x153);
    var x163: u32 = undefined;
    var x164: u1 = undefined;
    addcarryxU32(&x163, &x164, x162, x154, x151);
    var x165: u32 = undefined;
    var x166: u1 = undefined;
    addcarryxU32(&x165, &x166, x164, x152, x149);
    var x167: u32 = undefined;
    var x168: u1 = undefined;
    addcarryxU32(&x167, &x168, x166, x150, x147);
    var x169: u32 = undefined;
    var x170: u1 = undefined;
    addcarryxU32(&x169, &x170, x168, x148, x145);
    var x171: u32 = undefined;
    var x172: u1 = undefined;
    addcarryxU32(&x171, &x172, x170, x146, x143);
    var x173: u32 = undefined;
    var x174: u1 = undefined;
    addcarryxU32(&x173, &x174, x172, x144, x141);
    var x175: u32 = undefined;
    var x176: u1 = undefined;
    addcarryxU32(&x175, &x176, x174, x142, x139);
    var x177: u32 = undefined;
    var x178: u1 = undefined;
    addcarryxU32(&x177, &x178, x176, x140, x137);
    var x179: u32 = undefined;
    var x180: u1 = undefined;
    addcarryxU32(&x179, &x180, x178, x138, x135);
    const x181 = (cast(u32, x180) + x136);
    var x182: u32 = undefined;
    var x183: u1 = undefined;
    addcarryxU32(&x182, &x183, 0x0, x111, x157);
    var x184: u32 = undefined;
    var x185: u1 = undefined;
    addcarryxU32(&x184, &x185, x183, x113, x159);
    var x186: u32 = undefined;
    var x187: u1 = undefined;
    addcarryxU32(&x186, &x187, x185, x115, x161);
    var x188: u32 = undefined;
    var x189: u1 = undefined;
    addcarryxU32(&x188, &x189, x187, x117, x163);
    var x190: u32 = undefined;
    var x191: u1 = undefined;
    addcarryxU32(&x190, &x191, x189, x119, x165);
    var x192: u32 = undefined;
    var x193: u1 = undefined;
    addcarryxU32(&x192, &x193, x191, x121, x167);
    var x194: u32 = undefined;
    var x195: u1 = undefined;
    addcarryxU32(&x194, &x195, x193, x123, x169);
    var x196: u32 = undefined;
    var x197: u1 = undefined;
    addcarryxU32(&x196, &x197, x195, x125, x171);
    var x198: u32 = undefined;
    var x199: u1 = undefined;
    addcarryxU32(&x198, &x199, x197, x127, x173);
    var x200: u32 = undefined;
    var x201: u1 = undefined;
    addcarryxU32(&x200, &x201, x199, x129, x175);
    var x202: u32 = undefined;
    var x203: u1 = undefined;
    addcarryxU32(&x202, &x203, x201, x131, x177);
    var x204: u32 = undefined;
    var x205: u1 = undefined;
    addcarryxU32(&x204, &x205, x203, x133, x179);
    var x206: u32 = undefined;
    var x207: u1 = undefined;
    addcarryxU32(&x206, &x207, x205, cast(u32, x134), x181);
    var x208: u32 = undefined;
    var x209: u32 = undefined;
    mulxU32(&x208, &x209, x182, 0xe88fdc45);
    var x210: u32 = undefined;
    var x211: u32 = undefined;
    mulxU32(&x210, &x211, x208, 0xffffffff);
    var x212: u32 = undefined;
    var x213: u32 = undefined;
    mulxU32(&x212, &x213, x208, 0xffffffff);
    var x214: u32 = undefined;
    var x215: u32 = undefined;
    mulxU32(&x214, &x215, x208, 0xffffffff);
    var x216: u32 = undefined;
    var x217: u32 = undefined;
    mulxU32(&x216, &x217, x208, 0xffffffff);
    var x218: u32 = undefined;
    var x219: u32 = undefined;
    mulxU32(&x218, &x219, x208, 0xffffffff);
    var x220: u32 = undefined;
    var x221: u32 = undefined;
    mulxU32(&x220, &x221, x208, 0xffffffff);
    var x222: u32 = undefined;
    var x223: u32 = undefined;
    mulxU32(&x222, &x223, x208, 0xc7634d81);
    var x224: u32 = undefined;
    var x225: u32 = undefined;
    mulxU32(&x224, &x225, x208, 0xf4372ddf);
    var x226: u32 = undefined;
    var x227: u32 = undefined;
    mulxU32(&x226, &x227, x208, 0x581a0db2);
    var x228: u32 = undefined;
    var x229: u32 = undefined;
    mulxU32(&x228, &x229, x208, 0x48b0a77a);
    var x230: u32 = undefined;
    var x231: u32 = undefined;
    mulxU32(&x230, &x231, x208, 0xecec196a);
    var x232: u32 = undefined;
    var x233: u32 = undefined;
    mulxU32(&x232, &x233, x208, 0xccc52973);
    var x234: u32 = undefined;
    var x235: u1 = undefined;
    addcarryxU32(&x234, &x235, 0x0, x233, x230);
    var x236: u32 = undefined;
    var x237: u1 = undefined;
    addcarryxU32(&x236, &x237, x235, x231, x228);
    var x238: u32 = undefined;
    var x239: u1 = undefined;
    addcarryxU32(&x238, &x239, x237, x229, x226);
    var x240: u32 = undefined;
    var x241: u1 = undefined;
    addcarryxU32(&x240, &x241, x239, x227, x224);
    var x242: u32 = undefined;
    var x243: u1 = undefined;
    addcarryxU32(&x242, &x243, x241, x225, x222);
    var x244: u32 = undefined;
    var x245: u1 = undefined;
    addcarryxU32(&x244, &x245, x243, x223, x220);
    var x246: u32 = undefined;
    var x247: u1 = undefined;
    addcarryxU32(&x246, &x247, x245, x221, x218);
    var x248: u32 = undefined;
    var x249: u1 = undefined;
    addcarryxU32(&x248, &x249, x247, x219, x216);
    var x250: u32 = undefined;
    var x251: u1 = undefined;
    addcarryxU32(&x250, &x251, x249, x217, x214);
    var x252: u32 = undefined;
    var x253: u1 = undefined;
    addcarryxU32(&x252, &x253, x251, x215, x212);
    var x254: u32 = undefined;
    var x255: u1 = undefined;
    addcarryxU32(&x254, &x255, x253, x213, x210);
    const x256 = (cast(u32, x255) + x211);
    var x257: u32 = undefined;
    var x258: u1 = undefined;
    addcarryxU32(&x257, &x258, 0x0, x182, x232);
    var x259: u32 = undefined;
    var x260: u1 = undefined;
    addcarryxU32(&x259, &x260, x258, x184, x234);
    var x261: u32 = undefined;
    var x262: u1 = undefined;
    addcarryxU32(&x261, &x262, x260, x186, x236);
    var x263: u32 = undefined;
    var x264: u1 = undefined;
    addcarryxU32(&x263, &x264, x262, x188, x238);
    var x265: u32 = undefined;
    var x266: u1 = undefined;
    addcarryxU32(&x265, &x266, x264, x190, x240);
    var x267: u32 = undefined;
    var x268: u1 = undefined;
    addcarryxU32(&x267, &x268, x266, x192, x242);
    var x269: u32 = undefined;
    var x270: u1 = undefined;
    addcarryxU32(&x269, &x270, x268, x194, x244);
    var x271: u32 = undefined;
    var x272: u1 = undefined;
    addcarryxU32(&x271, &x272, x270, x196, x246);
    var x273: u32 = undefined;
    var x274: u1 = undefined;
    addcarryxU32(&x273, &x274, x272, x198, x248);
    var x275: u32 = undefined;
    var x276: u1 = undefined;
    addcarryxU32(&x275, &x276, x274, x200, x250);
    var x277: u32 = undefined;
    var x278: u1 = undefined;
    addcarryxU32(&x277, &x278, x276, x202, x252);
    var x279: u32 = undefined;
    var x280: u1 = undefined;
    addcarryxU32(&x279, &x280, x278, x204, x254);
    var x281: u32 = undefined;
    var x282: u1 = undefined;
    addcarryxU32(&x281, &x282, x280, x206, x256);
    const x283 = (cast(u32, x282) + cast(u32, x207));
    var x284: u32 = undefined;
    var x285: u32 = undefined;
    mulxU32(&x284, &x285, x2, (arg2[11]));
    var x286: u32 = undefined;
    var x287: u32 = undefined;
    mulxU32(&x286, &x287, x2, (arg2[10]));
    var x288: u32 = undefined;
    var x289: u32 = undefined;
    mulxU32(&x288, &x289, x2, (arg2[9]));
    var x290: u32 = undefined;
    var x291: u32 = undefined;
    mulxU32(&x290, &x291, x2, (arg2[8]));
    var x292: u32 = undefined;
    var x293: u32 = undefined;
    mulxU32(&x292, &x293, x2, (arg2[7]));
    var x294: u32 = undefined;
    var x295: u32 = undefined;
    mulxU32(&x294, &x295, x2, (arg2[6]));
    var x296: u32 = undefined;
    var x297: u32 = undefined;
    mulxU32(&x296, &x297, x2, (arg2[5]));
    var x298: u32 = undefined;
    var x299: u32 = undefined;
    mulxU32(&x298, &x299, x2, (arg2[4]));
    var x300: u32 = undefined;
    var x301: u32 = undefined;
    mulxU32(&x300, &x301, x2, (arg2[3]));
    var x302: u32 = undefined;
    var x303: u32 = undefined;
    mulxU32(&x302, &x303, x2, (arg2[2]));
    var x304: u32 = undefined;
    var x305: u32 = undefined;
    mulxU32(&x304, &x305, x2, (arg2[1]));
    var x306: u32 = undefined;
    var x307: u32 = undefined;
    mulxU32(&x306, &x307, x2, (arg2[0]));
    var x308: u32 = undefined;
    var x309: u1 = undefined;
    addcarryxU32(&x308, &x309, 0x0, x307, x304);
    var x310: u32 = undefined;
    var x311: u1 = undefined;
    addcarryxU32(&x310, &x311, x309, x305, x302);
    var x312: u32 = undefined;
    var x313: u1 = undefined;
    addcarryxU32(&x312, &x313, x311, x303, x300);
    var x314: u32 = undefined;
    var x315: u1 = undefined;
    addcarryxU32(&x314, &x315, x313, x301, x298);
    var x316: u32 = undefined;
    var x317: u1 = undefined;
    addcarryxU32(&x316, &x317, x315, x299, x296);
    var x318: u32 = undefined;
    var x319: u1 = undefined;
    addcarryxU32(&x318, &x319, x317, x297, x294);
    var x320: u32 = undefined;
    var x321: u1 = undefined;
    addcarryxU32(&x320, &x321, x319, x295, x292);
    var x322: u32 = undefined;
    var x323: u1 = undefined;
    addcarryxU32(&x322, &x323, x321, x293, x290);
    var x324: u32 = undefined;
    var x325: u1 = undefined;
    addcarryxU32(&x324, &x325, x323, x291, x288);
    var x326: u32 = undefined;
    var x327: u1 = undefined;
    addcarryxU32(&x326, &x327, x325, x289, x286);
    var x328: u32 = undefined;
    var x329: u1 = undefined;
    addcarryxU32(&x328, &x329, x327, x287, x284);
    const x330 = (cast(u32, x329) + x285);
    var x331: u32 = undefined;
    var x332: u1 = undefined;
    addcarryxU32(&x331, &x332, 0x0, x259, x306);
    var x333: u32 = undefined;
    var x334: u1 = undefined;
    addcarryxU32(&x333, &x334, x332, x261, x308);
    var x335: u32 = undefined;
    var x336: u1 = undefined;
    addcarryxU32(&x335, &x336, x334, x263, x310);
    var x337: u32 = undefined;
    var x338: u1 = undefined;
    addcarryxU32(&x337, &x338, x336, x265, x312);
    var x339: u32 = undefined;
    var x340: u1 = undefined;
    addcarryxU32(&x339, &x340, x338, x267, x314);
    var x341: u32 = undefined;
    var x342: u1 = undefined;
    addcarryxU32(&x341, &x342, x340, x269, x316);
    var x343: u32 = undefined;
    var x344: u1 = undefined;
    addcarryxU32(&x343, &x344, x342, x271, x318);
    var x345: u32 = undefined;
    var x346: u1 = undefined;
    addcarryxU32(&x345, &x346, x344, x273, x320);
    var x347: u32 = undefined;
    var x348: u1 = undefined;
    addcarryxU32(&x347, &x348, x346, x275, x322);
    var x349: u32 = undefined;
    var x350: u1 = undefined;
    addcarryxU32(&x349, &x350, x348, x277, x324);
    var x351: u32 = undefined;
    var x352: u1 = undefined;
    addcarryxU32(&x351, &x352, x350, x279, x326);
    var x353: u32 = undefined;
    var x354: u1 = undefined;
    addcarryxU32(&x353, &x354, x352, x281, x328);
    var x355: u32 = undefined;
    var x356: u1 = undefined;
    addcarryxU32(&x355, &x356, x354, x283, x330);
    var x357: u32 = undefined;
    var x358: u32 = undefined;
    mulxU32(&x357, &x358, x331, 0xe88fdc45);
    var x359: u32 = undefined;
    var x360: u32 = undefined;
    mulxU32(&x359, &x360, x357, 0xffffffff);
    var x361: u32 = undefined;
    var x362: u32 = undefined;
    mulxU32(&x361, &x362, x357, 0xffffffff);
    var x363: u32 = undefined;
    var x364: u32 = undefined;
    mulxU32(&x363, &x364, x357, 0xffffffff);
    var x365: u32 = undefined;
    var x366: u32 = undefined;
    mulxU32(&x365, &x366, x357, 0xffffffff);
    var x367: u32 = undefined;
    var x368: u32 = undefined;
    mulxU32(&x367, &x368, x357, 0xffffffff);
    var x369: u32 = undefined;
    var x370: u32 = undefined;
    mulxU32(&x369, &x370, x357, 0xffffffff);
    var x371: u32 = undefined;
    var x372: u32 = undefined;
    mulxU32(&x371, &x372, x357, 0xc7634d81);
    var x373: u32 = undefined;
    var x374: u32 = undefined;
    mulxU32(&x373, &x374, x357, 0xf4372ddf);
    var x375: u32 = undefined;
    var x376: u32 = undefined;
    mulxU32(&x375, &x376, x357, 0x581a0db2);
    var x377: u32 = undefined;
    var x378: u32 = undefined;
    mulxU32(&x377, &x378, x357, 0x48b0a77a);
    var x379: u32 = undefined;
    var x380: u32 = undefined;
    mulxU32(&x379, &x380, x357, 0xecec196a);
    var x381: u32 = undefined;
    var x382: u32 = undefined;
    mulxU32(&x381, &x382, x357, 0xccc52973);
    var x383: u32 = undefined;
    var x384: u1 = undefined;
    addcarryxU32(&x383, &x384, 0x0, x382, x379);
    var x385: u32 = undefined;
    var x386: u1 = undefined;
    addcarryxU32(&x385, &x386, x384, x380, x377);
    var x387: u32 = undefined;
    var x388: u1 = undefined;
    addcarryxU32(&x387, &x388, x386, x378, x375);
    var x389: u32 = undefined;
    var x390: u1 = undefined;
    addcarryxU32(&x389, &x390, x388, x376, x373);
    var x391: u32 = undefined;
    var x392: u1 = undefined;
    addcarryxU32(&x391, &x392, x390, x374, x371);
    var x393: u32 = undefined;
    var x394: u1 = undefined;
    addcarryxU32(&x393, &x394, x392, x372, x369);
    var x395: u32 = undefined;
    var x396: u1 = undefined;
    addcarryxU32(&x395, &x396, x394, x370, x367);
    var x397: u32 = undefined;
    var x398: u1 = undefined;
    addcarryxU32(&x397, &x398, x396, x368, x365);
    var x399: u32 = undefined;
    var x400: u1 = undefined;
    addcarryxU32(&x399, &x400, x398, x366, x363);
    var x401: u32 = undefined;
    var x402: u1 = undefined;
    addcarryxU32(&x401, &x402, x400, x364, x361);
    var x403: u32 = undefined;
    var x404: u1 = undefined;
    addcarryxU32(&x403, &x404, x402, x362, x359);
    const x405 = (cast(u32, x404) + x360);
    var x406: u32 = undefined;
    var x407: u1 = undefined;
    addcarryxU32(&x406, &x407, 0x0, x331, x381);
    var x408: u32 = undefined;
    var x409: u1 = undefined;
    addcarryxU32(&x408, &x409, x407, x333, x383);
    var x410: u32 = undefined;
    var x411: u1 = undefined;
    addcarryxU32(&x410, &x411, x409, x335, x385);
    var x412: u32 = undefined;
    var x413: u1 = undefined;
    addcarryxU32(&x412, &x413, x411, x337, x387);
    var x414: u32 = undefined;
    var x415: u1 = undefined;
    addcarryxU32(&x414, &x415, x413, x339, x389);
    var x416: u32 = undefined;
    var x417: u1 = undefined;
    addcarryxU32(&x416, &x417, x415, x341, x391);
    var x418: u32 = undefined;
    var x419: u1 = undefined;
    addcarryxU32(&x418, &x419, x417, x343, x393);
    var x420: u32 = undefined;
    var x421: u1 = undefined;
    addcarryxU32(&x420, &x421, x419, x345, x395);
    var x422: u32 = undefined;
    var x423: u1 = undefined;
    addcarryxU32(&x422, &x423, x421, x347, x397);
    var x424: u32 = undefined;
    var x425: u1 = undefined;
    addcarryxU32(&x424, &x425, x423, x349, x399);
    var x426: u32 = undefined;
    var x427: u1 = undefined;
    addcarryxU32(&x426, &x427, x425, x351, x401);
    var x428: u32 = undefined;
    var x429: u1 = undefined;
    addcarryxU32(&x428, &x429, x427, x353, x403);
    var x430: u32 = undefined;
    var x431: u1 = undefined;
    addcarryxU32(&x430, &x431, x429, x355, x405);
    const x432 = (cast(u32, x431) + cast(u32, x356));
    var x433: u32 = undefined;
    var x434: u32 = undefined;
    mulxU32(&x433, &x434, x3, (arg2[11]));
    var x435: u32 = undefined;
    var x436: u32 = undefined;
    mulxU32(&x435, &x436, x3, (arg2[10]));
    var x437: u32 = undefined;
    var x438: u32 = undefined;
    mulxU32(&x437, &x438, x3, (arg2[9]));
    var x439: u32 = undefined;
    var x440: u32 = undefined;
    mulxU32(&x439, &x440, x3, (arg2[8]));
    var x441: u32 = undefined;
    var x442: u32 = undefined;
    mulxU32(&x441, &x442, x3, (arg2[7]));
    var x443: u32 = undefined;
    var x444: u32 = undefined;
    mulxU32(&x443, &x444, x3, (arg2[6]));
    var x445: u32 = undefined;
    var x446: u32 = undefined;
    mulxU32(&x445, &x446, x3, (arg2[5]));
    var x447: u32 = undefined;
    var x448: u32 = undefined;
    mulxU32(&x447, &x448, x3, (arg2[4]));
    var x449: u32 = undefined;
    var x450: u32 = undefined;
    mulxU32(&x449, &x450, x3, (arg2[3]));
    var x451: u32 = undefined;
    var x452: u32 = undefined;
    mulxU32(&x451, &x452, x3, (arg2[2]));
    var x453: u32 = undefined;
    var x454: u32 = undefined;
    mulxU32(&x453, &x454, x3, (arg2[1]));
    var x455: u32 = undefined;
    var x456: u32 = undefined;
    mulxU32(&x455, &x456, x3, (arg2[0]));
    var x457: u32 = undefined;
    var x458: u1 = undefined;
    addcarryxU32(&x457, &x458, 0x0, x456, x453);
    var x459: u32 = undefined;
    var x460: u1 = undefined;
    addcarryxU32(&x459, &x460, x458, x454, x451);
    var x461: u32 = undefined;
    var x462: u1 = undefined;
    addcarryxU32(&x461, &x462, x460, x452, x449);
    var x463: u32 = undefined;
    var x464: u1 = undefined;
    addcarryxU32(&x463, &x464, x462, x450, x447);
    var x465: u32 = undefined;
    var x466: u1 = undefined;
    addcarryxU32(&x465, &x466, x464, x448, x445);
    var x467: u32 = undefined;
    var x468: u1 = undefined;
    addcarryxU32(&x467, &x468, x466, x446, x443);
    var x469: u32 = undefined;
    var x470: u1 = undefined;
    addcarryxU32(&x469, &x470, x468, x444, x441);
    var x471: u32 = undefined;
    var x472: u1 = undefined;
    addcarryxU32(&x471, &x472, x470, x442, x439);
    var x473: u32 = undefined;
    var x474: u1 = undefined;
    addcarryxU32(&x473, &x474, x472, x440, x437);
    var x475: u32 = undefined;
    var x476: u1 = undefined;
    addcarryxU32(&x475, &x476, x474, x438, x435);
    var x477: u32 = undefined;
    var x478: u1 = undefined;
    addcarryxU32(&x477, &x478, x476, x436, x433);
    const x479 = (cast(u32, x478) + x434);
    var x480: u32 = undefined;
    var x481: u1 = undefined;
    addcarryxU32(&x480, &x481, 0x0, x408, x455);
    var x482: u32 = undefined;
    var x483: u1 = undefined;
    addcarryxU32(&x482, &x483, x481, x410, x457);
    var x484: u32 = undefined;
    var x485: u1 = undefined;
    addcarryxU32(&x484, &x485, x483, x412, x459);
    var x486: u32 = undefined;
    var x487: u1 = undefined;
    addcarryxU32(&x486, &x487, x485, x414, x461);
    var x488: u32 = undefined;
    var x489: u1 = undefined;
    addcarryxU32(&x488, &x489, x487, x416, x463);
    var x490: u32 = undefined;
    var x491: u1 = undefined;
    addcarryxU32(&x490, &x491, x489, x418, x465);
    var x492: u32 = undefined;
    var x493: u1 = undefined;
    addcarryxU32(&x492, &x493, x491, x420, x467);
    var x494: u32 = undefined;
    var x495: u1 = undefined;
    addcarryxU32(&x494, &x495, x493, x422, x469);
    var x496: u32 = undefined;
    var x497: u1 = undefined;
    addcarryxU32(&x496, &x497, x495, x424, x471);
    var x498: u32 = undefined;
    var x499: u1 = undefined;
    addcarryxU32(&x498, &x499, x497, x426, x473);
    var x500: u32 = undefined;
    var x501: u1 = undefined;
    addcarryxU32(&x500, &x501, x499, x428, x475);
    var x502: u32 = undefined;
    var x503: u1 = undefined;
    addcarryxU32(&x502, &x503, x501, x430, x477);
    var x504: u32 = undefined;
    var x505: u1 = undefined;
    addcarryxU32(&x504, &x505, x503, x432, x479);
    var x506: u32 = undefined;
    var x507: u32 = undefined;
    mulxU32(&x506, &x507, x480, 0xe88fdc45);
    var x508: u32 = undefined;
    var x509: u32 = undefined;
    mulxU32(&x508, &x509, x506, 0xffffffff);
    var x510: u32 = undefined;
    var x511: u32 = undefined;
    mulxU32(&x510, &x511, x506, 0xffffffff);
    var x512: u32 = undefined;
    var x513: u32 = undefined;
    mulxU32(&x512, &x513, x506, 0xffffffff);
    var x514: u32 = undefined;
    var x515: u32 = undefined;
    mulxU32(&x514, &x515, x506, 0xffffffff);
    var x516: u32 = undefined;
    var x517: u32 = undefined;
    mulxU32(&x516, &x517, x506, 0xffffffff);
    var x518: u32 = undefined;
    var x519: u32 = undefined;
    mulxU32(&x518, &x519, x506, 0xffffffff);
    var x520: u32 = undefined;
    var x521: u32 = undefined;
    mulxU32(&x520, &x521, x506, 0xc7634d81);
    var x522: u32 = undefined;
    var x523: u32 = undefined;
    mulxU32(&x522, &x523, x506, 0xf4372ddf);
    var x524: u32 = undefined;
    var x525: u32 = undefined;
    mulxU32(&x524, &x525, x506, 0x581a0db2);
    var x526: u32 = undefined;
    var x527: u32 = undefined;
    mulxU32(&x526, &x527, x506, 0x48b0a77a);
    var x528: u32 = undefined;
    var x529: u32 = undefined;
    mulxU32(&x528, &x529, x506, 0xecec196a);
    var x530: u32 = undefined;
    var x531: u32 = undefined;
    mulxU32(&x530, &x531, x506, 0xccc52973);
    var x532: u32 = undefined;
    var x533: u1 = undefined;
    addcarryxU32(&x532, &x533, 0x0, x531, x528);
    var x534: u32 = undefined;
    var x535: u1 = undefined;
    addcarryxU32(&x534, &x535, x533, x529, x526);
    var x536: u32 = undefined;
    var x537: u1 = undefined;
    addcarryxU32(&x536, &x537, x535, x527, x524);
    var x538: u32 = undefined;
    var x539: u1 = undefined;
    addcarryxU32(&x538, &x539, x537, x525, x522);
    var x540: u32 = undefined;
    var x541: u1 = undefined;
    addcarryxU32(&x540, &x541, x539, x523, x520);
    var x542: u32 = undefined;
    var x543: u1 = undefined;
    addcarryxU32(&x542, &x543, x541, x521, x518);
    var x544: u32 = undefined;
    var x545: u1 = undefined;
    addcarryxU32(&x544, &x545, x543, x519, x516);
    var x546: u32 = undefined;
    var x547: u1 = undefined;
    addcarryxU32(&x546, &x547, x545, x517, x514);
    var x548: u32 = undefined;
    var x549: u1 = undefined;
    addcarryxU32(&x548, &x549, x547, x515, x512);
    var x550: u32 = undefined;
    var x551: u1 = undefined;
    addcarryxU32(&x550, &x551, x549, x513, x510);
    var x552: u32 = undefined;
    var x553: u1 = undefined;
    addcarryxU32(&x552, &x553, x551, x511, x508);
    const x554 = (cast(u32, x553) + x509);
    var x555: u32 = undefined;
    var x556: u1 = undefined;
    addcarryxU32(&x555, &x556, 0x0, x480, x530);
    var x557: u32 = undefined;
    var x558: u1 = undefined;
    addcarryxU32(&x557, &x558, x556, x482, x532);
    var x559: u32 = undefined;
    var x560: u1 = undefined;
    addcarryxU32(&x559, &x560, x558, x484, x534);
    var x561: u32 = undefined;
    var x562: u1 = undefined;
    addcarryxU32(&x561, &x562, x560, x486, x536);
    var x563: u32 = undefined;
    var x564: u1 = undefined;
    addcarryxU32(&x563, &x564, x562, x488, x538);
    var x565: u32 = undefined;
    var x566: u1 = undefined;
    addcarryxU32(&x565, &x566, x564, x490, x540);
    var x567: u32 = undefined;
    var x568: u1 = undefined;
    addcarryxU32(&x567, &x568, x566, x492, x542);
    var x569: u32 = undefined;
    var x570: u1 = undefined;
    addcarryxU32(&x569, &x570, x568, x494, x544);
    var x571: u32 = undefined;
    var x572: u1 = undefined;
    addcarryxU32(&x571, &x572, x570, x496, x546);
    var x573: u32 = undefined;
    var x574: u1 = undefined;
    addcarryxU32(&x573, &x574, x572, x498, x548);
    var x575: u32 = undefined;
    var x576: u1 = undefined;
    addcarryxU32(&x575, &x576, x574, x500, x550);
    var x577: u32 = undefined;
    var x578: u1 = undefined;
    addcarryxU32(&x577, &x578, x576, x502, x552);
    var x579: u32 = undefined;
    var x580: u1 = undefined;
    addcarryxU32(&x579, &x580, x578, x504, x554);
    const x581 = (cast(u32, x580) + cast(u32, x505));
    var x582: u32 = undefined;
    var x583: u32 = undefined;
    mulxU32(&x582, &x583, x4, (arg2[11]));
    var x584: u32 = undefined;
    var x585: u32 = undefined;
    mulxU32(&x584, &x585, x4, (arg2[10]));
    var x586: u32 = undefined;
    var x587: u32 = undefined;
    mulxU32(&x586, &x587, x4, (arg2[9]));
    var x588: u32 = undefined;
    var x589: u32 = undefined;
    mulxU32(&x588, &x589, x4, (arg2[8]));
    var x590: u32 = undefined;
    var x591: u32 = undefined;
    mulxU32(&x590, &x591, x4, (arg2[7]));
    var x592: u32 = undefined;
    var x593: u32 = undefined;
    mulxU32(&x592, &x593, x4, (arg2[6]));
    var x594: u32 = undefined;
    var x595: u32 = undefined;
    mulxU32(&x594, &x595, x4, (arg2[5]));
    var x596: u32 = undefined;
    var x597: u32 = undefined;
    mulxU32(&x596, &x597, x4, (arg2[4]));
    var x598: u32 = undefined;
    var x599: u32 = undefined;
    mulxU32(&x598, &x599, x4, (arg2[3]));
    var x600: u32 = undefined;
    var x601: u32 = undefined;
    mulxU32(&x600, &x601, x4, (arg2[2]));
    var x602: u32 = undefined;
    var x603: u32 = undefined;
    mulxU32(&x602, &x603, x4, (arg2[1]));
    var x604: u32 = undefined;
    var x605: u32 = undefined;
    mulxU32(&x604, &x605, x4, (arg2[0]));
    var x606: u32 = undefined;
    var x607: u1 = undefined;
    addcarryxU32(&x606, &x607, 0x0, x605, x602);
    var x608: u32 = undefined;
    var x609: u1 = undefined;
    addcarryxU32(&x608, &x609, x607, x603, x600);
    var x610: u32 = undefined;
    var x611: u1 = undefined;
    addcarryxU32(&x610, &x611, x609, x601, x598);
    var x612: u32 = undefined;
    var x613: u1 = undefined;
    addcarryxU32(&x612, &x613, x611, x599, x596);
    var x614: u32 = undefined;
    var x615: u1 = undefined;
    addcarryxU32(&x614, &x615, x613, x597, x594);
    var x616: u32 = undefined;
    var x617: u1 = undefined;
    addcarryxU32(&x616, &x617, x615, x595, x592);
    var x618: u32 = undefined;
    var x619: u1 = undefined;
    addcarryxU32(&x618, &x619, x617, x593, x590);
    var x620: u32 = undefined;
    var x621: u1 = undefined;
    addcarryxU32(&x620, &x621, x619, x591, x588);
    var x622: u32 = undefined;
    var x623: u1 = undefined;
    addcarryxU32(&x622, &x623, x621, x589, x586);
    var x624: u32 = undefined;
    var x625: u1 = undefined;
    addcarryxU32(&x624, &x625, x623, x587, x584);
    var x626: u32 = undefined;
    var x627: u1 = undefined;
    addcarryxU32(&x626, &x627, x625, x585, x582);
    const x628 = (cast(u32, x627) + x583);
    var x629: u32 = undefined;
    var x630: u1 = undefined;
    addcarryxU32(&x629, &x630, 0x0, x557, x604);
    var x631: u32 = undefined;
    var x632: u1 = undefined;
    addcarryxU32(&x631, &x632, x630, x559, x606);
    var x633: u32 = undefined;
    var x634: u1 = undefined;
    addcarryxU32(&x633, &x634, x632, x561, x608);
    var x635: u32 = undefined;
    var x636: u1 = undefined;
    addcarryxU32(&x635, &x636, x634, x563, x610);
    var x637: u32 = undefined;
    var x638: u1 = undefined;
    addcarryxU32(&x637, &x638, x636, x565, x612);
    var x639: u32 = undefined;
    var x640: u1 = undefined;
    addcarryxU32(&x639, &x640, x638, x567, x614);
    var x641: u32 = undefined;
    var x642: u1 = undefined;
    addcarryxU32(&x641, &x642, x640, x569, x616);
    var x643: u32 = undefined;
    var x644: u1 = undefined;
    addcarryxU32(&x643, &x644, x642, x571, x618);
    var x645: u32 = undefined;
    var x646: u1 = undefined;
    addcarryxU32(&x645, &x646, x644, x573, x620);
    var x647: u32 = undefined;
    var x648: u1 = undefined;
    addcarryxU32(&x647, &x648, x646, x575, x622);
    var x649: u32 = undefined;
    var x650: u1 = undefined;
    addcarryxU32(&x649, &x650, x648, x577, x624);
    var x651: u32 = undefined;
    var x652: u1 = undefined;
    addcarryxU32(&x651, &x652, x650, x579, x626);
    var x653: u32 = undefined;
    var x654: u1 = undefined;
    addcarryxU32(&x653, &x654, x652, x581, x628);
    var x655: u32 = undefined;
    var x656: u32 = undefined;
    mulxU32(&x655, &x656, x629, 0xe88fdc45);
    var x657: u32 = undefined;
    var x658: u32 = undefined;
    mulxU32(&x657, &x658, x655, 0xffffffff);
    var x659: u32 = undefined;
    var x660: u32 = undefined;
    mulxU32(&x659, &x660, x655, 0xffffffff);
    var x661: u32 = undefined;
    var x662: u32 = undefined;
    mulxU32(&x661, &x662, x655, 0xffffffff);
    var x663: u32 = undefined;
    var x664: u32 = undefined;
    mulxU32(&x663, &x664, x655, 0xffffffff);
    var x665: u32 = undefined;
    var x666: u32 = undefined;
    mulxU32(&x665, &x666, x655, 0xffffffff);
    var x667: u32 = undefined;
    var x668: u32 = undefined;
    mulxU32(&x667, &x668, x655, 0xffffffff);
    var x669: u32 = undefined;
    var x670: u32 = undefined;
    mulxU32(&x669, &x670, x655, 0xc7634d81);
    var x671: u32 = undefined;
    var x672: u32 = undefined;
    mulxU32(&x671, &x672, x655, 0xf4372ddf);
    var x673: u32 = undefined;
    var x674: u32 = undefined;
    mulxU32(&x673, &x674, x655, 0x581a0db2);
    var x675: u32 = undefined;
    var x676: u32 = undefined;
    mulxU32(&x675, &x676, x655, 0x48b0a77a);
    var x677: u32 = undefined;
    var x678: u32 = undefined;
    mulxU32(&x677, &x678, x655, 0xecec196a);
    var x679: u32 = undefined;
    var x680: u32 = undefined;
    mulxU32(&x679, &x680, x655, 0xccc52973);
    var x681: u32 = undefined;
    var x682: u1 = undefined;
    addcarryxU32(&x681, &x682, 0x0, x680, x677);
    var x683: u32 = undefined;
    var x684: u1 = undefined;
    addcarryxU32(&x683, &x684, x682, x678, x675);
    var x685: u32 = undefined;
    var x686: u1 = undefined;
    addcarryxU32(&x685, &x686, x684, x676, x673);
    var x687: u32 = undefined;
    var x688: u1 = undefined;
    addcarryxU32(&x687, &x688, x686, x674, x671);
    var x689: u32 = undefined;
    var x690: u1 = undefined;
    addcarryxU32(&x689, &x690, x688, x672, x669);
    var x691: u32 = undefined;
    var x692: u1 = undefined;
    addcarryxU32(&x691, &x692, x690, x670, x667);
    var x693: u32 = undefined;
    var x694: u1 = undefined;
    addcarryxU32(&x693, &x694, x692, x668, x665);
    var x695: u32 = undefined;
    var x696: u1 = undefined;
    addcarryxU32(&x695, &x696, x694, x666, x663);
    var x697: u32 = undefined;
    var x698: u1 = undefined;
    addcarryxU32(&x697, &x698, x696, x664, x661);
    var x699: u32 = undefined;
    var x700: u1 = undefined;
    addcarryxU32(&x699, &x700, x698, x662, x659);
    var x701: u32 = undefined;
    var x702: u1 = undefined;
    addcarryxU32(&x701, &x702, x700, x660, x657);
    const x703 = (cast(u32, x702) + x658);
    var x704: u32 = undefined;
    var x705: u1 = undefined;
    addcarryxU32(&x704, &x705, 0x0, x629, x679);
    var x706: u32 = undefined;
    var x707: u1 = undefined;
    addcarryxU32(&x706, &x707, x705, x631, x681);
    var x708: u32 = undefined;
    var x709: u1 = undefined;
    addcarryxU32(&x708, &x709, x707, x633, x683);
    var x710: u32 = undefined;
    var x711: u1 = undefined;
    addcarryxU32(&x710, &x711, x709, x635, x685);
    var x712: u32 = undefined;
    var x713: u1 = undefined;
    addcarryxU32(&x712, &x713, x711, x637, x687);
    var x714: u32 = undefined;
    var x715: u1 = undefined;
    addcarryxU32(&x714, &x715, x713, x639, x689);
    var x716: u32 = undefined;
    var x717: u1 = undefined;
    addcarryxU32(&x716, &x717, x715, x641, x691);
    var x718: u32 = undefined;
    var x719: u1 = undefined;
    addcarryxU32(&x718, &x719, x717, x643, x693);
    var x720: u32 = undefined;
    var x721: u1 = undefined;
    addcarryxU32(&x720, &x721, x719, x645, x695);
    var x722: u32 = undefined;
    var x723: u1 = undefined;
    addcarryxU32(&x722, &x723, x721, x647, x697);
    var x724: u32 = undefined;
    var x725: u1 = undefined;
    addcarryxU32(&x724, &x725, x723, x649, x699);
    var x726: u32 = undefined;
    var x727: u1 = undefined;
    addcarryxU32(&x726, &x727, x725, x651, x701);
    var x728: u32 = undefined;
    var x729: u1 = undefined;
    addcarryxU32(&x728, &x729, x727, x653, x703);
    const x730 = (cast(u32, x729) + cast(u32, x654));
    var x731: u32 = undefined;
    var x732: u32 = undefined;
    mulxU32(&x731, &x732, x5, (arg2[11]));
    var x733: u32 = undefined;
    var x734: u32 = undefined;
    mulxU32(&x733, &x734, x5, (arg2[10]));
    var x735: u32 = undefined;
    var x736: u32 = undefined;
    mulxU32(&x735, &x736, x5, (arg2[9]));
    var x737: u32 = undefined;
    var x738: u32 = undefined;
    mulxU32(&x737, &x738, x5, (arg2[8]));
    var x739: u32 = undefined;
    var x740: u32 = undefined;
    mulxU32(&x739, &x740, x5, (arg2[7]));
    var x741: u32 = undefined;
    var x742: u32 = undefined;
    mulxU32(&x741, &x742, x5, (arg2[6]));
    var x743: u32 = undefined;
    var x744: u32 = undefined;
    mulxU32(&x743, &x744, x5, (arg2[5]));
    var x745: u32 = undefined;
    var x746: u32 = undefined;
    mulxU32(&x745, &x746, x5, (arg2[4]));
    var x747: u32 = undefined;
    var x748: u32 = undefined;
    mulxU32(&x747, &x748, x5, (arg2[3]));
    var x749: u32 = undefined;
    var x750: u32 = undefined;
    mulxU32(&x749, &x750, x5, (arg2[2]));
    var x751: u32 = undefined;
    var x752: u32 = undefined;
    mulxU32(&x751, &x752, x5, (arg2[1]));
    var x753: u32 = undefined;
    var x754: u32 = undefined;
    mulxU32(&x753, &x754, x5, (arg2[0]));
    var x755: u32 = undefined;
    var x756: u1 = undefined;
    addcarryxU32(&x755, &x756, 0x0, x754, x751);
    var x757: u32 = undefined;
    var x758: u1 = undefined;
    addcarryxU32(&x757, &x758, x756, x752, x749);
    var x759: u32 = undefined;
    var x760: u1 = undefined;
    addcarryxU32(&x759, &x760, x758, x750, x747);
    var x761: u32 = undefined;
    var x762: u1 = undefined;
    addcarryxU32(&x761, &x762, x760, x748, x745);
    var x763: u32 = undefined;
    var x764: u1 = undefined;
    addcarryxU32(&x763, &x764, x762, x746, x743);
    var x765: u32 = undefined;
    var x766: u1 = undefined;
    addcarryxU32(&x765, &x766, x764, x744, x741);
    var x767: u32 = undefined;
    var x768: u1 = undefined;
    addcarryxU32(&x767, &x768, x766, x742, x739);
    var x769: u32 = undefined;
    var x770: u1 = undefined;
    addcarryxU32(&x769, &x770, x768, x740, x737);
    var x771: u32 = undefined;
    var x772: u1 = undefined;
    addcarryxU32(&x771, &x772, x770, x738, x735);
    var x773: u32 = undefined;
    var x774: u1 = undefined;
    addcarryxU32(&x773, &x774, x772, x736, x733);
    var x775: u32 = undefined;
    var x776: u1 = undefined;
    addcarryxU32(&x775, &x776, x774, x734, x731);
    const x777 = (cast(u32, x776) + x732);
    var x778: u32 = undefined;
    var x779: u1 = undefined;
    addcarryxU32(&x778, &x779, 0x0, x706, x753);
    var x780: u32 = undefined;
    var x781: u1 = undefined;
    addcarryxU32(&x780, &x781, x779, x708, x755);
    var x782: u32 = undefined;
    var x783: u1 = undefined;
    addcarryxU32(&x782, &x783, x781, x710, x757);
    var x784: u32 = undefined;
    var x785: u1 = undefined;
    addcarryxU32(&x784, &x785, x783, x712, x759);
    var x786: u32 = undefined;
    var x787: u1 = undefined;
    addcarryxU32(&x786, &x787, x785, x714, x761);
    var x788: u32 = undefined;
    var x789: u1 = undefined;
    addcarryxU32(&x788, &x789, x787, x716, x763);
    var x790: u32 = undefined;
    var x791: u1 = undefined;
    addcarryxU32(&x790, &x791, x789, x718, x765);
    var x792: u32 = undefined;
    var x793: u1 = undefined;
    addcarryxU32(&x792, &x793, x791, x720, x767);
    var x794: u32 = undefined;
    var x795: u1 = undefined;
    addcarryxU32(&x794, &x795, x793, x722, x769);
    var x796: u32 = undefined;
    var x797: u1 = undefined;
    addcarryxU32(&x796, &x797, x795, x724, x771);
    var x798: u32 = undefined;
    var x799: u1 = undefined;
    addcarryxU32(&x798, &x799, x797, x726, x773);
    var x800: u32 = undefined;
    var x801: u1 = undefined;
    addcarryxU32(&x800, &x801, x799, x728, x775);
    var x802: u32 = undefined;
    var x803: u1 = undefined;
    addcarryxU32(&x802, &x803, x801, x730, x777);
    var x804: u32 = undefined;
    var x805: u32 = undefined;
    mulxU32(&x804, &x805, x778, 0xe88fdc45);
    var x806: u32 = undefined;
    var x807: u32 = undefined;
    mulxU32(&x806, &x807, x804, 0xffffffff);
    var x808: u32 = undefined;
    var x809: u32 = undefined;
    mulxU32(&x808, &x809, x804, 0xffffffff);
    var x810: u32 = undefined;
    var x811: u32 = undefined;
    mulxU32(&x810, &x811, x804, 0xffffffff);
    var x812: u32 = undefined;
    var x813: u32 = undefined;
    mulxU32(&x812, &x813, x804, 0xffffffff);
    var x814: u32 = undefined;
    var x815: u32 = undefined;
    mulxU32(&x814, &x815, x804, 0xffffffff);
    var x816: u32 = undefined;
    var x817: u32 = undefined;
    mulxU32(&x816, &x817, x804, 0xffffffff);
    var x818: u32 = undefined;
    var x819: u32 = undefined;
    mulxU32(&x818, &x819, x804, 0xc7634d81);
    var x820: u32 = undefined;
    var x821: u32 = undefined;
    mulxU32(&x820, &x821, x804, 0xf4372ddf);
    var x822: u32 = undefined;
    var x823: u32 = undefined;
    mulxU32(&x822, &x823, x804, 0x581a0db2);
    var x824: u32 = undefined;
    var x825: u32 = undefined;
    mulxU32(&x824, &x825, x804, 0x48b0a77a);
    var x826: u32 = undefined;
    var x827: u32 = undefined;
    mulxU32(&x826, &x827, x804, 0xecec196a);
    var x828: u32 = undefined;
    var x829: u32 = undefined;
    mulxU32(&x828, &x829, x804, 0xccc52973);
    var x830: u32 = undefined;
    var x831: u1 = undefined;
    addcarryxU32(&x830, &x831, 0x0, x829, x826);
    var x832: u32 = undefined;
    var x833: u1 = undefined;
    addcarryxU32(&x832, &x833, x831, x827, x824);
    var x834: u32 = undefined;
    var x835: u1 = undefined;
    addcarryxU32(&x834, &x835, x833, x825, x822);
    var x836: u32 = undefined;
    var x837: u1 = undefined;
    addcarryxU32(&x836, &x837, x835, x823, x820);
    var x838: u32 = undefined;
    var x839: u1 = undefined;
    addcarryxU32(&x838, &x839, x837, x821, x818);
    var x840: u32 = undefined;
    var x841: u1 = undefined;
    addcarryxU32(&x840, &x841, x839, x819, x816);
    var x842: u32 = undefined;
    var x843: u1 = undefined;
    addcarryxU32(&x842, &x843, x841, x817, x814);
    var x844: u32 = undefined;
    var x845: u1 = undefined;
    addcarryxU32(&x844, &x845, x843, x815, x812);
    var x846: u32 = undefined;
    var x847: u1 = undefined;
    addcarryxU32(&x846, &x847, x845, x813, x810);
    var x848: u32 = undefined;
    var x849: u1 = undefined;
    addcarryxU32(&x848, &x849, x847, x811, x808);
    var x850: u32 = undefined;
    var x851: u1 = undefined;
    addcarryxU32(&x850, &x851, x849, x809, x806);
    const x852 = (cast(u32, x851) + x807);
    var x853: u32 = undefined;
    var x854: u1 = undefined;
    addcarryxU32(&x853, &x854, 0x0, x778, x828);
    var x855: u32 = undefined;
    var x856: u1 = undefined;
    addcarryxU32(&x855, &x856, x854, x780, x830);
    var x857: u32 = undefined;
    var x858: u1 = undefined;
    addcarryxU32(&x857, &x858, x856, x782, x832);
    var x859: u32 = undefined;
    var x860: u1 = undefined;
    addcarryxU32(&x859, &x860, x858, x784, x834);
    var x861: u32 = undefined;
    var x862: u1 = undefined;
    addcarryxU32(&x861, &x862, x860, x786, x836);
    var x863: u32 = undefined;
    var x864: u1 = undefined;
    addcarryxU32(&x863, &x864, x862, x788, x838);
    var x865: u32 = undefined;
    var x866: u1 = undefined;
    addcarryxU32(&x865, &x866, x864, x790, x840);
    var x867: u32 = undefined;
    var x868: u1 = undefined;
    addcarryxU32(&x867, &x868, x866, x792, x842);
    var x869: u32 = undefined;
    var x870: u1 = undefined;
    addcarryxU32(&x869, &x870, x868, x794, x844);
    var x871: u32 = undefined;
    var x872: u1 = undefined;
    addcarryxU32(&x871, &x872, x870, x796, x846);
    var x873: u32 = undefined;
    var x874: u1 = undefined;
    addcarryxU32(&x873, &x874, x872, x798, x848);
    var x875: u32 = undefined;
    var x876: u1 = undefined;
    addcarryxU32(&x875, &x876, x874, x800, x850);
    var x877: u32 = undefined;
    var x878: u1 = undefined;
    addcarryxU32(&x877, &x878, x876, x802, x852);
    const x879 = (cast(u32, x878) + cast(u32, x803));
    var x880: u32 = undefined;
    var x881: u32 = undefined;
    mulxU32(&x880, &x881, x6, (arg2[11]));
    var x882: u32 = undefined;
    var x883: u32 = undefined;
    mulxU32(&x882, &x883, x6, (arg2[10]));
    var x884: u32 = undefined;
    var x885: u32 = undefined;
    mulxU32(&x884, &x885, x6, (arg2[9]));
    var x886: u32 = undefined;
    var x887: u32 = undefined;
    mulxU32(&x886, &x887, x6, (arg2[8]));
    var x888: u32 = undefined;
    var x889: u32 = undefined;
    mulxU32(&x888, &x889, x6, (arg2[7]));
    var x890: u32 = undefined;
    var x891: u32 = undefined;
    mulxU32(&x890, &x891, x6, (arg2[6]));
    var x892: u32 = undefined;
    var x893: u32 = undefined;
    mulxU32(&x892, &x893, x6, (arg2[5]));
    var x894: u32 = undefined;
    var x895: u32 = undefined;
    mulxU32(&x894, &x895, x6, (arg2[4]));
    var x896: u32 = undefined;
    var x897: u32 = undefined;
    mulxU32(&x896, &x897, x6, (arg2[3]));
    var x898: u32 = undefined;
    var x899: u32 = undefined;
    mulxU32(&x898, &x899, x6, (arg2[2]));
    var x900: u32 = undefined;
    var x901: u32 = undefined;
    mulxU32(&x900, &x901, x6, (arg2[1]));
    var x902: u32 = undefined;
    var x903: u32 = undefined;
    mulxU32(&x902, &x903, x6, (arg2[0]));
    var x904: u32 = undefined;
    var x905: u1 = undefined;
    addcarryxU32(&x904, &x905, 0x0, x903, x900);
    var x906: u32 = undefined;
    var x907: u1 = undefined;
    addcarryxU32(&x906, &x907, x905, x901, x898);
    var x908: u32 = undefined;
    var x909: u1 = undefined;
    addcarryxU32(&x908, &x909, x907, x899, x896);
    var x910: u32 = undefined;
    var x911: u1 = undefined;
    addcarryxU32(&x910, &x911, x909, x897, x894);
    var x912: u32 = undefined;
    var x913: u1 = undefined;
    addcarryxU32(&x912, &x913, x911, x895, x892);
    var x914: u32 = undefined;
    var x915: u1 = undefined;
    addcarryxU32(&x914, &x915, x913, x893, x890);
    var x916: u32 = undefined;
    var x917: u1 = undefined;
    addcarryxU32(&x916, &x917, x915, x891, x888);
    var x918: u32 = undefined;
    var x919: u1 = undefined;
    addcarryxU32(&x918, &x919, x917, x889, x886);
    var x920: u32 = undefined;
    var x921: u1 = undefined;
    addcarryxU32(&x920, &x921, x919, x887, x884);
    var x922: u32 = undefined;
    var x923: u1 = undefined;
    addcarryxU32(&x922, &x923, x921, x885, x882);
    var x924: u32 = undefined;
    var x925: u1 = undefined;
    addcarryxU32(&x924, &x925, x923, x883, x880);
    const x926 = (cast(u32, x925) + x881);
    var x927: u32 = undefined;
    var x928: u1 = undefined;
    addcarryxU32(&x927, &x928, 0x0, x855, x902);
    var x929: u32 = undefined;
    var x930: u1 = undefined;
    addcarryxU32(&x929, &x930, x928, x857, x904);
    var x931: u32 = undefined;
    var x932: u1 = undefined;
    addcarryxU32(&x931, &x932, x930, x859, x906);
    var x933: u32 = undefined;
    var x934: u1 = undefined;
    addcarryxU32(&x933, &x934, x932, x861, x908);
    var x935: u32 = undefined;
    var x936: u1 = undefined;
    addcarryxU32(&x935, &x936, x934, x863, x910);
    var x937: u32 = undefined;
    var x938: u1 = undefined;
    addcarryxU32(&x937, &x938, x936, x865, x912);
    var x939: u32 = undefined;
    var x940: u1 = undefined;
    addcarryxU32(&x939, &x940, x938, x867, x914);
    var x941: u32 = undefined;
    var x942: u1 = undefined;
    addcarryxU32(&x941, &x942, x940, x869, x916);
    var x943: u32 = undefined;
    var x944: u1 = undefined;
    addcarryxU32(&x943, &x944, x942, x871, x918);
    var x945: u32 = undefined;
    var x946: u1 = undefined;
    addcarryxU32(&x945, &x946, x944, x873, x920);
    var x947: u32 = undefined;
    var x948: u1 = undefined;
    addcarryxU32(&x947, &x948, x946, x875, x922);
    var x949: u32 = undefined;
    var x950: u1 = undefined;
    addcarryxU32(&x949, &x950, x948, x877, x924);
    var x951: u32 = undefined;
    var x952: u1 = undefined;
    addcarryxU32(&x951, &x952, x950, x879, x926);
    var x953: u32 = undefined;
    var x954: u32 = undefined;
    mulxU32(&x953, &x954, x927, 0xe88fdc45);
    var x955: u32 = undefined;
    var x956: u32 = undefined;
    mulxU32(&x955, &x956, x953, 0xffffffff);
    var x957: u32 = undefined;
    var x958: u32 = undefined;
    mulxU32(&x957, &x958, x953, 0xffffffff);
    var x959: u32 = undefined;
    var x960: u32 = undefined;
    mulxU32(&x959, &x960, x953, 0xffffffff);
    var x961: u32 = undefined;
    var x962: u32 = undefined;
    mulxU32(&x961, &x962, x953, 0xffffffff);
    var x963: u32 = undefined;
    var x964: u32 = undefined;
    mulxU32(&x963, &x964, x953, 0xffffffff);
    var x965: u32 = undefined;
    var x966: u32 = undefined;
    mulxU32(&x965, &x966, x953, 0xffffffff);
    var x967: u32 = undefined;
    var x968: u32 = undefined;
    mulxU32(&x967, &x968, x953, 0xc7634d81);
    var x969: u32 = undefined;
    var x970: u32 = undefined;
    mulxU32(&x969, &x970, x953, 0xf4372ddf);
    var x971: u32 = undefined;
    var x972: u32 = undefined;
    mulxU32(&x971, &x972, x953, 0x581a0db2);
    var x973: u32 = undefined;
    var x974: u32 = undefined;
    mulxU32(&x973, &x974, x953, 0x48b0a77a);
    var x975: u32 = undefined;
    var x976: u32 = undefined;
    mulxU32(&x975, &x976, x953, 0xecec196a);
    var x977: u32 = undefined;
    var x978: u32 = undefined;
    mulxU32(&x977, &x978, x953, 0xccc52973);
    var x979: u32 = undefined;
    var x980: u1 = undefined;
    addcarryxU32(&x979, &x980, 0x0, x978, x975);
    var x981: u32 = undefined;
    var x982: u1 = undefined;
    addcarryxU32(&x981, &x982, x980, x976, x973);
    var x983: u32 = undefined;
    var x984: u1 = undefined;
    addcarryxU32(&x983, &x984, x982, x974, x971);
    var x985: u32 = undefined;
    var x986: u1 = undefined;
    addcarryxU32(&x985, &x986, x984, x972, x969);
    var x987: u32 = undefined;
    var x988: u1 = undefined;
    addcarryxU32(&x987, &x988, x986, x970, x967);
    var x989: u32 = undefined;
    var x990: u1 = undefined;
    addcarryxU32(&x989, &x990, x988, x968, x965);
    var x991: u32 = undefined;
    var x992: u1 = undefined;
    addcarryxU32(&x991, &x992, x990, x966, x963);
    var x993: u32 = undefined;
    var x994: u1 = undefined;
    addcarryxU32(&x993, &x994, x992, x964, x961);
    var x995: u32 = undefined;
    var x996: u1 = undefined;
    addcarryxU32(&x995, &x996, x994, x962, x959);
    var x997: u32 = undefined;
    var x998: u1 = undefined;
    addcarryxU32(&x997, &x998, x996, x960, x957);
    var x999: u32 = undefined;
    var x1000: u1 = undefined;
    addcarryxU32(&x999, &x1000, x998, x958, x955);
    const x1001 = (cast(u32, x1000) + x956);
    var x1002: u32 = undefined;
    var x1003: u1 = undefined;
    addcarryxU32(&x1002, &x1003, 0x0, x927, x977);
    var x1004: u32 = undefined;
    var x1005: u1 = undefined;
    addcarryxU32(&x1004, &x1005, x1003, x929, x979);
    var x1006: u32 = undefined;
    var x1007: u1 = undefined;
    addcarryxU32(&x1006, &x1007, x1005, x931, x981);
    var x1008: u32 = undefined;
    var x1009: u1 = undefined;
    addcarryxU32(&x1008, &x1009, x1007, x933, x983);
    var x1010: u32 = undefined;
    var x1011: u1 = undefined;
    addcarryxU32(&x1010, &x1011, x1009, x935, x985);
    var x1012: u32 = undefined;
    var x1013: u1 = undefined;
    addcarryxU32(&x1012, &x1013, x1011, x937, x987);
    var x1014: u32 = undefined;
    var x1015: u1 = undefined;
    addcarryxU32(&x1014, &x1015, x1013, x939, x989);
    var x1016: u32 = undefined;
    var x1017: u1 = undefined;
    addcarryxU32(&x1016, &x1017, x1015, x941, x991);
    var x1018: u32 = undefined;
    var x1019: u1 = undefined;
    addcarryxU32(&x1018, &x1019, x1017, x943, x993);
    var x1020: u32 = undefined;
    var x1021: u1 = undefined;
    addcarryxU32(&x1020, &x1021, x1019, x945, x995);
    var x1022: u32 = undefined;
    var x1023: u1 = undefined;
    addcarryxU32(&x1022, &x1023, x1021, x947, x997);
    var x1024: u32 = undefined;
    var x1025: u1 = undefined;
    addcarryxU32(&x1024, &x1025, x1023, x949, x999);
    var x1026: u32 = undefined;
    var x1027: u1 = undefined;
    addcarryxU32(&x1026, &x1027, x1025, x951, x1001);
    const x1028 = (cast(u32, x1027) + cast(u32, x952));
    var x1029: u32 = undefined;
    var x1030: u32 = undefined;
    mulxU32(&x1029, &x1030, x7, (arg2[11]));
    var x1031: u32 = undefined;
    var x1032: u32 = undefined;
    mulxU32(&x1031, &x1032, x7, (arg2[10]));
    var x1033: u32 = undefined;
    var x1034: u32 = undefined;
    mulxU32(&x1033, &x1034, x7, (arg2[9]));
    var x1035: u32 = undefined;
    var x1036: u32 = undefined;
    mulxU32(&x1035, &x1036, x7, (arg2[8]));
    var x1037: u32 = undefined;
    var x1038: u32 = undefined;
    mulxU32(&x1037, &x1038, x7, (arg2[7]));
    var x1039: u32 = undefined;
    var x1040: u32 = undefined;
    mulxU32(&x1039, &x1040, x7, (arg2[6]));
    var x1041: u32 = undefined;
    var x1042: u32 = undefined;
    mulxU32(&x1041, &x1042, x7, (arg2[5]));
    var x1043: u32 = undefined;
    var x1044: u32 = undefined;
    mulxU32(&x1043, &x1044, x7, (arg2[4]));
    var x1045: u32 = undefined;
    var x1046: u32 = undefined;
    mulxU32(&x1045, &x1046, x7, (arg2[3]));
    var x1047: u32 = undefined;
    var x1048: u32 = undefined;
    mulxU32(&x1047, &x1048, x7, (arg2[2]));
    var x1049: u32 = undefined;
    var x1050: u32 = undefined;
    mulxU32(&x1049, &x1050, x7, (arg2[1]));
    var x1051: u32 = undefined;
    var x1052: u32 = undefined;
    mulxU32(&x1051, &x1052, x7, (arg2[0]));
    var x1053: u32 = undefined;
    var x1054: u1 = undefined;
    addcarryxU32(&x1053, &x1054, 0x0, x1052, x1049);
    var x1055: u32 = undefined;
    var x1056: u1 = undefined;
    addcarryxU32(&x1055, &x1056, x1054, x1050, x1047);
    var x1057: u32 = undefined;
    var x1058: u1 = undefined;
    addcarryxU32(&x1057, &x1058, x1056, x1048, x1045);
    var x1059: u32 = undefined;
    var x1060: u1 = undefined;
    addcarryxU32(&x1059, &x1060, x1058, x1046, x1043);
    var x1061: u32 = undefined;
    var x1062: u1 = undefined;
    addcarryxU32(&x1061, &x1062, x1060, x1044, x1041);
    var x1063: u32 = undefined;
    var x1064: u1 = undefined;
    addcarryxU32(&x1063, &x1064, x1062, x1042, x1039);
    var x1065: u32 = undefined;
    var x1066: u1 = undefined;
    addcarryxU32(&x1065, &x1066, x1064, x1040, x1037);
    var x1067: u32 = undefined;
    var x1068: u1 = undefined;
    addcarryxU32(&x1067, &x1068, x1066, x1038, x1035);
    var x1069: u32 = undefined;
    var x1070: u1 = undefined;
    addcarryxU32(&x1069, &x1070, x1068, x1036, x1033);
    var x1071: u32 = undefined;
    var x1072: u1 = undefined;
    addcarryxU32(&x1071, &x1072, x1070, x1034, x1031);
    var x1073: u32 = undefined;
    var x1074: u1 = undefined;
    addcarryxU32(&x1073, &x1074, x1072, x1032, x1029);
    const x1075 = (cast(u32, x1074) + x1030);
    var x1076: u32 = undefined;
    var x1077: u1 = undefined;
    addcarryxU32(&x1076, &x1077, 0x0, x1004, x1051);
    var x1078: u32 = undefined;
    var x1079: u1 = undefined;
    addcarryxU32(&x1078, &x1079, x1077, x1006, x1053);
    var x1080: u32 = undefined;
    var x1081: u1 = undefined;
    addcarryxU32(&x1080, &x1081, x1079, x1008, x1055);
    var x1082: u32 = undefined;
    var x1083: u1 = undefined;
    addcarryxU32(&x1082, &x1083, x1081, x1010, x1057);
    var x1084: u32 = undefined;
    var x1085: u1 = undefined;
    addcarryxU32(&x1084, &x1085, x1083, x1012, x1059);
    var x1086: u32 = undefined;
    var x1087: u1 = undefined;
    addcarryxU32(&x1086, &x1087, x1085, x1014, x1061);
    var x1088: u32 = undefined;
    var x1089: u1 = undefined;
    addcarryxU32(&x1088, &x1089, x1087, x1016, x1063);
    var x1090: u32 = undefined;
    var x1091: u1 = undefined;
    addcarryxU32(&x1090, &x1091, x1089, x1018, x1065);
    var x1092: u32 = undefined;
    var x1093: u1 = undefined;
    addcarryxU32(&x1092, &x1093, x1091, x1020, x1067);
    var x1094: u32 = undefined;
    var x1095: u1 = undefined;
    addcarryxU32(&x1094, &x1095, x1093, x1022, x1069);
    var x1096: u32 = undefined;
    var x1097: u1 = undefined;
    addcarryxU32(&x1096, &x1097, x1095, x1024, x1071);
    var x1098: u32 = undefined;
    var x1099: u1 = undefined;
    addcarryxU32(&x1098, &x1099, x1097, x1026, x1073);
    var x1100: u32 = undefined;
    var x1101: u1 = undefined;
    addcarryxU32(&x1100, &x1101, x1099, x1028, x1075);
    var x1102: u32 = undefined;
    var x1103: u32 = undefined;
    mulxU32(&x1102, &x1103, x1076, 0xe88fdc45);
    var x1104: u32 = undefined;
    var x1105: u32 = undefined;
    mulxU32(&x1104, &x1105, x1102, 0xffffffff);
    var x1106: u32 = undefined;
    var x1107: u32 = undefined;
    mulxU32(&x1106, &x1107, x1102, 0xffffffff);
    var x1108: u32 = undefined;
    var x1109: u32 = undefined;
    mulxU32(&x1108, &x1109, x1102, 0xffffffff);
    var x1110: u32 = undefined;
    var x1111: u32 = undefined;
    mulxU32(&x1110, &x1111, x1102, 0xffffffff);
    var x1112: u32 = undefined;
    var x1113: u32 = undefined;
    mulxU32(&x1112, &x1113, x1102, 0xffffffff);
    var x1114: u32 = undefined;
    var x1115: u32 = undefined;
    mulxU32(&x1114, &x1115, x1102, 0xffffffff);
    var x1116: u32 = undefined;
    var x1117: u32 = undefined;
    mulxU32(&x1116, &x1117, x1102, 0xc7634d81);
    var x1118: u32 = undefined;
    var x1119: u32 = undefined;
    mulxU32(&x1118, &x1119, x1102, 0xf4372ddf);
    var x1120: u32 = undefined;
    var x1121: u32 = undefined;
    mulxU32(&x1120, &x1121, x1102, 0x581a0db2);
    var x1122: u32 = undefined;
    var x1123: u32 = undefined;
    mulxU32(&x1122, &x1123, x1102, 0x48b0a77a);
    var x1124: u32 = undefined;
    var x1125: u32 = undefined;
    mulxU32(&x1124, &x1125, x1102, 0xecec196a);
    var x1126: u32 = undefined;
    var x1127: u32 = undefined;
    mulxU32(&x1126, &x1127, x1102, 0xccc52973);
    var x1128: u32 = undefined;
    var x1129: u1 = undefined;
    addcarryxU32(&x1128, &x1129, 0x0, x1127, x1124);
    var x1130: u32 = undefined;
    var x1131: u1 = undefined;
    addcarryxU32(&x1130, &x1131, x1129, x1125, x1122);
    var x1132: u32 = undefined;
    var x1133: u1 = undefined;
    addcarryxU32(&x1132, &x1133, x1131, x1123, x1120);
    var x1134: u32 = undefined;
    var x1135: u1 = undefined;
    addcarryxU32(&x1134, &x1135, x1133, x1121, x1118);
    var x1136: u32 = undefined;
    var x1137: u1 = undefined;
    addcarryxU32(&x1136, &x1137, x1135, x1119, x1116);
    var x1138: u32 = undefined;
    var x1139: u1 = undefined;
    addcarryxU32(&x1138, &x1139, x1137, x1117, x1114);
    var x1140: u32 = undefined;
    var x1141: u1 = undefined;
    addcarryxU32(&x1140, &x1141, x1139, x1115, x1112);
    var x1142: u32 = undefined;
    var x1143: u1 = undefined;
    addcarryxU32(&x1142, &x1143, x1141, x1113, x1110);
    var x1144: u32 = undefined;
    var x1145: u1 = undefined;
    addcarryxU32(&x1144, &x1145, x1143, x1111, x1108);
    var x1146: u32 = undefined;
    var x1147: u1 = undefined;
    addcarryxU32(&x1146, &x1147, x1145, x1109, x1106);
    var x1148: u32 = undefined;
    var x1149: u1 = undefined;
    addcarryxU32(&x1148, &x1149, x1147, x1107, x1104);
    const x1150 = (cast(u32, x1149) + x1105);
    var x1151: u32 = undefined;
    var x1152: u1 = undefined;
    addcarryxU32(&x1151, &x1152, 0x0, x1076, x1126);
    var x1153: u32 = undefined;
    var x1154: u1 = undefined;
    addcarryxU32(&x1153, &x1154, x1152, x1078, x1128);
    var x1155: u32 = undefined;
    var x1156: u1 = undefined;
    addcarryxU32(&x1155, &x1156, x1154, x1080, x1130);
    var x1157: u32 = undefined;
    var x1158: u1 = undefined;
    addcarryxU32(&x1157, &x1158, x1156, x1082, x1132);
    var x1159: u32 = undefined;
    var x1160: u1 = undefined;
    addcarryxU32(&x1159, &x1160, x1158, x1084, x1134);
    var x1161: u32 = undefined;
    var x1162: u1 = undefined;
    addcarryxU32(&x1161, &x1162, x1160, x1086, x1136);
    var x1163: u32 = undefined;
    var x1164: u1 = undefined;
    addcarryxU32(&x1163, &x1164, x1162, x1088, x1138);
    var x1165: u32 = undefined;
    var x1166: u1 = undefined;
    addcarryxU32(&x1165, &x1166, x1164, x1090, x1140);
    var x1167: u32 = undefined;
    var x1168: u1 = undefined;
    addcarryxU32(&x1167, &x1168, x1166, x1092, x1142);
    var x1169: u32 = undefined;
    var x1170: u1 = undefined;
    addcarryxU32(&x1169, &x1170, x1168, x1094, x1144);
    var x1171: u32 = undefined;
    var x1172: u1 = undefined;
    addcarryxU32(&x1171, &x1172, x1170, x1096, x1146);
    var x1173: u32 = undefined;
    var x1174: u1 = undefined;
    addcarryxU32(&x1173, &x1174, x1172, x1098, x1148);
    var x1175: u32 = undefined;
    var x1176: u1 = undefined;
    addcarryxU32(&x1175, &x1176, x1174, x1100, x1150);
    const x1177 = (cast(u32, x1176) + cast(u32, x1101));
    var x1178: u32 = undefined;
    var x1179: u32 = undefined;
    mulxU32(&x1178, &x1179, x8, (arg2[11]));
    var x1180: u32 = undefined;
    var x1181: u32 = undefined;
    mulxU32(&x1180, &x1181, x8, (arg2[10]));
    var x1182: u32 = undefined;
    var x1183: u32 = undefined;
    mulxU32(&x1182, &x1183, x8, (arg2[9]));
    var x1184: u32 = undefined;
    var x1185: u32 = undefined;
    mulxU32(&x1184, &x1185, x8, (arg2[8]));
    var x1186: u32 = undefined;
    var x1187: u32 = undefined;
    mulxU32(&x1186, &x1187, x8, (arg2[7]));
    var x1188: u32 = undefined;
    var x1189: u32 = undefined;
    mulxU32(&x1188, &x1189, x8, (arg2[6]));
    var x1190: u32 = undefined;
    var x1191: u32 = undefined;
    mulxU32(&x1190, &x1191, x8, (arg2[5]));
    var x1192: u32 = undefined;
    var x1193: u32 = undefined;
    mulxU32(&x1192, &x1193, x8, (arg2[4]));
    var x1194: u32 = undefined;
    var x1195: u32 = undefined;
    mulxU32(&x1194, &x1195, x8, (arg2[3]));
    var x1196: u32 = undefined;
    var x1197: u32 = undefined;
    mulxU32(&x1196, &x1197, x8, (arg2[2]));
    var x1198: u32 = undefined;
    var x1199: u32 = undefined;
    mulxU32(&x1198, &x1199, x8, (arg2[1]));
    var x1200: u32 = undefined;
    var x1201: u32 = undefined;
    mulxU32(&x1200, &x1201, x8, (arg2[0]));
    var x1202: u32 = undefined;
    var x1203: u1 = undefined;
    addcarryxU32(&x1202, &x1203, 0x0, x1201, x1198);
    var x1204: u32 = undefined;
    var x1205: u1 = undefined;
    addcarryxU32(&x1204, &x1205, x1203, x1199, x1196);
    var x1206: u32 = undefined;
    var x1207: u1 = undefined;
    addcarryxU32(&x1206, &x1207, x1205, x1197, x1194);
    var x1208: u32 = undefined;
    var x1209: u1 = undefined;
    addcarryxU32(&x1208, &x1209, x1207, x1195, x1192);
    var x1210: u32 = undefined;
    var x1211: u1 = undefined;
    addcarryxU32(&x1210, &x1211, x1209, x1193, x1190);
    var x1212: u32 = undefined;
    var x1213: u1 = undefined;
    addcarryxU32(&x1212, &x1213, x1211, x1191, x1188);
    var x1214: u32 = undefined;
    var x1215: u1 = undefined;
    addcarryxU32(&x1214, &x1215, x1213, x1189, x1186);
    var x1216: u32 = undefined;
    var x1217: u1 = undefined;
    addcarryxU32(&x1216, &x1217, x1215, x1187, x1184);
    var x1218: u32 = undefined;
    var x1219: u1 = undefined;
    addcarryxU32(&x1218, &x1219, x1217, x1185, x1182);
    var x1220: u32 = undefined;
    var x1221: u1 = undefined;
    addcarryxU32(&x1220, &x1221, x1219, x1183, x1180);
    var x1222: u32 = undefined;
    var x1223: u1 = undefined;
    addcarryxU32(&x1222, &x1223, x1221, x1181, x1178);
    const x1224 = (cast(u32, x1223) + x1179);
    var x1225: u32 = undefined;
    var x1226: u1 = undefined;
    addcarryxU32(&x1225, &x1226, 0x0, x1153, x1200);
    var x1227: u32 = undefined;
    var x1228: u1 = undefined;
    addcarryxU32(&x1227, &x1228, x1226, x1155, x1202);
    var x1229: u32 = undefined;
    var x1230: u1 = undefined;
    addcarryxU32(&x1229, &x1230, x1228, x1157, x1204);
    var x1231: u32 = undefined;
    var x1232: u1 = undefined;
    addcarryxU32(&x1231, &x1232, x1230, x1159, x1206);
    var x1233: u32 = undefined;
    var x1234: u1 = undefined;
    addcarryxU32(&x1233, &x1234, x1232, x1161, x1208);
    var x1235: u32 = undefined;
    var x1236: u1 = undefined;
    addcarryxU32(&x1235, &x1236, x1234, x1163, x1210);
    var x1237: u32 = undefined;
    var x1238: u1 = undefined;
    addcarryxU32(&x1237, &x1238, x1236, x1165, x1212);
    var x1239: u32 = undefined;
    var x1240: u1 = undefined;
    addcarryxU32(&x1239, &x1240, x1238, x1167, x1214);
    var x1241: u32 = undefined;
    var x1242: u1 = undefined;
    addcarryxU32(&x1241, &x1242, x1240, x1169, x1216);
    var x1243: u32 = undefined;
    var x1244: u1 = undefined;
    addcarryxU32(&x1243, &x1244, x1242, x1171, x1218);
    var x1245: u32 = undefined;
    var x1246: u1 = undefined;
    addcarryxU32(&x1245, &x1246, x1244, x1173, x1220);
    var x1247: u32 = undefined;
    var x1248: u1 = undefined;
    addcarryxU32(&x1247, &x1248, x1246, x1175, x1222);
    var x1249: u32 = undefined;
    var x1250: u1 = undefined;
    addcarryxU32(&x1249, &x1250, x1248, x1177, x1224);
    var x1251: u32 = undefined;
    var x1252: u32 = undefined;
    mulxU32(&x1251, &x1252, x1225, 0xe88fdc45);
    var x1253: u32 = undefined;
    var x1254: u32 = undefined;
    mulxU32(&x1253, &x1254, x1251, 0xffffffff);
    var x1255: u32 = undefined;
    var x1256: u32 = undefined;
    mulxU32(&x1255, &x1256, x1251, 0xffffffff);
    var x1257: u32 = undefined;
    var x1258: u32 = undefined;
    mulxU32(&x1257, &x1258, x1251, 0xffffffff);
    var x1259: u32 = undefined;
    var x1260: u32 = undefined;
    mulxU32(&x1259, &x1260, x1251, 0xffffffff);
    var x1261: u32 = undefined;
    var x1262: u32 = undefined;
    mulxU32(&x1261, &x1262, x1251, 0xffffffff);
    var x1263: u32 = undefined;
    var x1264: u32 = undefined;
    mulxU32(&x1263, &x1264, x1251, 0xffffffff);
    var x1265: u32 = undefined;
    var x1266: u32 = undefined;
    mulxU32(&x1265, &x1266, x1251, 0xc7634d81);
    var x1267: u32 = undefined;
    var x1268: u32 = undefined;
    mulxU32(&x1267, &x1268, x1251, 0xf4372ddf);
    var x1269: u32 = undefined;
    var x1270: u32 = undefined;
    mulxU32(&x1269, &x1270, x1251, 0x581a0db2);
    var x1271: u32 = undefined;
    var x1272: u32 = undefined;
    mulxU32(&x1271, &x1272, x1251, 0x48b0a77a);
    var x1273: u32 = undefined;
    var x1274: u32 = undefined;
    mulxU32(&x1273, &x1274, x1251, 0xecec196a);
    var x1275: u32 = undefined;
    var x1276: u32 = undefined;
    mulxU32(&x1275, &x1276, x1251, 0xccc52973);
    var x1277: u32 = undefined;
    var x1278: u1 = undefined;
    addcarryxU32(&x1277, &x1278, 0x0, x1276, x1273);
    var x1279: u32 = undefined;
    var x1280: u1 = undefined;
    addcarryxU32(&x1279, &x1280, x1278, x1274, x1271);
    var x1281: u32 = undefined;
    var x1282: u1 = undefined;
    addcarryxU32(&x1281, &x1282, x1280, x1272, x1269);
    var x1283: u32 = undefined;
    var x1284: u1 = undefined;
    addcarryxU32(&x1283, &x1284, x1282, x1270, x1267);
    var x1285: u32 = undefined;
    var x1286: u1 = undefined;
    addcarryxU32(&x1285, &x1286, x1284, x1268, x1265);
    var x1287: u32 = undefined;
    var x1288: u1 = undefined;
    addcarryxU32(&x1287, &x1288, x1286, x1266, x1263);
    var x1289: u32 = undefined;
    var x1290: u1 = undefined;
    addcarryxU32(&x1289, &x1290, x1288, x1264, x1261);
    var x1291: u32 = undefined;
    var x1292: u1 = undefined;
    addcarryxU32(&x1291, &x1292, x1290, x1262, x1259);
    var x1293: u32 = undefined;
    var x1294: u1 = undefined;
    addcarryxU32(&x1293, &x1294, x1292, x1260, x1257);
    var x1295: u32 = undefined;
    var x1296: u1 = undefined;
    addcarryxU32(&x1295, &x1296, x1294, x1258, x1255);
    var x1297: u32 = undefined;
    var x1298: u1 = undefined;
    addcarryxU32(&x1297, &x1298, x1296, x1256, x1253);
    const x1299 = (cast(u32, x1298) + x1254);
    var x1300: u32 = undefined;
    var x1301: u1 = undefined;
    addcarryxU32(&x1300, &x1301, 0x0, x1225, x1275);
    var x1302: u32 = undefined;
    var x1303: u1 = undefined;
    addcarryxU32(&x1302, &x1303, x1301, x1227, x1277);
    var x1304: u32 = undefined;
    var x1305: u1 = undefined;
    addcarryxU32(&x1304, &x1305, x1303, x1229, x1279);
    var x1306: u32 = undefined;
    var x1307: u1 = undefined;
    addcarryxU32(&x1306, &x1307, x1305, x1231, x1281);
    var x1308: u32 = undefined;
    var x1309: u1 = undefined;
    addcarryxU32(&x1308, &x1309, x1307, x1233, x1283);
    var x1310: u32 = undefined;
    var x1311: u1 = undefined;
    addcarryxU32(&x1310, &x1311, x1309, x1235, x1285);
    var x1312: u32 = undefined;
    var x1313: u1 = undefined;
    addcarryxU32(&x1312, &x1313, x1311, x1237, x1287);
    var x1314: u32 = undefined;
    var x1315: u1 = undefined;
    addcarryxU32(&x1314, &x1315, x1313, x1239, x1289);
    var x1316: u32 = undefined;
    var x1317: u1 = undefined;
    addcarryxU32(&x1316, &x1317, x1315, x1241, x1291);
    var x1318: u32 = undefined;
    var x1319: u1 = undefined;
    addcarryxU32(&x1318, &x1319, x1317, x1243, x1293);
    var x1320: u32 = undefined;
    var x1321: u1 = undefined;
    addcarryxU32(&x1320, &x1321, x1319, x1245, x1295);
    var x1322: u32 = undefined;
    var x1323: u1 = undefined;
    addcarryxU32(&x1322, &x1323, x1321, x1247, x1297);
    var x1324: u32 = undefined;
    var x1325: u1 = undefined;
    addcarryxU32(&x1324, &x1325, x1323, x1249, x1299);
    const x1326 = (cast(u32, x1325) + cast(u32, x1250));
    var x1327: u32 = undefined;
    var x1328: u32 = undefined;
    mulxU32(&x1327, &x1328, x9, (arg2[11]));
    var x1329: u32 = undefined;
    var x1330: u32 = undefined;
    mulxU32(&x1329, &x1330, x9, (arg2[10]));
    var x1331: u32 = undefined;
    var x1332: u32 = undefined;
    mulxU32(&x1331, &x1332, x9, (arg2[9]));
    var x1333: u32 = undefined;
    var x1334: u32 = undefined;
    mulxU32(&x1333, &x1334, x9, (arg2[8]));
    var x1335: u32 = undefined;
    var x1336: u32 = undefined;
    mulxU32(&x1335, &x1336, x9, (arg2[7]));
    var x1337: u32 = undefined;
    var x1338: u32 = undefined;
    mulxU32(&x1337, &x1338, x9, (arg2[6]));
    var x1339: u32 = undefined;
    var x1340: u32 = undefined;
    mulxU32(&x1339, &x1340, x9, (arg2[5]));
    var x1341: u32 = undefined;
    var x1342: u32 = undefined;
    mulxU32(&x1341, &x1342, x9, (arg2[4]));
    var x1343: u32 = undefined;
    var x1344: u32 = undefined;
    mulxU32(&x1343, &x1344, x9, (arg2[3]));
    var x1345: u32 = undefined;
    var x1346: u32 = undefined;
    mulxU32(&x1345, &x1346, x9, (arg2[2]));
    var x1347: u32 = undefined;
    var x1348: u32 = undefined;
    mulxU32(&x1347, &x1348, x9, (arg2[1]));
    var x1349: u32 = undefined;
    var x1350: u32 = undefined;
    mulxU32(&x1349, &x1350, x9, (arg2[0]));
    var x1351: u32 = undefined;
    var x1352: u1 = undefined;
    addcarryxU32(&x1351, &x1352, 0x0, x1350, x1347);
    var x1353: u32 = undefined;
    var x1354: u1 = undefined;
    addcarryxU32(&x1353, &x1354, x1352, x1348, x1345);
    var x1355: u32 = undefined;
    var x1356: u1 = undefined;
    addcarryxU32(&x1355, &x1356, x1354, x1346, x1343);
    var x1357: u32 = undefined;
    var x1358: u1 = undefined;
    addcarryxU32(&x1357, &x1358, x1356, x1344, x1341);
    var x1359: u32 = undefined;
    var x1360: u1 = undefined;
    addcarryxU32(&x1359, &x1360, x1358, x1342, x1339);
    var x1361: u32 = undefined;
    var x1362: u1 = undefined;
    addcarryxU32(&x1361, &x1362, x1360, x1340, x1337);
    var x1363: u32 = undefined;
    var x1364: u1 = undefined;
    addcarryxU32(&x1363, &x1364, x1362, x1338, x1335);
    var x1365: u32 = undefined;
    var x1366: u1 = undefined;
    addcarryxU32(&x1365, &x1366, x1364, x1336, x1333);
    var x1367: u32 = undefined;
    var x1368: u1 = undefined;
    addcarryxU32(&x1367, &x1368, x1366, x1334, x1331);
    var x1369: u32 = undefined;
    var x1370: u1 = undefined;
    addcarryxU32(&x1369, &x1370, x1368, x1332, x1329);
    var x1371: u32 = undefined;
    var x1372: u1 = undefined;
    addcarryxU32(&x1371, &x1372, x1370, x1330, x1327);
    const x1373 = (cast(u32, x1372) + x1328);
    var x1374: u32 = undefined;
    var x1375: u1 = undefined;
    addcarryxU32(&x1374, &x1375, 0x0, x1302, x1349);
    var x1376: u32 = undefined;
    var x1377: u1 = undefined;
    addcarryxU32(&x1376, &x1377, x1375, x1304, x1351);
    var x1378: u32 = undefined;
    var x1379: u1 = undefined;
    addcarryxU32(&x1378, &x1379, x1377, x1306, x1353);
    var x1380: u32 = undefined;
    var x1381: u1 = undefined;
    addcarryxU32(&x1380, &x1381, x1379, x1308, x1355);
    var x1382: u32 = undefined;
    var x1383: u1 = undefined;
    addcarryxU32(&x1382, &x1383, x1381, x1310, x1357);
    var x1384: u32 = undefined;
    var x1385: u1 = undefined;
    addcarryxU32(&x1384, &x1385, x1383, x1312, x1359);
    var x1386: u32 = undefined;
    var x1387: u1 = undefined;
    addcarryxU32(&x1386, &x1387, x1385, x1314, x1361);
    var x1388: u32 = undefined;
    var x1389: u1 = undefined;
    addcarryxU32(&x1388, &x1389, x1387, x1316, x1363);
    var x1390: u32 = undefined;
    var x1391: u1 = undefined;
    addcarryxU32(&x1390, &x1391, x1389, x1318, x1365);
    var x1392: u32 = undefined;
    var x1393: u1 = undefined;
    addcarryxU32(&x1392, &x1393, x1391, x1320, x1367);
    var x1394: u32 = undefined;
    var x1395: u1 = undefined;
    addcarryxU32(&x1394, &x1395, x1393, x1322, x1369);
    var x1396: u32 = undefined;
    var x1397: u1 = undefined;
    addcarryxU32(&x1396, &x1397, x1395, x1324, x1371);
    var x1398: u32 = undefined;
    var x1399: u1 = undefined;
    addcarryxU32(&x1398, &x1399, x1397, x1326, x1373);
    var x1400: u32 = undefined;
    var x1401: u32 = undefined;
    mulxU32(&x1400, &x1401, x1374, 0xe88fdc45);
    var x1402: u32 = undefined;
    var x1403: u32 = undefined;
    mulxU32(&x1402, &x1403, x1400, 0xffffffff);
    var x1404: u32 = undefined;
    var x1405: u32 = undefined;
    mulxU32(&x1404, &x1405, x1400, 0xffffffff);
    var x1406: u32 = undefined;
    var x1407: u32 = undefined;
    mulxU32(&x1406, &x1407, x1400, 0xffffffff);
    var x1408: u32 = undefined;
    var x1409: u32 = undefined;
    mulxU32(&x1408, &x1409, x1400, 0xffffffff);
    var x1410: u32 = undefined;
    var x1411: u32 = undefined;
    mulxU32(&x1410, &x1411, x1400, 0xffffffff);
    var x1412: u32 = undefined;
    var x1413: u32 = undefined;
    mulxU32(&x1412, &x1413, x1400, 0xffffffff);
    var x1414: u32 = undefined;
    var x1415: u32 = undefined;
    mulxU32(&x1414, &x1415, x1400, 0xc7634d81);
    var x1416: u32 = undefined;
    var x1417: u32 = undefined;
    mulxU32(&x1416, &x1417, x1400, 0xf4372ddf);
    var x1418: u32 = undefined;
    var x1419: u32 = undefined;
    mulxU32(&x1418, &x1419, x1400, 0x581a0db2);
    var x1420: u32 = undefined;
    var x1421: u32 = undefined;
    mulxU32(&x1420, &x1421, x1400, 0x48b0a77a);
    var x1422: u32 = undefined;
    var x1423: u32 = undefined;
    mulxU32(&x1422, &x1423, x1400, 0xecec196a);
    var x1424: u32 = undefined;
    var x1425: u32 = undefined;
    mulxU32(&x1424, &x1425, x1400, 0xccc52973);
    var x1426: u32 = undefined;
    var x1427: u1 = undefined;
    addcarryxU32(&x1426, &x1427, 0x0, x1425, x1422);
    var x1428: u32 = undefined;
    var x1429: u1 = undefined;
    addcarryxU32(&x1428, &x1429, x1427, x1423, x1420);
    var x1430: u32 = undefined;
    var x1431: u1 = undefined;
    addcarryxU32(&x1430, &x1431, x1429, x1421, x1418);
    var x1432: u32 = undefined;
    var x1433: u1 = undefined;
    addcarryxU32(&x1432, &x1433, x1431, x1419, x1416);
    var x1434: u32 = undefined;
    var x1435: u1 = undefined;
    addcarryxU32(&x1434, &x1435, x1433, x1417, x1414);
    var x1436: u32 = undefined;
    var x1437: u1 = undefined;
    addcarryxU32(&x1436, &x1437, x1435, x1415, x1412);
    var x1438: u32 = undefined;
    var x1439: u1 = undefined;
    addcarryxU32(&x1438, &x1439, x1437, x1413, x1410);
    var x1440: u32 = undefined;
    var x1441: u1 = undefined;
    addcarryxU32(&x1440, &x1441, x1439, x1411, x1408);
    var x1442: u32 = undefined;
    var x1443: u1 = undefined;
    addcarryxU32(&x1442, &x1443, x1441, x1409, x1406);
    var x1444: u32 = undefined;
    var x1445: u1 = undefined;
    addcarryxU32(&x1444, &x1445, x1443, x1407, x1404);
    var x1446: u32 = undefined;
    var x1447: u1 = undefined;
    addcarryxU32(&x1446, &x1447, x1445, x1405, x1402);
    const x1448 = (cast(u32, x1447) + x1403);
    var x1449: u32 = undefined;
    var x1450: u1 = undefined;
    addcarryxU32(&x1449, &x1450, 0x0, x1374, x1424);
    var x1451: u32 = undefined;
    var x1452: u1 = undefined;
    addcarryxU32(&x1451, &x1452, x1450, x1376, x1426);
    var x1453: u32 = undefined;
    var x1454: u1 = undefined;
    addcarryxU32(&x1453, &x1454, x1452, x1378, x1428);
    var x1455: u32 = undefined;
    var x1456: u1 = undefined;
    addcarryxU32(&x1455, &x1456, x1454, x1380, x1430);
    var x1457: u32 = undefined;
    var x1458: u1 = undefined;
    addcarryxU32(&x1457, &x1458, x1456, x1382, x1432);
    var x1459: u32 = undefined;
    var x1460: u1 = undefined;
    addcarryxU32(&x1459, &x1460, x1458, x1384, x1434);
    var x1461: u32 = undefined;
    var x1462: u1 = undefined;
    addcarryxU32(&x1461, &x1462, x1460, x1386, x1436);
    var x1463: u32 = undefined;
    var x1464: u1 = undefined;
    addcarryxU32(&x1463, &x1464, x1462, x1388, x1438);
    var x1465: u32 = undefined;
    var x1466: u1 = undefined;
    addcarryxU32(&x1465, &x1466, x1464, x1390, x1440);
    var x1467: u32 = undefined;
    var x1468: u1 = undefined;
    addcarryxU32(&x1467, &x1468, x1466, x1392, x1442);
    var x1469: u32 = undefined;
    var x1470: u1 = undefined;
    addcarryxU32(&x1469, &x1470, x1468, x1394, x1444);
    var x1471: u32 = undefined;
    var x1472: u1 = undefined;
    addcarryxU32(&x1471, &x1472, x1470, x1396, x1446);
    var x1473: u32 = undefined;
    var x1474: u1 = undefined;
    addcarryxU32(&x1473, &x1474, x1472, x1398, x1448);
    const x1475 = (cast(u32, x1474) + cast(u32, x1399));
    var x1476: u32 = undefined;
    var x1477: u32 = undefined;
    mulxU32(&x1476, &x1477, x10, (arg2[11]));
    var x1478: u32 = undefined;
    var x1479: u32 = undefined;
    mulxU32(&x1478, &x1479, x10, (arg2[10]));
    var x1480: u32 = undefined;
    var x1481: u32 = undefined;
    mulxU32(&x1480, &x1481, x10, (arg2[9]));
    var x1482: u32 = undefined;
    var x1483: u32 = undefined;
    mulxU32(&x1482, &x1483, x10, (arg2[8]));
    var x1484: u32 = undefined;
    var x1485: u32 = undefined;
    mulxU32(&x1484, &x1485, x10, (arg2[7]));
    var x1486: u32 = undefined;
    var x1487: u32 = undefined;
    mulxU32(&x1486, &x1487, x10, (arg2[6]));
    var x1488: u32 = undefined;
    var x1489: u32 = undefined;
    mulxU32(&x1488, &x1489, x10, (arg2[5]));
    var x1490: u32 = undefined;
    var x1491: u32 = undefined;
    mulxU32(&x1490, &x1491, x10, (arg2[4]));
    var x1492: u32 = undefined;
    var x1493: u32 = undefined;
    mulxU32(&x1492, &x1493, x10, (arg2[3]));
    var x1494: u32 = undefined;
    var x1495: u32 = undefined;
    mulxU32(&x1494, &x1495, x10, (arg2[2]));
    var x1496: u32 = undefined;
    var x1497: u32 = undefined;
    mulxU32(&x1496, &x1497, x10, (arg2[1]));
    var x1498: u32 = undefined;
    var x1499: u32 = undefined;
    mulxU32(&x1498, &x1499, x10, (arg2[0]));
    var x1500: u32 = undefined;
    var x1501: u1 = undefined;
    addcarryxU32(&x1500, &x1501, 0x0, x1499, x1496);
    var x1502: u32 = undefined;
    var x1503: u1 = undefined;
    addcarryxU32(&x1502, &x1503, x1501, x1497, x1494);
    var x1504: u32 = undefined;
    var x1505: u1 = undefined;
    addcarryxU32(&x1504, &x1505, x1503, x1495, x1492);
    var x1506: u32 = undefined;
    var x1507: u1 = undefined;
    addcarryxU32(&x1506, &x1507, x1505, x1493, x1490);
    var x1508: u32 = undefined;
    var x1509: u1 = undefined;
    addcarryxU32(&x1508, &x1509, x1507, x1491, x1488);
    var x1510: u32 = undefined;
    var x1511: u1 = undefined;
    addcarryxU32(&x1510, &x1511, x1509, x1489, x1486);
    var x1512: u32 = undefined;
    var x1513: u1 = undefined;
    addcarryxU32(&x1512, &x1513, x1511, x1487, x1484);
    var x1514: u32 = undefined;
    var x1515: u1 = undefined;
    addcarryxU32(&x1514, &x1515, x1513, x1485, x1482);
    var x1516: u32 = undefined;
    var x1517: u1 = undefined;
    addcarryxU32(&x1516, &x1517, x1515, x1483, x1480);
    var x1518: u32 = undefined;
    var x1519: u1 = undefined;
    addcarryxU32(&x1518, &x1519, x1517, x1481, x1478);
    var x1520: u32 = undefined;
    var x1521: u1 = undefined;
    addcarryxU32(&x1520, &x1521, x1519, x1479, x1476);
    const x1522 = (cast(u32, x1521) + x1477);
    var x1523: u32 = undefined;
    var x1524: u1 = undefined;
    addcarryxU32(&x1523, &x1524, 0x0, x1451, x1498);
    var x1525: u32 = undefined;
    var x1526: u1 = undefined;
    addcarryxU32(&x1525, &x1526, x1524, x1453, x1500);
    var x1527: u32 = undefined;
    var x1528: u1 = undefined;
    addcarryxU32(&x1527, &x1528, x1526, x1455, x1502);
    var x1529: u32 = undefined;
    var x1530: u1 = undefined;
    addcarryxU32(&x1529, &x1530, x1528, x1457, x1504);
    var x1531: u32 = undefined;
    var x1532: u1 = undefined;
    addcarryxU32(&x1531, &x1532, x1530, x1459, x1506);
    var x1533: u32 = undefined;
    var x1534: u1 = undefined;
    addcarryxU32(&x1533, &x1534, x1532, x1461, x1508);
    var x1535: u32 = undefined;
    var x1536: u1 = undefined;
    addcarryxU32(&x1535, &x1536, x1534, x1463, x1510);
    var x1537: u32 = undefined;
    var x1538: u1 = undefined;
    addcarryxU32(&x1537, &x1538, x1536, x1465, x1512);
    var x1539: u32 = undefined;
    var x1540: u1 = undefined;
    addcarryxU32(&x1539, &x1540, x1538, x1467, x1514);
    var x1541: u32 = undefined;
    var x1542: u1 = undefined;
    addcarryxU32(&x1541, &x1542, x1540, x1469, x1516);
    var x1543: u32 = undefined;
    var x1544: u1 = undefined;
    addcarryxU32(&x1543, &x1544, x1542, x1471, x1518);
    var x1545: u32 = undefined;
    var x1546: u1 = undefined;
    addcarryxU32(&x1545, &x1546, x1544, x1473, x1520);
    var x1547: u32 = undefined;
    var x1548: u1 = undefined;
    addcarryxU32(&x1547, &x1548, x1546, x1475, x1522);
    var x1549: u32 = undefined;
    var x1550: u32 = undefined;
    mulxU32(&x1549, &x1550, x1523, 0xe88fdc45);
    var x1551: u32 = undefined;
    var x1552: u32 = undefined;
    mulxU32(&x1551, &x1552, x1549, 0xffffffff);
    var x1553: u32 = undefined;
    var x1554: u32 = undefined;
    mulxU32(&x1553, &x1554, x1549, 0xffffffff);
    var x1555: u32 = undefined;
    var x1556: u32 = undefined;
    mulxU32(&x1555, &x1556, x1549, 0xffffffff);
    var x1557: u32 = undefined;
    var x1558: u32 = undefined;
    mulxU32(&x1557, &x1558, x1549, 0xffffffff);
    var x1559: u32 = undefined;
    var x1560: u32 = undefined;
    mulxU32(&x1559, &x1560, x1549, 0xffffffff);
    var x1561: u32 = undefined;
    var x1562: u32 = undefined;
    mulxU32(&x1561, &x1562, x1549, 0xffffffff);
    var x1563: u32 = undefined;
    var x1564: u32 = undefined;
    mulxU32(&x1563, &x1564, x1549, 0xc7634d81);
    var x1565: u32 = undefined;
    var x1566: u32 = undefined;
    mulxU32(&x1565, &x1566, x1549, 0xf4372ddf);
    var x1567: u32 = undefined;
    var x1568: u32 = undefined;
    mulxU32(&x1567, &x1568, x1549, 0x581a0db2);
    var x1569: u32 = undefined;
    var x1570: u32 = undefined;
    mulxU32(&x1569, &x1570, x1549, 0x48b0a77a);
    var x1571: u32 = undefined;
    var x1572: u32 = undefined;
    mulxU32(&x1571, &x1572, x1549, 0xecec196a);
    var x1573: u32 = undefined;
    var x1574: u32 = undefined;
    mulxU32(&x1573, &x1574, x1549, 0xccc52973);
    var x1575: u32 = undefined;
    var x1576: u1 = undefined;
    addcarryxU32(&x1575, &x1576, 0x0, x1574, x1571);
    var x1577: u32 = undefined;
    var x1578: u1 = undefined;
    addcarryxU32(&x1577, &x1578, x1576, x1572, x1569);
    var x1579: u32 = undefined;
    var x1580: u1 = undefined;
    addcarryxU32(&x1579, &x1580, x1578, x1570, x1567);
    var x1581: u32 = undefined;
    var x1582: u1 = undefined;
    addcarryxU32(&x1581, &x1582, x1580, x1568, x1565);
    var x1583: u32 = undefined;
    var x1584: u1 = undefined;
    addcarryxU32(&x1583, &x1584, x1582, x1566, x1563);
    var x1585: u32 = undefined;
    var x1586: u1 = undefined;
    addcarryxU32(&x1585, &x1586, x1584, x1564, x1561);
    var x1587: u32 = undefined;
    var x1588: u1 = undefined;
    addcarryxU32(&x1587, &x1588, x1586, x1562, x1559);
    var x1589: u32 = undefined;
    var x1590: u1 = undefined;
    addcarryxU32(&x1589, &x1590, x1588, x1560, x1557);
    var x1591: u32 = undefined;
    var x1592: u1 = undefined;
    addcarryxU32(&x1591, &x1592, x1590, x1558, x1555);
    var x1593: u32 = undefined;
    var x1594: u1 = undefined;
    addcarryxU32(&x1593, &x1594, x1592, x1556, x1553);
    var x1595: u32 = undefined;
    var x1596: u1 = undefined;
    addcarryxU32(&x1595, &x1596, x1594, x1554, x1551);
    const x1597 = (cast(u32, x1596) + x1552);
    var x1598: u32 = undefined;
    var x1599: u1 = undefined;
    addcarryxU32(&x1598, &x1599, 0x0, x1523, x1573);
    var x1600: u32 = undefined;
    var x1601: u1 = undefined;
    addcarryxU32(&x1600, &x1601, x1599, x1525, x1575);
    var x1602: u32 = undefined;
    var x1603: u1 = undefined;
    addcarryxU32(&x1602, &x1603, x1601, x1527, x1577);
    var x1604: u32 = undefined;
    var x1605: u1 = undefined;
    addcarryxU32(&x1604, &x1605, x1603, x1529, x1579);
    var x1606: u32 = undefined;
    var x1607: u1 = undefined;
    addcarryxU32(&x1606, &x1607, x1605, x1531, x1581);
    var x1608: u32 = undefined;
    var x1609: u1 = undefined;
    addcarryxU32(&x1608, &x1609, x1607, x1533, x1583);
    var x1610: u32 = undefined;
    var x1611: u1 = undefined;
    addcarryxU32(&x1610, &x1611, x1609, x1535, x1585);
    var x1612: u32 = undefined;
    var x1613: u1 = undefined;
    addcarryxU32(&x1612, &x1613, x1611, x1537, x1587);
    var x1614: u32 = undefined;
    var x1615: u1 = undefined;
    addcarryxU32(&x1614, &x1615, x1613, x1539, x1589);
    var x1616: u32 = undefined;
    var x1617: u1 = undefined;
    addcarryxU32(&x1616, &x1617, x1615, x1541, x1591);
    var x1618: u32 = undefined;
    var x1619: u1 = undefined;
    addcarryxU32(&x1618, &x1619, x1617, x1543, x1593);
    var x1620: u32 = undefined;
    var x1621: u1 = undefined;
    addcarryxU32(&x1620, &x1621, x1619, x1545, x1595);
    var x1622: u32 = undefined;
    var x1623: u1 = undefined;
    addcarryxU32(&x1622, &x1623, x1621, x1547, x1597);
    const x1624 = (cast(u32, x1623) + cast(u32, x1548));
    var x1625: u32 = undefined;
    var x1626: u32 = undefined;
    mulxU32(&x1625, &x1626, x11, (arg2[11]));
    var x1627: u32 = undefined;
    var x1628: u32 = undefined;
    mulxU32(&x1627, &x1628, x11, (arg2[10]));
    var x1629: u32 = undefined;
    var x1630: u32 = undefined;
    mulxU32(&x1629, &x1630, x11, (arg2[9]));
    var x1631: u32 = undefined;
    var x1632: u32 = undefined;
    mulxU32(&x1631, &x1632, x11, (arg2[8]));
    var x1633: u32 = undefined;
    var x1634: u32 = undefined;
    mulxU32(&x1633, &x1634, x11, (arg2[7]));
    var x1635: u32 = undefined;
    var x1636: u32 = undefined;
    mulxU32(&x1635, &x1636, x11, (arg2[6]));
    var x1637: u32 = undefined;
    var x1638: u32 = undefined;
    mulxU32(&x1637, &x1638, x11, (arg2[5]));
    var x1639: u32 = undefined;
    var x1640: u32 = undefined;
    mulxU32(&x1639, &x1640, x11, (arg2[4]));
    var x1641: u32 = undefined;
    var x1642: u32 = undefined;
    mulxU32(&x1641, &x1642, x11, (arg2[3]));
    var x1643: u32 = undefined;
    var x1644: u32 = undefined;
    mulxU32(&x1643, &x1644, x11, (arg2[2]));
    var x1645: u32 = undefined;
    var x1646: u32 = undefined;
    mulxU32(&x1645, &x1646, x11, (arg2[1]));
    var x1647: u32 = undefined;
    var x1648: u32 = undefined;
    mulxU32(&x1647, &x1648, x11, (arg2[0]));
    var x1649: u32 = undefined;
    var x1650: u1 = undefined;
    addcarryxU32(&x1649, &x1650, 0x0, x1648, x1645);
    var x1651: u32 = undefined;
    var x1652: u1 = undefined;
    addcarryxU32(&x1651, &x1652, x1650, x1646, x1643);
    var x1653: u32 = undefined;
    var x1654: u1 = undefined;
    addcarryxU32(&x1653, &x1654, x1652, x1644, x1641);
    var x1655: u32 = undefined;
    var x1656: u1 = undefined;
    addcarryxU32(&x1655, &x1656, x1654, x1642, x1639);
    var x1657: u32 = undefined;
    var x1658: u1 = undefined;
    addcarryxU32(&x1657, &x1658, x1656, x1640, x1637);
    var x1659: u32 = undefined;
    var x1660: u1 = undefined;
    addcarryxU32(&x1659, &x1660, x1658, x1638, x1635);
    var x1661: u32 = undefined;
    var x1662: u1 = undefined;
    addcarryxU32(&x1661, &x1662, x1660, x1636, x1633);
    var x1663: u32 = undefined;
    var x1664: u1 = undefined;
    addcarryxU32(&x1663, &x1664, x1662, x1634, x1631);
    var x1665: u32 = undefined;
    var x1666: u1 = undefined;
    addcarryxU32(&x1665, &x1666, x1664, x1632, x1629);
    var x1667: u32 = undefined;
    var x1668: u1 = undefined;
    addcarryxU32(&x1667, &x1668, x1666, x1630, x1627);
    var x1669: u32 = undefined;
    var x1670: u1 = undefined;
    addcarryxU32(&x1669, &x1670, x1668, x1628, x1625);
    const x1671 = (cast(u32, x1670) + x1626);
    var x1672: u32 = undefined;
    var x1673: u1 = undefined;
    addcarryxU32(&x1672, &x1673, 0x0, x1600, x1647);
    var x1674: u32 = undefined;
    var x1675: u1 = undefined;
    addcarryxU32(&x1674, &x1675, x1673, x1602, x1649);
    var x1676: u32 = undefined;
    var x1677: u1 = undefined;
    addcarryxU32(&x1676, &x1677, x1675, x1604, x1651);
    var x1678: u32 = undefined;
    var x1679: u1 = undefined;
    addcarryxU32(&x1678, &x1679, x1677, x1606, x1653);
    var x1680: u32 = undefined;
    var x1681: u1 = undefined;
    addcarryxU32(&x1680, &x1681, x1679, x1608, x1655);
    var x1682: u32 = undefined;
    var x1683: u1 = undefined;
    addcarryxU32(&x1682, &x1683, x1681, x1610, x1657);
    var x1684: u32 = undefined;
    var x1685: u1 = undefined;
    addcarryxU32(&x1684, &x1685, x1683, x1612, x1659);
    var x1686: u32 = undefined;
    var x1687: u1 = undefined;
    addcarryxU32(&x1686, &x1687, x1685, x1614, x1661);
    var x1688: u32 = undefined;
    var x1689: u1 = undefined;
    addcarryxU32(&x1688, &x1689, x1687, x1616, x1663);
    var x1690: u32 = undefined;
    var x1691: u1 = undefined;
    addcarryxU32(&x1690, &x1691, x1689, x1618, x1665);
    var x1692: u32 = undefined;
    var x1693: u1 = undefined;
    addcarryxU32(&x1692, &x1693, x1691, x1620, x1667);
    var x1694: u32 = undefined;
    var x1695: u1 = undefined;
    addcarryxU32(&x1694, &x1695, x1693, x1622, x1669);
    var x1696: u32 = undefined;
    var x1697: u1 = undefined;
    addcarryxU32(&x1696, &x1697, x1695, x1624, x1671);
    var x1698: u32 = undefined;
    var x1699: u32 = undefined;
    mulxU32(&x1698, &x1699, x1672, 0xe88fdc45);
    var x1700: u32 = undefined;
    var x1701: u32 = undefined;
    mulxU32(&x1700, &x1701, x1698, 0xffffffff);
    var x1702: u32 = undefined;
    var x1703: u32 = undefined;
    mulxU32(&x1702, &x1703, x1698, 0xffffffff);
    var x1704: u32 = undefined;
    var x1705: u32 = undefined;
    mulxU32(&x1704, &x1705, x1698, 0xffffffff);
    var x1706: u32 = undefined;
    var x1707: u32 = undefined;
    mulxU32(&x1706, &x1707, x1698, 0xffffffff);
    var x1708: u32 = undefined;
    var x1709: u32 = undefined;
    mulxU32(&x1708, &x1709, x1698, 0xffffffff);
    var x1710: u32 = undefined;
    var x1711: u32 = undefined;
    mulxU32(&x1710, &x1711, x1698, 0xffffffff);
    var x1712: u32 = undefined;
    var x1713: u32 = undefined;
    mulxU32(&x1712, &x1713, x1698, 0xc7634d81);
    var x1714: u32 = undefined;
    var x1715: u32 = undefined;
    mulxU32(&x1714, &x1715, x1698, 0xf4372ddf);
    var x1716: u32 = undefined;
    var x1717: u32 = undefined;
    mulxU32(&x1716, &x1717, x1698, 0x581a0db2);
    var x1718: u32 = undefined;
    var x1719: u32 = undefined;
    mulxU32(&x1718, &x1719, x1698, 0x48b0a77a);
    var x1720: u32 = undefined;
    var x1721: u32 = undefined;
    mulxU32(&x1720, &x1721, x1698, 0xecec196a);
    var x1722: u32 = undefined;
    var x1723: u32 = undefined;
    mulxU32(&x1722, &x1723, x1698, 0xccc52973);
    var x1724: u32 = undefined;
    var x1725: u1 = undefined;
    addcarryxU32(&x1724, &x1725, 0x0, x1723, x1720);
    var x1726: u32 = undefined;
    var x1727: u1 = undefined;
    addcarryxU32(&x1726, &x1727, x1725, x1721, x1718);
    var x1728: u32 = undefined;
    var x1729: u1 = undefined;
    addcarryxU32(&x1728, &x1729, x1727, x1719, x1716);
    var x1730: u32 = undefined;
    var x1731: u1 = undefined;
    addcarryxU32(&x1730, &x1731, x1729, x1717, x1714);
    var x1732: u32 = undefined;
    var x1733: u1 = undefined;
    addcarryxU32(&x1732, &x1733, x1731, x1715, x1712);
    var x1734: u32 = undefined;
    var x1735: u1 = undefined;
    addcarryxU32(&x1734, &x1735, x1733, x1713, x1710);
    var x1736: u32 = undefined;
    var x1737: u1 = undefined;
    addcarryxU32(&x1736, &x1737, x1735, x1711, x1708);
    var x1738: u32 = undefined;
    var x1739: u1 = undefined;
    addcarryxU32(&x1738, &x1739, x1737, x1709, x1706);
    var x1740: u32 = undefined;
    var x1741: u1 = undefined;
    addcarryxU32(&x1740, &x1741, x1739, x1707, x1704);
    var x1742: u32 = undefined;
    var x1743: u1 = undefined;
    addcarryxU32(&x1742, &x1743, x1741, x1705, x1702);
    var x1744: u32 = undefined;
    var x1745: u1 = undefined;
    addcarryxU32(&x1744, &x1745, x1743, x1703, x1700);
    const x1746 = (cast(u32, x1745) + x1701);
    var x1747: u32 = undefined;
    var x1748: u1 = undefined;
    addcarryxU32(&x1747, &x1748, 0x0, x1672, x1722);
    var x1749: u32 = undefined;
    var x1750: u1 = undefined;
    addcarryxU32(&x1749, &x1750, x1748, x1674, x1724);
    var x1751: u32 = undefined;
    var x1752: u1 = undefined;
    addcarryxU32(&x1751, &x1752, x1750, x1676, x1726);
    var x1753: u32 = undefined;
    var x1754: u1 = undefined;
    addcarryxU32(&x1753, &x1754, x1752, x1678, x1728);
    var x1755: u32 = undefined;
    var x1756: u1 = undefined;
    addcarryxU32(&x1755, &x1756, x1754, x1680, x1730);
    var x1757: u32 = undefined;
    var x1758: u1 = undefined;
    addcarryxU32(&x1757, &x1758, x1756, x1682, x1732);
    var x1759: u32 = undefined;
    var x1760: u1 = undefined;
    addcarryxU32(&x1759, &x1760, x1758, x1684, x1734);
    var x1761: u32 = undefined;
    var x1762: u1 = undefined;
    addcarryxU32(&x1761, &x1762, x1760, x1686, x1736);
    var x1763: u32 = undefined;
    var x1764: u1 = undefined;
    addcarryxU32(&x1763, &x1764, x1762, x1688, x1738);
    var x1765: u32 = undefined;
    var x1766: u1 = undefined;
    addcarryxU32(&x1765, &x1766, x1764, x1690, x1740);
    var x1767: u32 = undefined;
    var x1768: u1 = undefined;
    addcarryxU32(&x1767, &x1768, x1766, x1692, x1742);
    var x1769: u32 = undefined;
    var x1770: u1 = undefined;
    addcarryxU32(&x1769, &x1770, x1768, x1694, x1744);
    var x1771: u32 = undefined;
    var x1772: u1 = undefined;
    addcarryxU32(&x1771, &x1772, x1770, x1696, x1746);
    const x1773 = (cast(u32, x1772) + cast(u32, x1697));
    var x1774: u32 = undefined;
    var x1775: u1 = undefined;
    subborrowxU32(&x1774, &x1775, 0x0, x1749, 0xccc52973);
    var x1776: u32 = undefined;
    var x1777: u1 = undefined;
    subborrowxU32(&x1776, &x1777, x1775, x1751, 0xecec196a);
    var x1778: u32 = undefined;
    var x1779: u1 = undefined;
    subborrowxU32(&x1778, &x1779, x1777, x1753, 0x48b0a77a);
    var x1780: u32 = undefined;
    var x1781: u1 = undefined;
    subborrowxU32(&x1780, &x1781, x1779, x1755, 0x581a0db2);
    var x1782: u32 = undefined;
    var x1783: u1 = undefined;
    subborrowxU32(&x1782, &x1783, x1781, x1757, 0xf4372ddf);
    var x1784: u32 = undefined;
    var x1785: u1 = undefined;
    subborrowxU32(&x1784, &x1785, x1783, x1759, 0xc7634d81);
    var x1786: u32 = undefined;
    var x1787: u1 = undefined;
    subborrowxU32(&x1786, &x1787, x1785, x1761, 0xffffffff);
    var x1788: u32 = undefined;
    var x1789: u1 = undefined;
    subborrowxU32(&x1788, &x1789, x1787, x1763, 0xffffffff);
    var x1790: u32 = undefined;
    var x1791: u1 = undefined;
    subborrowxU32(&x1790, &x1791, x1789, x1765, 0xffffffff);
    var x1792: u32 = undefined;
    var x1793: u1 = undefined;
    subborrowxU32(&x1792, &x1793, x1791, x1767, 0xffffffff);
    var x1794: u32 = undefined;
    var x1795: u1 = undefined;
    subborrowxU32(&x1794, &x1795, x1793, x1769, 0xffffffff);
    var x1796: u32 = undefined;
    var x1797: u1 = undefined;
    subborrowxU32(&x1796, &x1797, x1795, x1771, 0xffffffff);
    var x1798: u32 = undefined;
    var x1799: u1 = undefined;
    subborrowxU32(&x1798, &x1799, x1797, x1773, cast(u32, 0x0));
    var x1800: u32 = undefined;
    cmovznzU32(&x1800, x1799, x1774, x1749);
    var x1801: u32 = undefined;
    cmovznzU32(&x1801, x1799, x1776, x1751);
    var x1802: u32 = undefined;
    cmovznzU32(&x1802, x1799, x1778, x1753);
    var x1803: u32 = undefined;
    cmovznzU32(&x1803, x1799, x1780, x1755);
    var x1804: u32 = undefined;
    cmovznzU32(&x1804, x1799, x1782, x1757);
    var x1805: u32 = undefined;
    cmovznzU32(&x1805, x1799, x1784, x1759);
    var x1806: u32 = undefined;
    cmovznzU32(&x1806, x1799, x1786, x1761);
    var x1807: u32 = undefined;
    cmovznzU32(&x1807, x1799, x1788, x1763);
    var x1808: u32 = undefined;
    cmovznzU32(&x1808, x1799, x1790, x1765);
    var x1809: u32 = undefined;
    cmovznzU32(&x1809, x1799, x1792, x1767);
    var x1810: u32 = undefined;
    cmovznzU32(&x1810, x1799, x1794, x1769);
    var x1811: u32 = undefined;
    cmovznzU32(&x1811, x1799, x1796, x1771);
    out1[0] = x1800;
    out1[1] = x1801;
    out1[2] = x1802;
    out1[3] = x1803;
    out1[4] = x1804;
    out1[5] = x1805;
    out1[6] = x1806;
    out1[7] = x1807;
    out1[8] = x1808;
    out1[9] = x1809;
    out1[10] = x1810;
    out1[11] = x1811;
}

/// The function square squares a field element in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) * eval (from_montgomery arg1)) mod m
///   0 ≤ eval out1 < m
///
pub fn square(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[1]);
    const x2 = (arg1[2]);
    const x3 = (arg1[3]);
    const x4 = (arg1[4]);
    const x5 = (arg1[5]);
    const x6 = (arg1[6]);
    const x7 = (arg1[7]);
    const x8 = (arg1[8]);
    const x9 = (arg1[9]);
    const x10 = (arg1[10]);
    const x11 = (arg1[11]);
    const x12 = (arg1[0]);
    var x13: u32 = undefined;
    var x14: u32 = undefined;
    mulxU32(&x13, &x14, x12, (arg1[11]));
    var x15: u32 = undefined;
    var x16: u32 = undefined;
    mulxU32(&x15, &x16, x12, (arg1[10]));
    var x17: u32 = undefined;
    var x18: u32 = undefined;
    mulxU32(&x17, &x18, x12, (arg1[9]));
    var x19: u32 = undefined;
    var x20: u32 = undefined;
    mulxU32(&x19, &x20, x12, (arg1[8]));
    var x21: u32 = undefined;
    var x22: u32 = undefined;
    mulxU32(&x21, &x22, x12, (arg1[7]));
    var x23: u32 = undefined;
    var x24: u32 = undefined;
    mulxU32(&x23, &x24, x12, (arg1[6]));
    var x25: u32 = undefined;
    var x26: u32 = undefined;
    mulxU32(&x25, &x26, x12, (arg1[5]));
    var x27: u32 = undefined;
    var x28: u32 = undefined;
    mulxU32(&x27, &x28, x12, (arg1[4]));
    var x29: u32 = undefined;
    var x30: u32 = undefined;
    mulxU32(&x29, &x30, x12, (arg1[3]));
    var x31: u32 = undefined;
    var x32: u32 = undefined;
    mulxU32(&x31, &x32, x12, (arg1[2]));
    var x33: u32 = undefined;
    var x34: u32 = undefined;
    mulxU32(&x33, &x34, x12, (arg1[1]));
    var x35: u32 = undefined;
    var x36: u32 = undefined;
    mulxU32(&x35, &x36, x12, (arg1[0]));
    var x37: u32 = undefined;
    var x38: u1 = undefined;
    addcarryxU32(&x37, &x38, 0x0, x36, x33);
    var x39: u32 = undefined;
    var x40: u1 = undefined;
    addcarryxU32(&x39, &x40, x38, x34, x31);
    var x41: u32 = undefined;
    var x42: u1 = undefined;
    addcarryxU32(&x41, &x42, x40, x32, x29);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    addcarryxU32(&x43, &x44, x42, x30, x27);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    addcarryxU32(&x45, &x46, x44, x28, x25);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    addcarryxU32(&x47, &x48, x46, x26, x23);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    addcarryxU32(&x49, &x50, x48, x24, x21);
    var x51: u32 = undefined;
    var x52: u1 = undefined;
    addcarryxU32(&x51, &x52, x50, x22, x19);
    var x53: u32 = undefined;
    var x54: u1 = undefined;
    addcarryxU32(&x53, &x54, x52, x20, x17);
    var x55: u32 = undefined;
    var x56: u1 = undefined;
    addcarryxU32(&x55, &x56, x54, x18, x15);
    var x57: u32 = undefined;
    var x58: u1 = undefined;
    addcarryxU32(&x57, &x58, x56, x16, x13);
    const x59 = (cast(u32, x58) + x14);
    var x60: u32 = undefined;
    var x61: u32 = undefined;
    mulxU32(&x60, &x61, x35, 0xe88fdc45);
    var x62: u32 = undefined;
    var x63: u32 = undefined;
    mulxU32(&x62, &x63, x60, 0xffffffff);
    var x64: u32 = undefined;
    var x65: u32 = undefined;
    mulxU32(&x64, &x65, x60, 0xffffffff);
    var x66: u32 = undefined;
    var x67: u32 = undefined;
    mulxU32(&x66, &x67, x60, 0xffffffff);
    var x68: u32 = undefined;
    var x69: u32 = undefined;
    mulxU32(&x68, &x69, x60, 0xffffffff);
    var x70: u32 = undefined;
    var x71: u32 = undefined;
    mulxU32(&x70, &x71, x60, 0xffffffff);
    var x72: u32 = undefined;
    var x73: u32 = undefined;
    mulxU32(&x72, &x73, x60, 0xffffffff);
    var x74: u32 = undefined;
    var x75: u32 = undefined;
    mulxU32(&x74, &x75, x60, 0xc7634d81);
    var x76: u32 = undefined;
    var x77: u32 = undefined;
    mulxU32(&x76, &x77, x60, 0xf4372ddf);
    var x78: u32 = undefined;
    var x79: u32 = undefined;
    mulxU32(&x78, &x79, x60, 0x581a0db2);
    var x80: u32 = undefined;
    var x81: u32 = undefined;
    mulxU32(&x80, &x81, x60, 0x48b0a77a);
    var x82: u32 = undefined;
    var x83: u32 = undefined;
    mulxU32(&x82, &x83, x60, 0xecec196a);
    var x84: u32 = undefined;
    var x85: u32 = undefined;
    mulxU32(&x84, &x85, x60, 0xccc52973);
    var x86: u32 = undefined;
    var x87: u1 = undefined;
    addcarryxU32(&x86, &x87, 0x0, x85, x82);
    var x88: u32 = undefined;
    var x89: u1 = undefined;
    addcarryxU32(&x88, &x89, x87, x83, x80);
    var x90: u32 = undefined;
    var x91: u1 = undefined;
    addcarryxU32(&x90, &x91, x89, x81, x78);
    var x92: u32 = undefined;
    var x93: u1 = undefined;
    addcarryxU32(&x92, &x93, x91, x79, x76);
    var x94: u32 = undefined;
    var x95: u1 = undefined;
    addcarryxU32(&x94, &x95, x93, x77, x74);
    var x96: u32 = undefined;
    var x97: u1 = undefined;
    addcarryxU32(&x96, &x97, x95, x75, x72);
    var x98: u32 = undefined;
    var x99: u1 = undefined;
    addcarryxU32(&x98, &x99, x97, x73, x70);
    var x100: u32 = undefined;
    var x101: u1 = undefined;
    addcarryxU32(&x100, &x101, x99, x71, x68);
    var x102: u32 = undefined;
    var x103: u1 = undefined;
    addcarryxU32(&x102, &x103, x101, x69, x66);
    var x104: u32 = undefined;
    var x105: u1 = undefined;
    addcarryxU32(&x104, &x105, x103, x67, x64);
    var x106: u32 = undefined;
    var x107: u1 = undefined;
    addcarryxU32(&x106, &x107, x105, x65, x62);
    const x108 = (cast(u32, x107) + x63);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    addcarryxU32(&x109, &x110, 0x0, x35, x84);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    addcarryxU32(&x111, &x112, x110, x37, x86);
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    addcarryxU32(&x113, &x114, x112, x39, x88);
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    addcarryxU32(&x115, &x116, x114, x41, x90);
    var x117: u32 = undefined;
    var x118: u1 = undefined;
    addcarryxU32(&x117, &x118, x116, x43, x92);
    var x119: u32 = undefined;
    var x120: u1 = undefined;
    addcarryxU32(&x119, &x120, x118, x45, x94);
    var x121: u32 = undefined;
    var x122: u1 = undefined;
    addcarryxU32(&x121, &x122, x120, x47, x96);
    var x123: u32 = undefined;
    var x124: u1 = undefined;
    addcarryxU32(&x123, &x124, x122, x49, x98);
    var x125: u32 = undefined;
    var x126: u1 = undefined;
    addcarryxU32(&x125, &x126, x124, x51, x100);
    var x127: u32 = undefined;
    var x128: u1 = undefined;
    addcarryxU32(&x127, &x128, x126, x53, x102);
    var x129: u32 = undefined;
    var x130: u1 = undefined;
    addcarryxU32(&x129, &x130, x128, x55, x104);
    var x131: u32 = undefined;
    var x132: u1 = undefined;
    addcarryxU32(&x131, &x132, x130, x57, x106);
    var x133: u32 = undefined;
    var x134: u1 = undefined;
    addcarryxU32(&x133, &x134, x132, x59, x108);
    var x135: u32 = undefined;
    var x136: u32 = undefined;
    mulxU32(&x135, &x136, x1, (arg1[11]));
    var x137: u32 = undefined;
    var x138: u32 = undefined;
    mulxU32(&x137, &x138, x1, (arg1[10]));
    var x139: u32 = undefined;
    var x140: u32 = undefined;
    mulxU32(&x139, &x140, x1, (arg1[9]));
    var x141: u32 = undefined;
    var x142: u32 = undefined;
    mulxU32(&x141, &x142, x1, (arg1[8]));
    var x143: u32 = undefined;
    var x144: u32 = undefined;
    mulxU32(&x143, &x144, x1, (arg1[7]));
    var x145: u32 = undefined;
    var x146: u32 = undefined;
    mulxU32(&x145, &x146, x1, (arg1[6]));
    var x147: u32 = undefined;
    var x148: u32 = undefined;
    mulxU32(&x147, &x148, x1, (arg1[5]));
    var x149: u32 = undefined;
    var x150: u32 = undefined;
    mulxU32(&x149, &x150, x1, (arg1[4]));
    var x151: u32 = undefined;
    var x152: u32 = undefined;
    mulxU32(&x151, &x152, x1, (arg1[3]));
    var x153: u32 = undefined;
    var x154: u32 = undefined;
    mulxU32(&x153, &x154, x1, (arg1[2]));
    var x155: u32 = undefined;
    var x156: u32 = undefined;
    mulxU32(&x155, &x156, x1, (arg1[1]));
    var x157: u32 = undefined;
    var x158: u32 = undefined;
    mulxU32(&x157, &x158, x1, (arg1[0]));
    var x159: u32 = undefined;
    var x160: u1 = undefined;
    addcarryxU32(&x159, &x160, 0x0, x158, x155);
    var x161: u32 = undefined;
    var x162: u1 = undefined;
    addcarryxU32(&x161, &x162, x160, x156, x153);
    var x163: u32 = undefined;
    var x164: u1 = undefined;
    addcarryxU32(&x163, &x164, x162, x154, x151);
    var x165: u32 = undefined;
    var x166: u1 = undefined;
    addcarryxU32(&x165, &x166, x164, x152, x149);
    var x167: u32 = undefined;
    var x168: u1 = undefined;
    addcarryxU32(&x167, &x168, x166, x150, x147);
    var x169: u32 = undefined;
    var x170: u1 = undefined;
    addcarryxU32(&x169, &x170, x168, x148, x145);
    var x171: u32 = undefined;
    var x172: u1 = undefined;
    addcarryxU32(&x171, &x172, x170, x146, x143);
    var x173: u32 = undefined;
    var x174: u1 = undefined;
    addcarryxU32(&x173, &x174, x172, x144, x141);
    var x175: u32 = undefined;
    var x176: u1 = undefined;
    addcarryxU32(&x175, &x176, x174, x142, x139);
    var x177: u32 = undefined;
    var x178: u1 = undefined;
    addcarryxU32(&x177, &x178, x176, x140, x137);
    var x179: u32 = undefined;
    var x180: u1 = undefined;
    addcarryxU32(&x179, &x180, x178, x138, x135);
    const x181 = (cast(u32, x180) + x136);
    var x182: u32 = undefined;
    var x183: u1 = undefined;
    addcarryxU32(&x182, &x183, 0x0, x111, x157);
    var x184: u32 = undefined;
    var x185: u1 = undefined;
    addcarryxU32(&x184, &x185, x183, x113, x159);
    var x186: u32 = undefined;
    var x187: u1 = undefined;
    addcarryxU32(&x186, &x187, x185, x115, x161);
    var x188: u32 = undefined;
    var x189: u1 = undefined;
    addcarryxU32(&x188, &x189, x187, x117, x163);
    var x190: u32 = undefined;
    var x191: u1 = undefined;
    addcarryxU32(&x190, &x191, x189, x119, x165);
    var x192: u32 = undefined;
    var x193: u1 = undefined;
    addcarryxU32(&x192, &x193, x191, x121, x167);
    var x194: u32 = undefined;
    var x195: u1 = undefined;
    addcarryxU32(&x194, &x195, x193, x123, x169);
    var x196: u32 = undefined;
    var x197: u1 = undefined;
    addcarryxU32(&x196, &x197, x195, x125, x171);
    var x198: u32 = undefined;
    var x199: u1 = undefined;
    addcarryxU32(&x198, &x199, x197, x127, x173);
    var x200: u32 = undefined;
    var x201: u1 = undefined;
    addcarryxU32(&x200, &x201, x199, x129, x175);
    var x202: u32 = undefined;
    var x203: u1 = undefined;
    addcarryxU32(&x202, &x203, x201, x131, x177);
    var x204: u32 = undefined;
    var x205: u1 = undefined;
    addcarryxU32(&x204, &x205, x203, x133, x179);
    var x206: u32 = undefined;
    var x207: u1 = undefined;
    addcarryxU32(&x206, &x207, x205, cast(u32, x134), x181);
    var x208: u32 = undefined;
    var x209: u32 = undefined;
    mulxU32(&x208, &x209, x182, 0xe88fdc45);
    var x210: u32 = undefined;
    var x211: u32 = undefined;
    mulxU32(&x210, &x211, x208, 0xffffffff);
    var x212: u32 = undefined;
    var x213: u32 = undefined;
    mulxU32(&x212, &x213, x208, 0xffffffff);
    var x214: u32 = undefined;
    var x215: u32 = undefined;
    mulxU32(&x214, &x215, x208, 0xffffffff);
    var x216: u32 = undefined;
    var x217: u32 = undefined;
    mulxU32(&x216, &x217, x208, 0xffffffff);
    var x218: u32 = undefined;
    var x219: u32 = undefined;
    mulxU32(&x218, &x219, x208, 0xffffffff);
    var x220: u32 = undefined;
    var x221: u32 = undefined;
    mulxU32(&x220, &x221, x208, 0xffffffff);
    var x222: u32 = undefined;
    var x223: u32 = undefined;
    mulxU32(&x222, &x223, x208, 0xc7634d81);
    var x224: u32 = undefined;
    var x225: u32 = undefined;
    mulxU32(&x224, &x225, x208, 0xf4372ddf);
    var x226: u32 = undefined;
    var x227: u32 = undefined;
    mulxU32(&x226, &x227, x208, 0x581a0db2);
    var x228: u32 = undefined;
    var x229: u32 = undefined;
    mulxU32(&x228, &x229, x208, 0x48b0a77a);
    var x230: u32 = undefined;
    var x231: u32 = undefined;
    mulxU32(&x230, &x231, x208, 0xecec196a);
    var x232: u32 = undefined;
    var x233: u32 = undefined;
    mulxU32(&x232, &x233, x208, 0xccc52973);
    var x234: u32 = undefined;
    var x235: u1 = undefined;
    addcarryxU32(&x234, &x235, 0x0, x233, x230);
    var x236: u32 = undefined;
    var x237: u1 = undefined;
    addcarryxU32(&x236, &x237, x235, x231, x228);
    var x238: u32 = undefined;
    var x239: u1 = undefined;
    addcarryxU32(&x238, &x239, x237, x229, x226);
    var x240: u32 = undefined;
    var x241: u1 = undefined;
    addcarryxU32(&x240, &x241, x239, x227, x224);
    var x242: u32 = undefined;
    var x243: u1 = undefined;
    addcarryxU32(&x242, &x243, x241, x225, x222);
    var x244: u32 = undefined;
    var x245: u1 = undefined;
    addcarryxU32(&x244, &x245, x243, x223, x220);
    var x246: u32 = undefined;
    var x247: u1 = undefined;
    addcarryxU32(&x246, &x247, x245, x221, x218);
    var x248: u32 = undefined;
    var x249: u1 = undefined;
    addcarryxU32(&x248, &x249, x247, x219, x216);
    var x250: u32 = undefined;
    var x251: u1 = undefined;
    addcarryxU32(&x250, &x251, x249, x217, x214);
    var x252: u32 = undefined;
    var x253: u1 = undefined;
    addcarryxU32(&x252, &x253, x251, x215, x212);
    var x254: u32 = undefined;
    var x255: u1 = undefined;
    addcarryxU32(&x254, &x255, x253, x213, x210);
    const x256 = (cast(u32, x255) + x211);
    var x257: u32 = undefined;
    var x258: u1 = undefined;
    addcarryxU32(&x257, &x258, 0x0, x182, x232);
    var x259: u32 = undefined;
    var x260: u1 = undefined;
    addcarryxU32(&x259, &x260, x258, x184, x234);
    var x261: u32 = undefined;
    var x262: u1 = undefined;
    addcarryxU32(&x261, &x262, x260, x186, x236);
    var x263: u32 = undefined;
    var x264: u1 = undefined;
    addcarryxU32(&x263, &x264, x262, x188, x238);
    var x265: u32 = undefined;
    var x266: u1 = undefined;
    addcarryxU32(&x265, &x266, x264, x190, x240);
    var x267: u32 = undefined;
    var x268: u1 = undefined;
    addcarryxU32(&x267, &x268, x266, x192, x242);
    var x269: u32 = undefined;
    var x270: u1 = undefined;
    addcarryxU32(&x269, &x270, x268, x194, x244);
    var x271: u32 = undefined;
    var x272: u1 = undefined;
    addcarryxU32(&x271, &x272, x270, x196, x246);
    var x273: u32 = undefined;
    var x274: u1 = undefined;
    addcarryxU32(&x273, &x274, x272, x198, x248);
    var x275: u32 = undefined;
    var x276: u1 = undefined;
    addcarryxU32(&x275, &x276, x274, x200, x250);
    var x277: u32 = undefined;
    var x278: u1 = undefined;
    addcarryxU32(&x277, &x278, x276, x202, x252);
    var x279: u32 = undefined;
    var x280: u1 = undefined;
    addcarryxU32(&x279, &x280, x278, x204, x254);
    var x281: u32 = undefined;
    var x282: u1 = undefined;
    addcarryxU32(&x281, &x282, x280, x206, x256);
    const x283 = (cast(u32, x282) + cast(u32, x207));
    var x284: u32 = undefined;
    var x285: u32 = undefined;
    mulxU32(&x284, &x285, x2, (arg1[11]));
    var x286: u32 = undefined;
    var x287: u32 = undefined;
    mulxU32(&x286, &x287, x2, (arg1[10]));
    var x288: u32 = undefined;
    var x289: u32 = undefined;
    mulxU32(&x288, &x289, x2, (arg1[9]));
    var x290: u32 = undefined;
    var x291: u32 = undefined;
    mulxU32(&x290, &x291, x2, (arg1[8]));
    var x292: u32 = undefined;
    var x293: u32 = undefined;
    mulxU32(&x292, &x293, x2, (arg1[7]));
    var x294: u32 = undefined;
    var x295: u32 = undefined;
    mulxU32(&x294, &x295, x2, (arg1[6]));
    var x296: u32 = undefined;
    var x297: u32 = undefined;
    mulxU32(&x296, &x297, x2, (arg1[5]));
    var x298: u32 = undefined;
    var x299: u32 = undefined;
    mulxU32(&x298, &x299, x2, (arg1[4]));
    var x300: u32 = undefined;
    var x301: u32 = undefined;
    mulxU32(&x300, &x301, x2, (arg1[3]));
    var x302: u32 = undefined;
    var x303: u32 = undefined;
    mulxU32(&x302, &x303, x2, (arg1[2]));
    var x304: u32 = undefined;
    var x305: u32 = undefined;
    mulxU32(&x304, &x305, x2, (arg1[1]));
    var x306: u32 = undefined;
    var x307: u32 = undefined;
    mulxU32(&x306, &x307, x2, (arg1[0]));
    var x308: u32 = undefined;
    var x309: u1 = undefined;
    addcarryxU32(&x308, &x309, 0x0, x307, x304);
    var x310: u32 = undefined;
    var x311: u1 = undefined;
    addcarryxU32(&x310, &x311, x309, x305, x302);
    var x312: u32 = undefined;
    var x313: u1 = undefined;
    addcarryxU32(&x312, &x313, x311, x303, x300);
    var x314: u32 = undefined;
    var x315: u1 = undefined;
    addcarryxU32(&x314, &x315, x313, x301, x298);
    var x316: u32 = undefined;
    var x317: u1 = undefined;
    addcarryxU32(&x316, &x317, x315, x299, x296);
    var x318: u32 = undefined;
    var x319: u1 = undefined;
    addcarryxU32(&x318, &x319, x317, x297, x294);
    var x320: u32 = undefined;
    var x321: u1 = undefined;
    addcarryxU32(&x320, &x321, x319, x295, x292);
    var x322: u32 = undefined;
    var x323: u1 = undefined;
    addcarryxU32(&x322, &x323, x321, x293, x290);
    var x324: u32 = undefined;
    var x325: u1 = undefined;
    addcarryxU32(&x324, &x325, x323, x291, x288);
    var x326: u32 = undefined;
    var x327: u1 = undefined;
    addcarryxU32(&x326, &x327, x325, x289, x286);
    var x328: u32 = undefined;
    var x329: u1 = undefined;
    addcarryxU32(&x328, &x329, x327, x287, x284);
    const x330 = (cast(u32, x329) + x285);
    var x331: u32 = undefined;
    var x332: u1 = undefined;
    addcarryxU32(&x331, &x332, 0x0, x259, x306);
    var x333: u32 = undefined;
    var x334: u1 = undefined;
    addcarryxU32(&x333, &x334, x332, x261, x308);
    var x335: u32 = undefined;
    var x336: u1 = undefined;
    addcarryxU32(&x335, &x336, x334, x263, x310);
    var x337: u32 = undefined;
    var x338: u1 = undefined;
    addcarryxU32(&x337, &x338, x336, x265, x312);
    var x339: u32 = undefined;
    var x340: u1 = undefined;
    addcarryxU32(&x339, &x340, x338, x267, x314);
    var x341: u32 = undefined;
    var x342: u1 = undefined;
    addcarryxU32(&x341, &x342, x340, x269, x316);
    var x343: u32 = undefined;
    var x344: u1 = undefined;
    addcarryxU32(&x343, &x344, x342, x271, x318);
    var x345: u32 = undefined;
    var x346: u1 = undefined;
    addcarryxU32(&x345, &x346, x344, x273, x320);
    var x347: u32 = undefined;
    var x348: u1 = undefined;
    addcarryxU32(&x347, &x348, x346, x275, x322);
    var x349: u32 = undefined;
    var x350: u1 = undefined;
    addcarryxU32(&x349, &x350, x348, x277, x324);
    var x351: u32 = undefined;
    var x352: u1 = undefined;
    addcarryxU32(&x351, &x352, x350, x279, x326);
    var x353: u32 = undefined;
    var x354: u1 = undefined;
    addcarryxU32(&x353, &x354, x352, x281, x328);
    var x355: u32 = undefined;
    var x356: u1 = undefined;
    addcarryxU32(&x355, &x356, x354, x283, x330);
    var x357: u32 = undefined;
    var x358: u32 = undefined;
    mulxU32(&x357, &x358, x331, 0xe88fdc45);
    var x359: u32 = undefined;
    var x360: u32 = undefined;
    mulxU32(&x359, &x360, x357, 0xffffffff);
    var x361: u32 = undefined;
    var x362: u32 = undefined;
    mulxU32(&x361, &x362, x357, 0xffffffff);
    var x363: u32 = undefined;
    var x364: u32 = undefined;
    mulxU32(&x363, &x364, x357, 0xffffffff);
    var x365: u32 = undefined;
    var x366: u32 = undefined;
    mulxU32(&x365, &x366, x357, 0xffffffff);
    var x367: u32 = undefined;
    var x368: u32 = undefined;
    mulxU32(&x367, &x368, x357, 0xffffffff);
    var x369: u32 = undefined;
    var x370: u32 = undefined;
    mulxU32(&x369, &x370, x357, 0xffffffff);
    var x371: u32 = undefined;
    var x372: u32 = undefined;
    mulxU32(&x371, &x372, x357, 0xc7634d81);
    var x373: u32 = undefined;
    var x374: u32 = undefined;
    mulxU32(&x373, &x374, x357, 0xf4372ddf);
    var x375: u32 = undefined;
    var x376: u32 = undefined;
    mulxU32(&x375, &x376, x357, 0x581a0db2);
    var x377: u32 = undefined;
    var x378: u32 = undefined;
    mulxU32(&x377, &x378, x357, 0x48b0a77a);
    var x379: u32 = undefined;
    var x380: u32 = undefined;
    mulxU32(&x379, &x380, x357, 0xecec196a);
    var x381: u32 = undefined;
    var x382: u32 = undefined;
    mulxU32(&x381, &x382, x357, 0xccc52973);
    var x383: u32 = undefined;
    var x384: u1 = undefined;
    addcarryxU32(&x383, &x384, 0x0, x382, x379);
    var x385: u32 = undefined;
    var x386: u1 = undefined;
    addcarryxU32(&x385, &x386, x384, x380, x377);
    var x387: u32 = undefined;
    var x388: u1 = undefined;
    addcarryxU32(&x387, &x388, x386, x378, x375);
    var x389: u32 = undefined;
    var x390: u1 = undefined;
    addcarryxU32(&x389, &x390, x388, x376, x373);
    var x391: u32 = undefined;
    var x392: u1 = undefined;
    addcarryxU32(&x391, &x392, x390, x374, x371);
    var x393: u32 = undefined;
    var x394: u1 = undefined;
    addcarryxU32(&x393, &x394, x392, x372, x369);
    var x395: u32 = undefined;
    var x396: u1 = undefined;
    addcarryxU32(&x395, &x396, x394, x370, x367);
    var x397: u32 = undefined;
    var x398: u1 = undefined;
    addcarryxU32(&x397, &x398, x396, x368, x365);
    var x399: u32 = undefined;
    var x400: u1 = undefined;
    addcarryxU32(&x399, &x400, x398, x366, x363);
    var x401: u32 = undefined;
    var x402: u1 = undefined;
    addcarryxU32(&x401, &x402, x400, x364, x361);
    var x403: u32 = undefined;
    var x404: u1 = undefined;
    addcarryxU32(&x403, &x404, x402, x362, x359);
    const x405 = (cast(u32, x404) + x360);
    var x406: u32 = undefined;
    var x407: u1 = undefined;
    addcarryxU32(&x406, &x407, 0x0, x331, x381);
    var x408: u32 = undefined;
    var x409: u1 = undefined;
    addcarryxU32(&x408, &x409, x407, x333, x383);
    var x410: u32 = undefined;
    var x411: u1 = undefined;
    addcarryxU32(&x410, &x411, x409, x335, x385);
    var x412: u32 = undefined;
    var x413: u1 = undefined;
    addcarryxU32(&x412, &x413, x411, x337, x387);
    var x414: u32 = undefined;
    var x415: u1 = undefined;
    addcarryxU32(&x414, &x415, x413, x339, x389);
    var x416: u32 = undefined;
    var x417: u1 = undefined;
    addcarryxU32(&x416, &x417, x415, x341, x391);
    var x418: u32 = undefined;
    var x419: u1 = undefined;
    addcarryxU32(&x418, &x419, x417, x343, x393);
    var x420: u32 = undefined;
    var x421: u1 = undefined;
    addcarryxU32(&x420, &x421, x419, x345, x395);
    var x422: u32 = undefined;
    var x423: u1 = undefined;
    addcarryxU32(&x422, &x423, x421, x347, x397);
    var x424: u32 = undefined;
    var x425: u1 = undefined;
    addcarryxU32(&x424, &x425, x423, x349, x399);
    var x426: u32 = undefined;
    var x427: u1 = undefined;
    addcarryxU32(&x426, &x427, x425, x351, x401);
    var x428: u32 = undefined;
    var x429: u1 = undefined;
    addcarryxU32(&x428, &x429, x427, x353, x403);
    var x430: u32 = undefined;
    var x431: u1 = undefined;
    addcarryxU32(&x430, &x431, x429, x355, x405);
    const x432 = (cast(u32, x431) + cast(u32, x356));
    var x433: u32 = undefined;
    var x434: u32 = undefined;
    mulxU32(&x433, &x434, x3, (arg1[11]));
    var x435: u32 = undefined;
    var x436: u32 = undefined;
    mulxU32(&x435, &x436, x3, (arg1[10]));
    var x437: u32 = undefined;
    var x438: u32 = undefined;
    mulxU32(&x437, &x438, x3, (arg1[9]));
    var x439: u32 = undefined;
    var x440: u32 = undefined;
    mulxU32(&x439, &x440, x3, (arg1[8]));
    var x441: u32 = undefined;
    var x442: u32 = undefined;
    mulxU32(&x441, &x442, x3, (arg1[7]));
    var x443: u32 = undefined;
    var x444: u32 = undefined;
    mulxU32(&x443, &x444, x3, (arg1[6]));
    var x445: u32 = undefined;
    var x446: u32 = undefined;
    mulxU32(&x445, &x446, x3, (arg1[5]));
    var x447: u32 = undefined;
    var x448: u32 = undefined;
    mulxU32(&x447, &x448, x3, (arg1[4]));
    var x449: u32 = undefined;
    var x450: u32 = undefined;
    mulxU32(&x449, &x450, x3, (arg1[3]));
    var x451: u32 = undefined;
    var x452: u32 = undefined;
    mulxU32(&x451, &x452, x3, (arg1[2]));
    var x453: u32 = undefined;
    var x454: u32 = undefined;
    mulxU32(&x453, &x454, x3, (arg1[1]));
    var x455: u32 = undefined;
    var x456: u32 = undefined;
    mulxU32(&x455, &x456, x3, (arg1[0]));
    var x457: u32 = undefined;
    var x458: u1 = undefined;
    addcarryxU32(&x457, &x458, 0x0, x456, x453);
    var x459: u32 = undefined;
    var x460: u1 = undefined;
    addcarryxU32(&x459, &x460, x458, x454, x451);
    var x461: u32 = undefined;
    var x462: u1 = undefined;
    addcarryxU32(&x461, &x462, x460, x452, x449);
    var x463: u32 = undefined;
    var x464: u1 = undefined;
    addcarryxU32(&x463, &x464, x462, x450, x447);
    var x465: u32 = undefined;
    var x466: u1 = undefined;
    addcarryxU32(&x465, &x466, x464, x448, x445);
    var x467: u32 = undefined;
    var x468: u1 = undefined;
    addcarryxU32(&x467, &x468, x466, x446, x443);
    var x469: u32 = undefined;
    var x470: u1 = undefined;
    addcarryxU32(&x469, &x470, x468, x444, x441);
    var x471: u32 = undefined;
    var x472: u1 = undefined;
    addcarryxU32(&x471, &x472, x470, x442, x439);
    var x473: u32 = undefined;
    var x474: u1 = undefined;
    addcarryxU32(&x473, &x474, x472, x440, x437);
    var x475: u32 = undefined;
    var x476: u1 = undefined;
    addcarryxU32(&x475, &x476, x474, x438, x435);
    var x477: u32 = undefined;
    var x478: u1 = undefined;
    addcarryxU32(&x477, &x478, x476, x436, x433);
    const x479 = (cast(u32, x478) + x434);
    var x480: u32 = undefined;
    var x481: u1 = undefined;
    addcarryxU32(&x480, &x481, 0x0, x408, x455);
    var x482: u32 = undefined;
    var x483: u1 = undefined;
    addcarryxU32(&x482, &x483, x481, x410, x457);
    var x484: u32 = undefined;
    var x485: u1 = undefined;
    addcarryxU32(&x484, &x485, x483, x412, x459);
    var x486: u32 = undefined;
    var x487: u1 = undefined;
    addcarryxU32(&x486, &x487, x485, x414, x461);
    var x488: u32 = undefined;
    var x489: u1 = undefined;
    addcarryxU32(&x488, &x489, x487, x416, x463);
    var x490: u32 = undefined;
    var x491: u1 = undefined;
    addcarryxU32(&x490, &x491, x489, x418, x465);
    var x492: u32 = undefined;
    var x493: u1 = undefined;
    addcarryxU32(&x492, &x493, x491, x420, x467);
    var x494: u32 = undefined;
    var x495: u1 = undefined;
    addcarryxU32(&x494, &x495, x493, x422, x469);
    var x496: u32 = undefined;
    var x497: u1 = undefined;
    addcarryxU32(&x496, &x497, x495, x424, x471);
    var x498: u32 = undefined;
    var x499: u1 = undefined;
    addcarryxU32(&x498, &x499, x497, x426, x473);
    var x500: u32 = undefined;
    var x501: u1 = undefined;
    addcarryxU32(&x500, &x501, x499, x428, x475);
    var x502: u32 = undefined;
    var x503: u1 = undefined;
    addcarryxU32(&x502, &x503, x501, x430, x477);
    var x504: u32 = undefined;
    var x505: u1 = undefined;
    addcarryxU32(&x504, &x505, x503, x432, x479);
    var x506: u32 = undefined;
    var x507: u32 = undefined;
    mulxU32(&x506, &x507, x480, 0xe88fdc45);
    var x508: u32 = undefined;
    var x509: u32 = undefined;
    mulxU32(&x508, &x509, x506, 0xffffffff);
    var x510: u32 = undefined;
    var x511: u32 = undefined;
    mulxU32(&x510, &x511, x506, 0xffffffff);
    var x512: u32 = undefined;
    var x513: u32 = undefined;
    mulxU32(&x512, &x513, x506, 0xffffffff);
    var x514: u32 = undefined;
    var x515: u32 = undefined;
    mulxU32(&x514, &x515, x506, 0xffffffff);
    var x516: u32 = undefined;
    var x517: u32 = undefined;
    mulxU32(&x516, &x517, x506, 0xffffffff);
    var x518: u32 = undefined;
    var x519: u32 = undefined;
    mulxU32(&x518, &x519, x506, 0xffffffff);
    var x520: u32 = undefined;
    var x521: u32 = undefined;
    mulxU32(&x520, &x521, x506, 0xc7634d81);
    var x522: u32 = undefined;
    var x523: u32 = undefined;
    mulxU32(&x522, &x523, x506, 0xf4372ddf);
    var x524: u32 = undefined;
    var x525: u32 = undefined;
    mulxU32(&x524, &x525, x506, 0x581a0db2);
    var x526: u32 = undefined;
    var x527: u32 = undefined;
    mulxU32(&x526, &x527, x506, 0x48b0a77a);
    var x528: u32 = undefined;
    var x529: u32 = undefined;
    mulxU32(&x528, &x529, x506, 0xecec196a);
    var x530: u32 = undefined;
    var x531: u32 = undefined;
    mulxU32(&x530, &x531, x506, 0xccc52973);
    var x532: u32 = undefined;
    var x533: u1 = undefined;
    addcarryxU32(&x532, &x533, 0x0, x531, x528);
    var x534: u32 = undefined;
    var x535: u1 = undefined;
    addcarryxU32(&x534, &x535, x533, x529, x526);
    var x536: u32 = undefined;
    var x537: u1 = undefined;
    addcarryxU32(&x536, &x537, x535, x527, x524);
    var x538: u32 = undefined;
    var x539: u1 = undefined;
    addcarryxU32(&x538, &x539, x537, x525, x522);
    var x540: u32 = undefined;
    var x541: u1 = undefined;
    addcarryxU32(&x540, &x541, x539, x523, x520);
    var x542: u32 = undefined;
    var x543: u1 = undefined;
    addcarryxU32(&x542, &x543, x541, x521, x518);
    var x544: u32 = undefined;
    var x545: u1 = undefined;
    addcarryxU32(&x544, &x545, x543, x519, x516);
    var x546: u32 = undefined;
    var x547: u1 = undefined;
    addcarryxU32(&x546, &x547, x545, x517, x514);
    var x548: u32 = undefined;
    var x549: u1 = undefined;
    addcarryxU32(&x548, &x549, x547, x515, x512);
    var x550: u32 = undefined;
    var x551: u1 = undefined;
    addcarryxU32(&x550, &x551, x549, x513, x510);
    var x552: u32 = undefined;
    var x553: u1 = undefined;
    addcarryxU32(&x552, &x553, x551, x511, x508);
    const x554 = (cast(u32, x553) + x509);
    var x555: u32 = undefined;
    var x556: u1 = undefined;
    addcarryxU32(&x555, &x556, 0x0, x480, x530);
    var x557: u32 = undefined;
    var x558: u1 = undefined;
    addcarryxU32(&x557, &x558, x556, x482, x532);
    var x559: u32 = undefined;
    var x560: u1 = undefined;
    addcarryxU32(&x559, &x560, x558, x484, x534);
    var x561: u32 = undefined;
    var x562: u1 = undefined;
    addcarryxU32(&x561, &x562, x560, x486, x536);
    var x563: u32 = undefined;
    var x564: u1 = undefined;
    addcarryxU32(&x563, &x564, x562, x488, x538);
    var x565: u32 = undefined;
    var x566: u1 = undefined;
    addcarryxU32(&x565, &x566, x564, x490, x540);
    var x567: u32 = undefined;
    var x568: u1 = undefined;
    addcarryxU32(&x567, &x568, x566, x492, x542);
    var x569: u32 = undefined;
    var x570: u1 = undefined;
    addcarryxU32(&x569, &x570, x568, x494, x544);
    var x571: u32 = undefined;
    var x572: u1 = undefined;
    addcarryxU32(&x571, &x572, x570, x496, x546);
    var x573: u32 = undefined;
    var x574: u1 = undefined;
    addcarryxU32(&x573, &x574, x572, x498, x548);
    var x575: u32 = undefined;
    var x576: u1 = undefined;
    addcarryxU32(&x575, &x576, x574, x500, x550);
    var x577: u32 = undefined;
    var x578: u1 = undefined;
    addcarryxU32(&x577, &x578, x576, x502, x552);
    var x579: u32 = undefined;
    var x580: u1 = undefined;
    addcarryxU32(&x579, &x580, x578, x504, x554);
    const x581 = (cast(u32, x580) + cast(u32, x505));
    var x582: u32 = undefined;
    var x583: u32 = undefined;
    mulxU32(&x582, &x583, x4, (arg1[11]));
    var x584: u32 = undefined;
    var x585: u32 = undefined;
    mulxU32(&x584, &x585, x4, (arg1[10]));
    var x586: u32 = undefined;
    var x587: u32 = undefined;
    mulxU32(&x586, &x587, x4, (arg1[9]));
    var x588: u32 = undefined;
    var x589: u32 = undefined;
    mulxU32(&x588, &x589, x4, (arg1[8]));
    var x590: u32 = undefined;
    var x591: u32 = undefined;
    mulxU32(&x590, &x591, x4, (arg1[7]));
    var x592: u32 = undefined;
    var x593: u32 = undefined;
    mulxU32(&x592, &x593, x4, (arg1[6]));
    var x594: u32 = undefined;
    var x595: u32 = undefined;
    mulxU32(&x594, &x595, x4, (arg1[5]));
    var x596: u32 = undefined;
    var x597: u32 = undefined;
    mulxU32(&x596, &x597, x4, (arg1[4]));
    var x598: u32 = undefined;
    var x599: u32 = undefined;
    mulxU32(&x598, &x599, x4, (arg1[3]));
    var x600: u32 = undefined;
    var x601: u32 = undefined;
    mulxU32(&x600, &x601, x4, (arg1[2]));
    var x602: u32 = undefined;
    var x603: u32 = undefined;
    mulxU32(&x602, &x603, x4, (arg1[1]));
    var x604: u32 = undefined;
    var x605: u32 = undefined;
    mulxU32(&x604, &x605, x4, (arg1[0]));
    var x606: u32 = undefined;
    var x607: u1 = undefined;
    addcarryxU32(&x606, &x607, 0x0, x605, x602);
    var x608: u32 = undefined;
    var x609: u1 = undefined;
    addcarryxU32(&x608, &x609, x607, x603, x600);
    var x610: u32 = undefined;
    var x611: u1 = undefined;
    addcarryxU32(&x610, &x611, x609, x601, x598);
    var x612: u32 = undefined;
    var x613: u1 = undefined;
    addcarryxU32(&x612, &x613, x611, x599, x596);
    var x614: u32 = undefined;
    var x615: u1 = undefined;
    addcarryxU32(&x614, &x615, x613, x597, x594);
    var x616: u32 = undefined;
    var x617: u1 = undefined;
    addcarryxU32(&x616, &x617, x615, x595, x592);
    var x618: u32 = undefined;
    var x619: u1 = undefined;
    addcarryxU32(&x618, &x619, x617, x593, x590);
    var x620: u32 = undefined;
    var x621: u1 = undefined;
    addcarryxU32(&x620, &x621, x619, x591, x588);
    var x622: u32 = undefined;
    var x623: u1 = undefined;
    addcarryxU32(&x622, &x623, x621, x589, x586);
    var x624: u32 = undefined;
    var x625: u1 = undefined;
    addcarryxU32(&x624, &x625, x623, x587, x584);
    var x626: u32 = undefined;
    var x627: u1 = undefined;
    addcarryxU32(&x626, &x627, x625, x585, x582);
    const x628 = (cast(u32, x627) + x583);
    var x629: u32 = undefined;
    var x630: u1 = undefined;
    addcarryxU32(&x629, &x630, 0x0, x557, x604);
    var x631: u32 = undefined;
    var x632: u1 = undefined;
    addcarryxU32(&x631, &x632, x630, x559, x606);
    var x633: u32 = undefined;
    var x634: u1 = undefined;
    addcarryxU32(&x633, &x634, x632, x561, x608);
    var x635: u32 = undefined;
    var x636: u1 = undefined;
    addcarryxU32(&x635, &x636, x634, x563, x610);
    var x637: u32 = undefined;
    var x638: u1 = undefined;
    addcarryxU32(&x637, &x638, x636, x565, x612);
    var x639: u32 = undefined;
    var x640: u1 = undefined;
    addcarryxU32(&x639, &x640, x638, x567, x614);
    var x641: u32 = undefined;
    var x642: u1 = undefined;
    addcarryxU32(&x641, &x642, x640, x569, x616);
    var x643: u32 = undefined;
    var x644: u1 = undefined;
    addcarryxU32(&x643, &x644, x642, x571, x618);
    var x645: u32 = undefined;
    var x646: u1 = undefined;
    addcarryxU32(&x645, &x646, x644, x573, x620);
    var x647: u32 = undefined;
    var x648: u1 = undefined;
    addcarryxU32(&x647, &x648, x646, x575, x622);
    var x649: u32 = undefined;
    var x650: u1 = undefined;
    addcarryxU32(&x649, &x650, x648, x577, x624);
    var x651: u32 = undefined;
    var x652: u1 = undefined;
    addcarryxU32(&x651, &x652, x650, x579, x626);
    var x653: u32 = undefined;
    var x654: u1 = undefined;
    addcarryxU32(&x653, &x654, x652, x581, x628);
    var x655: u32 = undefined;
    var x656: u32 = undefined;
    mulxU32(&x655, &x656, x629, 0xe88fdc45);
    var x657: u32 = undefined;
    var x658: u32 = undefined;
    mulxU32(&x657, &x658, x655, 0xffffffff);
    var x659: u32 = undefined;
    var x660: u32 = undefined;
    mulxU32(&x659, &x660, x655, 0xffffffff);
    var x661: u32 = undefined;
    var x662: u32 = undefined;
    mulxU32(&x661, &x662, x655, 0xffffffff);
    var x663: u32 = undefined;
    var x664: u32 = undefined;
    mulxU32(&x663, &x664, x655, 0xffffffff);
    var x665: u32 = undefined;
    var x666: u32 = undefined;
    mulxU32(&x665, &x666, x655, 0xffffffff);
    var x667: u32 = undefined;
    var x668: u32 = undefined;
    mulxU32(&x667, &x668, x655, 0xffffffff);
    var x669: u32 = undefined;
    var x670: u32 = undefined;
    mulxU32(&x669, &x670, x655, 0xc7634d81);
    var x671: u32 = undefined;
    var x672: u32 = undefined;
    mulxU32(&x671, &x672, x655, 0xf4372ddf);
    var x673: u32 = undefined;
    var x674: u32 = undefined;
    mulxU32(&x673, &x674, x655, 0x581a0db2);
    var x675: u32 = undefined;
    var x676: u32 = undefined;
    mulxU32(&x675, &x676, x655, 0x48b0a77a);
    var x677: u32 = undefined;
    var x678: u32 = undefined;
    mulxU32(&x677, &x678, x655, 0xecec196a);
    var x679: u32 = undefined;
    var x680: u32 = undefined;
    mulxU32(&x679, &x680, x655, 0xccc52973);
    var x681: u32 = undefined;
    var x682: u1 = undefined;
    addcarryxU32(&x681, &x682, 0x0, x680, x677);
    var x683: u32 = undefined;
    var x684: u1 = undefined;
    addcarryxU32(&x683, &x684, x682, x678, x675);
    var x685: u32 = undefined;
    var x686: u1 = undefined;
    addcarryxU32(&x685, &x686, x684, x676, x673);
    var x687: u32 = undefined;
    var x688: u1 = undefined;
    addcarryxU32(&x687, &x688, x686, x674, x671);
    var x689: u32 = undefined;
    var x690: u1 = undefined;
    addcarryxU32(&x689, &x690, x688, x672, x669);
    var x691: u32 = undefined;
    var x692: u1 = undefined;
    addcarryxU32(&x691, &x692, x690, x670, x667);
    var x693: u32 = undefined;
    var x694: u1 = undefined;
    addcarryxU32(&x693, &x694, x692, x668, x665);
    var x695: u32 = undefined;
    var x696: u1 = undefined;
    addcarryxU32(&x695, &x696, x694, x666, x663);
    var x697: u32 = undefined;
    var x698: u1 = undefined;
    addcarryxU32(&x697, &x698, x696, x664, x661);
    var x699: u32 = undefined;
    var x700: u1 = undefined;
    addcarryxU32(&x699, &x700, x698, x662, x659);
    var x701: u32 = undefined;
    var x702: u1 = undefined;
    addcarryxU32(&x701, &x702, x700, x660, x657);
    const x703 = (cast(u32, x702) + x658);
    var x704: u32 = undefined;
    var x705: u1 = undefined;
    addcarryxU32(&x704, &x705, 0x0, x629, x679);
    var x706: u32 = undefined;
    var x707: u1 = undefined;
    addcarryxU32(&x706, &x707, x705, x631, x681);
    var x708: u32 = undefined;
    var x709: u1 = undefined;
    addcarryxU32(&x708, &x709, x707, x633, x683);
    var x710: u32 = undefined;
    var x711: u1 = undefined;
    addcarryxU32(&x710, &x711, x709, x635, x685);
    var x712: u32 = undefined;
    var x713: u1 = undefined;
    addcarryxU32(&x712, &x713, x711, x637, x687);
    var x714: u32 = undefined;
    var x715: u1 = undefined;
    addcarryxU32(&x714, &x715, x713, x639, x689);
    var x716: u32 = undefined;
    var x717: u1 = undefined;
    addcarryxU32(&x716, &x717, x715, x641, x691);
    var x718: u32 = undefined;
    var x719: u1 = undefined;
    addcarryxU32(&x718, &x719, x717, x643, x693);
    var x720: u32 = undefined;
    var x721: u1 = undefined;
    addcarryxU32(&x720, &x721, x719, x645, x695);
    var x722: u32 = undefined;
    var x723: u1 = undefined;
    addcarryxU32(&x722, &x723, x721, x647, x697);
    var x724: u32 = undefined;
    var x725: u1 = undefined;
    addcarryxU32(&x724, &x725, x723, x649, x699);
    var x726: u32 = undefined;
    var x727: u1 = undefined;
    addcarryxU32(&x726, &x727, x725, x651, x701);
    var x728: u32 = undefined;
    var x729: u1 = undefined;
    addcarryxU32(&x728, &x729, x727, x653, x703);
    const x730 = (cast(u32, x729) + cast(u32, x654));
    var x731: u32 = undefined;
    var x732: u32 = undefined;
    mulxU32(&x731, &x732, x5, (arg1[11]));
    var x733: u32 = undefined;
    var x734: u32 = undefined;
    mulxU32(&x733, &x734, x5, (arg1[10]));
    var x735: u32 = undefined;
    var x736: u32 = undefined;
    mulxU32(&x735, &x736, x5, (arg1[9]));
    var x737: u32 = undefined;
    var x738: u32 = undefined;
    mulxU32(&x737, &x738, x5, (arg1[8]));
    var x739: u32 = undefined;
    var x740: u32 = undefined;
    mulxU32(&x739, &x740, x5, (arg1[7]));
    var x741: u32 = undefined;
    var x742: u32 = undefined;
    mulxU32(&x741, &x742, x5, (arg1[6]));
    var x743: u32 = undefined;
    var x744: u32 = undefined;
    mulxU32(&x743, &x744, x5, (arg1[5]));
    var x745: u32 = undefined;
    var x746: u32 = undefined;
    mulxU32(&x745, &x746, x5, (arg1[4]));
    var x747: u32 = undefined;
    var x748: u32 = undefined;
    mulxU32(&x747, &x748, x5, (arg1[3]));
    var x749: u32 = undefined;
    var x750: u32 = undefined;
    mulxU32(&x749, &x750, x5, (arg1[2]));
    var x751: u32 = undefined;
    var x752: u32 = undefined;
    mulxU32(&x751, &x752, x5, (arg1[1]));
    var x753: u32 = undefined;
    var x754: u32 = undefined;
    mulxU32(&x753, &x754, x5, (arg1[0]));
    var x755: u32 = undefined;
    var x756: u1 = undefined;
    addcarryxU32(&x755, &x756, 0x0, x754, x751);
    var x757: u32 = undefined;
    var x758: u1 = undefined;
    addcarryxU32(&x757, &x758, x756, x752, x749);
    var x759: u32 = undefined;
    var x760: u1 = undefined;
    addcarryxU32(&x759, &x760, x758, x750, x747);
    var x761: u32 = undefined;
    var x762: u1 = undefined;
    addcarryxU32(&x761, &x762, x760, x748, x745);
    var x763: u32 = undefined;
    var x764: u1 = undefined;
    addcarryxU32(&x763, &x764, x762, x746, x743);
    var x765: u32 = undefined;
    var x766: u1 = undefined;
    addcarryxU32(&x765, &x766, x764, x744, x741);
    var x767: u32 = undefined;
    var x768: u1 = undefined;
    addcarryxU32(&x767, &x768, x766, x742, x739);
    var x769: u32 = undefined;
    var x770: u1 = undefined;
    addcarryxU32(&x769, &x770, x768, x740, x737);
    var x771: u32 = undefined;
    var x772: u1 = undefined;
    addcarryxU32(&x771, &x772, x770, x738, x735);
    var x773: u32 = undefined;
    var x774: u1 = undefined;
    addcarryxU32(&x773, &x774, x772, x736, x733);
    var x775: u32 = undefined;
    var x776: u1 = undefined;
    addcarryxU32(&x775, &x776, x774, x734, x731);
    const x777 = (cast(u32, x776) + x732);
    var x778: u32 = undefined;
    var x779: u1 = undefined;
    addcarryxU32(&x778, &x779, 0x0, x706, x753);
    var x780: u32 = undefined;
    var x781: u1 = undefined;
    addcarryxU32(&x780, &x781, x779, x708, x755);
    var x782: u32 = undefined;
    var x783: u1 = undefined;
    addcarryxU32(&x782, &x783, x781, x710, x757);
    var x784: u32 = undefined;
    var x785: u1 = undefined;
    addcarryxU32(&x784, &x785, x783, x712, x759);
    var x786: u32 = undefined;
    var x787: u1 = undefined;
    addcarryxU32(&x786, &x787, x785, x714, x761);
    var x788: u32 = undefined;
    var x789: u1 = undefined;
    addcarryxU32(&x788, &x789, x787, x716, x763);
    var x790: u32 = undefined;
    var x791: u1 = undefined;
    addcarryxU32(&x790, &x791, x789, x718, x765);
    var x792: u32 = undefined;
    var x793: u1 = undefined;
    addcarryxU32(&x792, &x793, x791, x720, x767);
    var x794: u32 = undefined;
    var x795: u1 = undefined;
    addcarryxU32(&x794, &x795, x793, x722, x769);
    var x796: u32 = undefined;
    var x797: u1 = undefined;
    addcarryxU32(&x796, &x797, x795, x724, x771);
    var x798: u32 = undefined;
    var x799: u1 = undefined;
    addcarryxU32(&x798, &x799, x797, x726, x773);
    var x800: u32 = undefined;
    var x801: u1 = undefined;
    addcarryxU32(&x800, &x801, x799, x728, x775);
    var x802: u32 = undefined;
    var x803: u1 = undefined;
    addcarryxU32(&x802, &x803, x801, x730, x777);
    var x804: u32 = undefined;
    var x805: u32 = undefined;
    mulxU32(&x804, &x805, x778, 0xe88fdc45);
    var x806: u32 = undefined;
    var x807: u32 = undefined;
    mulxU32(&x806, &x807, x804, 0xffffffff);
    var x808: u32 = undefined;
    var x809: u32 = undefined;
    mulxU32(&x808, &x809, x804, 0xffffffff);
    var x810: u32 = undefined;
    var x811: u32 = undefined;
    mulxU32(&x810, &x811, x804, 0xffffffff);
    var x812: u32 = undefined;
    var x813: u32 = undefined;
    mulxU32(&x812, &x813, x804, 0xffffffff);
    var x814: u32 = undefined;
    var x815: u32 = undefined;
    mulxU32(&x814, &x815, x804, 0xffffffff);
    var x816: u32 = undefined;
    var x817: u32 = undefined;
    mulxU32(&x816, &x817, x804, 0xffffffff);
    var x818: u32 = undefined;
    var x819: u32 = undefined;
    mulxU32(&x818, &x819, x804, 0xc7634d81);
    var x820: u32 = undefined;
    var x821: u32 = undefined;
    mulxU32(&x820, &x821, x804, 0xf4372ddf);
    var x822: u32 = undefined;
    var x823: u32 = undefined;
    mulxU32(&x822, &x823, x804, 0x581a0db2);
    var x824: u32 = undefined;
    var x825: u32 = undefined;
    mulxU32(&x824, &x825, x804, 0x48b0a77a);
    var x826: u32 = undefined;
    var x827: u32 = undefined;
    mulxU32(&x826, &x827, x804, 0xecec196a);
    var x828: u32 = undefined;
    var x829: u32 = undefined;
    mulxU32(&x828, &x829, x804, 0xccc52973);
    var x830: u32 = undefined;
    var x831: u1 = undefined;
    addcarryxU32(&x830, &x831, 0x0, x829, x826);
    var x832: u32 = undefined;
    var x833: u1 = undefined;
    addcarryxU32(&x832, &x833, x831, x827, x824);
    var x834: u32 = undefined;
    var x835: u1 = undefined;
    addcarryxU32(&x834, &x835, x833, x825, x822);
    var x836: u32 = undefined;
    var x837: u1 = undefined;
    addcarryxU32(&x836, &x837, x835, x823, x820);
    var x838: u32 = undefined;
    var x839: u1 = undefined;
    addcarryxU32(&x838, &x839, x837, x821, x818);
    var x840: u32 = undefined;
    var x841: u1 = undefined;
    addcarryxU32(&x840, &x841, x839, x819, x816);
    var x842: u32 = undefined;
    var x843: u1 = undefined;
    addcarryxU32(&x842, &x843, x841, x817, x814);
    var x844: u32 = undefined;
    var x845: u1 = undefined;
    addcarryxU32(&x844, &x845, x843, x815, x812);
    var x846: u32 = undefined;
    var x847: u1 = undefined;
    addcarryxU32(&x846, &x847, x845, x813, x810);
    var x848: u32 = undefined;
    var x849: u1 = undefined;
    addcarryxU32(&x848, &x849, x847, x811, x808);
    var x850: u32 = undefined;
    var x851: u1 = undefined;
    addcarryxU32(&x850, &x851, x849, x809, x806);
    const x852 = (cast(u32, x851) + x807);
    var x853: u32 = undefined;
    var x854: u1 = undefined;
    addcarryxU32(&x853, &x854, 0x0, x778, x828);
    var x855: u32 = undefined;
    var x856: u1 = undefined;
    addcarryxU32(&x855, &x856, x854, x780, x830);
    var x857: u32 = undefined;
    var x858: u1 = undefined;
    addcarryxU32(&x857, &x858, x856, x782, x832);
    var x859: u32 = undefined;
    var x860: u1 = undefined;
    addcarryxU32(&x859, &x860, x858, x784, x834);
    var x861: u32 = undefined;
    var x862: u1 = undefined;
    addcarryxU32(&x861, &x862, x860, x786, x836);
    var x863: u32 = undefined;
    var x864: u1 = undefined;
    addcarryxU32(&x863, &x864, x862, x788, x838);
    var x865: u32 = undefined;
    var x866: u1 = undefined;
    addcarryxU32(&x865, &x866, x864, x790, x840);
    var x867: u32 = undefined;
    var x868: u1 = undefined;
    addcarryxU32(&x867, &x868, x866, x792, x842);
    var x869: u32 = undefined;
    var x870: u1 = undefined;
    addcarryxU32(&x869, &x870, x868, x794, x844);
    var x871: u32 = undefined;
    var x872: u1 = undefined;
    addcarryxU32(&x871, &x872, x870, x796, x846);
    var x873: u32 = undefined;
    var x874: u1 = undefined;
    addcarryxU32(&x873, &x874, x872, x798, x848);
    var x875: u32 = undefined;
    var x876: u1 = undefined;
    addcarryxU32(&x875, &x876, x874, x800, x850);
    var x877: u32 = undefined;
    var x878: u1 = undefined;
    addcarryxU32(&x877, &x878, x876, x802, x852);
    const x879 = (cast(u32, x878) + cast(u32, x803));
    var x880: u32 = undefined;
    var x881: u32 = undefined;
    mulxU32(&x880, &x881, x6, (arg1[11]));
    var x882: u32 = undefined;
    var x883: u32 = undefined;
    mulxU32(&x882, &x883, x6, (arg1[10]));
    var x884: u32 = undefined;
    var x885: u32 = undefined;
    mulxU32(&x884, &x885, x6, (arg1[9]));
    var x886: u32 = undefined;
    var x887: u32 = undefined;
    mulxU32(&x886, &x887, x6, (arg1[8]));
    var x888: u32 = undefined;
    var x889: u32 = undefined;
    mulxU32(&x888, &x889, x6, (arg1[7]));
    var x890: u32 = undefined;
    var x891: u32 = undefined;
    mulxU32(&x890, &x891, x6, (arg1[6]));
    var x892: u32 = undefined;
    var x893: u32 = undefined;
    mulxU32(&x892, &x893, x6, (arg1[5]));
    var x894: u32 = undefined;
    var x895: u32 = undefined;
    mulxU32(&x894, &x895, x6, (arg1[4]));
    var x896: u32 = undefined;
    var x897: u32 = undefined;
    mulxU32(&x896, &x897, x6, (arg1[3]));
    var x898: u32 = undefined;
    var x899: u32 = undefined;
    mulxU32(&x898, &x899, x6, (arg1[2]));
    var x900: u32 = undefined;
    var x901: u32 = undefined;
    mulxU32(&x900, &x901, x6, (arg1[1]));
    var x902: u32 = undefined;
    var x903: u32 = undefined;
    mulxU32(&x902, &x903, x6, (arg1[0]));
    var x904: u32 = undefined;
    var x905: u1 = undefined;
    addcarryxU32(&x904, &x905, 0x0, x903, x900);
    var x906: u32 = undefined;
    var x907: u1 = undefined;
    addcarryxU32(&x906, &x907, x905, x901, x898);
    var x908: u32 = undefined;
    var x909: u1 = undefined;
    addcarryxU32(&x908, &x909, x907, x899, x896);
    var x910: u32 = undefined;
    var x911: u1 = undefined;
    addcarryxU32(&x910, &x911, x909, x897, x894);
    var x912: u32 = undefined;
    var x913: u1 = undefined;
    addcarryxU32(&x912, &x913, x911, x895, x892);
    var x914: u32 = undefined;
    var x915: u1 = undefined;
    addcarryxU32(&x914, &x915, x913, x893, x890);
    var x916: u32 = undefined;
    var x917: u1 = undefined;
    addcarryxU32(&x916, &x917, x915, x891, x888);
    var x918: u32 = undefined;
    var x919: u1 = undefined;
    addcarryxU32(&x918, &x919, x917, x889, x886);
    var x920: u32 = undefined;
    var x921: u1 = undefined;
    addcarryxU32(&x920, &x921, x919, x887, x884);
    var x922: u32 = undefined;
    var x923: u1 = undefined;
    addcarryxU32(&x922, &x923, x921, x885, x882);
    var x924: u32 = undefined;
    var x925: u1 = undefined;
    addcarryxU32(&x924, &x925, x923, x883, x880);
    const x926 = (cast(u32, x925) + x881);
    var x927: u32 = undefined;
    var x928: u1 = undefined;
    addcarryxU32(&x927, &x928, 0x0, x855, x902);
    var x929: u32 = undefined;
    var x930: u1 = undefined;
    addcarryxU32(&x929, &x930, x928, x857, x904);
    var x931: u32 = undefined;
    var x932: u1 = undefined;
    addcarryxU32(&x931, &x932, x930, x859, x906);
    var x933: u32 = undefined;
    var x934: u1 = undefined;
    addcarryxU32(&x933, &x934, x932, x861, x908);
    var x935: u32 = undefined;
    var x936: u1 = undefined;
    addcarryxU32(&x935, &x936, x934, x863, x910);
    var x937: u32 = undefined;
    var x938: u1 = undefined;
    addcarryxU32(&x937, &x938, x936, x865, x912);
    var x939: u32 = undefined;
    var x940: u1 = undefined;
    addcarryxU32(&x939, &x940, x938, x867, x914);
    var x941: u32 = undefined;
    var x942: u1 = undefined;
    addcarryxU32(&x941, &x942, x940, x869, x916);
    var x943: u32 = undefined;
    var x944: u1 = undefined;
    addcarryxU32(&x943, &x944, x942, x871, x918);
    var x945: u32 = undefined;
    var x946: u1 = undefined;
    addcarryxU32(&x945, &x946, x944, x873, x920);
    var x947: u32 = undefined;
    var x948: u1 = undefined;
    addcarryxU32(&x947, &x948, x946, x875, x922);
    var x949: u32 = undefined;
    var x950: u1 = undefined;
    addcarryxU32(&x949, &x950, x948, x877, x924);
    var x951: u32 = undefined;
    var x952: u1 = undefined;
    addcarryxU32(&x951, &x952, x950, x879, x926);
    var x953: u32 = undefined;
    var x954: u32 = undefined;
    mulxU32(&x953, &x954, x927, 0xe88fdc45);
    var x955: u32 = undefined;
    var x956: u32 = undefined;
    mulxU32(&x955, &x956, x953, 0xffffffff);
    var x957: u32 = undefined;
    var x958: u32 = undefined;
    mulxU32(&x957, &x958, x953, 0xffffffff);
    var x959: u32 = undefined;
    var x960: u32 = undefined;
    mulxU32(&x959, &x960, x953, 0xffffffff);
    var x961: u32 = undefined;
    var x962: u32 = undefined;
    mulxU32(&x961, &x962, x953, 0xffffffff);
    var x963: u32 = undefined;
    var x964: u32 = undefined;
    mulxU32(&x963, &x964, x953, 0xffffffff);
    var x965: u32 = undefined;
    var x966: u32 = undefined;
    mulxU32(&x965, &x966, x953, 0xffffffff);
    var x967: u32 = undefined;
    var x968: u32 = undefined;
    mulxU32(&x967, &x968, x953, 0xc7634d81);
    var x969: u32 = undefined;
    var x970: u32 = undefined;
    mulxU32(&x969, &x970, x953, 0xf4372ddf);
    var x971: u32 = undefined;
    var x972: u32 = undefined;
    mulxU32(&x971, &x972, x953, 0x581a0db2);
    var x973: u32 = undefined;
    var x974: u32 = undefined;
    mulxU32(&x973, &x974, x953, 0x48b0a77a);
    var x975: u32 = undefined;
    var x976: u32 = undefined;
    mulxU32(&x975, &x976, x953, 0xecec196a);
    var x977: u32 = undefined;
    var x978: u32 = undefined;
    mulxU32(&x977, &x978, x953, 0xccc52973);
    var x979: u32 = undefined;
    var x980: u1 = undefined;
    addcarryxU32(&x979, &x980, 0x0, x978, x975);
    var x981: u32 = undefined;
    var x982: u1 = undefined;
    addcarryxU32(&x981, &x982, x980, x976, x973);
    var x983: u32 = undefined;
    var x984: u1 = undefined;
    addcarryxU32(&x983, &x984, x982, x974, x971);
    var x985: u32 = undefined;
    var x986: u1 = undefined;
    addcarryxU32(&x985, &x986, x984, x972, x969);
    var x987: u32 = undefined;
    var x988: u1 = undefined;
    addcarryxU32(&x987, &x988, x986, x970, x967);
    var x989: u32 = undefined;
    var x990: u1 = undefined;
    addcarryxU32(&x989, &x990, x988, x968, x965);
    var x991: u32 = undefined;
    var x992: u1 = undefined;
    addcarryxU32(&x991, &x992, x990, x966, x963);
    var x993: u32 = undefined;
    var x994: u1 = undefined;
    addcarryxU32(&x993, &x994, x992, x964, x961);
    var x995: u32 = undefined;
    var x996: u1 = undefined;
    addcarryxU32(&x995, &x996, x994, x962, x959);
    var x997: u32 = undefined;
    var x998: u1 = undefined;
    addcarryxU32(&x997, &x998, x996, x960, x957);
    var x999: u32 = undefined;
    var x1000: u1 = undefined;
    addcarryxU32(&x999, &x1000, x998, x958, x955);
    const x1001 = (cast(u32, x1000) + x956);
    var x1002: u32 = undefined;
    var x1003: u1 = undefined;
    addcarryxU32(&x1002, &x1003, 0x0, x927, x977);
    var x1004: u32 = undefined;
    var x1005: u1 = undefined;
    addcarryxU32(&x1004, &x1005, x1003, x929, x979);
    var x1006: u32 = undefined;
    var x1007: u1 = undefined;
    addcarryxU32(&x1006, &x1007, x1005, x931, x981);
    var x1008: u32 = undefined;
    var x1009: u1 = undefined;
    addcarryxU32(&x1008, &x1009, x1007, x933, x983);
    var x1010: u32 = undefined;
    var x1011: u1 = undefined;
    addcarryxU32(&x1010, &x1011, x1009, x935, x985);
    var x1012: u32 = undefined;
    var x1013: u1 = undefined;
    addcarryxU32(&x1012, &x1013, x1011, x937, x987);
    var x1014: u32 = undefined;
    var x1015: u1 = undefined;
    addcarryxU32(&x1014, &x1015, x1013, x939, x989);
    var x1016: u32 = undefined;
    var x1017: u1 = undefined;
    addcarryxU32(&x1016, &x1017, x1015, x941, x991);
    var x1018: u32 = undefined;
    var x1019: u1 = undefined;
    addcarryxU32(&x1018, &x1019, x1017, x943, x993);
    var x1020: u32 = undefined;
    var x1021: u1 = undefined;
    addcarryxU32(&x1020, &x1021, x1019, x945, x995);
    var x1022: u32 = undefined;
    var x1023: u1 = undefined;
    addcarryxU32(&x1022, &x1023, x1021, x947, x997);
    var x1024: u32 = undefined;
    var x1025: u1 = undefined;
    addcarryxU32(&x1024, &x1025, x1023, x949, x999);
    var x1026: u32 = undefined;
    var x1027: u1 = undefined;
    addcarryxU32(&x1026, &x1027, x1025, x951, x1001);
    const x1028 = (cast(u32, x1027) + cast(u32, x952));
    var x1029: u32 = undefined;
    var x1030: u32 = undefined;
    mulxU32(&x1029, &x1030, x7, (arg1[11]));
    var x1031: u32 = undefined;
    var x1032: u32 = undefined;
    mulxU32(&x1031, &x1032, x7, (arg1[10]));
    var x1033: u32 = undefined;
    var x1034: u32 = undefined;
    mulxU32(&x1033, &x1034, x7, (arg1[9]));
    var x1035: u32 = undefined;
    var x1036: u32 = undefined;
    mulxU32(&x1035, &x1036, x7, (arg1[8]));
    var x1037: u32 = undefined;
    var x1038: u32 = undefined;
    mulxU32(&x1037, &x1038, x7, (arg1[7]));
    var x1039: u32 = undefined;
    var x1040: u32 = undefined;
    mulxU32(&x1039, &x1040, x7, (arg1[6]));
    var x1041: u32 = undefined;
    var x1042: u32 = undefined;
    mulxU32(&x1041, &x1042, x7, (arg1[5]));
    var x1043: u32 = undefined;
    var x1044: u32 = undefined;
    mulxU32(&x1043, &x1044, x7, (arg1[4]));
    var x1045: u32 = undefined;
    var x1046: u32 = undefined;
    mulxU32(&x1045, &x1046, x7, (arg1[3]));
    var x1047: u32 = undefined;
    var x1048: u32 = undefined;
    mulxU32(&x1047, &x1048, x7, (arg1[2]));
    var x1049: u32 = undefined;
    var x1050: u32 = undefined;
    mulxU32(&x1049, &x1050, x7, (arg1[1]));
    var x1051: u32 = undefined;
    var x1052: u32 = undefined;
    mulxU32(&x1051, &x1052, x7, (arg1[0]));
    var x1053: u32 = undefined;
    var x1054: u1 = undefined;
    addcarryxU32(&x1053, &x1054, 0x0, x1052, x1049);
    var x1055: u32 = undefined;
    var x1056: u1 = undefined;
    addcarryxU32(&x1055, &x1056, x1054, x1050, x1047);
    var x1057: u32 = undefined;
    var x1058: u1 = undefined;
    addcarryxU32(&x1057, &x1058, x1056, x1048, x1045);
    var x1059: u32 = undefined;
    var x1060: u1 = undefined;
    addcarryxU32(&x1059, &x1060, x1058, x1046, x1043);
    var x1061: u32 = undefined;
    var x1062: u1 = undefined;
    addcarryxU32(&x1061, &x1062, x1060, x1044, x1041);
    var x1063: u32 = undefined;
    var x1064: u1 = undefined;
    addcarryxU32(&x1063, &x1064, x1062, x1042, x1039);
    var x1065: u32 = undefined;
    var x1066: u1 = undefined;
    addcarryxU32(&x1065, &x1066, x1064, x1040, x1037);
    var x1067: u32 = undefined;
    var x1068: u1 = undefined;
    addcarryxU32(&x1067, &x1068, x1066, x1038, x1035);
    var x1069: u32 = undefined;
    var x1070: u1 = undefined;
    addcarryxU32(&x1069, &x1070, x1068, x1036, x1033);
    var x1071: u32 = undefined;
    var x1072: u1 = undefined;
    addcarryxU32(&x1071, &x1072, x1070, x1034, x1031);
    var x1073: u32 = undefined;
    var x1074: u1 = undefined;
    addcarryxU32(&x1073, &x1074, x1072, x1032, x1029);
    const x1075 = (cast(u32, x1074) + x1030);
    var x1076: u32 = undefined;
    var x1077: u1 = undefined;
    addcarryxU32(&x1076, &x1077, 0x0, x1004, x1051);
    var x1078: u32 = undefined;
    var x1079: u1 = undefined;
    addcarryxU32(&x1078, &x1079, x1077, x1006, x1053);
    var x1080: u32 = undefined;
    var x1081: u1 = undefined;
    addcarryxU32(&x1080, &x1081, x1079, x1008, x1055);
    var x1082: u32 = undefined;
    var x1083: u1 = undefined;
    addcarryxU32(&x1082, &x1083, x1081, x1010, x1057);
    var x1084: u32 = undefined;
    var x1085: u1 = undefined;
    addcarryxU32(&x1084, &x1085, x1083, x1012, x1059);
    var x1086: u32 = undefined;
    var x1087: u1 = undefined;
    addcarryxU32(&x1086, &x1087, x1085, x1014, x1061);
    var x1088: u32 = undefined;
    var x1089: u1 = undefined;
    addcarryxU32(&x1088, &x1089, x1087, x1016, x1063);
    var x1090: u32 = undefined;
    var x1091: u1 = undefined;
    addcarryxU32(&x1090, &x1091, x1089, x1018, x1065);
    var x1092: u32 = undefined;
    var x1093: u1 = undefined;
    addcarryxU32(&x1092, &x1093, x1091, x1020, x1067);
    var x1094: u32 = undefined;
    var x1095: u1 = undefined;
    addcarryxU32(&x1094, &x1095, x1093, x1022, x1069);
    var x1096: u32 = undefined;
    var x1097: u1 = undefined;
    addcarryxU32(&x1096, &x1097, x1095, x1024, x1071);
    var x1098: u32 = undefined;
    var x1099: u1 = undefined;
    addcarryxU32(&x1098, &x1099, x1097, x1026, x1073);
    var x1100: u32 = undefined;
    var x1101: u1 = undefined;
    addcarryxU32(&x1100, &x1101, x1099, x1028, x1075);
    var x1102: u32 = undefined;
    var x1103: u32 = undefined;
    mulxU32(&x1102, &x1103, x1076, 0xe88fdc45);
    var x1104: u32 = undefined;
    var x1105: u32 = undefined;
    mulxU32(&x1104, &x1105, x1102, 0xffffffff);
    var x1106: u32 = undefined;
    var x1107: u32 = undefined;
    mulxU32(&x1106, &x1107, x1102, 0xffffffff);
    var x1108: u32 = undefined;
    var x1109: u32 = undefined;
    mulxU32(&x1108, &x1109, x1102, 0xffffffff);
    var x1110: u32 = undefined;
    var x1111: u32 = undefined;
    mulxU32(&x1110, &x1111, x1102, 0xffffffff);
    var x1112: u32 = undefined;
    var x1113: u32 = undefined;
    mulxU32(&x1112, &x1113, x1102, 0xffffffff);
    var x1114: u32 = undefined;
    var x1115: u32 = undefined;
    mulxU32(&x1114, &x1115, x1102, 0xffffffff);
    var x1116: u32 = undefined;
    var x1117: u32 = undefined;
    mulxU32(&x1116, &x1117, x1102, 0xc7634d81);
    var x1118: u32 = undefined;
    var x1119: u32 = undefined;
    mulxU32(&x1118, &x1119, x1102, 0xf4372ddf);
    var x1120: u32 = undefined;
    var x1121: u32 = undefined;
    mulxU32(&x1120, &x1121, x1102, 0x581a0db2);
    var x1122: u32 = undefined;
    var x1123: u32 = undefined;
    mulxU32(&x1122, &x1123, x1102, 0x48b0a77a);
    var x1124: u32 = undefined;
    var x1125: u32 = undefined;
    mulxU32(&x1124, &x1125, x1102, 0xecec196a);
    var x1126: u32 = undefined;
    var x1127: u32 = undefined;
    mulxU32(&x1126, &x1127, x1102, 0xccc52973);
    var x1128: u32 = undefined;
    var x1129: u1 = undefined;
    addcarryxU32(&x1128, &x1129, 0x0, x1127, x1124);
    var x1130: u32 = undefined;
    var x1131: u1 = undefined;
    addcarryxU32(&x1130, &x1131, x1129, x1125, x1122);
    var x1132: u32 = undefined;
    var x1133: u1 = undefined;
    addcarryxU32(&x1132, &x1133, x1131, x1123, x1120);
    var x1134: u32 = undefined;
    var x1135: u1 = undefined;
    addcarryxU32(&x1134, &x1135, x1133, x1121, x1118);
    var x1136: u32 = undefined;
    var x1137: u1 = undefined;
    addcarryxU32(&x1136, &x1137, x1135, x1119, x1116);
    var x1138: u32 = undefined;
    var x1139: u1 = undefined;
    addcarryxU32(&x1138, &x1139, x1137, x1117, x1114);
    var x1140: u32 = undefined;
    var x1141: u1 = undefined;
    addcarryxU32(&x1140, &x1141, x1139, x1115, x1112);
    var x1142: u32 = undefined;
    var x1143: u1 = undefined;
    addcarryxU32(&x1142, &x1143, x1141, x1113, x1110);
    var x1144: u32 = undefined;
    var x1145: u1 = undefined;
    addcarryxU32(&x1144, &x1145, x1143, x1111, x1108);
    var x1146: u32 = undefined;
    var x1147: u1 = undefined;
    addcarryxU32(&x1146, &x1147, x1145, x1109, x1106);
    var x1148: u32 = undefined;
    var x1149: u1 = undefined;
    addcarryxU32(&x1148, &x1149, x1147, x1107, x1104);
    const x1150 = (cast(u32, x1149) + x1105);
    var x1151: u32 = undefined;
    var x1152: u1 = undefined;
    addcarryxU32(&x1151, &x1152, 0x0, x1076, x1126);
    var x1153: u32 = undefined;
    var x1154: u1 = undefined;
    addcarryxU32(&x1153, &x1154, x1152, x1078, x1128);
    var x1155: u32 = undefined;
    var x1156: u1 = undefined;
    addcarryxU32(&x1155, &x1156, x1154, x1080, x1130);
    var x1157: u32 = undefined;
    var x1158: u1 = undefined;
    addcarryxU32(&x1157, &x1158, x1156, x1082, x1132);
    var x1159: u32 = undefined;
    var x1160: u1 = undefined;
    addcarryxU32(&x1159, &x1160, x1158, x1084, x1134);
    var x1161: u32 = undefined;
    var x1162: u1 = undefined;
    addcarryxU32(&x1161, &x1162, x1160, x1086, x1136);
    var x1163: u32 = undefined;
    var x1164: u1 = undefined;
    addcarryxU32(&x1163, &x1164, x1162, x1088, x1138);
    var x1165: u32 = undefined;
    var x1166: u1 = undefined;
    addcarryxU32(&x1165, &x1166, x1164, x1090, x1140);
    var x1167: u32 = undefined;
    var x1168: u1 = undefined;
    addcarryxU32(&x1167, &x1168, x1166, x1092, x1142);
    var x1169: u32 = undefined;
    var x1170: u1 = undefined;
    addcarryxU32(&x1169, &x1170, x1168, x1094, x1144);
    var x1171: u32 = undefined;
    var x1172: u1 = undefined;
    addcarryxU32(&x1171, &x1172, x1170, x1096, x1146);
    var x1173: u32 = undefined;
    var x1174: u1 = undefined;
    addcarryxU32(&x1173, &x1174, x1172, x1098, x1148);
    var x1175: u32 = undefined;
    var x1176: u1 = undefined;
    addcarryxU32(&x1175, &x1176, x1174, x1100, x1150);
    const x1177 = (cast(u32, x1176) + cast(u32, x1101));
    var x1178: u32 = undefined;
    var x1179: u32 = undefined;
    mulxU32(&x1178, &x1179, x8, (arg1[11]));
    var x1180: u32 = undefined;
    var x1181: u32 = undefined;
    mulxU32(&x1180, &x1181, x8, (arg1[10]));
    var x1182: u32 = undefined;
    var x1183: u32 = undefined;
    mulxU32(&x1182, &x1183, x8, (arg1[9]));
    var x1184: u32 = undefined;
    var x1185: u32 = undefined;
    mulxU32(&x1184, &x1185, x8, (arg1[8]));
    var x1186: u32 = undefined;
    var x1187: u32 = undefined;
    mulxU32(&x1186, &x1187, x8, (arg1[7]));
    var x1188: u32 = undefined;
    var x1189: u32 = undefined;
    mulxU32(&x1188, &x1189, x8, (arg1[6]));
    var x1190: u32 = undefined;
    var x1191: u32 = undefined;
    mulxU32(&x1190, &x1191, x8, (arg1[5]));
    var x1192: u32 = undefined;
    var x1193: u32 = undefined;
    mulxU32(&x1192, &x1193, x8, (arg1[4]));
    var x1194: u32 = undefined;
    var x1195: u32 = undefined;
    mulxU32(&x1194, &x1195, x8, (arg1[3]));
    var x1196: u32 = undefined;
    var x1197: u32 = undefined;
    mulxU32(&x1196, &x1197, x8, (arg1[2]));
    var x1198: u32 = undefined;
    var x1199: u32 = undefined;
    mulxU32(&x1198, &x1199, x8, (arg1[1]));
    var x1200: u32 = undefined;
    var x1201: u32 = undefined;
    mulxU32(&x1200, &x1201, x8, (arg1[0]));
    var x1202: u32 = undefined;
    var x1203: u1 = undefined;
    addcarryxU32(&x1202, &x1203, 0x0, x1201, x1198);
    var x1204: u32 = undefined;
    var x1205: u1 = undefined;
    addcarryxU32(&x1204, &x1205, x1203, x1199, x1196);
    var x1206: u32 = undefined;
    var x1207: u1 = undefined;
    addcarryxU32(&x1206, &x1207, x1205, x1197, x1194);
    var x1208: u32 = undefined;
    var x1209: u1 = undefined;
    addcarryxU32(&x1208, &x1209, x1207, x1195, x1192);
    var x1210: u32 = undefined;
    var x1211: u1 = undefined;
    addcarryxU32(&x1210, &x1211, x1209, x1193, x1190);
    var x1212: u32 = undefined;
    var x1213: u1 = undefined;
    addcarryxU32(&x1212, &x1213, x1211, x1191, x1188);
    var x1214: u32 = undefined;
    var x1215: u1 = undefined;
    addcarryxU32(&x1214, &x1215, x1213, x1189, x1186);
    var x1216: u32 = undefined;
    var x1217: u1 = undefined;
    addcarryxU32(&x1216, &x1217, x1215, x1187, x1184);
    var x1218: u32 = undefined;
    var x1219: u1 = undefined;
    addcarryxU32(&x1218, &x1219, x1217, x1185, x1182);
    var x1220: u32 = undefined;
    var x1221: u1 = undefined;
    addcarryxU32(&x1220, &x1221, x1219, x1183, x1180);
    var x1222: u32 = undefined;
    var x1223: u1 = undefined;
    addcarryxU32(&x1222, &x1223, x1221, x1181, x1178);
    const x1224 = (cast(u32, x1223) + x1179);
    var x1225: u32 = undefined;
    var x1226: u1 = undefined;
    addcarryxU32(&x1225, &x1226, 0x0, x1153, x1200);
    var x1227: u32 = undefined;
    var x1228: u1 = undefined;
    addcarryxU32(&x1227, &x1228, x1226, x1155, x1202);
    var x1229: u32 = undefined;
    var x1230: u1 = undefined;
    addcarryxU32(&x1229, &x1230, x1228, x1157, x1204);
    var x1231: u32 = undefined;
    var x1232: u1 = undefined;
    addcarryxU32(&x1231, &x1232, x1230, x1159, x1206);
    var x1233: u32 = undefined;
    var x1234: u1 = undefined;
    addcarryxU32(&x1233, &x1234, x1232, x1161, x1208);
    var x1235: u32 = undefined;
    var x1236: u1 = undefined;
    addcarryxU32(&x1235, &x1236, x1234, x1163, x1210);
    var x1237: u32 = undefined;
    var x1238: u1 = undefined;
    addcarryxU32(&x1237, &x1238, x1236, x1165, x1212);
    var x1239: u32 = undefined;
    var x1240: u1 = undefined;
    addcarryxU32(&x1239, &x1240, x1238, x1167, x1214);
    var x1241: u32 = undefined;
    var x1242: u1 = undefined;
    addcarryxU32(&x1241, &x1242, x1240, x1169, x1216);
    var x1243: u32 = undefined;
    var x1244: u1 = undefined;
    addcarryxU32(&x1243, &x1244, x1242, x1171, x1218);
    var x1245: u32 = undefined;
    var x1246: u1 = undefined;
    addcarryxU32(&x1245, &x1246, x1244, x1173, x1220);
    var x1247: u32 = undefined;
    var x1248: u1 = undefined;
    addcarryxU32(&x1247, &x1248, x1246, x1175, x1222);
    var x1249: u32 = undefined;
    var x1250: u1 = undefined;
    addcarryxU32(&x1249, &x1250, x1248, x1177, x1224);
    var x1251: u32 = undefined;
    var x1252: u32 = undefined;
    mulxU32(&x1251, &x1252, x1225, 0xe88fdc45);
    var x1253: u32 = undefined;
    var x1254: u32 = undefined;
    mulxU32(&x1253, &x1254, x1251, 0xffffffff);
    var x1255: u32 = undefined;
    var x1256: u32 = undefined;
    mulxU32(&x1255, &x1256, x1251, 0xffffffff);
    var x1257: u32 = undefined;
    var x1258: u32 = undefined;
    mulxU32(&x1257, &x1258, x1251, 0xffffffff);
    var x1259: u32 = undefined;
    var x1260: u32 = undefined;
    mulxU32(&x1259, &x1260, x1251, 0xffffffff);
    var x1261: u32 = undefined;
    var x1262: u32 = undefined;
    mulxU32(&x1261, &x1262, x1251, 0xffffffff);
    var x1263: u32 = undefined;
    var x1264: u32 = undefined;
    mulxU32(&x1263, &x1264, x1251, 0xffffffff);
    var x1265: u32 = undefined;
    var x1266: u32 = undefined;
    mulxU32(&x1265, &x1266, x1251, 0xc7634d81);
    var x1267: u32 = undefined;
    var x1268: u32 = undefined;
    mulxU32(&x1267, &x1268, x1251, 0xf4372ddf);
    var x1269: u32 = undefined;
    var x1270: u32 = undefined;
    mulxU32(&x1269, &x1270, x1251, 0x581a0db2);
    var x1271: u32 = undefined;
    var x1272: u32 = undefined;
    mulxU32(&x1271, &x1272, x1251, 0x48b0a77a);
    var x1273: u32 = undefined;
    var x1274: u32 = undefined;
    mulxU32(&x1273, &x1274, x1251, 0xecec196a);
    var x1275: u32 = undefined;
    var x1276: u32 = undefined;
    mulxU32(&x1275, &x1276, x1251, 0xccc52973);
    var x1277: u32 = undefined;
    var x1278: u1 = undefined;
    addcarryxU32(&x1277, &x1278, 0x0, x1276, x1273);
    var x1279: u32 = undefined;
    var x1280: u1 = undefined;
    addcarryxU32(&x1279, &x1280, x1278, x1274, x1271);
    var x1281: u32 = undefined;
    var x1282: u1 = undefined;
    addcarryxU32(&x1281, &x1282, x1280, x1272, x1269);
    var x1283: u32 = undefined;
    var x1284: u1 = undefined;
    addcarryxU32(&x1283, &x1284, x1282, x1270, x1267);
    var x1285: u32 = undefined;
    var x1286: u1 = undefined;
    addcarryxU32(&x1285, &x1286, x1284, x1268, x1265);
    var x1287: u32 = undefined;
    var x1288: u1 = undefined;
    addcarryxU32(&x1287, &x1288, x1286, x1266, x1263);
    var x1289: u32 = undefined;
    var x1290: u1 = undefined;
    addcarryxU32(&x1289, &x1290, x1288, x1264, x1261);
    var x1291: u32 = undefined;
    var x1292: u1 = undefined;
    addcarryxU32(&x1291, &x1292, x1290, x1262, x1259);
    var x1293: u32 = undefined;
    var x1294: u1 = undefined;
    addcarryxU32(&x1293, &x1294, x1292, x1260, x1257);
    var x1295: u32 = undefined;
    var x1296: u1 = undefined;
    addcarryxU32(&x1295, &x1296, x1294, x1258, x1255);
    var x1297: u32 = undefined;
    var x1298: u1 = undefined;
    addcarryxU32(&x1297, &x1298, x1296, x1256, x1253);
    const x1299 = (cast(u32, x1298) + x1254);
    var x1300: u32 = undefined;
    var x1301: u1 = undefined;
    addcarryxU32(&x1300, &x1301, 0x0, x1225, x1275);
    var x1302: u32 = undefined;
    var x1303: u1 = undefined;
    addcarryxU32(&x1302, &x1303, x1301, x1227, x1277);
    var x1304: u32 = undefined;
    var x1305: u1 = undefined;
    addcarryxU32(&x1304, &x1305, x1303, x1229, x1279);
    var x1306: u32 = undefined;
    var x1307: u1 = undefined;
    addcarryxU32(&x1306, &x1307, x1305, x1231, x1281);
    var x1308: u32 = undefined;
    var x1309: u1 = undefined;
    addcarryxU32(&x1308, &x1309, x1307, x1233, x1283);
    var x1310: u32 = undefined;
    var x1311: u1 = undefined;
    addcarryxU32(&x1310, &x1311, x1309, x1235, x1285);
    var x1312: u32 = undefined;
    var x1313: u1 = undefined;
    addcarryxU32(&x1312, &x1313, x1311, x1237, x1287);
    var x1314: u32 = undefined;
    var x1315: u1 = undefined;
    addcarryxU32(&x1314, &x1315, x1313, x1239, x1289);
    var x1316: u32 = undefined;
    var x1317: u1 = undefined;
    addcarryxU32(&x1316, &x1317, x1315, x1241, x1291);
    var x1318: u32 = undefined;
    var x1319: u1 = undefined;
    addcarryxU32(&x1318, &x1319, x1317, x1243, x1293);
    var x1320: u32 = undefined;
    var x1321: u1 = undefined;
    addcarryxU32(&x1320, &x1321, x1319, x1245, x1295);
    var x1322: u32 = undefined;
    var x1323: u1 = undefined;
    addcarryxU32(&x1322, &x1323, x1321, x1247, x1297);
    var x1324: u32 = undefined;
    var x1325: u1 = undefined;
    addcarryxU32(&x1324, &x1325, x1323, x1249, x1299);
    const x1326 = (cast(u32, x1325) + cast(u32, x1250));
    var x1327: u32 = undefined;
    var x1328: u32 = undefined;
    mulxU32(&x1327, &x1328, x9, (arg1[11]));
    var x1329: u32 = undefined;
    var x1330: u32 = undefined;
    mulxU32(&x1329, &x1330, x9, (arg1[10]));
    var x1331: u32 = undefined;
    var x1332: u32 = undefined;
    mulxU32(&x1331, &x1332, x9, (arg1[9]));
    var x1333: u32 = undefined;
    var x1334: u32 = undefined;
    mulxU32(&x1333, &x1334, x9, (arg1[8]));
    var x1335: u32 = undefined;
    var x1336: u32 = undefined;
    mulxU32(&x1335, &x1336, x9, (arg1[7]));
    var x1337: u32 = undefined;
    var x1338: u32 = undefined;
    mulxU32(&x1337, &x1338, x9, (arg1[6]));
    var x1339: u32 = undefined;
    var x1340: u32 = undefined;
    mulxU32(&x1339, &x1340, x9, (arg1[5]));
    var x1341: u32 = undefined;
    var x1342: u32 = undefined;
    mulxU32(&x1341, &x1342, x9, (arg1[4]));
    var x1343: u32 = undefined;
    var x1344: u32 = undefined;
    mulxU32(&x1343, &x1344, x9, (arg1[3]));
    var x1345: u32 = undefined;
    var x1346: u32 = undefined;
    mulxU32(&x1345, &x1346, x9, (arg1[2]));
    var x1347: u32 = undefined;
    var x1348: u32 = undefined;
    mulxU32(&x1347, &x1348, x9, (arg1[1]));
    var x1349: u32 = undefined;
    var x1350: u32 = undefined;
    mulxU32(&x1349, &x1350, x9, (arg1[0]));
    var x1351: u32 = undefined;
    var x1352: u1 = undefined;
    addcarryxU32(&x1351, &x1352, 0x0, x1350, x1347);
    var x1353: u32 = undefined;
    var x1354: u1 = undefined;
    addcarryxU32(&x1353, &x1354, x1352, x1348, x1345);
    var x1355: u32 = undefined;
    var x1356: u1 = undefined;
    addcarryxU32(&x1355, &x1356, x1354, x1346, x1343);
    var x1357: u32 = undefined;
    var x1358: u1 = undefined;
    addcarryxU32(&x1357, &x1358, x1356, x1344, x1341);
    var x1359: u32 = undefined;
    var x1360: u1 = undefined;
    addcarryxU32(&x1359, &x1360, x1358, x1342, x1339);
    var x1361: u32 = undefined;
    var x1362: u1 = undefined;
    addcarryxU32(&x1361, &x1362, x1360, x1340, x1337);
    var x1363: u32 = undefined;
    var x1364: u1 = undefined;
    addcarryxU32(&x1363, &x1364, x1362, x1338, x1335);
    var x1365: u32 = undefined;
    var x1366: u1 = undefined;
    addcarryxU32(&x1365, &x1366, x1364, x1336, x1333);
    var x1367: u32 = undefined;
    var x1368: u1 = undefined;
    addcarryxU32(&x1367, &x1368, x1366, x1334, x1331);
    var x1369: u32 = undefined;
    var x1370: u1 = undefined;
    addcarryxU32(&x1369, &x1370, x1368, x1332, x1329);
    var x1371: u32 = undefined;
    var x1372: u1 = undefined;
    addcarryxU32(&x1371, &x1372, x1370, x1330, x1327);
    const x1373 = (cast(u32, x1372) + x1328);
    var x1374: u32 = undefined;
    var x1375: u1 = undefined;
    addcarryxU32(&x1374, &x1375, 0x0, x1302, x1349);
    var x1376: u32 = undefined;
    var x1377: u1 = undefined;
    addcarryxU32(&x1376, &x1377, x1375, x1304, x1351);
    var x1378: u32 = undefined;
    var x1379: u1 = undefined;
    addcarryxU32(&x1378, &x1379, x1377, x1306, x1353);
    var x1380: u32 = undefined;
    var x1381: u1 = undefined;
    addcarryxU32(&x1380, &x1381, x1379, x1308, x1355);
    var x1382: u32 = undefined;
    var x1383: u1 = undefined;
    addcarryxU32(&x1382, &x1383, x1381, x1310, x1357);
    var x1384: u32 = undefined;
    var x1385: u1 = undefined;
    addcarryxU32(&x1384, &x1385, x1383, x1312, x1359);
    var x1386: u32 = undefined;
    var x1387: u1 = undefined;
    addcarryxU32(&x1386, &x1387, x1385, x1314, x1361);
    var x1388: u32 = undefined;
    var x1389: u1 = undefined;
    addcarryxU32(&x1388, &x1389, x1387, x1316, x1363);
    var x1390: u32 = undefined;
    var x1391: u1 = undefined;
    addcarryxU32(&x1390, &x1391, x1389, x1318, x1365);
    var x1392: u32 = undefined;
    var x1393: u1 = undefined;
    addcarryxU32(&x1392, &x1393, x1391, x1320, x1367);
    var x1394: u32 = undefined;
    var x1395: u1 = undefined;
    addcarryxU32(&x1394, &x1395, x1393, x1322, x1369);
    var x1396: u32 = undefined;
    var x1397: u1 = undefined;
    addcarryxU32(&x1396, &x1397, x1395, x1324, x1371);
    var x1398: u32 = undefined;
    var x1399: u1 = undefined;
    addcarryxU32(&x1398, &x1399, x1397, x1326, x1373);
    var x1400: u32 = undefined;
    var x1401: u32 = undefined;
    mulxU32(&x1400, &x1401, x1374, 0xe88fdc45);
    var x1402: u32 = undefined;
    var x1403: u32 = undefined;
    mulxU32(&x1402, &x1403, x1400, 0xffffffff);
    var x1404: u32 = undefined;
    var x1405: u32 = undefined;
    mulxU32(&x1404, &x1405, x1400, 0xffffffff);
    var x1406: u32 = undefined;
    var x1407: u32 = undefined;
    mulxU32(&x1406, &x1407, x1400, 0xffffffff);
    var x1408: u32 = undefined;
    var x1409: u32 = undefined;
    mulxU32(&x1408, &x1409, x1400, 0xffffffff);
    var x1410: u32 = undefined;
    var x1411: u32 = undefined;
    mulxU32(&x1410, &x1411, x1400, 0xffffffff);
    var x1412: u32 = undefined;
    var x1413: u32 = undefined;
    mulxU32(&x1412, &x1413, x1400, 0xffffffff);
    var x1414: u32 = undefined;
    var x1415: u32 = undefined;
    mulxU32(&x1414, &x1415, x1400, 0xc7634d81);
    var x1416: u32 = undefined;
    var x1417: u32 = undefined;
    mulxU32(&x1416, &x1417, x1400, 0xf4372ddf);
    var x1418: u32 = undefined;
    var x1419: u32 = undefined;
    mulxU32(&x1418, &x1419, x1400, 0x581a0db2);
    var x1420: u32 = undefined;
    var x1421: u32 = undefined;
    mulxU32(&x1420, &x1421, x1400, 0x48b0a77a);
    var x1422: u32 = undefined;
    var x1423: u32 = undefined;
    mulxU32(&x1422, &x1423, x1400, 0xecec196a);
    var x1424: u32 = undefined;
    var x1425: u32 = undefined;
    mulxU32(&x1424, &x1425, x1400, 0xccc52973);
    var x1426: u32 = undefined;
    var x1427: u1 = undefined;
    addcarryxU32(&x1426, &x1427, 0x0, x1425, x1422);
    var x1428: u32 = undefined;
    var x1429: u1 = undefined;
    addcarryxU32(&x1428, &x1429, x1427, x1423, x1420);
    var x1430: u32 = undefined;
    var x1431: u1 = undefined;
    addcarryxU32(&x1430, &x1431, x1429, x1421, x1418);
    var x1432: u32 = undefined;
    var x1433: u1 = undefined;
    addcarryxU32(&x1432, &x1433, x1431, x1419, x1416);
    var x1434: u32 = undefined;
    var x1435: u1 = undefined;
    addcarryxU32(&x1434, &x1435, x1433, x1417, x1414);
    var x1436: u32 = undefined;
    var x1437: u1 = undefined;
    addcarryxU32(&x1436, &x1437, x1435, x1415, x1412);
    var x1438: u32 = undefined;
    var x1439: u1 = undefined;
    addcarryxU32(&x1438, &x1439, x1437, x1413, x1410);
    var x1440: u32 = undefined;
    var x1441: u1 = undefined;
    addcarryxU32(&x1440, &x1441, x1439, x1411, x1408);
    var x1442: u32 = undefined;
    var x1443: u1 = undefined;
    addcarryxU32(&x1442, &x1443, x1441, x1409, x1406);
    var x1444: u32 = undefined;
    var x1445: u1 = undefined;
    addcarryxU32(&x1444, &x1445, x1443, x1407, x1404);
    var x1446: u32 = undefined;
    var x1447: u1 = undefined;
    addcarryxU32(&x1446, &x1447, x1445, x1405, x1402);
    const x1448 = (cast(u32, x1447) + x1403);
    var x1449: u32 = undefined;
    var x1450: u1 = undefined;
    addcarryxU32(&x1449, &x1450, 0x0, x1374, x1424);
    var x1451: u32 = undefined;
    var x1452: u1 = undefined;
    addcarryxU32(&x1451, &x1452, x1450, x1376, x1426);
    var x1453: u32 = undefined;
    var x1454: u1 = undefined;
    addcarryxU32(&x1453, &x1454, x1452, x1378, x1428);
    var x1455: u32 = undefined;
    var x1456: u1 = undefined;
    addcarryxU32(&x1455, &x1456, x1454, x1380, x1430);
    var x1457: u32 = undefined;
    var x1458: u1 = undefined;
    addcarryxU32(&x1457, &x1458, x1456, x1382, x1432);
    var x1459: u32 = undefined;
    var x1460: u1 = undefined;
    addcarryxU32(&x1459, &x1460, x1458, x1384, x1434);
    var x1461: u32 = undefined;
    var x1462: u1 = undefined;
    addcarryxU32(&x1461, &x1462, x1460, x1386, x1436);
    var x1463: u32 = undefined;
    var x1464: u1 = undefined;
    addcarryxU32(&x1463, &x1464, x1462, x1388, x1438);
    var x1465: u32 = undefined;
    var x1466: u1 = undefined;
    addcarryxU32(&x1465, &x1466, x1464, x1390, x1440);
    var x1467: u32 = undefined;
    var x1468: u1 = undefined;
    addcarryxU32(&x1467, &x1468, x1466, x1392, x1442);
    var x1469: u32 = undefined;
    var x1470: u1 = undefined;
    addcarryxU32(&x1469, &x1470, x1468, x1394, x1444);
    var x1471: u32 = undefined;
    var x1472: u1 = undefined;
    addcarryxU32(&x1471, &x1472, x1470, x1396, x1446);
    var x1473: u32 = undefined;
    var x1474: u1 = undefined;
    addcarryxU32(&x1473, &x1474, x1472, x1398, x1448);
    const x1475 = (cast(u32, x1474) + cast(u32, x1399));
    var x1476: u32 = undefined;
    var x1477: u32 = undefined;
    mulxU32(&x1476, &x1477, x10, (arg1[11]));
    var x1478: u32 = undefined;
    var x1479: u32 = undefined;
    mulxU32(&x1478, &x1479, x10, (arg1[10]));
    var x1480: u32 = undefined;
    var x1481: u32 = undefined;
    mulxU32(&x1480, &x1481, x10, (arg1[9]));
    var x1482: u32 = undefined;
    var x1483: u32 = undefined;
    mulxU32(&x1482, &x1483, x10, (arg1[8]));
    var x1484: u32 = undefined;
    var x1485: u32 = undefined;
    mulxU32(&x1484, &x1485, x10, (arg1[7]));
    var x1486: u32 = undefined;
    var x1487: u32 = undefined;
    mulxU32(&x1486, &x1487, x10, (arg1[6]));
    var x1488: u32 = undefined;
    var x1489: u32 = undefined;
    mulxU32(&x1488, &x1489, x10, (arg1[5]));
    var x1490: u32 = undefined;
    var x1491: u32 = undefined;
    mulxU32(&x1490, &x1491, x10, (arg1[4]));
    var x1492: u32 = undefined;
    var x1493: u32 = undefined;
    mulxU32(&x1492, &x1493, x10, (arg1[3]));
    var x1494: u32 = undefined;
    var x1495: u32 = undefined;
    mulxU32(&x1494, &x1495, x10, (arg1[2]));
    var x1496: u32 = undefined;
    var x1497: u32 = undefined;
    mulxU32(&x1496, &x1497, x10, (arg1[1]));
    var x1498: u32 = undefined;
    var x1499: u32 = undefined;
    mulxU32(&x1498, &x1499, x10, (arg1[0]));
    var x1500: u32 = undefined;
    var x1501: u1 = undefined;
    addcarryxU32(&x1500, &x1501, 0x0, x1499, x1496);
    var x1502: u32 = undefined;
    var x1503: u1 = undefined;
    addcarryxU32(&x1502, &x1503, x1501, x1497, x1494);
    var x1504: u32 = undefined;
    var x1505: u1 = undefined;
    addcarryxU32(&x1504, &x1505, x1503, x1495, x1492);
    var x1506: u32 = undefined;
    var x1507: u1 = undefined;
    addcarryxU32(&x1506, &x1507, x1505, x1493, x1490);
    var x1508: u32 = undefined;
    var x1509: u1 = undefined;
    addcarryxU32(&x1508, &x1509, x1507, x1491, x1488);
    var x1510: u32 = undefined;
    var x1511: u1 = undefined;
    addcarryxU32(&x1510, &x1511, x1509, x1489, x1486);
    var x1512: u32 = undefined;
    var x1513: u1 = undefined;
    addcarryxU32(&x1512, &x1513, x1511, x1487, x1484);
    var x1514: u32 = undefined;
    var x1515: u1 = undefined;
    addcarryxU32(&x1514, &x1515, x1513, x1485, x1482);
    var x1516: u32 = undefined;
    var x1517: u1 = undefined;
    addcarryxU32(&x1516, &x1517, x1515, x1483, x1480);
    var x1518: u32 = undefined;
    var x1519: u1 = undefined;
    addcarryxU32(&x1518, &x1519, x1517, x1481, x1478);
    var x1520: u32 = undefined;
    var x1521: u1 = undefined;
    addcarryxU32(&x1520, &x1521, x1519, x1479, x1476);
    const x1522 = (cast(u32, x1521) + x1477);
    var x1523: u32 = undefined;
    var x1524: u1 = undefined;
    addcarryxU32(&x1523, &x1524, 0x0, x1451, x1498);
    var x1525: u32 = undefined;
    var x1526: u1 = undefined;
    addcarryxU32(&x1525, &x1526, x1524, x1453, x1500);
    var x1527: u32 = undefined;
    var x1528: u1 = undefined;
    addcarryxU32(&x1527, &x1528, x1526, x1455, x1502);
    var x1529: u32 = undefined;
    var x1530: u1 = undefined;
    addcarryxU32(&x1529, &x1530, x1528, x1457, x1504);
    var x1531: u32 = undefined;
    var x1532: u1 = undefined;
    addcarryxU32(&x1531, &x1532, x1530, x1459, x1506);
    var x1533: u32 = undefined;
    var x1534: u1 = undefined;
    addcarryxU32(&x1533, &x1534, x1532, x1461, x1508);
    var x1535: u32 = undefined;
    var x1536: u1 = undefined;
    addcarryxU32(&x1535, &x1536, x1534, x1463, x1510);
    var x1537: u32 = undefined;
    var x1538: u1 = undefined;
    addcarryxU32(&x1537, &x1538, x1536, x1465, x1512);
    var x1539: u32 = undefined;
    var x1540: u1 = undefined;
    addcarryxU32(&x1539, &x1540, x1538, x1467, x1514);
    var x1541: u32 = undefined;
    var x1542: u1 = undefined;
    addcarryxU32(&x1541, &x1542, x1540, x1469, x1516);
    var x1543: u32 = undefined;
    var x1544: u1 = undefined;
    addcarryxU32(&x1543, &x1544, x1542, x1471, x1518);
    var x1545: u32 = undefined;
    var x1546: u1 = undefined;
    addcarryxU32(&x1545, &x1546, x1544, x1473, x1520);
    var x1547: u32 = undefined;
    var x1548: u1 = undefined;
    addcarryxU32(&x1547, &x1548, x1546, x1475, x1522);
    var x1549: u32 = undefined;
    var x1550: u32 = undefined;
    mulxU32(&x1549, &x1550, x1523, 0xe88fdc45);
    var x1551: u32 = undefined;
    var x1552: u32 = undefined;
    mulxU32(&x1551, &x1552, x1549, 0xffffffff);
    var x1553: u32 = undefined;
    var x1554: u32 = undefined;
    mulxU32(&x1553, &x1554, x1549, 0xffffffff);
    var x1555: u32 = undefined;
    var x1556: u32 = undefined;
    mulxU32(&x1555, &x1556, x1549, 0xffffffff);
    var x1557: u32 = undefined;
    var x1558: u32 = undefined;
    mulxU32(&x1557, &x1558, x1549, 0xffffffff);
    var x1559: u32 = undefined;
    var x1560: u32 = undefined;
    mulxU32(&x1559, &x1560, x1549, 0xffffffff);
    var x1561: u32 = undefined;
    var x1562: u32 = undefined;
    mulxU32(&x1561, &x1562, x1549, 0xffffffff);
    var x1563: u32 = undefined;
    var x1564: u32 = undefined;
    mulxU32(&x1563, &x1564, x1549, 0xc7634d81);
    var x1565: u32 = undefined;
    var x1566: u32 = undefined;
    mulxU32(&x1565, &x1566, x1549, 0xf4372ddf);
    var x1567: u32 = undefined;
    var x1568: u32 = undefined;
    mulxU32(&x1567, &x1568, x1549, 0x581a0db2);
    var x1569: u32 = undefined;
    var x1570: u32 = undefined;
    mulxU32(&x1569, &x1570, x1549, 0x48b0a77a);
    var x1571: u32 = undefined;
    var x1572: u32 = undefined;
    mulxU32(&x1571, &x1572, x1549, 0xecec196a);
    var x1573: u32 = undefined;
    var x1574: u32 = undefined;
    mulxU32(&x1573, &x1574, x1549, 0xccc52973);
    var x1575: u32 = undefined;
    var x1576: u1 = undefined;
    addcarryxU32(&x1575, &x1576, 0x0, x1574, x1571);
    var x1577: u32 = undefined;
    var x1578: u1 = undefined;
    addcarryxU32(&x1577, &x1578, x1576, x1572, x1569);
    var x1579: u32 = undefined;
    var x1580: u1 = undefined;
    addcarryxU32(&x1579, &x1580, x1578, x1570, x1567);
    var x1581: u32 = undefined;
    var x1582: u1 = undefined;
    addcarryxU32(&x1581, &x1582, x1580, x1568, x1565);
    var x1583: u32 = undefined;
    var x1584: u1 = undefined;
    addcarryxU32(&x1583, &x1584, x1582, x1566, x1563);
    var x1585: u32 = undefined;
    var x1586: u1 = undefined;
    addcarryxU32(&x1585, &x1586, x1584, x1564, x1561);
    var x1587: u32 = undefined;
    var x1588: u1 = undefined;
    addcarryxU32(&x1587, &x1588, x1586, x1562, x1559);
    var x1589: u32 = undefined;
    var x1590: u1 = undefined;
    addcarryxU32(&x1589, &x1590, x1588, x1560, x1557);
    var x1591: u32 = undefined;
    var x1592: u1 = undefined;
    addcarryxU32(&x1591, &x1592, x1590, x1558, x1555);
    var x1593: u32 = undefined;
    var x1594: u1 = undefined;
    addcarryxU32(&x1593, &x1594, x1592, x1556, x1553);
    var x1595: u32 = undefined;
    var x1596: u1 = undefined;
    addcarryxU32(&x1595, &x1596, x1594, x1554, x1551);
    const x1597 = (cast(u32, x1596) + x1552);
    var x1598: u32 = undefined;
    var x1599: u1 = undefined;
    addcarryxU32(&x1598, &x1599, 0x0, x1523, x1573);
    var x1600: u32 = undefined;
    var x1601: u1 = undefined;
    addcarryxU32(&x1600, &x1601, x1599, x1525, x1575);
    var x1602: u32 = undefined;
    var x1603: u1 = undefined;
    addcarryxU32(&x1602, &x1603, x1601, x1527, x1577);
    var x1604: u32 = undefined;
    var x1605: u1 = undefined;
    addcarryxU32(&x1604, &x1605, x1603, x1529, x1579);
    var x1606: u32 = undefined;
    var x1607: u1 = undefined;
    addcarryxU32(&x1606, &x1607, x1605, x1531, x1581);
    var x1608: u32 = undefined;
    var x1609: u1 = undefined;
    addcarryxU32(&x1608, &x1609, x1607, x1533, x1583);
    var x1610: u32 = undefined;
    var x1611: u1 = undefined;
    addcarryxU32(&x1610, &x1611, x1609, x1535, x1585);
    var x1612: u32 = undefined;
    var x1613: u1 = undefined;
    addcarryxU32(&x1612, &x1613, x1611, x1537, x1587);
    var x1614: u32 = undefined;
    var x1615: u1 = undefined;
    addcarryxU32(&x1614, &x1615, x1613, x1539, x1589);
    var x1616: u32 = undefined;
    var x1617: u1 = undefined;
    addcarryxU32(&x1616, &x1617, x1615, x1541, x1591);
    var x1618: u32 = undefined;
    var x1619: u1 = undefined;
    addcarryxU32(&x1618, &x1619, x1617, x1543, x1593);
    var x1620: u32 = undefined;
    var x1621: u1 = undefined;
    addcarryxU32(&x1620, &x1621, x1619, x1545, x1595);
    var x1622: u32 = undefined;
    var x1623: u1 = undefined;
    addcarryxU32(&x1622, &x1623, x1621, x1547, x1597);
    const x1624 = (cast(u32, x1623) + cast(u32, x1548));
    var x1625: u32 = undefined;
    var x1626: u32 = undefined;
    mulxU32(&x1625, &x1626, x11, (arg1[11]));
    var x1627: u32 = undefined;
    var x1628: u32 = undefined;
    mulxU32(&x1627, &x1628, x11, (arg1[10]));
    var x1629: u32 = undefined;
    var x1630: u32 = undefined;
    mulxU32(&x1629, &x1630, x11, (arg1[9]));
    var x1631: u32 = undefined;
    var x1632: u32 = undefined;
    mulxU32(&x1631, &x1632, x11, (arg1[8]));
    var x1633: u32 = undefined;
    var x1634: u32 = undefined;
    mulxU32(&x1633, &x1634, x11, (arg1[7]));
    var x1635: u32 = undefined;
    var x1636: u32 = undefined;
    mulxU32(&x1635, &x1636, x11, (arg1[6]));
    var x1637: u32 = undefined;
    var x1638: u32 = undefined;
    mulxU32(&x1637, &x1638, x11, (arg1[5]));
    var x1639: u32 = undefined;
    var x1640: u32 = undefined;
    mulxU32(&x1639, &x1640, x11, (arg1[4]));
    var x1641: u32 = undefined;
    var x1642: u32 = undefined;
    mulxU32(&x1641, &x1642, x11, (arg1[3]));
    var x1643: u32 = undefined;
    var x1644: u32 = undefined;
    mulxU32(&x1643, &x1644, x11, (arg1[2]));
    var x1645: u32 = undefined;
    var x1646: u32 = undefined;
    mulxU32(&x1645, &x1646, x11, (arg1[1]));
    var x1647: u32 = undefined;
    var x1648: u32 = undefined;
    mulxU32(&x1647, &x1648, x11, (arg1[0]));
    var x1649: u32 = undefined;
    var x1650: u1 = undefined;
    addcarryxU32(&x1649, &x1650, 0x0, x1648, x1645);
    var x1651: u32 = undefined;
    var x1652: u1 = undefined;
    addcarryxU32(&x1651, &x1652, x1650, x1646, x1643);
    var x1653: u32 = undefined;
    var x1654: u1 = undefined;
    addcarryxU32(&x1653, &x1654, x1652, x1644, x1641);
    var x1655: u32 = undefined;
    var x1656: u1 = undefined;
    addcarryxU32(&x1655, &x1656, x1654, x1642, x1639);
    var x1657: u32 = undefined;
    var x1658: u1 = undefined;
    addcarryxU32(&x1657, &x1658, x1656, x1640, x1637);
    var x1659: u32 = undefined;
    var x1660: u1 = undefined;
    addcarryxU32(&x1659, &x1660, x1658, x1638, x1635);
    var x1661: u32 = undefined;
    var x1662: u1 = undefined;
    addcarryxU32(&x1661, &x1662, x1660, x1636, x1633);
    var x1663: u32 = undefined;
    var x1664: u1 = undefined;
    addcarryxU32(&x1663, &x1664, x1662, x1634, x1631);
    var x1665: u32 = undefined;
    var x1666: u1 = undefined;
    addcarryxU32(&x1665, &x1666, x1664, x1632, x1629);
    var x1667: u32 = undefined;
    var x1668: u1 = undefined;
    addcarryxU32(&x1667, &x1668, x1666, x1630, x1627);
    var x1669: u32 = undefined;
    var x1670: u1 = undefined;
    addcarryxU32(&x1669, &x1670, x1668, x1628, x1625);
    const x1671 = (cast(u32, x1670) + x1626);
    var x1672: u32 = undefined;
    var x1673: u1 = undefined;
    addcarryxU32(&x1672, &x1673, 0x0, x1600, x1647);
    var x1674: u32 = undefined;
    var x1675: u1 = undefined;
    addcarryxU32(&x1674, &x1675, x1673, x1602, x1649);
    var x1676: u32 = undefined;
    var x1677: u1 = undefined;
    addcarryxU32(&x1676, &x1677, x1675, x1604, x1651);
    var x1678: u32 = undefined;
    var x1679: u1 = undefined;
    addcarryxU32(&x1678, &x1679, x1677, x1606, x1653);
    var x1680: u32 = undefined;
    var x1681: u1 = undefined;
    addcarryxU32(&x1680, &x1681, x1679, x1608, x1655);
    var x1682: u32 = undefined;
    var x1683: u1 = undefined;
    addcarryxU32(&x1682, &x1683, x1681, x1610, x1657);
    var x1684: u32 = undefined;
    var x1685: u1 = undefined;
    addcarryxU32(&x1684, &x1685, x1683, x1612, x1659);
    var x1686: u32 = undefined;
    var x1687: u1 = undefined;
    addcarryxU32(&x1686, &x1687, x1685, x1614, x1661);
    var x1688: u32 = undefined;
    var x1689: u1 = undefined;
    addcarryxU32(&x1688, &x1689, x1687, x1616, x1663);
    var x1690: u32 = undefined;
    var x1691: u1 = undefined;
    addcarryxU32(&x1690, &x1691, x1689, x1618, x1665);
    var x1692: u32 = undefined;
    var x1693: u1 = undefined;
    addcarryxU32(&x1692, &x1693, x1691, x1620, x1667);
    var x1694: u32 = undefined;
    var x1695: u1 = undefined;
    addcarryxU32(&x1694, &x1695, x1693, x1622, x1669);
    var x1696: u32 = undefined;
    var x1697: u1 = undefined;
    addcarryxU32(&x1696, &x1697, x1695, x1624, x1671);
    var x1698: u32 = undefined;
    var x1699: u32 = undefined;
    mulxU32(&x1698, &x1699, x1672, 0xe88fdc45);
    var x1700: u32 = undefined;
    var x1701: u32 = undefined;
    mulxU32(&x1700, &x1701, x1698, 0xffffffff);
    var x1702: u32 = undefined;
    var x1703: u32 = undefined;
    mulxU32(&x1702, &x1703, x1698, 0xffffffff);
    var x1704: u32 = undefined;
    var x1705: u32 = undefined;
    mulxU32(&x1704, &x1705, x1698, 0xffffffff);
    var x1706: u32 = undefined;
    var x1707: u32 = undefined;
    mulxU32(&x1706, &x1707, x1698, 0xffffffff);
    var x1708: u32 = undefined;
    var x1709: u32 = undefined;
    mulxU32(&x1708, &x1709, x1698, 0xffffffff);
    var x1710: u32 = undefined;
    var x1711: u32 = undefined;
    mulxU32(&x1710, &x1711, x1698, 0xffffffff);
    var x1712: u32 = undefined;
    var x1713: u32 = undefined;
    mulxU32(&x1712, &x1713, x1698, 0xc7634d81);
    var x1714: u32 = undefined;
    var x1715: u32 = undefined;
    mulxU32(&x1714, &x1715, x1698, 0xf4372ddf);
    var x1716: u32 = undefined;
    var x1717: u32 = undefined;
    mulxU32(&x1716, &x1717, x1698, 0x581a0db2);
    var x1718: u32 = undefined;
    var x1719: u32 = undefined;
    mulxU32(&x1718, &x1719, x1698, 0x48b0a77a);
    var x1720: u32 = undefined;
    var x1721: u32 = undefined;
    mulxU32(&x1720, &x1721, x1698, 0xecec196a);
    var x1722: u32 = undefined;
    var x1723: u32 = undefined;
    mulxU32(&x1722, &x1723, x1698, 0xccc52973);
    var x1724: u32 = undefined;
    var x1725: u1 = undefined;
    addcarryxU32(&x1724, &x1725, 0x0, x1723, x1720);
    var x1726: u32 = undefined;
    var x1727: u1 = undefined;
    addcarryxU32(&x1726, &x1727, x1725, x1721, x1718);
    var x1728: u32 = undefined;
    var x1729: u1 = undefined;
    addcarryxU32(&x1728, &x1729, x1727, x1719, x1716);
    var x1730: u32 = undefined;
    var x1731: u1 = undefined;
    addcarryxU32(&x1730, &x1731, x1729, x1717, x1714);
    var x1732: u32 = undefined;
    var x1733: u1 = undefined;
    addcarryxU32(&x1732, &x1733, x1731, x1715, x1712);
    var x1734: u32 = undefined;
    var x1735: u1 = undefined;
    addcarryxU32(&x1734, &x1735, x1733, x1713, x1710);
    var x1736: u32 = undefined;
    var x1737: u1 = undefined;
    addcarryxU32(&x1736, &x1737, x1735, x1711, x1708);
    var x1738: u32 = undefined;
    var x1739: u1 = undefined;
    addcarryxU32(&x1738, &x1739, x1737, x1709, x1706);
    var x1740: u32 = undefined;
    var x1741: u1 = undefined;
    addcarryxU32(&x1740, &x1741, x1739, x1707, x1704);
    var x1742: u32 = undefined;
    var x1743: u1 = undefined;
    addcarryxU32(&x1742, &x1743, x1741, x1705, x1702);
    var x1744: u32 = undefined;
    var x1745: u1 = undefined;
    addcarryxU32(&x1744, &x1745, x1743, x1703, x1700);
    const x1746 = (cast(u32, x1745) + x1701);
    var x1747: u32 = undefined;
    var x1748: u1 = undefined;
    addcarryxU32(&x1747, &x1748, 0x0, x1672, x1722);
    var x1749: u32 = undefined;
    var x1750: u1 = undefined;
    addcarryxU32(&x1749, &x1750, x1748, x1674, x1724);
    var x1751: u32 = undefined;
    var x1752: u1 = undefined;
    addcarryxU32(&x1751, &x1752, x1750, x1676, x1726);
    var x1753: u32 = undefined;
    var x1754: u1 = undefined;
    addcarryxU32(&x1753, &x1754, x1752, x1678, x1728);
    var x1755: u32 = undefined;
    var x1756: u1 = undefined;
    addcarryxU32(&x1755, &x1756, x1754, x1680, x1730);
    var x1757: u32 = undefined;
    var x1758: u1 = undefined;
    addcarryxU32(&x1757, &x1758, x1756, x1682, x1732);
    var x1759: u32 = undefined;
    var x1760: u1 = undefined;
    addcarryxU32(&x1759, &x1760, x1758, x1684, x1734);
    var x1761: u32 = undefined;
    var x1762: u1 = undefined;
    addcarryxU32(&x1761, &x1762, x1760, x1686, x1736);
    var x1763: u32 = undefined;
    var x1764: u1 = undefined;
    addcarryxU32(&x1763, &x1764, x1762, x1688, x1738);
    var x1765: u32 = undefined;
    var x1766: u1 = undefined;
    addcarryxU32(&x1765, &x1766, x1764, x1690, x1740);
    var x1767: u32 = undefined;
    var x1768: u1 = undefined;
    addcarryxU32(&x1767, &x1768, x1766, x1692, x1742);
    var x1769: u32 = undefined;
    var x1770: u1 = undefined;
    addcarryxU32(&x1769, &x1770, x1768, x1694, x1744);
    var x1771: u32 = undefined;
    var x1772: u1 = undefined;
    addcarryxU32(&x1771, &x1772, x1770, x1696, x1746);
    const x1773 = (cast(u32, x1772) + cast(u32, x1697));
    var x1774: u32 = undefined;
    var x1775: u1 = undefined;
    subborrowxU32(&x1774, &x1775, 0x0, x1749, 0xccc52973);
    var x1776: u32 = undefined;
    var x1777: u1 = undefined;
    subborrowxU32(&x1776, &x1777, x1775, x1751, 0xecec196a);
    var x1778: u32 = undefined;
    var x1779: u1 = undefined;
    subborrowxU32(&x1778, &x1779, x1777, x1753, 0x48b0a77a);
    var x1780: u32 = undefined;
    var x1781: u1 = undefined;
    subborrowxU32(&x1780, &x1781, x1779, x1755, 0x581a0db2);
    var x1782: u32 = undefined;
    var x1783: u1 = undefined;
    subborrowxU32(&x1782, &x1783, x1781, x1757, 0xf4372ddf);
    var x1784: u32 = undefined;
    var x1785: u1 = undefined;
    subborrowxU32(&x1784, &x1785, x1783, x1759, 0xc7634d81);
    var x1786: u32 = undefined;
    var x1787: u1 = undefined;
    subborrowxU32(&x1786, &x1787, x1785, x1761, 0xffffffff);
    var x1788: u32 = undefined;
    var x1789: u1 = undefined;
    subborrowxU32(&x1788, &x1789, x1787, x1763, 0xffffffff);
    var x1790: u32 = undefined;
    var x1791: u1 = undefined;
    subborrowxU32(&x1790, &x1791, x1789, x1765, 0xffffffff);
    var x1792: u32 = undefined;
    var x1793: u1 = undefined;
    subborrowxU32(&x1792, &x1793, x1791, x1767, 0xffffffff);
    var x1794: u32 = undefined;
    var x1795: u1 = undefined;
    subborrowxU32(&x1794, &x1795, x1793, x1769, 0xffffffff);
    var x1796: u32 = undefined;
    var x1797: u1 = undefined;
    subborrowxU32(&x1796, &x1797, x1795, x1771, 0xffffffff);
    var x1798: u32 = undefined;
    var x1799: u1 = undefined;
    subborrowxU32(&x1798, &x1799, x1797, x1773, cast(u32, 0x0));
    var x1800: u32 = undefined;
    cmovznzU32(&x1800, x1799, x1774, x1749);
    var x1801: u32 = undefined;
    cmovznzU32(&x1801, x1799, x1776, x1751);
    var x1802: u32 = undefined;
    cmovznzU32(&x1802, x1799, x1778, x1753);
    var x1803: u32 = undefined;
    cmovznzU32(&x1803, x1799, x1780, x1755);
    var x1804: u32 = undefined;
    cmovznzU32(&x1804, x1799, x1782, x1757);
    var x1805: u32 = undefined;
    cmovznzU32(&x1805, x1799, x1784, x1759);
    var x1806: u32 = undefined;
    cmovznzU32(&x1806, x1799, x1786, x1761);
    var x1807: u32 = undefined;
    cmovznzU32(&x1807, x1799, x1788, x1763);
    var x1808: u32 = undefined;
    cmovznzU32(&x1808, x1799, x1790, x1765);
    var x1809: u32 = undefined;
    cmovznzU32(&x1809, x1799, x1792, x1767);
    var x1810: u32 = undefined;
    cmovznzU32(&x1810, x1799, x1794, x1769);
    var x1811: u32 = undefined;
    cmovznzU32(&x1811, x1799, x1796, x1771);
    out1[0] = x1800;
    out1[1] = x1801;
    out1[2] = x1802;
    out1[3] = x1803;
    out1[4] = x1804;
    out1[5] = x1805;
    out1[6] = x1806;
    out1[7] = x1807;
    out1[8] = x1808;
    out1[9] = x1809;
    out1[10] = x1810;
    out1[11] = x1811;
}

/// The function add adds two field elements in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
///   0 ≤ eval arg2 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) + eval (from_montgomery arg2)) mod m
///   0 ≤ eval out1 < m
///
pub fn add(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement, arg2: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    addcarryxU32(&x1, &x2, 0x0, (arg1[0]), (arg2[0]));
    var x3: u32 = undefined;
    var x4: u1 = undefined;
    addcarryxU32(&x3, &x4, x2, (arg1[1]), (arg2[1]));
    var x5: u32 = undefined;
    var x6: u1 = undefined;
    addcarryxU32(&x5, &x6, x4, (arg1[2]), (arg2[2]));
    var x7: u32 = undefined;
    var x8: u1 = undefined;
    addcarryxU32(&x7, &x8, x6, (arg1[3]), (arg2[3]));
    var x9: u32 = undefined;
    var x10: u1 = undefined;
    addcarryxU32(&x9, &x10, x8, (arg1[4]), (arg2[4]));
    var x11: u32 = undefined;
    var x12: u1 = undefined;
    addcarryxU32(&x11, &x12, x10, (arg1[5]), (arg2[5]));
    var x13: u32 = undefined;
    var x14: u1 = undefined;
    addcarryxU32(&x13, &x14, x12, (arg1[6]), (arg2[6]));
    var x15: u32 = undefined;
    var x16: u1 = undefined;
    addcarryxU32(&x15, &x16, x14, (arg1[7]), (arg2[7]));
    var x17: u32 = undefined;
    var x18: u1 = undefined;
    addcarryxU32(&x17, &x18, x16, (arg1[8]), (arg2[8]));
    var x19: u32 = undefined;
    var x20: u1 = undefined;
    addcarryxU32(&x19, &x20, x18, (arg1[9]), (arg2[9]));
    var x21: u32 = undefined;
    var x22: u1 = undefined;
    addcarryxU32(&x21, &x22, x20, (arg1[10]), (arg2[10]));
    var x23: u32 = undefined;
    var x24: u1 = undefined;
    addcarryxU32(&x23, &x24, x22, (arg1[11]), (arg2[11]));
    var x25: u32 = undefined;
    var x26: u1 = undefined;
    subborrowxU32(&x25, &x26, 0x0, x1, 0xccc52973);
    var x27: u32 = undefined;
    var x28: u1 = undefined;
    subborrowxU32(&x27, &x28, x26, x3, 0xecec196a);
    var x29: u32 = undefined;
    var x30: u1 = undefined;
    subborrowxU32(&x29, &x30, x28, x5, 0x48b0a77a);
    var x31: u32 = undefined;
    var x32: u1 = undefined;
    subborrowxU32(&x31, &x32, x30, x7, 0x581a0db2);
    var x33: u32 = undefined;
    var x34: u1 = undefined;
    subborrowxU32(&x33, &x34, x32, x9, 0xf4372ddf);
    var x35: u32 = undefined;
    var x36: u1 = undefined;
    subborrowxU32(&x35, &x36, x34, x11, 0xc7634d81);
    var x37: u32 = undefined;
    var x38: u1 = undefined;
    subborrowxU32(&x37, &x38, x36, x13, 0xffffffff);
    var x39: u32 = undefined;
    var x40: u1 = undefined;
    subborrowxU32(&x39, &x40, x38, x15, 0xffffffff);
    var x41: u32 = undefined;
    var x42: u1 = undefined;
    subborrowxU32(&x41, &x42, x40, x17, 0xffffffff);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    subborrowxU32(&x43, &x44, x42, x19, 0xffffffff);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    subborrowxU32(&x45, &x46, x44, x21, 0xffffffff);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    subborrowxU32(&x47, &x48, x46, x23, 0xffffffff);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    subborrowxU32(&x49, &x50, x48, cast(u32, x24), cast(u32, 0x0));
    var x51: u32 = undefined;
    cmovznzU32(&x51, x50, x25, x1);
    var x52: u32 = undefined;
    cmovznzU32(&x52, x50, x27, x3);
    var x53: u32 = undefined;
    cmovznzU32(&x53, x50, x29, x5);
    var x54: u32 = undefined;
    cmovznzU32(&x54, x50, x31, x7);
    var x55: u32 = undefined;
    cmovznzU32(&x55, x50, x33, x9);
    var x56: u32 = undefined;
    cmovznzU32(&x56, x50, x35, x11);
    var x57: u32 = undefined;
    cmovznzU32(&x57, x50, x37, x13);
    var x58: u32 = undefined;
    cmovznzU32(&x58, x50, x39, x15);
    var x59: u32 = undefined;
    cmovznzU32(&x59, x50, x41, x17);
    var x60: u32 = undefined;
    cmovznzU32(&x60, x50, x43, x19);
    var x61: u32 = undefined;
    cmovznzU32(&x61, x50, x45, x21);
    var x62: u32 = undefined;
    cmovznzU32(&x62, x50, x47, x23);
    out1[0] = x51;
    out1[1] = x52;
    out1[2] = x53;
    out1[3] = x54;
    out1[4] = x55;
    out1[5] = x56;
    out1[6] = x57;
    out1[7] = x58;
    out1[8] = x59;
    out1[9] = x60;
    out1[10] = x61;
    out1[11] = x62;
}

/// The function sub subtracts two field elements in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
///   0 ≤ eval arg2 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) - eval (from_montgomery arg2)) mod m
///   0 ≤ eval out1 < m
///
pub fn sub(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement, arg2: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    subborrowxU32(&x1, &x2, 0x0, (arg1[0]), (arg2[0]));
    var x3: u32 = undefined;
    var x4: u1 = undefined;
    subborrowxU32(&x3, &x4, x2, (arg1[1]), (arg2[1]));
    var x5: u32 = undefined;
    var x6: u1 = undefined;
    subborrowxU32(&x5, &x6, x4, (arg1[2]), (arg2[2]));
    var x7: u32 = undefined;
    var x8: u1 = undefined;
    subborrowxU32(&x7, &x8, x6, (arg1[3]), (arg2[3]));
    var x9: u32 = undefined;
    var x10: u1 = undefined;
    subborrowxU32(&x9, &x10, x8, (arg1[4]), (arg2[4]));
    var x11: u32 = undefined;
    var x12: u1 = undefined;
    subborrowxU32(&x11, &x12, x10, (arg1[5]), (arg2[5]));
    var x13: u32 = undefined;
    var x14: u1 = undefined;
    subborrowxU32(&x13, &x14, x12, (arg1[6]), (arg2[6]));
    var x15: u32 = undefined;
    var x16: u1 = undefined;
    subborrowxU32(&x15, &x16, x14, (arg1[7]), (arg2[7]));
    var x17: u32 = undefined;
    var x18: u1 = undefined;
    subborrowxU32(&x17, &x18, x16, (arg1[8]), (arg2[8]));
    var x19: u32 = undefined;
    var x20: u1 = undefined;
    subborrowxU32(&x19, &x20, x18, (arg1[9]), (arg2[9]));
    var x21: u32 = undefined;
    var x22: u1 = undefined;
    subborrowxU32(&x21, &x22, x20, (arg1[10]), (arg2[10]));
    var x23: u32 = undefined;
    var x24: u1 = undefined;
    subborrowxU32(&x23, &x24, x22, (arg1[11]), (arg2[11]));
    var x25: u32 = undefined;
    cmovznzU32(&x25, x24, cast(u32, 0x0), 0xffffffff);
    var x26: u32 = undefined;
    var x27: u1 = undefined;
    addcarryxU32(&x26, &x27, 0x0, x1, (x25 & 0xccc52973));
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, x27, x3, (x25 & 0xecec196a));
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, x5, (x25 & 0x48b0a77a));
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, x7, (x25 & 0x581a0db2));
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, x9, (x25 & 0xf4372ddf));
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, x11, (x25 & 0xc7634d81));
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, x37, x13, x25);
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, x39, x15, x25);
    var x42: u32 = undefined;
    var x43: u1 = undefined;
    addcarryxU32(&x42, &x43, x41, x17, x25);
    var x44: u32 = undefined;
    var x45: u1 = undefined;
    addcarryxU32(&x44, &x45, x43, x19, x25);
    var x46: u32 = undefined;
    var x47: u1 = undefined;
    addcarryxU32(&x46, &x47, x45, x21, x25);
    var x48: u32 = undefined;
    var x49: u1 = undefined;
    addcarryxU32(&x48, &x49, x47, x23, x25);
    out1[0] = x26;
    out1[1] = x28;
    out1[2] = x30;
    out1[3] = x32;
    out1[4] = x34;
    out1[5] = x36;
    out1[6] = x38;
    out1[7] = x40;
    out1[8] = x42;
    out1[9] = x44;
    out1[10] = x46;
    out1[11] = x48;
}

/// The function opp negates a field element in the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = -eval (from_montgomery arg1) mod m
///   0 ≤ eval out1 < m
///
pub fn opp(out1: *MontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    subborrowxU32(&x1, &x2, 0x0, cast(u32, 0x0), (arg1[0]));
    var x3: u32 = undefined;
    var x4: u1 = undefined;
    subborrowxU32(&x3, &x4, x2, cast(u32, 0x0), (arg1[1]));
    var x5: u32 = undefined;
    var x6: u1 = undefined;
    subborrowxU32(&x5, &x6, x4, cast(u32, 0x0), (arg1[2]));
    var x7: u32 = undefined;
    var x8: u1 = undefined;
    subborrowxU32(&x7, &x8, x6, cast(u32, 0x0), (arg1[3]));
    var x9: u32 = undefined;
    var x10: u1 = undefined;
    subborrowxU32(&x9, &x10, x8, cast(u32, 0x0), (arg1[4]));
    var x11: u32 = undefined;
    var x12: u1 = undefined;
    subborrowxU32(&x11, &x12, x10, cast(u32, 0x0), (arg1[5]));
    var x13: u32 = undefined;
    var x14: u1 = undefined;
    subborrowxU32(&x13, &x14, x12, cast(u32, 0x0), (arg1[6]));
    var x15: u32 = undefined;
    var x16: u1 = undefined;
    subborrowxU32(&x15, &x16, x14, cast(u32, 0x0), (arg1[7]));
    var x17: u32 = undefined;
    var x18: u1 = undefined;
    subborrowxU32(&x17, &x18, x16, cast(u32, 0x0), (arg1[8]));
    var x19: u32 = undefined;
    var x20: u1 = undefined;
    subborrowxU32(&x19, &x20, x18, cast(u32, 0x0), (arg1[9]));
    var x21: u32 = undefined;
    var x22: u1 = undefined;
    subborrowxU32(&x21, &x22, x20, cast(u32, 0x0), (arg1[10]));
    var x23: u32 = undefined;
    var x24: u1 = undefined;
    subborrowxU32(&x23, &x24, x22, cast(u32, 0x0), (arg1[11]));
    var x25: u32 = undefined;
    cmovznzU32(&x25, x24, cast(u32, 0x0), 0xffffffff);
    var x26: u32 = undefined;
    var x27: u1 = undefined;
    addcarryxU32(&x26, &x27, 0x0, x1, (x25 & 0xccc52973));
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, x27, x3, (x25 & 0xecec196a));
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, x5, (x25 & 0x48b0a77a));
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, x7, (x25 & 0x581a0db2));
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, x9, (x25 & 0xf4372ddf));
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, x11, (x25 & 0xc7634d81));
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, x37, x13, x25);
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, x39, x15, x25);
    var x42: u32 = undefined;
    var x43: u1 = undefined;
    addcarryxU32(&x42, &x43, x41, x17, x25);
    var x44: u32 = undefined;
    var x45: u1 = undefined;
    addcarryxU32(&x44, &x45, x43, x19, x25);
    var x46: u32 = undefined;
    var x47: u1 = undefined;
    addcarryxU32(&x46, &x47, x45, x21, x25);
    var x48: u32 = undefined;
    var x49: u1 = undefined;
    addcarryxU32(&x48, &x49, x47, x23, x25);
    out1[0] = x26;
    out1[1] = x28;
    out1[2] = x30;
    out1[3] = x32;
    out1[4] = x34;
    out1[5] = x36;
    out1[6] = x38;
    out1[7] = x40;
    out1[8] = x42;
    out1[9] = x44;
    out1[10] = x46;
    out1[11] = x48;
}

/// The function fromMontgomery translates a field element out of the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval out1 mod m = (eval arg1 * ((2^32)⁻¹ mod m)^12) mod m
///   0 ≤ eval out1 < m
///
pub fn fromMontgomery(out1: *NonMontgomeryDomainFieldElement, arg1: MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[0]);
    var x2: u32 = undefined;
    var x3: u32 = undefined;
    mulxU32(&x2, &x3, x1, 0xe88fdc45);
    var x4: u32 = undefined;
    var x5: u32 = undefined;
    mulxU32(&x4, &x5, x2, 0xffffffff);
    var x6: u32 = undefined;
    var x7: u32 = undefined;
    mulxU32(&x6, &x7, x2, 0xffffffff);
    var x8: u32 = undefined;
    var x9: u32 = undefined;
    mulxU32(&x8, &x9, x2, 0xffffffff);
    var x10: u32 = undefined;
    var x11: u32 = undefined;
    mulxU32(&x10, &x11, x2, 0xffffffff);
    var x12: u32 = undefined;
    var x13: u32 = undefined;
    mulxU32(&x12, &x13, x2, 0xffffffff);
    var x14: u32 = undefined;
    var x15: u32 = undefined;
    mulxU32(&x14, &x15, x2, 0xffffffff);
    var x16: u32 = undefined;
    var x17: u32 = undefined;
    mulxU32(&x16, &x17, x2, 0xc7634d81);
    var x18: u32 = undefined;
    var x19: u32 = undefined;
    mulxU32(&x18, &x19, x2, 0xf4372ddf);
    var x20: u32 = undefined;
    var x21: u32 = undefined;
    mulxU32(&x20, &x21, x2, 0x581a0db2);
    var x22: u32 = undefined;
    var x23: u32 = undefined;
    mulxU32(&x22, &x23, x2, 0x48b0a77a);
    var x24: u32 = undefined;
    var x25: u32 = undefined;
    mulxU32(&x24, &x25, x2, 0xecec196a);
    var x26: u32 = undefined;
    var x27: u32 = undefined;
    mulxU32(&x26, &x27, x2, 0xccc52973);
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, 0x0, x27, x24);
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, x25, x22);
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, x23, x20);
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, x21, x18);
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, x19, x16);
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, x37, x17, x14);
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, x39, x15, x12);
    var x42: u32 = undefined;
    var x43: u1 = undefined;
    addcarryxU32(&x42, &x43, x41, x13, x10);
    var x44: u32 = undefined;
    var x45: u1 = undefined;
    addcarryxU32(&x44, &x45, x43, x11, x8);
    var x46: u32 = undefined;
    var x47: u1 = undefined;
    addcarryxU32(&x46, &x47, x45, x9, x6);
    var x48: u32 = undefined;
    var x49: u1 = undefined;
    addcarryxU32(&x48, &x49, x47, x7, x4);
    var x50: u32 = undefined;
    var x51: u1 = undefined;
    addcarryxU32(&x50, &x51, 0x0, x1, x26);
    var x52: u32 = undefined;
    var x53: u1 = undefined;
    addcarryxU32(&x52, &x53, x51, cast(u32, 0x0), x28);
    var x54: u32 = undefined;
    var x55: u1 = undefined;
    addcarryxU32(&x54, &x55, x53, cast(u32, 0x0), x30);
    var x56: u32 = undefined;
    var x57: u1 = undefined;
    addcarryxU32(&x56, &x57, x55, cast(u32, 0x0), x32);
    var x58: u32 = undefined;
    var x59: u1 = undefined;
    addcarryxU32(&x58, &x59, x57, cast(u32, 0x0), x34);
    var x60: u32 = undefined;
    var x61: u1 = undefined;
    addcarryxU32(&x60, &x61, x59, cast(u32, 0x0), x36);
    var x62: u32 = undefined;
    var x63: u1 = undefined;
    addcarryxU32(&x62, &x63, x61, cast(u32, 0x0), x38);
    var x64: u32 = undefined;
    var x65: u1 = undefined;
    addcarryxU32(&x64, &x65, x63, cast(u32, 0x0), x40);
    var x66: u32 = undefined;
    var x67: u1 = undefined;
    addcarryxU32(&x66, &x67, x65, cast(u32, 0x0), x42);
    var x68: u32 = undefined;
    var x69: u1 = undefined;
    addcarryxU32(&x68, &x69, x67, cast(u32, 0x0), x44);
    var x70: u32 = undefined;
    var x71: u1 = undefined;
    addcarryxU32(&x70, &x71, x69, cast(u32, 0x0), x46);
    var x72: u32 = undefined;
    var x73: u1 = undefined;
    addcarryxU32(&x72, &x73, x71, cast(u32, 0x0), x48);
    var x74: u32 = undefined;
    var x75: u1 = undefined;
    addcarryxU32(&x74, &x75, x73, cast(u32, 0x0), (cast(u32, x49) + x5));
    var x76: u32 = undefined;
    var x77: u1 = undefined;
    addcarryxU32(&x76, &x77, 0x0, x52, (arg1[1]));
    var x78: u32 = undefined;
    var x79: u1 = undefined;
    addcarryxU32(&x78, &x79, x77, x54, cast(u32, 0x0));
    var x80: u32 = undefined;
    var x81: u1 = undefined;
    addcarryxU32(&x80, &x81, x79, x56, cast(u32, 0x0));
    var x82: u32 = undefined;
    var x83: u1 = undefined;
    addcarryxU32(&x82, &x83, x81, x58, cast(u32, 0x0));
    var x84: u32 = undefined;
    var x85: u1 = undefined;
    addcarryxU32(&x84, &x85, x83, x60, cast(u32, 0x0));
    var x86: u32 = undefined;
    var x87: u1 = undefined;
    addcarryxU32(&x86, &x87, x85, x62, cast(u32, 0x0));
    var x88: u32 = undefined;
    var x89: u1 = undefined;
    addcarryxU32(&x88, &x89, x87, x64, cast(u32, 0x0));
    var x90: u32 = undefined;
    var x91: u1 = undefined;
    addcarryxU32(&x90, &x91, x89, x66, cast(u32, 0x0));
    var x92: u32 = undefined;
    var x93: u1 = undefined;
    addcarryxU32(&x92, &x93, x91, x68, cast(u32, 0x0));
    var x94: u32 = undefined;
    var x95: u1 = undefined;
    addcarryxU32(&x94, &x95, x93, x70, cast(u32, 0x0));
    var x96: u32 = undefined;
    var x97: u1 = undefined;
    addcarryxU32(&x96, &x97, x95, x72, cast(u32, 0x0));
    var x98: u32 = undefined;
    var x99: u1 = undefined;
    addcarryxU32(&x98, &x99, x97, x74, cast(u32, 0x0));
    var x100: u32 = undefined;
    var x101: u32 = undefined;
    mulxU32(&x100, &x101, x76, 0xe88fdc45);
    var x102: u32 = undefined;
    var x103: u32 = undefined;
    mulxU32(&x102, &x103, x100, 0xffffffff);
    var x104: u32 = undefined;
    var x105: u32 = undefined;
    mulxU32(&x104, &x105, x100, 0xffffffff);
    var x106: u32 = undefined;
    var x107: u32 = undefined;
    mulxU32(&x106, &x107, x100, 0xffffffff);
    var x108: u32 = undefined;
    var x109: u32 = undefined;
    mulxU32(&x108, &x109, x100, 0xffffffff);
    var x110: u32 = undefined;
    var x111: u32 = undefined;
    mulxU32(&x110, &x111, x100, 0xffffffff);
    var x112: u32 = undefined;
    var x113: u32 = undefined;
    mulxU32(&x112, &x113, x100, 0xffffffff);
    var x114: u32 = undefined;
    var x115: u32 = undefined;
    mulxU32(&x114, &x115, x100, 0xc7634d81);
    var x116: u32 = undefined;
    var x117: u32 = undefined;
    mulxU32(&x116, &x117, x100, 0xf4372ddf);
    var x118: u32 = undefined;
    var x119: u32 = undefined;
    mulxU32(&x118, &x119, x100, 0x581a0db2);
    var x120: u32 = undefined;
    var x121: u32 = undefined;
    mulxU32(&x120, &x121, x100, 0x48b0a77a);
    var x122: u32 = undefined;
    var x123: u32 = undefined;
    mulxU32(&x122, &x123, x100, 0xecec196a);
    var x124: u32 = undefined;
    var x125: u32 = undefined;
    mulxU32(&x124, &x125, x100, 0xccc52973);
    var x126: u32 = undefined;
    var x127: u1 = undefined;
    addcarryxU32(&x126, &x127, 0x0, x125, x122);
    var x128: u32 = undefined;
    var x129: u1 = undefined;
    addcarryxU32(&x128, &x129, x127, x123, x120);
    var x130: u32 = undefined;
    var x131: u1 = undefined;
    addcarryxU32(&x130, &x131, x129, x121, x118);
    var x132: u32 = undefined;
    var x133: u1 = undefined;
    addcarryxU32(&x132, &x133, x131, x119, x116);
    var x134: u32 = undefined;
    var x135: u1 = undefined;
    addcarryxU32(&x134, &x135, x133, x117, x114);
    var x136: u32 = undefined;
    var x137: u1 = undefined;
    addcarryxU32(&x136, &x137, x135, x115, x112);
    var x138: u32 = undefined;
    var x139: u1 = undefined;
    addcarryxU32(&x138, &x139, x137, x113, x110);
    var x140: u32 = undefined;
    var x141: u1 = undefined;
    addcarryxU32(&x140, &x141, x139, x111, x108);
    var x142: u32 = undefined;
    var x143: u1 = undefined;
    addcarryxU32(&x142, &x143, x141, x109, x106);
    var x144: u32 = undefined;
    var x145: u1 = undefined;
    addcarryxU32(&x144, &x145, x143, x107, x104);
    var x146: u32 = undefined;
    var x147: u1 = undefined;
    addcarryxU32(&x146, &x147, x145, x105, x102);
    var x148: u32 = undefined;
    var x149: u1 = undefined;
    addcarryxU32(&x148, &x149, 0x0, x76, x124);
    var x150: u32 = undefined;
    var x151: u1 = undefined;
    addcarryxU32(&x150, &x151, x149, x78, x126);
    var x152: u32 = undefined;
    var x153: u1 = undefined;
    addcarryxU32(&x152, &x153, x151, x80, x128);
    var x154: u32 = undefined;
    var x155: u1 = undefined;
    addcarryxU32(&x154, &x155, x153, x82, x130);
    var x156: u32 = undefined;
    var x157: u1 = undefined;
    addcarryxU32(&x156, &x157, x155, x84, x132);
    var x158: u32 = undefined;
    var x159: u1 = undefined;
    addcarryxU32(&x158, &x159, x157, x86, x134);
    var x160: u32 = undefined;
    var x161: u1 = undefined;
    addcarryxU32(&x160, &x161, x159, x88, x136);
    var x162: u32 = undefined;
    var x163: u1 = undefined;
    addcarryxU32(&x162, &x163, x161, x90, x138);
    var x164: u32 = undefined;
    var x165: u1 = undefined;
    addcarryxU32(&x164, &x165, x163, x92, x140);
    var x166: u32 = undefined;
    var x167: u1 = undefined;
    addcarryxU32(&x166, &x167, x165, x94, x142);
    var x168: u32 = undefined;
    var x169: u1 = undefined;
    addcarryxU32(&x168, &x169, x167, x96, x144);
    var x170: u32 = undefined;
    var x171: u1 = undefined;
    addcarryxU32(&x170, &x171, x169, x98, x146);
    var x172: u32 = undefined;
    var x173: u1 = undefined;
    addcarryxU32(&x172, &x173, x171, (cast(u32, x99) + cast(u32, x75)), (cast(u32, x147) + x103));
    var x174: u32 = undefined;
    var x175: u1 = undefined;
    addcarryxU32(&x174, &x175, 0x0, x150, (arg1[2]));
    var x176: u32 = undefined;
    var x177: u1 = undefined;
    addcarryxU32(&x176, &x177, x175, x152, cast(u32, 0x0));
    var x178: u32 = undefined;
    var x179: u1 = undefined;
    addcarryxU32(&x178, &x179, x177, x154, cast(u32, 0x0));
    var x180: u32 = undefined;
    var x181: u1 = undefined;
    addcarryxU32(&x180, &x181, x179, x156, cast(u32, 0x0));
    var x182: u32 = undefined;
    var x183: u1 = undefined;
    addcarryxU32(&x182, &x183, x181, x158, cast(u32, 0x0));
    var x184: u32 = undefined;
    var x185: u1 = undefined;
    addcarryxU32(&x184, &x185, x183, x160, cast(u32, 0x0));
    var x186: u32 = undefined;
    var x187: u1 = undefined;
    addcarryxU32(&x186, &x187, x185, x162, cast(u32, 0x0));
    var x188: u32 = undefined;
    var x189: u1 = undefined;
    addcarryxU32(&x188, &x189, x187, x164, cast(u32, 0x0));
    var x190: u32 = undefined;
    var x191: u1 = undefined;
    addcarryxU32(&x190, &x191, x189, x166, cast(u32, 0x0));
    var x192: u32 = undefined;
    var x193: u1 = undefined;
    addcarryxU32(&x192, &x193, x191, x168, cast(u32, 0x0));
    var x194: u32 = undefined;
    var x195: u1 = undefined;
    addcarryxU32(&x194, &x195, x193, x170, cast(u32, 0x0));
    var x196: u32 = undefined;
    var x197: u1 = undefined;
    addcarryxU32(&x196, &x197, x195, x172, cast(u32, 0x0));
    var x198: u32 = undefined;
    var x199: u32 = undefined;
    mulxU32(&x198, &x199, x174, 0xe88fdc45);
    var x200: u32 = undefined;
    var x201: u32 = undefined;
    mulxU32(&x200, &x201, x198, 0xffffffff);
    var x202: u32 = undefined;
    var x203: u32 = undefined;
    mulxU32(&x202, &x203, x198, 0xffffffff);
    var x204: u32 = undefined;
    var x205: u32 = undefined;
    mulxU32(&x204, &x205, x198, 0xffffffff);
    var x206: u32 = undefined;
    var x207: u32 = undefined;
    mulxU32(&x206, &x207, x198, 0xffffffff);
    var x208: u32 = undefined;
    var x209: u32 = undefined;
    mulxU32(&x208, &x209, x198, 0xffffffff);
    var x210: u32 = undefined;
    var x211: u32 = undefined;
    mulxU32(&x210, &x211, x198, 0xffffffff);
    var x212: u32 = undefined;
    var x213: u32 = undefined;
    mulxU32(&x212, &x213, x198, 0xc7634d81);
    var x214: u32 = undefined;
    var x215: u32 = undefined;
    mulxU32(&x214, &x215, x198, 0xf4372ddf);
    var x216: u32 = undefined;
    var x217: u32 = undefined;
    mulxU32(&x216, &x217, x198, 0x581a0db2);
    var x218: u32 = undefined;
    var x219: u32 = undefined;
    mulxU32(&x218, &x219, x198, 0x48b0a77a);
    var x220: u32 = undefined;
    var x221: u32 = undefined;
    mulxU32(&x220, &x221, x198, 0xecec196a);
    var x222: u32 = undefined;
    var x223: u32 = undefined;
    mulxU32(&x222, &x223, x198, 0xccc52973);
    var x224: u32 = undefined;
    var x225: u1 = undefined;
    addcarryxU32(&x224, &x225, 0x0, x223, x220);
    var x226: u32 = undefined;
    var x227: u1 = undefined;
    addcarryxU32(&x226, &x227, x225, x221, x218);
    var x228: u32 = undefined;
    var x229: u1 = undefined;
    addcarryxU32(&x228, &x229, x227, x219, x216);
    var x230: u32 = undefined;
    var x231: u1 = undefined;
    addcarryxU32(&x230, &x231, x229, x217, x214);
    var x232: u32 = undefined;
    var x233: u1 = undefined;
    addcarryxU32(&x232, &x233, x231, x215, x212);
    var x234: u32 = undefined;
    var x235: u1 = undefined;
    addcarryxU32(&x234, &x235, x233, x213, x210);
    var x236: u32 = undefined;
    var x237: u1 = undefined;
    addcarryxU32(&x236, &x237, x235, x211, x208);
    var x238: u32 = undefined;
    var x239: u1 = undefined;
    addcarryxU32(&x238, &x239, x237, x209, x206);
    var x240: u32 = undefined;
    var x241: u1 = undefined;
    addcarryxU32(&x240, &x241, x239, x207, x204);
    var x242: u32 = undefined;
    var x243: u1 = undefined;
    addcarryxU32(&x242, &x243, x241, x205, x202);
    var x244: u32 = undefined;
    var x245: u1 = undefined;
    addcarryxU32(&x244, &x245, x243, x203, x200);
    var x246: u32 = undefined;
    var x247: u1 = undefined;
    addcarryxU32(&x246, &x247, 0x0, x174, x222);
    var x248: u32 = undefined;
    var x249: u1 = undefined;
    addcarryxU32(&x248, &x249, x247, x176, x224);
    var x250: u32 = undefined;
    var x251: u1 = undefined;
    addcarryxU32(&x250, &x251, x249, x178, x226);
    var x252: u32 = undefined;
    var x253: u1 = undefined;
    addcarryxU32(&x252, &x253, x251, x180, x228);
    var x254: u32 = undefined;
    var x255: u1 = undefined;
    addcarryxU32(&x254, &x255, x253, x182, x230);
    var x256: u32 = undefined;
    var x257: u1 = undefined;
    addcarryxU32(&x256, &x257, x255, x184, x232);
    var x258: u32 = undefined;
    var x259: u1 = undefined;
    addcarryxU32(&x258, &x259, x257, x186, x234);
    var x260: u32 = undefined;
    var x261: u1 = undefined;
    addcarryxU32(&x260, &x261, x259, x188, x236);
    var x262: u32 = undefined;
    var x263: u1 = undefined;
    addcarryxU32(&x262, &x263, x261, x190, x238);
    var x264: u32 = undefined;
    var x265: u1 = undefined;
    addcarryxU32(&x264, &x265, x263, x192, x240);
    var x266: u32 = undefined;
    var x267: u1 = undefined;
    addcarryxU32(&x266, &x267, x265, x194, x242);
    var x268: u32 = undefined;
    var x269: u1 = undefined;
    addcarryxU32(&x268, &x269, x267, x196, x244);
    var x270: u32 = undefined;
    var x271: u1 = undefined;
    addcarryxU32(&x270, &x271, x269, (cast(u32, x197) + cast(u32, x173)), (cast(u32, x245) + x201));
    var x272: u32 = undefined;
    var x273: u1 = undefined;
    addcarryxU32(&x272, &x273, 0x0, x248, (arg1[3]));
    var x274: u32 = undefined;
    var x275: u1 = undefined;
    addcarryxU32(&x274, &x275, x273, x250, cast(u32, 0x0));
    var x276: u32 = undefined;
    var x277: u1 = undefined;
    addcarryxU32(&x276, &x277, x275, x252, cast(u32, 0x0));
    var x278: u32 = undefined;
    var x279: u1 = undefined;
    addcarryxU32(&x278, &x279, x277, x254, cast(u32, 0x0));
    var x280: u32 = undefined;
    var x281: u1 = undefined;
    addcarryxU32(&x280, &x281, x279, x256, cast(u32, 0x0));
    var x282: u32 = undefined;
    var x283: u1 = undefined;
    addcarryxU32(&x282, &x283, x281, x258, cast(u32, 0x0));
    var x284: u32 = undefined;
    var x285: u1 = undefined;
    addcarryxU32(&x284, &x285, x283, x260, cast(u32, 0x0));
    var x286: u32 = undefined;
    var x287: u1 = undefined;
    addcarryxU32(&x286, &x287, x285, x262, cast(u32, 0x0));
    var x288: u32 = undefined;
    var x289: u1 = undefined;
    addcarryxU32(&x288, &x289, x287, x264, cast(u32, 0x0));
    var x290: u32 = undefined;
    var x291: u1 = undefined;
    addcarryxU32(&x290, &x291, x289, x266, cast(u32, 0x0));
    var x292: u32 = undefined;
    var x293: u1 = undefined;
    addcarryxU32(&x292, &x293, x291, x268, cast(u32, 0x0));
    var x294: u32 = undefined;
    var x295: u1 = undefined;
    addcarryxU32(&x294, &x295, x293, x270, cast(u32, 0x0));
    var x296: u32 = undefined;
    var x297: u32 = undefined;
    mulxU32(&x296, &x297, x272, 0xe88fdc45);
    var x298: u32 = undefined;
    var x299: u32 = undefined;
    mulxU32(&x298, &x299, x296, 0xffffffff);
    var x300: u32 = undefined;
    var x301: u32 = undefined;
    mulxU32(&x300, &x301, x296, 0xffffffff);
    var x302: u32 = undefined;
    var x303: u32 = undefined;
    mulxU32(&x302, &x303, x296, 0xffffffff);
    var x304: u32 = undefined;
    var x305: u32 = undefined;
    mulxU32(&x304, &x305, x296, 0xffffffff);
    var x306: u32 = undefined;
    var x307: u32 = undefined;
    mulxU32(&x306, &x307, x296, 0xffffffff);
    var x308: u32 = undefined;
    var x309: u32 = undefined;
    mulxU32(&x308, &x309, x296, 0xffffffff);
    var x310: u32 = undefined;
    var x311: u32 = undefined;
    mulxU32(&x310, &x311, x296, 0xc7634d81);
    var x312: u32 = undefined;
    var x313: u32 = undefined;
    mulxU32(&x312, &x313, x296, 0xf4372ddf);
    var x314: u32 = undefined;
    var x315: u32 = undefined;
    mulxU32(&x314, &x315, x296, 0x581a0db2);
    var x316: u32 = undefined;
    var x317: u32 = undefined;
    mulxU32(&x316, &x317, x296, 0x48b0a77a);
    var x318: u32 = undefined;
    var x319: u32 = undefined;
    mulxU32(&x318, &x319, x296, 0xecec196a);
    var x320: u32 = undefined;
    var x321: u32 = undefined;
    mulxU32(&x320, &x321, x296, 0xccc52973);
    var x322: u32 = undefined;
    var x323: u1 = undefined;
    addcarryxU32(&x322, &x323, 0x0, x321, x318);
    var x324: u32 = undefined;
    var x325: u1 = undefined;
    addcarryxU32(&x324, &x325, x323, x319, x316);
    var x326: u32 = undefined;
    var x327: u1 = undefined;
    addcarryxU32(&x326, &x327, x325, x317, x314);
    var x328: u32 = undefined;
    var x329: u1 = undefined;
    addcarryxU32(&x328, &x329, x327, x315, x312);
    var x330: u32 = undefined;
    var x331: u1 = undefined;
    addcarryxU32(&x330, &x331, x329, x313, x310);
    var x332: u32 = undefined;
    var x333: u1 = undefined;
    addcarryxU32(&x332, &x333, x331, x311, x308);
    var x334: u32 = undefined;
    var x335: u1 = undefined;
    addcarryxU32(&x334, &x335, x333, x309, x306);
    var x336: u32 = undefined;
    var x337: u1 = undefined;
    addcarryxU32(&x336, &x337, x335, x307, x304);
    var x338: u32 = undefined;
    var x339: u1 = undefined;
    addcarryxU32(&x338, &x339, x337, x305, x302);
    var x340: u32 = undefined;
    var x341: u1 = undefined;
    addcarryxU32(&x340, &x341, x339, x303, x300);
    var x342: u32 = undefined;
    var x343: u1 = undefined;
    addcarryxU32(&x342, &x343, x341, x301, x298);
    var x344: u32 = undefined;
    var x345: u1 = undefined;
    addcarryxU32(&x344, &x345, 0x0, x272, x320);
    var x346: u32 = undefined;
    var x347: u1 = undefined;
    addcarryxU32(&x346, &x347, x345, x274, x322);
    var x348: u32 = undefined;
    var x349: u1 = undefined;
    addcarryxU32(&x348, &x349, x347, x276, x324);
    var x350: u32 = undefined;
    var x351: u1 = undefined;
    addcarryxU32(&x350, &x351, x349, x278, x326);
    var x352: u32 = undefined;
    var x353: u1 = undefined;
    addcarryxU32(&x352, &x353, x351, x280, x328);
    var x354: u32 = undefined;
    var x355: u1 = undefined;
    addcarryxU32(&x354, &x355, x353, x282, x330);
    var x356: u32 = undefined;
    var x357: u1 = undefined;
    addcarryxU32(&x356, &x357, x355, x284, x332);
    var x358: u32 = undefined;
    var x359: u1 = undefined;
    addcarryxU32(&x358, &x359, x357, x286, x334);
    var x360: u32 = undefined;
    var x361: u1 = undefined;
    addcarryxU32(&x360, &x361, x359, x288, x336);
    var x362: u32 = undefined;
    var x363: u1 = undefined;
    addcarryxU32(&x362, &x363, x361, x290, x338);
    var x364: u32 = undefined;
    var x365: u1 = undefined;
    addcarryxU32(&x364, &x365, x363, x292, x340);
    var x366: u32 = undefined;
    var x367: u1 = undefined;
    addcarryxU32(&x366, &x367, x365, x294, x342);
    var x368: u32 = undefined;
    var x369: u1 = undefined;
    addcarryxU32(&x368, &x369, x367, (cast(u32, x295) + cast(u32, x271)), (cast(u32, x343) + x299));
    var x370: u32 = undefined;
    var x371: u1 = undefined;
    addcarryxU32(&x370, &x371, 0x0, x346, (arg1[4]));
    var x372: u32 = undefined;
    var x373: u1 = undefined;
    addcarryxU32(&x372, &x373, x371, x348, cast(u32, 0x0));
    var x374: u32 = undefined;
    var x375: u1 = undefined;
    addcarryxU32(&x374, &x375, x373, x350, cast(u32, 0x0));
    var x376: u32 = undefined;
    var x377: u1 = undefined;
    addcarryxU32(&x376, &x377, x375, x352, cast(u32, 0x0));
    var x378: u32 = undefined;
    var x379: u1 = undefined;
    addcarryxU32(&x378, &x379, x377, x354, cast(u32, 0x0));
    var x380: u32 = undefined;
    var x381: u1 = undefined;
    addcarryxU32(&x380, &x381, x379, x356, cast(u32, 0x0));
    var x382: u32 = undefined;
    var x383: u1 = undefined;
    addcarryxU32(&x382, &x383, x381, x358, cast(u32, 0x0));
    var x384: u32 = undefined;
    var x385: u1 = undefined;
    addcarryxU32(&x384, &x385, x383, x360, cast(u32, 0x0));
    var x386: u32 = undefined;
    var x387: u1 = undefined;
    addcarryxU32(&x386, &x387, x385, x362, cast(u32, 0x0));
    var x388: u32 = undefined;
    var x389: u1 = undefined;
    addcarryxU32(&x388, &x389, x387, x364, cast(u32, 0x0));
    var x390: u32 = undefined;
    var x391: u1 = undefined;
    addcarryxU32(&x390, &x391, x389, x366, cast(u32, 0x0));
    var x392: u32 = undefined;
    var x393: u1 = undefined;
    addcarryxU32(&x392, &x393, x391, x368, cast(u32, 0x0));
    var x394: u32 = undefined;
    var x395: u32 = undefined;
    mulxU32(&x394, &x395, x370, 0xe88fdc45);
    var x396: u32 = undefined;
    var x397: u32 = undefined;
    mulxU32(&x396, &x397, x394, 0xffffffff);
    var x398: u32 = undefined;
    var x399: u32 = undefined;
    mulxU32(&x398, &x399, x394, 0xffffffff);
    var x400: u32 = undefined;
    var x401: u32 = undefined;
    mulxU32(&x400, &x401, x394, 0xffffffff);
    var x402: u32 = undefined;
    var x403: u32 = undefined;
    mulxU32(&x402, &x403, x394, 0xffffffff);
    var x404: u32 = undefined;
    var x405: u32 = undefined;
    mulxU32(&x404, &x405, x394, 0xffffffff);
    var x406: u32 = undefined;
    var x407: u32 = undefined;
    mulxU32(&x406, &x407, x394, 0xffffffff);
    var x408: u32 = undefined;
    var x409: u32 = undefined;
    mulxU32(&x408, &x409, x394, 0xc7634d81);
    var x410: u32 = undefined;
    var x411: u32 = undefined;
    mulxU32(&x410, &x411, x394, 0xf4372ddf);
    var x412: u32 = undefined;
    var x413: u32 = undefined;
    mulxU32(&x412, &x413, x394, 0x581a0db2);
    var x414: u32 = undefined;
    var x415: u32 = undefined;
    mulxU32(&x414, &x415, x394, 0x48b0a77a);
    var x416: u32 = undefined;
    var x417: u32 = undefined;
    mulxU32(&x416, &x417, x394, 0xecec196a);
    var x418: u32 = undefined;
    var x419: u32 = undefined;
    mulxU32(&x418, &x419, x394, 0xccc52973);
    var x420: u32 = undefined;
    var x421: u1 = undefined;
    addcarryxU32(&x420, &x421, 0x0, x419, x416);
    var x422: u32 = undefined;
    var x423: u1 = undefined;
    addcarryxU32(&x422, &x423, x421, x417, x414);
    var x424: u32 = undefined;
    var x425: u1 = undefined;
    addcarryxU32(&x424, &x425, x423, x415, x412);
    var x426: u32 = undefined;
    var x427: u1 = undefined;
    addcarryxU32(&x426, &x427, x425, x413, x410);
    var x428: u32 = undefined;
    var x429: u1 = undefined;
    addcarryxU32(&x428, &x429, x427, x411, x408);
    var x430: u32 = undefined;
    var x431: u1 = undefined;
    addcarryxU32(&x430, &x431, x429, x409, x406);
    var x432: u32 = undefined;
    var x433: u1 = undefined;
    addcarryxU32(&x432, &x433, x431, x407, x404);
    var x434: u32 = undefined;
    var x435: u1 = undefined;
    addcarryxU32(&x434, &x435, x433, x405, x402);
    var x436: u32 = undefined;
    var x437: u1 = undefined;
    addcarryxU32(&x436, &x437, x435, x403, x400);
    var x438: u32 = undefined;
    var x439: u1 = undefined;
    addcarryxU32(&x438, &x439, x437, x401, x398);
    var x440: u32 = undefined;
    var x441: u1 = undefined;
    addcarryxU32(&x440, &x441, x439, x399, x396);
    var x442: u32 = undefined;
    var x443: u1 = undefined;
    addcarryxU32(&x442, &x443, 0x0, x370, x418);
    var x444: u32 = undefined;
    var x445: u1 = undefined;
    addcarryxU32(&x444, &x445, x443, x372, x420);
    var x446: u32 = undefined;
    var x447: u1 = undefined;
    addcarryxU32(&x446, &x447, x445, x374, x422);
    var x448: u32 = undefined;
    var x449: u1 = undefined;
    addcarryxU32(&x448, &x449, x447, x376, x424);
    var x450: u32 = undefined;
    var x451: u1 = undefined;
    addcarryxU32(&x450, &x451, x449, x378, x426);
    var x452: u32 = undefined;
    var x453: u1 = undefined;
    addcarryxU32(&x452, &x453, x451, x380, x428);
    var x454: u32 = undefined;
    var x455: u1 = undefined;
    addcarryxU32(&x454, &x455, x453, x382, x430);
    var x456: u32 = undefined;
    var x457: u1 = undefined;
    addcarryxU32(&x456, &x457, x455, x384, x432);
    var x458: u32 = undefined;
    var x459: u1 = undefined;
    addcarryxU32(&x458, &x459, x457, x386, x434);
    var x460: u32 = undefined;
    var x461: u1 = undefined;
    addcarryxU32(&x460, &x461, x459, x388, x436);
    var x462: u32 = undefined;
    var x463: u1 = undefined;
    addcarryxU32(&x462, &x463, x461, x390, x438);
    var x464: u32 = undefined;
    var x465: u1 = undefined;
    addcarryxU32(&x464, &x465, x463, x392, x440);
    var x466: u32 = undefined;
    var x467: u1 = undefined;
    addcarryxU32(&x466, &x467, x465, (cast(u32, x393) + cast(u32, x369)), (cast(u32, x441) + x397));
    var x468: u32 = undefined;
    var x469: u1 = undefined;
    addcarryxU32(&x468, &x469, 0x0, x444, (arg1[5]));
    var x470: u32 = undefined;
    var x471: u1 = undefined;
    addcarryxU32(&x470, &x471, x469, x446, cast(u32, 0x0));
    var x472: u32 = undefined;
    var x473: u1 = undefined;
    addcarryxU32(&x472, &x473, x471, x448, cast(u32, 0x0));
    var x474: u32 = undefined;
    var x475: u1 = undefined;
    addcarryxU32(&x474, &x475, x473, x450, cast(u32, 0x0));
    var x476: u32 = undefined;
    var x477: u1 = undefined;
    addcarryxU32(&x476, &x477, x475, x452, cast(u32, 0x0));
    var x478: u32 = undefined;
    var x479: u1 = undefined;
    addcarryxU32(&x478, &x479, x477, x454, cast(u32, 0x0));
    var x480: u32 = undefined;
    var x481: u1 = undefined;
    addcarryxU32(&x480, &x481, x479, x456, cast(u32, 0x0));
    var x482: u32 = undefined;
    var x483: u1 = undefined;
    addcarryxU32(&x482, &x483, x481, x458, cast(u32, 0x0));
    var x484: u32 = undefined;
    var x485: u1 = undefined;
    addcarryxU32(&x484, &x485, x483, x460, cast(u32, 0x0));
    var x486: u32 = undefined;
    var x487: u1 = undefined;
    addcarryxU32(&x486, &x487, x485, x462, cast(u32, 0x0));
    var x488: u32 = undefined;
    var x489: u1 = undefined;
    addcarryxU32(&x488, &x489, x487, x464, cast(u32, 0x0));
    var x490: u32 = undefined;
    var x491: u1 = undefined;
    addcarryxU32(&x490, &x491, x489, x466, cast(u32, 0x0));
    var x492: u32 = undefined;
    var x493: u32 = undefined;
    mulxU32(&x492, &x493, x468, 0xe88fdc45);
    var x494: u32 = undefined;
    var x495: u32 = undefined;
    mulxU32(&x494, &x495, x492, 0xffffffff);
    var x496: u32 = undefined;
    var x497: u32 = undefined;
    mulxU32(&x496, &x497, x492, 0xffffffff);
    var x498: u32 = undefined;
    var x499: u32 = undefined;
    mulxU32(&x498, &x499, x492, 0xffffffff);
    var x500: u32 = undefined;
    var x501: u32 = undefined;
    mulxU32(&x500, &x501, x492, 0xffffffff);
    var x502: u32 = undefined;
    var x503: u32 = undefined;
    mulxU32(&x502, &x503, x492, 0xffffffff);
    var x504: u32 = undefined;
    var x505: u32 = undefined;
    mulxU32(&x504, &x505, x492, 0xffffffff);
    var x506: u32 = undefined;
    var x507: u32 = undefined;
    mulxU32(&x506, &x507, x492, 0xc7634d81);
    var x508: u32 = undefined;
    var x509: u32 = undefined;
    mulxU32(&x508, &x509, x492, 0xf4372ddf);
    var x510: u32 = undefined;
    var x511: u32 = undefined;
    mulxU32(&x510, &x511, x492, 0x581a0db2);
    var x512: u32 = undefined;
    var x513: u32 = undefined;
    mulxU32(&x512, &x513, x492, 0x48b0a77a);
    var x514: u32 = undefined;
    var x515: u32 = undefined;
    mulxU32(&x514, &x515, x492, 0xecec196a);
    var x516: u32 = undefined;
    var x517: u32 = undefined;
    mulxU32(&x516, &x517, x492, 0xccc52973);
    var x518: u32 = undefined;
    var x519: u1 = undefined;
    addcarryxU32(&x518, &x519, 0x0, x517, x514);
    var x520: u32 = undefined;
    var x521: u1 = undefined;
    addcarryxU32(&x520, &x521, x519, x515, x512);
    var x522: u32 = undefined;
    var x523: u1 = undefined;
    addcarryxU32(&x522, &x523, x521, x513, x510);
    var x524: u32 = undefined;
    var x525: u1 = undefined;
    addcarryxU32(&x524, &x525, x523, x511, x508);
    var x526: u32 = undefined;
    var x527: u1 = undefined;
    addcarryxU32(&x526, &x527, x525, x509, x506);
    var x528: u32 = undefined;
    var x529: u1 = undefined;
    addcarryxU32(&x528, &x529, x527, x507, x504);
    var x530: u32 = undefined;
    var x531: u1 = undefined;
    addcarryxU32(&x530, &x531, x529, x505, x502);
    var x532: u32 = undefined;
    var x533: u1 = undefined;
    addcarryxU32(&x532, &x533, x531, x503, x500);
    var x534: u32 = undefined;
    var x535: u1 = undefined;
    addcarryxU32(&x534, &x535, x533, x501, x498);
    var x536: u32 = undefined;
    var x537: u1 = undefined;
    addcarryxU32(&x536, &x537, x535, x499, x496);
    var x538: u32 = undefined;
    var x539: u1 = undefined;
    addcarryxU32(&x538, &x539, x537, x497, x494);
    var x540: u32 = undefined;
    var x541: u1 = undefined;
    addcarryxU32(&x540, &x541, 0x0, x468, x516);
    var x542: u32 = undefined;
    var x543: u1 = undefined;
    addcarryxU32(&x542, &x543, x541, x470, x518);
    var x544: u32 = undefined;
    var x545: u1 = undefined;
    addcarryxU32(&x544, &x545, x543, x472, x520);
    var x546: u32 = undefined;
    var x547: u1 = undefined;
    addcarryxU32(&x546, &x547, x545, x474, x522);
    var x548: u32 = undefined;
    var x549: u1 = undefined;
    addcarryxU32(&x548, &x549, x547, x476, x524);
    var x550: u32 = undefined;
    var x551: u1 = undefined;
    addcarryxU32(&x550, &x551, x549, x478, x526);
    var x552: u32 = undefined;
    var x553: u1 = undefined;
    addcarryxU32(&x552, &x553, x551, x480, x528);
    var x554: u32 = undefined;
    var x555: u1 = undefined;
    addcarryxU32(&x554, &x555, x553, x482, x530);
    var x556: u32 = undefined;
    var x557: u1 = undefined;
    addcarryxU32(&x556, &x557, x555, x484, x532);
    var x558: u32 = undefined;
    var x559: u1 = undefined;
    addcarryxU32(&x558, &x559, x557, x486, x534);
    var x560: u32 = undefined;
    var x561: u1 = undefined;
    addcarryxU32(&x560, &x561, x559, x488, x536);
    var x562: u32 = undefined;
    var x563: u1 = undefined;
    addcarryxU32(&x562, &x563, x561, x490, x538);
    var x564: u32 = undefined;
    var x565: u1 = undefined;
    addcarryxU32(&x564, &x565, x563, (cast(u32, x491) + cast(u32, x467)), (cast(u32, x539) + x495));
    var x566: u32 = undefined;
    var x567: u1 = undefined;
    addcarryxU32(&x566, &x567, 0x0, x542, (arg1[6]));
    var x568: u32 = undefined;
    var x569: u1 = undefined;
    addcarryxU32(&x568, &x569, x567, x544, cast(u32, 0x0));
    var x570: u32 = undefined;
    var x571: u1 = undefined;
    addcarryxU32(&x570, &x571, x569, x546, cast(u32, 0x0));
    var x572: u32 = undefined;
    var x573: u1 = undefined;
    addcarryxU32(&x572, &x573, x571, x548, cast(u32, 0x0));
    var x574: u32 = undefined;
    var x575: u1 = undefined;
    addcarryxU32(&x574, &x575, x573, x550, cast(u32, 0x0));
    var x576: u32 = undefined;
    var x577: u1 = undefined;
    addcarryxU32(&x576, &x577, x575, x552, cast(u32, 0x0));
    var x578: u32 = undefined;
    var x579: u1 = undefined;
    addcarryxU32(&x578, &x579, x577, x554, cast(u32, 0x0));
    var x580: u32 = undefined;
    var x581: u1 = undefined;
    addcarryxU32(&x580, &x581, x579, x556, cast(u32, 0x0));
    var x582: u32 = undefined;
    var x583: u1 = undefined;
    addcarryxU32(&x582, &x583, x581, x558, cast(u32, 0x0));
    var x584: u32 = undefined;
    var x585: u1 = undefined;
    addcarryxU32(&x584, &x585, x583, x560, cast(u32, 0x0));
    var x586: u32 = undefined;
    var x587: u1 = undefined;
    addcarryxU32(&x586, &x587, x585, x562, cast(u32, 0x0));
    var x588: u32 = undefined;
    var x589: u1 = undefined;
    addcarryxU32(&x588, &x589, x587, x564, cast(u32, 0x0));
    var x590: u32 = undefined;
    var x591: u32 = undefined;
    mulxU32(&x590, &x591, x566, 0xe88fdc45);
    var x592: u32 = undefined;
    var x593: u32 = undefined;
    mulxU32(&x592, &x593, x590, 0xffffffff);
    var x594: u32 = undefined;
    var x595: u32 = undefined;
    mulxU32(&x594, &x595, x590, 0xffffffff);
    var x596: u32 = undefined;
    var x597: u32 = undefined;
    mulxU32(&x596, &x597, x590, 0xffffffff);
    var x598: u32 = undefined;
    var x599: u32 = undefined;
    mulxU32(&x598, &x599, x590, 0xffffffff);
    var x600: u32 = undefined;
    var x601: u32 = undefined;
    mulxU32(&x600, &x601, x590, 0xffffffff);
    var x602: u32 = undefined;
    var x603: u32 = undefined;
    mulxU32(&x602, &x603, x590, 0xffffffff);
    var x604: u32 = undefined;
    var x605: u32 = undefined;
    mulxU32(&x604, &x605, x590, 0xc7634d81);
    var x606: u32 = undefined;
    var x607: u32 = undefined;
    mulxU32(&x606, &x607, x590, 0xf4372ddf);
    var x608: u32 = undefined;
    var x609: u32 = undefined;
    mulxU32(&x608, &x609, x590, 0x581a0db2);
    var x610: u32 = undefined;
    var x611: u32 = undefined;
    mulxU32(&x610, &x611, x590, 0x48b0a77a);
    var x612: u32 = undefined;
    var x613: u32 = undefined;
    mulxU32(&x612, &x613, x590, 0xecec196a);
    var x614: u32 = undefined;
    var x615: u32 = undefined;
    mulxU32(&x614, &x615, x590, 0xccc52973);
    var x616: u32 = undefined;
    var x617: u1 = undefined;
    addcarryxU32(&x616, &x617, 0x0, x615, x612);
    var x618: u32 = undefined;
    var x619: u1 = undefined;
    addcarryxU32(&x618, &x619, x617, x613, x610);
    var x620: u32 = undefined;
    var x621: u1 = undefined;
    addcarryxU32(&x620, &x621, x619, x611, x608);
    var x622: u32 = undefined;
    var x623: u1 = undefined;
    addcarryxU32(&x622, &x623, x621, x609, x606);
    var x624: u32 = undefined;
    var x625: u1 = undefined;
    addcarryxU32(&x624, &x625, x623, x607, x604);
    var x626: u32 = undefined;
    var x627: u1 = undefined;
    addcarryxU32(&x626, &x627, x625, x605, x602);
    var x628: u32 = undefined;
    var x629: u1 = undefined;
    addcarryxU32(&x628, &x629, x627, x603, x600);
    var x630: u32 = undefined;
    var x631: u1 = undefined;
    addcarryxU32(&x630, &x631, x629, x601, x598);
    var x632: u32 = undefined;
    var x633: u1 = undefined;
    addcarryxU32(&x632, &x633, x631, x599, x596);
    var x634: u32 = undefined;
    var x635: u1 = undefined;
    addcarryxU32(&x634, &x635, x633, x597, x594);
    var x636: u32 = undefined;
    var x637: u1 = undefined;
    addcarryxU32(&x636, &x637, x635, x595, x592);
    var x638: u32 = undefined;
    var x639: u1 = undefined;
    addcarryxU32(&x638, &x639, 0x0, x566, x614);
    var x640: u32 = undefined;
    var x641: u1 = undefined;
    addcarryxU32(&x640, &x641, x639, x568, x616);
    var x642: u32 = undefined;
    var x643: u1 = undefined;
    addcarryxU32(&x642, &x643, x641, x570, x618);
    var x644: u32 = undefined;
    var x645: u1 = undefined;
    addcarryxU32(&x644, &x645, x643, x572, x620);
    var x646: u32 = undefined;
    var x647: u1 = undefined;
    addcarryxU32(&x646, &x647, x645, x574, x622);
    var x648: u32 = undefined;
    var x649: u1 = undefined;
    addcarryxU32(&x648, &x649, x647, x576, x624);
    var x650: u32 = undefined;
    var x651: u1 = undefined;
    addcarryxU32(&x650, &x651, x649, x578, x626);
    var x652: u32 = undefined;
    var x653: u1 = undefined;
    addcarryxU32(&x652, &x653, x651, x580, x628);
    var x654: u32 = undefined;
    var x655: u1 = undefined;
    addcarryxU32(&x654, &x655, x653, x582, x630);
    var x656: u32 = undefined;
    var x657: u1 = undefined;
    addcarryxU32(&x656, &x657, x655, x584, x632);
    var x658: u32 = undefined;
    var x659: u1 = undefined;
    addcarryxU32(&x658, &x659, x657, x586, x634);
    var x660: u32 = undefined;
    var x661: u1 = undefined;
    addcarryxU32(&x660, &x661, x659, x588, x636);
    var x662: u32 = undefined;
    var x663: u1 = undefined;
    addcarryxU32(&x662, &x663, x661, (cast(u32, x589) + cast(u32, x565)), (cast(u32, x637) + x593));
    var x664: u32 = undefined;
    var x665: u1 = undefined;
    addcarryxU32(&x664, &x665, 0x0, x640, (arg1[7]));
    var x666: u32 = undefined;
    var x667: u1 = undefined;
    addcarryxU32(&x666, &x667, x665, x642, cast(u32, 0x0));
    var x668: u32 = undefined;
    var x669: u1 = undefined;
    addcarryxU32(&x668, &x669, x667, x644, cast(u32, 0x0));
    var x670: u32 = undefined;
    var x671: u1 = undefined;
    addcarryxU32(&x670, &x671, x669, x646, cast(u32, 0x0));
    var x672: u32 = undefined;
    var x673: u1 = undefined;
    addcarryxU32(&x672, &x673, x671, x648, cast(u32, 0x0));
    var x674: u32 = undefined;
    var x675: u1 = undefined;
    addcarryxU32(&x674, &x675, x673, x650, cast(u32, 0x0));
    var x676: u32 = undefined;
    var x677: u1 = undefined;
    addcarryxU32(&x676, &x677, x675, x652, cast(u32, 0x0));
    var x678: u32 = undefined;
    var x679: u1 = undefined;
    addcarryxU32(&x678, &x679, x677, x654, cast(u32, 0x0));
    var x680: u32 = undefined;
    var x681: u1 = undefined;
    addcarryxU32(&x680, &x681, x679, x656, cast(u32, 0x0));
    var x682: u32 = undefined;
    var x683: u1 = undefined;
    addcarryxU32(&x682, &x683, x681, x658, cast(u32, 0x0));
    var x684: u32 = undefined;
    var x685: u1 = undefined;
    addcarryxU32(&x684, &x685, x683, x660, cast(u32, 0x0));
    var x686: u32 = undefined;
    var x687: u1 = undefined;
    addcarryxU32(&x686, &x687, x685, x662, cast(u32, 0x0));
    var x688: u32 = undefined;
    var x689: u32 = undefined;
    mulxU32(&x688, &x689, x664, 0xe88fdc45);
    var x690: u32 = undefined;
    var x691: u32 = undefined;
    mulxU32(&x690, &x691, x688, 0xffffffff);
    var x692: u32 = undefined;
    var x693: u32 = undefined;
    mulxU32(&x692, &x693, x688, 0xffffffff);
    var x694: u32 = undefined;
    var x695: u32 = undefined;
    mulxU32(&x694, &x695, x688, 0xffffffff);
    var x696: u32 = undefined;
    var x697: u32 = undefined;
    mulxU32(&x696, &x697, x688, 0xffffffff);
    var x698: u32 = undefined;
    var x699: u32 = undefined;
    mulxU32(&x698, &x699, x688, 0xffffffff);
    var x700: u32 = undefined;
    var x701: u32 = undefined;
    mulxU32(&x700, &x701, x688, 0xffffffff);
    var x702: u32 = undefined;
    var x703: u32 = undefined;
    mulxU32(&x702, &x703, x688, 0xc7634d81);
    var x704: u32 = undefined;
    var x705: u32 = undefined;
    mulxU32(&x704, &x705, x688, 0xf4372ddf);
    var x706: u32 = undefined;
    var x707: u32 = undefined;
    mulxU32(&x706, &x707, x688, 0x581a0db2);
    var x708: u32 = undefined;
    var x709: u32 = undefined;
    mulxU32(&x708, &x709, x688, 0x48b0a77a);
    var x710: u32 = undefined;
    var x711: u32 = undefined;
    mulxU32(&x710, &x711, x688, 0xecec196a);
    var x712: u32 = undefined;
    var x713: u32 = undefined;
    mulxU32(&x712, &x713, x688, 0xccc52973);
    var x714: u32 = undefined;
    var x715: u1 = undefined;
    addcarryxU32(&x714, &x715, 0x0, x713, x710);
    var x716: u32 = undefined;
    var x717: u1 = undefined;
    addcarryxU32(&x716, &x717, x715, x711, x708);
    var x718: u32 = undefined;
    var x719: u1 = undefined;
    addcarryxU32(&x718, &x719, x717, x709, x706);
    var x720: u32 = undefined;
    var x721: u1 = undefined;
    addcarryxU32(&x720, &x721, x719, x707, x704);
    var x722: u32 = undefined;
    var x723: u1 = undefined;
    addcarryxU32(&x722, &x723, x721, x705, x702);
    var x724: u32 = undefined;
    var x725: u1 = undefined;
    addcarryxU32(&x724, &x725, x723, x703, x700);
    var x726: u32 = undefined;
    var x727: u1 = undefined;
    addcarryxU32(&x726, &x727, x725, x701, x698);
    var x728: u32 = undefined;
    var x729: u1 = undefined;
    addcarryxU32(&x728, &x729, x727, x699, x696);
    var x730: u32 = undefined;
    var x731: u1 = undefined;
    addcarryxU32(&x730, &x731, x729, x697, x694);
    var x732: u32 = undefined;
    var x733: u1 = undefined;
    addcarryxU32(&x732, &x733, x731, x695, x692);
    var x734: u32 = undefined;
    var x735: u1 = undefined;
    addcarryxU32(&x734, &x735, x733, x693, x690);
    var x736: u32 = undefined;
    var x737: u1 = undefined;
    addcarryxU32(&x736, &x737, 0x0, x664, x712);
    var x738: u32 = undefined;
    var x739: u1 = undefined;
    addcarryxU32(&x738, &x739, x737, x666, x714);
    var x740: u32 = undefined;
    var x741: u1 = undefined;
    addcarryxU32(&x740, &x741, x739, x668, x716);
    var x742: u32 = undefined;
    var x743: u1 = undefined;
    addcarryxU32(&x742, &x743, x741, x670, x718);
    var x744: u32 = undefined;
    var x745: u1 = undefined;
    addcarryxU32(&x744, &x745, x743, x672, x720);
    var x746: u32 = undefined;
    var x747: u1 = undefined;
    addcarryxU32(&x746, &x747, x745, x674, x722);
    var x748: u32 = undefined;
    var x749: u1 = undefined;
    addcarryxU32(&x748, &x749, x747, x676, x724);
    var x750: u32 = undefined;
    var x751: u1 = undefined;
    addcarryxU32(&x750, &x751, x749, x678, x726);
    var x752: u32 = undefined;
    var x753: u1 = undefined;
    addcarryxU32(&x752, &x753, x751, x680, x728);
    var x754: u32 = undefined;
    var x755: u1 = undefined;
    addcarryxU32(&x754, &x755, x753, x682, x730);
    var x756: u32 = undefined;
    var x757: u1 = undefined;
    addcarryxU32(&x756, &x757, x755, x684, x732);
    var x758: u32 = undefined;
    var x759: u1 = undefined;
    addcarryxU32(&x758, &x759, x757, x686, x734);
    var x760: u32 = undefined;
    var x761: u1 = undefined;
    addcarryxU32(&x760, &x761, x759, (cast(u32, x687) + cast(u32, x663)), (cast(u32, x735) + x691));
    var x762: u32 = undefined;
    var x763: u1 = undefined;
    addcarryxU32(&x762, &x763, 0x0, x738, (arg1[8]));
    var x764: u32 = undefined;
    var x765: u1 = undefined;
    addcarryxU32(&x764, &x765, x763, x740, cast(u32, 0x0));
    var x766: u32 = undefined;
    var x767: u1 = undefined;
    addcarryxU32(&x766, &x767, x765, x742, cast(u32, 0x0));
    var x768: u32 = undefined;
    var x769: u1 = undefined;
    addcarryxU32(&x768, &x769, x767, x744, cast(u32, 0x0));
    var x770: u32 = undefined;
    var x771: u1 = undefined;
    addcarryxU32(&x770, &x771, x769, x746, cast(u32, 0x0));
    var x772: u32 = undefined;
    var x773: u1 = undefined;
    addcarryxU32(&x772, &x773, x771, x748, cast(u32, 0x0));
    var x774: u32 = undefined;
    var x775: u1 = undefined;
    addcarryxU32(&x774, &x775, x773, x750, cast(u32, 0x0));
    var x776: u32 = undefined;
    var x777: u1 = undefined;
    addcarryxU32(&x776, &x777, x775, x752, cast(u32, 0x0));
    var x778: u32 = undefined;
    var x779: u1 = undefined;
    addcarryxU32(&x778, &x779, x777, x754, cast(u32, 0x0));
    var x780: u32 = undefined;
    var x781: u1 = undefined;
    addcarryxU32(&x780, &x781, x779, x756, cast(u32, 0x0));
    var x782: u32 = undefined;
    var x783: u1 = undefined;
    addcarryxU32(&x782, &x783, x781, x758, cast(u32, 0x0));
    var x784: u32 = undefined;
    var x785: u1 = undefined;
    addcarryxU32(&x784, &x785, x783, x760, cast(u32, 0x0));
    var x786: u32 = undefined;
    var x787: u32 = undefined;
    mulxU32(&x786, &x787, x762, 0xe88fdc45);
    var x788: u32 = undefined;
    var x789: u32 = undefined;
    mulxU32(&x788, &x789, x786, 0xffffffff);
    var x790: u32 = undefined;
    var x791: u32 = undefined;
    mulxU32(&x790, &x791, x786, 0xffffffff);
    var x792: u32 = undefined;
    var x793: u32 = undefined;
    mulxU32(&x792, &x793, x786, 0xffffffff);
    var x794: u32 = undefined;
    var x795: u32 = undefined;
    mulxU32(&x794, &x795, x786, 0xffffffff);
    var x796: u32 = undefined;
    var x797: u32 = undefined;
    mulxU32(&x796, &x797, x786, 0xffffffff);
    var x798: u32 = undefined;
    var x799: u32 = undefined;
    mulxU32(&x798, &x799, x786, 0xffffffff);
    var x800: u32 = undefined;
    var x801: u32 = undefined;
    mulxU32(&x800, &x801, x786, 0xc7634d81);
    var x802: u32 = undefined;
    var x803: u32 = undefined;
    mulxU32(&x802, &x803, x786, 0xf4372ddf);
    var x804: u32 = undefined;
    var x805: u32 = undefined;
    mulxU32(&x804, &x805, x786, 0x581a0db2);
    var x806: u32 = undefined;
    var x807: u32 = undefined;
    mulxU32(&x806, &x807, x786, 0x48b0a77a);
    var x808: u32 = undefined;
    var x809: u32 = undefined;
    mulxU32(&x808, &x809, x786, 0xecec196a);
    var x810: u32 = undefined;
    var x811: u32 = undefined;
    mulxU32(&x810, &x811, x786, 0xccc52973);
    var x812: u32 = undefined;
    var x813: u1 = undefined;
    addcarryxU32(&x812, &x813, 0x0, x811, x808);
    var x814: u32 = undefined;
    var x815: u1 = undefined;
    addcarryxU32(&x814, &x815, x813, x809, x806);
    var x816: u32 = undefined;
    var x817: u1 = undefined;
    addcarryxU32(&x816, &x817, x815, x807, x804);
    var x818: u32 = undefined;
    var x819: u1 = undefined;
    addcarryxU32(&x818, &x819, x817, x805, x802);
    var x820: u32 = undefined;
    var x821: u1 = undefined;
    addcarryxU32(&x820, &x821, x819, x803, x800);
    var x822: u32 = undefined;
    var x823: u1 = undefined;
    addcarryxU32(&x822, &x823, x821, x801, x798);
    var x824: u32 = undefined;
    var x825: u1 = undefined;
    addcarryxU32(&x824, &x825, x823, x799, x796);
    var x826: u32 = undefined;
    var x827: u1 = undefined;
    addcarryxU32(&x826, &x827, x825, x797, x794);
    var x828: u32 = undefined;
    var x829: u1 = undefined;
    addcarryxU32(&x828, &x829, x827, x795, x792);
    var x830: u32 = undefined;
    var x831: u1 = undefined;
    addcarryxU32(&x830, &x831, x829, x793, x790);
    var x832: u32 = undefined;
    var x833: u1 = undefined;
    addcarryxU32(&x832, &x833, x831, x791, x788);
    var x834: u32 = undefined;
    var x835: u1 = undefined;
    addcarryxU32(&x834, &x835, 0x0, x762, x810);
    var x836: u32 = undefined;
    var x837: u1 = undefined;
    addcarryxU32(&x836, &x837, x835, x764, x812);
    var x838: u32 = undefined;
    var x839: u1 = undefined;
    addcarryxU32(&x838, &x839, x837, x766, x814);
    var x840: u32 = undefined;
    var x841: u1 = undefined;
    addcarryxU32(&x840, &x841, x839, x768, x816);
    var x842: u32 = undefined;
    var x843: u1 = undefined;
    addcarryxU32(&x842, &x843, x841, x770, x818);
    var x844: u32 = undefined;
    var x845: u1 = undefined;
    addcarryxU32(&x844, &x845, x843, x772, x820);
    var x846: u32 = undefined;
    var x847: u1 = undefined;
    addcarryxU32(&x846, &x847, x845, x774, x822);
    var x848: u32 = undefined;
    var x849: u1 = undefined;
    addcarryxU32(&x848, &x849, x847, x776, x824);
    var x850: u32 = undefined;
    var x851: u1 = undefined;
    addcarryxU32(&x850, &x851, x849, x778, x826);
    var x852: u32 = undefined;
    var x853: u1 = undefined;
    addcarryxU32(&x852, &x853, x851, x780, x828);
    var x854: u32 = undefined;
    var x855: u1 = undefined;
    addcarryxU32(&x854, &x855, x853, x782, x830);
    var x856: u32 = undefined;
    var x857: u1 = undefined;
    addcarryxU32(&x856, &x857, x855, x784, x832);
    var x858: u32 = undefined;
    var x859: u1 = undefined;
    addcarryxU32(&x858, &x859, x857, (cast(u32, x785) + cast(u32, x761)), (cast(u32, x833) + x789));
    var x860: u32 = undefined;
    var x861: u1 = undefined;
    addcarryxU32(&x860, &x861, 0x0, x836, (arg1[9]));
    var x862: u32 = undefined;
    var x863: u1 = undefined;
    addcarryxU32(&x862, &x863, x861, x838, cast(u32, 0x0));
    var x864: u32 = undefined;
    var x865: u1 = undefined;
    addcarryxU32(&x864, &x865, x863, x840, cast(u32, 0x0));
    var x866: u32 = undefined;
    var x867: u1 = undefined;
    addcarryxU32(&x866, &x867, x865, x842, cast(u32, 0x0));
    var x868: u32 = undefined;
    var x869: u1 = undefined;
    addcarryxU32(&x868, &x869, x867, x844, cast(u32, 0x0));
    var x870: u32 = undefined;
    var x871: u1 = undefined;
    addcarryxU32(&x870, &x871, x869, x846, cast(u32, 0x0));
    var x872: u32 = undefined;
    var x873: u1 = undefined;
    addcarryxU32(&x872, &x873, x871, x848, cast(u32, 0x0));
    var x874: u32 = undefined;
    var x875: u1 = undefined;
    addcarryxU32(&x874, &x875, x873, x850, cast(u32, 0x0));
    var x876: u32 = undefined;
    var x877: u1 = undefined;
    addcarryxU32(&x876, &x877, x875, x852, cast(u32, 0x0));
    var x878: u32 = undefined;
    var x879: u1 = undefined;
    addcarryxU32(&x878, &x879, x877, x854, cast(u32, 0x0));
    var x880: u32 = undefined;
    var x881: u1 = undefined;
    addcarryxU32(&x880, &x881, x879, x856, cast(u32, 0x0));
    var x882: u32 = undefined;
    var x883: u1 = undefined;
    addcarryxU32(&x882, &x883, x881, x858, cast(u32, 0x0));
    var x884: u32 = undefined;
    var x885: u32 = undefined;
    mulxU32(&x884, &x885, x860, 0xe88fdc45);
    var x886: u32 = undefined;
    var x887: u32 = undefined;
    mulxU32(&x886, &x887, x884, 0xffffffff);
    var x888: u32 = undefined;
    var x889: u32 = undefined;
    mulxU32(&x888, &x889, x884, 0xffffffff);
    var x890: u32 = undefined;
    var x891: u32 = undefined;
    mulxU32(&x890, &x891, x884, 0xffffffff);
    var x892: u32 = undefined;
    var x893: u32 = undefined;
    mulxU32(&x892, &x893, x884, 0xffffffff);
    var x894: u32 = undefined;
    var x895: u32 = undefined;
    mulxU32(&x894, &x895, x884, 0xffffffff);
    var x896: u32 = undefined;
    var x897: u32 = undefined;
    mulxU32(&x896, &x897, x884, 0xffffffff);
    var x898: u32 = undefined;
    var x899: u32 = undefined;
    mulxU32(&x898, &x899, x884, 0xc7634d81);
    var x900: u32 = undefined;
    var x901: u32 = undefined;
    mulxU32(&x900, &x901, x884, 0xf4372ddf);
    var x902: u32 = undefined;
    var x903: u32 = undefined;
    mulxU32(&x902, &x903, x884, 0x581a0db2);
    var x904: u32 = undefined;
    var x905: u32 = undefined;
    mulxU32(&x904, &x905, x884, 0x48b0a77a);
    var x906: u32 = undefined;
    var x907: u32 = undefined;
    mulxU32(&x906, &x907, x884, 0xecec196a);
    var x908: u32 = undefined;
    var x909: u32 = undefined;
    mulxU32(&x908, &x909, x884, 0xccc52973);
    var x910: u32 = undefined;
    var x911: u1 = undefined;
    addcarryxU32(&x910, &x911, 0x0, x909, x906);
    var x912: u32 = undefined;
    var x913: u1 = undefined;
    addcarryxU32(&x912, &x913, x911, x907, x904);
    var x914: u32 = undefined;
    var x915: u1 = undefined;
    addcarryxU32(&x914, &x915, x913, x905, x902);
    var x916: u32 = undefined;
    var x917: u1 = undefined;
    addcarryxU32(&x916, &x917, x915, x903, x900);
    var x918: u32 = undefined;
    var x919: u1 = undefined;
    addcarryxU32(&x918, &x919, x917, x901, x898);
    var x920: u32 = undefined;
    var x921: u1 = undefined;
    addcarryxU32(&x920, &x921, x919, x899, x896);
    var x922: u32 = undefined;
    var x923: u1 = undefined;
    addcarryxU32(&x922, &x923, x921, x897, x894);
    var x924: u32 = undefined;
    var x925: u1 = undefined;
    addcarryxU32(&x924, &x925, x923, x895, x892);
    var x926: u32 = undefined;
    var x927: u1 = undefined;
    addcarryxU32(&x926, &x927, x925, x893, x890);
    var x928: u32 = undefined;
    var x929: u1 = undefined;
    addcarryxU32(&x928, &x929, x927, x891, x888);
    var x930: u32 = undefined;
    var x931: u1 = undefined;
    addcarryxU32(&x930, &x931, x929, x889, x886);
    var x932: u32 = undefined;
    var x933: u1 = undefined;
    addcarryxU32(&x932, &x933, 0x0, x860, x908);
    var x934: u32 = undefined;
    var x935: u1 = undefined;
    addcarryxU32(&x934, &x935, x933, x862, x910);
    var x936: u32 = undefined;
    var x937: u1 = undefined;
    addcarryxU32(&x936, &x937, x935, x864, x912);
    var x938: u32 = undefined;
    var x939: u1 = undefined;
    addcarryxU32(&x938, &x939, x937, x866, x914);
    var x940: u32 = undefined;
    var x941: u1 = undefined;
    addcarryxU32(&x940, &x941, x939, x868, x916);
    var x942: u32 = undefined;
    var x943: u1 = undefined;
    addcarryxU32(&x942, &x943, x941, x870, x918);
    var x944: u32 = undefined;
    var x945: u1 = undefined;
    addcarryxU32(&x944, &x945, x943, x872, x920);
    var x946: u32 = undefined;
    var x947: u1 = undefined;
    addcarryxU32(&x946, &x947, x945, x874, x922);
    var x948: u32 = undefined;
    var x949: u1 = undefined;
    addcarryxU32(&x948, &x949, x947, x876, x924);
    var x950: u32 = undefined;
    var x951: u1 = undefined;
    addcarryxU32(&x950, &x951, x949, x878, x926);
    var x952: u32 = undefined;
    var x953: u1 = undefined;
    addcarryxU32(&x952, &x953, x951, x880, x928);
    var x954: u32 = undefined;
    var x955: u1 = undefined;
    addcarryxU32(&x954, &x955, x953, x882, x930);
    var x956: u32 = undefined;
    var x957: u1 = undefined;
    addcarryxU32(&x956, &x957, x955, (cast(u32, x883) + cast(u32, x859)), (cast(u32, x931) + x887));
    var x958: u32 = undefined;
    var x959: u1 = undefined;
    addcarryxU32(&x958, &x959, 0x0, x934, (arg1[10]));
    var x960: u32 = undefined;
    var x961: u1 = undefined;
    addcarryxU32(&x960, &x961, x959, x936, cast(u32, 0x0));
    var x962: u32 = undefined;
    var x963: u1 = undefined;
    addcarryxU32(&x962, &x963, x961, x938, cast(u32, 0x0));
    var x964: u32 = undefined;
    var x965: u1 = undefined;
    addcarryxU32(&x964, &x965, x963, x940, cast(u32, 0x0));
    var x966: u32 = undefined;
    var x967: u1 = undefined;
    addcarryxU32(&x966, &x967, x965, x942, cast(u32, 0x0));
    var x968: u32 = undefined;
    var x969: u1 = undefined;
    addcarryxU32(&x968, &x969, x967, x944, cast(u32, 0x0));
    var x970: u32 = undefined;
    var x971: u1 = undefined;
    addcarryxU32(&x970, &x971, x969, x946, cast(u32, 0x0));
    var x972: u32 = undefined;
    var x973: u1 = undefined;
    addcarryxU32(&x972, &x973, x971, x948, cast(u32, 0x0));
    var x974: u32 = undefined;
    var x975: u1 = undefined;
    addcarryxU32(&x974, &x975, x973, x950, cast(u32, 0x0));
    var x976: u32 = undefined;
    var x977: u1 = undefined;
    addcarryxU32(&x976, &x977, x975, x952, cast(u32, 0x0));
    var x978: u32 = undefined;
    var x979: u1 = undefined;
    addcarryxU32(&x978, &x979, x977, x954, cast(u32, 0x0));
    var x980: u32 = undefined;
    var x981: u1 = undefined;
    addcarryxU32(&x980, &x981, x979, x956, cast(u32, 0x0));
    var x982: u32 = undefined;
    var x983: u32 = undefined;
    mulxU32(&x982, &x983, x958, 0xe88fdc45);
    var x984: u32 = undefined;
    var x985: u32 = undefined;
    mulxU32(&x984, &x985, x982, 0xffffffff);
    var x986: u32 = undefined;
    var x987: u32 = undefined;
    mulxU32(&x986, &x987, x982, 0xffffffff);
    var x988: u32 = undefined;
    var x989: u32 = undefined;
    mulxU32(&x988, &x989, x982, 0xffffffff);
    var x990: u32 = undefined;
    var x991: u32 = undefined;
    mulxU32(&x990, &x991, x982, 0xffffffff);
    var x992: u32 = undefined;
    var x993: u32 = undefined;
    mulxU32(&x992, &x993, x982, 0xffffffff);
    var x994: u32 = undefined;
    var x995: u32 = undefined;
    mulxU32(&x994, &x995, x982, 0xffffffff);
    var x996: u32 = undefined;
    var x997: u32 = undefined;
    mulxU32(&x996, &x997, x982, 0xc7634d81);
    var x998: u32 = undefined;
    var x999: u32 = undefined;
    mulxU32(&x998, &x999, x982, 0xf4372ddf);
    var x1000: u32 = undefined;
    var x1001: u32 = undefined;
    mulxU32(&x1000, &x1001, x982, 0x581a0db2);
    var x1002: u32 = undefined;
    var x1003: u32 = undefined;
    mulxU32(&x1002, &x1003, x982, 0x48b0a77a);
    var x1004: u32 = undefined;
    var x1005: u32 = undefined;
    mulxU32(&x1004, &x1005, x982, 0xecec196a);
    var x1006: u32 = undefined;
    var x1007: u32 = undefined;
    mulxU32(&x1006, &x1007, x982, 0xccc52973);
    var x1008: u32 = undefined;
    var x1009: u1 = undefined;
    addcarryxU32(&x1008, &x1009, 0x0, x1007, x1004);
    var x1010: u32 = undefined;
    var x1011: u1 = undefined;
    addcarryxU32(&x1010, &x1011, x1009, x1005, x1002);
    var x1012: u32 = undefined;
    var x1013: u1 = undefined;
    addcarryxU32(&x1012, &x1013, x1011, x1003, x1000);
    var x1014: u32 = undefined;
    var x1015: u1 = undefined;
    addcarryxU32(&x1014, &x1015, x1013, x1001, x998);
    var x1016: u32 = undefined;
    var x1017: u1 = undefined;
    addcarryxU32(&x1016, &x1017, x1015, x999, x996);
    var x1018: u32 = undefined;
    var x1019: u1 = undefined;
    addcarryxU32(&x1018, &x1019, x1017, x997, x994);
    var x1020: u32 = undefined;
    var x1021: u1 = undefined;
    addcarryxU32(&x1020, &x1021, x1019, x995, x992);
    var x1022: u32 = undefined;
    var x1023: u1 = undefined;
    addcarryxU32(&x1022, &x1023, x1021, x993, x990);
    var x1024: u32 = undefined;
    var x1025: u1 = undefined;
    addcarryxU32(&x1024, &x1025, x1023, x991, x988);
    var x1026: u32 = undefined;
    var x1027: u1 = undefined;
    addcarryxU32(&x1026, &x1027, x1025, x989, x986);
    var x1028: u32 = undefined;
    var x1029: u1 = undefined;
    addcarryxU32(&x1028, &x1029, x1027, x987, x984);
    var x1030: u32 = undefined;
    var x1031: u1 = undefined;
    addcarryxU32(&x1030, &x1031, 0x0, x958, x1006);
    var x1032: u32 = undefined;
    var x1033: u1 = undefined;
    addcarryxU32(&x1032, &x1033, x1031, x960, x1008);
    var x1034: u32 = undefined;
    var x1035: u1 = undefined;
    addcarryxU32(&x1034, &x1035, x1033, x962, x1010);
    var x1036: u32 = undefined;
    var x1037: u1 = undefined;
    addcarryxU32(&x1036, &x1037, x1035, x964, x1012);
    var x1038: u32 = undefined;
    var x1039: u1 = undefined;
    addcarryxU32(&x1038, &x1039, x1037, x966, x1014);
    var x1040: u32 = undefined;
    var x1041: u1 = undefined;
    addcarryxU32(&x1040, &x1041, x1039, x968, x1016);
    var x1042: u32 = undefined;
    var x1043: u1 = undefined;
    addcarryxU32(&x1042, &x1043, x1041, x970, x1018);
    var x1044: u32 = undefined;
    var x1045: u1 = undefined;
    addcarryxU32(&x1044, &x1045, x1043, x972, x1020);
    var x1046: u32 = undefined;
    var x1047: u1 = undefined;
    addcarryxU32(&x1046, &x1047, x1045, x974, x1022);
    var x1048: u32 = undefined;
    var x1049: u1 = undefined;
    addcarryxU32(&x1048, &x1049, x1047, x976, x1024);
    var x1050: u32 = undefined;
    var x1051: u1 = undefined;
    addcarryxU32(&x1050, &x1051, x1049, x978, x1026);
    var x1052: u32 = undefined;
    var x1053: u1 = undefined;
    addcarryxU32(&x1052, &x1053, x1051, x980, x1028);
    var x1054: u32 = undefined;
    var x1055: u1 = undefined;
    addcarryxU32(&x1054, &x1055, x1053, (cast(u32, x981) + cast(u32, x957)), (cast(u32, x1029) + x985));
    var x1056: u32 = undefined;
    var x1057: u1 = undefined;
    addcarryxU32(&x1056, &x1057, 0x0, x1032, (arg1[11]));
    var x1058: u32 = undefined;
    var x1059: u1 = undefined;
    addcarryxU32(&x1058, &x1059, x1057, x1034, cast(u32, 0x0));
    var x1060: u32 = undefined;
    var x1061: u1 = undefined;
    addcarryxU32(&x1060, &x1061, x1059, x1036, cast(u32, 0x0));
    var x1062: u32 = undefined;
    var x1063: u1 = undefined;
    addcarryxU32(&x1062, &x1063, x1061, x1038, cast(u32, 0x0));
    var x1064: u32 = undefined;
    var x1065: u1 = undefined;
    addcarryxU32(&x1064, &x1065, x1063, x1040, cast(u32, 0x0));
    var x1066: u32 = undefined;
    var x1067: u1 = undefined;
    addcarryxU32(&x1066, &x1067, x1065, x1042, cast(u32, 0x0));
    var x1068: u32 = undefined;
    var x1069: u1 = undefined;
    addcarryxU32(&x1068, &x1069, x1067, x1044, cast(u32, 0x0));
    var x1070: u32 = undefined;
    var x1071: u1 = undefined;
    addcarryxU32(&x1070, &x1071, x1069, x1046, cast(u32, 0x0));
    var x1072: u32 = undefined;
    var x1073: u1 = undefined;
    addcarryxU32(&x1072, &x1073, x1071, x1048, cast(u32, 0x0));
    var x1074: u32 = undefined;
    var x1075: u1 = undefined;
    addcarryxU32(&x1074, &x1075, x1073, x1050, cast(u32, 0x0));
    var x1076: u32 = undefined;
    var x1077: u1 = undefined;
    addcarryxU32(&x1076, &x1077, x1075, x1052, cast(u32, 0x0));
    var x1078: u32 = undefined;
    var x1079: u1 = undefined;
    addcarryxU32(&x1078, &x1079, x1077, x1054, cast(u32, 0x0));
    var x1080: u32 = undefined;
    var x1081: u32 = undefined;
    mulxU32(&x1080, &x1081, x1056, 0xe88fdc45);
    var x1082: u32 = undefined;
    var x1083: u32 = undefined;
    mulxU32(&x1082, &x1083, x1080, 0xffffffff);
    var x1084: u32 = undefined;
    var x1085: u32 = undefined;
    mulxU32(&x1084, &x1085, x1080, 0xffffffff);
    var x1086: u32 = undefined;
    var x1087: u32 = undefined;
    mulxU32(&x1086, &x1087, x1080, 0xffffffff);
    var x1088: u32 = undefined;
    var x1089: u32 = undefined;
    mulxU32(&x1088, &x1089, x1080, 0xffffffff);
    var x1090: u32 = undefined;
    var x1091: u32 = undefined;
    mulxU32(&x1090, &x1091, x1080, 0xffffffff);
    var x1092: u32 = undefined;
    var x1093: u32 = undefined;
    mulxU32(&x1092, &x1093, x1080, 0xffffffff);
    var x1094: u32 = undefined;
    var x1095: u32 = undefined;
    mulxU32(&x1094, &x1095, x1080, 0xc7634d81);
    var x1096: u32 = undefined;
    var x1097: u32 = undefined;
    mulxU32(&x1096, &x1097, x1080, 0xf4372ddf);
    var x1098: u32 = undefined;
    var x1099: u32 = undefined;
    mulxU32(&x1098, &x1099, x1080, 0x581a0db2);
    var x1100: u32 = undefined;
    var x1101: u32 = undefined;
    mulxU32(&x1100, &x1101, x1080, 0x48b0a77a);
    var x1102: u32 = undefined;
    var x1103: u32 = undefined;
    mulxU32(&x1102, &x1103, x1080, 0xecec196a);
    var x1104: u32 = undefined;
    var x1105: u32 = undefined;
    mulxU32(&x1104, &x1105, x1080, 0xccc52973);
    var x1106: u32 = undefined;
    var x1107: u1 = undefined;
    addcarryxU32(&x1106, &x1107, 0x0, x1105, x1102);
    var x1108: u32 = undefined;
    var x1109: u1 = undefined;
    addcarryxU32(&x1108, &x1109, x1107, x1103, x1100);
    var x1110: u32 = undefined;
    var x1111: u1 = undefined;
    addcarryxU32(&x1110, &x1111, x1109, x1101, x1098);
    var x1112: u32 = undefined;
    var x1113: u1 = undefined;
    addcarryxU32(&x1112, &x1113, x1111, x1099, x1096);
    var x1114: u32 = undefined;
    var x1115: u1 = undefined;
    addcarryxU32(&x1114, &x1115, x1113, x1097, x1094);
    var x1116: u32 = undefined;
    var x1117: u1 = undefined;
    addcarryxU32(&x1116, &x1117, x1115, x1095, x1092);
    var x1118: u32 = undefined;
    var x1119: u1 = undefined;
    addcarryxU32(&x1118, &x1119, x1117, x1093, x1090);
    var x1120: u32 = undefined;
    var x1121: u1 = undefined;
    addcarryxU32(&x1120, &x1121, x1119, x1091, x1088);
    var x1122: u32 = undefined;
    var x1123: u1 = undefined;
    addcarryxU32(&x1122, &x1123, x1121, x1089, x1086);
    var x1124: u32 = undefined;
    var x1125: u1 = undefined;
    addcarryxU32(&x1124, &x1125, x1123, x1087, x1084);
    var x1126: u32 = undefined;
    var x1127: u1 = undefined;
    addcarryxU32(&x1126, &x1127, x1125, x1085, x1082);
    var x1128: u32 = undefined;
    var x1129: u1 = undefined;
    addcarryxU32(&x1128, &x1129, 0x0, x1056, x1104);
    var x1130: u32 = undefined;
    var x1131: u1 = undefined;
    addcarryxU32(&x1130, &x1131, x1129, x1058, x1106);
    var x1132: u32 = undefined;
    var x1133: u1 = undefined;
    addcarryxU32(&x1132, &x1133, x1131, x1060, x1108);
    var x1134: u32 = undefined;
    var x1135: u1 = undefined;
    addcarryxU32(&x1134, &x1135, x1133, x1062, x1110);
    var x1136: u32 = undefined;
    var x1137: u1 = undefined;
    addcarryxU32(&x1136, &x1137, x1135, x1064, x1112);
    var x1138: u32 = undefined;
    var x1139: u1 = undefined;
    addcarryxU32(&x1138, &x1139, x1137, x1066, x1114);
    var x1140: u32 = undefined;
    var x1141: u1 = undefined;
    addcarryxU32(&x1140, &x1141, x1139, x1068, x1116);
    var x1142: u32 = undefined;
    var x1143: u1 = undefined;
    addcarryxU32(&x1142, &x1143, x1141, x1070, x1118);
    var x1144: u32 = undefined;
    var x1145: u1 = undefined;
    addcarryxU32(&x1144, &x1145, x1143, x1072, x1120);
    var x1146: u32 = undefined;
    var x1147: u1 = undefined;
    addcarryxU32(&x1146, &x1147, x1145, x1074, x1122);
    var x1148: u32 = undefined;
    var x1149: u1 = undefined;
    addcarryxU32(&x1148, &x1149, x1147, x1076, x1124);
    var x1150: u32 = undefined;
    var x1151: u1 = undefined;
    addcarryxU32(&x1150, &x1151, x1149, x1078, x1126);
    var x1152: u32 = undefined;
    var x1153: u1 = undefined;
    addcarryxU32(&x1152, &x1153, x1151, (cast(u32, x1079) + cast(u32, x1055)), (cast(u32, x1127) + x1083));
    var x1154: u32 = undefined;
    var x1155: u1 = undefined;
    subborrowxU32(&x1154, &x1155, 0x0, x1130, 0xccc52973);
    var x1156: u32 = undefined;
    var x1157: u1 = undefined;
    subborrowxU32(&x1156, &x1157, x1155, x1132, 0xecec196a);
    var x1158: u32 = undefined;
    var x1159: u1 = undefined;
    subborrowxU32(&x1158, &x1159, x1157, x1134, 0x48b0a77a);
    var x1160: u32 = undefined;
    var x1161: u1 = undefined;
    subborrowxU32(&x1160, &x1161, x1159, x1136, 0x581a0db2);
    var x1162: u32 = undefined;
    var x1163: u1 = undefined;
    subborrowxU32(&x1162, &x1163, x1161, x1138, 0xf4372ddf);
    var x1164: u32 = undefined;
    var x1165: u1 = undefined;
    subborrowxU32(&x1164, &x1165, x1163, x1140, 0xc7634d81);
    var x1166: u32 = undefined;
    var x1167: u1 = undefined;
    subborrowxU32(&x1166, &x1167, x1165, x1142, 0xffffffff);
    var x1168: u32 = undefined;
    var x1169: u1 = undefined;
    subborrowxU32(&x1168, &x1169, x1167, x1144, 0xffffffff);
    var x1170: u32 = undefined;
    var x1171: u1 = undefined;
    subborrowxU32(&x1170, &x1171, x1169, x1146, 0xffffffff);
    var x1172: u32 = undefined;
    var x1173: u1 = undefined;
    subborrowxU32(&x1172, &x1173, x1171, x1148, 0xffffffff);
    var x1174: u32 = undefined;
    var x1175: u1 = undefined;
    subborrowxU32(&x1174, &x1175, x1173, x1150, 0xffffffff);
    var x1176: u32 = undefined;
    var x1177: u1 = undefined;
    subborrowxU32(&x1176, &x1177, x1175, x1152, 0xffffffff);
    var x1178: u32 = undefined;
    var x1179: u1 = undefined;
    subborrowxU32(&x1178, &x1179, x1177, cast(u32, x1153), cast(u32, 0x0));
    var x1180: u32 = undefined;
    cmovznzU32(&x1180, x1179, x1154, x1130);
    var x1181: u32 = undefined;
    cmovznzU32(&x1181, x1179, x1156, x1132);
    var x1182: u32 = undefined;
    cmovznzU32(&x1182, x1179, x1158, x1134);
    var x1183: u32 = undefined;
    cmovznzU32(&x1183, x1179, x1160, x1136);
    var x1184: u32 = undefined;
    cmovznzU32(&x1184, x1179, x1162, x1138);
    var x1185: u32 = undefined;
    cmovznzU32(&x1185, x1179, x1164, x1140);
    var x1186: u32 = undefined;
    cmovznzU32(&x1186, x1179, x1166, x1142);
    var x1187: u32 = undefined;
    cmovznzU32(&x1187, x1179, x1168, x1144);
    var x1188: u32 = undefined;
    cmovznzU32(&x1188, x1179, x1170, x1146);
    var x1189: u32 = undefined;
    cmovznzU32(&x1189, x1179, x1172, x1148);
    var x1190: u32 = undefined;
    cmovznzU32(&x1190, x1179, x1174, x1150);
    var x1191: u32 = undefined;
    cmovznzU32(&x1191, x1179, x1176, x1152);
    out1[0] = x1180;
    out1[1] = x1181;
    out1[2] = x1182;
    out1[3] = x1183;
    out1[4] = x1184;
    out1[5] = x1185;
    out1[6] = x1186;
    out1[7] = x1187;
    out1[8] = x1188;
    out1[9] = x1189;
    out1[10] = x1190;
    out1[11] = x1191;
}

/// The function toMontgomery translates a field element into the Montgomery domain.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   eval (from_montgomery out1) mod m = eval arg1 mod m
///   0 ≤ eval out1 < m
///
pub fn toMontgomery(out1: *MontgomeryDomainFieldElement, arg1: NonMontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[1]);
    const x2 = (arg1[2]);
    const x3 = (arg1[3]);
    const x4 = (arg1[4]);
    const x5 = (arg1[5]);
    const x6 = (arg1[6]);
    const x7 = (arg1[7]);
    const x8 = (arg1[8]);
    const x9 = (arg1[9]);
    const x10 = (arg1[10]);
    const x11 = (arg1[11]);
    const x12 = (arg1[0]);
    var x13: u32 = undefined;
    var x14: u32 = undefined;
    mulxU32(&x13, &x14, x12, 0xc84ee01);
    var x15: u32 = undefined;
    var x16: u32 = undefined;
    mulxU32(&x15, &x16, x12, 0x2b39bf21);
    var x17: u32 = undefined;
    var x18: u32 = undefined;
    mulxU32(&x17, &x18, x12, 0x3fb05b7a);
    var x19: u32 = undefined;
    var x20: u32 = undefined;
    mulxU32(&x19, &x20, x12, 0x28266895);
    var x21: u32 = undefined;
    var x22: u32 = undefined;
    mulxU32(&x21, &x22, x12, 0xd40d4917);
    var x23: u32 = undefined;
    var x24: u32 = undefined;
    mulxU32(&x23, &x24, x12, 0x4aab1cc5);
    var x25: u32 = undefined;
    var x26: u32 = undefined;
    mulxU32(&x25, &x26, x12, 0xbc3e483a);
    var x27: u32 = undefined;
    var x28: u32 = undefined;
    mulxU32(&x27, &x28, x12, 0xfcb82947);
    var x29: u32 = undefined;
    var x30: u32 = undefined;
    mulxU32(&x29, &x30, x12, 0xff3d81e5);
    var x31: u32 = undefined;
    var x32: u32 = undefined;
    mulxU32(&x31, &x32, x12, 0xdf1aa419);
    var x33: u32 = undefined;
    var x34: u32 = undefined;
    mulxU32(&x33, &x34, x12, 0x2d319b24);
    var x35: u32 = undefined;
    var x36: u32 = undefined;
    mulxU32(&x35, &x36, x12, 0x19b409a9);
    var x37: u32 = undefined;
    var x38: u1 = undefined;
    addcarryxU32(&x37, &x38, 0x0, x36, x33);
    var x39: u32 = undefined;
    var x40: u1 = undefined;
    addcarryxU32(&x39, &x40, x38, x34, x31);
    var x41: u32 = undefined;
    var x42: u1 = undefined;
    addcarryxU32(&x41, &x42, x40, x32, x29);
    var x43: u32 = undefined;
    var x44: u1 = undefined;
    addcarryxU32(&x43, &x44, x42, x30, x27);
    var x45: u32 = undefined;
    var x46: u1 = undefined;
    addcarryxU32(&x45, &x46, x44, x28, x25);
    var x47: u32 = undefined;
    var x48: u1 = undefined;
    addcarryxU32(&x47, &x48, x46, x26, x23);
    var x49: u32 = undefined;
    var x50: u1 = undefined;
    addcarryxU32(&x49, &x50, x48, x24, x21);
    var x51: u32 = undefined;
    var x52: u1 = undefined;
    addcarryxU32(&x51, &x52, x50, x22, x19);
    var x53: u32 = undefined;
    var x54: u1 = undefined;
    addcarryxU32(&x53, &x54, x52, x20, x17);
    var x55: u32 = undefined;
    var x56: u1 = undefined;
    addcarryxU32(&x55, &x56, x54, x18, x15);
    var x57: u32 = undefined;
    var x58: u1 = undefined;
    addcarryxU32(&x57, &x58, x56, x16, x13);
    var x59: u32 = undefined;
    var x60: u32 = undefined;
    mulxU32(&x59, &x60, x35, 0xe88fdc45);
    var x61: u32 = undefined;
    var x62: u32 = undefined;
    mulxU32(&x61, &x62, x59, 0xffffffff);
    var x63: u32 = undefined;
    var x64: u32 = undefined;
    mulxU32(&x63, &x64, x59, 0xffffffff);
    var x65: u32 = undefined;
    var x66: u32 = undefined;
    mulxU32(&x65, &x66, x59, 0xffffffff);
    var x67: u32 = undefined;
    var x68: u32 = undefined;
    mulxU32(&x67, &x68, x59, 0xffffffff);
    var x69: u32 = undefined;
    var x70: u32 = undefined;
    mulxU32(&x69, &x70, x59, 0xffffffff);
    var x71: u32 = undefined;
    var x72: u32 = undefined;
    mulxU32(&x71, &x72, x59, 0xffffffff);
    var x73: u32 = undefined;
    var x74: u32 = undefined;
    mulxU32(&x73, &x74, x59, 0xc7634d81);
    var x75: u32 = undefined;
    var x76: u32 = undefined;
    mulxU32(&x75, &x76, x59, 0xf4372ddf);
    var x77: u32 = undefined;
    var x78: u32 = undefined;
    mulxU32(&x77, &x78, x59, 0x581a0db2);
    var x79: u32 = undefined;
    var x80: u32 = undefined;
    mulxU32(&x79, &x80, x59, 0x48b0a77a);
    var x81: u32 = undefined;
    var x82: u32 = undefined;
    mulxU32(&x81, &x82, x59, 0xecec196a);
    var x83: u32 = undefined;
    var x84: u32 = undefined;
    mulxU32(&x83, &x84, x59, 0xccc52973);
    var x85: u32 = undefined;
    var x86: u1 = undefined;
    addcarryxU32(&x85, &x86, 0x0, x84, x81);
    var x87: u32 = undefined;
    var x88: u1 = undefined;
    addcarryxU32(&x87, &x88, x86, x82, x79);
    var x89: u32 = undefined;
    var x90: u1 = undefined;
    addcarryxU32(&x89, &x90, x88, x80, x77);
    var x91: u32 = undefined;
    var x92: u1 = undefined;
    addcarryxU32(&x91, &x92, x90, x78, x75);
    var x93: u32 = undefined;
    var x94: u1 = undefined;
    addcarryxU32(&x93, &x94, x92, x76, x73);
    var x95: u32 = undefined;
    var x96: u1 = undefined;
    addcarryxU32(&x95, &x96, x94, x74, x71);
    var x97: u32 = undefined;
    var x98: u1 = undefined;
    addcarryxU32(&x97, &x98, x96, x72, x69);
    var x99: u32 = undefined;
    var x100: u1 = undefined;
    addcarryxU32(&x99, &x100, x98, x70, x67);
    var x101: u32 = undefined;
    var x102: u1 = undefined;
    addcarryxU32(&x101, &x102, x100, x68, x65);
    var x103: u32 = undefined;
    var x104: u1 = undefined;
    addcarryxU32(&x103, &x104, x102, x66, x63);
    var x105: u32 = undefined;
    var x106: u1 = undefined;
    addcarryxU32(&x105, &x106, x104, x64, x61);
    var x107: u32 = undefined;
    var x108: u1 = undefined;
    addcarryxU32(&x107, &x108, 0x0, x35, x83);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    addcarryxU32(&x109, &x110, x108, x37, x85);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    addcarryxU32(&x111, &x112, x110, x39, x87);
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    addcarryxU32(&x113, &x114, x112, x41, x89);
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    addcarryxU32(&x115, &x116, x114, x43, x91);
    var x117: u32 = undefined;
    var x118: u1 = undefined;
    addcarryxU32(&x117, &x118, x116, x45, x93);
    var x119: u32 = undefined;
    var x120: u1 = undefined;
    addcarryxU32(&x119, &x120, x118, x47, x95);
    var x121: u32 = undefined;
    var x122: u1 = undefined;
    addcarryxU32(&x121, &x122, x120, x49, x97);
    var x123: u32 = undefined;
    var x124: u1 = undefined;
    addcarryxU32(&x123, &x124, x122, x51, x99);
    var x125: u32 = undefined;
    var x126: u1 = undefined;
    addcarryxU32(&x125, &x126, x124, x53, x101);
    var x127: u32 = undefined;
    var x128: u1 = undefined;
    addcarryxU32(&x127, &x128, x126, x55, x103);
    var x129: u32 = undefined;
    var x130: u1 = undefined;
    addcarryxU32(&x129, &x130, x128, x57, x105);
    var x131: u32 = undefined;
    var x132: u1 = undefined;
    addcarryxU32(&x131, &x132, x130, (cast(u32, x58) + x14), (cast(u32, x106) + x62));
    var x133: u32 = undefined;
    var x134: u32 = undefined;
    mulxU32(&x133, &x134, x1, 0xc84ee01);
    var x135: u32 = undefined;
    var x136: u32 = undefined;
    mulxU32(&x135, &x136, x1, 0x2b39bf21);
    var x137: u32 = undefined;
    var x138: u32 = undefined;
    mulxU32(&x137, &x138, x1, 0x3fb05b7a);
    var x139: u32 = undefined;
    var x140: u32 = undefined;
    mulxU32(&x139, &x140, x1, 0x28266895);
    var x141: u32 = undefined;
    var x142: u32 = undefined;
    mulxU32(&x141, &x142, x1, 0xd40d4917);
    var x143: u32 = undefined;
    var x144: u32 = undefined;
    mulxU32(&x143, &x144, x1, 0x4aab1cc5);
    var x145: u32 = undefined;
    var x146: u32 = undefined;
    mulxU32(&x145, &x146, x1, 0xbc3e483a);
    var x147: u32 = undefined;
    var x148: u32 = undefined;
    mulxU32(&x147, &x148, x1, 0xfcb82947);
    var x149: u32 = undefined;
    var x150: u32 = undefined;
    mulxU32(&x149, &x150, x1, 0xff3d81e5);
    var x151: u32 = undefined;
    var x152: u32 = undefined;
    mulxU32(&x151, &x152, x1, 0xdf1aa419);
    var x153: u32 = undefined;
    var x154: u32 = undefined;
    mulxU32(&x153, &x154, x1, 0x2d319b24);
    var x155: u32 = undefined;
    var x156: u32 = undefined;
    mulxU32(&x155, &x156, x1, 0x19b409a9);
    var x157: u32 = undefined;
    var x158: u1 = undefined;
    addcarryxU32(&x157, &x158, 0x0, x156, x153);
    var x159: u32 = undefined;
    var x160: u1 = undefined;
    addcarryxU32(&x159, &x160, x158, x154, x151);
    var x161: u32 = undefined;
    var x162: u1 = undefined;
    addcarryxU32(&x161, &x162, x160, x152, x149);
    var x163: u32 = undefined;
    var x164: u1 = undefined;
    addcarryxU32(&x163, &x164, x162, x150, x147);
    var x165: u32 = undefined;
    var x166: u1 = undefined;
    addcarryxU32(&x165, &x166, x164, x148, x145);
    var x167: u32 = undefined;
    var x168: u1 = undefined;
    addcarryxU32(&x167, &x168, x166, x146, x143);
    var x169: u32 = undefined;
    var x170: u1 = undefined;
    addcarryxU32(&x169, &x170, x168, x144, x141);
    var x171: u32 = undefined;
    var x172: u1 = undefined;
    addcarryxU32(&x171, &x172, x170, x142, x139);
    var x173: u32 = undefined;
    var x174: u1 = undefined;
    addcarryxU32(&x173, &x174, x172, x140, x137);
    var x175: u32 = undefined;
    var x176: u1 = undefined;
    addcarryxU32(&x175, &x176, x174, x138, x135);
    var x177: u32 = undefined;
    var x178: u1 = undefined;
    addcarryxU32(&x177, &x178, x176, x136, x133);
    var x179: u32 = undefined;
    var x180: u1 = undefined;
    addcarryxU32(&x179, &x180, 0x0, x109, x155);
    var x181: u32 = undefined;
    var x182: u1 = undefined;
    addcarryxU32(&x181, &x182, x180, x111, x157);
    var x183: u32 = undefined;
    var x184: u1 = undefined;
    addcarryxU32(&x183, &x184, x182, x113, x159);
    var x185: u32 = undefined;
    var x186: u1 = undefined;
    addcarryxU32(&x185, &x186, x184, x115, x161);
    var x187: u32 = undefined;
    var x188: u1 = undefined;
    addcarryxU32(&x187, &x188, x186, x117, x163);
    var x189: u32 = undefined;
    var x190: u1 = undefined;
    addcarryxU32(&x189, &x190, x188, x119, x165);
    var x191: u32 = undefined;
    var x192: u1 = undefined;
    addcarryxU32(&x191, &x192, x190, x121, x167);
    var x193: u32 = undefined;
    var x194: u1 = undefined;
    addcarryxU32(&x193, &x194, x192, x123, x169);
    var x195: u32 = undefined;
    var x196: u1 = undefined;
    addcarryxU32(&x195, &x196, x194, x125, x171);
    var x197: u32 = undefined;
    var x198: u1 = undefined;
    addcarryxU32(&x197, &x198, x196, x127, x173);
    var x199: u32 = undefined;
    var x200: u1 = undefined;
    addcarryxU32(&x199, &x200, x198, x129, x175);
    var x201: u32 = undefined;
    var x202: u1 = undefined;
    addcarryxU32(&x201, &x202, x200, x131, x177);
    var x203: u32 = undefined;
    var x204: u32 = undefined;
    mulxU32(&x203, &x204, x179, 0xe88fdc45);
    var x205: u32 = undefined;
    var x206: u32 = undefined;
    mulxU32(&x205, &x206, x203, 0xffffffff);
    var x207: u32 = undefined;
    var x208: u32 = undefined;
    mulxU32(&x207, &x208, x203, 0xffffffff);
    var x209: u32 = undefined;
    var x210: u32 = undefined;
    mulxU32(&x209, &x210, x203, 0xffffffff);
    var x211: u32 = undefined;
    var x212: u32 = undefined;
    mulxU32(&x211, &x212, x203, 0xffffffff);
    var x213: u32 = undefined;
    var x214: u32 = undefined;
    mulxU32(&x213, &x214, x203, 0xffffffff);
    var x215: u32 = undefined;
    var x216: u32 = undefined;
    mulxU32(&x215, &x216, x203, 0xffffffff);
    var x217: u32 = undefined;
    var x218: u32 = undefined;
    mulxU32(&x217, &x218, x203, 0xc7634d81);
    var x219: u32 = undefined;
    var x220: u32 = undefined;
    mulxU32(&x219, &x220, x203, 0xf4372ddf);
    var x221: u32 = undefined;
    var x222: u32 = undefined;
    mulxU32(&x221, &x222, x203, 0x581a0db2);
    var x223: u32 = undefined;
    var x224: u32 = undefined;
    mulxU32(&x223, &x224, x203, 0x48b0a77a);
    var x225: u32 = undefined;
    var x226: u32 = undefined;
    mulxU32(&x225, &x226, x203, 0xecec196a);
    var x227: u32 = undefined;
    var x228: u32 = undefined;
    mulxU32(&x227, &x228, x203, 0xccc52973);
    var x229: u32 = undefined;
    var x230: u1 = undefined;
    addcarryxU32(&x229, &x230, 0x0, x228, x225);
    var x231: u32 = undefined;
    var x232: u1 = undefined;
    addcarryxU32(&x231, &x232, x230, x226, x223);
    var x233: u32 = undefined;
    var x234: u1 = undefined;
    addcarryxU32(&x233, &x234, x232, x224, x221);
    var x235: u32 = undefined;
    var x236: u1 = undefined;
    addcarryxU32(&x235, &x236, x234, x222, x219);
    var x237: u32 = undefined;
    var x238: u1 = undefined;
    addcarryxU32(&x237, &x238, x236, x220, x217);
    var x239: u32 = undefined;
    var x240: u1 = undefined;
    addcarryxU32(&x239, &x240, x238, x218, x215);
    var x241: u32 = undefined;
    var x242: u1 = undefined;
    addcarryxU32(&x241, &x242, x240, x216, x213);
    var x243: u32 = undefined;
    var x244: u1 = undefined;
    addcarryxU32(&x243, &x244, x242, x214, x211);
    var x245: u32 = undefined;
    var x246: u1 = undefined;
    addcarryxU32(&x245, &x246, x244, x212, x209);
    var x247: u32 = undefined;
    var x248: u1 = undefined;
    addcarryxU32(&x247, &x248, x246, x210, x207);
    var x249: u32 = undefined;
    var x250: u1 = undefined;
    addcarryxU32(&x249, &x250, x248, x208, x205);
    var x251: u32 = undefined;
    var x252: u1 = undefined;
    addcarryxU32(&x251, &x252, 0x0, x179, x227);
    var x253: u32 = undefined;
    var x254: u1 = undefined;
    addcarryxU32(&x253, &x254, x252, x181, x229);
    var x255: u32 = undefined;
    var x256: u1 = undefined;
    addcarryxU32(&x255, &x256, x254, x183, x231);
    var x257: u32 = undefined;
    var x258: u1 = undefined;
    addcarryxU32(&x257, &x258, x256, x185, x233);
    var x259: u32 = undefined;
    var x260: u1 = undefined;
    addcarryxU32(&x259, &x260, x258, x187, x235);
    var x261: u32 = undefined;
    var x262: u1 = undefined;
    addcarryxU32(&x261, &x262, x260, x189, x237);
    var x263: u32 = undefined;
    var x264: u1 = undefined;
    addcarryxU32(&x263, &x264, x262, x191, x239);
    var x265: u32 = undefined;
    var x266: u1 = undefined;
    addcarryxU32(&x265, &x266, x264, x193, x241);
    var x267: u32 = undefined;
    var x268: u1 = undefined;
    addcarryxU32(&x267, &x268, x266, x195, x243);
    var x269: u32 = undefined;
    var x270: u1 = undefined;
    addcarryxU32(&x269, &x270, x268, x197, x245);
    var x271: u32 = undefined;
    var x272: u1 = undefined;
    addcarryxU32(&x271, &x272, x270, x199, x247);
    var x273: u32 = undefined;
    var x274: u1 = undefined;
    addcarryxU32(&x273, &x274, x272, x201, x249);
    var x275: u32 = undefined;
    var x276: u1 = undefined;
    addcarryxU32(&x275, &x276, x274, ((cast(u32, x202) + cast(u32, x132)) + (cast(u32, x178) + x134)), (cast(u32, x250) + x206));
    var x277: u32 = undefined;
    var x278: u32 = undefined;
    mulxU32(&x277, &x278, x2, 0xc84ee01);
    var x279: u32 = undefined;
    var x280: u32 = undefined;
    mulxU32(&x279, &x280, x2, 0x2b39bf21);
    var x281: u32 = undefined;
    var x282: u32 = undefined;
    mulxU32(&x281, &x282, x2, 0x3fb05b7a);
    var x283: u32 = undefined;
    var x284: u32 = undefined;
    mulxU32(&x283, &x284, x2, 0x28266895);
    var x285: u32 = undefined;
    var x286: u32 = undefined;
    mulxU32(&x285, &x286, x2, 0xd40d4917);
    var x287: u32 = undefined;
    var x288: u32 = undefined;
    mulxU32(&x287, &x288, x2, 0x4aab1cc5);
    var x289: u32 = undefined;
    var x290: u32 = undefined;
    mulxU32(&x289, &x290, x2, 0xbc3e483a);
    var x291: u32 = undefined;
    var x292: u32 = undefined;
    mulxU32(&x291, &x292, x2, 0xfcb82947);
    var x293: u32 = undefined;
    var x294: u32 = undefined;
    mulxU32(&x293, &x294, x2, 0xff3d81e5);
    var x295: u32 = undefined;
    var x296: u32 = undefined;
    mulxU32(&x295, &x296, x2, 0xdf1aa419);
    var x297: u32 = undefined;
    var x298: u32 = undefined;
    mulxU32(&x297, &x298, x2, 0x2d319b24);
    var x299: u32 = undefined;
    var x300: u32 = undefined;
    mulxU32(&x299, &x300, x2, 0x19b409a9);
    var x301: u32 = undefined;
    var x302: u1 = undefined;
    addcarryxU32(&x301, &x302, 0x0, x300, x297);
    var x303: u32 = undefined;
    var x304: u1 = undefined;
    addcarryxU32(&x303, &x304, x302, x298, x295);
    var x305: u32 = undefined;
    var x306: u1 = undefined;
    addcarryxU32(&x305, &x306, x304, x296, x293);
    var x307: u32 = undefined;
    var x308: u1 = undefined;
    addcarryxU32(&x307, &x308, x306, x294, x291);
    var x309: u32 = undefined;
    var x310: u1 = undefined;
    addcarryxU32(&x309, &x310, x308, x292, x289);
    var x311: u32 = undefined;
    var x312: u1 = undefined;
    addcarryxU32(&x311, &x312, x310, x290, x287);
    var x313: u32 = undefined;
    var x314: u1 = undefined;
    addcarryxU32(&x313, &x314, x312, x288, x285);
    var x315: u32 = undefined;
    var x316: u1 = undefined;
    addcarryxU32(&x315, &x316, x314, x286, x283);
    var x317: u32 = undefined;
    var x318: u1 = undefined;
    addcarryxU32(&x317, &x318, x316, x284, x281);
    var x319: u32 = undefined;
    var x320: u1 = undefined;
    addcarryxU32(&x319, &x320, x318, x282, x279);
    var x321: u32 = undefined;
    var x322: u1 = undefined;
    addcarryxU32(&x321, &x322, x320, x280, x277);
    var x323: u32 = undefined;
    var x324: u1 = undefined;
    addcarryxU32(&x323, &x324, 0x0, x253, x299);
    var x325: u32 = undefined;
    var x326: u1 = undefined;
    addcarryxU32(&x325, &x326, x324, x255, x301);
    var x327: u32 = undefined;
    var x328: u1 = undefined;
    addcarryxU32(&x327, &x328, x326, x257, x303);
    var x329: u32 = undefined;
    var x330: u1 = undefined;
    addcarryxU32(&x329, &x330, x328, x259, x305);
    var x331: u32 = undefined;
    var x332: u1 = undefined;
    addcarryxU32(&x331, &x332, x330, x261, x307);
    var x333: u32 = undefined;
    var x334: u1 = undefined;
    addcarryxU32(&x333, &x334, x332, x263, x309);
    var x335: u32 = undefined;
    var x336: u1 = undefined;
    addcarryxU32(&x335, &x336, x334, x265, x311);
    var x337: u32 = undefined;
    var x338: u1 = undefined;
    addcarryxU32(&x337, &x338, x336, x267, x313);
    var x339: u32 = undefined;
    var x340: u1 = undefined;
    addcarryxU32(&x339, &x340, x338, x269, x315);
    var x341: u32 = undefined;
    var x342: u1 = undefined;
    addcarryxU32(&x341, &x342, x340, x271, x317);
    var x343: u32 = undefined;
    var x344: u1 = undefined;
    addcarryxU32(&x343, &x344, x342, x273, x319);
    var x345: u32 = undefined;
    var x346: u1 = undefined;
    addcarryxU32(&x345, &x346, x344, x275, x321);
    var x347: u32 = undefined;
    var x348: u32 = undefined;
    mulxU32(&x347, &x348, x323, 0xe88fdc45);
    var x349: u32 = undefined;
    var x350: u32 = undefined;
    mulxU32(&x349, &x350, x347, 0xffffffff);
    var x351: u32 = undefined;
    var x352: u32 = undefined;
    mulxU32(&x351, &x352, x347, 0xffffffff);
    var x353: u32 = undefined;
    var x354: u32 = undefined;
    mulxU32(&x353, &x354, x347, 0xffffffff);
    var x355: u32 = undefined;
    var x356: u32 = undefined;
    mulxU32(&x355, &x356, x347, 0xffffffff);
    var x357: u32 = undefined;
    var x358: u32 = undefined;
    mulxU32(&x357, &x358, x347, 0xffffffff);
    var x359: u32 = undefined;
    var x360: u32 = undefined;
    mulxU32(&x359, &x360, x347, 0xffffffff);
    var x361: u32 = undefined;
    var x362: u32 = undefined;
    mulxU32(&x361, &x362, x347, 0xc7634d81);
    var x363: u32 = undefined;
    var x364: u32 = undefined;
    mulxU32(&x363, &x364, x347, 0xf4372ddf);
    var x365: u32 = undefined;
    var x366: u32 = undefined;
    mulxU32(&x365, &x366, x347, 0x581a0db2);
    var x367: u32 = undefined;
    var x368: u32 = undefined;
    mulxU32(&x367, &x368, x347, 0x48b0a77a);
    var x369: u32 = undefined;
    var x370: u32 = undefined;
    mulxU32(&x369, &x370, x347, 0xecec196a);
    var x371: u32 = undefined;
    var x372: u32 = undefined;
    mulxU32(&x371, &x372, x347, 0xccc52973);
    var x373: u32 = undefined;
    var x374: u1 = undefined;
    addcarryxU32(&x373, &x374, 0x0, x372, x369);
    var x375: u32 = undefined;
    var x376: u1 = undefined;
    addcarryxU32(&x375, &x376, x374, x370, x367);
    var x377: u32 = undefined;
    var x378: u1 = undefined;
    addcarryxU32(&x377, &x378, x376, x368, x365);
    var x379: u32 = undefined;
    var x380: u1 = undefined;
    addcarryxU32(&x379, &x380, x378, x366, x363);
    var x381: u32 = undefined;
    var x382: u1 = undefined;
    addcarryxU32(&x381, &x382, x380, x364, x361);
    var x383: u32 = undefined;
    var x384: u1 = undefined;
    addcarryxU32(&x383, &x384, x382, x362, x359);
    var x385: u32 = undefined;
    var x386: u1 = undefined;
    addcarryxU32(&x385, &x386, x384, x360, x357);
    var x387: u32 = undefined;
    var x388: u1 = undefined;
    addcarryxU32(&x387, &x388, x386, x358, x355);
    var x389: u32 = undefined;
    var x390: u1 = undefined;
    addcarryxU32(&x389, &x390, x388, x356, x353);
    var x391: u32 = undefined;
    var x392: u1 = undefined;
    addcarryxU32(&x391, &x392, x390, x354, x351);
    var x393: u32 = undefined;
    var x394: u1 = undefined;
    addcarryxU32(&x393, &x394, x392, x352, x349);
    var x395: u32 = undefined;
    var x396: u1 = undefined;
    addcarryxU32(&x395, &x396, 0x0, x323, x371);
    var x397: u32 = undefined;
    var x398: u1 = undefined;
    addcarryxU32(&x397, &x398, x396, x325, x373);
    var x399: u32 = undefined;
    var x400: u1 = undefined;
    addcarryxU32(&x399, &x400, x398, x327, x375);
    var x401: u32 = undefined;
    var x402: u1 = undefined;
    addcarryxU32(&x401, &x402, x400, x329, x377);
    var x403: u32 = undefined;
    var x404: u1 = undefined;
    addcarryxU32(&x403, &x404, x402, x331, x379);
    var x405: u32 = undefined;
    var x406: u1 = undefined;
    addcarryxU32(&x405, &x406, x404, x333, x381);
    var x407: u32 = undefined;
    var x408: u1 = undefined;
    addcarryxU32(&x407, &x408, x406, x335, x383);
    var x409: u32 = undefined;
    var x410: u1 = undefined;
    addcarryxU32(&x409, &x410, x408, x337, x385);
    var x411: u32 = undefined;
    var x412: u1 = undefined;
    addcarryxU32(&x411, &x412, x410, x339, x387);
    var x413: u32 = undefined;
    var x414: u1 = undefined;
    addcarryxU32(&x413, &x414, x412, x341, x389);
    var x415: u32 = undefined;
    var x416: u1 = undefined;
    addcarryxU32(&x415, &x416, x414, x343, x391);
    var x417: u32 = undefined;
    var x418: u1 = undefined;
    addcarryxU32(&x417, &x418, x416, x345, x393);
    var x419: u32 = undefined;
    var x420: u1 = undefined;
    addcarryxU32(&x419, &x420, x418, ((cast(u32, x346) + cast(u32, x276)) + (cast(u32, x322) + x278)), (cast(u32, x394) + x350));
    var x421: u32 = undefined;
    var x422: u32 = undefined;
    mulxU32(&x421, &x422, x3, 0xc84ee01);
    var x423: u32 = undefined;
    var x424: u32 = undefined;
    mulxU32(&x423, &x424, x3, 0x2b39bf21);
    var x425: u32 = undefined;
    var x426: u32 = undefined;
    mulxU32(&x425, &x426, x3, 0x3fb05b7a);
    var x427: u32 = undefined;
    var x428: u32 = undefined;
    mulxU32(&x427, &x428, x3, 0x28266895);
    var x429: u32 = undefined;
    var x430: u32 = undefined;
    mulxU32(&x429, &x430, x3, 0xd40d4917);
    var x431: u32 = undefined;
    var x432: u32 = undefined;
    mulxU32(&x431, &x432, x3, 0x4aab1cc5);
    var x433: u32 = undefined;
    var x434: u32 = undefined;
    mulxU32(&x433, &x434, x3, 0xbc3e483a);
    var x435: u32 = undefined;
    var x436: u32 = undefined;
    mulxU32(&x435, &x436, x3, 0xfcb82947);
    var x437: u32 = undefined;
    var x438: u32 = undefined;
    mulxU32(&x437, &x438, x3, 0xff3d81e5);
    var x439: u32 = undefined;
    var x440: u32 = undefined;
    mulxU32(&x439, &x440, x3, 0xdf1aa419);
    var x441: u32 = undefined;
    var x442: u32 = undefined;
    mulxU32(&x441, &x442, x3, 0x2d319b24);
    var x443: u32 = undefined;
    var x444: u32 = undefined;
    mulxU32(&x443, &x444, x3, 0x19b409a9);
    var x445: u32 = undefined;
    var x446: u1 = undefined;
    addcarryxU32(&x445, &x446, 0x0, x444, x441);
    var x447: u32 = undefined;
    var x448: u1 = undefined;
    addcarryxU32(&x447, &x448, x446, x442, x439);
    var x449: u32 = undefined;
    var x450: u1 = undefined;
    addcarryxU32(&x449, &x450, x448, x440, x437);
    var x451: u32 = undefined;
    var x452: u1 = undefined;
    addcarryxU32(&x451, &x452, x450, x438, x435);
    var x453: u32 = undefined;
    var x454: u1 = undefined;
    addcarryxU32(&x453, &x454, x452, x436, x433);
    var x455: u32 = undefined;
    var x456: u1 = undefined;
    addcarryxU32(&x455, &x456, x454, x434, x431);
    var x457: u32 = undefined;
    var x458: u1 = undefined;
    addcarryxU32(&x457, &x458, x456, x432, x429);
    var x459: u32 = undefined;
    var x460: u1 = undefined;
    addcarryxU32(&x459, &x460, x458, x430, x427);
    var x461: u32 = undefined;
    var x462: u1 = undefined;
    addcarryxU32(&x461, &x462, x460, x428, x425);
    var x463: u32 = undefined;
    var x464: u1 = undefined;
    addcarryxU32(&x463, &x464, x462, x426, x423);
    var x465: u32 = undefined;
    var x466: u1 = undefined;
    addcarryxU32(&x465, &x466, x464, x424, x421);
    var x467: u32 = undefined;
    var x468: u1 = undefined;
    addcarryxU32(&x467, &x468, 0x0, x397, x443);
    var x469: u32 = undefined;
    var x470: u1 = undefined;
    addcarryxU32(&x469, &x470, x468, x399, x445);
    var x471: u32 = undefined;
    var x472: u1 = undefined;
    addcarryxU32(&x471, &x472, x470, x401, x447);
    var x473: u32 = undefined;
    var x474: u1 = undefined;
    addcarryxU32(&x473, &x474, x472, x403, x449);
    var x475: u32 = undefined;
    var x476: u1 = undefined;
    addcarryxU32(&x475, &x476, x474, x405, x451);
    var x477: u32 = undefined;
    var x478: u1 = undefined;
    addcarryxU32(&x477, &x478, x476, x407, x453);
    var x479: u32 = undefined;
    var x480: u1 = undefined;
    addcarryxU32(&x479, &x480, x478, x409, x455);
    var x481: u32 = undefined;
    var x482: u1 = undefined;
    addcarryxU32(&x481, &x482, x480, x411, x457);
    var x483: u32 = undefined;
    var x484: u1 = undefined;
    addcarryxU32(&x483, &x484, x482, x413, x459);
    var x485: u32 = undefined;
    var x486: u1 = undefined;
    addcarryxU32(&x485, &x486, x484, x415, x461);
    var x487: u32 = undefined;
    var x488: u1 = undefined;
    addcarryxU32(&x487, &x488, x486, x417, x463);
    var x489: u32 = undefined;
    var x490: u1 = undefined;
    addcarryxU32(&x489, &x490, x488, x419, x465);
    var x491: u32 = undefined;
    var x492: u32 = undefined;
    mulxU32(&x491, &x492, x467, 0xe88fdc45);
    var x493: u32 = undefined;
    var x494: u32 = undefined;
    mulxU32(&x493, &x494, x491, 0xffffffff);
    var x495: u32 = undefined;
    var x496: u32 = undefined;
    mulxU32(&x495, &x496, x491, 0xffffffff);
    var x497: u32 = undefined;
    var x498: u32 = undefined;
    mulxU32(&x497, &x498, x491, 0xffffffff);
    var x499: u32 = undefined;
    var x500: u32 = undefined;
    mulxU32(&x499, &x500, x491, 0xffffffff);
    var x501: u32 = undefined;
    var x502: u32 = undefined;
    mulxU32(&x501, &x502, x491, 0xffffffff);
    var x503: u32 = undefined;
    var x504: u32 = undefined;
    mulxU32(&x503, &x504, x491, 0xffffffff);
    var x505: u32 = undefined;
    var x506: u32 = undefined;
    mulxU32(&x505, &x506, x491, 0xc7634d81);
    var x507: u32 = undefined;
    var x508: u32 = undefined;
    mulxU32(&x507, &x508, x491, 0xf4372ddf);
    var x509: u32 = undefined;
    var x510: u32 = undefined;
    mulxU32(&x509, &x510, x491, 0x581a0db2);
    var x511: u32 = undefined;
    var x512: u32 = undefined;
    mulxU32(&x511, &x512, x491, 0x48b0a77a);
    var x513: u32 = undefined;
    var x514: u32 = undefined;
    mulxU32(&x513, &x514, x491, 0xecec196a);
    var x515: u32 = undefined;
    var x516: u32 = undefined;
    mulxU32(&x515, &x516, x491, 0xccc52973);
    var x517: u32 = undefined;
    var x518: u1 = undefined;
    addcarryxU32(&x517, &x518, 0x0, x516, x513);
    var x519: u32 = undefined;
    var x520: u1 = undefined;
    addcarryxU32(&x519, &x520, x518, x514, x511);
    var x521: u32 = undefined;
    var x522: u1 = undefined;
    addcarryxU32(&x521, &x522, x520, x512, x509);
    var x523: u32 = undefined;
    var x524: u1 = undefined;
    addcarryxU32(&x523, &x524, x522, x510, x507);
    var x525: u32 = undefined;
    var x526: u1 = undefined;
    addcarryxU32(&x525, &x526, x524, x508, x505);
    var x527: u32 = undefined;
    var x528: u1 = undefined;
    addcarryxU32(&x527, &x528, x526, x506, x503);
    var x529: u32 = undefined;
    var x530: u1 = undefined;
    addcarryxU32(&x529, &x530, x528, x504, x501);
    var x531: u32 = undefined;
    var x532: u1 = undefined;
    addcarryxU32(&x531, &x532, x530, x502, x499);
    var x533: u32 = undefined;
    var x534: u1 = undefined;
    addcarryxU32(&x533, &x534, x532, x500, x497);
    var x535: u32 = undefined;
    var x536: u1 = undefined;
    addcarryxU32(&x535, &x536, x534, x498, x495);
    var x537: u32 = undefined;
    var x538: u1 = undefined;
    addcarryxU32(&x537, &x538, x536, x496, x493);
    var x539: u32 = undefined;
    var x540: u1 = undefined;
    addcarryxU32(&x539, &x540, 0x0, x467, x515);
    var x541: u32 = undefined;
    var x542: u1 = undefined;
    addcarryxU32(&x541, &x542, x540, x469, x517);
    var x543: u32 = undefined;
    var x544: u1 = undefined;
    addcarryxU32(&x543, &x544, x542, x471, x519);
    var x545: u32 = undefined;
    var x546: u1 = undefined;
    addcarryxU32(&x545, &x546, x544, x473, x521);
    var x547: u32 = undefined;
    var x548: u1 = undefined;
    addcarryxU32(&x547, &x548, x546, x475, x523);
    var x549: u32 = undefined;
    var x550: u1 = undefined;
    addcarryxU32(&x549, &x550, x548, x477, x525);
    var x551: u32 = undefined;
    var x552: u1 = undefined;
    addcarryxU32(&x551, &x552, x550, x479, x527);
    var x553: u32 = undefined;
    var x554: u1 = undefined;
    addcarryxU32(&x553, &x554, x552, x481, x529);
    var x555: u32 = undefined;
    var x556: u1 = undefined;
    addcarryxU32(&x555, &x556, x554, x483, x531);
    var x557: u32 = undefined;
    var x558: u1 = undefined;
    addcarryxU32(&x557, &x558, x556, x485, x533);
    var x559: u32 = undefined;
    var x560: u1 = undefined;
    addcarryxU32(&x559, &x560, x558, x487, x535);
    var x561: u32 = undefined;
    var x562: u1 = undefined;
    addcarryxU32(&x561, &x562, x560, x489, x537);
    var x563: u32 = undefined;
    var x564: u1 = undefined;
    addcarryxU32(&x563, &x564, x562, ((cast(u32, x490) + cast(u32, x420)) + (cast(u32, x466) + x422)), (cast(u32, x538) + x494));
    var x565: u32 = undefined;
    var x566: u32 = undefined;
    mulxU32(&x565, &x566, x4, 0xc84ee01);
    var x567: u32 = undefined;
    var x568: u32 = undefined;
    mulxU32(&x567, &x568, x4, 0x2b39bf21);
    var x569: u32 = undefined;
    var x570: u32 = undefined;
    mulxU32(&x569, &x570, x4, 0x3fb05b7a);
    var x571: u32 = undefined;
    var x572: u32 = undefined;
    mulxU32(&x571, &x572, x4, 0x28266895);
    var x573: u32 = undefined;
    var x574: u32 = undefined;
    mulxU32(&x573, &x574, x4, 0xd40d4917);
    var x575: u32 = undefined;
    var x576: u32 = undefined;
    mulxU32(&x575, &x576, x4, 0x4aab1cc5);
    var x577: u32 = undefined;
    var x578: u32 = undefined;
    mulxU32(&x577, &x578, x4, 0xbc3e483a);
    var x579: u32 = undefined;
    var x580: u32 = undefined;
    mulxU32(&x579, &x580, x4, 0xfcb82947);
    var x581: u32 = undefined;
    var x582: u32 = undefined;
    mulxU32(&x581, &x582, x4, 0xff3d81e5);
    var x583: u32 = undefined;
    var x584: u32 = undefined;
    mulxU32(&x583, &x584, x4, 0xdf1aa419);
    var x585: u32 = undefined;
    var x586: u32 = undefined;
    mulxU32(&x585, &x586, x4, 0x2d319b24);
    var x587: u32 = undefined;
    var x588: u32 = undefined;
    mulxU32(&x587, &x588, x4, 0x19b409a9);
    var x589: u32 = undefined;
    var x590: u1 = undefined;
    addcarryxU32(&x589, &x590, 0x0, x588, x585);
    var x591: u32 = undefined;
    var x592: u1 = undefined;
    addcarryxU32(&x591, &x592, x590, x586, x583);
    var x593: u32 = undefined;
    var x594: u1 = undefined;
    addcarryxU32(&x593, &x594, x592, x584, x581);
    var x595: u32 = undefined;
    var x596: u1 = undefined;
    addcarryxU32(&x595, &x596, x594, x582, x579);
    var x597: u32 = undefined;
    var x598: u1 = undefined;
    addcarryxU32(&x597, &x598, x596, x580, x577);
    var x599: u32 = undefined;
    var x600: u1 = undefined;
    addcarryxU32(&x599, &x600, x598, x578, x575);
    var x601: u32 = undefined;
    var x602: u1 = undefined;
    addcarryxU32(&x601, &x602, x600, x576, x573);
    var x603: u32 = undefined;
    var x604: u1 = undefined;
    addcarryxU32(&x603, &x604, x602, x574, x571);
    var x605: u32 = undefined;
    var x606: u1 = undefined;
    addcarryxU32(&x605, &x606, x604, x572, x569);
    var x607: u32 = undefined;
    var x608: u1 = undefined;
    addcarryxU32(&x607, &x608, x606, x570, x567);
    var x609: u32 = undefined;
    var x610: u1 = undefined;
    addcarryxU32(&x609, &x610, x608, x568, x565);
    var x611: u32 = undefined;
    var x612: u1 = undefined;
    addcarryxU32(&x611, &x612, 0x0, x541, x587);
    var x613: u32 = undefined;
    var x614: u1 = undefined;
    addcarryxU32(&x613, &x614, x612, x543, x589);
    var x615: u32 = undefined;
    var x616: u1 = undefined;
    addcarryxU32(&x615, &x616, x614, x545, x591);
    var x617: u32 = undefined;
    var x618: u1 = undefined;
    addcarryxU32(&x617, &x618, x616, x547, x593);
    var x619: u32 = undefined;
    var x620: u1 = undefined;
    addcarryxU32(&x619, &x620, x618, x549, x595);
    var x621: u32 = undefined;
    var x622: u1 = undefined;
    addcarryxU32(&x621, &x622, x620, x551, x597);
    var x623: u32 = undefined;
    var x624: u1 = undefined;
    addcarryxU32(&x623, &x624, x622, x553, x599);
    var x625: u32 = undefined;
    var x626: u1 = undefined;
    addcarryxU32(&x625, &x626, x624, x555, x601);
    var x627: u32 = undefined;
    var x628: u1 = undefined;
    addcarryxU32(&x627, &x628, x626, x557, x603);
    var x629: u32 = undefined;
    var x630: u1 = undefined;
    addcarryxU32(&x629, &x630, x628, x559, x605);
    var x631: u32 = undefined;
    var x632: u1 = undefined;
    addcarryxU32(&x631, &x632, x630, x561, x607);
    var x633: u32 = undefined;
    var x634: u1 = undefined;
    addcarryxU32(&x633, &x634, x632, x563, x609);
    var x635: u32 = undefined;
    var x636: u32 = undefined;
    mulxU32(&x635, &x636, x611, 0xe88fdc45);
    var x637: u32 = undefined;
    var x638: u32 = undefined;
    mulxU32(&x637, &x638, x635, 0xffffffff);
    var x639: u32 = undefined;
    var x640: u32 = undefined;
    mulxU32(&x639, &x640, x635, 0xffffffff);
    var x641: u32 = undefined;
    var x642: u32 = undefined;
    mulxU32(&x641, &x642, x635, 0xffffffff);
    var x643: u32 = undefined;
    var x644: u32 = undefined;
    mulxU32(&x643, &x644, x635, 0xffffffff);
    var x645: u32 = undefined;
    var x646: u32 = undefined;
    mulxU32(&x645, &x646, x635, 0xffffffff);
    var x647: u32 = undefined;
    var x648: u32 = undefined;
    mulxU32(&x647, &x648, x635, 0xffffffff);
    var x649: u32 = undefined;
    var x650: u32 = undefined;
    mulxU32(&x649, &x650, x635, 0xc7634d81);
    var x651: u32 = undefined;
    var x652: u32 = undefined;
    mulxU32(&x651, &x652, x635, 0xf4372ddf);
    var x653: u32 = undefined;
    var x654: u32 = undefined;
    mulxU32(&x653, &x654, x635, 0x581a0db2);
    var x655: u32 = undefined;
    var x656: u32 = undefined;
    mulxU32(&x655, &x656, x635, 0x48b0a77a);
    var x657: u32 = undefined;
    var x658: u32 = undefined;
    mulxU32(&x657, &x658, x635, 0xecec196a);
    var x659: u32 = undefined;
    var x660: u32 = undefined;
    mulxU32(&x659, &x660, x635, 0xccc52973);
    var x661: u32 = undefined;
    var x662: u1 = undefined;
    addcarryxU32(&x661, &x662, 0x0, x660, x657);
    var x663: u32 = undefined;
    var x664: u1 = undefined;
    addcarryxU32(&x663, &x664, x662, x658, x655);
    var x665: u32 = undefined;
    var x666: u1 = undefined;
    addcarryxU32(&x665, &x666, x664, x656, x653);
    var x667: u32 = undefined;
    var x668: u1 = undefined;
    addcarryxU32(&x667, &x668, x666, x654, x651);
    var x669: u32 = undefined;
    var x670: u1 = undefined;
    addcarryxU32(&x669, &x670, x668, x652, x649);
    var x671: u32 = undefined;
    var x672: u1 = undefined;
    addcarryxU32(&x671, &x672, x670, x650, x647);
    var x673: u32 = undefined;
    var x674: u1 = undefined;
    addcarryxU32(&x673, &x674, x672, x648, x645);
    var x675: u32 = undefined;
    var x676: u1 = undefined;
    addcarryxU32(&x675, &x676, x674, x646, x643);
    var x677: u32 = undefined;
    var x678: u1 = undefined;
    addcarryxU32(&x677, &x678, x676, x644, x641);
    var x679: u32 = undefined;
    var x680: u1 = undefined;
    addcarryxU32(&x679, &x680, x678, x642, x639);
    var x681: u32 = undefined;
    var x682: u1 = undefined;
    addcarryxU32(&x681, &x682, x680, x640, x637);
    var x683: u32 = undefined;
    var x684: u1 = undefined;
    addcarryxU32(&x683, &x684, 0x0, x611, x659);
    var x685: u32 = undefined;
    var x686: u1 = undefined;
    addcarryxU32(&x685, &x686, x684, x613, x661);
    var x687: u32 = undefined;
    var x688: u1 = undefined;
    addcarryxU32(&x687, &x688, x686, x615, x663);
    var x689: u32 = undefined;
    var x690: u1 = undefined;
    addcarryxU32(&x689, &x690, x688, x617, x665);
    var x691: u32 = undefined;
    var x692: u1 = undefined;
    addcarryxU32(&x691, &x692, x690, x619, x667);
    var x693: u32 = undefined;
    var x694: u1 = undefined;
    addcarryxU32(&x693, &x694, x692, x621, x669);
    var x695: u32 = undefined;
    var x696: u1 = undefined;
    addcarryxU32(&x695, &x696, x694, x623, x671);
    var x697: u32 = undefined;
    var x698: u1 = undefined;
    addcarryxU32(&x697, &x698, x696, x625, x673);
    var x699: u32 = undefined;
    var x700: u1 = undefined;
    addcarryxU32(&x699, &x700, x698, x627, x675);
    var x701: u32 = undefined;
    var x702: u1 = undefined;
    addcarryxU32(&x701, &x702, x700, x629, x677);
    var x703: u32 = undefined;
    var x704: u1 = undefined;
    addcarryxU32(&x703, &x704, x702, x631, x679);
    var x705: u32 = undefined;
    var x706: u1 = undefined;
    addcarryxU32(&x705, &x706, x704, x633, x681);
    var x707: u32 = undefined;
    var x708: u1 = undefined;
    addcarryxU32(&x707, &x708, x706, ((cast(u32, x634) + cast(u32, x564)) + (cast(u32, x610) + x566)), (cast(u32, x682) + x638));
    var x709: u32 = undefined;
    var x710: u32 = undefined;
    mulxU32(&x709, &x710, x5, 0xc84ee01);
    var x711: u32 = undefined;
    var x712: u32 = undefined;
    mulxU32(&x711, &x712, x5, 0x2b39bf21);
    var x713: u32 = undefined;
    var x714: u32 = undefined;
    mulxU32(&x713, &x714, x5, 0x3fb05b7a);
    var x715: u32 = undefined;
    var x716: u32 = undefined;
    mulxU32(&x715, &x716, x5, 0x28266895);
    var x717: u32 = undefined;
    var x718: u32 = undefined;
    mulxU32(&x717, &x718, x5, 0xd40d4917);
    var x719: u32 = undefined;
    var x720: u32 = undefined;
    mulxU32(&x719, &x720, x5, 0x4aab1cc5);
    var x721: u32 = undefined;
    var x722: u32 = undefined;
    mulxU32(&x721, &x722, x5, 0xbc3e483a);
    var x723: u32 = undefined;
    var x724: u32 = undefined;
    mulxU32(&x723, &x724, x5, 0xfcb82947);
    var x725: u32 = undefined;
    var x726: u32 = undefined;
    mulxU32(&x725, &x726, x5, 0xff3d81e5);
    var x727: u32 = undefined;
    var x728: u32 = undefined;
    mulxU32(&x727, &x728, x5, 0xdf1aa419);
    var x729: u32 = undefined;
    var x730: u32 = undefined;
    mulxU32(&x729, &x730, x5, 0x2d319b24);
    var x731: u32 = undefined;
    var x732: u32 = undefined;
    mulxU32(&x731, &x732, x5, 0x19b409a9);
    var x733: u32 = undefined;
    var x734: u1 = undefined;
    addcarryxU32(&x733, &x734, 0x0, x732, x729);
    var x735: u32 = undefined;
    var x736: u1 = undefined;
    addcarryxU32(&x735, &x736, x734, x730, x727);
    var x737: u32 = undefined;
    var x738: u1 = undefined;
    addcarryxU32(&x737, &x738, x736, x728, x725);
    var x739: u32 = undefined;
    var x740: u1 = undefined;
    addcarryxU32(&x739, &x740, x738, x726, x723);
    var x741: u32 = undefined;
    var x742: u1 = undefined;
    addcarryxU32(&x741, &x742, x740, x724, x721);
    var x743: u32 = undefined;
    var x744: u1 = undefined;
    addcarryxU32(&x743, &x744, x742, x722, x719);
    var x745: u32 = undefined;
    var x746: u1 = undefined;
    addcarryxU32(&x745, &x746, x744, x720, x717);
    var x747: u32 = undefined;
    var x748: u1 = undefined;
    addcarryxU32(&x747, &x748, x746, x718, x715);
    var x749: u32 = undefined;
    var x750: u1 = undefined;
    addcarryxU32(&x749, &x750, x748, x716, x713);
    var x751: u32 = undefined;
    var x752: u1 = undefined;
    addcarryxU32(&x751, &x752, x750, x714, x711);
    var x753: u32 = undefined;
    var x754: u1 = undefined;
    addcarryxU32(&x753, &x754, x752, x712, x709);
    var x755: u32 = undefined;
    var x756: u1 = undefined;
    addcarryxU32(&x755, &x756, 0x0, x685, x731);
    var x757: u32 = undefined;
    var x758: u1 = undefined;
    addcarryxU32(&x757, &x758, x756, x687, x733);
    var x759: u32 = undefined;
    var x760: u1 = undefined;
    addcarryxU32(&x759, &x760, x758, x689, x735);
    var x761: u32 = undefined;
    var x762: u1 = undefined;
    addcarryxU32(&x761, &x762, x760, x691, x737);
    var x763: u32 = undefined;
    var x764: u1 = undefined;
    addcarryxU32(&x763, &x764, x762, x693, x739);
    var x765: u32 = undefined;
    var x766: u1 = undefined;
    addcarryxU32(&x765, &x766, x764, x695, x741);
    var x767: u32 = undefined;
    var x768: u1 = undefined;
    addcarryxU32(&x767, &x768, x766, x697, x743);
    var x769: u32 = undefined;
    var x770: u1 = undefined;
    addcarryxU32(&x769, &x770, x768, x699, x745);
    var x771: u32 = undefined;
    var x772: u1 = undefined;
    addcarryxU32(&x771, &x772, x770, x701, x747);
    var x773: u32 = undefined;
    var x774: u1 = undefined;
    addcarryxU32(&x773, &x774, x772, x703, x749);
    var x775: u32 = undefined;
    var x776: u1 = undefined;
    addcarryxU32(&x775, &x776, x774, x705, x751);
    var x777: u32 = undefined;
    var x778: u1 = undefined;
    addcarryxU32(&x777, &x778, x776, x707, x753);
    var x779: u32 = undefined;
    var x780: u32 = undefined;
    mulxU32(&x779, &x780, x755, 0xe88fdc45);
    var x781: u32 = undefined;
    var x782: u32 = undefined;
    mulxU32(&x781, &x782, x779, 0xffffffff);
    var x783: u32 = undefined;
    var x784: u32 = undefined;
    mulxU32(&x783, &x784, x779, 0xffffffff);
    var x785: u32 = undefined;
    var x786: u32 = undefined;
    mulxU32(&x785, &x786, x779, 0xffffffff);
    var x787: u32 = undefined;
    var x788: u32 = undefined;
    mulxU32(&x787, &x788, x779, 0xffffffff);
    var x789: u32 = undefined;
    var x790: u32 = undefined;
    mulxU32(&x789, &x790, x779, 0xffffffff);
    var x791: u32 = undefined;
    var x792: u32 = undefined;
    mulxU32(&x791, &x792, x779, 0xffffffff);
    var x793: u32 = undefined;
    var x794: u32 = undefined;
    mulxU32(&x793, &x794, x779, 0xc7634d81);
    var x795: u32 = undefined;
    var x796: u32 = undefined;
    mulxU32(&x795, &x796, x779, 0xf4372ddf);
    var x797: u32 = undefined;
    var x798: u32 = undefined;
    mulxU32(&x797, &x798, x779, 0x581a0db2);
    var x799: u32 = undefined;
    var x800: u32 = undefined;
    mulxU32(&x799, &x800, x779, 0x48b0a77a);
    var x801: u32 = undefined;
    var x802: u32 = undefined;
    mulxU32(&x801, &x802, x779, 0xecec196a);
    var x803: u32 = undefined;
    var x804: u32 = undefined;
    mulxU32(&x803, &x804, x779, 0xccc52973);
    var x805: u32 = undefined;
    var x806: u1 = undefined;
    addcarryxU32(&x805, &x806, 0x0, x804, x801);
    var x807: u32 = undefined;
    var x808: u1 = undefined;
    addcarryxU32(&x807, &x808, x806, x802, x799);
    var x809: u32 = undefined;
    var x810: u1 = undefined;
    addcarryxU32(&x809, &x810, x808, x800, x797);
    var x811: u32 = undefined;
    var x812: u1 = undefined;
    addcarryxU32(&x811, &x812, x810, x798, x795);
    var x813: u32 = undefined;
    var x814: u1 = undefined;
    addcarryxU32(&x813, &x814, x812, x796, x793);
    var x815: u32 = undefined;
    var x816: u1 = undefined;
    addcarryxU32(&x815, &x816, x814, x794, x791);
    var x817: u32 = undefined;
    var x818: u1 = undefined;
    addcarryxU32(&x817, &x818, x816, x792, x789);
    var x819: u32 = undefined;
    var x820: u1 = undefined;
    addcarryxU32(&x819, &x820, x818, x790, x787);
    var x821: u32 = undefined;
    var x822: u1 = undefined;
    addcarryxU32(&x821, &x822, x820, x788, x785);
    var x823: u32 = undefined;
    var x824: u1 = undefined;
    addcarryxU32(&x823, &x824, x822, x786, x783);
    var x825: u32 = undefined;
    var x826: u1 = undefined;
    addcarryxU32(&x825, &x826, x824, x784, x781);
    var x827: u32 = undefined;
    var x828: u1 = undefined;
    addcarryxU32(&x827, &x828, 0x0, x755, x803);
    var x829: u32 = undefined;
    var x830: u1 = undefined;
    addcarryxU32(&x829, &x830, x828, x757, x805);
    var x831: u32 = undefined;
    var x832: u1 = undefined;
    addcarryxU32(&x831, &x832, x830, x759, x807);
    var x833: u32 = undefined;
    var x834: u1 = undefined;
    addcarryxU32(&x833, &x834, x832, x761, x809);
    var x835: u32 = undefined;
    var x836: u1 = undefined;
    addcarryxU32(&x835, &x836, x834, x763, x811);
    var x837: u32 = undefined;
    var x838: u1 = undefined;
    addcarryxU32(&x837, &x838, x836, x765, x813);
    var x839: u32 = undefined;
    var x840: u1 = undefined;
    addcarryxU32(&x839, &x840, x838, x767, x815);
    var x841: u32 = undefined;
    var x842: u1 = undefined;
    addcarryxU32(&x841, &x842, x840, x769, x817);
    var x843: u32 = undefined;
    var x844: u1 = undefined;
    addcarryxU32(&x843, &x844, x842, x771, x819);
    var x845: u32 = undefined;
    var x846: u1 = undefined;
    addcarryxU32(&x845, &x846, x844, x773, x821);
    var x847: u32 = undefined;
    var x848: u1 = undefined;
    addcarryxU32(&x847, &x848, x846, x775, x823);
    var x849: u32 = undefined;
    var x850: u1 = undefined;
    addcarryxU32(&x849, &x850, x848, x777, x825);
    var x851: u32 = undefined;
    var x852: u1 = undefined;
    addcarryxU32(&x851, &x852, x850, ((cast(u32, x778) + cast(u32, x708)) + (cast(u32, x754) + x710)), (cast(u32, x826) + x782));
    var x853: u32 = undefined;
    var x854: u32 = undefined;
    mulxU32(&x853, &x854, x6, 0xc84ee01);
    var x855: u32 = undefined;
    var x856: u32 = undefined;
    mulxU32(&x855, &x856, x6, 0x2b39bf21);
    var x857: u32 = undefined;
    var x858: u32 = undefined;
    mulxU32(&x857, &x858, x6, 0x3fb05b7a);
    var x859: u32 = undefined;
    var x860: u32 = undefined;
    mulxU32(&x859, &x860, x6, 0x28266895);
    var x861: u32 = undefined;
    var x862: u32 = undefined;
    mulxU32(&x861, &x862, x6, 0xd40d4917);
    var x863: u32 = undefined;
    var x864: u32 = undefined;
    mulxU32(&x863, &x864, x6, 0x4aab1cc5);
    var x865: u32 = undefined;
    var x866: u32 = undefined;
    mulxU32(&x865, &x866, x6, 0xbc3e483a);
    var x867: u32 = undefined;
    var x868: u32 = undefined;
    mulxU32(&x867, &x868, x6, 0xfcb82947);
    var x869: u32 = undefined;
    var x870: u32 = undefined;
    mulxU32(&x869, &x870, x6, 0xff3d81e5);
    var x871: u32 = undefined;
    var x872: u32 = undefined;
    mulxU32(&x871, &x872, x6, 0xdf1aa419);
    var x873: u32 = undefined;
    var x874: u32 = undefined;
    mulxU32(&x873, &x874, x6, 0x2d319b24);
    var x875: u32 = undefined;
    var x876: u32 = undefined;
    mulxU32(&x875, &x876, x6, 0x19b409a9);
    var x877: u32 = undefined;
    var x878: u1 = undefined;
    addcarryxU32(&x877, &x878, 0x0, x876, x873);
    var x879: u32 = undefined;
    var x880: u1 = undefined;
    addcarryxU32(&x879, &x880, x878, x874, x871);
    var x881: u32 = undefined;
    var x882: u1 = undefined;
    addcarryxU32(&x881, &x882, x880, x872, x869);
    var x883: u32 = undefined;
    var x884: u1 = undefined;
    addcarryxU32(&x883, &x884, x882, x870, x867);
    var x885: u32 = undefined;
    var x886: u1 = undefined;
    addcarryxU32(&x885, &x886, x884, x868, x865);
    var x887: u32 = undefined;
    var x888: u1 = undefined;
    addcarryxU32(&x887, &x888, x886, x866, x863);
    var x889: u32 = undefined;
    var x890: u1 = undefined;
    addcarryxU32(&x889, &x890, x888, x864, x861);
    var x891: u32 = undefined;
    var x892: u1 = undefined;
    addcarryxU32(&x891, &x892, x890, x862, x859);
    var x893: u32 = undefined;
    var x894: u1 = undefined;
    addcarryxU32(&x893, &x894, x892, x860, x857);
    var x895: u32 = undefined;
    var x896: u1 = undefined;
    addcarryxU32(&x895, &x896, x894, x858, x855);
    var x897: u32 = undefined;
    var x898: u1 = undefined;
    addcarryxU32(&x897, &x898, x896, x856, x853);
    var x899: u32 = undefined;
    var x900: u1 = undefined;
    addcarryxU32(&x899, &x900, 0x0, x829, x875);
    var x901: u32 = undefined;
    var x902: u1 = undefined;
    addcarryxU32(&x901, &x902, x900, x831, x877);
    var x903: u32 = undefined;
    var x904: u1 = undefined;
    addcarryxU32(&x903, &x904, x902, x833, x879);
    var x905: u32 = undefined;
    var x906: u1 = undefined;
    addcarryxU32(&x905, &x906, x904, x835, x881);
    var x907: u32 = undefined;
    var x908: u1 = undefined;
    addcarryxU32(&x907, &x908, x906, x837, x883);
    var x909: u32 = undefined;
    var x910: u1 = undefined;
    addcarryxU32(&x909, &x910, x908, x839, x885);
    var x911: u32 = undefined;
    var x912: u1 = undefined;
    addcarryxU32(&x911, &x912, x910, x841, x887);
    var x913: u32 = undefined;
    var x914: u1 = undefined;
    addcarryxU32(&x913, &x914, x912, x843, x889);
    var x915: u32 = undefined;
    var x916: u1 = undefined;
    addcarryxU32(&x915, &x916, x914, x845, x891);
    var x917: u32 = undefined;
    var x918: u1 = undefined;
    addcarryxU32(&x917, &x918, x916, x847, x893);
    var x919: u32 = undefined;
    var x920: u1 = undefined;
    addcarryxU32(&x919, &x920, x918, x849, x895);
    var x921: u32 = undefined;
    var x922: u1 = undefined;
    addcarryxU32(&x921, &x922, x920, x851, x897);
    var x923: u32 = undefined;
    var x924: u32 = undefined;
    mulxU32(&x923, &x924, x899, 0xe88fdc45);
    var x925: u32 = undefined;
    var x926: u32 = undefined;
    mulxU32(&x925, &x926, x923, 0xffffffff);
    var x927: u32 = undefined;
    var x928: u32 = undefined;
    mulxU32(&x927, &x928, x923, 0xffffffff);
    var x929: u32 = undefined;
    var x930: u32 = undefined;
    mulxU32(&x929, &x930, x923, 0xffffffff);
    var x931: u32 = undefined;
    var x932: u32 = undefined;
    mulxU32(&x931, &x932, x923, 0xffffffff);
    var x933: u32 = undefined;
    var x934: u32 = undefined;
    mulxU32(&x933, &x934, x923, 0xffffffff);
    var x935: u32 = undefined;
    var x936: u32 = undefined;
    mulxU32(&x935, &x936, x923, 0xffffffff);
    var x937: u32 = undefined;
    var x938: u32 = undefined;
    mulxU32(&x937, &x938, x923, 0xc7634d81);
    var x939: u32 = undefined;
    var x940: u32 = undefined;
    mulxU32(&x939, &x940, x923, 0xf4372ddf);
    var x941: u32 = undefined;
    var x942: u32 = undefined;
    mulxU32(&x941, &x942, x923, 0x581a0db2);
    var x943: u32 = undefined;
    var x944: u32 = undefined;
    mulxU32(&x943, &x944, x923, 0x48b0a77a);
    var x945: u32 = undefined;
    var x946: u32 = undefined;
    mulxU32(&x945, &x946, x923, 0xecec196a);
    var x947: u32 = undefined;
    var x948: u32 = undefined;
    mulxU32(&x947, &x948, x923, 0xccc52973);
    var x949: u32 = undefined;
    var x950: u1 = undefined;
    addcarryxU32(&x949, &x950, 0x0, x948, x945);
    var x951: u32 = undefined;
    var x952: u1 = undefined;
    addcarryxU32(&x951, &x952, x950, x946, x943);
    var x953: u32 = undefined;
    var x954: u1 = undefined;
    addcarryxU32(&x953, &x954, x952, x944, x941);
    var x955: u32 = undefined;
    var x956: u1 = undefined;
    addcarryxU32(&x955, &x956, x954, x942, x939);
    var x957: u32 = undefined;
    var x958: u1 = undefined;
    addcarryxU32(&x957, &x958, x956, x940, x937);
    var x959: u32 = undefined;
    var x960: u1 = undefined;
    addcarryxU32(&x959, &x960, x958, x938, x935);
    var x961: u32 = undefined;
    var x962: u1 = undefined;
    addcarryxU32(&x961, &x962, x960, x936, x933);
    var x963: u32 = undefined;
    var x964: u1 = undefined;
    addcarryxU32(&x963, &x964, x962, x934, x931);
    var x965: u32 = undefined;
    var x966: u1 = undefined;
    addcarryxU32(&x965, &x966, x964, x932, x929);
    var x967: u32 = undefined;
    var x968: u1 = undefined;
    addcarryxU32(&x967, &x968, x966, x930, x927);
    var x969: u32 = undefined;
    var x970: u1 = undefined;
    addcarryxU32(&x969, &x970, x968, x928, x925);
    var x971: u32 = undefined;
    var x972: u1 = undefined;
    addcarryxU32(&x971, &x972, 0x0, x899, x947);
    var x973: u32 = undefined;
    var x974: u1 = undefined;
    addcarryxU32(&x973, &x974, x972, x901, x949);
    var x975: u32 = undefined;
    var x976: u1 = undefined;
    addcarryxU32(&x975, &x976, x974, x903, x951);
    var x977: u32 = undefined;
    var x978: u1 = undefined;
    addcarryxU32(&x977, &x978, x976, x905, x953);
    var x979: u32 = undefined;
    var x980: u1 = undefined;
    addcarryxU32(&x979, &x980, x978, x907, x955);
    var x981: u32 = undefined;
    var x982: u1 = undefined;
    addcarryxU32(&x981, &x982, x980, x909, x957);
    var x983: u32 = undefined;
    var x984: u1 = undefined;
    addcarryxU32(&x983, &x984, x982, x911, x959);
    var x985: u32 = undefined;
    var x986: u1 = undefined;
    addcarryxU32(&x985, &x986, x984, x913, x961);
    var x987: u32 = undefined;
    var x988: u1 = undefined;
    addcarryxU32(&x987, &x988, x986, x915, x963);
    var x989: u32 = undefined;
    var x990: u1 = undefined;
    addcarryxU32(&x989, &x990, x988, x917, x965);
    var x991: u32 = undefined;
    var x992: u1 = undefined;
    addcarryxU32(&x991, &x992, x990, x919, x967);
    var x993: u32 = undefined;
    var x994: u1 = undefined;
    addcarryxU32(&x993, &x994, x992, x921, x969);
    var x995: u32 = undefined;
    var x996: u1 = undefined;
    addcarryxU32(&x995, &x996, x994, ((cast(u32, x922) + cast(u32, x852)) + (cast(u32, x898) + x854)), (cast(u32, x970) + x926));
    var x997: u32 = undefined;
    var x998: u32 = undefined;
    mulxU32(&x997, &x998, x7, 0xc84ee01);
    var x999: u32 = undefined;
    var x1000: u32 = undefined;
    mulxU32(&x999, &x1000, x7, 0x2b39bf21);
    var x1001: u32 = undefined;
    var x1002: u32 = undefined;
    mulxU32(&x1001, &x1002, x7, 0x3fb05b7a);
    var x1003: u32 = undefined;
    var x1004: u32 = undefined;
    mulxU32(&x1003, &x1004, x7, 0x28266895);
    var x1005: u32 = undefined;
    var x1006: u32 = undefined;
    mulxU32(&x1005, &x1006, x7, 0xd40d4917);
    var x1007: u32 = undefined;
    var x1008: u32 = undefined;
    mulxU32(&x1007, &x1008, x7, 0x4aab1cc5);
    var x1009: u32 = undefined;
    var x1010: u32 = undefined;
    mulxU32(&x1009, &x1010, x7, 0xbc3e483a);
    var x1011: u32 = undefined;
    var x1012: u32 = undefined;
    mulxU32(&x1011, &x1012, x7, 0xfcb82947);
    var x1013: u32 = undefined;
    var x1014: u32 = undefined;
    mulxU32(&x1013, &x1014, x7, 0xff3d81e5);
    var x1015: u32 = undefined;
    var x1016: u32 = undefined;
    mulxU32(&x1015, &x1016, x7, 0xdf1aa419);
    var x1017: u32 = undefined;
    var x1018: u32 = undefined;
    mulxU32(&x1017, &x1018, x7, 0x2d319b24);
    var x1019: u32 = undefined;
    var x1020: u32 = undefined;
    mulxU32(&x1019, &x1020, x7, 0x19b409a9);
    var x1021: u32 = undefined;
    var x1022: u1 = undefined;
    addcarryxU32(&x1021, &x1022, 0x0, x1020, x1017);
    var x1023: u32 = undefined;
    var x1024: u1 = undefined;
    addcarryxU32(&x1023, &x1024, x1022, x1018, x1015);
    var x1025: u32 = undefined;
    var x1026: u1 = undefined;
    addcarryxU32(&x1025, &x1026, x1024, x1016, x1013);
    var x1027: u32 = undefined;
    var x1028: u1 = undefined;
    addcarryxU32(&x1027, &x1028, x1026, x1014, x1011);
    var x1029: u32 = undefined;
    var x1030: u1 = undefined;
    addcarryxU32(&x1029, &x1030, x1028, x1012, x1009);
    var x1031: u32 = undefined;
    var x1032: u1 = undefined;
    addcarryxU32(&x1031, &x1032, x1030, x1010, x1007);
    var x1033: u32 = undefined;
    var x1034: u1 = undefined;
    addcarryxU32(&x1033, &x1034, x1032, x1008, x1005);
    var x1035: u32 = undefined;
    var x1036: u1 = undefined;
    addcarryxU32(&x1035, &x1036, x1034, x1006, x1003);
    var x1037: u32 = undefined;
    var x1038: u1 = undefined;
    addcarryxU32(&x1037, &x1038, x1036, x1004, x1001);
    var x1039: u32 = undefined;
    var x1040: u1 = undefined;
    addcarryxU32(&x1039, &x1040, x1038, x1002, x999);
    var x1041: u32 = undefined;
    var x1042: u1 = undefined;
    addcarryxU32(&x1041, &x1042, x1040, x1000, x997);
    var x1043: u32 = undefined;
    var x1044: u1 = undefined;
    addcarryxU32(&x1043, &x1044, 0x0, x973, x1019);
    var x1045: u32 = undefined;
    var x1046: u1 = undefined;
    addcarryxU32(&x1045, &x1046, x1044, x975, x1021);
    var x1047: u32 = undefined;
    var x1048: u1 = undefined;
    addcarryxU32(&x1047, &x1048, x1046, x977, x1023);
    var x1049: u32 = undefined;
    var x1050: u1 = undefined;
    addcarryxU32(&x1049, &x1050, x1048, x979, x1025);
    var x1051: u32 = undefined;
    var x1052: u1 = undefined;
    addcarryxU32(&x1051, &x1052, x1050, x981, x1027);
    var x1053: u32 = undefined;
    var x1054: u1 = undefined;
    addcarryxU32(&x1053, &x1054, x1052, x983, x1029);
    var x1055: u32 = undefined;
    var x1056: u1 = undefined;
    addcarryxU32(&x1055, &x1056, x1054, x985, x1031);
    var x1057: u32 = undefined;
    var x1058: u1 = undefined;
    addcarryxU32(&x1057, &x1058, x1056, x987, x1033);
    var x1059: u32 = undefined;
    var x1060: u1 = undefined;
    addcarryxU32(&x1059, &x1060, x1058, x989, x1035);
    var x1061: u32 = undefined;
    var x1062: u1 = undefined;
    addcarryxU32(&x1061, &x1062, x1060, x991, x1037);
    var x1063: u32 = undefined;
    var x1064: u1 = undefined;
    addcarryxU32(&x1063, &x1064, x1062, x993, x1039);
    var x1065: u32 = undefined;
    var x1066: u1 = undefined;
    addcarryxU32(&x1065, &x1066, x1064, x995, x1041);
    var x1067: u32 = undefined;
    var x1068: u32 = undefined;
    mulxU32(&x1067, &x1068, x1043, 0xe88fdc45);
    var x1069: u32 = undefined;
    var x1070: u32 = undefined;
    mulxU32(&x1069, &x1070, x1067, 0xffffffff);
    var x1071: u32 = undefined;
    var x1072: u32 = undefined;
    mulxU32(&x1071, &x1072, x1067, 0xffffffff);
    var x1073: u32 = undefined;
    var x1074: u32 = undefined;
    mulxU32(&x1073, &x1074, x1067, 0xffffffff);
    var x1075: u32 = undefined;
    var x1076: u32 = undefined;
    mulxU32(&x1075, &x1076, x1067, 0xffffffff);
    var x1077: u32 = undefined;
    var x1078: u32 = undefined;
    mulxU32(&x1077, &x1078, x1067, 0xffffffff);
    var x1079: u32 = undefined;
    var x1080: u32 = undefined;
    mulxU32(&x1079, &x1080, x1067, 0xffffffff);
    var x1081: u32 = undefined;
    var x1082: u32 = undefined;
    mulxU32(&x1081, &x1082, x1067, 0xc7634d81);
    var x1083: u32 = undefined;
    var x1084: u32 = undefined;
    mulxU32(&x1083, &x1084, x1067, 0xf4372ddf);
    var x1085: u32 = undefined;
    var x1086: u32 = undefined;
    mulxU32(&x1085, &x1086, x1067, 0x581a0db2);
    var x1087: u32 = undefined;
    var x1088: u32 = undefined;
    mulxU32(&x1087, &x1088, x1067, 0x48b0a77a);
    var x1089: u32 = undefined;
    var x1090: u32 = undefined;
    mulxU32(&x1089, &x1090, x1067, 0xecec196a);
    var x1091: u32 = undefined;
    var x1092: u32 = undefined;
    mulxU32(&x1091, &x1092, x1067, 0xccc52973);
    var x1093: u32 = undefined;
    var x1094: u1 = undefined;
    addcarryxU32(&x1093, &x1094, 0x0, x1092, x1089);
    var x1095: u32 = undefined;
    var x1096: u1 = undefined;
    addcarryxU32(&x1095, &x1096, x1094, x1090, x1087);
    var x1097: u32 = undefined;
    var x1098: u1 = undefined;
    addcarryxU32(&x1097, &x1098, x1096, x1088, x1085);
    var x1099: u32 = undefined;
    var x1100: u1 = undefined;
    addcarryxU32(&x1099, &x1100, x1098, x1086, x1083);
    var x1101: u32 = undefined;
    var x1102: u1 = undefined;
    addcarryxU32(&x1101, &x1102, x1100, x1084, x1081);
    var x1103: u32 = undefined;
    var x1104: u1 = undefined;
    addcarryxU32(&x1103, &x1104, x1102, x1082, x1079);
    var x1105: u32 = undefined;
    var x1106: u1 = undefined;
    addcarryxU32(&x1105, &x1106, x1104, x1080, x1077);
    var x1107: u32 = undefined;
    var x1108: u1 = undefined;
    addcarryxU32(&x1107, &x1108, x1106, x1078, x1075);
    var x1109: u32 = undefined;
    var x1110: u1 = undefined;
    addcarryxU32(&x1109, &x1110, x1108, x1076, x1073);
    var x1111: u32 = undefined;
    var x1112: u1 = undefined;
    addcarryxU32(&x1111, &x1112, x1110, x1074, x1071);
    var x1113: u32 = undefined;
    var x1114: u1 = undefined;
    addcarryxU32(&x1113, &x1114, x1112, x1072, x1069);
    var x1115: u32 = undefined;
    var x1116: u1 = undefined;
    addcarryxU32(&x1115, &x1116, 0x0, x1043, x1091);
    var x1117: u32 = undefined;
    var x1118: u1 = undefined;
    addcarryxU32(&x1117, &x1118, x1116, x1045, x1093);
    var x1119: u32 = undefined;
    var x1120: u1 = undefined;
    addcarryxU32(&x1119, &x1120, x1118, x1047, x1095);
    var x1121: u32 = undefined;
    var x1122: u1 = undefined;
    addcarryxU32(&x1121, &x1122, x1120, x1049, x1097);
    var x1123: u32 = undefined;
    var x1124: u1 = undefined;
    addcarryxU32(&x1123, &x1124, x1122, x1051, x1099);
    var x1125: u32 = undefined;
    var x1126: u1 = undefined;
    addcarryxU32(&x1125, &x1126, x1124, x1053, x1101);
    var x1127: u32 = undefined;
    var x1128: u1 = undefined;
    addcarryxU32(&x1127, &x1128, x1126, x1055, x1103);
    var x1129: u32 = undefined;
    var x1130: u1 = undefined;
    addcarryxU32(&x1129, &x1130, x1128, x1057, x1105);
    var x1131: u32 = undefined;
    var x1132: u1 = undefined;
    addcarryxU32(&x1131, &x1132, x1130, x1059, x1107);
    var x1133: u32 = undefined;
    var x1134: u1 = undefined;
    addcarryxU32(&x1133, &x1134, x1132, x1061, x1109);
    var x1135: u32 = undefined;
    var x1136: u1 = undefined;
    addcarryxU32(&x1135, &x1136, x1134, x1063, x1111);
    var x1137: u32 = undefined;
    var x1138: u1 = undefined;
    addcarryxU32(&x1137, &x1138, x1136, x1065, x1113);
    var x1139: u32 = undefined;
    var x1140: u1 = undefined;
    addcarryxU32(&x1139, &x1140, x1138, ((cast(u32, x1066) + cast(u32, x996)) + (cast(u32, x1042) + x998)), (cast(u32, x1114) + x1070));
    var x1141: u32 = undefined;
    var x1142: u32 = undefined;
    mulxU32(&x1141, &x1142, x8, 0xc84ee01);
    var x1143: u32 = undefined;
    var x1144: u32 = undefined;
    mulxU32(&x1143, &x1144, x8, 0x2b39bf21);
    var x1145: u32 = undefined;
    var x1146: u32 = undefined;
    mulxU32(&x1145, &x1146, x8, 0x3fb05b7a);
    var x1147: u32 = undefined;
    var x1148: u32 = undefined;
    mulxU32(&x1147, &x1148, x8, 0x28266895);
    var x1149: u32 = undefined;
    var x1150: u32 = undefined;
    mulxU32(&x1149, &x1150, x8, 0xd40d4917);
    var x1151: u32 = undefined;
    var x1152: u32 = undefined;
    mulxU32(&x1151, &x1152, x8, 0x4aab1cc5);
    var x1153: u32 = undefined;
    var x1154: u32 = undefined;
    mulxU32(&x1153, &x1154, x8, 0xbc3e483a);
    var x1155: u32 = undefined;
    var x1156: u32 = undefined;
    mulxU32(&x1155, &x1156, x8, 0xfcb82947);
    var x1157: u32 = undefined;
    var x1158: u32 = undefined;
    mulxU32(&x1157, &x1158, x8, 0xff3d81e5);
    var x1159: u32 = undefined;
    var x1160: u32 = undefined;
    mulxU32(&x1159, &x1160, x8, 0xdf1aa419);
    var x1161: u32 = undefined;
    var x1162: u32 = undefined;
    mulxU32(&x1161, &x1162, x8, 0x2d319b24);
    var x1163: u32 = undefined;
    var x1164: u32 = undefined;
    mulxU32(&x1163, &x1164, x8, 0x19b409a9);
    var x1165: u32 = undefined;
    var x1166: u1 = undefined;
    addcarryxU32(&x1165, &x1166, 0x0, x1164, x1161);
    var x1167: u32 = undefined;
    var x1168: u1 = undefined;
    addcarryxU32(&x1167, &x1168, x1166, x1162, x1159);
    var x1169: u32 = undefined;
    var x1170: u1 = undefined;
    addcarryxU32(&x1169, &x1170, x1168, x1160, x1157);
    var x1171: u32 = undefined;
    var x1172: u1 = undefined;
    addcarryxU32(&x1171, &x1172, x1170, x1158, x1155);
    var x1173: u32 = undefined;
    var x1174: u1 = undefined;
    addcarryxU32(&x1173, &x1174, x1172, x1156, x1153);
    var x1175: u32 = undefined;
    var x1176: u1 = undefined;
    addcarryxU32(&x1175, &x1176, x1174, x1154, x1151);
    var x1177: u32 = undefined;
    var x1178: u1 = undefined;
    addcarryxU32(&x1177, &x1178, x1176, x1152, x1149);
    var x1179: u32 = undefined;
    var x1180: u1 = undefined;
    addcarryxU32(&x1179, &x1180, x1178, x1150, x1147);
    var x1181: u32 = undefined;
    var x1182: u1 = undefined;
    addcarryxU32(&x1181, &x1182, x1180, x1148, x1145);
    var x1183: u32 = undefined;
    var x1184: u1 = undefined;
    addcarryxU32(&x1183, &x1184, x1182, x1146, x1143);
    var x1185: u32 = undefined;
    var x1186: u1 = undefined;
    addcarryxU32(&x1185, &x1186, x1184, x1144, x1141);
    var x1187: u32 = undefined;
    var x1188: u1 = undefined;
    addcarryxU32(&x1187, &x1188, 0x0, x1117, x1163);
    var x1189: u32 = undefined;
    var x1190: u1 = undefined;
    addcarryxU32(&x1189, &x1190, x1188, x1119, x1165);
    var x1191: u32 = undefined;
    var x1192: u1 = undefined;
    addcarryxU32(&x1191, &x1192, x1190, x1121, x1167);
    var x1193: u32 = undefined;
    var x1194: u1 = undefined;
    addcarryxU32(&x1193, &x1194, x1192, x1123, x1169);
    var x1195: u32 = undefined;
    var x1196: u1 = undefined;
    addcarryxU32(&x1195, &x1196, x1194, x1125, x1171);
    var x1197: u32 = undefined;
    var x1198: u1 = undefined;
    addcarryxU32(&x1197, &x1198, x1196, x1127, x1173);
    var x1199: u32 = undefined;
    var x1200: u1 = undefined;
    addcarryxU32(&x1199, &x1200, x1198, x1129, x1175);
    var x1201: u32 = undefined;
    var x1202: u1 = undefined;
    addcarryxU32(&x1201, &x1202, x1200, x1131, x1177);
    var x1203: u32 = undefined;
    var x1204: u1 = undefined;
    addcarryxU32(&x1203, &x1204, x1202, x1133, x1179);
    var x1205: u32 = undefined;
    var x1206: u1 = undefined;
    addcarryxU32(&x1205, &x1206, x1204, x1135, x1181);
    var x1207: u32 = undefined;
    var x1208: u1 = undefined;
    addcarryxU32(&x1207, &x1208, x1206, x1137, x1183);
    var x1209: u32 = undefined;
    var x1210: u1 = undefined;
    addcarryxU32(&x1209, &x1210, x1208, x1139, x1185);
    var x1211: u32 = undefined;
    var x1212: u32 = undefined;
    mulxU32(&x1211, &x1212, x1187, 0xe88fdc45);
    var x1213: u32 = undefined;
    var x1214: u32 = undefined;
    mulxU32(&x1213, &x1214, x1211, 0xffffffff);
    var x1215: u32 = undefined;
    var x1216: u32 = undefined;
    mulxU32(&x1215, &x1216, x1211, 0xffffffff);
    var x1217: u32 = undefined;
    var x1218: u32 = undefined;
    mulxU32(&x1217, &x1218, x1211, 0xffffffff);
    var x1219: u32 = undefined;
    var x1220: u32 = undefined;
    mulxU32(&x1219, &x1220, x1211, 0xffffffff);
    var x1221: u32 = undefined;
    var x1222: u32 = undefined;
    mulxU32(&x1221, &x1222, x1211, 0xffffffff);
    var x1223: u32 = undefined;
    var x1224: u32 = undefined;
    mulxU32(&x1223, &x1224, x1211, 0xffffffff);
    var x1225: u32 = undefined;
    var x1226: u32 = undefined;
    mulxU32(&x1225, &x1226, x1211, 0xc7634d81);
    var x1227: u32 = undefined;
    var x1228: u32 = undefined;
    mulxU32(&x1227, &x1228, x1211, 0xf4372ddf);
    var x1229: u32 = undefined;
    var x1230: u32 = undefined;
    mulxU32(&x1229, &x1230, x1211, 0x581a0db2);
    var x1231: u32 = undefined;
    var x1232: u32 = undefined;
    mulxU32(&x1231, &x1232, x1211, 0x48b0a77a);
    var x1233: u32 = undefined;
    var x1234: u32 = undefined;
    mulxU32(&x1233, &x1234, x1211, 0xecec196a);
    var x1235: u32 = undefined;
    var x1236: u32 = undefined;
    mulxU32(&x1235, &x1236, x1211, 0xccc52973);
    var x1237: u32 = undefined;
    var x1238: u1 = undefined;
    addcarryxU32(&x1237, &x1238, 0x0, x1236, x1233);
    var x1239: u32 = undefined;
    var x1240: u1 = undefined;
    addcarryxU32(&x1239, &x1240, x1238, x1234, x1231);
    var x1241: u32 = undefined;
    var x1242: u1 = undefined;
    addcarryxU32(&x1241, &x1242, x1240, x1232, x1229);
    var x1243: u32 = undefined;
    var x1244: u1 = undefined;
    addcarryxU32(&x1243, &x1244, x1242, x1230, x1227);
    var x1245: u32 = undefined;
    var x1246: u1 = undefined;
    addcarryxU32(&x1245, &x1246, x1244, x1228, x1225);
    var x1247: u32 = undefined;
    var x1248: u1 = undefined;
    addcarryxU32(&x1247, &x1248, x1246, x1226, x1223);
    var x1249: u32 = undefined;
    var x1250: u1 = undefined;
    addcarryxU32(&x1249, &x1250, x1248, x1224, x1221);
    var x1251: u32 = undefined;
    var x1252: u1 = undefined;
    addcarryxU32(&x1251, &x1252, x1250, x1222, x1219);
    var x1253: u32 = undefined;
    var x1254: u1 = undefined;
    addcarryxU32(&x1253, &x1254, x1252, x1220, x1217);
    var x1255: u32 = undefined;
    var x1256: u1 = undefined;
    addcarryxU32(&x1255, &x1256, x1254, x1218, x1215);
    var x1257: u32 = undefined;
    var x1258: u1 = undefined;
    addcarryxU32(&x1257, &x1258, x1256, x1216, x1213);
    var x1259: u32 = undefined;
    var x1260: u1 = undefined;
    addcarryxU32(&x1259, &x1260, 0x0, x1187, x1235);
    var x1261: u32 = undefined;
    var x1262: u1 = undefined;
    addcarryxU32(&x1261, &x1262, x1260, x1189, x1237);
    var x1263: u32 = undefined;
    var x1264: u1 = undefined;
    addcarryxU32(&x1263, &x1264, x1262, x1191, x1239);
    var x1265: u32 = undefined;
    var x1266: u1 = undefined;
    addcarryxU32(&x1265, &x1266, x1264, x1193, x1241);
    var x1267: u32 = undefined;
    var x1268: u1 = undefined;
    addcarryxU32(&x1267, &x1268, x1266, x1195, x1243);
    var x1269: u32 = undefined;
    var x1270: u1 = undefined;
    addcarryxU32(&x1269, &x1270, x1268, x1197, x1245);
    var x1271: u32 = undefined;
    var x1272: u1 = undefined;
    addcarryxU32(&x1271, &x1272, x1270, x1199, x1247);
    var x1273: u32 = undefined;
    var x1274: u1 = undefined;
    addcarryxU32(&x1273, &x1274, x1272, x1201, x1249);
    var x1275: u32 = undefined;
    var x1276: u1 = undefined;
    addcarryxU32(&x1275, &x1276, x1274, x1203, x1251);
    var x1277: u32 = undefined;
    var x1278: u1 = undefined;
    addcarryxU32(&x1277, &x1278, x1276, x1205, x1253);
    var x1279: u32 = undefined;
    var x1280: u1 = undefined;
    addcarryxU32(&x1279, &x1280, x1278, x1207, x1255);
    var x1281: u32 = undefined;
    var x1282: u1 = undefined;
    addcarryxU32(&x1281, &x1282, x1280, x1209, x1257);
    var x1283: u32 = undefined;
    var x1284: u1 = undefined;
    addcarryxU32(&x1283, &x1284, x1282, ((cast(u32, x1210) + cast(u32, x1140)) + (cast(u32, x1186) + x1142)), (cast(u32, x1258) + x1214));
    var x1285: u32 = undefined;
    var x1286: u32 = undefined;
    mulxU32(&x1285, &x1286, x9, 0xc84ee01);
    var x1287: u32 = undefined;
    var x1288: u32 = undefined;
    mulxU32(&x1287, &x1288, x9, 0x2b39bf21);
    var x1289: u32 = undefined;
    var x1290: u32 = undefined;
    mulxU32(&x1289, &x1290, x9, 0x3fb05b7a);
    var x1291: u32 = undefined;
    var x1292: u32 = undefined;
    mulxU32(&x1291, &x1292, x9, 0x28266895);
    var x1293: u32 = undefined;
    var x1294: u32 = undefined;
    mulxU32(&x1293, &x1294, x9, 0xd40d4917);
    var x1295: u32 = undefined;
    var x1296: u32 = undefined;
    mulxU32(&x1295, &x1296, x9, 0x4aab1cc5);
    var x1297: u32 = undefined;
    var x1298: u32 = undefined;
    mulxU32(&x1297, &x1298, x9, 0xbc3e483a);
    var x1299: u32 = undefined;
    var x1300: u32 = undefined;
    mulxU32(&x1299, &x1300, x9, 0xfcb82947);
    var x1301: u32 = undefined;
    var x1302: u32 = undefined;
    mulxU32(&x1301, &x1302, x9, 0xff3d81e5);
    var x1303: u32 = undefined;
    var x1304: u32 = undefined;
    mulxU32(&x1303, &x1304, x9, 0xdf1aa419);
    var x1305: u32 = undefined;
    var x1306: u32 = undefined;
    mulxU32(&x1305, &x1306, x9, 0x2d319b24);
    var x1307: u32 = undefined;
    var x1308: u32 = undefined;
    mulxU32(&x1307, &x1308, x9, 0x19b409a9);
    var x1309: u32 = undefined;
    var x1310: u1 = undefined;
    addcarryxU32(&x1309, &x1310, 0x0, x1308, x1305);
    var x1311: u32 = undefined;
    var x1312: u1 = undefined;
    addcarryxU32(&x1311, &x1312, x1310, x1306, x1303);
    var x1313: u32 = undefined;
    var x1314: u1 = undefined;
    addcarryxU32(&x1313, &x1314, x1312, x1304, x1301);
    var x1315: u32 = undefined;
    var x1316: u1 = undefined;
    addcarryxU32(&x1315, &x1316, x1314, x1302, x1299);
    var x1317: u32 = undefined;
    var x1318: u1 = undefined;
    addcarryxU32(&x1317, &x1318, x1316, x1300, x1297);
    var x1319: u32 = undefined;
    var x1320: u1 = undefined;
    addcarryxU32(&x1319, &x1320, x1318, x1298, x1295);
    var x1321: u32 = undefined;
    var x1322: u1 = undefined;
    addcarryxU32(&x1321, &x1322, x1320, x1296, x1293);
    var x1323: u32 = undefined;
    var x1324: u1 = undefined;
    addcarryxU32(&x1323, &x1324, x1322, x1294, x1291);
    var x1325: u32 = undefined;
    var x1326: u1 = undefined;
    addcarryxU32(&x1325, &x1326, x1324, x1292, x1289);
    var x1327: u32 = undefined;
    var x1328: u1 = undefined;
    addcarryxU32(&x1327, &x1328, x1326, x1290, x1287);
    var x1329: u32 = undefined;
    var x1330: u1 = undefined;
    addcarryxU32(&x1329, &x1330, x1328, x1288, x1285);
    var x1331: u32 = undefined;
    var x1332: u1 = undefined;
    addcarryxU32(&x1331, &x1332, 0x0, x1261, x1307);
    var x1333: u32 = undefined;
    var x1334: u1 = undefined;
    addcarryxU32(&x1333, &x1334, x1332, x1263, x1309);
    var x1335: u32 = undefined;
    var x1336: u1 = undefined;
    addcarryxU32(&x1335, &x1336, x1334, x1265, x1311);
    var x1337: u32 = undefined;
    var x1338: u1 = undefined;
    addcarryxU32(&x1337, &x1338, x1336, x1267, x1313);
    var x1339: u32 = undefined;
    var x1340: u1 = undefined;
    addcarryxU32(&x1339, &x1340, x1338, x1269, x1315);
    var x1341: u32 = undefined;
    var x1342: u1 = undefined;
    addcarryxU32(&x1341, &x1342, x1340, x1271, x1317);
    var x1343: u32 = undefined;
    var x1344: u1 = undefined;
    addcarryxU32(&x1343, &x1344, x1342, x1273, x1319);
    var x1345: u32 = undefined;
    var x1346: u1 = undefined;
    addcarryxU32(&x1345, &x1346, x1344, x1275, x1321);
    var x1347: u32 = undefined;
    var x1348: u1 = undefined;
    addcarryxU32(&x1347, &x1348, x1346, x1277, x1323);
    var x1349: u32 = undefined;
    var x1350: u1 = undefined;
    addcarryxU32(&x1349, &x1350, x1348, x1279, x1325);
    var x1351: u32 = undefined;
    var x1352: u1 = undefined;
    addcarryxU32(&x1351, &x1352, x1350, x1281, x1327);
    var x1353: u32 = undefined;
    var x1354: u1 = undefined;
    addcarryxU32(&x1353, &x1354, x1352, x1283, x1329);
    var x1355: u32 = undefined;
    var x1356: u32 = undefined;
    mulxU32(&x1355, &x1356, x1331, 0xe88fdc45);
    var x1357: u32 = undefined;
    var x1358: u32 = undefined;
    mulxU32(&x1357, &x1358, x1355, 0xffffffff);
    var x1359: u32 = undefined;
    var x1360: u32 = undefined;
    mulxU32(&x1359, &x1360, x1355, 0xffffffff);
    var x1361: u32 = undefined;
    var x1362: u32 = undefined;
    mulxU32(&x1361, &x1362, x1355, 0xffffffff);
    var x1363: u32 = undefined;
    var x1364: u32 = undefined;
    mulxU32(&x1363, &x1364, x1355, 0xffffffff);
    var x1365: u32 = undefined;
    var x1366: u32 = undefined;
    mulxU32(&x1365, &x1366, x1355, 0xffffffff);
    var x1367: u32 = undefined;
    var x1368: u32 = undefined;
    mulxU32(&x1367, &x1368, x1355, 0xffffffff);
    var x1369: u32 = undefined;
    var x1370: u32 = undefined;
    mulxU32(&x1369, &x1370, x1355, 0xc7634d81);
    var x1371: u32 = undefined;
    var x1372: u32 = undefined;
    mulxU32(&x1371, &x1372, x1355, 0xf4372ddf);
    var x1373: u32 = undefined;
    var x1374: u32 = undefined;
    mulxU32(&x1373, &x1374, x1355, 0x581a0db2);
    var x1375: u32 = undefined;
    var x1376: u32 = undefined;
    mulxU32(&x1375, &x1376, x1355, 0x48b0a77a);
    var x1377: u32 = undefined;
    var x1378: u32 = undefined;
    mulxU32(&x1377, &x1378, x1355, 0xecec196a);
    var x1379: u32 = undefined;
    var x1380: u32 = undefined;
    mulxU32(&x1379, &x1380, x1355, 0xccc52973);
    var x1381: u32 = undefined;
    var x1382: u1 = undefined;
    addcarryxU32(&x1381, &x1382, 0x0, x1380, x1377);
    var x1383: u32 = undefined;
    var x1384: u1 = undefined;
    addcarryxU32(&x1383, &x1384, x1382, x1378, x1375);
    var x1385: u32 = undefined;
    var x1386: u1 = undefined;
    addcarryxU32(&x1385, &x1386, x1384, x1376, x1373);
    var x1387: u32 = undefined;
    var x1388: u1 = undefined;
    addcarryxU32(&x1387, &x1388, x1386, x1374, x1371);
    var x1389: u32 = undefined;
    var x1390: u1 = undefined;
    addcarryxU32(&x1389, &x1390, x1388, x1372, x1369);
    var x1391: u32 = undefined;
    var x1392: u1 = undefined;
    addcarryxU32(&x1391, &x1392, x1390, x1370, x1367);
    var x1393: u32 = undefined;
    var x1394: u1 = undefined;
    addcarryxU32(&x1393, &x1394, x1392, x1368, x1365);
    var x1395: u32 = undefined;
    var x1396: u1 = undefined;
    addcarryxU32(&x1395, &x1396, x1394, x1366, x1363);
    var x1397: u32 = undefined;
    var x1398: u1 = undefined;
    addcarryxU32(&x1397, &x1398, x1396, x1364, x1361);
    var x1399: u32 = undefined;
    var x1400: u1 = undefined;
    addcarryxU32(&x1399, &x1400, x1398, x1362, x1359);
    var x1401: u32 = undefined;
    var x1402: u1 = undefined;
    addcarryxU32(&x1401, &x1402, x1400, x1360, x1357);
    var x1403: u32 = undefined;
    var x1404: u1 = undefined;
    addcarryxU32(&x1403, &x1404, 0x0, x1331, x1379);
    var x1405: u32 = undefined;
    var x1406: u1 = undefined;
    addcarryxU32(&x1405, &x1406, x1404, x1333, x1381);
    var x1407: u32 = undefined;
    var x1408: u1 = undefined;
    addcarryxU32(&x1407, &x1408, x1406, x1335, x1383);
    var x1409: u32 = undefined;
    var x1410: u1 = undefined;
    addcarryxU32(&x1409, &x1410, x1408, x1337, x1385);
    var x1411: u32 = undefined;
    var x1412: u1 = undefined;
    addcarryxU32(&x1411, &x1412, x1410, x1339, x1387);
    var x1413: u32 = undefined;
    var x1414: u1 = undefined;
    addcarryxU32(&x1413, &x1414, x1412, x1341, x1389);
    var x1415: u32 = undefined;
    var x1416: u1 = undefined;
    addcarryxU32(&x1415, &x1416, x1414, x1343, x1391);
    var x1417: u32 = undefined;
    var x1418: u1 = undefined;
    addcarryxU32(&x1417, &x1418, x1416, x1345, x1393);
    var x1419: u32 = undefined;
    var x1420: u1 = undefined;
    addcarryxU32(&x1419, &x1420, x1418, x1347, x1395);
    var x1421: u32 = undefined;
    var x1422: u1 = undefined;
    addcarryxU32(&x1421, &x1422, x1420, x1349, x1397);
    var x1423: u32 = undefined;
    var x1424: u1 = undefined;
    addcarryxU32(&x1423, &x1424, x1422, x1351, x1399);
    var x1425: u32 = undefined;
    var x1426: u1 = undefined;
    addcarryxU32(&x1425, &x1426, x1424, x1353, x1401);
    var x1427: u32 = undefined;
    var x1428: u1 = undefined;
    addcarryxU32(&x1427, &x1428, x1426, ((cast(u32, x1354) + cast(u32, x1284)) + (cast(u32, x1330) + x1286)), (cast(u32, x1402) + x1358));
    var x1429: u32 = undefined;
    var x1430: u32 = undefined;
    mulxU32(&x1429, &x1430, x10, 0xc84ee01);
    var x1431: u32 = undefined;
    var x1432: u32 = undefined;
    mulxU32(&x1431, &x1432, x10, 0x2b39bf21);
    var x1433: u32 = undefined;
    var x1434: u32 = undefined;
    mulxU32(&x1433, &x1434, x10, 0x3fb05b7a);
    var x1435: u32 = undefined;
    var x1436: u32 = undefined;
    mulxU32(&x1435, &x1436, x10, 0x28266895);
    var x1437: u32 = undefined;
    var x1438: u32 = undefined;
    mulxU32(&x1437, &x1438, x10, 0xd40d4917);
    var x1439: u32 = undefined;
    var x1440: u32 = undefined;
    mulxU32(&x1439, &x1440, x10, 0x4aab1cc5);
    var x1441: u32 = undefined;
    var x1442: u32 = undefined;
    mulxU32(&x1441, &x1442, x10, 0xbc3e483a);
    var x1443: u32 = undefined;
    var x1444: u32 = undefined;
    mulxU32(&x1443, &x1444, x10, 0xfcb82947);
    var x1445: u32 = undefined;
    var x1446: u32 = undefined;
    mulxU32(&x1445, &x1446, x10, 0xff3d81e5);
    var x1447: u32 = undefined;
    var x1448: u32 = undefined;
    mulxU32(&x1447, &x1448, x10, 0xdf1aa419);
    var x1449: u32 = undefined;
    var x1450: u32 = undefined;
    mulxU32(&x1449, &x1450, x10, 0x2d319b24);
    var x1451: u32 = undefined;
    var x1452: u32 = undefined;
    mulxU32(&x1451, &x1452, x10, 0x19b409a9);
    var x1453: u32 = undefined;
    var x1454: u1 = undefined;
    addcarryxU32(&x1453, &x1454, 0x0, x1452, x1449);
    var x1455: u32 = undefined;
    var x1456: u1 = undefined;
    addcarryxU32(&x1455, &x1456, x1454, x1450, x1447);
    var x1457: u32 = undefined;
    var x1458: u1 = undefined;
    addcarryxU32(&x1457, &x1458, x1456, x1448, x1445);
    var x1459: u32 = undefined;
    var x1460: u1 = undefined;
    addcarryxU32(&x1459, &x1460, x1458, x1446, x1443);
    var x1461: u32 = undefined;
    var x1462: u1 = undefined;
    addcarryxU32(&x1461, &x1462, x1460, x1444, x1441);
    var x1463: u32 = undefined;
    var x1464: u1 = undefined;
    addcarryxU32(&x1463, &x1464, x1462, x1442, x1439);
    var x1465: u32 = undefined;
    var x1466: u1 = undefined;
    addcarryxU32(&x1465, &x1466, x1464, x1440, x1437);
    var x1467: u32 = undefined;
    var x1468: u1 = undefined;
    addcarryxU32(&x1467, &x1468, x1466, x1438, x1435);
    var x1469: u32 = undefined;
    var x1470: u1 = undefined;
    addcarryxU32(&x1469, &x1470, x1468, x1436, x1433);
    var x1471: u32 = undefined;
    var x1472: u1 = undefined;
    addcarryxU32(&x1471, &x1472, x1470, x1434, x1431);
    var x1473: u32 = undefined;
    var x1474: u1 = undefined;
    addcarryxU32(&x1473, &x1474, x1472, x1432, x1429);
    var x1475: u32 = undefined;
    var x1476: u1 = undefined;
    addcarryxU32(&x1475, &x1476, 0x0, x1405, x1451);
    var x1477: u32 = undefined;
    var x1478: u1 = undefined;
    addcarryxU32(&x1477, &x1478, x1476, x1407, x1453);
    var x1479: u32 = undefined;
    var x1480: u1 = undefined;
    addcarryxU32(&x1479, &x1480, x1478, x1409, x1455);
    var x1481: u32 = undefined;
    var x1482: u1 = undefined;
    addcarryxU32(&x1481, &x1482, x1480, x1411, x1457);
    var x1483: u32 = undefined;
    var x1484: u1 = undefined;
    addcarryxU32(&x1483, &x1484, x1482, x1413, x1459);
    var x1485: u32 = undefined;
    var x1486: u1 = undefined;
    addcarryxU32(&x1485, &x1486, x1484, x1415, x1461);
    var x1487: u32 = undefined;
    var x1488: u1 = undefined;
    addcarryxU32(&x1487, &x1488, x1486, x1417, x1463);
    var x1489: u32 = undefined;
    var x1490: u1 = undefined;
    addcarryxU32(&x1489, &x1490, x1488, x1419, x1465);
    var x1491: u32 = undefined;
    var x1492: u1 = undefined;
    addcarryxU32(&x1491, &x1492, x1490, x1421, x1467);
    var x1493: u32 = undefined;
    var x1494: u1 = undefined;
    addcarryxU32(&x1493, &x1494, x1492, x1423, x1469);
    var x1495: u32 = undefined;
    var x1496: u1 = undefined;
    addcarryxU32(&x1495, &x1496, x1494, x1425, x1471);
    var x1497: u32 = undefined;
    var x1498: u1 = undefined;
    addcarryxU32(&x1497, &x1498, x1496, x1427, x1473);
    var x1499: u32 = undefined;
    var x1500: u32 = undefined;
    mulxU32(&x1499, &x1500, x1475, 0xe88fdc45);
    var x1501: u32 = undefined;
    var x1502: u32 = undefined;
    mulxU32(&x1501, &x1502, x1499, 0xffffffff);
    var x1503: u32 = undefined;
    var x1504: u32 = undefined;
    mulxU32(&x1503, &x1504, x1499, 0xffffffff);
    var x1505: u32 = undefined;
    var x1506: u32 = undefined;
    mulxU32(&x1505, &x1506, x1499, 0xffffffff);
    var x1507: u32 = undefined;
    var x1508: u32 = undefined;
    mulxU32(&x1507, &x1508, x1499, 0xffffffff);
    var x1509: u32 = undefined;
    var x1510: u32 = undefined;
    mulxU32(&x1509, &x1510, x1499, 0xffffffff);
    var x1511: u32 = undefined;
    var x1512: u32 = undefined;
    mulxU32(&x1511, &x1512, x1499, 0xffffffff);
    var x1513: u32 = undefined;
    var x1514: u32 = undefined;
    mulxU32(&x1513, &x1514, x1499, 0xc7634d81);
    var x1515: u32 = undefined;
    var x1516: u32 = undefined;
    mulxU32(&x1515, &x1516, x1499, 0xf4372ddf);
    var x1517: u32 = undefined;
    var x1518: u32 = undefined;
    mulxU32(&x1517, &x1518, x1499, 0x581a0db2);
    var x1519: u32 = undefined;
    var x1520: u32 = undefined;
    mulxU32(&x1519, &x1520, x1499, 0x48b0a77a);
    var x1521: u32 = undefined;
    var x1522: u32 = undefined;
    mulxU32(&x1521, &x1522, x1499, 0xecec196a);
    var x1523: u32 = undefined;
    var x1524: u32 = undefined;
    mulxU32(&x1523, &x1524, x1499, 0xccc52973);
    var x1525: u32 = undefined;
    var x1526: u1 = undefined;
    addcarryxU32(&x1525, &x1526, 0x0, x1524, x1521);
    var x1527: u32 = undefined;
    var x1528: u1 = undefined;
    addcarryxU32(&x1527, &x1528, x1526, x1522, x1519);
    var x1529: u32 = undefined;
    var x1530: u1 = undefined;
    addcarryxU32(&x1529, &x1530, x1528, x1520, x1517);
    var x1531: u32 = undefined;
    var x1532: u1 = undefined;
    addcarryxU32(&x1531, &x1532, x1530, x1518, x1515);
    var x1533: u32 = undefined;
    var x1534: u1 = undefined;
    addcarryxU32(&x1533, &x1534, x1532, x1516, x1513);
    var x1535: u32 = undefined;
    var x1536: u1 = undefined;
    addcarryxU32(&x1535, &x1536, x1534, x1514, x1511);
    var x1537: u32 = undefined;
    var x1538: u1 = undefined;
    addcarryxU32(&x1537, &x1538, x1536, x1512, x1509);
    var x1539: u32 = undefined;
    var x1540: u1 = undefined;
    addcarryxU32(&x1539, &x1540, x1538, x1510, x1507);
    var x1541: u32 = undefined;
    var x1542: u1 = undefined;
    addcarryxU32(&x1541, &x1542, x1540, x1508, x1505);
    var x1543: u32 = undefined;
    var x1544: u1 = undefined;
    addcarryxU32(&x1543, &x1544, x1542, x1506, x1503);
    var x1545: u32 = undefined;
    var x1546: u1 = undefined;
    addcarryxU32(&x1545, &x1546, x1544, x1504, x1501);
    var x1547: u32 = undefined;
    var x1548: u1 = undefined;
    addcarryxU32(&x1547, &x1548, 0x0, x1475, x1523);
    var x1549: u32 = undefined;
    var x1550: u1 = undefined;
    addcarryxU32(&x1549, &x1550, x1548, x1477, x1525);
    var x1551: u32 = undefined;
    var x1552: u1 = undefined;
    addcarryxU32(&x1551, &x1552, x1550, x1479, x1527);
    var x1553: u32 = undefined;
    var x1554: u1 = undefined;
    addcarryxU32(&x1553, &x1554, x1552, x1481, x1529);
    var x1555: u32 = undefined;
    var x1556: u1 = undefined;
    addcarryxU32(&x1555, &x1556, x1554, x1483, x1531);
    var x1557: u32 = undefined;
    var x1558: u1 = undefined;
    addcarryxU32(&x1557, &x1558, x1556, x1485, x1533);
    var x1559: u32 = undefined;
    var x1560: u1 = undefined;
    addcarryxU32(&x1559, &x1560, x1558, x1487, x1535);
    var x1561: u32 = undefined;
    var x1562: u1 = undefined;
    addcarryxU32(&x1561, &x1562, x1560, x1489, x1537);
    var x1563: u32 = undefined;
    var x1564: u1 = undefined;
    addcarryxU32(&x1563, &x1564, x1562, x1491, x1539);
    var x1565: u32 = undefined;
    var x1566: u1 = undefined;
    addcarryxU32(&x1565, &x1566, x1564, x1493, x1541);
    var x1567: u32 = undefined;
    var x1568: u1 = undefined;
    addcarryxU32(&x1567, &x1568, x1566, x1495, x1543);
    var x1569: u32 = undefined;
    var x1570: u1 = undefined;
    addcarryxU32(&x1569, &x1570, x1568, x1497, x1545);
    var x1571: u32 = undefined;
    var x1572: u1 = undefined;
    addcarryxU32(&x1571, &x1572, x1570, ((cast(u32, x1498) + cast(u32, x1428)) + (cast(u32, x1474) + x1430)), (cast(u32, x1546) + x1502));
    var x1573: u32 = undefined;
    var x1574: u32 = undefined;
    mulxU32(&x1573, &x1574, x11, 0xc84ee01);
    var x1575: u32 = undefined;
    var x1576: u32 = undefined;
    mulxU32(&x1575, &x1576, x11, 0x2b39bf21);
    var x1577: u32 = undefined;
    var x1578: u32 = undefined;
    mulxU32(&x1577, &x1578, x11, 0x3fb05b7a);
    var x1579: u32 = undefined;
    var x1580: u32 = undefined;
    mulxU32(&x1579, &x1580, x11, 0x28266895);
    var x1581: u32 = undefined;
    var x1582: u32 = undefined;
    mulxU32(&x1581, &x1582, x11, 0xd40d4917);
    var x1583: u32 = undefined;
    var x1584: u32 = undefined;
    mulxU32(&x1583, &x1584, x11, 0x4aab1cc5);
    var x1585: u32 = undefined;
    var x1586: u32 = undefined;
    mulxU32(&x1585, &x1586, x11, 0xbc3e483a);
    var x1587: u32 = undefined;
    var x1588: u32 = undefined;
    mulxU32(&x1587, &x1588, x11, 0xfcb82947);
    var x1589: u32 = undefined;
    var x1590: u32 = undefined;
    mulxU32(&x1589, &x1590, x11, 0xff3d81e5);
    var x1591: u32 = undefined;
    var x1592: u32 = undefined;
    mulxU32(&x1591, &x1592, x11, 0xdf1aa419);
    var x1593: u32 = undefined;
    var x1594: u32 = undefined;
    mulxU32(&x1593, &x1594, x11, 0x2d319b24);
    var x1595: u32 = undefined;
    var x1596: u32 = undefined;
    mulxU32(&x1595, &x1596, x11, 0x19b409a9);
    var x1597: u32 = undefined;
    var x1598: u1 = undefined;
    addcarryxU32(&x1597, &x1598, 0x0, x1596, x1593);
    var x1599: u32 = undefined;
    var x1600: u1 = undefined;
    addcarryxU32(&x1599, &x1600, x1598, x1594, x1591);
    var x1601: u32 = undefined;
    var x1602: u1 = undefined;
    addcarryxU32(&x1601, &x1602, x1600, x1592, x1589);
    var x1603: u32 = undefined;
    var x1604: u1 = undefined;
    addcarryxU32(&x1603, &x1604, x1602, x1590, x1587);
    var x1605: u32 = undefined;
    var x1606: u1 = undefined;
    addcarryxU32(&x1605, &x1606, x1604, x1588, x1585);
    var x1607: u32 = undefined;
    var x1608: u1 = undefined;
    addcarryxU32(&x1607, &x1608, x1606, x1586, x1583);
    var x1609: u32 = undefined;
    var x1610: u1 = undefined;
    addcarryxU32(&x1609, &x1610, x1608, x1584, x1581);
    var x1611: u32 = undefined;
    var x1612: u1 = undefined;
    addcarryxU32(&x1611, &x1612, x1610, x1582, x1579);
    var x1613: u32 = undefined;
    var x1614: u1 = undefined;
    addcarryxU32(&x1613, &x1614, x1612, x1580, x1577);
    var x1615: u32 = undefined;
    var x1616: u1 = undefined;
    addcarryxU32(&x1615, &x1616, x1614, x1578, x1575);
    var x1617: u32 = undefined;
    var x1618: u1 = undefined;
    addcarryxU32(&x1617, &x1618, x1616, x1576, x1573);
    var x1619: u32 = undefined;
    var x1620: u1 = undefined;
    addcarryxU32(&x1619, &x1620, 0x0, x1549, x1595);
    var x1621: u32 = undefined;
    var x1622: u1 = undefined;
    addcarryxU32(&x1621, &x1622, x1620, x1551, x1597);
    var x1623: u32 = undefined;
    var x1624: u1 = undefined;
    addcarryxU32(&x1623, &x1624, x1622, x1553, x1599);
    var x1625: u32 = undefined;
    var x1626: u1 = undefined;
    addcarryxU32(&x1625, &x1626, x1624, x1555, x1601);
    var x1627: u32 = undefined;
    var x1628: u1 = undefined;
    addcarryxU32(&x1627, &x1628, x1626, x1557, x1603);
    var x1629: u32 = undefined;
    var x1630: u1 = undefined;
    addcarryxU32(&x1629, &x1630, x1628, x1559, x1605);
    var x1631: u32 = undefined;
    var x1632: u1 = undefined;
    addcarryxU32(&x1631, &x1632, x1630, x1561, x1607);
    var x1633: u32 = undefined;
    var x1634: u1 = undefined;
    addcarryxU32(&x1633, &x1634, x1632, x1563, x1609);
    var x1635: u32 = undefined;
    var x1636: u1 = undefined;
    addcarryxU32(&x1635, &x1636, x1634, x1565, x1611);
    var x1637: u32 = undefined;
    var x1638: u1 = undefined;
    addcarryxU32(&x1637, &x1638, x1636, x1567, x1613);
    var x1639: u32 = undefined;
    var x1640: u1 = undefined;
    addcarryxU32(&x1639, &x1640, x1638, x1569, x1615);
    var x1641: u32 = undefined;
    var x1642: u1 = undefined;
    addcarryxU32(&x1641, &x1642, x1640, x1571, x1617);
    var x1643: u32 = undefined;
    var x1644: u32 = undefined;
    mulxU32(&x1643, &x1644, x1619, 0xe88fdc45);
    var x1645: u32 = undefined;
    var x1646: u32 = undefined;
    mulxU32(&x1645, &x1646, x1643, 0xffffffff);
    var x1647: u32 = undefined;
    var x1648: u32 = undefined;
    mulxU32(&x1647, &x1648, x1643, 0xffffffff);
    var x1649: u32 = undefined;
    var x1650: u32 = undefined;
    mulxU32(&x1649, &x1650, x1643, 0xffffffff);
    var x1651: u32 = undefined;
    var x1652: u32 = undefined;
    mulxU32(&x1651, &x1652, x1643, 0xffffffff);
    var x1653: u32 = undefined;
    var x1654: u32 = undefined;
    mulxU32(&x1653, &x1654, x1643, 0xffffffff);
    var x1655: u32 = undefined;
    var x1656: u32 = undefined;
    mulxU32(&x1655, &x1656, x1643, 0xffffffff);
    var x1657: u32 = undefined;
    var x1658: u32 = undefined;
    mulxU32(&x1657, &x1658, x1643, 0xc7634d81);
    var x1659: u32 = undefined;
    var x1660: u32 = undefined;
    mulxU32(&x1659, &x1660, x1643, 0xf4372ddf);
    var x1661: u32 = undefined;
    var x1662: u32 = undefined;
    mulxU32(&x1661, &x1662, x1643, 0x581a0db2);
    var x1663: u32 = undefined;
    var x1664: u32 = undefined;
    mulxU32(&x1663, &x1664, x1643, 0x48b0a77a);
    var x1665: u32 = undefined;
    var x1666: u32 = undefined;
    mulxU32(&x1665, &x1666, x1643, 0xecec196a);
    var x1667: u32 = undefined;
    var x1668: u32 = undefined;
    mulxU32(&x1667, &x1668, x1643, 0xccc52973);
    var x1669: u32 = undefined;
    var x1670: u1 = undefined;
    addcarryxU32(&x1669, &x1670, 0x0, x1668, x1665);
    var x1671: u32 = undefined;
    var x1672: u1 = undefined;
    addcarryxU32(&x1671, &x1672, x1670, x1666, x1663);
    var x1673: u32 = undefined;
    var x1674: u1 = undefined;
    addcarryxU32(&x1673, &x1674, x1672, x1664, x1661);
    var x1675: u32 = undefined;
    var x1676: u1 = undefined;
    addcarryxU32(&x1675, &x1676, x1674, x1662, x1659);
    var x1677: u32 = undefined;
    var x1678: u1 = undefined;
    addcarryxU32(&x1677, &x1678, x1676, x1660, x1657);
    var x1679: u32 = undefined;
    var x1680: u1 = undefined;
    addcarryxU32(&x1679, &x1680, x1678, x1658, x1655);
    var x1681: u32 = undefined;
    var x1682: u1 = undefined;
    addcarryxU32(&x1681, &x1682, x1680, x1656, x1653);
    var x1683: u32 = undefined;
    var x1684: u1 = undefined;
    addcarryxU32(&x1683, &x1684, x1682, x1654, x1651);
    var x1685: u32 = undefined;
    var x1686: u1 = undefined;
    addcarryxU32(&x1685, &x1686, x1684, x1652, x1649);
    var x1687: u32 = undefined;
    var x1688: u1 = undefined;
    addcarryxU32(&x1687, &x1688, x1686, x1650, x1647);
    var x1689: u32 = undefined;
    var x1690: u1 = undefined;
    addcarryxU32(&x1689, &x1690, x1688, x1648, x1645);
    var x1691: u32 = undefined;
    var x1692: u1 = undefined;
    addcarryxU32(&x1691, &x1692, 0x0, x1619, x1667);
    var x1693: u32 = undefined;
    var x1694: u1 = undefined;
    addcarryxU32(&x1693, &x1694, x1692, x1621, x1669);
    var x1695: u32 = undefined;
    var x1696: u1 = undefined;
    addcarryxU32(&x1695, &x1696, x1694, x1623, x1671);
    var x1697: u32 = undefined;
    var x1698: u1 = undefined;
    addcarryxU32(&x1697, &x1698, x1696, x1625, x1673);
    var x1699: u32 = undefined;
    var x1700: u1 = undefined;
    addcarryxU32(&x1699, &x1700, x1698, x1627, x1675);
    var x1701: u32 = undefined;
    var x1702: u1 = undefined;
    addcarryxU32(&x1701, &x1702, x1700, x1629, x1677);
    var x1703: u32 = undefined;
    var x1704: u1 = undefined;
    addcarryxU32(&x1703, &x1704, x1702, x1631, x1679);
    var x1705: u32 = undefined;
    var x1706: u1 = undefined;
    addcarryxU32(&x1705, &x1706, x1704, x1633, x1681);
    var x1707: u32 = undefined;
    var x1708: u1 = undefined;
    addcarryxU32(&x1707, &x1708, x1706, x1635, x1683);
    var x1709: u32 = undefined;
    var x1710: u1 = undefined;
    addcarryxU32(&x1709, &x1710, x1708, x1637, x1685);
    var x1711: u32 = undefined;
    var x1712: u1 = undefined;
    addcarryxU32(&x1711, &x1712, x1710, x1639, x1687);
    var x1713: u32 = undefined;
    var x1714: u1 = undefined;
    addcarryxU32(&x1713, &x1714, x1712, x1641, x1689);
    var x1715: u32 = undefined;
    var x1716: u1 = undefined;
    addcarryxU32(&x1715, &x1716, x1714, ((cast(u32, x1642) + cast(u32, x1572)) + (cast(u32, x1618) + x1574)), (cast(u32, x1690) + x1646));
    var x1717: u32 = undefined;
    var x1718: u1 = undefined;
    subborrowxU32(&x1717, &x1718, 0x0, x1693, 0xccc52973);
    var x1719: u32 = undefined;
    var x1720: u1 = undefined;
    subborrowxU32(&x1719, &x1720, x1718, x1695, 0xecec196a);
    var x1721: u32 = undefined;
    var x1722: u1 = undefined;
    subborrowxU32(&x1721, &x1722, x1720, x1697, 0x48b0a77a);
    var x1723: u32 = undefined;
    var x1724: u1 = undefined;
    subborrowxU32(&x1723, &x1724, x1722, x1699, 0x581a0db2);
    var x1725: u32 = undefined;
    var x1726: u1 = undefined;
    subborrowxU32(&x1725, &x1726, x1724, x1701, 0xf4372ddf);
    var x1727: u32 = undefined;
    var x1728: u1 = undefined;
    subborrowxU32(&x1727, &x1728, x1726, x1703, 0xc7634d81);
    var x1729: u32 = undefined;
    var x1730: u1 = undefined;
    subborrowxU32(&x1729, &x1730, x1728, x1705, 0xffffffff);
    var x1731: u32 = undefined;
    var x1732: u1 = undefined;
    subborrowxU32(&x1731, &x1732, x1730, x1707, 0xffffffff);
    var x1733: u32 = undefined;
    var x1734: u1 = undefined;
    subborrowxU32(&x1733, &x1734, x1732, x1709, 0xffffffff);
    var x1735: u32 = undefined;
    var x1736: u1 = undefined;
    subborrowxU32(&x1735, &x1736, x1734, x1711, 0xffffffff);
    var x1737: u32 = undefined;
    var x1738: u1 = undefined;
    subborrowxU32(&x1737, &x1738, x1736, x1713, 0xffffffff);
    var x1739: u32 = undefined;
    var x1740: u1 = undefined;
    subborrowxU32(&x1739, &x1740, x1738, x1715, 0xffffffff);
    var x1741: u32 = undefined;
    var x1742: u1 = undefined;
    subborrowxU32(&x1741, &x1742, x1740, cast(u32, x1716), cast(u32, 0x0));
    var x1743: u32 = undefined;
    cmovznzU32(&x1743, x1742, x1717, x1693);
    var x1744: u32 = undefined;
    cmovznzU32(&x1744, x1742, x1719, x1695);
    var x1745: u32 = undefined;
    cmovznzU32(&x1745, x1742, x1721, x1697);
    var x1746: u32 = undefined;
    cmovznzU32(&x1746, x1742, x1723, x1699);
    var x1747: u32 = undefined;
    cmovznzU32(&x1747, x1742, x1725, x1701);
    var x1748: u32 = undefined;
    cmovznzU32(&x1748, x1742, x1727, x1703);
    var x1749: u32 = undefined;
    cmovznzU32(&x1749, x1742, x1729, x1705);
    var x1750: u32 = undefined;
    cmovznzU32(&x1750, x1742, x1731, x1707);
    var x1751: u32 = undefined;
    cmovznzU32(&x1751, x1742, x1733, x1709);
    var x1752: u32 = undefined;
    cmovznzU32(&x1752, x1742, x1735, x1711);
    var x1753: u32 = undefined;
    cmovznzU32(&x1753, x1742, x1737, x1713);
    var x1754: u32 = undefined;
    cmovznzU32(&x1754, x1742, x1739, x1715);
    out1[0] = x1743;
    out1[1] = x1744;
    out1[2] = x1745;
    out1[3] = x1746;
    out1[4] = x1747;
    out1[5] = x1748;
    out1[6] = x1749;
    out1[7] = x1750;
    out1[8] = x1751;
    out1[9] = x1752;
    out1[10] = x1753;
    out1[11] = x1754;
}

/// The function nonzero outputs a single non-zero word if the input is non-zero and zero otherwise.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   out1 = 0 ↔ eval (from_montgomery arg1) mod m = 0
///
/// Input Bounds:
///   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
pub fn nonzero(out1: *u32, arg1: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = ((arg1[0]) | ((arg1[1]) | ((arg1[2]) | ((arg1[3]) | ((arg1[4]) | ((arg1[5]) | ((arg1[6]) | ((arg1[7]) | ((arg1[8]) | ((arg1[9]) | ((arg1[10]) | (arg1[11]))))))))))));
    out1.* = x1;
}

/// The function selectznz is a multi-limb conditional select.
///
/// Postconditions:
///   out1 = (if arg1 = 0 then arg2 else arg3)
///
/// Input Bounds:
///   arg1: [0x0 ~> 0x1]
///   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn selectznz(out1: *[12]u32, arg1: u1, arg2: [12]u32, arg3: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    cmovznzU32(&x1, arg1, (arg2[0]), (arg3[0]));
    var x2: u32 = undefined;
    cmovznzU32(&x2, arg1, (arg2[1]), (arg3[1]));
    var x3: u32 = undefined;
    cmovznzU32(&x3, arg1, (arg2[2]), (arg3[2]));
    var x4: u32 = undefined;
    cmovznzU32(&x4, arg1, (arg2[3]), (arg3[3]));
    var x5: u32 = undefined;
    cmovznzU32(&x5, arg1, (arg2[4]), (arg3[4]));
    var x6: u32 = undefined;
    cmovznzU32(&x6, arg1, (arg2[5]), (arg3[5]));
    var x7: u32 = undefined;
    cmovznzU32(&x7, arg1, (arg2[6]), (arg3[6]));
    var x8: u32 = undefined;
    cmovznzU32(&x8, arg1, (arg2[7]), (arg3[7]));
    var x9: u32 = undefined;
    cmovznzU32(&x9, arg1, (arg2[8]), (arg3[8]));
    var x10: u32 = undefined;
    cmovznzU32(&x10, arg1, (arg2[9]), (arg3[9]));
    var x11: u32 = undefined;
    cmovznzU32(&x11, arg1, (arg2[10]), (arg3[10]));
    var x12: u32 = undefined;
    cmovznzU32(&x12, arg1, (arg2[11]), (arg3[11]));
    out1[0] = x1;
    out1[1] = x2;
    out1[2] = x3;
    out1[3] = x4;
    out1[4] = x5;
    out1[5] = x6;
    out1[6] = x7;
    out1[7] = x8;
    out1[8] = x9;
    out1[9] = x10;
    out1[10] = x11;
    out1[11] = x12;
}

/// The function toBytes serializes a field element NOT in the Montgomery domain to bytes in little-endian order.
///
/// Preconditions:
///   0 ≤ eval arg1 < m
/// Postconditions:
///   out1 = map (λ x, ⌊((eval arg1 mod m) mod 2^(8 * (x + 1))) / 2^(8 * x)⌋) [0..47]
///
/// Input Bounds:
///   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
pub fn toBytes(out1: *[48]u8, arg1: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (arg1[11]);
    const x2 = (arg1[10]);
    const x3 = (arg1[9]);
    const x4 = (arg1[8]);
    const x5 = (arg1[7]);
    const x6 = (arg1[6]);
    const x7 = (arg1[5]);
    const x8 = (arg1[4]);
    const x9 = (arg1[3]);
    const x10 = (arg1[2]);
    const x11 = (arg1[1]);
    const x12 = (arg1[0]);
    const x13 = cast(u8, (x12 & cast(u32, 0xff)));
    const x14 = (x12 >> 8);
    const x15 = cast(u8, (x14 & cast(u32, 0xff)));
    const x16 = (x14 >> 8);
    const x17 = cast(u8, (x16 & cast(u32, 0xff)));
    const x18 = cast(u8, (x16 >> 8));
    const x19 = cast(u8, (x11 & cast(u32, 0xff)));
    const x20 = (x11 >> 8);
    const x21 = cast(u8, (x20 & cast(u32, 0xff)));
    const x22 = (x20 >> 8);
    const x23 = cast(u8, (x22 & cast(u32, 0xff)));
    const x24 = cast(u8, (x22 >> 8));
    const x25 = cast(u8, (x10 & cast(u32, 0xff)));
    const x26 = (x10 >> 8);
    const x27 = cast(u8, (x26 & cast(u32, 0xff)));
    const x28 = (x26 >> 8);
    const x29 = cast(u8, (x28 & cast(u32, 0xff)));
    const x30 = cast(u8, (x28 >> 8));
    const x31 = cast(u8, (x9 & cast(u32, 0xff)));
    const x32 = (x9 >> 8);
    const x33 = cast(u8, (x32 & cast(u32, 0xff)));
    const x34 = (x32 >> 8);
    const x35 = cast(u8, (x34 & cast(u32, 0xff)));
    const x36 = cast(u8, (x34 >> 8));
    const x37 = cast(u8, (x8 & cast(u32, 0xff)));
    const x38 = (x8 >> 8);
    const x39 = cast(u8, (x38 & cast(u32, 0xff)));
    const x40 = (x38 >> 8);
    const x41 = cast(u8, (x40 & cast(u32, 0xff)));
    const x42 = cast(u8, (x40 >> 8));
    const x43 = cast(u8, (x7 & cast(u32, 0xff)));
    const x44 = (x7 >> 8);
    const x45 = cast(u8, (x44 & cast(u32, 0xff)));
    const x46 = (x44 >> 8);
    const x47 = cast(u8, (x46 & cast(u32, 0xff)));
    const x48 = cast(u8, (x46 >> 8));
    const x49 = cast(u8, (x6 & cast(u32, 0xff)));
    const x50 = (x6 >> 8);
    const x51 = cast(u8, (x50 & cast(u32, 0xff)));
    const x52 = (x50 >> 8);
    const x53 = cast(u8, (x52 & cast(u32, 0xff)));
    const x54 = cast(u8, (x52 >> 8));
    const x55 = cast(u8, (x5 & cast(u32, 0xff)));
    const x56 = (x5 >> 8);
    const x57 = cast(u8, (x56 & cast(u32, 0xff)));
    const x58 = (x56 >> 8);
    const x59 = cast(u8, (x58 & cast(u32, 0xff)));
    const x60 = cast(u8, (x58 >> 8));
    const x61 = cast(u8, (x4 & cast(u32, 0xff)));
    const x62 = (x4 >> 8);
    const x63 = cast(u8, (x62 & cast(u32, 0xff)));
    const x64 = (x62 >> 8);
    const x65 = cast(u8, (x64 & cast(u32, 0xff)));
    const x66 = cast(u8, (x64 >> 8));
    const x67 = cast(u8, (x3 & cast(u32, 0xff)));
    const x68 = (x3 >> 8);
    const x69 = cast(u8, (x68 & cast(u32, 0xff)));
    const x70 = (x68 >> 8);
    const x71 = cast(u8, (x70 & cast(u32, 0xff)));
    const x72 = cast(u8, (x70 >> 8));
    const x73 = cast(u8, (x2 & cast(u32, 0xff)));
    const x74 = (x2 >> 8);
    const x75 = cast(u8, (x74 & cast(u32, 0xff)));
    const x76 = (x74 >> 8);
    const x77 = cast(u8, (x76 & cast(u32, 0xff)));
    const x78 = cast(u8, (x76 >> 8));
    const x79 = cast(u8, (x1 & cast(u32, 0xff)));
    const x80 = (x1 >> 8);
    const x81 = cast(u8, (x80 & cast(u32, 0xff)));
    const x82 = (x80 >> 8);
    const x83 = cast(u8, (x82 & cast(u32, 0xff)));
    const x84 = cast(u8, (x82 >> 8));
    out1[0] = x13;
    out1[1] = x15;
    out1[2] = x17;
    out1[3] = x18;
    out1[4] = x19;
    out1[5] = x21;
    out1[6] = x23;
    out1[7] = x24;
    out1[8] = x25;
    out1[9] = x27;
    out1[10] = x29;
    out1[11] = x30;
    out1[12] = x31;
    out1[13] = x33;
    out1[14] = x35;
    out1[15] = x36;
    out1[16] = x37;
    out1[17] = x39;
    out1[18] = x41;
    out1[19] = x42;
    out1[20] = x43;
    out1[21] = x45;
    out1[22] = x47;
    out1[23] = x48;
    out1[24] = x49;
    out1[25] = x51;
    out1[26] = x53;
    out1[27] = x54;
    out1[28] = x55;
    out1[29] = x57;
    out1[30] = x59;
    out1[31] = x60;
    out1[32] = x61;
    out1[33] = x63;
    out1[34] = x65;
    out1[35] = x66;
    out1[36] = x67;
    out1[37] = x69;
    out1[38] = x71;
    out1[39] = x72;
    out1[40] = x73;
    out1[41] = x75;
    out1[42] = x77;
    out1[43] = x78;
    out1[44] = x79;
    out1[45] = x81;
    out1[46] = x83;
    out1[47] = x84;
}

/// The function fromBytes deserializes a field element NOT in the Montgomery domain from bytes in little-endian order.
///
/// Preconditions:
///   0 ≤ bytes_eval arg1 < m
/// Postconditions:
///   eval out1 mod m = bytes_eval arg1 mod m
///   0 ≤ eval out1 < m
///
/// Input Bounds:
///   arg1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn fromBytes(out1: *[12]u32, arg1: [48]u8) void {
    @setRuntimeSafety(mode == .Debug);

    const x1 = (cast(u32, (arg1[47])) << 24);
    const x2 = (cast(u32, (arg1[46])) << 16);
    const x3 = (cast(u32, (arg1[45])) << 8);
    const x4 = (arg1[44]);
    const x5 = (cast(u32, (arg1[43])) << 24);
    const x6 = (cast(u32, (arg1[42])) << 16);
    const x7 = (cast(u32, (arg1[41])) << 8);
    const x8 = (arg1[40]);
    const x9 = (cast(u32, (arg1[39])) << 24);
    const x10 = (cast(u32, (arg1[38])) << 16);
    const x11 = (cast(u32, (arg1[37])) << 8);
    const x12 = (arg1[36]);
    const x13 = (cast(u32, (arg1[35])) << 24);
    const x14 = (cast(u32, (arg1[34])) << 16);
    const x15 = (cast(u32, (arg1[33])) << 8);
    const x16 = (arg1[32]);
    const x17 = (cast(u32, (arg1[31])) << 24);
    const x18 = (cast(u32, (arg1[30])) << 16);
    const x19 = (cast(u32, (arg1[29])) << 8);
    const x20 = (arg1[28]);
    const x21 = (cast(u32, (arg1[27])) << 24);
    const x22 = (cast(u32, (arg1[26])) << 16);
    const x23 = (cast(u32, (arg1[25])) << 8);
    const x24 = (arg1[24]);
    const x25 = (cast(u32, (arg1[23])) << 24);
    const x26 = (cast(u32, (arg1[22])) << 16);
    const x27 = (cast(u32, (arg1[21])) << 8);
    const x28 = (arg1[20]);
    const x29 = (cast(u32, (arg1[19])) << 24);
    const x30 = (cast(u32, (arg1[18])) << 16);
    const x31 = (cast(u32, (arg1[17])) << 8);
    const x32 = (arg1[16]);
    const x33 = (cast(u32, (arg1[15])) << 24);
    const x34 = (cast(u32, (arg1[14])) << 16);
    const x35 = (cast(u32, (arg1[13])) << 8);
    const x36 = (arg1[12]);
    const x37 = (cast(u32, (arg1[11])) << 24);
    const x38 = (cast(u32, (arg1[10])) << 16);
    const x39 = (cast(u32, (arg1[9])) << 8);
    const x40 = (arg1[8]);
    const x41 = (cast(u32, (arg1[7])) << 24);
    const x42 = (cast(u32, (arg1[6])) << 16);
    const x43 = (cast(u32, (arg1[5])) << 8);
    const x44 = (arg1[4]);
    const x45 = (cast(u32, (arg1[3])) << 24);
    const x46 = (cast(u32, (arg1[2])) << 16);
    const x47 = (cast(u32, (arg1[1])) << 8);
    const x48 = (arg1[0]);
    const x49 = (x47 + cast(u32, x48));
    const x50 = (x46 + x49);
    const x51 = (x45 + x50);
    const x52 = (x43 + cast(u32, x44));
    const x53 = (x42 + x52);
    const x54 = (x41 + x53);
    const x55 = (x39 + cast(u32, x40));
    const x56 = (x38 + x55);
    const x57 = (x37 + x56);
    const x58 = (x35 + cast(u32, x36));
    const x59 = (x34 + x58);
    const x60 = (x33 + x59);
    const x61 = (x31 + cast(u32, x32));
    const x62 = (x30 + x61);
    const x63 = (x29 + x62);
    const x64 = (x27 + cast(u32, x28));
    const x65 = (x26 + x64);
    const x66 = (x25 + x65);
    const x67 = (x23 + cast(u32, x24));
    const x68 = (x22 + x67);
    const x69 = (x21 + x68);
    const x70 = (x19 + cast(u32, x20));
    const x71 = (x18 + x70);
    const x72 = (x17 + x71);
    const x73 = (x15 + cast(u32, x16));
    const x74 = (x14 + x73);
    const x75 = (x13 + x74);
    const x76 = (x11 + cast(u32, x12));
    const x77 = (x10 + x76);
    const x78 = (x9 + x77);
    const x79 = (x7 + cast(u32, x8));
    const x80 = (x6 + x79);
    const x81 = (x5 + x80);
    const x82 = (x3 + cast(u32, x4));
    const x83 = (x2 + x82);
    const x84 = (x1 + x83);
    out1[0] = x51;
    out1[1] = x54;
    out1[2] = x57;
    out1[3] = x60;
    out1[4] = x63;
    out1[5] = x66;
    out1[6] = x69;
    out1[7] = x72;
    out1[8] = x75;
    out1[9] = x78;
    out1[10] = x81;
    out1[11] = x84;
}

/// The function setOne returns the field element one in the Montgomery domain.
///
/// Postconditions:
///   eval (from_montgomery out1) mod m = 1 mod m
///   0 ≤ eval out1 < m
///
pub fn setOne(out1: *MontgomeryDomainFieldElement) void {
    @setRuntimeSafety(mode == .Debug);

    out1[0] = 0x333ad68d;
    out1[1] = 0x1313e695;
    out1[2] = 0xb74f5885;
    out1[3] = 0xa7e5f24d;
    out1[4] = 0xbc8d220;
    out1[5] = 0x389cb27e;
    out1[6] = cast(u32, 0x0);
    out1[7] = cast(u32, 0x0);
    out1[8] = cast(u32, 0x0);
    out1[9] = cast(u32, 0x0);
    out1[10] = cast(u32, 0x0);
    out1[11] = cast(u32, 0x0);
}

/// The function msat returns the saturated representation of the prime modulus.
///
/// Postconditions:
///   twos_complement_eval out1 = m
///   0 ≤ eval out1 < m
///
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn msat(out1: *[13]u32) void {
    @setRuntimeSafety(mode == .Debug);

    out1[0] = 0xccc52973;
    out1[1] = 0xecec196a;
    out1[2] = 0x48b0a77a;
    out1[3] = 0x581a0db2;
    out1[4] = 0xf4372ddf;
    out1[5] = 0xc7634d81;
    out1[6] = 0xffffffff;
    out1[7] = 0xffffffff;
    out1[8] = 0xffffffff;
    out1[9] = 0xffffffff;
    out1[10] = 0xffffffff;
    out1[11] = 0xffffffff;
    out1[12] = cast(u32, 0x0);
}

/// The function divstep computes a divstep.
///
/// Preconditions:
///   0 ≤ eval arg4 < m
///   0 ≤ eval arg5 < m
/// Postconditions:
///   out1 = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then 1 - arg1 else 1 + arg1)
///   twos_complement_eval out2 = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then twos_complement_eval arg3 else twos_complement_eval arg2)
///   twos_complement_eval out3 = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then ⌊(twos_complement_eval arg3 - twos_complement_eval arg2) / 2⌋ else ⌊(twos_complement_eval arg3 + (twos_complement_eval arg3 mod 2) * twos_complement_eval arg2) / 2⌋)
///   eval (from_montgomery out4) mod m = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then (2 * eval (from_montgomery arg5)) mod m else (2 * eval (from_montgomery arg4)) mod m)
///   eval (from_montgomery out5) mod m = (if 0 < arg1 ∧ (twos_complement_eval arg3) is odd then (eval (from_montgomery arg4) - eval (from_montgomery arg4)) mod m else (eval (from_montgomery arg5) + (twos_complement_eval arg3 mod 2) * eval (from_montgomery arg4)) mod m)
///   0 ≤ eval out5 < m
///   0 ≤ eval out5 < m
///   0 ≤ eval out2 < m
///   0 ≤ eval out3 < m
///
/// Input Bounds:
///   arg1: [0x0 ~> 0xffffffff]
///   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg4: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   arg5: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
/// Output Bounds:
///   out1: [0x0 ~> 0xffffffff]
///   out2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   out3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   out4: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
///   out5: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn divstep(out1: *u32, out2: *[13]u32, out3: *[13]u32, out4: *[12]u32, out5: *[12]u32, arg1: u32, arg2: [13]u32, arg3: [13]u32, arg4: [12]u32, arg5: [12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    var x1: u32 = undefined;
    var x2: u1 = undefined;
    addcarryxU32(&x1, &x2, 0x0, (~arg1), cast(u32, 0x1));
    const x3 = (cast(u1, (x1 >> 31)) & cast(u1, ((arg3[0]) & cast(u32, 0x1))));
    var x4: u32 = undefined;
    var x5: u1 = undefined;
    addcarryxU32(&x4, &x5, 0x0, (~arg1), cast(u32, 0x1));
    var x6: u32 = undefined;
    cmovznzU32(&x6, x3, arg1, x4);
    var x7: u32 = undefined;
    cmovznzU32(&x7, x3, (arg2[0]), (arg3[0]));
    var x8: u32 = undefined;
    cmovznzU32(&x8, x3, (arg2[1]), (arg3[1]));
    var x9: u32 = undefined;
    cmovznzU32(&x9, x3, (arg2[2]), (arg3[2]));
    var x10: u32 = undefined;
    cmovznzU32(&x10, x3, (arg2[3]), (arg3[3]));
    var x11: u32 = undefined;
    cmovznzU32(&x11, x3, (arg2[4]), (arg3[4]));
    var x12: u32 = undefined;
    cmovznzU32(&x12, x3, (arg2[5]), (arg3[5]));
    var x13: u32 = undefined;
    cmovznzU32(&x13, x3, (arg2[6]), (arg3[6]));
    var x14: u32 = undefined;
    cmovznzU32(&x14, x3, (arg2[7]), (arg3[7]));
    var x15: u32 = undefined;
    cmovznzU32(&x15, x3, (arg2[8]), (arg3[8]));
    var x16: u32 = undefined;
    cmovznzU32(&x16, x3, (arg2[9]), (arg3[9]));
    var x17: u32 = undefined;
    cmovznzU32(&x17, x3, (arg2[10]), (arg3[10]));
    var x18: u32 = undefined;
    cmovznzU32(&x18, x3, (arg2[11]), (arg3[11]));
    var x19: u32 = undefined;
    cmovznzU32(&x19, x3, (arg2[12]), (arg3[12]));
    var x20: u32 = undefined;
    var x21: u1 = undefined;
    addcarryxU32(&x20, &x21, 0x0, cast(u32, 0x1), (~(arg2[0])));
    var x22: u32 = undefined;
    var x23: u1 = undefined;
    addcarryxU32(&x22, &x23, x21, cast(u32, 0x0), (~(arg2[1])));
    var x24: u32 = undefined;
    var x25: u1 = undefined;
    addcarryxU32(&x24, &x25, x23, cast(u32, 0x0), (~(arg2[2])));
    var x26: u32 = undefined;
    var x27: u1 = undefined;
    addcarryxU32(&x26, &x27, x25, cast(u32, 0x0), (~(arg2[3])));
    var x28: u32 = undefined;
    var x29: u1 = undefined;
    addcarryxU32(&x28, &x29, x27, cast(u32, 0x0), (~(arg2[4])));
    var x30: u32 = undefined;
    var x31: u1 = undefined;
    addcarryxU32(&x30, &x31, x29, cast(u32, 0x0), (~(arg2[5])));
    var x32: u32 = undefined;
    var x33: u1 = undefined;
    addcarryxU32(&x32, &x33, x31, cast(u32, 0x0), (~(arg2[6])));
    var x34: u32 = undefined;
    var x35: u1 = undefined;
    addcarryxU32(&x34, &x35, x33, cast(u32, 0x0), (~(arg2[7])));
    var x36: u32 = undefined;
    var x37: u1 = undefined;
    addcarryxU32(&x36, &x37, x35, cast(u32, 0x0), (~(arg2[8])));
    var x38: u32 = undefined;
    var x39: u1 = undefined;
    addcarryxU32(&x38, &x39, x37, cast(u32, 0x0), (~(arg2[9])));
    var x40: u32 = undefined;
    var x41: u1 = undefined;
    addcarryxU32(&x40, &x41, x39, cast(u32, 0x0), (~(arg2[10])));
    var x42: u32 = undefined;
    var x43: u1 = undefined;
    addcarryxU32(&x42, &x43, x41, cast(u32, 0x0), (~(arg2[11])));
    var x44: u32 = undefined;
    var x45: u1 = undefined;
    addcarryxU32(&x44, &x45, x43, cast(u32, 0x0), (~(arg2[12])));
    var x46: u32 = undefined;
    cmovznzU32(&x46, x3, (arg3[0]), x20);
    var x47: u32 = undefined;
    cmovznzU32(&x47, x3, (arg3[1]), x22);
    var x48: u32 = undefined;
    cmovznzU32(&x48, x3, (arg3[2]), x24);
    var x49: u32 = undefined;
    cmovznzU32(&x49, x3, (arg3[3]), x26);
    var x50: u32 = undefined;
    cmovznzU32(&x50, x3, (arg3[4]), x28);
    var x51: u32 = undefined;
    cmovznzU32(&x51, x3, (arg3[5]), x30);
    var x52: u32 = undefined;
    cmovznzU32(&x52, x3, (arg3[6]), x32);
    var x53: u32 = undefined;
    cmovznzU32(&x53, x3, (arg3[7]), x34);
    var x54: u32 = undefined;
    cmovznzU32(&x54, x3, (arg3[8]), x36);
    var x55: u32 = undefined;
    cmovznzU32(&x55, x3, (arg3[9]), x38);
    var x56: u32 = undefined;
    cmovznzU32(&x56, x3, (arg3[10]), x40);
    var x57: u32 = undefined;
    cmovznzU32(&x57, x3, (arg3[11]), x42);
    var x58: u32 = undefined;
    cmovznzU32(&x58, x3, (arg3[12]), x44);
    var x59: u32 = undefined;
    cmovznzU32(&x59, x3, (arg4[0]), (arg5[0]));
    var x60: u32 = undefined;
    cmovznzU32(&x60, x3, (arg4[1]), (arg5[1]));
    var x61: u32 = undefined;
    cmovznzU32(&x61, x3, (arg4[2]), (arg5[2]));
    var x62: u32 = undefined;
    cmovznzU32(&x62, x3, (arg4[3]), (arg5[3]));
    var x63: u32 = undefined;
    cmovznzU32(&x63, x3, (arg4[4]), (arg5[4]));
    var x64: u32 = undefined;
    cmovznzU32(&x64, x3, (arg4[5]), (arg5[5]));
    var x65: u32 = undefined;
    cmovznzU32(&x65, x3, (arg4[6]), (arg5[6]));
    var x66: u32 = undefined;
    cmovznzU32(&x66, x3, (arg4[7]), (arg5[7]));
    var x67: u32 = undefined;
    cmovznzU32(&x67, x3, (arg4[8]), (arg5[8]));
    var x68: u32 = undefined;
    cmovznzU32(&x68, x3, (arg4[9]), (arg5[9]));
    var x69: u32 = undefined;
    cmovznzU32(&x69, x3, (arg4[10]), (arg5[10]));
    var x70: u32 = undefined;
    cmovznzU32(&x70, x3, (arg4[11]), (arg5[11]));
    var x71: u32 = undefined;
    var x72: u1 = undefined;
    addcarryxU32(&x71, &x72, 0x0, x59, x59);
    var x73: u32 = undefined;
    var x74: u1 = undefined;
    addcarryxU32(&x73, &x74, x72, x60, x60);
    var x75: u32 = undefined;
    var x76: u1 = undefined;
    addcarryxU32(&x75, &x76, x74, x61, x61);
    var x77: u32 = undefined;
    var x78: u1 = undefined;
    addcarryxU32(&x77, &x78, x76, x62, x62);
    var x79: u32 = undefined;
    var x80: u1 = undefined;
    addcarryxU32(&x79, &x80, x78, x63, x63);
    var x81: u32 = undefined;
    var x82: u1 = undefined;
    addcarryxU32(&x81, &x82, x80, x64, x64);
    var x83: u32 = undefined;
    var x84: u1 = undefined;
    addcarryxU32(&x83, &x84, x82, x65, x65);
    var x85: u32 = undefined;
    var x86: u1 = undefined;
    addcarryxU32(&x85, &x86, x84, x66, x66);
    var x87: u32 = undefined;
    var x88: u1 = undefined;
    addcarryxU32(&x87, &x88, x86, x67, x67);
    var x89: u32 = undefined;
    var x90: u1 = undefined;
    addcarryxU32(&x89, &x90, x88, x68, x68);
    var x91: u32 = undefined;
    var x92: u1 = undefined;
    addcarryxU32(&x91, &x92, x90, x69, x69);
    var x93: u32 = undefined;
    var x94: u1 = undefined;
    addcarryxU32(&x93, &x94, x92, x70, x70);
    var x95: u32 = undefined;
    var x96: u1 = undefined;
    subborrowxU32(&x95, &x96, 0x0, x71, 0xccc52973);
    var x97: u32 = undefined;
    var x98: u1 = undefined;
    subborrowxU32(&x97, &x98, x96, x73, 0xecec196a);
    var x99: u32 = undefined;
    var x100: u1 = undefined;
    subborrowxU32(&x99, &x100, x98, x75, 0x48b0a77a);
    var x101: u32 = undefined;
    var x102: u1 = undefined;
    subborrowxU32(&x101, &x102, x100, x77, 0x581a0db2);
    var x103: u32 = undefined;
    var x104: u1 = undefined;
    subborrowxU32(&x103, &x104, x102, x79, 0xf4372ddf);
    var x105: u32 = undefined;
    var x106: u1 = undefined;
    subborrowxU32(&x105, &x106, x104, x81, 0xc7634d81);
    var x107: u32 = undefined;
    var x108: u1 = undefined;
    subborrowxU32(&x107, &x108, x106, x83, 0xffffffff);
    var x109: u32 = undefined;
    var x110: u1 = undefined;
    subborrowxU32(&x109, &x110, x108, x85, 0xffffffff);
    var x111: u32 = undefined;
    var x112: u1 = undefined;
    subborrowxU32(&x111, &x112, x110, x87, 0xffffffff);
    var x113: u32 = undefined;
    var x114: u1 = undefined;
    subborrowxU32(&x113, &x114, x112, x89, 0xffffffff);
    var x115: u32 = undefined;
    var x116: u1 = undefined;
    subborrowxU32(&x115, &x116, x114, x91, 0xffffffff);
    var x117: u32 = undefined;
    var x118: u1 = undefined;
    subborrowxU32(&x117, &x118, x116, x93, 0xffffffff);
    var x119: u32 = undefined;
    var x120: u1 = undefined;
    subborrowxU32(&x119, &x120, x118, cast(u32, x94), cast(u32, 0x0));
    const x121 = (arg4[11]);
    const x122 = (arg4[10]);
    const x123 = (arg4[9]);
    const x124 = (arg4[8]);
    const x125 = (arg4[7]);
    const x126 = (arg4[6]);
    const x127 = (arg4[5]);
    const x128 = (arg4[4]);
    const x129 = (arg4[3]);
    const x130 = (arg4[2]);
    const x131 = (arg4[1]);
    const x132 = (arg4[0]);
    var x133: u32 = undefined;
    var x134: u1 = undefined;
    subborrowxU32(&x133, &x134, 0x0, cast(u32, 0x0), x132);
    var x135: u32 = undefined;
    var x136: u1 = undefined;
    subborrowxU32(&x135, &x136, x134, cast(u32, 0x0), x131);
    var x137: u32 = undefined;
    var x138: u1 = undefined;
    subborrowxU32(&x137, &x138, x136, cast(u32, 0x0), x130);
    var x139: u32 = undefined;
    var x140: u1 = undefined;
    subborrowxU32(&x139, &x140, x138, cast(u32, 0x0), x129);
    var x141: u32 = undefined;
    var x142: u1 = undefined;
    subborrowxU32(&x141, &x142, x140, cast(u32, 0x0), x128);
    var x143: u32 = undefined;
    var x144: u1 = undefined;
    subborrowxU32(&x143, &x144, x142, cast(u32, 0x0), x127);
    var x145: u32 = undefined;
    var x146: u1 = undefined;
    subborrowxU32(&x145, &x146, x144, cast(u32, 0x0), x126);
    var x147: u32 = undefined;
    var x148: u1 = undefined;
    subborrowxU32(&x147, &x148, x146, cast(u32, 0x0), x125);
    var x149: u32 = undefined;
    var x150: u1 = undefined;
    subborrowxU32(&x149, &x150, x148, cast(u32, 0x0), x124);
    var x151: u32 = undefined;
    var x152: u1 = undefined;
    subborrowxU32(&x151, &x152, x150, cast(u32, 0x0), x123);
    var x153: u32 = undefined;
    var x154: u1 = undefined;
    subborrowxU32(&x153, &x154, x152, cast(u32, 0x0), x122);
    var x155: u32 = undefined;
    var x156: u1 = undefined;
    subborrowxU32(&x155, &x156, x154, cast(u32, 0x0), x121);
    var x157: u32 = undefined;
    cmovznzU32(&x157, x156, cast(u32, 0x0), 0xffffffff);
    var x158: u32 = undefined;
    var x159: u1 = undefined;
    addcarryxU32(&x158, &x159, 0x0, x133, (x157 & 0xccc52973));
    var x160: u32 = undefined;
    var x161: u1 = undefined;
    addcarryxU32(&x160, &x161, x159, x135, (x157 & 0xecec196a));
    var x162: u32 = undefined;
    var x163: u1 = undefined;
    addcarryxU32(&x162, &x163, x161, x137, (x157 & 0x48b0a77a));
    var x164: u32 = undefined;
    var x165: u1 = undefined;
    addcarryxU32(&x164, &x165, x163, x139, (x157 & 0x581a0db2));
    var x166: u32 = undefined;
    var x167: u1 = undefined;
    addcarryxU32(&x166, &x167, x165, x141, (x157 & 0xf4372ddf));
    var x168: u32 = undefined;
    var x169: u1 = undefined;
    addcarryxU32(&x168, &x169, x167, x143, (x157 & 0xc7634d81));
    var x170: u32 = undefined;
    var x171: u1 = undefined;
    addcarryxU32(&x170, &x171, x169, x145, x157);
    var x172: u32 = undefined;
    var x173: u1 = undefined;
    addcarryxU32(&x172, &x173, x171, x147, x157);
    var x174: u32 = undefined;
    var x175: u1 = undefined;
    addcarryxU32(&x174, &x175, x173, x149, x157);
    var x176: u32 = undefined;
    var x177: u1 = undefined;
    addcarryxU32(&x176, &x177, x175, x151, x157);
    var x178: u32 = undefined;
    var x179: u1 = undefined;
    addcarryxU32(&x178, &x179, x177, x153, x157);
    var x180: u32 = undefined;
    var x181: u1 = undefined;
    addcarryxU32(&x180, &x181, x179, x155, x157);
    var x182: u32 = undefined;
    cmovznzU32(&x182, x3, (arg5[0]), x158);
    var x183: u32 = undefined;
    cmovznzU32(&x183, x3, (arg5[1]), x160);
    var x184: u32 = undefined;
    cmovznzU32(&x184, x3, (arg5[2]), x162);
    var x185: u32 = undefined;
    cmovznzU32(&x185, x3, (arg5[3]), x164);
    var x186: u32 = undefined;
    cmovznzU32(&x186, x3, (arg5[4]), x166);
    var x187: u32 = undefined;
    cmovznzU32(&x187, x3, (arg5[5]), x168);
    var x188: u32 = undefined;
    cmovznzU32(&x188, x3, (arg5[6]), x170);
    var x189: u32 = undefined;
    cmovznzU32(&x189, x3, (arg5[7]), x172);
    var x190: u32 = undefined;
    cmovznzU32(&x190, x3, (arg5[8]), x174);
    var x191: u32 = undefined;
    cmovznzU32(&x191, x3, (arg5[9]), x176);
    var x192: u32 = undefined;
    cmovznzU32(&x192, x3, (arg5[10]), x178);
    var x193: u32 = undefined;
    cmovznzU32(&x193, x3, (arg5[11]), x180);
    const x194 = cast(u1, (x46 & cast(u32, 0x1)));
    var x195: u32 = undefined;
    cmovznzU32(&x195, x194, cast(u32, 0x0), x7);
    var x196: u32 = undefined;
    cmovznzU32(&x196, x194, cast(u32, 0x0), x8);
    var x197: u32 = undefined;
    cmovznzU32(&x197, x194, cast(u32, 0x0), x9);
    var x198: u32 = undefined;
    cmovznzU32(&x198, x194, cast(u32, 0x0), x10);
    var x199: u32 = undefined;
    cmovznzU32(&x199, x194, cast(u32, 0x0), x11);
    var x200: u32 = undefined;
    cmovznzU32(&x200, x194, cast(u32, 0x0), x12);
    var x201: u32 = undefined;
    cmovznzU32(&x201, x194, cast(u32, 0x0), x13);
    var x202: u32 = undefined;
    cmovznzU32(&x202, x194, cast(u32, 0x0), x14);
    var x203: u32 = undefined;
    cmovznzU32(&x203, x194, cast(u32, 0x0), x15);
    var x204: u32 = undefined;
    cmovznzU32(&x204, x194, cast(u32, 0x0), x16);
    var x205: u32 = undefined;
    cmovznzU32(&x205, x194, cast(u32, 0x0), x17);
    var x206: u32 = undefined;
    cmovznzU32(&x206, x194, cast(u32, 0x0), x18);
    var x207: u32 = undefined;
    cmovznzU32(&x207, x194, cast(u32, 0x0), x19);
    var x208: u32 = undefined;
    var x209: u1 = undefined;
    addcarryxU32(&x208, &x209, 0x0, x46, x195);
    var x210: u32 = undefined;
    var x211: u1 = undefined;
    addcarryxU32(&x210, &x211, x209, x47, x196);
    var x212: u32 = undefined;
    var x213: u1 = undefined;
    addcarryxU32(&x212, &x213, x211, x48, x197);
    var x214: u32 = undefined;
    var x215: u1 = undefined;
    addcarryxU32(&x214, &x215, x213, x49, x198);
    var x216: u32 = undefined;
    var x217: u1 = undefined;
    addcarryxU32(&x216, &x217, x215, x50, x199);
    var x218: u32 = undefined;
    var x219: u1 = undefined;
    addcarryxU32(&x218, &x219, x217, x51, x200);
    var x220: u32 = undefined;
    var x221: u1 = undefined;
    addcarryxU32(&x220, &x221, x219, x52, x201);
    var x222: u32 = undefined;
    var x223: u1 = undefined;
    addcarryxU32(&x222, &x223, x221, x53, x202);
    var x224: u32 = undefined;
    var x225: u1 = undefined;
    addcarryxU32(&x224, &x225, x223, x54, x203);
    var x226: u32 = undefined;
    var x227: u1 = undefined;
    addcarryxU32(&x226, &x227, x225, x55, x204);
    var x228: u32 = undefined;
    var x229: u1 = undefined;
    addcarryxU32(&x228, &x229, x227, x56, x205);
    var x230: u32 = undefined;
    var x231: u1 = undefined;
    addcarryxU32(&x230, &x231, x229, x57, x206);
    var x232: u32 = undefined;
    var x233: u1 = undefined;
    addcarryxU32(&x232, &x233, x231, x58, x207);
    var x234: u32 = undefined;
    cmovznzU32(&x234, x194, cast(u32, 0x0), x59);
    var x235: u32 = undefined;
    cmovznzU32(&x235, x194, cast(u32, 0x0), x60);
    var x236: u32 = undefined;
    cmovznzU32(&x236, x194, cast(u32, 0x0), x61);
    var x237: u32 = undefined;
    cmovznzU32(&x237, x194, cast(u32, 0x0), x62);
    var x238: u32 = undefined;
    cmovznzU32(&x238, x194, cast(u32, 0x0), x63);
    var x239: u32 = undefined;
    cmovznzU32(&x239, x194, cast(u32, 0x0), x64);
    var x240: u32 = undefined;
    cmovznzU32(&x240, x194, cast(u32, 0x0), x65);
    var x241: u32 = undefined;
    cmovznzU32(&x241, x194, cast(u32, 0x0), x66);
    var x242: u32 = undefined;
    cmovznzU32(&x242, x194, cast(u32, 0x0), x67);
    var x243: u32 = undefined;
    cmovznzU32(&x243, x194, cast(u32, 0x0), x68);
    var x244: u32 = undefined;
    cmovznzU32(&x244, x194, cast(u32, 0x0), x69);
    var x245: u32 = undefined;
    cmovznzU32(&x245, x194, cast(u32, 0x0), x70);
    var x246: u32 = undefined;
    var x247: u1 = undefined;
    addcarryxU32(&x246, &x247, 0x0, x182, x234);
    var x248: u32 = undefined;
    var x249: u1 = undefined;
    addcarryxU32(&x248, &x249, x247, x183, x235);
    var x250: u32 = undefined;
    var x251: u1 = undefined;
    addcarryxU32(&x250, &x251, x249, x184, x236);
    var x252: u32 = undefined;
    var x253: u1 = undefined;
    addcarryxU32(&x252, &x253, x251, x185, x237);
    var x254: u32 = undefined;
    var x255: u1 = undefined;
    addcarryxU32(&x254, &x255, x253, x186, x238);
    var x256: u32 = undefined;
    var x257: u1 = undefined;
    addcarryxU32(&x256, &x257, x255, x187, x239);
    var x258: u32 = undefined;
    var x259: u1 = undefined;
    addcarryxU32(&x258, &x259, x257, x188, x240);
    var x260: u32 = undefined;
    var x261: u1 = undefined;
    addcarryxU32(&x260, &x261, x259, x189, x241);
    var x262: u32 = undefined;
    var x263: u1 = undefined;
    addcarryxU32(&x262, &x263, x261, x190, x242);
    var x264: u32 = undefined;
    var x265: u1 = undefined;
    addcarryxU32(&x264, &x265, x263, x191, x243);
    var x266: u32 = undefined;
    var x267: u1 = undefined;
    addcarryxU32(&x266, &x267, x265, x192, x244);
    var x268: u32 = undefined;
    var x269: u1 = undefined;
    addcarryxU32(&x268, &x269, x267, x193, x245);
    var x270: u32 = undefined;
    var x271: u1 = undefined;
    subborrowxU32(&x270, &x271, 0x0, x246, 0xccc52973);
    var x272: u32 = undefined;
    var x273: u1 = undefined;
    subborrowxU32(&x272, &x273, x271, x248, 0xecec196a);
    var x274: u32 = undefined;
    var x275: u1 = undefined;
    subborrowxU32(&x274, &x275, x273, x250, 0x48b0a77a);
    var x276: u32 = undefined;
    var x277: u1 = undefined;
    subborrowxU32(&x276, &x277, x275, x252, 0x581a0db2);
    var x278: u32 = undefined;
    var x279: u1 = undefined;
    subborrowxU32(&x278, &x279, x277, x254, 0xf4372ddf);
    var x280: u32 = undefined;
    var x281: u1 = undefined;
    subborrowxU32(&x280, &x281, x279, x256, 0xc7634d81);
    var x282: u32 = undefined;
    var x283: u1 = undefined;
    subborrowxU32(&x282, &x283, x281, x258, 0xffffffff);
    var x284: u32 = undefined;
    var x285: u1 = undefined;
    subborrowxU32(&x284, &x285, x283, x260, 0xffffffff);
    var x286: u32 = undefined;
    var x287: u1 = undefined;
    subborrowxU32(&x286, &x287, x285, x262, 0xffffffff);
    var x288: u32 = undefined;
    var x289: u1 = undefined;
    subborrowxU32(&x288, &x289, x287, x264, 0xffffffff);
    var x290: u32 = undefined;
    var x291: u1 = undefined;
    subborrowxU32(&x290, &x291, x289, x266, 0xffffffff);
    var x292: u32 = undefined;
    var x293: u1 = undefined;
    subborrowxU32(&x292, &x293, x291, x268, 0xffffffff);
    var x294: u32 = undefined;
    var x295: u1 = undefined;
    subborrowxU32(&x294, &x295, x293, cast(u32, x269), cast(u32, 0x0));
    var x296: u32 = undefined;
    var x297: u1 = undefined;
    addcarryxU32(&x296, &x297, 0x0, x6, cast(u32, 0x1));
    const x298 = ((x208 >> 1) | ((x210 << 31) & 0xffffffff));
    const x299 = ((x210 >> 1) | ((x212 << 31) & 0xffffffff));
    const x300 = ((x212 >> 1) | ((x214 << 31) & 0xffffffff));
    const x301 = ((x214 >> 1) | ((x216 << 31) & 0xffffffff));
    const x302 = ((x216 >> 1) | ((x218 << 31) & 0xffffffff));
    const x303 = ((x218 >> 1) | ((x220 << 31) & 0xffffffff));
    const x304 = ((x220 >> 1) | ((x222 << 31) & 0xffffffff));
    const x305 = ((x222 >> 1) | ((x224 << 31) & 0xffffffff));
    const x306 = ((x224 >> 1) | ((x226 << 31) & 0xffffffff));
    const x307 = ((x226 >> 1) | ((x228 << 31) & 0xffffffff));
    const x308 = ((x228 >> 1) | ((x230 << 31) & 0xffffffff));
    const x309 = ((x230 >> 1) | ((x232 << 31) & 0xffffffff));
    const x310 = ((x232 & 0x80000000) | (x232 >> 1));
    var x311: u32 = undefined;
    cmovznzU32(&x311, x120, x95, x71);
    var x312: u32 = undefined;
    cmovznzU32(&x312, x120, x97, x73);
    var x313: u32 = undefined;
    cmovznzU32(&x313, x120, x99, x75);
    var x314: u32 = undefined;
    cmovznzU32(&x314, x120, x101, x77);
    var x315: u32 = undefined;
    cmovznzU32(&x315, x120, x103, x79);
    var x316: u32 = undefined;
    cmovznzU32(&x316, x120, x105, x81);
    var x317: u32 = undefined;
    cmovznzU32(&x317, x120, x107, x83);
    var x318: u32 = undefined;
    cmovznzU32(&x318, x120, x109, x85);
    var x319: u32 = undefined;
    cmovznzU32(&x319, x120, x111, x87);
    var x320: u32 = undefined;
    cmovznzU32(&x320, x120, x113, x89);
    var x321: u32 = undefined;
    cmovznzU32(&x321, x120, x115, x91);
    var x322: u32 = undefined;
    cmovznzU32(&x322, x120, x117, x93);
    var x323: u32 = undefined;
    cmovznzU32(&x323, x295, x270, x246);
    var x324: u32 = undefined;
    cmovznzU32(&x324, x295, x272, x248);
    var x325: u32 = undefined;
    cmovznzU32(&x325, x295, x274, x250);
    var x326: u32 = undefined;
    cmovznzU32(&x326, x295, x276, x252);
    var x327: u32 = undefined;
    cmovznzU32(&x327, x295, x278, x254);
    var x328: u32 = undefined;
    cmovznzU32(&x328, x295, x280, x256);
    var x329: u32 = undefined;
    cmovznzU32(&x329, x295, x282, x258);
    var x330: u32 = undefined;
    cmovznzU32(&x330, x295, x284, x260);
    var x331: u32 = undefined;
    cmovznzU32(&x331, x295, x286, x262);
    var x332: u32 = undefined;
    cmovznzU32(&x332, x295, x288, x264);
    var x333: u32 = undefined;
    cmovznzU32(&x333, x295, x290, x266);
    var x334: u32 = undefined;
    cmovznzU32(&x334, x295, x292, x268);
    out1.* = x296;
    out2[0] = x7;
    out2[1] = x8;
    out2[2] = x9;
    out2[3] = x10;
    out2[4] = x11;
    out2[5] = x12;
    out2[6] = x13;
    out2[7] = x14;
    out2[8] = x15;
    out2[9] = x16;
    out2[10] = x17;
    out2[11] = x18;
    out2[12] = x19;
    out3[0] = x298;
    out3[1] = x299;
    out3[2] = x300;
    out3[3] = x301;
    out3[4] = x302;
    out3[5] = x303;
    out3[6] = x304;
    out3[7] = x305;
    out3[8] = x306;
    out3[9] = x307;
    out3[10] = x308;
    out3[11] = x309;
    out3[12] = x310;
    out4[0] = x311;
    out4[1] = x312;
    out4[2] = x313;
    out4[3] = x314;
    out4[4] = x315;
    out4[5] = x316;
    out4[6] = x317;
    out4[7] = x318;
    out4[8] = x319;
    out4[9] = x320;
    out4[10] = x321;
    out4[11] = x322;
    out5[0] = x323;
    out5[1] = x324;
    out5[2] = x325;
    out5[3] = x326;
    out5[4] = x327;
    out5[5] = x328;
    out5[6] = x329;
    out5[7] = x330;
    out5[8] = x331;
    out5[9] = x332;
    out5[10] = x333;
    out5[11] = x334;
}

/// The function divstepPrecomp returns the precomputed value for Bernstein-Yang-inversion (in montgomery form).
///
/// Postconditions:
///   eval (from_montgomery out1) = ⌊(m - 1) / 2⌋^(if ⌊log2 m⌋ + 1 < 46 then ⌊(49 * (⌊log2 m⌋ + 1) + 80) / 17⌋ else ⌊(49 * (⌊log2 m⌋ + 1) + 57) / 17⌋)
///   0 ≤ eval out1 < m
///
/// Output Bounds:
///   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
pub fn divstepPrecomp(out1: *[12]u32) void {
    @setRuntimeSafety(mode == .Debug);

    out1[0] = 0xe6045b6a;
    out1[1] = 0x49589ae0;
    out1[2] = 0x870040ed;
    out1[3] = 0x3c9a5352;
    out1[4] = 0x977dc242;
    out1[5] = 0xdacb097e;
    out1[6] = 0xd1ecbe36;
    out1[7] = 0xb5ab30a6;
    out1[8] = 0x1f959973;
    out1[9] = 0x97d7a108;
    out1[10] = 0xd27192bc;
    out1[11] = 0x2ba012f8;
}
